<?php

namespace admin;

use common\libraries\Translation;

/**
 * Abstract class that describes an action for a package which handles the package sources. Common functionality is
 * grouped here and must be overridden for more information
 * 
 * @author Sven Vanpoucke
 */
abstract class PackageSourceAction extends PackageAction
{
    /**
     * The installer source Keeps track of the package and the package.info from the given source
     * 
     * @var PackageSource
     */
    private $package_source;

    /**
     * Constructor Initializes the source
     * 
     * @param $source_type String
     */
    function __construct($source_type)
    {
        parent :: __construct();
        
        $this->package_source = PackageSource :: factory($this, $source_type);
    }

    /**
     * Returns the package source
     * 
     * @return PackageSource
     */
    function get_package_source()
    {
        return $this->package_source;
    }

    /**
     * Runs the package source action
     * 
     * @return boolean
     */
    function run()
    {
        if ($this->initialize() && $this->verify_dependencies() && $this->process_source() && $this->process())
        {
            return true;
        }
        
        if ($this->revert())
        {
            $this->action_successful('revert', Translation :: get('PackageRevertSuccess'));
        }
        
        return false;
    }

    /**
     * Initializes the package source action
     * 
     * @return boolean
     */
    function initialize()
    {
        $package_attributes = $this->package_source->get_attributes();
        
        if (! $package_attributes)
        {
            return $this->action_failed('initialization', Translation :: get('PackageAttributesNotFound'));
        }
        else
        {
            $this->add_message(Translation :: get('PackageAttributesFound'));
        }
        
        return true;
    }

    /**
     * Verifies the package dependencies
     * 
     * @return boolean
     */
    function verify_dependencies($package_attributes)
    {
        return parent :: verify_dependencies($this->package_source->get_attributes());
    }

    /**
     * Processes the source
     * 
     * @return boolean
     */
    function process_source()
    {
        if (! $this->package_source->process())
        {
            return $this->action_failed('source', Translation :: get('PackageSourceProcessingFailed'));
        }
        
        return $this->action_successful('source', Translation :: get('PackageSourceProcessed'));
    
    }

    /**
     * Reverts the package source action on failure
     * 
     * @return boolean
     */
    function revert()
    {
        if (! $this->package_source->revert())
        {
            $this->action_failed('revert', Translation :: get('PackageSourceRevertFailed'));
        }
        
        $this->add_message(Translation :: get('PackageSourceRevertSuccess'));
        
        return true;
    }

    /**
     * Delegation methods
     */
    
    /**
     * Returns the attributes of the package
     */
    public function get_attributes()
    {
        return $this->get_package_source()->get_attributes();
    }

    /**
     * Abstract functionality
     */
    
    /**
     * Processes the action
     */
    abstract function process();

    /**
     * Returns the name and type of the package
     * 
     * @param String[]
     */
    abstract function get_package_type_and_name();

}

?>
