<?php
namespace admin;

use common\libraries\Translation;

use common\libraries\Utilities;
use common\libraries\Path;
use Exception;

/**
 * Abstract class that describes the install process for packages. Must be extended by the available package types.
 * 
 * @author Hans De Bisschop - Erasmus Hogeschool Brussel
 * @author Magali Gillard - Erasmus Hogeschool Brussel
 * @author Sven Vanpoucke - Hogeschool Gent - Cleanup, code refactoring and bugfixes, comments
 */

abstract class PackageInstallerType
{
    /**
     * The parent installer
     * 
     * @var PackageInstaller
     */
    private $parent;

    /**
     * Constructor
     * 
     * @param $parent PackageInstaller
     */
    function __construct($parent)
    {
        $this->set_parent($parent);
    }

    /**
     * Invokes the constructor of the class that corresponds to the specified type of package installer type.
     * 
     * @param $parent PackageInstaller
     * @param $type String
     */
    static function factory($parent, $type)
    {
        if (! $parent instanceof PackageInstaller)
        {
            throw new Exception(Translation :: get('ParentMustBePackageInstaller'));
        }
        
        $class = __NAMESPACE__ . '\\' . 'PackageInstaller' . Utilities :: underscores_to_camelcase($type) . 'Type';
        
        if (! class_exists($class))
        {
            throw new Exception(Translation :: get('CouldNotFindPackageInstallerType', array('TYPE' => $type)));
        }
        
        return new $class($parent);
    }

    /**
     * Getters and setters
     */
    
    /**
     * Returns the parent PackageUpdater
     * 
     * @return PackageUpdater
     */
    function get_parent()
    {
        return $this->parent;
    }

    /**
     * Sets the parent PackageUpdater
     * 
     * @param $parent PackageUpdater
     */
    function set_parent($parent)
    {
        $this->parent = $parent;
    }

    /**
     * Delegation functions
     */
    
    /**
     * Adds a message to the message queue
     * 
     * @param $message String
     * @param $type String
     */
    function add_message($message, $type = PackageAction :: TYPE_NORMAL)
    {
        $this->get_parent()->add_message($message, $type);
    }

    /**
     * Sets the current block to failed
     * 
     * @param $type String
     * @param $message String
     *
     * @return false
     */
    function action_failed($type, $message)
    {
        return $this->get_parent()->action_failed($type, $message);
    }

    /**
     * Sets the current block to success
     * 
     * @param $type String
     * @param $message String
     *
     * @return true
     */
    function action_successful($type, $message)
    {
        return $this->get_parent()->action_successful($type, $message);
    }

    /**
     * Returns the package source
     * 
     * @return PackageSource
     */
    function get_source()
    {
        return $this->get_parent()->get_package_source();
    }

    /**
     * Abstract functionality
     */
    
    /**
     * Installs the package
     * 
     * @return boolean
     */
    abstract function install();

}
?>