<?php
namespace admin;

use common\libraries\Registration;
use common\libraries\CommonDataManager;
use common\libraries\Utilities;
use common\libraries\Translation;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\MessageLogger;

use repository\ContentObject;

/**
 * Abstract class that describes a chamilo package depedency, can be used for all elements (applications, objects,
 * tools, subapplications) from chamilo
 * 
 * @author Sven Vanpoucke - Hogeschool Gent
 * @package admin.lib.package_installer.dependency
 */

abstract class ChamiloPackageDependency extends PackageDependency
{
    const PROPERTY_VERSION = 'version';
    
    /**
     * Keeps track of the version information
     * 
     * @var String[]
     */
    private $version;

    /**
     * Constructor
     * 
     * @param $dependency String[]
     */
    function __construct($dependency)
    {
        parent :: __construct($dependency);
        $this->set_version($dependency['version']);
    }

    /**
     * Returns the version
     * 
     * @return String[]
     */
    public function get_version()
    {
        return $this->version;
    }

    /**
     * Returns the operator
     * 
     * @return String
     */
    public function get_operator()
    {
        return $this->version['type'];
    }

    /**
     * Returns the version number
     * 
     * @return String
     */
    public function get_version_number()
    {
        return $this->version['_content'];
    }

    /**
     * Sets the version parameter
     * 
     * @param $version String[]
     */
    public function set_version($version)
    {
        $this->version = $version;
    }

    /**
     * Creates a dependency information string as html
     * 
     * @return String
     */
    function as_html()
    {
        $version = $this->get_version();
        return $this->get_id() . ', ' . Translation :: get('Version', array(), Utilities :: COMMON_LIBRARIES) . ': ' . $version['_content'];
    }

    /**
     * Checks the dependency in the registration table of the administration
     * 
     * @return boolean
     */
    function check()
    {
        $version = $this->get_version();
        $type = $this->get_type();
        
        $message = Translation :: get('DependencyCheck' . Utilities :: underscores_to_camelcase($type)) . ': ' . $this->as_html() . ' ' . Translation :: get('Found', array(), Utilities :: COMMON_LIBRARIES) . ': ';
        
        $context = $this->get_context();
        
        $registration = CommonDataManager :: get_registration($context);
        
        if (! $registration instanceof Registration)
        {
            $message .= '--' . Translation :: get('Nothing', array(), Utilities :: COMMON_LIBRARIES) . '--';
            $this->logger->add_message($message);
            return false;
        }
        else
        {
            $target_version = $this->version_compare($version['type'], $version['_content'], $registration->get_version());
            if (! $target_version)
            {
                $message .= '--' . Translation :: get('WrongVersion', array(), Utilities :: COMMON_LIBRARIES) . '--';
                $this->logger->add_message($message);
                $this->logger->add_message(Translation :: get('DependencyObjectWrongVersion'), MessageLogger :: TYPE_WARNING);
                return false;
            }
            else
            {
                if (! $registration->is_active())
                {
                    $message .= '--' . Translation :: get('InactiveObject', array(), Utilities :: COMMON_LIBRARIES) . '--';
                    $this->logger->add_message($message);
                    $this->logger->add_message(Translation :: get('DependencyActivateObjectWarning'), MessageLogger :: TYPE_WARNING);
                }
                else
                {
                    $message .= $registration->get_version();
                    $this->logger->add_message($message);
                }
                
                return true;
            }
        }
    }

    abstract function get_type();

    abstract function get_context();
}
?>