<?php
namespace admin;

use common\libraries\Registration;

use Exception;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\Filesystem;

/**
 * Package Remover
 * 
 * @author Hans De Bisschop - Erasmus Hogeschool Brussel
 * @author Magali Gillard - Erasmus Hogeschool Brussel
 * @author Sven Vanpoucke - Hogeschool Gent - Cleanup, code refactoring and bugfixes, comments
 * @package admin.lib.package_installer
 */
abstract class PackageRemover extends PackageAction
{
    /**
     * The package registration
     * 
     * @var Registration
     */
    private $registration;

    /**
     * Constructor Intializes the registration
     * 
     * @param $registration Registration
     * @param $source String
     */
    function __construct(Registration $registration)
    {
        if (! $registration instanceof Registration)
        {
            throw new Exception(Translation :: get('ValidRegistrationRequired'));
        }
        
        $this->registration = $registration;
    }

    /**
     * Invokes the constructor of the class that corresponds to the specified type of package remover type.
     * 
     * @param $type String
     * @param $registration String
     */
    static function factory($type, $registration)
    {
        $class = __NAMESPACE__ . '\\' . 'Package' . Utilities :: underscores_to_camelcase($type) . 'Remover';
        
        if (! class_exists($class))
        {
            throw new Exception(Translation :: get('CouldNotFindPackageRemoverType', array('TYPE' => $type)));
        }
        
        return new $class($registration);
    }

    /**
     * Executes the removal process
     */
    function run()
    {
        if ($this->initialize() && $this->check_dependencies() && $this->delete() && $this->remove_package_folder() && $this->remove_package_registration())
        {
            return $this->action_successful('finished', Translation :: get('PackageSuccessfullyDeleted'));
        }
        
        return $this->action_failed('failed', Translation :: get('PackageDeletionFailed'));
    }

    /**
     * Initializes the remover
     * 
     * @param $with_success_message boolean - whether or not to add a success message after the initialization
     * @return boolean
     */
    function initialize($with_success_message = true)
    {
        if (! $this->registration->can_be_activated())
        {
            return $this->action_failed('initilization', Translation :: get('PackageTypeNotRemovable'));
        }
        
        if (! $this->registration->deactivate(true))
        {
            return $this->action_failed('initilization', Translation :: get('PackageDeactivationFailed'));
        }
        
        $this->add_message(Translation :: get('PackageDeactivated'));
        
        if ($with_success_message)
        {
            return $this->action_successful('initilization', Translation :: get('PackageRemovalInitialized'));
        }
        
        return true;
    }

    /**
     * Checks the dependencies
     * 
     * @return boolean;
     */
    function check_dependencies()
    {
        $package_info = new PackageInfo($this->registration->get_context());
        $package = $package_info->get_package();
        
        return $this->verify_dependencies($package);
    }

    /**
     * Removes the package folder
     */
    function remove_package_folder()
    {
        $path = $this->get_package_folder_path();
        if (! $path)
        {
            return $this->action_successful('folder', Translation :: get('NoPackageFolderFound'));
        }
        
        if (! Filesystem :: remove($path))
        {
            return $this->action_failed('folder', Translation :: get('CouldNotRemovePackagFiles'));
        }
        
        return $this->action_successful('folder', Translation :: get('RemovedPackageFiles'));
    }

    /**
     * Removes package registration
     */
    function remove_package_registration()
    {
        $registration = $this->get_registration();
        
        if (! $registration->delete())
        {
            $this->add_message(Translation :: get('CouldNotRemoveRegistration'));
        }
        
        $this->add_message(Translation :: get('RemovedRegistration'));
        
        return true;
    }

    /**
     * Executes the check on the dependency verifier for the current action
     * 
     * @param $package_dependency_verifier PackageDependencyVerifier
     */
    function execute_dependency_check(PackageDependencyVerifier $package_dependency_verifier)
    {
        return $package_dependency_verifier->is_removable();
    }

    /**
     * Getters and setters
     */
    
    /**
     * Sets the registration of the package
     * 
     * @return Registration $registration
     */
    public function get_registration()
    {
        return $this->registration;
    }

    /**
     * Sets the registration of the package
     * 
     * @param $registration Registration
     */
    public function set_registration($registration)
    {
        $this->registration = $registration;
    }

    /**
     * Abstract functionality
     */
    
    /**
     * Deletes the actual package
     * 
     * @return boolean
     */
    abstract function delete();

    /**
     * Dummy functionality
     */
    
    /**
     * Returns the path to the package folder
     * 
     * @return String or null if nothing has to be removed
     */
    function get_package_folder_path()
    {
        return null;
    }
}
?>