<?php
namespace admin;

use common\libraries\Setting;

use common\libraries\CommonDataManager;

use menu\ApplicationItem;

use common\libraries\MessageLogger;

use reporting\ReportingTemplateRegistration;
use reporting\ReportingBlockRegistration;
use reporting\ReportingDataManager;

use common\libraries\Path;
use common\libraries\WebApplication;
use common\libraries\Translation;
use common\libraries\EqualityCondition;
use common\libraries\Mdb2Database;
use common\libraries\Filesystem;

use DOMDocument;

use tracking\Event;
use tracking\TrackingDataManager;
use tracking\TrackerRegistration;

use menu\MenuDataManager;
use menu\NavigationItem;

use webservice\WebserviceDataManager;
use webservice\WebserviceRegistration;

use rights\RightsDataManager;
use rights\Location;

/**
 * Package Application Remover
 * 
 * @author Hans De Bisschop - Erasmus Hogeschool Brussel
 * @author Magali Gillard - Erasmus Hogeschool Brussel
 * @author Sven Vanpoucke - Hogeschool Gent - Cleanup, code refactoring and bugfixes, comments
 * @package admin.lib.package_installer
 */
class PackageApplicationRemover extends PackageRemover
{

    /**
     * Deletes the actual package
     * 
     * @return boolean
     */
    function delete()
    {
        // Deactivate the application, thus making it inaccesible
        $mdm = MenuDataManager :: get_instance();
        $this->add_message(Translation :: get('RemovingMenuItems'));
        
        $condition = new EqualityCondition(ApplicationItem :: PROPERTY_APPLICATION, $this->get_registration()->get_name());
        
        if ($mdm->delete_navigation_items($condition))
        {
            $this->action_successful('initilization', Translation :: get('ApplicationMenuItemSucessfullyRemoved'));
        }
        else
        {
            return $this->action_failed('initilization', Translation :: get('ApplicationMenuItemNotRemoved'));
        }
        
        // Remove webservices
        if (! $this->remove_webservices())
        {
            return $this->action_failed('webservice', Translation :: get('WebservicesDeletionFailed'));
        }
        else
        {
            $this->action_successful('webservice', Translation :: get('WebservicesSuccessfullyDeleted'));
        }
        
        // Remove reporting
        if (! $this->remove_reporting())
        {
            return $this->action_failed('reporting', Translation :: get('ReportingDeletionFailed'));
        }
        else
        {
            $this->action_successful('reporting', Translation :: get('ReportingSuccessfullyDeleted'));
        }
        
        // Remove tracking
        if (! $this->remove_tracking())
        {
            return $this->action_failed('tracking', Translation :: get('TrackingDeletionFailed'));
        }
        else
        {
            $this->action_successful('tracking', Translation :: get('TrackingSuccessfullyDeleted'));
        }
        
        // Remove roles and rights
        if (! $this->remove_rights())
        {
            return $this->action_failed('rights', Translation :: get('RightsDeletionFailed'));
        }
        else
        {
            $this->action_successful('rights', Translation :: get('RightsSuccessfullyDeleted'));
        }
        
        // Remove storage units
        if (! $this->remove_storage_units())
        {
            return $this->action_failed('database', Translation :: get('StorageUnitsDeletionFailed'));
        }
        else
        {
            $this->action_successful('database', Translation :: get('StorageUnitsSuccessfullyDeleted'));
        }
        
        // Remove application
        if (! $this->remove_settings())
        {
            return $this->action_failed('config', Translation :: get('CouldNotRemoveApplication'));
        }
        else
        {
            return $this->action_successful('config', Translation :: get('ApplicationRemoveProcessed'));
        }
    }

    /**
     * Removes the webservices
     * 
     * @return boolean
     */
    function remove_webservices()
    {
        $registration = $this->get_registration();
        
        $wdm = WebserviceDataManager :: get_instance();
        $condition = new EqualityCondition(WebserviceRegistration :: PROPERTY_NAME, $registration->get_name());
        $webservices = $wdm->retrieve_webservices($condition);
        
        while ($webservice = $webservices->next_result())
        {
            $message = Translation :: get('RemovingWebserviceRegistration') . ': ' . $webservice->get_name();
            $this->add_message($message);
            if (! $webservice->delete())
            {
                return false;
            }
        }
        
        // TODO: Delete categories added by the application
        
        return true;
    }

    /**
     * Removes the reporting
     * 
     * @return boolean
     */
    function remove_reporting()
    {
        $registration = $this->get_registration();
        
        $rdm = ReportingDataManager :: get_instance();
        
        $this->add_message(Translation :: get('RemovingReportingblocks'));
        $condition = new EqualityCondition(ReportingBlockRegistration :: PROPERTY_APPLICATION, $registration->get_name());
        
        if (! $rdm->delete_reporting_block_registrations($condition))
        {
            return false;
        }
        
        $this->add_message(Translation :: get('RemovingReportingTemplates'));
        $condition = new EqualityCondition(ReportingTemplateRegistration :: PROPERTY_APPLICATION, $registration->get_name());
        
        if (! $rdm->delete_reporting_template_registrations($condition))
        {
            return false;
        }
        
        if (! $rdm->delete_orphaned_block_template_relations())
        {
            $this->add_message(Translation :: get('DeletingOrphanedBlockTemplateRelationsFailed'), self :: TYPE_WARNING);
        }
        else
        {
            $this->add_message(Translation :: get('DeletingOrphanedBlockTemplateRelations'));
        }
        
        return true;
    }

    /**
     * Removes the tracking
     * 
     * @return boolean
     */
    function remove_tracking()
    {
        $registration = $this->get_registration();
        $base_path = Path :: get_application_path() . 'lib/' . $registration->get_name() . '/trackers/tracker_tables/';
        
        $database = new Mdb2Database();
        $database->set_prefix('tracking_');
        
        if (is_dir($base_path))
        {
            $files = Filesystem :: get_directory_content($base_path, Filesystem :: LIST_FILES);
            
            if (count($files) > 0)
            {
                foreach ($files as $file)
                {
                    if ((substr($file, - 3) == 'xml'))
                    {
                        $doc = new DOMDocument();
                        $doc->load($file);
                        $object = $doc->getElementsByTagname('object')->item(0);
                        $name = $object->getAttribute('name');
                        
                        $this->add_message(Translation :: get('DroppingTrackingStorageUnit') . ': <em>' . $name . '</em>');
                        
                        if (! $database->drop_storage_unit($name))
                        {
                            return false;
                        }
                    }
                }
            }
        }
        
        $condition = new EqualityCondition(Event :: PROPERTY_BLOCK, $registration->get_name());
        $tdm = TrackingDataManager :: get_instance();
        $this->add_message(Translation :: get('DeletingApplicationEvents'));
        if (! $tdm->delete_events($condition))
        {
            return false;
        }
        
        $condition = new EqualityCondition(TrackerRegistration :: PROPERTY_APPLICATION, $registration->get_name());
        $this->add_message(Translation :: get('DeletingApplicationTrackerRegistrations'));
        if (! $tdm->delete_tracker_registrations($condition))
        {
            return false;
        }
        
        if (! $tdm->delete_orphaned_event_rel_tracker())
        {
            $this->add_message(Translation :: get('DeletingOrphanedEventRelTrackersFailed'), self :: TYPE_WARNING);
        }
        
        return true;
    }

    /**
     * Removes the rights
     * 
     * @return boolean
     */
    function remove_rights()
    {
        $registration = $this->get_registration();
        $rdm = RightsDataManager :: get_instance();
        $condition = new EqualityCondition(Location :: PROPERTY_APPLICATION, $registration->get_name());
        $this->add_message(Translation :: get('DeletingApplicationLocations'));
        if (! $rdm->delete_locations($condition))
        {
            return false;
        }
        else
        {
            if (! $rdm->delete_orphaned_rights_template_right_locations())
            {
                $this->add_message(Translation :: get('DeletingOrphanedRoleRightLocationsFailed'), self :: TYPE_WARNING);
            }
            
            return true;
        }
    }

    /**
     * Removes the settings
     * 
     * @return boolean
     */
    function remove_settings()
    {
        $registration = $this->get_registration();
        $adm = AdminDataManager :: get_instance();
        $condition = new EqualityCondition(Setting :: PROPERTY_APPLICATION, $registration->get_name());
        
        if (CommonDataManager :: deletes(Setting :: class_name(), $condition))
        {
            $this->add_message(Translation :: get('RemovedApplicationSettings'));
            return true;
        }
        
        $this->add_message(Translation :: get('CouldNotRemoveApplicationSettings'), MessageLogger :: TYPE_ERROR);
        
        return false;
    }

    /**
     * Removes the storage units
     * 
     * @return boolean
     */
    function remove_storage_units()
    {
        $registration = $this->get_registration();
        $database = new Mdb2Database();
        $database->set_prefix($registration->get_name() . '_');
        
        $path = WebApplication :: get_application_class_path($registration->get_name()) . 'install/';
        $files = Filesystem :: get_directory_content($path, Filesystem :: LIST_FILES);
        
        foreach ($files as $file)
        {
            if ((substr($file, - 3) == 'xml'))
            {
                $doc = new DOMDocument();
                $doc->load($file);
                $object = $doc->getElementsByTagname('object')->item(0);
                $name = $object->getAttribute('name');
                
                $this->add_message(Translation :: get('DroppingStorageUnit') . ': <em>' . $name . '</em>');
                
                if (! $database->drop_storage_unit($name))
                {
                    return false;
                }
            }
        }
        
        return true;
    }

    /**
     * Returns the path to the package folder
     * 
     * @return String or null if nothing has to be removed
     */
    function get_package_folder_path()
    {
        return $path = Path :: get_application_path() . $this->get_registration()->get_name() . '/';
    }
}
?>