<?php
namespace admin;

use common\libraries\Translation;
use common\libraries\Utilities;

use Exception;

/**
 * Abstract class that describes the source of a package. Holds the package information.
 * 
 * @author Hans De Bisschop - Erasmus Hogeschool Brussel
 * @author Magali Gillard - Erasmus Hogeschool Brussel
 * @author Sven Vanpoucke - Hogeschool Gent - Cleanup, code refactoring and bugfixes, comments
 * @package admin.lib.package_source
 */

abstract class PackageSource
{
    /**
     * The parent object
     * 
     * @var PackageAction
     */
    private $parent;
    
    /**
     * The package attributes
     * 
     * @var String[]
     */
    private $attributes;

    /**
     * Constructor
     * 
     * @param $parent PackageAction
     */
    function __construct($parent)
    {
        $this->set_parent($parent);
    }

    /**
     * Invokes the constructor of the class that corresponds to the specified type of package source. @parent
     * PackageAction $parent @parent String $type
     */
    static function factory($parent, $type)
    {
        if (! $parent instanceof PackageSourceAction)
        {
            throw new Exception(Translation :: get('ParentMustBePackageSourceAction'));
        }
        
        $class = __NAMESPACE__ . '\Package' . Utilities :: underscores_to_camelcase($type) . 'Source';
        
        if (! class_exists($class))
        {
            throw new Exception(Translation :: get('CouldNotFindPackageSource', array('TYPE' => $type)));
        }
        
        return new $class($parent);
    }

    /**
     * Return the parent object
     * 
     * @return PackageAction
     */
    function get_parent()
    {
        return $this->parent;
    }

    /**
     * Sets the parent object
     * 
     * @param $parent PackageAction
     */
    function set_parent($parent)
    {
        $this->parent = $parent;
    }

    /**
     * Returns the attributes, if the attributes are not available, retrieve them from the package
     * 
     * @return String[]
     */
    function get_attributes()
    {
        if (! isset($this->attributes))
        {
            $this->attributes = $this->retrieve_attributes();
        }
        
        return $this->attributes;
    }

    /**
     * Sets the attributes
     * 
     * @param $attributes String[]
     */
    function set_attributes($attributes)
    {
        $this->attributes = $attributes;
    }

    /**
     * Delegation functions
     */
    function add_message($message, $type = PackageAction :: TYPE_NORMAL)
    {
        $this->get_parent()->add_message($message, $type);
    }

    /**
     * Delegates the function to add_message because we don't want the entire process to fail
     * 
     * @param $message String
     */
    function action_failed($message)
    {
        $this->add_message($message, PackageAction :: TYPE_ERROR);
        return false;
    }

    /**
     * Abstract functions
     */
    
    /**
     * Dummy function to process the source Standard function returns true
     */
    function process()
    {
        return true;
    }

    /**
     * Dummy function to revert the source process Some sources will work with temporary files wich need to be removed
     * in case of failure
     */
    function revert()
    {
        return true;
    }

    /**
     * Abstract function to retrieve the attributes
     */
    abstract function retrieve_attributes();

}
?>