<?php
namespace admin;

use common\libraries\Path;
use common\libraries\Translation;
use common\libraries\Request;
use common\libraries\PlatformSetting;

/**
 * Package Source file that can retrieve packages from an external source. The packages need to be synchronized before
 * they can be retrieved from the remote location. The attributes must already be stored in the RemotePackage Table from
 * the admin application
 * 
 * @author Hans De Bisschop - Erasmus Hogeschool Brussel
 * @author Magali Gillard - Erasmus Hogeschool Brussel
 * @author Sven Vanpoucke - Hogeschool Gent - Cleanup, code refactoring and bugfixes, comments
 * @package admin.lib.package_source.source
 */

class PackageRemoteSource extends PackageZippedSource
{

    /**
     * Retrieves the attributes
     */
    function retrieve_attributes()
    {
        list($package_section, $package_code) = $this->get_parent()->get_package_type_and_name();
        
        return AdminDataManager :: retrieve_remote_package_by_context($this->get_parent()->get_package_type_and_name());
    }

    /**
     * Retrieves the archive from the remote location
     * 
     * @return string - The path to the temporary folder
     */
    function get_archive()
    {
        $remote_file = $this->get_attributes()->get_filename();
        $online_repository = PlatformSetting :: get('package_repository');
        $remote_path = $online_repository . 'application/package/files/' . $remote_file;
        
        $local_hash = md5(time() . $remote_path);
        $local_file = $local_hash . '.zip';
        $local_path = Path :: get(SYS_TEMP_PATH) . $local_file;
        
        $remote_handle = fopen($remote_path, 'r');
        $local_handle = fopen($local_path, 'w+b');
        
        if ($remote_handle)
        {
            $this->add_message(Translation :: get('RemotePackageFound'));
            while ($line = fread($remote_handle, 1024))
            {
                fwrite($local_handle, $line);
            }
            
            fclose($remote_handle);
            fclose($local_handle);
        }
        else
        {
            return $this->action_failed(Translation :: get('RemotePackageNotFound'));
        }
        
        if (! $this->verify_hashes($local_path))
        {
            return $this->action_failed(Translation :: get('RemotePackageHashFail'));
        }
        else
        {
            $this->add_message(Translation :: get('RemotePackageHashVerified'));
        }
        
        return $local_path;
    }

    /**
     * Verifies the given hashes
     * 
     * @param $local_path String - Path to the file
     * @return boolean
     */
    private function verify_hashes($local_path)
    {
        $attributes = $this->get_attributes();
        
        if (md5_file($local_path) !== $attributes->get_md5())
        {
            return false;
        }
        
        if (sha1_file($local_path) !== $attributes->get_sha1())
        {
            return false;
        }
        
        if (hash_file('sha256', $local_path) !== $attributes->get_sha256())
        {
            return false;
        }
        
        if (hash_file('sha512', $local_path) !== $attributes->get_sha512())
        {
            return false;
        }
        
        return true;
    }
}
?>