<?php
namespace admin;

use common\libraries\Filecompression;
use common\libraries\Filesystem;
use common\libraries\Utilities;
use common\libraries\Path;
use common\libraries\Translation;
use common\libraries\Text;

/**
 * Abstract class that describeds a zipped package which needs to be moved and extracted
 * 
 * @author Sven Vanpoucke - Hogeschool Gent
 * @package admin.lib.package_source.source
 */

abstract class PackageZippedSource extends PackageSource
{
    /**
     * The path to the package zip file
     * 
     * @var String
     */
    private $package_file;
    
    /**
     * The path to the package backup file
     * 
     * @var String
     */
    private $package_backup_file;
    
    /**
     * The path to the package folder
     * 
     * @var String
     */
    private $package_folder;

    /**
     * Constructor
     * 
     * @param $parent PackageAction
     */
    function __construct($parent)
    {
        parent :: __construct($parent);
    }

    /**
     * Processes the source Retrieving zip file, unpacking zip file and moving the folder to the correct path (with use
     * of the context attribute of the package.info)
     * 
     * @return boolean
     */
    function process()
    {
        $package_folder = Path :: namespace_to_full_path($this->get_attributes()->get_context());
        $this->set_package_folder($package_folder);
        
        if (! $this->backup_package_files() || ! $this->remove_package_files())
        {
            return false;
        }
        
        $package_file = $this->get_archive();
        if (! $package_file)
        {
            return $this->action_failed(Translation :: get('PackageNotRetrieved'));
        }
        
        $this->set_package_file($package_file);
        $this->add_message(Translation :: get('PackageRetrieved'));
        $this->add_message();
        
        if (! $this->extract_archive_to_folder($package_file, $package_folder))
        {
            return false;
        }
        
        $this->cleanup();
        
        return true;
    }

    /**
     * Makes a backup of the files before the package source replaces the files
     * 
     * @return boolean
     */
    private function backup_package_files()
    {
        $compression = Filecompression :: factory();
        $compression->set_filename('package_backup_' . $this->get_attributes()->get_code(), 'zip');
        $archive_path = $compression->create_archive($this->get_package_folder());
        
        if (! $archive_path)
        {
            return $this->action_failed(Translation :: get('PackageFilesNotBackupped'));
        }
        
        $this->set_package_backup_file($archive_path);
        
        $this->add_message(Translation :: get('PackageFilesBackupped'));
        $this->add_message();
        
        return true;
    }

    /**
     * Removes the files in the php and resources folder of this package
     * 
     * @return boolean
     */
    private function remove_package_files()
    {
        $paths_to_remove = array('php', 'resources');
        $success = true;
        
        foreach ($paths_to_remove as $path_to_remove)
        {
            $full_path = $this->get_package_folder() . $path_to_remove;
            
            $success = $this->remove_path($full_path, Translation :: get('PackageFolderRemoved', array(
                    'FOLDER' => $path_to_remove)), Translation :: get('PackageFolderNotRemoved', array(
                    'FOLDER' => $path_to_remove)), Translation :: get('PackageFolderNotFound', array(
                    'FOLDER' => $path_to_remove)));
        
        }
        
        if (! $success)
        {
            return $this->action_failed(Translation :: get('PackageFilesNotRemoved'));
        }
        
        $this->add_message();
        
        return true;
    }

    /**
     * Extract a given archive and moves the content to the given folder
     * 
     * @param $archive_file String
     * @param $target_folder String
     *
     * @return boolean
     */
    private function extract_archive_to_folder($archive_file, $target_folder)
    {
        $extract_path = Filecompression :: factory()->extract_file($archive_file);
        if (! Text :: is_valid_path($extract_path))
        {
            return $this->action_failed(Translation :: get('PackageNotExtracted'));
        }
        
        $this->add_message(Translation :: get('PackageExtracted'));
        
        if (! Filesystem :: recurse_move($extract_path, $target_folder, true))
        {
            return $this->action_failed(Translation :: get('PackageMoveFailed'));
        }
        
        $this->add_message(Translation :: get('PackageMovedSucessfully'));
        
        return true;
    }

    /**
     * Removes the temporary files and folders
     * 
     * @return boolean
     */
    function cleanup()
    {
        $succes = $this->remove_path($this->get_package_file(), Translation :: get('RemovedZipFile'), Translation :: get('ZipfileNotRemoved'), Translation :: get('ZipFileNotFound'));
        
        $this->set_package_file(null);
        
        return $succes;
    }

    /**
     * Removes the package folder when an error occurs
     * 
     * @return boolean
     */
    function revert()
    {
        $succes = $this->cleanup();
        
        if (! $this->remove_package_files() || ! $this->extract_archive_to_folder($this->package_backup_file, $this->package_folder))
        {
            return false;
        }
        
        return $succes;
    }

    /**
     * Removes a file or a folder
     * 
     * @param $path String
     * @param $path_remove_translation String
     * @param $path_not_remove_translation String
     * @param $path_not_found_translation String
     */
    function remove_path($path, $path_remove_translation, $path_not_remove_translation, $path_not_found_translation)
    {
        if (Text :: is_valid_path($path) && file_exists($path))
        {
            if (Filesystem :: remove($path))
            {
                $this->add_message($path_remove_translation);
            }
            else
            {
                return $this->action_failed($path_not_remove_translation);
            }
        }
        else
        {
            $this->add_message($path_not_found_translation);
        }
        
        return true;
    }

    /**
     * Getters and setters
     */
    
    /**
     * Returns the path to the package zip file
     * 
     * @return String
     */
    function get_package_file()
    {
        return $this->package_file;
    }

    /**
     * Sets the path to the package zip file
     * 
     * @param $package_file String
     */
    function set_package_file($package_file)
    {
        $this->package_file = $package_file;
    }

    /**
     * Returns the path to the package backup zip file
     * 
     * @return String
     */
    function get_package_backup_file()
    {
        return $this->package_backup_file;
    }

    /**
     * Sets the path to the package backup zip file
     * 
     * @param $package_backup_file String
     */
    function set_package_backup_file($package_backup_file)
    {
        $this->package_backup_file = $package_backup_file;
    }

    /**
     * Returns the path to the package folder
     * 
     * @return String
     */
    function get_package_folder()
    {
        return $this->package_folder;
    }

    /**
     * Sets the path to the package folder
     * 
     * @param $package_folder Strings
     */
    function set_package_folder($package_folder)
    {
        $this->package_folder = $package_folder;
    }

    /**
     * Abstract functionality
     */
    
    /**
     * Retrieves the zip file
     */
    abstract function get_archive();
}
?>