<?php
namespace admin;

use common\libraries\Registration;

use common\libraries\Translation;
use common\libraries\Path;

use Exception;

/**
 * Package Update
 * 
 * @author Hans De Bisschop - Erasmus Hogeschool Brussel
 * @author Magali Gillard - Erasmus Hogeschool Brussel
 * @author Sven Vanpoucke - Hogeschool Gent - Cleanup, code refactoring and bugfixes, comments
 * @package admin.lib.package_updater
 */
class PackageUpdater extends PackageSourceAction
{
    /**
     * The package registration
     * 
     * @var Registration
     */
    private $registration;

    /**
     * Constructor Intializes the registration
     * 
     * @param $registration Registration
     * @param $source String
     */
    function __construct(Registration $registration, $source)
    {
        parent :: __construct($source);
        
        if (! $registration instanceof Registration)
        {
            throw new Exception(Translation :: get('ValidRegistrationRequired'));
        }
        
        $this->registration = $registration;
    }

    /**
     * Runs the package updater
     * 
     * @return boolean
     */
    function run()
    {
        if (parent :: run())
        {
            return $this->action_successful('finished', Translation :: get('PackageCompletelyUpdated'));
        }
        else
        {
            return $this->action_failed('failed', Translation :: get('PackageUpdateFailed'));
        }
    }

    /**
     * Initializes the package installer
     * 
     * @return boolean
     */
    function initialize()
    {
        if ($this->registration->deactivate(true))
        {
            $this->add_message(Translation :: get('PackageDeactivated'));
        }
        else
        {
            return $this->action_failed('initialization', Translation :: get('PackageDeactivationFailed'));
        }
        
        if (! parent :: initialize())
        {
            return false;
        }
        
        return $this->action_successful('initialization', Translation :: get('PackageInstallInitialized'));
    }

    /**
     * Executes the updatable and installable check on the package dependency verifier
     * 
     * @param $package_dependency_verifier PackageDependencyVerifier
     *
     * @return boolean
     */
    function execute_dependency_check(PackageDependencyVerifier $package_dependency_verifier)
    {
        $success_update = $package_dependency_verifier->is_updatable();
        $this->add_message($package_dependency_verifier->get_logger()->render());
        
        if (! $success_update)
        {
            return false;
        }
        
        $this->add_message();
        
        return $success_update && $package_dependency_verifier->is_installable();
    }

    /**
     * Updates the package
     * 
     * @return boolean
     */
    function process()
    {
        $package = PackageUpdaterType :: factory($this, $this->get_attributes()->get_section());
        if (! $package->update())
        {
            return false;
        }
        
        $this->update_registration_version();
        
        if (! $this->registration->activate(true))
        {
            return $this->action_failed('config', Translation :: get('PackageActivationFailed'));
        }
        
        $this->add_message(Translation :: get('PackageActivated'));
        return $this->action_successful('config', Translation :: get('PackageConfigurationSuccess'));
    }

    /**
     * Returns the name and type of the package
     * 
     * @param String[]
     */
    function get_package_type_and_name()
    {
        return $this->registration->get_context();
    }

    /**
     * Helper functionality
     */
    
    /**
     * Updates the registration to the new version
     * 
     * @param $new_version String
     *
     * @return boolean
     */
    public function update_registration_version($new_version)
    {
        $registration = $this->get_registration();
        
        $version = $new_version ? $new_version : $this->get_attributes()->get_version();
        $registration->set_version($new_version);
        
        $succes = $registration->update();
        $this->add_message(Translation :: get(($succes ? 'VersionSet' : 'VersionNotSet')));
        
        return $success;
    }

    /**
     * Getters and setters
     */
    
    /**
     * Sets the registration of the package
     * 
     * @return Registration $registration
     */
    public function get_registration()
    {
        return $this->registration;
    }

    /**
     * Sets the registration of the package
     * 
     * @param $registration Registration
     */
    public function set_registration($registration)
    {
        $this->registration = $registration;
    }
}
?>