<?php
namespace admin;

use common\libraries\Utilities;
use common\libraries\Path;
use common\libraries\Translation;
use common\libraries\Filesystem;

use Exception;

/**
 * Abstract class that describes the update process for packages. Must be extended by the available package types.
 * 
 * @author Hans De Bisschop - Erasmus Hogeschool Brussel
 * @author Magali Gillard - Erasmus Hogeschool Brussel
 * @author Sven Vanpoucke - Hogeschool Gent - Cleanup, code refactoring and bugfixes, comments
 */
abstract class PackageUpdaterType
{
    /**
     * The parent updater
     * 
     * @var PackageUpdater
     */
    private $parent;

    /**
     * Constructor
     * 
     * @param $parent PackageUpdater
     */
    function __construct($parent)
    {
        $this->set_parent($parent);
    }

    /**
     * Invokes the constructor of the class that corresponds to the specified type of package updater type.
     * 
     * @param $parent PackageUpdater
     * @param $type String
     */
    static function factory($parent, $type)
    {
        if (! $parent instanceof PackageUpdater)
        {
            throw new Exception(Translation :: get('ParentMustBePackageUpdater'));
        }
        
        $class = __NAMESPACE__ . '\PackageUpdater' . Utilities :: underscores_to_camelcase($type) . 'Type';
        
        if (! file_exists($class))
        {
            throw new Exception(Translation :: get('CouldNotFindPackageUpdaterType', array('TYPE' => $type)));
        }
        
        return new $class($parent);
    }

    /**
     * Update script. Searches for the update folders, filters the invalid folders and execute the update scripts
     * 
     * @return boolean
     */
    function update()
    {
        $code = $this->get_attributes()->get_code();
        $section = $this->get_attributes()->get_section();
        
        /**
         * Searches for folders
         */
        $folders = array();
        
        $path = $this->get_updater_folder_path($code);
        
        if (file_exists($path))
        {
            $folders = Filesystem :: get_directory_content($path, Filesystem :: LIST_DIRECTORIES, false);
        }
        
        if (count($folders) == 0)
        {
            $this->add_message(Translation :: get('NoUpdateFoldersFound'));
            $this->action_successful('processing', Translation :: get('FinishedUpgrade'));
        }
        
        /**
         * Validates folders
         */
        $update_folders = array();
        foreach ($folders as $folder)
        {
            if (version_compare($folder, $this->get_registration()->get_version(), '>') && version_compare(PackageInfo :: factory($section, $code)->get_package()->get_version(), $folder, '>='))
            {
                
                $update_folders[] = $folder;
            }
        }
        
        /**
         * Executes the update folder scripts
         */
        foreach ($update_folders as $update_folder)
        {
            $this->add_message();
            $this->add_message(Translation :: get('UpdateToVersion', array('VERSION' => $update_folder)));
            $this->add_message();
            
            if (! $this->update_folder($update_folder))
            {
                return false;
            }
            
            $this->update_registration_version($update_folder);
        }
        
        return true;
    }

    /**
     * Getters and setters
     */
    
    /**
     * Returns the parent PackageUpdater
     * 
     * @return PackageUpdater
     */
    function get_parent()
    {
        return $this->parent;
    }

    /**
     * Sets the parent PackageUpdater
     * 
     * @param $parent PackageUpdater
     */
    function set_parent($parent)
    {
        $this->parent = $parent;
    }

    /**
     * Delegation functions
     */
    
    /**
     * Adds a message to the message queue
     * 
     * @param $message String
     * @param $type String
     */
    function add_message($message, $type = PackageAction :: TYPE_NORMAL)
    {
        $this->get_parent()->add_message($message, $type);
    }

    /**
     * Sets the current block to failed
     * 
     * @param $type String
     * @param $message String
     *
     * @return false
     */
    function action_failed($type, $message)
    {
        return $this->get_parent()->action_failed($type, $message);
    }

    /**
     * Sets the current block to success
     * 
     * @param $type String
     * @param $message String
     *
     * @return true
     */
    function action_successful($type, $message)
    {
        return $this->get_parent()->action_successful($type, $message);
    }

    /**
     * Returns the package source
     * 
     * @return PackageSource
     */
    function get_source()
    {
        return $this->get_parent()->get_package_source();
    }

    /**
     * Returns the attributes of the package
     */
    function get_attributes()
    {
        return $this->get_parent()->get_attributes();
    }

    /**
     * Updates the registration to the new version
     * 
     * @param $new_version String
     *
     * @return boolean
     */
    function update_registration_version($new_version)
    {
        return $this->get_parent()->update_registration_version($new_version);
    }

    /**
     * Abstract functions
     */
    
    /**
     * Executes the update script from an update folder
     * 
     * @param $code String - the code of the package
     * @param $folder String - the update folder
     * @return boolean
     */
    abstract function update_folder($code, $folder);

    /**
     * Retrieves the path to the update folder
     * 
     * @param $code String - the code of the package
     * @return String
     */
    abstract function get_updater_folder_path($code);

}
?>