<?php

namespace application\personal_messenger;

use common\libraries\AjaxManager;
use common\libraries\DatetimeUtilities;
use common\libraries\JsonAjaxResult;
use common\libraries\Redirect;
use common\libraries\Request;
use common\libraries\Theme;
use common\libraries\Translation;
use common\libraries\Utilities;

/** * ***************************************************************************
 * This class handles the AJAX call for the retrieval of new replies on the 
 * discussion identified by the given id.
 *
 * @name Chamilo Personal Messenger > PersonalMessengerAjaxRetrieveNewReplies
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * **************************************************************************** */
class PersonalMessengerAjaxRetrieveNewReplies extends AjaxManager
{
    // input parameters
    const PARAM_MESSAGE_IDENTIFIER = 'message_id';
    const PARAM_LAST_RETRIEVED = 'last_retrieved';

    // global properties to return
    const PROPERTY_REPLIES = 'replies';
    const PROPERTY_ATTACHMENT_PREFIX = 'attachment_prefix';
    const PROPERTY_ATTACHMENT_SUFFIX = 'attachment_suffix';
    const PROPERTY_ATTACHMENT_SEPARATOR = 'attachment_separator';
    const PROPERTY_ATTACHMENT_IMG = 'attachment_image';
    const PROPERTY_LAST_RETRIEVED = 'last_retrieved';
    const PROPERTY_REPLY_BACKGROUND_IMG_STYLE = 'background_img_style';

    // replies consist of
    const PROPERTY_REPLY_AUTHOR = 'author';
    const PROPERTY_REPLY_TITLE = 'title';
    const PROPERTY_REPLY_CONTENT = 'content';
    const PROPERTY_REPLY_ATTACHMENTS = 'attachments';
    const PROPERTY_REPLY_MODIFICATION = 'modification';

    // attachments consist of
    const PROPERTY_REPLY_ATTACHMENT_URL = 'url';
    const PROPERTY_REPLY_ATTACHMENT_TITLE = 'title';
    const PROPERTY_REPLY_ATTACHMENT_TYPE = 'type';

    // required parameters
    function required_parameters()
    {
        return array(self :: PARAM_MESSAGE_IDENTIFIER, self :: PARAM_LAST_RETRIEVED);
    }

    // run
    function run()
    {
        // get replies
        $replies = PersonalMessengerDataManager :: retrieve_unread_replies(
                        $this->get_parameter(self :: PARAM_MESSAGE_IDENTIFIER), $this->get_parameter(self :: PARAM_LAST_RETRIEVED)
        );

        // save checktime
        $retrieved = time();

        // create replies
        $replies_array = array();
        while ($reply = $replies->next_result())
        {
            // each reply consists of an array containing individual properties
            $reply_array = array();

            // author
            $reply_array[self :: PROPERTY_REPLY_AUTHOR] = $reply->get_author_name();

            // title
            $reply_array[self :: PROPERTY_REPLY_TITLE] = $reply->get_title();

            // content
            $reply_array[self :: PROPERTY_REPLY_CONTENT] = $reply->get_content();

            // create attachments
            $reply_attachments = array();
            if ($reply->count_attachments() > 0)
            {
                foreach ($reply->get_message_attachments() as $reply_attachment)
                {
                    // get the content object and process it if it exists
                    $content_object = $reply_attachment->retieve_attachment_content_object();
                    if ($content_object)
                    {
                        // each attachment consists of an array containing individual properties
                        $reply_attachment = array();

                        // viewing url
                        $url = $this->get_attachment_viewing_link(
                                        $reply->get_id(), $content_object->get_id(), PersonalMessengerManagerViewerComponent :: ATTACHMENT_POPUP
                        );

                        // adjust link if popup
                        if (PersonalMessengerManagerViewerComponent :: ATTACHMENT_POPUP)
                        {
                            $url = 'javascript:openPopup(\'' . $url . '\'); return false;';
                            $reply_attachment[self :: PROPERTY_REPLY_ATTACHMENT_URL] = '<a href="#" onClick="' . $url . '">';
                        }
                        else
                        {
                            $reply_attachment[self :: PROPERTY_REPLY_ATTACHMENT_URL] = '<a href="' . $url . '">';
                        }

                        // title
                        $reply_attachment[self :: PROPERTY_REPLY_ATTACHMENT_TITLE] = $content_object->get_title();

                        // type
                        $reply_attachment[self :: PROPERTY_REPLY_ATTACHMENT_TYPE] = $content_object->get_type();

                        // add attachment
                        $reply_attachments[] = $reply_attachment;
                    }
                }
            }

            // add attachments
            $reply_array[self :: PROPERTY_REPLY_ATTACHMENTS] = $reply_attachments;

            // created - modified
            $creation = $reply->get_creation_time();
            $modification = $reply->get_modification_time();

            $mod = Translation :: get('CreatedOn') . ': ';
            $mod .= DatetimeUtilities :: format_locale_date(
                            Translation :: get('DateFormatShort', null, Utilities :: COMMON_LIBRARIES) .
                            ', ' . Translation :: get('TimeNoSecFormat', null, Utilities :: COMMON_LIBRARIES), $creation
            );
            if ($creation != $modification)
            {
                $mod .= '<br/>';
                $mod .= Translation :: get('LastModified') . ': ';
                $mod .= DatetimeUtilities :: format_locale_date(
                                Translation :: get('DateFormatShort', null, Utilities :: COMMON_LIBRARIES) .
                                ', ' . Translation :: get('TimeNoSecFormat', null, Utilities :: COMMON_LIBRARIES), $modification
                );
            }
            $reply_array[self :: PROPERTY_REPLY_MODIFICATION] = $mod;

            $replies_array[] = $reply_array;
        }

        // construct result
        $result = new JsonAjaxResult(200);

        // set properties
        $result->set_property(self :: PROPERTY_REPLIES, $replies_array);
        $result->set_property(self :: PROPERTY_LAST_RETRIEVED, $retrieved);
        $result->set_property(self :: PROPERTY_ATTACHMENT_IMG, Theme :: get_image_path() . 'has_attachments.png');
        $result->set_property(self :: PROPERTY_REPLY_BACKGROUND_IMG_STYLE, 'url(' . Theme :: get_image_path() . 'view_reply.png)');
        $result->set_property(self :: PROPERTY_ATTACHMENT_PREFIX, PersonalMessengerManagerViewerComponent :: ATTACHMENT_PREFIX);
        $result->set_property(self :: PROPERTY_ATTACHMENT_SUFFIX, PersonalMessengerManagerViewerComponent :: ATTACHMENT_SUFFIX);
        $result->set_property(self :: PROPERTY_ATTACHMENT_SEPARATOR, PersonalMessengerManagerViewerComponent :: ATTACHMENT_SEPARATOR);

        // update usermessage
        if (!PersonalMessengerDataManager :: read_my_message($this->get_parameter(self :: PARAM_MESSAGE_IDENTIFIER)))
        {
            // failure
            $result->set_result_code(500);
        }

        // return result
        $result->display();
    }

    /**
     * Returns a link to the viewer of the attachment.
     * @param int $message_id
     * @param int $content_object_id
     * @param boolean $popup
     * @return String The link to the viewer of the message
     */
    function get_attachment_viewing_link($message_id, $content_object_id, $popup)
    {
        // construct viewing parameters
        $params = array();
        $params[PersonalMessengerManager :: PARAM_ACTION] = PersonalMessengerManager :: ACTION_VIEW_ATTACHMENT;
        $params[PersonalMessengerManager :: PARAM_MESSAGE_ID] = $message_id;
        $params[PersonalMessengerManager :: PARAM_ATTACHMENT_ID] = $content_object_id;
        $params[PersonalMessengerManager :: PARAM_POPUP] = $popup;

        // return a link for the Personal Messenger application, with the given parameters
        return Redirect :: get_link(PersonalMessengerManager :: APPLICATION_NAME, $params, array(), false, Redirect :: TYPE_APPLICATION);
    }

}

?>