<?php

namespace application\personal_messenger;

use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Path;
use common\libraries\Session;
use common\libraries\Theme;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\extensions\category_manager\PlatformCategory;

/** * ***************************************************************************
 * This class represents a user defined category (CAT).
 *
 * CAT objects have a number of default properties:
 * - id (inherited): the numeric ID of the CAT;
 * - parent_id (inherited): the numeric ID identifying the parent of the CAT;
 * - user_id: the ID of the owner of the CAT;
 * - folder: enum specifies the folder in which the CAT is created;
 * - name (inherited): the name of the user defined CAT;
 * - display_order (inherited): the display order of the CAT;
 *
 * @name Chamilo Personal Messenger > UserCategory
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * **************************************************************************** */
class UserCategory extends PlatformCategory
{
    //**************************************************************************
    // GENERAL CONSTANTS
    //**************************************************************************
    const CLASS_NAME = __CLASS__;
    //const TABLE_NAME = 'user_category';
    //**************************************************************************
    // PROPERTY CONSTANTS
    //**************************************************************************
    const PROPERTY_USER_CATEGORY_USER_ID = 'user_id';
    const PROPERTY_USER_CATEGORY_FOLDER = 'folder';

    //**************************************************************************
    // GENERAL GETTERS
    //**************************************************************************
    /**
     * Get the default properties of all categories.
     * @return array The property names
     */
    static function get_default_property_names()
    {
        return parent :: get_default_property_names(array(
            self :: PROPERTY_USER_CATEGORY_USER_ID,
            self :: PROPERTY_USER_CATEGORY_FOLDER));
    }

    /**
     * inherited
     */
    function get_data_manager()
    {
        return PersonalMessengerDataManager :: get_instance();
    }

    //**************************************************************************
    // PROPERTY GETTERS
    //**************************************************************************
    /**
     * Returns the numeric ID of the owner of this CAT.
     * @return int The user ID
     */
    function get_user_id()
    {
        return $this->get_default_property(self :: PROPERTY_USER_CATEGORY_USER_ID);
    }

    /**
     * Returns the numeric ID of the folder belonging to this CAT.
     * @return int The folder
     */
    function get_folder()
    {
        return $this->get_default_property(self :: PROPERTY_USER_CATEGORY_FOLDER);
    }

    //**************************************************************************
    // PROPERTY SETTERS
    //**************************************************************************
    /**
     * Sets the numeric ID of the owner of this CAT.
     * @param int $user_id The user ID
     */
    function set_user_id($user_id)
    {
        $this->set_default_property(self :: PROPERTY_USER_CATEGORY_USER_ID, $user_id);
    }

    /**
     * Sets the numeric ID of the folder belonging to this CAT.
     * @param int $folder The folder
     */
    function set_folder($folder)
    {
        $this->set_default_property(self :: PROPERTY_USER_CATEGORY_FOLDER, $folder);
    }

    //**************************************************************************
    // OVERRIDE
    //**************************************************************************
    /**
     * Overrides the get_name of the parent adding the unread message count to the category name.
     * @return string
     */
    function get_name()
    {
        // count messages
        $unread_count = $this->count_unread_messages();

        // get the original name
        $name = parent :: get_name();

        // append if unread messages
        if ($unread_count > 0)
        {
            $name .= ' (' . $unread_count . ')';
        }

        // return full name
        return $name;
    }

    //**************************************************************************
    // DATABASE FUNCTIONS
    //**************************************************************************
    /**
     * Returns the numer of unread messages in this category.
     */
    function count_unread_messages()
    {
        // construct conditions
        $conditions = array();
        $conditions[] = new EqualityCondition(UserMessage :: PROPERTY_USER_MESSAGE_USER_ID, Session :: get_user_id());
        $conditions[] = new EqualityCondition(UserMessage :: PROPERTY_USER_MESSAGE_CATEGORY_ID, $this->get_id());
        $conditions[] = new EqualityCondition(UserMessage :: PROPERTY_USER_MESSAGE_STATUS, UserMessage :: STATUS_UNREAD);
        $condition = new AndCondition($conditions);

        // return count
        return PersonalMessengerDataManager :: get_instance()->count_user_message($condition);
    }

    /**
     * Gets the table name for this class.
     */
    static function get_table_name()
    {
        return Utilities :: get_classname_from_namespace(self :: CLASS_NAME, true);
    }

}

?>