<?php

namespace application\personal_messenger;

use common\extensions\category_manager\CategoryManager;
use common\libraries\AndCondition;
use common\libraries\Application;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\EqualityCondition;
use common\libraries\InCondition;
use common\libraries\Request;
use common\libraries\Session;
use common\libraries\Translation;
use common\libraries\Utilities;

/** * ***************************************************************************
 * This class manages the user defined categories.
 *
 * @name Chamilo Personal Messenger > UserCategoryManager
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * **************************************************************************** */
class UserCategoryManager extends CategoryManager
{

    //**************************************************************************
    // CONSTRUCTOR
    //**************************************************************************
    /**
     * Constructor.
     * @param <type> $parent
     * @param <type> $trail
     * @param <type> $is_subcategories_allowed
     */
    function __construct($parent, $trail = null, $is_subcategories_allowed = true)
    {
        $trail = BreadcrumbTrail :: get_instance();
        $trail->add(new Breadcrumb(
                        $parent->get_url(
                                array(
                                    Application :: PARAM_ACTION => PersonalMessengerManager :: ACTION_MANAGE_CATEGORIES
                                )
                        ),
                        Translation :: get('ManageCategories', null, Utilities::COMMON_LIBRARIES))
        );
        parent :: __construct($parent, $trail, $is_subcategories_allowed);
    }

    //**************************************************************************
    // INHERITED FUNCTIONS
    //**************************************************************************
    /**
     * Override.
     * Returns a UserCategory for the current folder.
     * @return UserCategory 
     */
    function get_category()
    {
        $user_category = new UserCategory();
        $user_category->set_folder($this->get_parent()->get_folder());
        $user_category->set_user_id($this->get_parent()->get_user_id());
        return $user_category;
    }

    /**
     * Inherited from CategoryManager:
     * Counts the number of user defined categories.
     * @param Condition $condition
     * @return int The number of categories
     */
    function count_categories($condition)
    {
        // append condition with folder
        $conditions = array();
        if ($condition)
        {
            $conditions[] = $condition;
        }
        $conditions[] = new EqualityCondition(UserCategory :: PROPERTY_USER_CATEGORY_FOLDER,
                        $this->get_parent()->get_folder()
        );
        $conditions[] = new EqualityCondition(UserCategory :: PROPERTY_USER_CATEGORY_USER_ID,
                        $this->get_parent()->get_user_id()
        );
        $condition = new AndCondition($conditions);

        // count
        return PersonalMessengerDataManager :: get_instance()->count_user_category($condition);
    }

    /**
     * Inherited from CategoryManager:
     * Retrieves all categories that match the given condition.
     * @param Condition $condition
     * @param int $offset
     * @param int $count
     * @param array $order_property
     * @return UserCategoryResultSet
     */
    function retrieve_categories($condition, $offset, $count, $order_property)
    {
        // append condition with folder
        $conditions = array();
        if ($condition)
        {
            $conditions[] = $condition;
        }
        $conditions[] = new EqualityCondition(UserCategory :: PROPERTY_USER_CATEGORY_FOLDER,
                        $this->get_parent()->get_folder()
        );
        $conditions[] = new EqualityCondition(UserCategory :: PROPERTY_USER_CATEGORY_USER_ID,
                        $this->get_parent()->get_user_id()
        );
        $condition = new AndCondition($conditions);

        // retrieve
        $dm = PersonalMessengerDataManager :: get_instance();
        return $dm->retrieve_all_user_categories($condition, $order_property, $offset, $count);
    }

    /**
     * Inherited from CategoryManager:
     * Determines the next display order number based on the given parent id.
     * @param int $parent_id
     * @return int The next display order number
     */
    function get_next_category_display_order($parent_id)
    {
        // construct condition
        $conditions = array();
        $conditions[] = new EqualityCondition(UserCategory :: PROPERTY_PARENT,
                        $parent_id
        );
        $conditions[] = new EqualityCondition(UserCategory :: PROPERTY_USER_CATEGORY_FOLDER,
                        $this->get_parent()->get_folder()
        );
        $conditions[] = new EqualityCondition(UserCategory :: PROPERTY_USER_CATEGORY_USER_ID,
                        $this->get_parent()->get_user_id()
        );
        $condition = new AndCondition($conditions);

        // retrieve
        $dm = PersonalMessengerDataManager :: get_instance();
        return $dm->retrieve_next_sort_value(UserCategory :: get_table_name(),
                UserCategory :: PROPERTY_DISPLAY_ORDER,
                $condition
        );
    }

    /**
     * Returns if the category can be deleted (contains no messages)
     * @param int $category_id
     */
    function allowed_to_delete_category($category_id)
    {
        $dm = PersonalMessengerDataManager :: get_instance();

        // check subcategories
        $condition = new EqualityCondition(UserCategory :: PROPERTY_PARENT, $category_id);
        if ($dm->count_user_category($condition) != 0)
        {
            return false;
        }

        // check messages
        // construct condition
        $conditions = array();
        $conditions[] = new EqualityCondition(UserMessage :: PROPERTY_USER_MESSAGE_USER_ID,
                        $this->get_parent()->get_user_id()
        );
        $conditions[] = new EqualityCondition(UserMessage :: PROPERTY_USER_MESSAGE_CATEGORY_ID,
                        $category_id
        );
        $condition = new AndCondition($conditions);

        // count
        return ($dm->count_user_message($condition) == 0);
    }

    //**************************************************************************
    // STATIC FUNCTIONS
    //**************************************************************************
    /**
     * Returns if the user has categories in the given folder.
     * @return boolean
     */
    static function user_has_categories_in_folder($user_id, $folder)
    {
        // construct condition
        $conditions = array();
        $conditions[] = new EqualityCondition(UserCategory :: PROPERTY_USER_CATEGORY_FOLDER, $folder);
        $conditions[] = new EqualityCondition(UserCategory :: PROPERTY_USER_CATEGORY_USER_ID, $user_id);
        $condition = new AndCondition($conditions);

        // count
        return (PersonalMessengerDataManager :: get_instance()->count_user_category($condition) > 0);
    }

}

?>