<?php
namespace application\personal_messenger;

use common\libraries\FormValidator;
use common\libraries\InCondition;
use common\libraries\Path;
use common\libraries\ResourceManager;
use common\libraries\Request;
use common\libraries\Session;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\WebApplication;
use group\GroupDataManager;
use repository\RepositoryDataManager;
use repository\ContentObject;
use user\UserDataManager;

require_once Path :: get_plugin_path() . 'html2text/class.html2text.inc';

/**
 * * *************************************************************************** This form offers the functionality to
 * create, update and reply on a message. Also: forward a message.
 *
 * @name Chamilo Personal Messenger > MessageForm
 * @version 1.0.0
 * @author Stijn Van Hoecke ****************************************************************************
 */
class MessageForm extends FormValidator
{
    // **************************************************************************
    // CONSTANTS
    // **************************************************************************
    const TYPE_CREATE = 1;
    const TYPE_CREATE_TEMPLATE = 2;
    const TYPE_EDIT = 3;
    const TYPE_REPLY = 4;
    const TYPE_REPLY_ALL = 5;
    const TYPE_FORWARD = 6;

    const RADIO_TYPE = 'type';
    const RADIO_TYPE_PERSONAL_MESSAGE = 'personal_message';
    const RADIO_TYPE_DISCUSSION = 'discussion';
    const CHECK_TEMPLATE = 'save_as_template';

    const SELECT_RECIPIENTS = 'select_recipients';
    const SELECT_ATTACHMENTS = 'select_attachments';

    const SUBARRAY_ELEMENT_FINDER_ELEMENT = 'lo';
    const SUBARRAY_USER_GROUP_FINDER_USER = 'user';
    const SUBARRAY_USER_GROUP_FINDER_GROUP = 'group';

    // **************************************************************************
    // VARIABLES
    // **************************************************************************
    /**
     * The message we will send.
     *
     * @var Message
     */
    private $message;
    /**
     * The manager to get referenced data.
     *
     * @var PersonalMessengerManager
     */
    private $manager;
    /**
     * Flag indicating if we are allowed change the type.
     *
     * @var boolean
     */
    private $allow_type_change;
    /**
     * Flag indicating if we are allowed to change the recipients.
     *
     * @var boolean
     */
    private $allow_recipient_change;

    // **************************************************************************
    // CONSTRUCTOR
    // **************************************************************************
    /**
     * Constructor.
     *
     * @param $form_type int
     * @param $message Message
     * @param $action String
     * @param $user User
     * @param $manager PersonalMessengerManager
     * @param $allow_type_change boolean
     * @param $allow_recipient_change boolean
     */
    function __construct($form_type, $message, $action, $user, $manager, $allow_type_change = true,
            $allow_recipient_change = true)
    {
        parent :: __construct('create_personal_message', 'post', $action);

        $this->form_type = $form_type;
        $this->message = $message;
        $this->manager = $manager;
        $this->allow_type_change = $allow_type_change;
        $this->allow_recipient_change = $allow_recipient_change;

        if ($this->form_type == self :: TYPE_EDIT)
        {
            $this->build_editing_form();
        }
        elseif ($this->form_type == self :: TYPE_FORWARD)
        {
            $this->build_forward_form();
        }
        elseif ($this->form_type == self :: TYPE_CREATE || $this->form_type == self :: TYPE_REPLY || $this->form_type == self :: TYPE_REPLY_ALL)
        {
            $this->build_creation_form();
        }

        $this->setDefaults();
    }

    // **************************************************************************
    // FORM FUNCTIONS
    // **************************************************************************
    /**
     * Builds the editing form.
     */
    function build_editing_form()
    {
        // basic form
        $this->build_basic_form();

        // message options
        $this->build_message_options();

        // update - clear buttons
        $this->build_update_buttons();
    }

    /**
     * Builds the forward form.
     */
    function build_forward_form()
    {
        // extended form
        $this->build_extended_form();

        // send - clear buttons
        $this->build_send_buttons();
    }

    /**
     * Builds the creation form.
     */
    function build_creation_form()
    {
        // extended form
        $this->build_extended_form();

        // message options
        $this->build_message_options();

        // send - clear buttons
        $this->build_send_buttons();
    }

    /**
     * Constructs a basic form.
     */
    function build_basic_form()
    {
        // title field
        $this->add_textfield(Message :: PROPERTY_MESSAGE_TITLE, Translation :: get('Title'), false,
                array("size" => "50"));

        // content HTML editor
        $this->add_html_editor(Message :: PROPERTY_MESSAGE_CONTENT, Translation :: get('Message'), true);
    }

    /**
     * Constructs a basic form, and extends it with additional fields.
     */
    function build_extended_form()
    {
        // basic form
        $this->build_basic_form();

        // attachment selector
        $url = Path :: get(WEB_PATH) . 'repository/php/xml_feed.php';
        $locale = array();
        $locale['Display'] = Translation :: get('AddAttachments');
        $locale['Searching'] = Translation :: get('Searching', null, Utilities :: COMMON_LIBRARIES);
        $locale['NoResults'] = Translation :: get('NoResults', null, Utilities :: COMMON_LIBRARIES);
        $locale['Error'] = Translation :: get('Error', null, Utilities :: COMMON_LIBRARIES);

        $this->addElement('html',
                ResourceManager :: get_instance()->get_resource_html(
                        Path :: get(WEB_PLUGIN_PATH) . 'jquery/uploadify2/swfobject.js'));
        $this->addElement('html',
                ResourceManager :: get_instance()->get_resource_html(
                        Path :: get(WEB_PLUGIN_PATH) . 'jquery/uploadify2/jquery.uploadify.v2.1.0.min.js'));
        $this->addElement('static', 'uploadify',
                Translation :: get('UploadDocument', null, Utilities :: COMMON_LIBRARIES), '<div id="uploadify"></div>');
        $elem = $this->addElement('element_finder', self :: SELECT_ATTACHMENTS,
                Translation :: get('SelectAttachment', null, Utilities :: COMMON_LIBRARIES), $url, $locale, array());
        $elem->setDefaultCollapsed(true);

        $this->addElement('html',
                ResourceManager :: get_instance()->get_resource_html(
                        WebApplication :: get_application_web_path('personal_messenger') . 'resources/javascript/message_form.js'));

        // recipient selector
        if ($this->allow_recipient_change)
        {
            $url = Path :: get(WEB_PATH) . 'common/libraries/php/shared/xml_feeds/xml_user_group_feed.php';
            $locale = array();
            $locale['Display'] = Translation :: get('SelectRecipients');
            $locale['Searching'] = Translation :: get('Searching', null, Utilities :: COMMON_LIBRARIES);
            $locale['NoResults'] = Translation :: get('NoResults', null, Utilities :: COMMON_LIBRARIES);
            $locale['Error'] = Translation :: get('Error', null, Utilities :: COMMON_LIBRARIES);

            $elem = $this->addElement('user_group_finder', self :: SELECT_RECIPIENTS, Translation :: get('Recipients'),
                    $url, $locale, array());
            $elem->excludeElements(array('user_' . Session :: get_user_id()));
            $elem->setDefaultCollapsed(true);
        }
    }

    /**
     * Builds the options to change the message type and create a template.
     */
    function build_message_options()
    {
        // type radios
        if ($this->allow_type_change)
        {
            $type_choices = array();
            $type_choices[] = $this->createElement('radio', self :: RADIO_TYPE, null, Translation :: get('Message'),
                    self :: RADIO_TYPE_PERSONAL_MESSAGE, $params);
            $type_choices[] = $this->createElement('radio', self :: RADIO_TYPE, null, Translation :: get('Discussion'),
                    self :: RADIO_TYPE_DISCUSSION, $params);
            $this->addGroup($type_choices, null, Translation :: get('Type'), '<br />', false);
        }

        // save as template
        $this->addElement('checkbox', self :: CHECK_TEMPLATE, Translation :: get('SaveAsTemplate'));
    }

    /**
     * Builds the buttons to update.
     */
    function build_update_buttons()
    {
        // update - clear buttons
        $buttons[] = $this->createElement('style_submit_button', 'submit',
                Translation :: get('Update', null, Utilities :: COMMON_LIBRARIES), array('class' => 'positive update'));
        $buttons[] = $this->createElement('style_reset_button', 'reset',
                Translation :: get('Reset', null, Utilities :: COMMON_LIBRARIES), array('class' => 'normal empty'));

        $this->addGroup($buttons, 'buttons', null, '&nbsp;', false);
    }

    /**
     * Builds the opdate to send.
     */
    function build_send_buttons()
    {
        // send - clear buttons
        $buttons[] = $this->createElement('style_submit_button', 'submit',
                Translation :: get('Send', null, Utilities :: COMMON_LIBRARIES), array('class' => 'positive send'));
        $buttons[] = $this->createElement('style_reset_button', 'reset',
                Translation :: get('Reset', null, Utilities :: COMMON_LIBRARIES), array('class' => 'normal empty'));

        $this->addGroup($buttons, 'buttons', null, '&nbsp;', false);
    }

    // **************************************************************************
    // OBJECT FUNCTIONS
    // **************************************************************************
    /**
     * Updates the message.
     *
     * @return Message
     */
    function update_message()
    {
        $message = $this->message;
        $values = $this->exportValues();

        // title
        if ($values[Message :: PROPERTY_MESSAGE_TITLE])
        {
            $message->set_title($values[Message :: PROPERTY_MESSAGE_TITLE]);
        }

        // content
        $message->set_content($values[Message :: PROPERTY_MESSAGE_CONTENT]);

        $value = $message->update();

        return $value;
    }

    /**
     * Creates the message.
     *
     * @return Message
     */
    function create_message()
    {
        $message = $this->message;
        $values = $this->exportValues();

        // title
        $message->set_title($values[Message :: PROPERTY_MESSAGE_TITLE]);

        // content
        $message->set_content($values[Message :: PROPERTY_MESSAGE_CONTENT]);

        // author
        $message->set_author_id($this->manager->get_user_id());

        // attachments (target)
        if ($values[self :: SELECT_ATTACHMENTS][self :: SUBARRAY_ELEMENT_FINDER_ELEMENT])
        {
            $message->set_attachments($values[self :: SELECT_ATTACHMENTS][self :: SUBARRAY_ELEMENT_FINDER_ELEMENT]);
        }

        // users (target)
        if ($values[self :: SELECT_RECIPIENTS][self :: SUBARRAY_USER_GROUP_FINDER_USER])
        {
            $message->set_target_users($values[self :: SELECT_RECIPIENTS][self :: SUBARRAY_USER_GROUP_FINDER_USER]);
        }

        // groups (target)
        if ($values[self :: SELECT_RECIPIENTS][self :: SUBARRAY_USER_GROUP_FINDER_GROUP])
        {
            $message->set_target_groups($values[self :: SELECT_RECIPIENTS][self :: SUBARRAY_USER_GROUP_FINDER_GROUP]);
        }

        // create message
        $value = $message->create();

        if ($this->form_type == self :: TYPE_CREATE || $this->form_type == self :: TYPE_CREATE_TEMPLATE)
        {
            // set parent if it is (the root of) a discussion (=>ROOT: discussion & no parent registered)
            if ($values[self :: RADIO_TYPE] == self :: RADIO_TYPE_DISCUSSION && ! $message->get_parent_id())
            {
                $message->set_parent_id($message->get_id());
                $message->update();
            }

            // save template
            if ($values[self :: CHECK_TEMPLATE] == true)
            {
                $this->manager->create_template($message);
            }
        }

        return $value;
    }

    // **************************************************************************
    // DEFAULTS
    // **************************************************************************
    /**
     * Sets default values.
     *
     * @param $defaults array Default values for this form's parameters.
     */
    function setDefaults($defaults = array())
    {
        $message = $this->message;

        // MESSAGE DEFAULTS
        $defaults[Message :: PROPERTY_ID] = $message->get_id();
        $defaults[Message :: PROPERTY_MESSAGE_AUTHOR_ID] = $message->get_author_id();
        $defaults[Message :: PROPERTY_MESSAGE_CONTENT] = $message->get_content();
        $defaults[Message :: PROPERTY_MESSAGE_CREATION_TIME] = $message->get_creation_time();
        $defaults[Message :: PROPERTY_MESSAGE_MODIFICATION_TIME] = $message->get_modification_time();
        $defaults[Message :: PROPERTY_MESSAGE_PARENT_ID] = $message->get_parent_id();
        $defaults[Message :: PROPERTY_MESSAGE_REPLY_ID] = $message->get_reply_id();
        $defaults[Message :: PROPERTY_MESSAGE_TITLE] = $message->get_title();

        // RADIO DEFAULTS
        if ($message->is_discussion())
        {
            $defaults[self :: RADIO_TYPE] = self :: RADIO_TYPE_DISCUSSION;
        }
        else
        {
            if (! $defaults[Message :: PROPERTY_ID] && Request :: get(PersonalMessengerManager :: PARAM_FOLDER) == PersonalMessengerFolder :: DISCUSSIONS)
            {
                $defaults[self :: RADIO_TYPE] = self :: RADIO_TYPE_DISCUSSION;
            }
            else
            {
                $defaults[self :: RADIO_TYPE] = self :: RADIO_TYPE_PERSONAL_MESSAGE;
            }
        }

        $defaults[self :: CHECK_TEMPLATE] = false;

        $repository_dm = RepositoryDataManager :: get_instance();
        $group_dm = GroupDataManager :: get_instance();
        $user_dm = UserDataManager :: get_instance();

        // ATTACHMENT DEFAULTS
        $defaults[self :: SELECT_ATTACHMENTS] = array();
        if ($message->get_attachments())
        {
            $condition = new InCondition(ContentObject :: PROPERTY_ID, $message->get_attachments(),
                    ContentObject :: get_table_name());
            $attached_objects = $repository_dm->retrieve_content_objects($condition)->as_array();
            $defaults[self :: SELECT_ATTACHMENTS] = Utilities :: content_objects_for_element_finder($attached_objects);
        }

        $active = $this->getElement(self :: SELECT_ATTACHMENTS);
        if ($active)
        {
            if ($active->_elements[0])
            {
                $active->_elements[0]->setValue(serialize($defaults[self :: SELECT_ATTACHMENTS]));
            }
        }

        // RECIPIENT DEFAULTS
        $defaults[self :: SELECT_RECIPIENTS] = array();
        foreach ($message->get_target_groups() as $target_group)
        {
            $group = $group_dm->retrieve_group($target_group);

            $selected_group = array();
            $selected_group['id'] = 'group_' . $group->get_id();
            $selected_group['classes'] = 'type type_group';
            $selected_group['title'] = $group->get_name();
            $selected_group['description'] = $group->get_description();

            $defaults[self :: SELECT_RECIPIENTS][$selected_group['id']] = $selected_group;
        }
        foreach ($message->get_target_users() as $target_user)
        {
            $user = $user_dm->retrieve_user($target_user);

            $selected_user = array();
            $selected_user['id'] = 'user_' . $user->get_id();
            $selected_user['classes'] = 'type type_user';
            $selected_user['title'] = $user->get_fullname();
            $selected_user['description'] = $user->get_username();

            $defaults[self :: SELECT_RECIPIENTS][$selected_user['id']] = $selected_user;
        }

        $active = $this->getElement(self :: SELECT_RECIPIENTS);
        if ($active)
        {
            if ($active->_elements[0])
            {
                $active->_elements[0]->setValue(serialize($defaults[self :: SELECT_RECIPIENTS]));
            }
        }

        parent :: setDefaults($defaults);
    }

}

?>