<?php

namespace application\personal_messenger;

use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\FormValidator;
use common\libraries\InCondition;
use common\libraries\Path;
use common\libraries\ResourceManager;
use common\libraries\Session;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\WebApplication;
use group\GroupDataManager;
use repository\RepositoryDataManager;
use repository\ContentObject;
use user\UserDataManager;

/** * ***************************************************************************
 * This form offers the functionality to move a message to a user defined category.
 *
 * @name Chamilo Personal Messenger > MoveForm
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * **************************************************************************** */
class MoveForm extends FormValidator
{
    //**************************************************************************
    // CONSTANTS
    //**************************************************************************
    const SELECT_CATEGORY = 'select_category';

    const SUBARRAY_ELEMENT_FINDER_ELEMENT = 'lo';
    const SUBARRAY_USER_GROUP_FINDER_USER = 'user';
    const SUBARRAY_USER_GROUP_FINDER_GROUP = 'group';

    const CATEGORY_TREE_ROOT_ID = 0;

    //**************************************************************************
    // VARIABLES
    //**************************************************************************
    /**
     * The manager to retrieve additional information.
     * @var PersonalMessengerManager
     */
    private $manager;
    /**
     * The folder in which the messages resides.
     * @var int
     */
    private $folder;
    /**
     * The tree to construct.
     * @var array
     */
    private $tree;

    //**************************************************************************
    // CONSTRUCTOR
    //**************************************************************************
    /**
     * Constructor.
     */
    function __construct($manager, $folder, $action)
    {
        parent :: __construct('move_personal_message', 'post', $action);
        $this->manager = $manager;
        $this->folder = $folder;
        $this->build_move_to_category_form();
    }

    //**************************************************************************
    // FORM FUNCTIONS
    //**************************************************************************
    /**
     * Builds the move form.
     */
    function build_move_to_category_form()
    {
        // build the tree
        $this->build_category_selection_tree();

        // build the selector element
        $this->addElement(
                'select',
                self :: SELECT_CATEGORY,
                Translation :: get('Category', null, Utilities:: COMMON_LIBRARIES),
                $this->tree
        );

        // create button
        $this->addElement(
                'style_submit_button',
                'submit',
                Translation :: get('Move', null, Utilities:: COMMON_LIBRARIES),
                array('class' => 'positive move')
        );
    }

    //**************************************************************************
    // ADDITIONAL FUNCTIONS
    //**************************************************************************
    /**
     * Creates a tree like structure to display the user defined categories.
     * @return Array
     */
    function build_category_selection_tree()
    {
        // initiate the tree
        $this->tree = array();

        // add the root (= folder name)
        $tree_name = '[ ';
        $tree_name .= Translation :: get(
                        Utilities :: underscores_to_camelcase(PersonalMessengerFolder :: get_folder_name($this->folder))
        );
        $tree_name .= ' ]';
        $this->tree[self :: CATEGORY_TREE_ROOT_ID] = $tree_name;

        // build subcategories
        $this->build_subcategories(self :: CATEGORY_TREE_ROOT_ID);
    }

    /**
     * Recursive method that is called to build the tree like view.
     * @param int $parent_id The parent for which the subcategories are fetched
     * @param int $level The level in the tree (for indentation)
     */
    function build_subcategories($parent_id, $level = 0)
    {
        // construct conditions
        $conditions = array();
        $conditions[] = new EqualityCondition(
                        UserCategory :: PROPERTY_USER_CATEGORY_USER_ID,
                        $this->manager->get_user_id()
        );
        $conditions[] = new EqualityCondition(
                        UserCategory :: PROPERTY_PARENT,
                        $parent_id
        );
        $conditions[] = new EqualityCondition(
                        UserCategory :: PROPERTY_USER_CATEGORY_FOLDER,
                        $this->folder
        );
        $condition = new AndCondition($conditions);

        // retrieve categories
        $categories = PersonalMessengerDataManager :: get_instance()->retrieve_all_user_categories($condition);

        // append tree
        $level++;
        while ($cat = $categories->next_result())
        {
            $this->tree[$cat->get_id()] = str_repeat('--', $level) . ' ' . $cat->get_name();
            $this->build_subcategories($cat->get_id(), $level);
        }
        $level--;
    }

    //**************************************************************************
    // OBJECT FUNCTIONS
    //**************************************************************************
    /**
     * Moves the message.
     * @return boolean
     */
    function move_message($message_id)
    {
        // get category
        $category_id = $this->get_destination_category_id();

        // construct conditions
        $conditions = array();
        $conditions[] = new EqualityCondition(
                        UserMessage :: PROPERTY_USER_MESSAGE_MESSAGE_ID,
                        $message_id
        );
        $conditions[] = new EqualityCondition(
                        UserMessage :: PROPERTY_USER_MESSAGE_USER_ID,
                        $this->manager->get_user_id()
        );
        $condition = new AndCondition($conditions);

        // retrieve the user message
        $user_message = PersonalMessengerDataManager :: get_instance()->retrieve_user_message($condition);

        // update the user message
        if ($user_message)
        {
            $user_message->set_category_id($category_id);
            return $user_message->update();
        }
        else
        {
            return false;
        }
    }

    /**
     * Returns the destination category id.
     * @return int
     */
    function get_destination_category_id()
    {
        return $this->exportValue(self :: SELECT_CATEGORY);
    }

}

?>