<?php

namespace application\personal_messenger;

use common\libraries\AndCondition;
use common\libraries\Configuration;
use common\libraries\EqualityCondition;
use common\libraries\InCondition;
use common\libraries\InequalityCondition;
use common\libraries\NotCondition;
use common\libraries\OrCondition;
use common\libraries\Session;
use common\libraries\Utilities;
use user\User;
use user\UserDataManager;

/** * ***************************************************************************
 * This class is global to all data managers. It combines a singleton and
 * factory to construct different implementations of the data manager.
 *
 * @name Chamilo Personal Messenger > PersonalMessengerDataManager
 * @version 1.0.0
 * @author Hans De Bisschop
 * @author Dieter De Neef
 * @author Stijn Van Hoecke
 * **************************************************************************** */
class PersonalMessengerDataManager
{

    //**************************************************************************
    // VARIABLES
    //**************************************************************************
    /**
     * Instance of the PersonalMessengerDataManager
     * @var <PersonalMessengerDataManager>
     */
    private static $instance;

    //**************************************************************************
    // CONSTRUCTOR
    //**************************************************************************
    /**
     * Constructor
     */
    public function __construct()
    {
        $this->initialize();
    }

    //**************************************************************************
    // GENERAL FUNCTIONS
    //**************************************************************************
    /**
     * Returns an instance of the PersonalMessengerDataManager as declared in the configuration.
     * All data managers currently have to reside in the '/data_manager/' folder and must end in
     * '_personal_messenger_data_manager.class.php'.
     * @return <PersonalMessengerDataManager> The data manager.
     */
    static function get_instance()
    {
        if (!isset(self :: $instance))
        {
            // get the type of data manager to instantiate
            $type = Configuration :: get_instance()->get_parameter('general', 'data_manager');

            // determine the name of the class
            $class = __NAMESPACE__ . '\\' . Utilities :: underscores_to_camelcase($type) . 'PersonalMessengerDataManager';

            // instantiate the class and store it as instance
            self :: $instance = new $class();
        }
        return self :: $instance;
    }

    /**
     * Returns the number of attachments linked to this message.
     * @return int The number of attachments
     */
    static function count_attachments($message_id)
    {
        // create condition
        $count_attachments_condition = new EqualityCondition(
                        MessageAttachment :: PROPERTY_MESSAGE_ATTACHMENT_MESSAGE_ID,
                        $message_id,
                        MessageAttachment :: get_table_name()
        );

        // get recipients
        return self :: get_instance()->count_message_attachment($count_attachments_condition);
    }

    /**
     * Returns the recipient names in an array.
     * @param int $message_id The message ID
     * @param array $excluded_user_ids The user IDs not to include (e.g. author_id)
     * @return array The recipients
     */
    static function get_recipient_names($message_id, $excluded_user_ids = array())
    {
        // create conditions
        $group_condition = new EqualityCondition(
                        MessageGroupRecipient :: PROPERTY_MESSAGE_RECIPIENT_MESSAGE_ID,
                        $message_id,
                        MessageGroupRecipient :: get_table_name()
        );
        $user_condition = new EqualityCondition(
                        MessageUserRecipient :: PROPERTY_MESSAGE_RECIPIENT_MESSAGE_ID,
                        $message_id,
                        MessageUserRecipient :: get_table_name()
        );

        // get recipients
        $group_recipients = self :: get_instance()->retrieve_message_group_recipients($group_condition);
        $user_recipients = self :: get_instance()->retrieve_message_user_recipients($user_condition);

        // construct name array
        $recipients = array();
        while ($recipient = $group_recipients->next_result())
        {
            $recipients[] = $recipient->get_recipient_name();
        }
        while ($recipient = $user_recipients->next_result())
        {
            // if the recipient is not in the exclude array, add him/her
            if (!in_array($recipient->get_user_id(), $excluded_user_ids))
            {
                $recipients[] = $recipient->get_recipient_name();
            }
        }

        // return the names
        return $recipients;
    }

    /**
     * Creates a new attachment link for the given message and attachment ID.
     * @param Message $message The message
     * @param int $content_object_id The attachment ID
     * @return MessageAttachment
     */
    static function message_add_attachment($message, $content_object_id)
    {
        // construct the object
        $message_attachment = new MessageAttachment();
        $message_attachment->set_message_id($message->get_id());
        $message_attachment->set_content_object_id($content_object_id);

        // create the link
        if (!$message_attachment->create())
        {
            return false;
        }

        return $message_attachment;
    }

    /**
     * Deletes an attachment link for the given message and attachment ID.
     * @param Message $message The message
     * @param int $content_object_id The attachment ID
     * @return MessageAttachment
     */
    static function message_delete_attachment($message, $content_object_id)
    {
        // construct the object
        $message_attachment = new MessageAttachment();
        $message_attachment->set_message_id($message->get_id());
        $message_attachment->set_content_object_id($content_object_id);

        // create the link
        if (!$message_attachment->delete())
        {
            return false;
        }

        return $message_attachment;
    }

    /**
     * Creates a new group recipient link for the given message and group ID.
     * @param Message $message The message
     * @param int $group_id The group ID
     * @return MessageGroupRecipient
     */
    static function message_add_group_recipient($message, $group_id)
    {
        // construct the object
        $message_group_recipient = new MessageGroupRecipient();
        $message_group_recipient->set_message_id($message->get_id());
        $message_group_recipient->set_group_id($group_id);

        // create the link
        if (!$message_group_recipient->create())
        {
            return false;
        }

        return $message_group_recipient;
    }

    /**
     * Creates a new user recipient link for the given message and user ID.
     * @param Message $message The message
     * @param int $user_id The user ID
     * @return MessageUserRecipient
     */
    static function message_add_user_recipient($message, $user_id)
    {
        // construct the object
        $message_user_recipient = new MessageUserRecipient();
        $message_user_recipient->set_message_id($message->get_id());
        $message_user_recipient->set_user_id($user_id);

        // create the link
        if (!$message_user_recipient->create())
        {
            return false;
        }

        return $message_user_recipient;
    }

    /**
     * Creates a new user message link for the given message and user ID.
     * @param Message $message The message
     * @param int $user_id The user ID
     * @return UserMessage
     */
    static function message_add_user_message_relation($message, $user_id, $own_read = true)
    {
        // cache to store the users who have ignored me so I can be so kind to set the status of my message to ignored
        $user_ids_who_ignore_me = array();

        // fill the ignore cache
        $condition = new EqualityCondition(
                        UserIgnore :: PROPERTY_USER_IGNORE_IGNORED_USER_ID,
                        $message->get_author_id()
        );
        $users_who_ignore_me = self :: get_instance()->retrieve_all_user_ignores($condition);
        while ($bastard = $users_who_ignore_me->next_result())
        {
            $user_ids_who_ignore_me[] = $bastard->get_user_id();
        }

        // create the new link and exclude the user from any further linking
        $user_message = new UserMessage();
        $user_message->set_message_id($message->get_id());
        $user_message->set_user_id($user_id);

        // adjust the status of the message if it should be ignored
        if (in_array($user_id, $user_ids_who_ignore_me))
        {
            $user_message->set_status(UserMessage :: STATUS_IGNORE);
        }

        // adjust the status of the message if I sent it
        if ($user_id == $message->get_author_id() && $own_read)
        {
            $user_message->set_status(UserMessage :: STATUS_READ);
        }

        // create the user message (link to the inbox)
        if (!$user_message->create())
        {
            return false;
        }

        return $user_message;
    }

    /**
     * Deletes a user message link based on the given message and user ID.
     * @param Message $message
     * @param int $user_id
     * @return boolean
     */
    static function message_delete_user_message_relation($message, $user_id)
    {
        // create conditions
        $conditions = array();
        $conditions[] = new EqualityCondition(
                        UserMessage :: PROPERTY_USER_MESSAGE_MESSAGE_ID,
                        $message->get_id()
        );
        $conditions[] = new EqualityCondition(
                        UserMessage :: PROPERTY_USER_MESSAGE_USER_ID,
                        $user_id
        );
        $condition = new AndCondition($conditions);

        $user_message = self :: get_instance()->retrieve_user_message($condition);
        return $user_message->delete();
    }

    /**
     * Determines if a user message relation exists between the given message ID and user ID.
     * @param int $message_id
     * @param int $user_id
     * @return boolean
     */
    static function user_message_relation_exists($message_id, $user_id)
    {
        // create conditions
        $conditions = array();
        $conditions[] = new EqualityCondition(
                        UserMessage :: PROPERTY_USER_MESSAGE_MESSAGE_ID,
                        $message_id
        );
        $conditions[] = new EqualityCondition(
                        UserMessage :: PROPERTY_USER_MESSAGE_USER_ID,
                        $user_id
        );
        $condition = new AndCondition($conditions);

        // determine if there is a link
        return (self :: get_instance()->count_user_message($condition) == 1);
    }

    static function is_user_ignored_by_me($user_id)
    {
        // create conditions
        $conditions = array();
        $conditions[] = new EqualityCondition(
                        UserIgnore :: PROPERTY_USER_IGNORE_USER_ID,
                        Session :: get_user_id()
        );
        $conditions[] = new EqualityCondition(
                        UserIgnore :: PROPERTY_USER_IGNORE_IGNORED_USER_ID,
                        $user_id
        );
        $condition = new AndCondition($conditions);

        // determine if te ignore link exists
        return (self :: get_instance()->count_user_ignore($condition) == 1);
    }

    /**
     * Gets the status of the usermessage.
     * @param int $message_id The message to change
     * @return int
     */
    static function get_user_message_status($message_id)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(
                        UserMessage :: PROPERTY_USER_MESSAGE_MESSAGE_ID,
                        $message_id
        );
        $conditions[] = new EqualityCondition(
                        UserMessage :: PROPERTY_USER_MESSAGE_USER_ID,
                        Session :: get_user_id()
        );
        $condition = new AndCondition($conditions);

        $user_message = self :: get_instance()->retrieve_user_message($condition);
        if ($user_message)
        {
            return $user_message->get_status();
        }
        else
        {
            return false;
        }
    }

    /**
     * Changes the status of the usermessage from the given original to the given new status.
     * @param int $message_id The message to change
     * @param int $original_status Restriction on the original status
     * @param int $new_status Switch to this status
     * @param boolean $include_me Indicates is my own link should be changes aswell
     * @return boolean
     */
    static function change_user_message_status($message_id, $original_status = UserMessage :: STATUS_READ, $new_status = UserMessage :: STATUS_UNREAD, $include_me = false)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(
                        UserMessage :: PROPERTY_USER_MESSAGE_MESSAGE_ID,
                        $message_id
        );
        $conditions[] = new EqualityCondition(
                        UserMessage :: PROPERTY_USER_MESSAGE_STATUS,
                        $original_status
        );

        if (!$include_me)
        {
            $conditions[] = new NotCondition(
                            new EqualityCondition(
                                    UserMessage :: PROPERTY_USER_MESSAGE_USER_ID,
                                    Session :: get_user_id()
                            )
            );
        }

        $condition = new AndCondition($conditions);
        $other_recipients = self :: get_instance()->retrieve_all_user_messages($condition);
        while ($recipient = $other_recipients->next_result())
        {
            $recipient->set_status($new_status);
            if (!$recipient->update())
            {
                return false;
            }
        }
        return true;
    }

    /**
     * Returns if there are new replies.
     * @param int $root_message_id The root message id
     * @return type
     */
    static function has_unread_replies($root_message_id)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(
                        UserMessage :: PROPERTY_USER_MESSAGE_USER_ID,
                        Session :: get_user_id()
        );
        $conditions[] = new EqualityCondition(
                        UserMessage :: PROPERTY_USER_MESSAGE_MESSAGE_ID,
                        $root_message_id
        );
        $conditions[] = new EqualityCondition(
                        UserMessage :: PROPERTY_USER_MESSAGE_STATUS,
                        UserMessage :: STATUS_UNREAD
        );
        $condition = new AndCondition($conditions);

        return self :: get_instance()->count_user_message($condition);
    }

    /**
     * Returns my unread replies on the given message since the last_checked date.
     * @param int $root_message_id The message on which is replied
     * @param int $last_checked The UNIX timestamp of the last time we checked
     */
    static function retrieve_unread_replies($root_message_id, $last_checked)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(
                        Message :: PROPERTY_MESSAGE_PARENT_ID,
                        $root_message_id
        );
        $conditions[] = new InequalityCondition(
                        Message :: PROPERTY_MESSAGE_MODIFICATION_TIME,
                        InequalityCondition :: GREATER_THAN,
                        $last_checked
        );
        $condition = new AndCondition($conditions);

        return self :: get_instance()->retrieve_all_messages($condition);
    }
    
    /**
     * Switches the status of the given message to read.
     * @param int $root_message_id
     * @return boolean
     */
    static function read_my_message($root_message_id)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(
                        UserMessage :: PROPERTY_USER_MESSAGE_MESSAGE_ID,
                        $root_message_id
        );
        $conditions[] = new EqualityCondition(
                        UserMessage :: PROPERTY_USER_MESSAGE_USER_ID,
                        Session :: get_user_id()
        );
        $condition = new AndCondition($conditions);

        $user_message = self :: get_instance()->retrieve_user_message($condition);
        if($user_message)
        {
            $user_message->set_status(UserMessage :: STATUS_READ);
            return $user_message->update();
        }
        return false;
    }
    
    /**
     * Retrieves a single message based on the given id.
     * @param int $message_id 
     */
    static function retrieve_message_by_id($message_id)
    {
        $condition = new EqualityCondition(Message :: PROPERTY_ID, $message_id);
        return self :: get_instance()->retrieve_message($condition);
    }

    /**
     * Counts the number of (unread) messages in the given folder and category.
     * @param type $folder The folder
     * @param type $category_id The category_id
     * @param type $unread Restrict to unread messages
     * @return int
     */
    static function count_messages($folder, $category_id = -1, $unread = false)
    {
        // create the basic condition for messages in the given folder
        $conditions = self :: get_folder_conditions($folder, $category_id);

        if ($unread)
        {
            $additional_conditions = array();
            $additional_conditions[] = $conditions;
            $additional_conditions[] = new EqualityCondition(
                            UserMessage :: PROPERTY_USER_MESSAGE_STATUS,
                            UserMessage :: STATUS_UNREAD,
                            UserMessage :: get_table_name()
            );
            $conditions = new AndCondition($additional_conditions);
        }

        // count
        return self :: get_instance()->count_user_messages($conditions);
    }
    
    /**
     * Counts all (unread) messages on the platform for the current user, regardless the folder or category.
     * @param boolean $unread Indicates if we count only unread messages
     * @return int 
     */
    static function count_all_messages($unread = true, $since = 0)
    {
        $conditions = array();
        
        // my own messages
        $conditions[] = new EqualityCondition(UserMessage :: PROPERTY_USER_MESSAGE_USER_ID, Session :: get_user_id(), UserMessage :: get_table_name());
        
        // since the given date
        $conditions[] = new InequalityCondition(Message :: PROPERTY_MESSAGE_CREATION_TIME, InequalityCondition :: GREATER_THAN, $since);
        
        // optionaly unread
        if($unread)
        {
            $conditions[] = new EqualityCondition(
                            UserMessage :: PROPERTY_USER_MESSAGE_STATUS,
                            UserMessage :: STATUS_UNREAD,
                            UserMessage :: get_table_name()
            );
        }
        
        $condition = new AndCondition($conditions);
        
        // count
        return self :: get_instance()->count_user_messages($condition);
    }

    /**
     * Creates the conditions with which messages must match for the given folder.
     * @param int $folder
     * @param int $category_id (optional, -1 for all categories)
     * @return Condition
     */
    static function get_folder_conditions($folder, $category_id = 0)
    {
        // always restrict to own messages
        $user_id = Session :: get_user_id();
        $user_id_condition = new EqualityCondition(
                        UserMessage :: PROPERTY_USER_MESSAGE_USER_ID,
                        $user_id,
                        UserMessage :: get_table_name()
        );

        // additional conditions
        $conditions = Array();
        $conditions[] = $user_id_condition;

        if (PersonalMessengerFolder :: supports_subcategories($folder))
        {
            // if the category_id is -1, get all categories
            if ($category_id != -1)
            {
                $conditions[] = new EqualityCondition(
                                UserMessage :: PROPERTY_USER_MESSAGE_CATEGORY_ID,
                                $category_id,
                                UserMessage :: get_table_name()
                );
            }
        }

        switch ($folder)
        {
            case PersonalMessengerFolder :: INBOX :
                // no parent for inbox
                $conditions[] = new EqualityCondition(
                                Message :: PROPERTY_MESSAGE_PARENT_ID,
                                null
                );

                // I am not the author
                $conditions[] = new NotCondition(
                                new EqualityCondition(
                                        Message :: PROPERTY_MESSAGE_AUTHOR_ID,
                                        $user_id
                                )
                );

                // status read or unread
                $status_conditions = Array();
                $status_conditions[] = new EqualityCondition(
                                UserMessage :: PROPERTY_USER_MESSAGE_STATUS,
                                UserMessage :: STATUS_READ,
                                UserMessage :: get_table_name()
                );
                $status_conditions[] = new EqualityCondition(
                                UserMessage :: PROPERTY_USER_MESSAGE_STATUS,
                                UserMessage :: STATUS_UNREAD,
                                UserMessage :: get_table_name()
                );
                $conditions[] = new OrCondition($status_conditions);
                break;

            case PersonalMessengerFolder :: DISCUSSIONS :
                // parent for discussion
                $conditions[] = new NotCondition(
                                new EqualityCondition(
                                        Message :: PROPERTY_MESSAGE_PARENT_ID,
                                        null
                                )
                );

                // status read or unread
                $status_conditions = Array();
                $status_conditions[] = new EqualityCondition(
                                UserMessage :: PROPERTY_USER_MESSAGE_STATUS,
                                UserMessage :: STATUS_READ,
                                UserMessage :: get_table_name()
                );
                $status_conditions[] = new EqualityCondition(
                                UserMessage :: PROPERTY_USER_MESSAGE_STATUS,
                                UserMessage :: STATUS_UNREAD,
                                UserMessage :: get_table_name()
                );
                $conditions[] = new OrCondition($status_conditions);
                break;

            case PersonalMessengerFolder :: OUTBOX :
                // no parent for outbox
                $conditions[] = new EqualityCondition(
                                Message :: PROPERTY_MESSAGE_PARENT_ID,
                                null
                );

                // I am author
                $conditions[] = new EqualityCondition(
                                Message :: PROPERTY_MESSAGE_AUTHOR_ID,
                                $user_id
                );

                // status read or unread
                $status_conditions = Array();
                $status_conditions[] = new EqualityCondition(
                                UserMessage :: PROPERTY_USER_MESSAGE_STATUS,
                                UserMessage :: STATUS_READ,
                                UserMessage :: get_table_name()
                );
                $status_conditions[] = new EqualityCondition(
                                UserMessage :: PROPERTY_USER_MESSAGE_STATUS,
                                UserMessage :: STATUS_UNREAD,
                                UserMessage :: get_table_name()
                );
                $conditions[] = new OrCondition($status_conditions);
                break;

            case PersonalMessengerFolder :: IGNORED :
                // status ignored
                $conditions[] = new EqualityCondition(
                                UserMessage :: PROPERTY_USER_MESSAGE_STATUS,
                                UserMessage :: STATUS_IGNORE,
                                UserMessage :: get_table_name()
                );
                break;

            case PersonalMessengerFolder :: TRASH :
                // status trashed
                $conditions[] = new EqualityCondition(
                                UserMessage :: PROPERTY_USER_MESSAGE_STATUS,
                                UserMessage :: STATUS_TRASH,
                                UserMessage :: get_table_name()
                );
                break;
        }

        // return global condition
        return new AndCondition($conditions);
    }

}

?>