<?php

namespace application\personal_messenger;

/** * ***************************************************************************
 * This interface is global to the data managers. It has some functions which
 * are obligated.
 * - COUNT
 * - C+R+U+D
 *
 * Each databaseobject has its proper functions that need to be implemented to
 * offer the above functionality.
 *
 * @name Chamilo Personal Messenger > PersonalMessengerDataManagerInterface
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * **************************************************************************** */
interface PersonalMessengerDataManagerInterface
{

    //**************************************************************************
    // GENERAL FUNCTIONS
    //**************************************************************************
    /**
     * Initializes the data manager.
     */
    function initialize();

    //**************************************************************************
    // MESSAGE FUNCTIONS
    //**************************************************************************
    /**
     * Counts the messages based on a given condition.
     * @param Condition $condition
     * @return int The number of messages
     */
    function count_message($condition = null);

    /**
     * Creates the message.
     * @param Message $message The message
     * @uses create_message_target_users()
     * @uses create_message_target_groups()
     * @return boolean
     */
    function create_message($message);

    /**
     * Retrieves a single message.
     * @param Condition $condition The condition
     * @return Message
     */
    function retrieve_message($condition = null);

    /**
     * Retrieves a series of messages.
     * @param Condition $condition The condition
     * @param array $order_by
     * @param int $offset The offset
     * @param int $max_objects The max number of objects to return
     * @return MessageResultSet
     */
    function retrieve_all_messages($condition = null, $order_by = array(), $offset = 0, $max_objects = -1);

    /**
     * Updates the given message.
     * @param Message $message The message
     * @return boolean
     */
    function update_message($message);

    /**
     * Deletes the given message.
     * @param Message $message The message
     * @return boolean
     */
    function delete_message($message);

    //**************************************************************************
    // MESSAGE_ATTACHMENT FUNCTIONS
    //**************************************************************************
    /**
     * Counts the message attachments based on a given condition.
     * @param Condition $condition
     * @return int The number of message attachments
     */
    function count_message_attachment($condition = null);

    /**
     * Creates the message attachment
     * @param MessageAttachment $message_attachment The message attachment
     * @return boolean
     */
    function create_message_attachment($message_attachment);

    /**
     * Retrieves a single message attachment.
     * @param Condition $condition The condition
     * @return MessageAttachment
     */
    function retrieve_message_attachment($condition = null);

    /**
     * Retrieves a series of message attachments.
     * @param Condition $condition The condition
     * @param array $order_by
     * @param int $offset The offset
     * @param int $max_objects The max number of objects to return
     * @return MessageAttachmentResultSet
     */
    function retrieve_all_message_attachments($condition = null, $order_by = array(), $offset = 0, $max_objects = -1);

    /**
     * Updates the given message.
     * @param MessageAttachment $message_attachment The message attachment
     * @return boolean
     */
    // !!! FEATURE DISABLED -> message_attachment is a linking table with only a primary key, no update possible !!!
    // function update_message_attachment($message_attachment);

    /**
     * Deletes the given message attachment.
     * @param MessageAttachment $message_attachment The message attachment
     * @return boolean
     */
    function delete_message_attachment($message_attachment);

    //**************************************************************************
    // MESSAGE_GROUP_RECIPIENTS FUNCTIONS
    //**************************************************************************
    /**
     * Counts the group relations based on a given condition.
     * @param Condition $condition
     * @return int The number of groups related to the message
     */
    function count_message_group_recipient($condition = null);

    /**
     * Creates the group relation
     * @param MessageGroupRecipient $message_group_recipient The group relation
     * @return boolean
     */
    function create_message_group_recipient($message_group_recipient);

    /**
     * Retrieves a single group relation.
     * @param Condition $condition The condition
     * @return MessageGroupRecipient
     */
    function retrieve_message_group_recipient($condition);

    /**
     * Retrieves a series of group relations.
     * @param Condition $condition The condition
     * @param array $order_by
     * @param int $offset The offset
     * @param int $max_objects The max number of objects to return
     * @return MessageGroupRecipientResultSet
     */
    function retrieve_all_message_group_recipients($condition = null, $order_by = array(), $offset = 0, $max_objects = -1);

    /**
     * Updates the given group relation.
     * @param MessageGroupRecipient $message_group_recipient The group relation
     * @return boolean
     */
    // !!! FEATURE DISABLED -> message_group_recipient is a linking table with only a primary key, no update possible !!!
    // function update_message_group_recipient($message_group_recipient);

    /**
     * Deletes the given group relation.
     * @param MessageGroupRecipient $message_group_recipient The group relation
     * @return boolean
     */
    function delete_message_group_recipient($message_group_recipient);

    //**************************************************************************
    // MESSAGE_USER_RECIPIENT FUNCTIONS
    //**************************************************************************
    /**
     * Counts the user relations based on a given condition.
     * @param Condition $condition
     * @return int The number of users related to the message
     */
    function count_message_user_recipient($condition = null);

    /**
     * Creates the user relation
     * @param MessageUserRecipient $message_user_recipient The user relation
     * @return boolean
     */
    function create_message_user_recipient($message_user_recipient);

    /**
     * Retrieves a single user relation.
     * @param Condition $condition The condition
     * @return MessageUserRecipient
     */
    function retrieve_message_user_recipient($condition = null);

    /**
     * Retrieves a series of user relations.
     * @param Condition $condition The condition
     * @param array $order_by
     * @param int $offset The offset
     * @param int $max_objects The max number of objects to return
     * @return MessageUserRecipientResultSet
     */
    function retrieve_all_message_user_recipients($condition = null, $order_by = array(), $offset = 0, $max_objects = -1);

    /**
     * Updates the given user relation.
     * @param MessageUserRecipient $message_user_recipient The user relation
     * @return boolean
     */
    // !!! FEATURE DISABLED -> message_user_recipient is a linking table with only a primary key, no update possible !!!
    // function update_message_user_recipient($message_user_recipient);

    /**
     * Deletes the given user relation.
     * @param MessageUserRecipient $message_user_recipient The user relation
     * @return boolean
     */
    function delete_message_user_recipient($message_user_recipient);

    //**************************************************************************
    // USER_MESSAGE FUNCTIONS
    //**************************************************************************
    /**
     * Counts the user messages based on a given condition.
     * @param Condition $condition
     * @return int The number of user message records
     */
    function count_user_message($condition = null);

    /**
     * Creates the user message
     * @param UserMessage $user_message The user message
     * @return boolean
     */
    function create_user_message($user_message);

    /**
     * Retrieves a single user message.
     * @param Condition $condition The condition
     * @return UserMessage
     */
    function retrieve_user_message($condition);

    /**
     * Retrieves a series of user messages.
     * @param Condition $condition The condition
     * @param array $order_by
     * @param int $offset The offset
     * @param int $max_objects The max number of objects to return
     * @return UserMessageResultSet
     */
    function retrieve_all_user_messages($condition = null, $order_by = array(), $offset = 0, $max_objects = -1);

    /**
     * Updates the given user message.
     * @param UserMessage $user_message The user message
     * @return boolean
     */
    function update_user_message($user_message);

    /**
     * Deletes the given user message.
     * @param UserMessage $user_message The user message
     * @return boolean
     */
    function delete_user_message($user_message);

    //**************************************************************************
    // USER_CATEGORY FUNCTIONS
    //**************************************************************************
    /**
     * Counts the user categories based on a given condition.
     * @param Condition $condition
     * @return int The number of user_categories
     */
    function count_user_category($condition = null);

    /**
     * Creates the user category
     * @param UserCategory $user_category The user category
     * @return boolean
     */
    function create_user_category($user_category);

    /**
     * Retrieves a single user category.
     * @param Condition $condition The condition
     * @return Message
     */
    function retrieve_user_category($condition);

    /**
     * Retrieves a series of user categories.
     * @param Condition $condition The condition
     * @param array $order_by
     * @param int $offset The offset
     * @param int $max_objects The max number of objects to return
     * @return UserCategoriesResultSet
     */
    function retrieve_all_user_categories($condition = null, $order_by = array(), $offset = 0, $max_objects = -1);

    /**
     * Updates the given user category.
     * @param UserCategory $user_category The user category
     * @return boolean
     */
    function update_user_category($user_category);

    /**
     * Deletes the given user category.
     * @param UserCategory $user_category The user category
     * @return boolean
     */
    function delete_user_category($user_category);

    //**************************************************************************
    // USER_IGNORE FUNCTIONS
    //**************************************************************************
    /**
     * Counts the ignored users based on a given condition.
     * @param Condition $condition
     * @return int The number of ignored users
     */
    function count_user_ignore($condition = null);

    /**
     * Creates the ignored user
     * @param UserIgnore $user_ignore The ignored user
     * @return boolean
     */
    function create_user_ignore($user_ignore);

    /**
     * Retrieves a single ignored user.
     * @param Condition $condition The condition
     * @return UserIgnore
     */
    function retrieve_user_ignore($condition);

    /**
     * Retrieves a series of ignored users.
     * @param Condition $condition The condition
     * @param array $order_by
     * @param int $offset The offset
     * @param int $max_objects The max number of objects to return
     * @return UserIgnoreResultSet
     */
    function retrieve_all_user_ignores($condition = null, $order_by = array(), $offset = 0, $max_objects = -1);

    /**
     * Updates the given ignored user.
     * @param UserIgnore $user_ignore The ignored user
     * @return boolean
     */
    // !!! FEATURE DISABLED -> user_ignore is a linking table with only a primary key, no update possible !!!
    // function update_user_ignore($user_ignore);

    /**
     * Deletes the given ignored user.
     * @param UserIgnore $user_ignore The ignored user
     * @return boolean
     */
    function delete_user_ignore($user_ignore);

    //**************************************************************************
    // ADVANCED FUNCTIONS
    //**************************************************************************
    /**
     * Retrieves all messages for a given user, depending on the given conditions.
     * @param Condition $condition
     * @param Array $order_by
     * @param int $offset
     * @param int $max_objects
     * @return MessageResultSet
     */
    function retrieve_user_messages($condition = null, $order_by = array(), $offset = 0, $max_objects = -1);

    /**
     * Counts all messages for a given user, depending on the given conditions.
     * @param Condition $condition
     * @return int
     */
    function count_user_messages($condition = null);

    /**
     * Retrieves all user recipients, depending on the given conditions.
     * @param Condition $condition
     * @param Array $order_by
     * @param int $offset
     * @param int $max_objects
     * @return MessageUserRecipientResultSet
     */
    function retrieve_message_user_recipients($condition = null, $order_by = array(), $offset = 0, $max_objects = -1);

    /**
     * Retrieves all group recipients, depending on the given conditions.
     * @param Condition $condition
     * @param Array $order_by
     * @param int $offset
     * @param int $max_objects
     * @return MessageGroupRecipientResultSet
     */
    function retrieve_message_group_recipients($condition = null, $order_by = array(), $offset = 0, $max_objects = -1);

    /**
     * Retrieves all ignored users, depending on the given conditions.
     * @param Condition $condition
     * @param array $order_by
     * @param int $offset
     * @param int $max_objects
     * @return UserResultSet
     */
    function retrieve_ignored_user_users($condition, $order_by = array(), $offset = 0, $max_objects = -1);

    /**
     * Retrieves all users currently active in a message, depending on the given conditions.
     * @param Condition $condition
     * @param array $order_by
     * @param int $offset
     * @param int $max_objects
     * @return UserResultSet
     */
    function retrieve_user_message_users($condition, $order_by = array(), $offset = 0, $max_objects = -1);
}

?>