<?php

namespace application\personal_messenger;

use common\libraries\ActionBarRenderer;
use common\libraries\AndCondition;
use common\libraries\Application;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\DynamicVisualTab;
use common\libraries\DynamicVisualTabsRenderer;
use common\libraries\EqualityCondition;
use common\libraries\OrCondition;
use common\libraries\PatternMatchCondition;
use common\libraries\Request;
use common\libraries\ResourceManager;
use common\libraries\Theme;
use common\libraries\Toolbar;
use common\libraries\ToolbarItem;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\WebApplication;
use user\UserDataManager;
use user\User;

/** * ***************************************************************************
 * This component offers the functionality to browse a series of messages.
 * It also displays all folders as tabs if the constant USE_TABS_VIEW is true.
 *
 * @name Chamilo Personal Messenger > PersonalMessengerManagerBrowserComponent
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * **************************************************************************** */
class PersonalMessengerManagerBrowserComponent extends PersonalMessengerManager
{
    //**************************************************************************
    // GENERAL CONSTANTS
    //**************************************************************************
    /**
     * Constant indicating the following statistics should be displayed after the tab name:
     * - inbox: the number of unread messages;
     * - discussion: the number of unread messages;
     * - outbox: nothing;
     * - ignored: the number of ignored messages;
     * - trash: nothing;
     */
    const SHOW_UNREAD_MESSAGE_COUNT_AFTER_TAB_NAME = true;

    /**
     * Constant indicating if the user category tree should be displayed next to, of above the table.
     */
    const SHOW_TREE_NEXT_TO_TABLE = false;

    //**************************************************************************
    // VARIABLES
    //**************************************************************************
    /**
     * Variable holding the action bar.
     * @var ActionBar
     */
    private $action_bar;
    /**
     * Variable indicating if there is a tree to display, this can be used to hide certain actions if no tree is present.
     * @var boolean
     */
    private $has_user_category_tree;

    //**************************************************************************
    // RUN
    //**************************************************************************
    /**
     * Runs this component and displays its output.
     */
    function run()
    {
        // create action bar
        $this->action_bar = $this->get_action_bar();

        // create breadcrubms
        $breadcrumbtrail = BreadcrumbTrail :: get_instance();

        // set folder (tab) name
        $folder_name = Translation :: get(Utilities :: underscores_to_camelcase(
                                PersonalMessengerFolder :: get_folder_name($this->get_folder())
                        ));

        // edit the breadcrumbtrail to include the folder (tab)
        $breadcrumbtrail->add(new Breadcrumb($this->get_browser_url($this->get_folder(), $category_id), $folder_name));

        // header
        $this->display_header($trail);

        // content
        echo $this->action_bar->as_html();

        // build view
        echo $this->get_folder_tabs();

        // footer
        $this->display_footer();
        
        // AJAX REFRESH
        // -- meta data for the AJAX postback
        $refresh_rate = 120;    // could come from user settings... now just take 2 minutes for refresh check
        echo '<script type="text/javascript">';
        echo 'var browser_refresh_interval = ' . $refresh_rate . '; ';
        echo 'var lastChecked = ' . time() . '; ';
        echo 'var ajaxUri = getPath("WEB_PATH") + "ajax.php";';
        echo '</script>';
        
        // script with AJAX refresh polling function
        echo ResourceManager :: get_instance()->get_resource_html(
                        WebApplication :: get_application_web_path('personal_messenger') . 'resources/javascript/has_new_messages.js');
    }

    //**************************************************************************
    // TABS FUNCTIONS
    //**************************************************************************
    /**
     * Creates the tab structure.
     * @return String HTML of the tab(s)
     */
    private function get_folder_tabs()
    {
        $folder_tabs = new DynamicVisualTabsRenderer('personal_messenger');
        $html = $this->get_folder_tabs_header($folder_tabs);
        $html .= $this->get_folder_tabs_content();
        $html .= $this->get_folder_tabs_footer($folder_tabs);
        return $html;
    }

    /**
     * Creates the header for the tabs.
     * @return String HTML of the header
     */
    private function get_folder_tabs_header($folder_tabs)
    {
        $html = array();

        foreach (PersonalMessengerFolder :: get_all_folders() as $folder)
        {
            $link = $this->get_browser_url($folder);
            $folder_name = Translation :: get(Utilities :: underscores_to_camelcase(
                                    PersonalMessengerFolder :: get_folder_name($folder)
                            ));

            if (self :: SHOW_UNREAD_MESSAGE_COUNT_AFTER_TAB_NAME)
            {
                $folder_name .= $this->get_folder_message_count_string($folder);
            }
            $folder_tabs->add_tab(new DynamicVisualTab(
                            $folder,
                            $folder_name,
                            PersonalMessengerFolder :: get_folder_icon_path($folder),
                            $link,
                            $this->get_folder() == $folder
            ));
        }

        $html[] = $folder_tabs->header();
        $html[] = DynamicVisualTabsRenderer :: body_header();

        return implode("\n", $html);
    }

    /**
     * Creates the content of the selected tab.
     * @return String HTML of the content
     */
    private function get_folder_tabs_content()
    {
        $html = array();
        $this->has_user_category_tree = false;

        // construct tree if allowed
        if (PersonalMessengerFolder :: supports_subcategories($this->get_folder()))
        {
            $tree = $this->get_menu_tree();

            // if the tree doesn't have children, don't show it
            $this->has_user_category_tree = $tree->node_has_children(0);
        }

        // show tree if there is one
        if ($this->has_user_category_tree)
        {
            $html[] = '<div ';
            if (self :: SHOW_TREE_NEXT_TO_TABLE)
            {
                $html[] = 'style="width: 12%; overflow: auto; float: left;">';
            }
            else
            {
                $html[] = 'class="user_category_tree"';
            }
            $html[] = '>';
            $html[] = $tree->render_as_tree();
            $html[] = '</div>';
            if (self :: SHOW_TREE_NEXT_TO_TABLE)
            {
                $html[] = '<div style="width: 85%; float: right;">';
            }
        }

        $html[] = $this->get_messages_browser_table_html();

        if ($this->has_user_category_tree && self :: SHOW_TREE_NEXT_TO_TABLE)
        {
            $html[] = '</div>';
        }

        return implode("\n", $html);
    }

    /**
     * Creates the footer for the tabs.
     * @return String HTML of the footer
     */
    private function get_folder_tabs_footer($folder_tabs)
    {
        $html = array();
        $html[] = DynamicVisualTabsRenderer :: body_footer();
        $html[] = $folder_tabs->footer();
        return implode("\n", $html);
    }

    //**************************************************************************
    // ADDITIONAL FUNCTIONS
    //**************************************************************************
    /**
     * Returns the HTML view of all messages in the specified folder.
     * @param int $folder
     * @return String HTML representation of the series of messages in the specified folder
     */
    private function get_messages_browser_table_html()
    {
        $parameters = $this->get_parameters();

        // construct condition
        $condition = PersonalMessengerFolder :: get_conditions($this->get_folder(), $this->get_category_id());

        // append condition using the search box
        if ($this->get_search_condition())
        {
            $conditions = array();
            $conditions[] = $condition;
            $conditions[] = $this->get_search_condition();
            $condition = new AndCondition($conditions);
        }

        // create table
        $table = new MessageBrowserTable($this, $parameters, $condition);

        // build html
        $html = array();
        $html[] = $table->as_html();

        return implode("\n", $html);
    }

    /**
     * Returns the CategoryRenderer of the generic tree containing all user defined categories.
     * @return UserCategoryRenderer
     */
    function get_menu_tree()
    {
        return new UserCategoryRenderer($this);
    }

    /**
     * Constructs the conditions for the search field.
     * @return Condition
     */
    function get_search_condition()
    {
        $query = $this->action_bar->get_query();
        if (isset($query) && $query != '')
        {
            $conditions = array();
            $conditions[] = new PatternMatchCondition(
                            Message :: PROPERTY_MESSAGE_TITLE,
                            '*' . $query . '*'
            );
            $conditions[] = new PatternMatchCondition(
                            Message :: PROPERTY_MESSAGE_CONTENT,
                            '*' . $query . '*'
            );
            if ($this->get_folder() != PersonalMessengerFolder :: OUTBOX)
            {
                /*
                 * We have to construct the alias for the User table here, because the database class constructs it with the wrong prefix.
                 * Summon the user data manager to get the alias and set the final parameter to true (to tell the db it is already an alias).
                 */
                $user_dm = UserDataManager :: get_instance();
                $conditions[] = new PatternMatchCondition(
                                User :: PROPERTY_FIRSTNAME,
                                '*' . $query . '*',
                                $user_dm->get_alias(User :: get_table_name()),
                                true
                );
                $conditions[] = new PatternMatchCondition(
                                User :: PROPERTY_LASTNAME,
                                '*' . $query . '*',
                                $user_dm->get_alias(User :: get_table_name()),
                                true
                );
            }
            return new OrCondition($conditions);
        }

        return null;
    }

    /**
     * Returns the action bar.
     * @return ActionBar
     */
    function get_action_bar()
    {
        // default bar
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);

        // additional functions
        // ACTION SEND
        $create_new_url = $this->get_message_create_url(Request :: get(self :: PARAM_REFERER));
        $create_new_item = new ToolbarItem(
                        Translation :: get(
                                Request :: get(self :: PARAM_FOLDER) == PersonalMessengerFolder :: DISCUSSIONS ? 'CreateDiscussion' : 'CreateMessage'),
                        Theme :: get_image_path() . 'action_create.png',
                        $create_new_url,
                        ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                        false
        );

        // ACTION SEND TEMPLATE
        $create_from_template_url = $this->get_message_create_from_template_url(Request :: get(self :: PARAM_REFERER));
        $create_from_template_item = new ToolbarItem(
                        Translation :: get(
                                Request :: get(self :: PARAM_FOLDER) == PersonalMessengerFolder :: DISCUSSIONS ? 'CreateDiscussionFromTemplate' : 'CreateMessageFromTemplate'),
                        Theme :: get_image_path() . 'action_create_from_template.png',
                        $create_from_template_url,
                        ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                        false
        );

        // MANAGE CATEGORIES
        $manage_cat_url = $this->get_url(array(Application :: PARAM_ACTION => self :: ACTION_MANAGE_CATEGORIES));
        $manage_cat_item = new ToolbarItem(
                        Translation :: get('ManageCategories', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_category.png',
                        $manage_cat_url,
                        ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                        false
        );

        // MANAGE CATEGORIES_NA
        $manage_cat_na_url = null;
        $manage_cat_na_item = new ToolbarItem(
                        Translation :: get('ManageCategoriesNA', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_category_na.png',
                        $manage_cat_na_url,
                        ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                        false
        );

        // CONSTRUCT TOOLBAR
        $action_bar->add_common_action($create_new_item);
        $action_bar->add_common_action($create_from_template_item);
        $action_bar->add_common_action(
                PersonalMessengerFolder :: supports_subcategories($this->get_folder()) ? $manage_cat_item : $manage_cat_na_item
        );

        if ($this->get_folder() == PersonalMessengerFolder :: IGNORED)
        {
            // ACTION MANAGE IGNORED USERS
            $manage_ignored_users_url = $this->get_ignored_user_manager_url(self :: ACTION_MANAGE_USERS_IGNORED);
            $manage_ignored_users_item = new ToolbarItem(
                            Translation :: get('ManageIgnoredUsers'),
                            Theme :: get_image_path() . 'action_manage_ignored_users.png',
                            $manage_ignored_users_url,
                            ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                            false
            );
            $action_bar->add_tool_action($manage_ignored_users_item);
        }

        // SEARCH
        $action_bar->set_search_url($this->get_url());
        $show_all_url = $this->get_url();
        $show_all_item = new ToolbarItem(
                        Translation :: get('ShowAll', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_browser.png',
                        $show_all_url,
                        ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                        false
        );
        $action_bar->add_common_action($show_all_item);

        // return
        return $action_bar;
    }

    function get_folder_message_count_string($folder)
    {
        // we don't want to display the number of messages in the outbox, nor the trash
        if ($folder == PersonalMessengerFolder :: OUTBOX || $folder == PersonalMessengerFolder :: TRASH)
        {
            return '';
        }

        if ($folder == PersonalMessengerFolder :: INBOX || $folder == PersonalMessengerFolder :: DISCUSSIONS)
        {
            $unread = true;
        }
        else
        {
            $unread = false;
        }
        
        $count = PersonalMessengerDataManager :: count_messages($folder, -1, $unread);

        // only display if we have messages
        $string = ($count > 0) ? ' (' . $count . ')' : '';
        return $string;
    }
}

?>