<?php

namespace application\personal_messenger;

use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\Request;
use common\libraries\Application;

/** * ***************************************************************************
 * This component offers the functionality to mark a (series of) message(s)
 * with a specific status. The status is determined by the actual implementation
 * of the marker.
 *
 * @name Chamilo Personal Messenger > PersonalMessengerManagerMarkerComponent
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * @see PersonalMessengerManagerMarkerReadComponent
 * @see PersonalMessengerManagerMarkerUnreadComponent
 * @see PersonalMessengerManagerMarkerIgnoreComponent
 * @see PersonalMessengerManagerMarkerUnignoreComponent
 * @see PersonalMessengerManagerMarkerTrashComponent
 * @see PersonalMessengerManagerMarkerUntrashComponent
 * **************************************************************************** */
abstract class PersonalMessengerManagerMarkerComponent extends PersonalMessengerManager
{

    //**************************************************************************
    // RUN
    //**************************************************************************
    /**
     * Runs this component.
     */
    function run()
    {
        $failures = 0;

        // get the IDs
        $message_ids = Request :: get(self :: PARAM_MESSAGE_ID);

        // checks
        if (!empty($message_ids))
        {
            // construct array if id is single element
            if (!is_array($message_ids))
            {
                $message_ids = array($message_ids);
            }

            // for each message, update
            foreach ($message_ids as $message_id)
            {
                // construct the condition
                $conditions = array();
                $conditions[] = new EqualityCondition(
                                UserMessage :: PROPERTY_USER_MESSAGE_USER_ID,
                                $this->get_user_id()
                );
                $conditions[] = new EqualityCondition(
                                UserMessage :: PROPERTY_USER_MESSAGE_MESSAGE_ID,
                                $message_id
                );
                $condition = new AndCondition($conditions);

                // get the UserMessage
                $user_message = $this->retrieve_user_message($condition);

                // update the status
                if (!$user_message)
                {
                    $failures++;
                }
                else
                {
                    $this->update_status($user_message);
                    if (!$user_message->update())
                    {
                        $failures++;
                    }
                }

                $update_folder_message = $message_id;
            }

            // update folder depending on the update_folder_message
            $message = $this->retrieve_message_by_id($update_folder_message);
            if ($message && Request :: get(self :: PARAM_SWITCH_CURRENT_FOLDER))
            {
                $this->set_folder($message->get_folder());
            }
        }

        // feedback
        if ($failures)
        {
            $feedback_message = Translation :: get(
                            count($message_ids) == 1 ? 'MessageNotMarked' : 'MessagesNotMarked',
                            array('STATUS' => Translation :: get($this->get_translation_status()))
            );
        }
        else
        {
            $feedback_message = Translation :: get(
                            count($message_ids) ? 'MessageMarked' : 'MessagesMarked',
                            array('STATUS' => Translation :: get($this->get_translation_status()))
            );
        }

        // go back
        $go_back_parameters = array();
        if (Request :: get(PersonalMessengerManager :: PARAM_REFERER))
        {
            $go_back_parameters[Application :: PARAM_ACTION] = Request :: get(self :: PARAM_REFERER);
            $go_back_parameters[self :: PARAM_MESSAGE_ID] = Request :: get(self :: PARAM_MESSAGE_ID);
        }
        else
        {
            $go_back_parameters[Application :: PARAM_ACTION] = self :: DEFAULT_ACTION;
        }

        $this->redirect($feedback_message, ($failures ? true : false), $go_back_parameters);
    }

    //**************************************************************************
    // ADDITIONAL FUNCTIONS
    //**************************************************************************
    /**
     * Adds the additional parameters to the URL.
     * @return array
     */
    function get_additional_parameters()
    {
        return array(self :: PARAM_REFERER, self :: PARAM_MESSAGE_ID);
    }

    //**************************************************************************
    // ABSTRACT FUNCTIONS
    //**************************************************************************
    /**
     * Updates the status of the message to the appropriate implementation.
     * @param UserMessage The user message
     */
    abstract function update_status($user_message);

    /**
     * Returns the status (identifier for translation) depending on the implementation.
     * @return string The status
     */
    abstract function get_translation_status();
}

?>