<?php

namespace application\personal_messenger;

use common\libraries\Theme;
use common\libraries\Toolbar;
use common\libraries\ToolbarItem;
use common\libraries\Translation;
use common\libraries\Utilities;

/** * ***************************************************************************
 * Cell renderer for the message browser table.
 *
 * @name Chamilo Personal Messenger > MessageBrowserTableCellRenderer
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * **************************************************************************** */
class MessageBrowserTableCellRenderer extends DefaultMessageTableCellRenderer
{

    //**************************************************************************
    // GENERAL VARIABLES
    //**************************************************************************
    /**
     * The messenge browser component
     */
    private $browser;

    //**************************************************************************
    // CONSTRUCTOR
    //**************************************************************************
    /**
     * Constructor
     * @param PersonalMessengerManagerBrowserComponent $browser The browser component
     * @param int $folder The folder to restrict actions
     * @param boolean $has_user_category_tree Indicates if there is a user category tree
     */
    function __construct($browser)
    {
        parent :: __construct($browser->get_folder(), $browser->get_category_id());
        $this->browser = $browser;
    }

    //**************************************************************************
    // GENERAL FUNCTIONS
    //**************************************************************************
    /**
     * Renders a table cell based on the given parameters column and message.
     * The message contains additional parameters obtained by a join between the
     * user message and actual message in the database data manager.
     * @param MessageTableColumn $column The column which should be rendered
     * @param Message $message_with_optional_properties The message to render
     * @return String A HTML representation of the rendered table cell
     */
    function render_cell($column, $message_with_optional_properties)
    {
        // get action links
        if ($column === MessageBrowserTableColumnModel :: get_modification_column())
        {
            return $this->get_modification_links($message_with_optional_properties);
        }

        // adds some special features to some columns, continues to the default if
        // the specified column is not found here.
        switch ($column->get_name())
        {
            // display image instead of status text with text as alt-attribute
            case UserMessage :: PROPERTY_USER_MESSAGE_STATUS :
                $status = Translation :: get(Utilities :: underscores_to_camelcase(UserMessage :: get_status_name(
                                                $message_with_optional_properties->get_optional_property(
                                                        UserMessage :: PROPERTY_USER_MESSAGE_STATUS)
                                        )
                                )
                );
                $return = '<img src="';
                $return .= UserMessage :: get_status_icon_path($message_with_optional_properties->get_optional_property(
                                        UserMessage :: PROPERTY_USER_MESSAGE_STATUS
                                ),
                                $message_with_optional_properties->is_discussion()
                );
                $return .= '" alt="';
                $return .= $status;
                $return .= '" title="';
                $return .= $status;
                $return .= '" />';
                return $return;

            // display image instead of attachment with number as alt-attribute
            case DefaultMessageTableColumnModel :: TABLE_HEADER_ATTACHMENT :
                $count = $message_with_optional_properties->count_attachments();
                if ($count > 0)
                {
                    $text = $count . ' ' . Translation :: get($count != 1 ? 'Attachments' : 'Attachments');
                    $return = '<img src="';
                    $return .= Theme :: get_image_path() . 'has_attachments.png';
                    $return .= '" alt="';
                    $return .= $text;
                    $return .= '" title="';
                    $return .= $text;
                    $return .= '" />';
                    return $return;
                }
                return ' ';

            // convert the boolean value of the function with some image
            case DefaultMessageTableColumnModel :: TABLE_HEADER_TYPE :
                $type = Translation :: get($message_with_optional_properties->is_discussion() ? 'Discussion' : 'Message');
                $return = '<img src="';
                $return .= $message_with_optional_properties->get_icon_path();
                $return .= '" alt="';
                $return .= $type;
                $return .= '" title="';
                $return .= $type;
                $return .= '" />';
                return $return;
        }

        return parent :: render_cell($column, $message_with_optional_properties);
    }

    /**
     * Gets the action links to display.
     * @param Message $message_with_optional_parameters The message for which the action links should be returned
     * @return String A HTML representation of the action links
     */
    private function get_modification_links($message_with_optional_properties)
    {
        // ACTION VIEW
        $view_url = $this->browser->get_message_view_url($message_with_optional_properties);
        $view_item = new ToolbarItem(
                        Translation :: get('View', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_image_path() . 'action_view.png',
                        $view_url,
                        ToolbarItem :: DISPLAY_ICON,
                        false
        );

        // ACTION MOVE
        $move_url = $this->browser->get_message_move_url($message_with_optional_properties, false);
        $move_item = new ToolbarItem(
                        Translation :: get('Move', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_move.png',
                        $move_url,
                        ToolbarItem :: DISPLAY_ICON,
                        false
        );

        // ACTION MOVE_NA
        $move_na_url = null;
        $move_na_item = new ToolbarItem(
                        Translation :: get('MoveNA', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_move_na.png',
                        $move_na_url,
                        ToolbarItem :: DISPLAY_ICON,
                        false
        );

        // ACTION UPDATE
        $update_url = $this->browser->get_message_update_url($message_with_optional_properties);
        $update_item = new ToolbarItem(
                        Translation :: get('Update', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_image_path() . 'action_edit.png',
                        $update_url,
                        ToolbarItem :: DISPLAY_ICON,
                        false
        );

        // ACTION REPLY
        $reply_url = $this->browser->get_message_reply_url($message_with_optional_properties);
        $reply_item = new ToolbarItem(
                        Translation :: get('Reply'),
                        Theme :: get_image_path() . 'action_reply.png',
                        $reply_url,
                        ToolbarItem :: DISPLAY_ICON,
                        false
        );

        // ACTION REPLY ALL
        $reply_all_url = $this->browser->get_message_reply_all_url($message_with_optional_properties);
        $reply_all_item = new ToolbarItem(
                        Translation :: get('ReplyAll'),
                        Theme :: get_image_path() . 'action_reply_all.png',
                        $reply_all_url,
                        ToolbarItem :: DISPLAY_ICON,
                        false
        );

        // ACTION FORWARD
        $forward_url = $this->browser->get_message_forward_url($message_with_optional_properties);
        $forward_item = new ToolbarItem(
                        Translation :: get('Forward'),
                        Theme :: get_image_path() . 'action_forward.png',
                        $forward_url,
                        ToolbarItem :: DISPLAY_ICON,
                        false
        );

        // ACTION MARK UNREAD
        $mark_unread_url = $this->browser->get_message_mark_unread_url($message_with_optional_properties, false);
        $mark_unread_item = new ToolbarItem(
                        Translation :: get('MarkSelectedUnread'),
                        Theme :: get_image_path() . 'action_mark_unread.png',
                        $mark_unread_url,
                        ToolbarItem :: DISPLAY_ICON,
                        false
        );

        // ACTION CREATE TEMPLATE
        $create_template_url = $this->browser->get_message_create_template_url($message_with_optional_properties);
        $create_template_item = new ToolbarItem(
                        Translation :: get('CreateTemplate'),
                        Theme :: get_image_path() . 'action_convert.png',
                        $create_template_url,
                        ToolbarItem :: DISPLAY_ICON,
                        false
        );

        // ACTION IGNORE
        $ignore_url = $this->browser->get_message_ignore_url($message_with_optional_properties, false);
        $ignore_item = new ToolbarItem(
                        Translation :: get('Ignore'),
                        Theme :: get_image_path() . 'action_ignore.png',
                        $ignore_url,
                        ToolbarItem :: DISPLAY_ICON,
                        false
        );

        // ACTION IGNORE AUTHOR
        $ignore_author_url = $this->browser->get_user_ignore_url($message_with_optional_properties->get_author_id());
        $ignore_author_item = new ToolbarItem(
                        Translation :: get('IgnoreAuthor'),
                        Theme :: get_image_path() . 'action_ignore_user.png',
                        $ignore_author_url,
                        ToolbarItem :: DISPLAY_ICON,
                        false
        );

        // ACTION UNIGNORE
        $unignore_url = $this->browser->get_message_unignore_url($message_with_optional_properties, false);
        $unignore_item = new ToolbarItem(
                        Translation :: get('Unignore'),
                        Theme :: get_image_path() . 'action_restore.png',
                        $unignore_url,
                        ToolbarItem :: DISPLAY_ICON,
                        false
        );

        // ACTION UNIGNORE AUTHOR
        $unignore_author_url = $this->browser->get_user_unignore_url($message_with_optional_properties->get_author_id());
        $unignore_author_item = new ToolbarItem(
                        Translation :: get('UnignoreAuthor'),
                        Theme :: get_image_path() . 'action_restore_user.png',
                        $unignore_author_url,
                        ToolbarItem :: DISPLAY_ICON,
                        false
        );

        // ACTION TRASH
        $trash_url = $this->browser->get_message_trash_url($message_with_optional_properties, false);
        $trash_item = new ToolbarItem(
                        Translation :: get('Trash'),
                        Theme :: get_image_path() . 'action_trash.png',
                        $trash_url,
                        ToolbarItem :: DISPLAY_ICON,
                        false
        );

        // ACTION UNTRASH
        $untrash_url = $this->browser->get_message_untrash_url($message_with_optional_properties, false);
        $untrash_item = new ToolbarItem(
                        Translation :: get('Untrash'),
                        Theme :: get_image_path() . 'action_untrash.png',
                        $untrash_url,
                        ToolbarItem :: DISPLAY_ICON,
                        false
        );

        // ACTION DELETE
        $delete_url = $this->browser->get_message_delete_url($message_with_optional_properties);
        $delete_item = new ToolbarItem(
                        Translation :: get('Delete', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_delete.png',
                        $delete_url,
                        ToolbarItem :: DISPLAY_ICON,
                        true
        );


        // CONSTRUCT TOOLBAR DEPENDING ON THE TYPE OF MESSAGE AND THE FOLDER
        $toolbar = new Toolbar(Toolbar :: TYPE_HORIZONTAL);

        // general available actions
        $toolbar->add_item($view_item);
        if (PersonalMessengerFolder :: supports_subcategories($this->folder))
        {
            $toolbar->add_item(UserCategoryManager :: user_has_categories_in_folder(
                            $this->browser->get_user_id(),
                            $this->folder) ? $move_item : $move_na_item
            );
        }

        // folder specific actions
        switch ($this->browser->get_folder())
        {
            case PersonalMessengerFolder :: INBOX :
                $toolbar->add_item($reply_item);
                $toolbar->add_item($reply_all_item);
                $toolbar->add_item($forward_item);
                $toolbar->add_item($ignore_item);
                $toolbar->add_item($create_template_item);
                $toolbar->add_item($trash_item);
                if ($message_with_optional_properties->get_optional_property(UserMessage :: PROPERTY_USER_MESSAGE_STATUS)
                        == UserMessage :: STATUS_READ
                )
                {
                    $toolbar->add_item($mark_unread_item);
                }
                break;

            case PersonalMessengerFolder :: DISCUSSIONS :
                $toolbar->add_item($reply_all_item);
                $toolbar->add_item($ignore_item);
                $toolbar->add_item($trash_item);
                if ($message_with_optional_properties->get_optional_property(UserMessage :: PROPERTY_USER_MESSAGE_STATUS)
                        == UserMessage :: STATUS_READ
                )
                {
                    $toolbar->add_item($mark_unread_item);
                }
                break;

            case PersonalMessengerFolder :: OUTBOX :
                $toolbar->add_item($forward_item);
                $toolbar->add_item($create_template_item);
                $toolbar->add_item($delete_item);
                break;

            case PersonalMessengerFolder :: IGNORED :
                $toolbar->add_item($unignore_item);
                if ($message_with_optional_properties->is_author_ignored())
                {
                    $toolbar->add_item($unignore_author_item);
                }
                else
                {
                    $toolbar->add_item($ignore_author_item);
                }
                $toolbar->add_item($delete_item);
                break;

            case PersonalMessengerFolder :: TRASH :
                $toolbar->add_item($untrash_item);
                $toolbar->add_item($delete_item);
                break;
        }

        return $toolbar->as_html();
    }

}

?>