<?php

namespace application\personal_messenger;

use common\libraries\ActionBarRenderer;
use common\libraries\Application;
use common\libraries\Display;
use common\libraries\EqualityCondition;
use common\libraries\Request;
use common\libraries\Theme;
use common\libraries\ToolbarItem;
use common\libraries\Translation;
use common\libraries\Utilities;
use user\UserDataManager;

/** * ***************************************************************************
 * This component offers the functionality to manage users.
 * It is the inteface class to:
 * - user_manager_ignored
 * - user_manager_discussion
 *
 * @name Chamilo Personal Messenger > PersonalMessengerManagerUserManagerComponent
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * @see PersonalMessengerManagerUserManagerIgnoredComponent
 * @sse PersonalMessengerManagerUserManagerDiscussionComponent
 * **************************************************************************** */
abstract class PersonalMessengerManagerUserManagerComponent extends PersonalMessengerManager
{

    //**************************************************************************
    // VARIABLES
    //**************************************************************************
    /**
     * Variable holding the action bar.
     * @var ActionBar
     */
    protected $action_bar;
    /**
     * The add form.
     * @var AddUserForm
     */
    protected $add_form;
    /**
     * The id of the message we want to manage.
     * @var int
     */
    protected $message_id;

    //**************************************************************************
    // CONSTRUCTOR
    //**************************************************************************
    /**
     * Constructor.
     */
    function __construct($user)
    {
        parent :: __construct($user);
        $this->message_id = Request :: get(self :: PARAM_MESSAGE_ID);
    }

    //**************************************************************************
    // RUN
    //**************************************************************************
    /**
     * Runs this component and displays its output.
     */
    function run()
    {
        // constructs the form
        $this->add_form = $this->get_add_form();

        // process form or display GUI
        if ($this->add_form->validate())
        {
            // process the form data
            $success = $this->add($this->add_form);

            // go back
            $go_back_parameters = array();
            if (Request :: get(self :: PARAM_REFERER))
            {
                $go_back_parameters[Application :: PARAM_ACTION] = Request :: get(self :: PARAM_REFERER);
                $go_back_parameters[self :: PARAM_MESSAGE_ID] = $this->message_id;
            }
            else
            {
                $go_back_parameters[Application :: PARAM_ACTION] = self :: DEFAULT_ACTION;
            }

            // redirect to this component
            $this->redirect(
                    Translation :: get(($success ? 'UsersUpdated' : 'UsersNotUpdated')),
                    ($success ? false : true),
                    $go_back_parameters
            );
        }
        else
        {
            // create action bar
            $this->action_bar = $this->get_action_bar();

            // header
            $this->display_header();

            // content
            echo $this->action_bar->as_html();

            // build view
            $this->display_content();

            // footer
            $this->display_footer();
        }
    }

    //**************************************************************************
    // GENERAL FUNCTIONS
    //**************************************************************************
    /**
     * Displays the content of this manager.
     */
    function display_content()
    {
        $html = array();

        // show before-usermanager GUI (depending on implementation)
        $html[] = $this->get_before_manager_gui();

        $html[] = '<div class="user_manager_division">';
        $html[] = '<span class="category">';
        $html[] = Translation :: get($this->get_form_category_text());
        $html[] = '</span>';
        $html[] = $this->add_form->toHtml();
        $html[] = '</div>';

        $html[] = '<div class="user_manager_division">';
        $html[] = '<span class="category">';
        $html[] = Translation :: get($this->get_table_category_text());
        $html[] = '</span>';
        $html[] = $this->get_users_table_html();
        $html[] = '</div>';

        // show after-usermanager GUI (depending on implementation)
        $html[] = $this->get_after_manager_gui();

        echo implode("\n", $html);
    }

    //**************************************************************************
    // ADDITIONAL FUNCTIONS
    //**************************************************************************
    /**
     * Returns the add form.
     * @return user_add_form
     */
    private function get_add_form()
    {
        return new SelectUserForm(
                $this,
                $this->get_url(),
                $this->get_submit_button_text(),
                $this->get_default_hidden_users(),
                $this->get_default_selected(),
                true
        );
    }

    /**
     * Returns the HTML representation of all users depending on the type of this manager.
     * @return String HTML
     */
    private function get_users_table_html()
    {
        $parameters = $this->get_parameters();

        // construct condition
        $condition = $this->get_base_condition();

        // append the base condition using the search box
        if ($this->get_search_condition())
        {
            $conditions = array();
            $conditions[] = $condition;
            $conditions[] = $this->get_search_condition();
            $condition = new AndCondition($conditions);
        }

        // create table
        $table = $this->get_table($parameters, $condition);
        return $table->as_html();
    }

    /**
     * Constructs the conditions for the search field.
     * @return Condition
     */
    function get_search_condition()
    {
        $query = $this->action_bar->get_query();
        if (isset($query) && $query != '')
        {
            $conditions = array();
            $conditions[] = new PatternMatchCondition(
                            User :: PROPERTY_FIRSTNAME,
                            '*' . $query . '*'
            );
            $conditions[] = new PatternMatchCondition(
                            User :: PROPERTY_LASTNAME,
                            '*' . $query . '*'
            );
            return new OrCondition($conditions);
        }

        return null;
    }

    /**
     * Returns the action bar.
     * @return ActionBar
     */
    function get_action_bar()
    {
        // default bar
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);

        // additional functions
        // SEARCH
        $action_bar->set_search_url($this->get_url());
        $show_all_url = $this->get_url();
        $show_all_item = new ToolbarItem(
                        Translation :: get('ShowAll', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_browser.png',
                        $show_all_url,
                        ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                        false
        );

        $action_bar->add_common_action($show_all_item);

        // return
        return $action_bar;
    }

    /**
     * Adds the additional parameters to the URL.
     * @return array
     */
    function get_additional_parameters()
    {
        return array(self :: PARAM_REFERER, self :: PARAM_MESSAGE_ID);
    }

    //**************************************************************************
    // FUNCTIONS TO OVERRIDE (IF NECESSARY)
    //**************************************************************************
    /**
     * Returns HTML of some specific content to display before the user manager GUI.
     * @return string HTML
     */
    function get_before_manager_gui()
    {
        // nothing to show...
    }

    /**
     * Returns HTML of some specific content to display after the user manager GUI.
     * @return string HTML
     */
    function get_after_manager_gui()
    {
        // nothing to show...
    }

    //**************************************************************************
    // ABSTRACT FUNCTIONS
    //**************************************************************************
    /**
     * Returns the base condition for this manager depending on the type.
     * @return Condition
     */
    abstract function get_base_condition();

    /**
     * Returns the table depending on the type.
     * @param array $parameters
     * @param Condition $condition
     * @return table
     */
    abstract function get_table($parameters, $condition);

    /**
     * Returns the default selected values in the user selector depending on the type.
     * @return array
     */
    abstract function get_default_selected();

    /**
     * Retuns the default hidden values in the user selector depending on the type.
     * @return array
     */
    abstract function get_default_hidden_users();

    /**
     * Returns the text for the category division of the form depending on the type.
     * @return string The translation key for the button text.
     */
    abstract function get_table_category_text();

    /**
     * Returns the text for the form category division.
     * @return string The translation key for the form category text.
     */
    abstract function get_form_category_text();

    /**
     * Returns the text for the submit button of the form depending on the type.
     * @return string The translation key for the button text.
     */
    abstract function get_submit_button_text();

    /**
     * Adds the given users depending on the type.
     * @param array $user_ids
     * @return boolean
     */
    abstract function add($form);

    /**
     * Deletes the given users depending on the type.
     * @param array $user_ids
     * @return boolean
     */
    abstract function delete($form);
}

?>