<?php

namespace application\personal_messenger;

use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\AndCondition;
use common\libraries\Display;
use common\libraries\EqualityCondition;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;
use group\GroupDataManager;
use user\UserDataManager;

/** * ***************************************************************************
 * Actual implementation of the UserManager for discussion users.
 *
 * @name Chamilo Personal Messenger > PersonalMessengerManagerUserManagerDiscussionComponent
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * @see PersonalMessengerManagerUserManagerComponent
 * **************************************************************************** */
class PersonalMessengerManagerUserManagerDiscussionComponent extends PersonalMessengerManagerUserManagerComponent
{

    //**************************************************************************
    // CONSTRUCTOR
    //**************************************************************************
    /**
     * Constructor.
     */
    function __construct($user)
    {
        parent :: __construct($user);
        Request :: set_get(self :: PARAM_REFERER, self :: ACTION_MANAGE_USERS_DISCUSSION);
        $this->check();
    }

    //**************************************************************************
    // INHERITED FUNCTIONS
    //**************************************************************************
    /**
     * Checks if all required data is present.
     * We have to have a message, and link to the message.
     */
    function check()
    {
        if (!$this->message_id)
        {
            $this->display_header();
            Display :: error_message(Translation :: get('NoMessageSelected'));
            $this->display_footer();
            exit();
        }
        else
        {
            $message = $this->retrieve_message_by_id($this->message_id);
            if (!$message->has_user_message_relation($this->get_user_id()))
            {
                $this->display_header();
                Display :: error_message(Translation :: get('NotAllowed', null, Utilities :: COMMON_LIBRARIES));
                $this->display_footer();
                exit();
            }
        }
    }

    /**
     * Override.
     * Returns HTML of some specific content to display before the user manager GUI.
     * @return string HTML
     */
    function get_before_manager_gui()
    {
        // show the discussion root information
        $message = $this->retrieve_message_by_id($this->message_id);
        $html[] = '<div class="user_manager_division">';
        $html[] = '<span class="category">';
        $html[] = Translation :: get('MessageDetails');
        $html[] = '</span>';
        $html[] = '<table>';
        $html[] = '<tr><td class="header_title">' . Translation :: get('DiscussionFrom');
        $html[] = '</td><td class="header_value">' . $message->get_author_name() . '</td></tr>';
        $html[] = '<tr><td class="header_title">' . Translation :: get('DiscussionTitle');
        $html[] = '</td><td class="header_value">' . $message->get_title() . '</td></tr>';
        $html[] = '<tr><td class="header_title">' . Translation :: get('AllDiscussionRecipients');
        $html[] = '</td><td class="header_value">' . implode(', ', $message->get_recipient_names()) . '</td></tr>';
        $html[] = '</table>';
        $html[] = '</div>';

        return implode("\n", $html);
    }

    /**
     * Returns the base condition for ignored user managers.
     * @return Condition
     */
    function get_base_condition()
    {
        $dm = PersonalMessengerDataManager :: get_instance();
        return new EqualityCondition(
                UserMessage :: PROPERTY_USER_MESSAGE_MESSAGE_ID,
                $this->message_id,
                $dm->get_alias(UserMessage :: get_table_name()),
                true
        );
    }

    /**
     * Returns a DiscussionUserTable.
     * @param array $parameters
     * @param Condition $condition
     * @return table
     */
    function get_table($parameters, $condition)
    {
        return new DiscussionUserTable($this, $this->retrieve_message_by_id($this->message_id), $parameters, $condition);
    }

    /**
     * Returns the default selected values in the user selector.
     * @return array
     */
    function get_default_selected()
    {
        return array();
    }

    /**
     * Retuns the default hidden values in the user selector.
     * @return array
     */
    function get_default_hidden_users()
    {
        // construct condition
        $condition = new EqualityCondition(
                        UserMessage :: PROPERTY_USER_MESSAGE_MESSAGE_ID,
                        $this->message_id
        );

        // retrieve users
        $users_messages = PersonalMessengerDataManager :: get_instance()->retrieve_all_user_messages($condition);

        // construct user_group_finder elements from users
        $default_hidden = array();
        while ($users_message = $users_messages->next_result())
        {
            $default_hidden[] = $users_message->get_user_id();
        }

        // return the ids
        return $default_hidden;
    }

    /**
     * Returns the text for the table category division.
     * @return string The translation key for the table category text.
     */
    function get_table_category_text()
    {
        return 'ActiveParticipants';
    }

    /**
     * Returns the text for the form category division.
     * @return string The translation key for the form category text.
     */
    function get_form_category_text()
    {
        return 'AddParticipant';
    }

    /**
     * Returns the text for the submit button.
     * @return string The translation key for the button text.
     */
    function get_submit_button_text()
    {
        return 'AddUsers';
    }

    /**
     * Add the given users.
     * @param array $user_ids
     * @return boolean
     */
    function add($form)
    {
        return ($this->add_users(
                $this->retrieve_message_by_id($this->message_id),
                $form->get_selected_users(),
                $form->get_selected_groups()
        ) == 0);
    }

    /**
     * Delete the given users.
     * @param array $user_ids
     * @return boolean
     */
    function delete($form)
    {
        return ($this->delete_users(
                $this->retrieve_message_by_id($this->message_id),
                $form->get_selected_users()
        ) == 0);
    }

    //**************************************************************************
    // ADDITIONAL FUNCTIONS
    //**************************************************************************
    /**
     * Adds the additional breadcrumbs for navigation.
     * @param BreadcrumbTrail $breadcrumbtrail
     */
    function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $browser_folder_url = $this->get_browser_url($this->get_folder(), $this->get_category_id());

        // browser
        $breadcrumbtrail->add(new Breadcrumb(
                        $browser_folder_url,
                        Translation :: get('PersonalMessengerManagerBrowserComponent')
        ));

        // folder
        $breadcrumbtrail->add(new Breadcrumb(
                        $browser_folder_url,
                        Translation :: get(Utilities:: underscores_to_camelcase(
                                        PersonalMessengerFolder :: get_folder_name($this->get_folder()))
                        )
        ));

        // message
        if ($this->message_id)
        {
            $message = $this->retrieve_message_by_id($this->message_id);
            if ($message)
            {
                $message_url = $this->get_message_view_url($message);
                $breadcrumbtrail->add(new Breadcrumb(
                                $message_url,
                                Translation :: get(
                                        $message->is_discussion() ? 'PersonalMessengerManagerViewerDiscussionComponent' : 'PersonalMessengerManagerViewerMessageComponent'
                                )
                ));
            }
        }
    }

}

?>