<?php

namespace application\personal_messenger;

use common\libraries\ActionBarRenderer;
use common\libraries\AndCondition;
use common\libraries\Application;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\DatetimeUtilities;
use common\libraries\Display;
use common\libraries\EqualityCondition;
use common\libraries\Request;
use common\libraries\Theme;
use common\libraries\ToolbarItem;
use common\libraries\Translation;
use common\libraries\Utilities;

/** * ***************************************************************************
 * This component offers the functionality to view a single message.
 * It is an abstract class, therefore a specific implementation must be provided.
 *
 * @name Chamilo Personal Messenger > PersonalMessengerManagerViewerComponent
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * @see ViewerDiscussion
 * @see ViewerMessage
 * **************************************************************************** */
abstract class PersonalMessengerManagerViewerComponent extends PersonalMessengerManager
{
    //**************************************************************************
    // GENERAL CONSTANTS
    //**************************************************************************
    /**
     * String put in front of the attachment link.
     * Used in retrieve_new_replies ajax.
     */
    const ATTACHMENT_PREFIX = '[ ';

    /**
     * String put at the end of the attachment link.
     * Used in retrieve_new_replies ajax.
     */
    const ATTACHMENT_SUFFIX = ' ]';

    /**
     * String put between 2 messages.
     * e.g. switch between ' ' and '<br/>' to display each attachment on a the same, or a separate line.
     * Used in retrieve_new_replies ajax.
     */
    const ATTACHMENT_SEPARATOR = '<br/>';

    /**
     * Indicates if attachments should be opened in a popup dialog.
     * Used in retrieve_new_replies ajax.
     */
    const ATTACHMENT_POPUP = true;
    
    /**
     * Indicates if the quick reply should use the AJAX form, or the FormValidator form.
     */
    const USE_AJAX_QUICK_REPLY = true;

    //**************************************************************************
    // VARIABLES
    //**************************************************************************
    /**
     * The message.
     * @var Message
     */
    protected $message;

    //**************************************************************************
    // RUN
    //**************************************************************************
    /**
     * Runs this component and displays its output.
     */
    function run()
    {
        // simple step by step building of the viewer, depending on abstract functions
        // to provide viewer component specific structures.
        $this->message = $this->get_message();
        $this->check_viewer_component();
        $this->process_forms();
        $this->display_header();
        echo $this->get_action_bar()->as_html();
        $this->start_message();
        $this->build_basic_viewer();
        $this->add_viewer_additionals();
        $this->end_message();
        $this->update_status();
        $this->display_footer();
    }

    //**************************************************************************
    // GENERAL FUNCTIONS
    //**************************************************************************
    /**
     * Returns the message after performing some checks.
     * @return Message
     */
    function get_message()
    {
        // get message id
        $message_id = Request :: get(PersonalMessengerManager :: PARAM_MESSAGE_ID);
        if (!$message_id)
        {
            $this->display_header();
            Display :: error_message(Translation :: get('NoMessageSelected'));
            $this->display_footer();
            exit();
        }
        else
        {
            // retrieve the entire message, or the root depending on the implementation
            $message = $this->get_root_message($message_id);

            // if there is no message
            if (!$message)
            {
                $this->display_header();
                Display :: error_message(Translation :: get('NoMessageSelected'));
                $this->display_footer();
                exit();
            }
            else
            {
                // create user message conditions
                $conditions = array();
                $conditions[] = new EqualityCondition(
                                UserMessage :: PROPERTY_USER_MESSAGE_MESSAGE_ID,
                                $message->get_id()
                );
                $conditions[] = new EqualityCondition(
                                UserMessage :: PROPERTY_USER_MESSAGE_USER_ID,
                                $this->get_user_id()
                );
                $user_message_condition = new AndCondition($conditions);

                // retrieve the user message
                $user_message = $this->retrieve_user_message($user_message_condition);

                // check if there is a user message
                if (!$user_message)
                {
                    $this->display_header();
                    Display :: error_message(Translation :: get('NotAllowed', null, Utilities :: COMMON_LIBRARIES));
                    $this->display_footer();
                    exit();
                }
                else
                {
                    // return the message
                    return $message;
                }
            }
        }
    }

    /**
     * Simple check to redirect to the proper viewer in case the user modifies the link.
     * @param Message $message
     */
    function check_viewer_component()
    {
        if ($this->message->is_discussion())
        {
            if ($this instanceof PersonalMessengerManagerViewerMessageComponent)
            {
                // we try to view a discussion in a message viewer -> redirect
                $this->simple_redirect(
                        array(
                            Application :: PARAM_ACTION => PersonalMessengerManager :: ACTION_VIEW_DISCUSSION,
                            PersonalMessengerManager :: PARAM_MESSAGE_ID => $this->message->get_id()
                        )
                );
            }
        }
        else
        {
            if ($this instanceof PersonalMessengerManagerViewerDiscussionComponent)
            {
                // we try to view a message in a discussion viewer -> redirect
                $this->simple_redirect(
                        array(
                            Application :: PARAM_ACTION => PersonalMessengerManager :: ACTION_VIEW_MESSAGE,
                            PersonalMessengerManager :: PARAM_MESSAGE_ID => $this->message->get_id()
                        )
                );
            }
        }
    }

    /**
     * Structural part 1: The start of the global message division.
     */
    function start_message()
    {
        echo $this->get_global_div();
    }

    /**
     * Structural part 2: The basic viewer that is the same in all viewing implementations.
     * @param Message $message
     */
    function build_basic_viewer()
    {
        //echo '<div class="clear"></div><br />';
        // display the message
        echo $this->get_message_html();
    }

    /*
     * Structural part 3: Additions (viewer dependant) -> see abstract functions.
     */

    /**
     * Structural part 4: The end of the message division.
     */
    function end_message()
    {
        echo '</div>';
    }

    /**
     * Update the message status.
     */
    function update_status()
    {
        // create user message conditions
        $conditions = array();
        $conditions[] = new EqualityCondition(
                        UserMessage :: PROPERTY_USER_MESSAGE_MESSAGE_ID,
                        $this->message->get_id()
        );
        $conditions[] = new EqualityCondition(
                        UserMessage :: PROPERTY_USER_MESSAGE_USER_ID,
                        $this->get_user_id()
        );
        $user_message_condition = new AndCondition($conditions);

        // retrieve the user message
        $user_message = $this->retrieve_user_message($user_message_condition);

        // update status
        if ($user_message->get_status(UserMessage :: PROPERTY_USER_MESSAGE_STATUS) == UserMessage :: STATUS_UNREAD)
        {
            $user_message->set_status(UserMessage :: STATUS_READ);
            $user_message->update();
        }
    }

    /**
     * Renders the message and returns its HTML.
     * @param Message $message
     * @return String HTML
     */
    function get_message_html()
    {
        $html = array();

        // header
        // - start DIV title
        $html[] = '<div class="title">';
        if ($this->message->get_title())
        {
            $html[] = $this->message->get_title();
        }
        else
        {
            $html[] = '< ' . Translation :: get('NoTitle') . ' >';
        }
        // - end DIV title
        $html[] = '</div>';

        // - start DIV headerdata
        $html[] = '<div class="headerdata">';

        $html[] = '<table>';

        // -- author
        $html[] = '<tr>';
        $html[] = '<td class="header_title">' . Translation :: get('MessageFrom') . '</td><td class="header_value">';
        $author = $this->message->get_author_name();
        if ($author)
        {
            $html[] = $author;
        }
        else
        {
            $html[] = Translation :: get('SenderUnknown');
        }
        $html[] = '</td>';

        $html[] = '</tr><tr>';

        // -- recipients
        $html[] = '<td class="header_title">' . Translation :: get('MessageTo') . '</td><td class="header_value">';
        $recipients = $this->message->get_recipient_names();
        if ($recipients)
        {
            $html[] = implode(', ', $recipients);
        }
        else
        {
            $html[] = Translation :: get('RecipientUnknown');
        }
        $html[] = '</td>';

        $html[] = '</tr><tr>';

        // -- date
        $html[] = '<td class="header_title">' . Translation :: get('MessageDate') . '</td><td class="header_value">';
        $html[] = DatetimeUtilities :: format_locale_date(
                        Translation :: get('DateFormatShort', null, Utilities :: COMMON_LIBRARIES) .
                        ', ' . Translation :: get('TimeNoSecFormat', null, Utilities :: COMMON_LIBRARIES),
                        $this->message->get_modification_time()
        );
        $html[] = '</td>';

        $html[] = '</tr><tr>';

        // -- attachments
        if ($this->message->count_attachments() > 0)
        {
            $html[] = '<td class="header_title">';
            $html[] = '<img src="' . Theme :: get_image_path() . 'has_attachments.png"';
            $html[] = 'alt="' . Translation :: get('Attachments') . '"/>';
            $html[] = '</td><td class="header_value">';
            $html[] = $this->render_attachments($this->message);
            $html[] = '</td>';
        }

        $html[] = '</tr></table>';

        // - end DIV headerdata
        $html[] = '</div>';

        $html[] = '<hr/>';

        // - start DIV message
        $html[] = '<div class="message">';
        // message
        $html[] = $this->message->get_content();
        // - end DIV title
        $html[] = '</div>';

        return implode("\n", $html);
    }

    /**
     * Renders the attachments of the given message, providing links to the attachment viewer component.
     * @param Message $message
     * @return String HTML
     */
    function render_attachments($message)
    {
        $html = array();

        // add each element + separator
        foreach ($message->get_message_attachments() as $message_attachment)
        {
            $content_object = $message_attachment->retieve_attachment_content_object();
            if ($content_object)
            {
                $url = $this->get_attachment_viewer_url(
                                $message->get_id(),
                                $content_object->get_id(),
                                self :: ATTACHMENT_POPUP
                );
                $html[] = self :: ATTACHMENT_PREFIX;
                if (self :: ATTACHMENT_POPUP)
                {
                    $url = 'javascript:openPopup(\'' . $url . '\'); return false;';
                    $html[] = '<a href="#" onClick="' . $url . '">';
                }
                else
                {
                    $html[] = '<a href="' . $url . '">';
                }
                $html[] = $content_object->get_title() . ' (' . $content_object->get_type() . ')</a>';
                $html[] = self :: ATTACHMENT_SUFFIX;
                $html[] = self :: ATTACHMENT_SEPARATOR;
            }
        }

        // remove last separator
        array_pop($html);

        return implode("\n", $html);
    }

    //**************************************************************************
    // ADDITIONAL FUNCTIONS
    //**************************************************************************
    /**
     * Returns the action bar.
     * @return ActionBar
     */
    function get_action_bar()
    {
        // default bar
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);

        $folder = $this->message->get_folder();

        // additional functions
        // REPLY
        $reply_url = $this->get_message_reply_url(
                        $this->message,
                        true,
                        true,
                        Request :: get(self :: PARAM_REFERER)
        );
        $reply_item = new ToolbarItem(
                        Translation :: get('Reply'),
                        Theme :: get_image_path() . 'action_reply.png',
                        $reply_url,
                        ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                        false
        );

        // REPLY ALL
        $reply_all_url = $this->get_message_reply_all_url(
                        $this->message,
                        (!$this->message->is_discussion()),
                        (!$this->message->is_discussion()),
                        Request :: get(self :: PARAM_REFERER)
        );
        $reply_all_item = new ToolbarItem(
                        Translation :: get('ReplyAll'),
                        Theme :: get_image_path() . 'action_reply_all.png',
                        $reply_all_url,
                        ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                        false
        );

        // FORWARD
        $forward_url = $this->get_message_forward_url(
                        $this->message,
                        Request :: get(self :: PARAM_REFERER)
        );
        $forward_item = new ToolbarItem(
                        Translation :: get('Forward'),
                        Theme :: get_image_path() . 'action_forward.png',
                        $forward_url,
                        ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                        false
        );

        // IGNORE
        $ignore_url = $this->get_message_ignore_url(
                        $this->message,
                        true,
                        Request :: get(self :: PARAM_REFERER)
        );
        $ignore_item = new ToolbarItem(
                        Translation :: get('Ignore'),
                        Theme :: get_image_path() . 'action_ignore.png',
                        $ignore_url,
                        ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                        false
        );

        // IGNORE AUTHOR
        $ignore_author_url = $this->get_user_ignore_url(
                        $this->message->get_author_id(),
                        Request :: get(self :: PARAM_REFERER)
        );
        $ignore_author_item = new ToolbarItem(
                        Translation :: get('IgnoreAuthor'),
                        Theme :: get_image_path() . 'action_ignore_user.png',
                        $ignore_author_url,
                        ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                        false
        );

        // UNIGNORE
        $unignore_url = $this->get_message_unignore_url(
                        $this->message,
                        true,
                        Request :: get(self :: PARAM_REFERER)
        );
        $unignore_item = new ToolbarItem(
                        Translation :: get('Unignore'),
                        Theme :: get_image_path() . 'action_restore.png',
                        $unignore_url,
                        ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                        false
        );

        // UNIGNORE AUTHOR
        $unignore_author_url = $this->get_user_unignore_url(
                        $this->message->get_author_id(),
                        Request :: get(self :: PARAM_REFERER)
        );
        $unignore_author_item = new ToolbarItem(
                        Translation :: get('UnignoreAuthor'),
                        Theme :: get_image_path() . 'action_restore_user.png',
                        $unignore_author_url,
                        ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                        false
        );

        // TRASH
        $trash_url = $this->get_message_trash_url(
                        $this->message,
                        true,
                        Request :: get(self :: PARAM_REFERER)
        );
        $trash_item = new ToolbarItem(
                        Translation :: get('Trash'),
                        Theme :: get_image_path() . 'action_trash.png',
                        $trash_url,
                        ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                        false
        );

        // UNTRASH
        $untrash_url = $this->get_message_untrash_url(
                        $this->message,
                        true,
                        Request :: get(self :: PARAM_REFERER)
        );
        $untrash_item = new ToolbarItem(
                        Translation :: get('Untrash'),
                        Theme :: get_image_path() . 'action_untrash.png',
                        $untrash_url,
                        ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                        false
        );

        // DELETE
        $delete_url = $this->get_message_delete_url(
                        $this->message
        );
        $delete_item = new ToolbarItem(
                        Translation :: get('Delete', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_delete.png',
                        $delete_url,
                        ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                        true
        );

        // CONSTRUCT TOOLBAR
        // actions specific for messages / discussions
        if ($this->message->is_discussion())
        {
            $action_bar->add_common_action($reply_all_item);
        }
        else
        {
            $action_bar->add_common_action($reply_item);
            $action_bar->add_common_action($reply_all_item);
            $action_bar->add_common_action($forward_item);
        }

        // actions determined by status
        switch ($this->message->get_status())
        {
            case UserMessage :: STATUS_READ :
                $action_bar->add_common_action($trash_item);
                $action_bar->add_common_action($ignore_item);
                break;
            case UserMessage :: STATUS_UNREAD :
                $action_bar->add_common_action($trash_item);
                $action_bar->add_common_action($ignore_item);
                break;
            case UserMessage :: STATUS_IGNORE :
                $action_bar->add_common_action($unignore_item);
                break;
            case UserMessage :: STATUS_TRASH :
                $action_bar->add_common_action($untrash_item);
                break;
            default :
                break;
        }

        // author ignore
        if ($this->message->is_author_ignored())
        {
            $action_bar->add_tool_action($unignore_author_item);
        }
        else
        {
            $action_bar->add_tool_action($ignore_author_item);
        }

        // if outbox/ignored/trash -> delete
        if ($folder == PersonalMessengerFolder :: OUTBOX
                || $folder == PersonalMessengerFolder :: IGNORED
                || $folder == PersonalMessengerFolder :: TRASH)
        {
            $action_bar->add_tool_action($delete_item);
        }

        // return
        return $action_bar;
    }

    /**
     * Adds the additional parameters to the URL.
     * @return array
     */
    function get_additional_parameters()
    {
        return array(self :: PARAM_REFERER, self :: PARAM_MESSAGE_ID, self :: PARAM_REPLY_ID);
    }

    /**
     * Adds the additional breadcrumbs for navigation.
     * @param BreadcrumbTrail $breadcrumbtrail
     */
    function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $browser_folder_url = $this->get_browser_url($this->get_folder(), $this->get_category_id());

        // browser
        $breadcrumbtrail->add(new Breadcrumb(
                        $browser_folder_url,
                        Translation :: get('PersonalMessengerManagerBrowserComponent')
        ));

        // folder
        $breadcrumbtrail->add(new Breadcrumb(
                        $browser_folder_url,
                        Translation :: get(Utilities:: underscores_to_camelcase(
                                        PersonalMessengerFolder :: get_folder_name($this->get_folder())
                        ))));
    }

    //**************************************************************************
    // ABSTRACT FUNCTIONS
    //**************************************************************************
    /**
     * Returns the root message.
     * - MessageViewer: no changes
     * - DiscussionViewer: always returns the root
     * @return Message
     */
    abstract function get_root_message($message_id);

    /**
     * Returns the global division declaration
     * @return String HTML
     */
    abstract function get_global_div();

    /**
     * Adds additional features specific to the viewer implementation.
     */
    abstract function add_viewer_additionals();

    /**
     * Processes the forms and redirect before headers are sent.
     */
    abstract function process_forms();
}

?>