<?php
namespace application\weblcms\course;

use common\libraries\Configuration;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\SubselectCondition;
use common\libraries\AndCondition;
use common\libraries\InCondition;
use common\libraries\NotCondition;

use application\weblcms\WeblcmsDataManager;
use application\weblcms\CourseSettingsController;
use application\weblcms\CourseSetting;
use application\weblcms\CourseTool;
use application\weblcms\CourseSection;

/**
 * This class describes the datamanager for the course submanager (extension on weblcms data manager)
 *
 * @package \application\weblcms\course
 * @author Sven Vanpoucke - Hogeschool Gent
 */
class CourseDataManager extends WeblcmsDataManager
{
    /**
     * Instance of the class, for the singleton pattern.
     */
    private static $instance;

    /**
     * **************************************************************************************************************
     * Inherited Functionality *
     * **************************************************************************************************************
     */

    /**
     * Creates and returns an instance of a course data manager implementation
     *
     * @return CourseDataManagerInterface
     */
    static function get_instance()
    {
        if (! isset(self :: $instance))
        {
            $type = Configuration :: get_instance()->get_parameter('general', 'data_manager');
            $file = dirname(__FILE__) . '/implementation/' . strtolower($type) . '.class.php';

            if (! file_exists($file))
            {
                throw new \Exception(Translation :: get('DataManagerImplementationNotFound', array('TYPE' => $type), Utilities :: COMMON_LIBRARIES));
            }

            $class = __NAMESPACE__ . '\\' . Utilities :: underscores_to_camelcase($type) . 'CourseDataManager';

            self :: $instance = new $class();
        }

        return self :: $instance;
    }

    /**
     * **************************************************************************************************************
     * Course Functionality *
     * **************************************************************************************************************
     */

    /**
     * Updates the courses from a given course type with the given properties
     *
     * @param $course_type_id int
     * @param $properties string[]
     *
     * @return boolean
     */
    static function update_courses_from_course_type_with_properties($course_type_id = 0, $properties = array())
    {
        if (count($properties) == 0)
        {
            throw new \Exception('No properties selected to update the courses from a course type');
        }

        $instance = self :: get_instance();

        foreach ($properties as $key => $property)
        {
            $properties[$key] = $instance->quote($property);
        }

        $condition = new EqualityCondition(Course :: PROPERTY_COURSE_TYPE_ID, $course_type_id);

        return $instance->updates(Course :: class_name(), $properties, $condition);
    }

    /**
     * Counts the courses for a given course type
     *
     * @param $course_type_id int - [OPTIONAL] default: 0
     */
    static function count_courses_from_course_type($course_type_id = 0)
    {
        $condition = new EqualityCondition(Course :: PROPERTY_COURSE_TYPE_ID, $course_type_id);

        return self :: get_instance()->count_courses($condition);
    }

    /**
     * **************************************************************************************************************
     * CourseUserRelation Functionality *
     * **************************************************************************************************************
     */

    /**
     * Deletes the course user relations for given user and course(s)
     *
     * @param $user_id int
     * @param $course_ids int[] - [OPTIONAL] default empty array
     *
     * @return boolean
     */
    static function delete_course_user_relations_for_user_and_courses($user_id, $course_ids = array())
    {
        $conditions = array();

        $conditions[] = new EqualityCondition(CourseUserRelation :: PROPERTY_USER_ID, $user_id);

        $course_ids = (array) $course_ids;

        if (count($course_ids) > 0)
        {
            $conditions[] = new InCondition(CourseUserRelation :: PROPERTY_COURSE_ID, $course_ids);
        }

        $condition = new AndCondition($conditions);

        return self :: get_instance()->delete(CourseUserRelation :: get_table_name(), $condition);
    }

    /**
     * Returns whether or not a user is directly subscribed to a course
     *
     * @param $user_id int
     * @param $course_id int
     *
     * @return boolean
     */
    static function is_user_direct_subscribed_to_course($user_id, $course_id)
    {
        $conditions = array();

        $conditions[] = new EqualityCondition(CourseUserRelation :: PROPERTY_USER_ID, $user_id);
        $conditions[] = new EqualityCondition(CourseUserRelation :: PROPERTY_COURSE_ID, $course_id);

        $condition = new AndCondition($conditions);

        return (self :: get_instance()->count_objects(CourseUserRelation :: get_table_name(), $condition) > 0);
    }

    /**
     * **************************************************************************************************************
     * CourseRelCourseSettingValue Functionality *
     * **************************************************************************************************************
     */

    /**
     * Copies the course settings to all the courses from a given course type Can be limited by a given course setting
     *
     * @param $course_type_id int
     * @param $course_setting_id int - [OPTIONAL] default null
     */
    static function copy_course_settings_from_course_type($course_type_id, $course_setting_id = null)
    {
        $course_settings_controller = CourseSettingsController :: get_instance();

        if (! self :: delete_values_for_course_setting_and_course_type($course_type_id, $course_setting_id))
        {
            return false;
        }

        $course_setting_relations = self :: retrieve_course_setting_relations_from_course_type($course_type_id, $course_setting_id);

        $succes = true;

        while ($course_setting_relation = $course_setting_relations->next_result())
        {
            $course_setting_id = $course_setting_relation->get_course_setting_id();
            $course_setting = $course_settings_controller->get_course_setting_by_id($course_setting_id);

            $values = $course_settings_controller->get_course_type_setting($course_type_id, $course_setting->get_name(), $course_setting->get_tool_id());

            $values = (array) $values;

            foreach ($values as $value)
            {
                try
                {
                    $course_setting_relation->add_course_setting_value($value);
                }
                catch (Exception $e)
                {
                    $succes = false;
                }
            }
        }

        return $succes;
    }

    /**
     * Deletes the values of a given course setting for all the courses that belong to the given course type id
     *
     * @param $course_type_id int - [OPTIONAL] default 0
     * @param $course_setting_id int - [OPTIONAL] default null
     * @return boolean
     */
    static function delete_values_for_course_setting_and_course_type($course_type_id = 0, $course_setting_id = null)
    {
        $course_rel_course_setting_condition = self :: get_condition_for_course_settings_from_course_type($course_type_id, $course_setting_id);

        /**
         * Course rel course setting value condition
         */
        $condition = new SubselectCondition(CourseRelCourseSettingValue :: PROPERTY_COURSE_REL_COURSE_SETTING_ID, CourseRelCourseSetting :: PROPERTY_ID, CourseRelCourseSetting :: get_table_name(), $course_rel_course_setting_condition);

        /**
         * Delete values
         */
        return self :: get_instance()->delete(CourseRelCourseSettingValue :: get_table_name(), $condition);
    }

    /**
     * Returns the course setting relations for a given course type Can be limited by a given course setting
     *
     * @param $course_type_id int - [OPTIONAL] default 0
     * @param $course_setting_id int - [OPTIONAL] default null
     * @return ResultSet<CourseRelCourseSetting>
     */
    static function retrieve_course_setting_relations_from_course_type($course_type_id = 0, $course_setting_id = null)
    {
        $condition = self :: get_condition_for_course_settings_from_course_type($course_type_id, $course_setting_id);

        return self :: get_instance()->retrieve_objects(CourseRelCourseSetting :: get_table_name(), $condition);
    }

    /**
     * **************************************************************************************************************
     * CourseRelCourseSettingValue Helper Functionality *
     * **************************************************************************************************************
     */

    /**
     * Returns the condition for the course settings for all the courses connected to a given course type Can be limited
     * by a given course setting
     *
     * @param $course_type_id int - [OPTIONAL] default 0
     * @param $course_setting_id int - [OPTIONAL] default null
     * @return Condition
     */
    static function get_condition_for_course_settings_from_course_type($course_type_id = 0, $course_setting_id = null)
    {
        /**
         * Courses condition
         */
        $courses_condition = new EqualityCondition(Course :: PROPERTY_COURSE_TYPE_ID, $course_type_id);

        /**
         * Course rel course setting conditions
         */
        $course_rel_course_setting_conditions = array();

        $course_rel_course_setting_conditions[] = new SubselectCondition(CourseRelCourseSetting :: PROPERTY_COURSE_ID, Course :: PROPERTY_ID, Course :: get_table_name(), $courses_condition);

        if (! is_null($course_setting_id))
        {
            $course_rel_course_setting_conditions[] = new EqualityCondition(CourseRelCourseSetting :: PROPERTY_COURSE_SETTING_ID, $course_setting_id);
        }

        return new AndCondition($course_rel_course_setting_conditions);
    }
    
    
    /*
     * set the tool visible = 1 / invisible = 0 by course_id and tool_name
     * if tool_name is null, all the tools in the course will be set (except Home tool)
     */
    static function set_tool_visibility_by_tool_name($course_id, $tool_name = null, $visible = 1)
    {
        $tools_condition = new NotCondition( //filter Home tool, always visible!
            new EqualityCondition(
                CourseTool :: PROPERTY_SECTION_TYPE,
                CourseSection :: TYPE_CUSTOM
            )
        );
        $tools = WeblcmsDataManager :: get_instance()->retrieve_course_tools($tools_condition);
        while ($tool = $tools->next_result())
        {
            if(!$tool_name || $tool_name == $tool->get_name()){
                 self :: set_tool_visibility_by_tool_id($course_id,$tool->get_id(),$visible);
            }
        }
    }
    
    static function set_tool_visibility_by_tool_id($course_id, $tool_id, $visible = 1)
    {
        $course_settings_controller = CourseSettingsController :: get_instance();
        $course_setting = $course_settings_controller->get_course_setting_object_from_name_and_tool(CourseSetting :: COURSE_SETTING_TOOL_VISIBLE, $tool_id);
        if($course_setting)
        {
            $course = self :: get_instance()->retrieve_course($course_id);
            if($course)
            {
                $course_setting_relation = $course->retrieve_course_setting_relation($course_setting);

                if($course_setting_relation)
                {
                    $course_setting_relation->truncate_values();
                    $course_setting_relation->add_course_setting_value($visible);
                }
            }
        }
    }
    
}
?>
