<?php

namespace application\weblcms\course;

use application\weblcms\Mdb2WeblcmsDataManager;
use application\weblcms\course_type\CourseType;

use common\libraries\EqualityCondition;

/**
 * This class describes the mdb2 datamanager implementation for the course
 * submanager (extension on weblcms data manager)
 *
 * @package \application\weblcms\course
 *         
 * @author Sven Vanpoucke - Hogeschool Gent
 */
class Mdb2CourseDataManager extends Mdb2WeblcmsDataManager implements CourseDataManagerInterface
{

    /**
     * **************************************************************************************************************
     * Course Dataclass Functionality *
     * **************************************************************************************************************
     */
    
    /**
     * Retrieves a course with the given id
     *
     * @param $id int           
     *
     * @return Course
     */
    function retrieve_course($id)
    {
        $condition = new EqualityCondition(Course :: PROPERTY_ID, $id);
        return $this->retrieve_object(Course :: get_table_name(), $condition);
    }

    /**
     * Retrieves a list of available courses
     *
     * @param $condition \common\libraries\Condition           
     * @param $offset int           
     * @param $max_objects int           
     * @param $order_by \common\libraries\ObjectTableOrder           
     *
     * @return \common\libraries\ResultSet<Course>
     */
    function retrieve_courses($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        $course_alias = $this->get_alias(Course :: get_table_name());
        $course_type_alias = $this->get_alias(CourseType :: get_table_name());
        
        $course_all_columns = $course_alias . '.*';
        $course_type_title_column = $this->escape_column_name(CourseType :: PROPERTY_TITLE, $course_type_alias);
        
        $query = 'SELECT ' . $course_all_columns . ', ' . $course_type_title_column . ' AS ';
        $query .= Course :: PROPERTY_COURSE_TYPE_TITLE . ' ' . $this->get_course_base_query();
        
        return $this->retrieve_object_set($query, Course :: get_table_name(), $condition, $offset, $max_objects, $order_by);
    }

    /**
     * Counts the courses
     *
     * @param $condition \common\libraries\Condition           
     *
     * @return int
     */
    function count_courses($condition = null)
    {
        $query = 'SELECT COUNT(*) ' . $this->get_course_base_query();
        
        return $this->count_result_set($query, Course :: get_table_name(), $condition);
    }

    /**
     * **************************************************************************************************************
     * Course Dataclass Helper Functionality *
     * **************************************************************************************************************
     */
    
    /**
     * Returns the base query for select queries for the course table.
     * This query is already joined with the course type table.
     *
     * @return String
     */
    private function get_course_base_query()
    {
        $course_alias = $this->get_alias(Course :: get_table_name());
        $course_table = $this->get_table_name(Course :: get_table_name());
        $course_type_alias = $this->get_alias(CourseType :: get_table_name());
        $course_type_table = $this->get_table_name(CourseType :: get_table_name());
        
        $course_course_type_id_column = $this->escape_column_name(Course :: PROPERTY_COURSE_TYPE_ID, $course_alias);
        $course_type_id_column = $this->escape_column_name(CourseType :: PROPERTY_ID, $course_type_alias);
        
        $query = 'FROM ' . $course_table . ' AS ' . $course_alias;
        $query .= ' LEFT JOIN ' . $course_type_table . ' AS ' . $course_type_alias;
        $query .= ' ON ' . $course_course_type_id_column . ' = ' . $course_type_id_column;
        
        return $query;
    }

    /**
     * **************************************************************************************************************
     * CourseRelCourseSetting Dataclass Functionality *
     * **************************************************************************************************************
     */
    
    /**
     * Retrieves a course rel course setting with the given id
     *
     * @param $id int           
     *
     * @return CourseRelCourseSetting
     */
    function retrieve_course_rel_course_setting($id)
    {
        $condition = new EqualityCondition(CourseRelCourseSetting :: PROPERTY_ID, $id);
        return $this->retrieve_object(CourseRelCourseSetting :: get_table_name(), $condition);
    }

    /**
     * Retrieves a list of available course rel course settings
     *
     * @param $condition \common\libraries\Condition           
     * @param $offset int           
     * @param $max_objects int           
     * @param $order_by \common\libraries\ObjectTableOrder           
     *
     * @return \common\libraries\ResultSet <CourseRelCourseSetting>
     */
    function retrieve_course_rel_course_settings($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        return $this->retrieve_objects(CourseRelCourseSetting :: get_table_name(), $condition, $offset, $max_objects, $order_by);
    }

    /**
     * Counts the course rel course settings
     *
     * @param $condition \common\libraries\Condition           
     *
     * @return int
     */
    function count_course_rel_course_settings($condition = null)
    {
        return $this->count_objects(CourseRelCourseSetting :: get_table_name(), $condition);
    }

    /**
     * **************************************************************************************************************
     * CourseRelCourseSettingValue Dataclass Functionality *
     * **************************************************************************************************************
     */
    
    /**
     * Retrieves a course rel course setting value with the given id
     *
     * @param $id int           
     *
     * @return CourseRelCourseSettingValue
     */
    function retrieve_course_rel_course_setting_value($id)
    {
        $condition = new EqualityCondition(CourseRelCourseSettingValue :: PROPERTY_ID, $id);
        return $this->retrieve_object(CourseRelCourseSettingValue :: get_table_name(), $condition);
    }

    /**
     * Retrieves a list of available course rel course setting values
     *
     * @param $condition \common\libraries\Condition           
     * @param $offset int           
     * @param $max_objects int           
     * @param $order_by \common\libraries\ObjectTableOrder           
     *
     * @return \common\libraries\ResultSet <CourseRelCourseSettingValue>
     */
    function retrieve_course_rel_course_setting_values($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        return $this->retrieve_objects(CourseRelCourseSettingValue :: get_table_name(), $condition, $offset, $max_objects, $order_by);
    }

    /**
     * Counts the course rel course setting values
     *
     * @param $condition \common\libraries\Condition           
     *
     * @return int
     */
    function count_course_rel_course_setting_values($condition = null)
    {
        return $this->count_objects(CourseRelCourseSettingValue :: get_table_name(), $condition);
    }

}
?>
