<?php

namespace application\weblcms\course;

use user\UserDataManager;
use common\libraries\Session;
use common\libraries\Utilities;
use common\libraries\ObjectTableOrder;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\Path;
use common\libraries\DataClass;
use common\libraries\Translation;
use common\libraries\Text;

use application\weblcms\WeblcmsDataManager;
use application\weblcms\CourseSection;
use application\weblcms\CourseGroup;
use application\weblcms\CourseManagementRights;
use application\weblcms\ContentObjectPublicationCategory;
use application\weblcms\CourseModule;
use application\weblcms\WeblcmsManager;
use application\weblcms\course_type\CourseTypeDataManager;
use application\weblcms\CourseSettingsController;
use application\weblcms\Tool;
use application\weblcms\CourseSetting;

/**
 * This class represents a course in the weblcms.
 *
 * @package application\weblcms\course;
 *
 * @author Previously Author Unknown
 * @author Yannick & Tristan
 * @author Sven Vanpoucke - Hogeschool Gent - Refactoring
 */
class Course extends DataClass
{
    /****************************************************************************************************************
    * Table Properties												    *
    ****************************************************************************************************************/

    const PROPERTY_COURSE_TYPE_ID = 'course_type_id';
    const PROPERTY_TITULAR_ID = 'titular_id';
    const PROPERTY_VISUAL_CODE = 'visual_code';
    const PROPERTY_TITLE = 'title';
    const PROPERTY_LAST_VISIT = 'last_visit';
    const PROPERTY_LAST_EDIT = 'last_edit';
    const PROPERTY_CREATION_DATE = 'creation_date';
    const PROPERTY_EXPIRATION_DATE = 'expiration_date';
    const PROPERTY_LANGUAGE = 'language';
    const PROPERTY_CATEGORY_ID = 'category_id';

    /****************************************************************************************************************
    * Foreign properties											    *
    ****************************************************************************************************************/

    const FOREIGN_PROPERTY_COURSE_TYPE = 'course_type';
    
    /****************************************************************************************************************
    * Additional properties											    *
    ****************************************************************************************************************/
    
    /**
     * Property only used when joining with the course type table
     */
    const PROPERTY_COURSE_TYPE_TITLE = 'course_type_title';

    /****************************************************************************************************************
    * Cache properties												    *	
    ****************************************************************************************************************/

    /**
     * Caches the course admin checks
     *
     * @var boolean[]
     */
    private $is_course_admin_cache;

    /**
     * Caches the users that are course admins in this course
     *
     * @var \application\weblcms\course\CourseUserRelation[]
     */
    private $course_admin_users_cache;

    /**
     * Caches the groups that are course admins in this course
     *
     * @var \application\weblcms\course\CourseGroupRelation[]
     */
    private $course_admin_groups_cache;

    /****************************************************************************************************************
    * Old properties												    *
    ****************************************************************************************************************/

    /**
     * Keeps track of the previous course type id before updating so the location can be moved if a different course
     * type is selected
     *
     * @var int
     */
    private $old_course_type_id;

    /****************************************************************************************************************
    * Inherited Functionality											    *
    ****************************************************************************************************************/

    /**
     * Returns the default properties of this dataclass
     *
     * @return String[] - The property names.
     */
    static function  get_default_property_names($extended_properties = array())
    {
        $extended_properties[] = self :: PROPERTY_COURSE_TYPE_ID;
        $extended_properties[] = self :: PROPERTY_TITULAR_ID;
        $extended_properties[] = self :: PROPERTY_TITLE;
        $extended_properties[] = self :: PROPERTY_VISUAL_CODE;
        $extended_properties[] = self :: PROPERTY_CREATION_DATE;
        $extended_properties[] = self :: PROPERTY_EXPIRATION_DATE;
        $extended_properties[] = self :: PROPERTY_LAST_EDIT;
        $extended_properties[] = self :: PROPERTY_LAST_VISIT;
        $extended_properties[] = self :: PROPERTY_CATEGORY_ID;
        $extended_properties[] = self :: PROPERTY_LANGUAGE;

	return parent :: get_default_property_names($extended_properties);
    }

    /**
     * Returns the datamanager for this dataclass
     *
     * @return WeblcmsDataManager
     */
    function get_data_manager()
    {
        return CourseDataManager :: get_instance();
    }

    /**
     * Returns the table name of this dataclass
     * Inherited
     *
     * @return String
     */
    static function get_table_name()
    {
        return Utilities :: get_classname_from_namespace(__CLASS__, true);
    }

    /**
     * Creates this course object
     * 
     * @return boolean
     */
    function create($automated_values = true, $create_in_batch = false)
    {
        if ($automated_values)
        {
            $now = time();
            $this->set_last_visit($now);
            $this->set_last_edit($now);
            $this->set_creation_date($now);
            $this->set_expiration_date($now);
        }

        if (!parent :: create())
        {
            return false;
        }

        if (!$this->initialize_course_sections())
        {
            return false;
        }

        if (!$this->create_locations($create_in_batch))
        {
            return false;
        }

        if (!$this->create_root_course_group())
        {
            return false;
        }

        return true;
    }

    /**
     * Updates this course object
     *
     * If the name of the course changes, the name of the root chamilo course group needs to change
     *
     * @return boolean
     */
    function update()
    {
        $course_group = $this->get_data_manager()->retrieve_course_group_root($this->get_id());

        if($course_group)
        {
            if($course_group->get_name() != $this->get_title())
            {
                $course_group->set_name($this->get_title());
                $course_group->update();
            }
        }

        $old_course_type_id = $this->old_course_type_id;

        if($old_course_type_id)
        {
            $location = $this->get_rights_location();
            
            if(!$location->move($this->get_parent_rights_location()->get_id()))
            {
                return false;
            }
        }

        return parent :: update();
    }

    /**
     * Deletes this course object
     *
     * @return boolean
     */
    function delete()
    {
        // Remove subtree location
	$location = CourseManagementRights :: get_instance()->get_courses_subtree_root($this->get_id());
        if ($location)
        {
            if (!$location->delete())
            {
                return false;
            }
        }

	// Remove location in root tree
	$location = $this->get_rights_location();

        if ($location)
        {
            if (!$location->delete())
            {
                return false;
            }
        }

        $dm = $this->get_data_manager();
        return $dm->delete_course($this->get_id());
    }

    /****************************************************************************************************************
    * Rights Functionality											    *
    ****************************************************************************************************************/

    /**
     * Returns the rights location for this object
     *
     * @return RightsLocation
     */
    function get_rights_location()
    {
	return CourseManagementRights :: get_instance()->get_weblcms_location_by_identifier_from_courses_subtree
		(CourseManagementRights :: TYPE_COURSE, $this->get_id(), 0);
    }

    /**
     * Returns the (possible) parent rights location depending on this objects properties
     *
     * @return RightsLocation
     */
    function get_parent_rights_location()
    {
        $parent = CourseManagementRights :: get_instance()->get_weblcms_location_by_identifier_from_courses_subtree(
		CourseManagementRights :: TYPE_COURSE_TYPE, $this->get_course_type_id(), 0);

        if (!$parent)
        {
            $parent = CourseManagementRights :: get_instance()->get_courses_subtree_root(0);
        }

        return $parent;
    }

    /**
     * Returns the available rights for the course management
     *
     * @return int[string]
     */
    function get_available_course_management_rights()
    {
	return CourseManagementRights :: get_instance()->get_base_course_management_rights();
    }

    /**
     * Checks whether or not this object can change the given course management right
     *
     * @param int $right_id
     *
     * @return boolean
     */
    function can_change_course_management_right($right_id)
    {
	$available_rights = $this->get_available_course_management_rights();
        if(!in_array($right_id, $available_rights))
        {
            return false;
        }
        
        $course_type = $this->get_course_type();
	if(!$course_type)
	{
	    return true;
	}

	return !CourseManagementRights :: get_instance()->is_right_locked_for_base_object($course_type, $right_id);
    }

    /****************************************************************************************************************
    * Course Settings Functionality										    *
    ****************************************************************************************************************/

    /**
     * Retrieves course setting values for the given setting name and tool id
     * 
     * @param string $setting_name
     * @param int $tool_id
     *
     * @return string[]
     */
    function get_course_setting($setting_name, $tool_id = 0)
    {
        return CourseSettingsController :: get_instance()->get_course_setting($this->get_id(), $setting_name, $tool_id);
    }

    /**
     * Retrieves the default values for a given course setting
     *
     * @param string $setting_name
     * @param int $tool_id
     *
     * @return string[]
     */
    function get_default_course_setting($setting_name, $tool_id)
    {
        return CourseSettingsController :: get_instance()->get_course_type_setting($this->get_course_type_id(), 
                $setting_name, $tool_id);
    }

    /**
     * Delegation function to create course settings from given values
     *
     * @param string[] $values
     * @param boolean $force [OPTIONAL] - default false - Sets the values even if the base object is not allowed to.
     *
     * @return boolean
     */
    function create_course_settings_from_values($values, $force = false)
    {
	return CourseSettingsController :: get_instance()->handle_settings_for_object_with_given_values($this, $values,
                CourseSettingsController :: SETTING_ACTION_CREATE, $force);
    }

    /**
     * Delegation function to update course settings from given values
     *
     * @param string $values
     *
     * @return boolean
     */
    function update_course_settings_from_values($values)
    {
	return CourseSettingsController :: get_instance()->handle_settings_for_object_with_given_values($this, $values,
		CourseSettingsController :: SETTING_ACTION_UPDATE);
    }

    /**
     * Creates a course setting relation for the given course setting object
     *
     * @param CourseSetting $course_setting
     * @param boolean $locked
     *
     * @return CourseRelCourseSetting
     */
    function create_course_setting_relation(CourseSetting $course_setting, $locked)
    {
	$course_rel_setting = new CourseRelCourseSetting();
	$course_rel_setting->set_course_setting($course_setting);
	$course_rel_setting->set_course($this);

	if(!$course_rel_setting->create())
	{
	    throw new \Exception(Translation :: get('CouldNotCreateCourseRelCourseSetting'));
	}

	return $course_rel_setting;
    }

    /**
     * Updates a course setting relation for the given course setting object
     *
     * @param CourseSetting $course_setting
     * @param boolean $locked
     *
     * @return CourseTypeRelCourseSetting
     */
    function update_course_setting_relation(CourseSetting $course_setting, $locked)
    {
	return $this->retrieve_course_setting_relation($course_setting);
    }

    /**
     * Retrieves a course setting relation object for the given course setting object
     *
     * @param CourseSetting $course_setting
     *
     * @return CourseRelCourseSetting
     */
    function retrieve_course_setting_relation(CourseSetting $course_setting)
    {
	$conditions = array();

	$conditions[] = new EqualityCondition(CourseRelCourseSetting :: PROPERTY_COURSE_SETTING_ID,
		$course_setting->get_id());
	$conditions[] = new EqualityCondition(CourseRelCourseSetting :: PROPERTY_COURSE_ID, $this->get_id());

	$condition = new AndCondition($conditions);

	return CourseDataManager :: get_instance()->retrieve_course_rel_course_settings($condition)->next_result();
    }

    /**
     * Returns whether or not a given course setting is locked for this object.
     * Since courses do not support locking of course settings the course settings are never locked
     * 
     * @param CourseSetting $course_setting
     * 
     * @return boolean
     */
    function is_course_setting_locked(CourseSetting $course_setting)
    {
	return false;
    }

    /**
     * Returns whether or not a given course setting can be changed by this object
     *
     * @param CourseSetting $course_setting
     *
     * @return boolean
     */
    function can_change_course_setting(CourseSetting $course_setting)
    {
	$course_type = $this->get_course_type();
	if(!$course_type)
	{
	    return true;
	}
	
	return !$course_type->is_course_setting_locked($course_setting);
	
    }

    /****************************************************************************************************************
    * Helper Functionality											    *
    ****************************************************************************************************************/

    /**
     * Creates the necessary locations for this dataclass
     *
     * Creates a location for the given course in the courses tree
     * Creates a root location for the course subtree
     * Set view right for everyone on root location
     * Creates a location for each tool in the course subtree
     *
     * @param boolean $create_in_batch - Whether or not the left and right values should be calculated
     *
     * @return boolean
     */
    private function create_locations($create_in_batch = false)
    {
        // Create location in the course subtree
	$parent_id = $this->get_parent_rights_location()->get_id();

        if(!CourseManagementRights :: get_instance()->create_location_in_courses_subtree(
		CourseManagementRights :: TYPE_COURSE, $this->get_id(), $parent_id, 0, $create_in_batch, 0))
	{
	    return false;
	}

	// Create course subtree root location
	$course_subtree_root_location = CourseManagementRights :: get_instance()->create_subtree_root_location(
		$this->get_id(), CourseManagementRights :: TREE_TYPE_COURSE, true);

	if(!$course_subtree_root_location)
	{
	    return false;
	}

	$course_subtree_root_location_id = $course_subtree_root_location->get_id();

	// Set view right for everyone on root location
	if(!CourseManagementRights :: get_instance()->invert_location_entity_right(
		CourseManagementRights :: VIEW_RIGHT, 0, 0,
		CourseManagementRights :: get_instance()->get_courses_subtree_root_id($this->get_id())))
        {
            return false;
        }

	// Create a location for each tool
	$tools = WeblcmsDataManager :: get_instance()->retrieve_course_tools();
	while($tool = $tools->next_result())
	{
	    if(!CourseManagementRights :: get_instance()->create_location_in_courses_subtree(
		    CourseManagementRights :: TYPE_COURSE_MODULE, $tool->get_id(),
		    $course_subtree_root_location_id, $this->get_id(), $create_in_batch))
	    {
		return false;
	    }
	}

	return true;
        
    }

    /**
     * Initializes the course sections for this course
     *
     * @return boolean
     */
    private function initialize_course_sections()
    {
        $sections = array();
        $sections[] = array('name' => Translation :: get('SectionTools'), 'type' => 1, 'order' => 1);
        $sections[] = array('name' => Translation :: get('SectionLinks'), 'type' => 2, 'order' => 2);
        $sections[] = array('name' => Translation :: get('SectionDisabled'), 'type' => 0, 'order' => 3);
        $sections[] = array('name' => Translation :: get('SectionCourseAdministration'), 'type' => 3, 'order' => 4);

        foreach ($sections as $section)
        {
            $course_section = new CourseSection();
            $course_section->set_course_id($this->get_id());
            $course_section->set_name($section['name']);
            $course_section->set_type($section['type']);
            $course_section->set_visible(true);
            if (!$course_section->create())
            {
                return false;
            }
        }

        return true;
    }

    /**
     * Creates a course group in the course with the same name as the course
     *
     * @return boolean
     */
    private function create_root_course_group()
    {
        $group = new CourseGroup();
        $group->set_course_code($this->get_id());
        $group->set_name($this->get_title());
        $group->set_parent_id(0);
        return $group->create();
    }

    /****************************************************************************************************************
    * Delegation Functionality											    *
    ****************************************************************************************************************/

    /**
     * Checks whether the given user is a course admin in this course
     *
     * @param int $user_id
     *
     * @return boolean
     */
    function is_course_admin($user)
    {
        //fix for view as
        $va_id = Session :: get(Tool :: PARAM_VIEW_AS_ID);
        $course_id = Session :: get(Tool :: PARAM_VIEW_AS_COURSE_ID);
        $id = $user->get_id();
        if (isset($va_id) && isset($course_id))
        {
            if ($course_id == $this->get_id())
            {
                $id = $va_id;
            }
            else
            {
                if ($user->is_platform_admin())
                {
                    return true;
                }
            }
        }
        else
        {
            if ($user->is_platform_admin())
            {
                return true;
            }
        }

        if (is_null($this->is_course_admin_cache[$id]))
        {
            $studentview = Session :: retrieve('studentview');

            if ($studentview)
            {
                return false;
            }

            $this->is_course_admin_cache[$id] = $this->is_subscribed_as_course_admin($user);
        }
        
        return $this->is_course_admin_cache[$id];
    }

     /**
     * Checks whether the current user is subscribed as course admin of the given course
     *
     * @param Course $course
     *
     * @return boolean
     */
    function is_subscribed_as_course_admin($user)
    {
        $wdm = WeblcmsDataManager :: get_instance();

        if($wdm->is_course_admin($this, $user->get_id()))
        {
            return true;
        }

        if(WeblcmsDataManager :: is_teacher_through_platform_groups($this, $user))
        {
            return true;
        }

        return false;
    }

    /**
     * Retrieves the course admin users for this course
     *
     * @return \user\User[]
     */
    function get_course_admin_users()
    {
        if(!$this->course_admin_users_cache)
        {
            $this->course_admin_users_cache = WeblcmsDataManager :: retrieve_course_admin_users($this->get_id());
        }
        return $this->course_admin_users_cache;
    }

    /**
     * Retrieves the course admin groups for this course
     *
     * @return \group\Group[]
     */
    function get_course_admin_groups()
    {
        if(!$this->course_admin_groups_cache)
        {
            $this->course_admin_groups_cache = WeblcmsDataManager :: retrieve_course_admin_groups($this->get_id());
        }
        return $this->course_admin_groups_cache;
    }

    /**
     * Returns if this course has subscribed users
     *
     * @return boolean
     */
    function has_subscribed_users()
    {
        $relation_condition = new EqualityCondition(CourseUserRelation :: PROPERTY_COURSE_ID, $this->get_id());
        return ($this->get_data_manager()->count_course_user_relations($relation_condition) > 0);
    }

    /**
     * Gets the subscribed users of this course
     *
     * @return \application\weblcms\course\CourseUserRelation[]
     */
    function get_subscribed_users()
    {
        $relation_condition = new EqualityCondition(CourseUserRelation :: PROPERTY_COURSE_ID, $this->get_id());
        return $this->get_data_manager()->retrieve_course_user_relations($relation_condition)->as_array();
    }

    /**
     * Returns if this course has subscribed groups
     *
     * @return boolean
     */
    function has_subscribed_groups()
    {
        $relation_condition = new EqualityCondition(CourseGroupRelation :: PROPERTY_COURSE_ID, $this->get_id());
        return $this->get_data_manager()->count_course_group_relations($relation_condition);
    }

    /**
     * Gets the subscribed groups of this course
     *
     * @return \application\weblcms\course\CourseGroupRelation[]
     */
    function get_subscribed_groups()
    {
        $relation_condition = new EqualityCondition(CourseGroupRelation :: PROPERTY_COURSE_ID, $this->get_id());
        return $this->get_data_manager()->retrieve_course_group_relations($relation_condition)->as_array();
    }

    /**
     * Gets the course groups of this course
     *
     * @return \application\weblcms\CourseGroup[]
     */
    function get_course_groups($as_array = true)
    {
        $wdm = WeblcmsDataManager :: get_instance();
	
        $condition = new EqualityCondition(CourseGroup :: PROPERTY_COURSE_CODE, $this->get_id());
        $result = $wdm->retrieve_course_groups($condition, null, null, array(new ObjectTableOrder(
		CourseGroup :: PROPERTY_TITLE)));

        return ($as_array ? $result->as_array() : $result);
    }

    /****************************************************************************************************************
    * Getters and Setters											    *
    ****************************************************************************************************************/

    /**
     * Returns the course type id of this course object
     *
     * @return int
     */
    function get_course_type_id()
    {
        return $this->get_default_property(self :: PROPERTY_COURSE_TYPE_ID);
    }

    /**
     * Returns the titular id of this course object
     *
     * @return int
     */
    function get_titular_id()
    {
        return $this->get_default_property(self :: PROPERTY_TITULAR_ID);
    }

    /**
     * Returns the visual code of this course object
     *
     * @return String
     */
    function get_visual_code()
    {
        return $this->get_default_property(self :: PROPERTY_VISUAL_CODE);
    }

    /**
     * Returns the title of this course object
     *
     * @return String
     */
    function get_title()
    {
        return $this->get_default_property(self :: PROPERTY_TITLE);
    }

    /**
     * Returns the creation date of this course object
     *
     * @return int
     */
    function get_creation_date()
    {
        return $this->get_default_property(self :: PROPERTY_CREATION_DATE);
    }

    /**
     * Returns the expiration date of this course
     *
     * @return int
     */
    function get_expiration_date()
    {
        return $this->get_default_property(self :: PROPERTY_EXPIRATION_DATE);
    }

    /**
     * Returns the last modification date of this course
     *
     * @return int
     */
    function get_last_edit()
    {
        return $this->get_default_property(self :: PROPERTY_LAST_EDIT);
    }

    /**
     * Returns the last visit date of this course
     *
     * @return int
     */
    function get_last_visit()
    {
        return $this->get_default_property(self :: PROPERTY_LAST_VISIT);
    }

    /**
     * Sets the course type id of this course object
     *
     * @param int $course_type_id
     */
    function set_course_type_id($course_type_id)
    {
        $old_course_type_id = $this->get_course_type_id();
        
        if($course_type_id == $old_course_type_id)
        {
            return;
        }

        $this->old_course_type_id = $old_course_type_id;

        $this->set_default_property(self :: PROPERTY_COURSE_TYPE_ID, $course_type_id);
    }

    /**
     * Sets the titular id of this course object
     *
     * @param int $titular_id
     */
    function set_titular_id($titular_id)
    {
        $this->set_default_property(self :: PROPERTY_TITULAR_ID, $titular_id);
    }

    /**
     * Sets the visual code of this course object
     *
     * @param String $visual_code
     */
    function set_visual_code($visual_code)
    {
        $this->set_default_property(self :: PROPERTY_VISUAL_CODE, $visual_code);
    }

    /**
     * Sets the course title of this course object
     *
     * @param String $title
     */
    function set_title($title)
    {
        $this->set_default_property(self :: PROPERTY_TITLE, $title);
    }

    /**
     * Sets the creation date of this course
     *
     * @param int $creation_date
     */
    function set_creation_date($creation_date)
    {
        $this->set_default_property(self :: PROPERTY_CREATION_DATE, $creation_date);
    }

    /**
     * Sets the expiration date of this course
     *
     * @param int $expiration_date
     */
    function set_expiration_date($expiration_date)
    {
        $this->set_default_property(self :: PROPERTY_EXPIRATION_DATE, $expiration_date);
    }

    /**
     * Sets the last edit date of this course
     *
     * @param int $last_edit
     */
    function set_last_edit($last_edit)
    {
        $this->set_default_property(self :: PROPERTY_LAST_EDIT, $last_edit);
    }

    /**
     * Sets the last visit date of this course
     *
     * @param int $last_visit
     */
    function set_last_visit($last_visit)
    {
        $this->set_default_property(self :: PROPERTY_LAST_VISIT, $last_visit);
    }
    
    /**
     * Sets the language of this course object
     *
     * @param String $language
     */
    function set_language($language)
    {
        $this->set_default_property(self :: PROPERTY_LANGUAGE, $language);
    }

    /**
     * gets the language of this course object
     *
     * @return String $language
    */
    function get_language()
    {
        return $this->get_default_property(self :: PROPERTY_LANGUAGE);
    }
    
     /**
     * Sets the category_id of this course object
     *
     * @param int $category_id
     */
    function set_category_id($category_id)
    {
        $this->set_default_property(self :: PROPERTY_CATEGORY_ID, $category_id);
    }

    /**
     * gets the category_id of this course object
     *
     * @return String $language
    */
    function get_category_id()
    {
        return $this->get_default_property(self :: PROPERTY_CATEGORY_ID);
    }


    /****************************************************************************************************************
    * Additional Properties Setters / Getters									    *
    ****************************************************************************************************************/
    
    /**
     * Returns the title for the course type foreign object
     *
     * @return String
     */
    public function get_course_type_title()
    {
	$course_type_title = $this->get_optional_property(self :: PROPERTY_COURSE_TYPE_TITLE);
	if(!$course_type_title)
	{
	    $course_type = $this->get_course_type();
	    return ($course_type ? $course_type->get_title() : Translation :: get('NoCourseType'));
	}

	return $course_type_title;
    }
    
    /****************************************************************************************************************
    * Foreign Properties Setters / Getters									    *
    ****************************************************************************************************************/

    /**
     * Returns the course type of this course object (lazy loading)
     *
     * @return \application\weblcms\course_type\CourseType
     */
    public function get_course_type()
    {
        if($this->get_course_type_id() == 0)
	{
	    return null;
	}

	return $this->get_foreign_property(self :: FOREIGN_PROPERTY_COURSE_TYPE,
                CourseTypeDataManager :: get_instance());
    }

    /**
     * Sets the course type of this course object
     *
     * @param \application\weblcms\course_type\CourseType $course_type
     */
    public function set_course_type(\application\weblcms\course_type\CourseType $course_type)
    {
        if(!is_null($course_type))
        {
            $old_course_type_id = $this->get_course_type_id();
            
            if($course_type->get_id() != $old_course_type_id)
            {
                $this->old_course_type_id = $old_course_type_id;
            }
        }
        
        $this->set_foreign_property(self :: FOREIGN_PROPERTY_COURSE_TYPE, $course_type);
    }
}
?>
