<?php
namespace application\weblcms\course_type;

use common\libraries\Configuration;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\InequalityCondition;
use common\libraries\AndCondition;
use common\libraries\OrCondition;
use common\libraries\NotCondition;
use common\libraries\SubselectCondition;
use common\libraries\ObjectTableOrder;

use application\weblcms\WeblcmsDataManager;
use application\weblcms\course\CourseDataManager;

/**
 * This class describes the datamanager for the course type submanager (extension on weblcms data manager)
 *
 * @package \application\weblcms\course_type
 * @author Sven Vanpoucke - Hogeschool Gent
 */
class CourseTypeDataManager extends WeblcmsDataManager
{
    /**
     * Instance of the class, for the singleton pattern.
     */
    private static $instance;

    /**
     * **************************************************************************************************************
     * Inherited Functionality *
     * **************************************************************************************************************
     */

    /**
     * Creates and returns an instance of a course type data manager implementation
     *
     * @return CourseTypeDataManagerInterface
     */
    static function get_instance()
    {
        if (! isset(self :: $instance))
        {
            $type = Configuration :: get_instance()->get_parameter('general', 'data_manager');
            $file = dirname(__FILE__) . '/implementation/' . strtolower($type) . '.class.php';

            if (! file_exists($file))
            {
                throw new \Exception(Translation :: get('DataManagerImplementationNotFound', array('TYPE' => $type), Utilities :: COMMON_LIBRARIES));
            }

            $class = __NAMESPACE__ . '\\' . Utilities :: underscores_to_camelcase($type) . 'CourseTypeDataManager';

            self :: $instance = new $class();
        }

        return self :: $instance;
    }

    /**
     * **************************************************************************************************************
     * CourseType Dataclass Functionality *
     * **************************************************************************************************************
     */

    /**
     * Retrieves the active course types
     *
     * @return ResultSet<CourseType>
     */
    static function retrieve_active_course_types()
    {
        $condition = new EqualityCondition(CourseType :: PROPERTY_ACTIVE, 1);

        $order = array(new ObjectTableOrder(CourseType :: PROPERTY_DISPLAY_ORDER));

        return self :: get_instance()->retrieve_course_types($condition, null, null, $order);
    }

    /**
     * Retrieves the active course types ordered by the given user display order
     *
     * @param $user_id int
     *
     * @return ResultSet<CourseType>
     */
    static function retrieve_active_course_types_with_user_order($user_id)
    {
        $condition = new EqualityCondition(CourseType :: PROPERTY_ACTIVE, 1);

        $join_conditions = array();
        $join_conditions[] = new EqualityCondition(CourseTypeUserOrder :: PROPERTY_USER_ID, $user_id, CourseTypeUserOrder :: get_table_name());
        $join_conditions[] = new EqualityCondition(CourseTypeUserOrder :: PROPERTY_USER_ID, null, CourseTypeUserOrder :: get_table_name());

        $join_condition = new OrCondition($join_conditions);

        $course_type_user_order_alias = self :: get_instance()->get_alias(CourseTypeUserOrder :: get_table_name());

        $order = array();

        $order[] = new ObjectTableOrder(CourseTypeUserOrder :: PROPERTY_DISPLAY_ORDER, SORT_ASC, $course_type_user_order_alias);
        $order[] = new ObjectTableOrder(CourseType :: PROPERTY_DISPLAY_ORDER);

        return self :: get_instance()->retrieve_course_types_with_user_order($join_condition, $condition, null, null, $order);
    }

    /**
     * Counts the active course types
     *
     * @return int
     */
    static function count_active_course_types()
    {
        $condition = new EqualityCondition(CourseType :: PROPERTY_ACTIVE, 1);
        return self :: get_instance()->count_course_types($condition);
    }

    /**
     * Returns the highest display order for the course types
     *
     * @return int
     */
    static function get_max_display_order_for_course_types()
    {
        return self :: get_instance()->retrieve_max_sort_value(CourseType :: get_table_name(), CourseType :: PROPERTY_DISPLAY_ORDER);
    }

    /**
     * Fixes all the display order values between the given start and end display order with the number with which the
     * display order must be fixed for the course types
     *
     * @param $start_display_order int
     * @param $end int
     * @param $count int
     * @param $direction int
     *
     * @return boolean
     */
    static function fix_course_type_display_order($start_display_order = 1, $end_display_order = null, $count = 1, $direction = self :: DISPLAY_ORDER_DIRECTION_UP)
    {
        return self :: fix_display_order(CourseType :: class_name(), CourseType :: PROPERTY_DISPLAY_ORDER, null, $start_display_order, $end_display_order, $count, $direction);
    }

    /**
     * Returns whether or not a course type has courses
     *
     * @param $course_type_id int
     *
     * @return int
     */
    static function has_course_type_courses($course_type_id = 0)
    {
        return CourseDataManager :: count_courses_from_course_type($course_type_id) > 0;
    }

    /**
     * **************************************************************************************************************
     * CourseTypeUserOrder Dataclass Functionality *
     * **************************************************************************************************************
     */

    /**
     * Retrieves a CourseTypeUserOrder for a given course type and user
     *
     * @param $course_type_id int
     * @param $user_id int
     *
     * @return CourseTypeUserOrder
     */
    static function retrieve_user_order_for_course_type($course_type_id, $user_id)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(CourseTypeUserOrder :: PROPERTY_COURSE_TYPE_ID, $course_type_id);
        $conditions[] = new EqualityCondition(CourseTypeUserOrder :: PROPERTY_USER_ID, $user_id);
        $condition = new AndCondition($conditions);

        return self :: get_instance()->retrieve_course_type_user_orders($condition)->next_result();
    }

    /**
     * Returns the highest display order for the course type user orders for a given user
     *
     * @param $user_id int
     *
     * @return int
     */
    static function get_max_display_order_for_course_type_user_orders($user_id)
    {
        $condition = new EqualityCondition(CourseTypeUserOrder :: PROPERTY_USER_ID, $user_id);

        return self :: get_instance()->retrieve_max_sort_value(CourseTypeUserOrder :: get_table_name(), CourseTypeUserOrder :: PROPERTY_DISPLAY_ORDER, $condition);
    }

    /**
     * Fixes all the display order values between the given start and end display order with the number with which the
     * display order must be fixed for the course type user orders
     *
     * @param $condition Condition
     * @param $start_display_order int
     * @param $end int
     * @param $count int
     * @param $direction int return boolean
     */
    static function fix_course_type_user_order_display_order($condition, $start_display_order = 1, $end_display_order = null, $count = 1, $direction = self :: DISPLAY_ORDER_DIRECTION_UP)
    {
        return self :: fix_display_order(CourseTypeUserOrder :: class_name(), CourseTypeUserOrder :: PROPERTY_DISPLAY_ORDER, $condition, $start_display_order, $end_display_order, $count, $direction);
    }

    /**
     * **************************************************************************************************************
     * Helper Functionality *
     * **************************************************************************************************************
     */

    /**
     * Creates the course type user orders for a given user with the current course types
     *
     * @param $user_id int
     *
     * @return boolean
     */
    static function create_course_type_user_orders_for_user($user_id)
    {
        $course_types = self :: retrieve_active_course_types();

        while ($course_type = $course_types->next_result())
        {
            $course_type_user_order = new CourseTypeUserOrder();
            $course_type_user_order->set_course_type_id($course_type->get_id());
            $course_type_user_order->set_user_id($user_id);
            if (! $course_type_user_order->create())
            {
                return false;
            }
        }

        return true;
    }

    /**
     * Fixes the course tabs user orders when they are broken due to for example new course tabs, removal or
     * (in)activation of course tabs. When new course types are found / activated, new course type user orders are added
     * with highest order (last) When coures types are deleted or inactivated, the course type user oders are deleted
     * and the display orders are fixed.
     *
     * @param $user_id int
     *
     * @return boolean
     */
    static function fix_course_tab_user_orders_for_user($user_id)
    {
        $user_condition = new EqualityCondition(CourseTypeUserOrder :: PROPERTY_USER_ID, $user_id, CourseTypeUserOrder :: get_table_name());
        $instance = self :: get_instance();

        if ($instance->count_course_type_user_orders($user_condition) == 0)
        {
            return true;
        }

        /**
         * Fix all the new / activated course types
         */
        $subcondition = $user_condition;

        $conditions = array();

        $conditions[] = new EqualityCondition(CourseType :: PROPERTY_ACTIVE, 1);
        $conditions[] = new NotCondition(new SubselectCondition(CourseType :: PROPERTY_ID, CourseTypeUserOrder :: PROPERTY_COURSE_TYPE_ID, CourseTypeUserOrder :: get_table_name(), $subcondition));

        $condition = new AndCondition($conditions);

        $active_course_types = $instance->retrieve_course_types($condition);

        while ($course_type = $active_course_types->next_result())
        {
            $course_type_user_order = new CourseTypeUserOrder();
            $course_type_user_order->set_course_type_id($course_type->get_id());
            $course_type_user_order->set_user_id($user_id);
            if (! $course_type_user_order->create())
            {
                return false;
            }
        }

        /**
         * Fix all the deleted / deactivated course types
         */
        $subcondition = new EqualityCondition(CourseType :: PROPERTY_ACTIVE, 1);

        $conditions = array();

        $conditions[] = new NotCondition(new SubselectCondition(CourseTypeUserOrder :: PROPERTY_COURSE_TYPE_ID, CourseType :: PROPERTY_ID, CourseType :: get_table_name(), $subcondition));
        $conditions[] = $user_condition;

        $condition = new AndCondition($conditions);

        while ($course_type_user_order = $instance->retrieve_course_type_user_orders($condition, 0, 1)->next_result())
        {
            if (! $course_type_user_order->delete())
            {
                return false;
            }
        }

        return true;
    }

    /**
     * Checks if the given course type exists
     *
     * @param $course_type_name type
     * @return type
     */
    static function is_course_type_valid($course_type_name)
    {
        $condition = new EqualityCondition(CourseType :: PROPERTY_TITLE, $course_type_name, CourseType :: get_table_name());
        return (self :: get_instance()->count_course_types($condition) != 0);
    }

    /**
     * Retrieves a course type by name
     *
     * @param $course_type_name type
     * @return type
     */
    static function retrieve_course_type_by_name($course_type_name)
    {
        $condition = new EqualityCondition(CourseType :: PROPERTY_TITLE, $course_type_name, CourseType :: get_table_name());
        $types = self :: get_instance()->retrieve_course_types($condition);
        if ($types)
        {
            return $types->next_result();
        }
        else
        {
            return null;
        }
    }
}
?>
