<?php

namespace application\weblcms\course_type;

use application\weblcms\Mdb2WeblcmsDataManager;

use common\libraries\EqualityCondition;
use common\libraries\Condition;
use common\libraries\ConditionTranslator;

/**
 * This class describes the mdb2 datamanager implementation for the course type
 * submanager (extension on weblcms data manager)
 *
 * @package \application\weblcms\course_type
 *         
 * @author Sven Vanpoucke - Hogeschool Gent
 */
class Mdb2CourseTypeDataManager extends Mdb2WeblcmsDataManager implements CourseTypeDataManagerInterface
{

    /**
     * **************************************************************************************************************
     * CourseType Dataclass Functionality *
     * **************************************************************************************************************
     */
    
    /**
     * Retrieves a course type with the given id
     *
     * @param $id int           
     *
     * @return CourseType
     */
    function retrieve_course_type($id)
    {
        $condition = new EqualityCondition(CourseType :: PROPERTY_ID, $id);
        return $this->retrieve_object(CourseType :: get_table_name(), $condition);
    }

    /**
     * Retrieves a list of available course types
     *
     * @param $condition \common\libraries\Condition           
     * @param $offset int           
     * @param $max_objects int           
     * @param $order_by \common\libraries\ObjectTableOrder           
     *
     * @return \common\libraries\ResultSet<CourseType>
     */
    function retrieve_course_types($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        return $this->retrieve_objects(CourseType :: get_table_name(), $condition, $offset, $max_objects, $order_by);
    }

    /**
     * Retrieves a list of available course types joined with course type user
     * order table
     *
     * @param $join_condition \common\libraries\Condition           
     * @param $condition \common\libraries\Condition           
     * @param $offset int           
     * @param $max_objects int           
     * @param $order_by \common\libraries\ObjectTableOrder           
     *
     * @return \common\libraries\ResultSet <CourseType>
     */
    function retrieve_course_types_with_user_order($join_condition = null, $condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        $course_type_table = $this->get_table_name(CourseType :: get_table_name());
        $course_type_alias = $this->get_alias(CourseType :: get_table_name());
        $course_type_user_order_table = $this->get_table_name(CourseTypeUserOrder :: get_table_name());
        $course_type_user_order_alias = $this->get_alias(CourseTypeUserOrder :: get_table_name());
        
        $course_type_table_id = $this->escape_column_name(CourseType :: PROPERTY_ID, $course_type_alias);
        $course_type_user_order_table_course_type_id = $this->escape_column_name(CourseTypeUserOrder :: PROPERTY_COURSE_TYPE_ID, $course_type_user_order_alias);
        
        $query = 'SELECT ' . $course_type_alias . '.* FROM ' . $course_type_table . ' AS ' . $course_type_alias;
        $query .= ' LEFT JOIN ' . $course_type_user_order_table . ' AS ' . $course_type_user_order_alias;
        $query .= ' ON ' . $course_type_table_id . ' = ' . $course_type_user_order_table_course_type_id;
        
        if ($join_condition instanceof Condition)
        {
            $condition_translator = ConditionTranslator :: factory($this);
            $query .= ' AND ' . $condition_translator->translate($join_condition);
        }
        
        return $this->retrieve_object_set($query, CourseType :: get_table_name(), $condition, $offset, $max_objects, $order_by);
    }

    /**
     * Counts the course types
     *
     * @param $condition \common\libraries\Condition           
     *
     * @return int
     */
    function count_course_types($condition = null)
    {
        return $this->count_objects(CourseType :: get_table_name(), $condition);
    }

    /**
     * **************************************************************************************************************
     * CourseTypeRelCourseSetting Dataclass Functionality *
     * **************************************************************************************************************
     */
    
    /**
     * Retrieves a course type rel course setting with the given id
     *
     * @param $id int           
     *
     * @return CourseTypeRelCourseSetting
     */
    function retrieve_course_type_rel_course_setting($id)
    {
        $condition = new EqualityCondition(CourseTypeRelCourseSetting :: PROPERTY_ID, $id);
        return $this->retrieve_object(CourseTypeRelCourseSetting :: get_table_name(), $condition);
    }

    /**
     * Retrieves a list of available course type rel course settings
     *
     * @param $condition \common\libraries\Condition           
     * @param $offset int           
     * @param $max_objects int           
     * @param $order_by \common\libraries\ObjectTableOrder           
     *
     * @return \common\libraries\ResultSet <CourseTypeRelCourseSetting>
     */
    function retrieve_course_type_rel_course_settings($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        return $this->retrieve_objects(CourseTypeRelCourseSetting :: get_table_name(), $condition, $offset, $max_objects, $order_by);
    }

    /**
     * Counts the course type rel course settings
     *
     * @param $condition \common\libraries\Condition           
     *
     * @return int
     */
    function count_course_type_rel_course_settings($condition = null)
    {
        return $this->count_objects(CourseTypeRelCourseSetting :: get_table_name(), $condition);
    }

    /**
     * **************************************************************************************************************
     * CourseTypeRelCourseSettingValue Dataclass Functionality *
     * **************************************************************************************************************
     */
    
    /**
     * Retrieves a course type rel course setting value with the given id
     *
     * @param $id int           
     *
     * @return CourseTypeRelCourseSettingValue
     */
    function retrieve_course_type_rel_course_setting_value($id)
    {
        $condition = new EqualityCondition(CourseTypeRelCourseSettingValue :: PROPERTY_ID, $id);
        return $this->retrieve_object(CourseTypeRelCourseSettingValue :: get_table_name(), $condition);
    }

    /**
     * Retrieves a list of available course type rel course setting values
     *
     * @param $condition \common\libraries\Condition           
     * @param $offset int           
     * @param $max_objects int           
     * @param $order_by \common\libraries\ObjectTableOrder           
     *
     * @return \common\libraries\ResultSet <CourseTypeRelCourseSettingValue>
     */
    function retrieve_course_type_rel_course_setting_values($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        return $this->retrieve_objects(CourseTypeRelCourseSettingValue :: get_table_name(), $condition, $offset, $max_objects, $order_by);
    }

    /**
     * Counts the course type rel course setting values
     *
     * @param $condition \common\libraries\Condition           
     *
     * @return int
     */
    function count_course_type_rel_course_setting_values($condition = null)
    {
        return $this->count_objects(CourseTypeRelCourseSettingValue :: get_table_name(), $condition);
    }

    /**
     * **************************************************************************************************************
     * CourseTypeUserOrder Dataclass Functionality *
     * **************************************************************************************************************
     */
    
    /**
     * Retrieves a course type user order with the given id
     *
     * @param $id int           
     *
     * @return CourseType
     */
    function retrieve_course_type_user_order($id)
    {
        $condition = new EqualityCondition(CourseTypeUserOrder :: PROPERTY_ID, $id);
        return $this->retrieve_object(CourseTypeUserOrder :: get_table_name(), $condition);
    }

    /**
     * Retrieves a list of available course type user orders
     *
     * @param $condition \common\libraries\Condition           
     * @param $offset int           
     * @param $max_objects int           
     * @param $order_by \common\libraries\ObjectTableOrder           
     *
     * @return \common\libraries\ResultSet<CourseType>
     */
    function retrieve_course_type_user_orders($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        return $this->retrieve_objects(CourseTypeUserOrder :: get_table_name(), $condition, $offset, $max_objects, $order_by);
    }

    /**
     * Counts the course type user orders
     *
     * @param $condition \common\libraries\Condition           
     *
     * @return int
     */
    function count_course_type_user_orders($condition = null)
    {
        return $this->count_objects(CourseTypeUserOrder :: get_table_name(), $condition);
    }
}
?>
