<?php

namespace application\weblcms\course_type;

use common\libraries\DataClass;
use common\libraries\Utilities;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Translation;

use application\weblcms\WeblcmsDataManager;
use application\weblcms\CourseManagementRights;
use application\weblcms\CourseSettingsController;
use application\weblcms\CourseSetting;

use application\weblcms\course\CourseDataManager;
use application\weblcms\CourseSettingsConnector;

/**
 * Describes a course type (templating and rights system for courses)
 *
 * @package application\weblcms\course_type;
 *         
 * @author Yannick & Tristan
 * @author Sven Vanpoucke - Hogeschool Gent - Refactoring
 */
class CourseType extends DataClass
{
    /**
     * **************************************************************************************************************
     * Table Properties *
     * **************************************************************************************************************
     */
    
    const PROPERTY_TITLE = 'title';
    const PROPERTY_DESCRIPTION = 'description';
    const PROPERTY_ACTIVE = 'active';
    const PROPERTY_DISPLAY_ORDER = 'display_order';
    
    /**
     * **************************************************************************************************************
     * Display Order Properties *
     * **************************************************************************************************************
     */
    
    /**
     * When changing the display order of this object, the old display order is
     * still available so we can fix the
     * other display orders
     *
     * @var int
     */
    private $old_display_order;

    /**
     * **************************************************************************************************************
     * Inherited Functionality *
     * **************************************************************************************************************
     */
    
    /**
     * Returns the default properties of this dataclass
     *
     * @return String[] - The property names.
     */
    static function get_default_property_names($extended_property_names = array())
    {
        return parent :: get_default_property_names(array(self :: PROPERTY_TITLE, self :: PROPERTY_ACTIVE, 
                self :: PROPERTY_DESCRIPTION, self :: PROPERTY_DISPLAY_ORDER));
    }

    /**
     * Returns the datamanager for this dataclass
     *
     * @return CourseTypeDataManager
     */
    function get_data_manager()
    {
        return CourseTypeDataManager :: get_instance();
    }

    /**
     * **************************************************************************************************************
     * CRUD Functionality *
     * **************************************************************************************************************
     */
    
    /**
     * Sets the default display order and creates this object in the database
     *
     * @return boolean
     */
    function create($create_in_batch = false)
    {
        $this->set_display_order($this->get_data_manager()->retrieve_next_sort_value(self :: get_table_name(),
                self :: PROPERTY_DISPLAY_ORDER));
        
        if (! parent :: create())
        {
            return false;
        }
        
        $parent_id = $this->get_parent_rights_location()->get_id();
        
        if (! CourseManagementRights :: get_instance()->create_location_in_courses_subtree(
                CourseManagementRights :: TYPE_COURSE_TYPE, $this->get_id(), $parent_id, 0, $create_in_batch, 0))
        {
            return false;
        }
        
        return true;
    }

    /**
     * Updates the object and fixes the display orders if necessary
     *
     * @return boolean
     */
    function update()
    {
        $old_display_order = $this->get_old_display_order();
        if ($old_display_order)
        {
            $new_display_order = $this->get_display_order();
            $direction = ($new_display_order > $old_display_order) ? CourseTypeDataManager :: DISPLAY_ORDER_DIRECTION_UP : CourseTypeDataManager :: DISPLAY_ORDER_DIRECTION_DOWN;
            
            CourseTypeDataManager :: fix_course_type_display_order($old_display_order, $new_display_order, 1, $direction);
            
            $this->set_old_display_order();
        }
        
        return parent :: update();
    }

    /**
     * Deletes the object and fixes the display orders
     *
     * @return boolean
     */
    function delete()
    {
        $location = $this->get_rights_location();
        
        if ($location)
        {
            if (! $location->delete())
            {
                return false;
            }
        }
        
        $succes = parent :: delete();
        
        if (! $succes)
        {
            return false;
        }
        
        CourseTypeDataManager :: fix_course_type_display_order($this->get_display_order());
        
        return true;
    }

    /**
     * **************************************************************************************************************
     * Rights Functionality *
     * **************************************************************************************************************
     */
    
    /**
     * Returns the rights location for this object
     *
     * @return RightsLocation
     */
    function get_rights_location()
    {
        return CourseManagementRights :: get_instance()->get_weblcms_location_by_identifier_from_courses_subtree(CourseManagementRights :: TYPE_COURSE_TYPE, $this->get_id(), 0);
    }

    /**
     * Returns the (possible) parent rights location depending on this objects
     * properties
     *
     * @return RightsLocation
     */
    function get_parent_rights_location()
    {
        return CourseManagementRights :: get_instance()->get_courses_subtree_root(0);
    }

    /**
     * Returns the available rights for the course management
     *
     * @return int[string]
     */
    function get_available_course_management_rights()
    {
        return CourseManagementRights :: get_instance()->get_all_course_management_rights();
    }

    /**
     * Checks whether or not this object can change the given course management
     * right
     * Returns always true because a course type is not limited
     *
     * @param $right_id int           
     *
     * @return boolean
     */
    function can_change_course_management_right($right_id)
    {
        return true;
    }

    /**
     * Forces all the course rights to the courses connected to this course
     * type.
     *
     * @return boolean
     */
    function force_rights_to_courses()
    {
        return CourseManagementRights :: get_instance()->copy_rights_to_child_locations($this->get_rights_location());
    }

    /**
     * **************************************************************************************************************
     * Course Settings Functionality *
     * **************************************************************************************************************
     */
    
    /**
     * Retrieves course setting values for the given setting name and tool id
     *
     * @param $setting_name string           
     * @param $tool_id int           
     *
     * @return string[]
     */
    function get_course_setting($setting_name, $tool_id = 0)
    {
        return CourseSettingsController :: get_instance()->get_course_type_setting($this->get_id(), $setting_name, $tool_id);
    }

    /**
     * Retrieves the default values for a given course setting
     *
     * @param $setting_name string           
     * @param $tool_id int           
     *
     * @return string[]
     */
    function get_default_course_setting($setting_name, $tool_id)
    {
        return CourseSettingsController :: get_instance()->get_default_setting($setting_name, $tool_id);
    }

    /**
     * Delegation function to create course settings from given values
     *
     * @param $values string[]           
     *
     * @return boolean
     */
    function create_course_settings_from_values($values)
    {
        return CourseSettingsController :: get_instance()->handle_settings_for_object_with_given_values($this, $values);
    }

    /**
     * Delegation function to update course settings from given values
     *
     * @param $values string           
     *
     * @return boolean
     */
    function update_course_settings_from_values($values)
    {
        return CourseSettingsController :: get_instance()->handle_settings_for_object_with_given_values($this, $values, CourseSettingsController :: SETTING_ACTION_UPDATE);
    }

    /**
     * Creates a course setting relation for the given course setting object
     *
     * @param $course_setting CourseSetting           
     * @param $locked boolean           
     *
     * @return CourseTypeRelCourseSetting
     */
    function create_course_setting_relation(CourseSetting $course_setting, $locked)
    {
        $course_type_rel_setting = new CourseTypeRelCourseSetting();
        $course_type_rel_setting->set_course_setting($course_setting);
        $course_type_rel_setting->set_course_type($this);
        $course_type_rel_setting->set_locked($locked);
        
        if (! $course_type_rel_setting->create())
        {
            throw new \Exception(Translation :: get('CouldNotCreateCourseRelCourseTypeSetting'));
        }
        
        return $course_type_rel_setting;
    }

    /**
     * Updates a course setting relation for the given course setting object
     *
     * @param $course_setting CourseSetting           
     * @param $locked boolean           
     *
     * @return CourseTypeRelCourseSetting
     */
    function update_course_setting_relation(CourseSetting $course_setting, $locked)
    {
        $course_setting_relation = $this->retrieve_course_setting_relation($course_setting);
        
        if ($course_setting_relation && $course_setting_relation->is_locked() != $locked)
        {
            $course_setting_relation->set_locked($locked);
            if (! $course_setting_relation->update())
            {
                throw new \Exception(Translation :: get('CouldNotUpdateCourseRelCourseTypeSetting'));
            }
        }
        
        return $course_setting_relation;
    }

    /**
     * Retrieves a course setting relation object for the given course setting
     * object
     *
     * @param $course_setting CourseSetting           
     *
     * @return CourseTypeRelCourseSetting
     */
    function retrieve_course_setting_relation(CourseSetting $course_setting)
    {
        $conditions = array();
        
        $conditions[] = new EqualityCondition(CourseTypeRelCourseSetting :: PROPERTY_COURSE_SETTING_ID, $course_setting->get_id());
        $conditions[] = new EqualityCondition(CourseTypeRelCourseSetting :: PROPERTY_COURSE_TYPE_ID, $this->get_id());
        
        $condition = new AndCondition($conditions);
        
        return CourseTypeDataManager :: get_instance()->retrieve_course_type_rel_course_settings($condition)->next_result();
    }

    /**
     * Returns whether or not a given course setting is locked for this object.
     *
     * @param $course_setting CourseSetting           
     *
     * @return boolean
     */
    function is_course_setting_locked(CourseSetting $course_setting)
    {
        $course_setting_relation = $this->retrieve_course_setting_relation($course_setting);
        
        if (! $course_setting_relation)
        {
            return false;
        }
        
        return $course_setting_relation->is_locked();
    }

    /**
     * Returns whether or not a given course setting can be changed by this
     * object
     *
     * @param $course_setting CourseSetting           
     *
     * @return boolean
     */
    function can_change_course_setting(CourseSetting $course_setting)
    {
        return true;
    
    }

    /**
     * Forces all the course settings to the courses connected to this course
     * type.
     * Handles the copied settings for the course table
     *
     * @return boolean
     */
    function force_course_settings_to_courses()
    {
        CourseSettingsController :: get_instance()->clear_cache_for_type_and_object(CourseSettingsController :: SETTING_TYPE_COURSE_TYPE, $this->get_id());
        
        if (! CourseDataManager :: copy_course_settings_from_course_type($this->get_id()))
        {
            return false;
        }
        
        $properties = array();
        
        $copied_settings = CourseSettingsConnector :: get_copied_settings_for_course();
        foreach ($copied_settings as $setting_name => $property)
        {
            $properties[$property] = $this->get_course_setting($setting_name);
        }
        
        return CourseDataManager :: update_courses_from_course_type_with_properties($this->get_id(), $properties);
    }

    /**
     * **************************************************************************************************************
     * Helper Functionality *
     * **************************************************************************************************************
     */
    
    /**
     * Updates the current display order added with the given value
     *
     * @param $count int           
     */
    function update_display_order_with_count($count)
    {
        $this->set_display_order($this->get_display_order() + $count);
    }

    /**
     * **************************************************************************************************************
     * Getters and Setters *
     * **************************************************************************************************************
     */
    
    /**
     * Returns the title of this CourseType object
     *
     * @return String
     */
    function get_title()
    {
        return $this->get_default_property(self :: PROPERTY_TITLE);
    }

    /**
     * Sets the title of this CourseType object
     *
     * @param $title String           
     */
    function set_title($title)
    {
        $this->set_default_property(self :: PROPERTY_TITLE, $title);
    }

    /**
     * Returns the description of this CourseType object
     *
     * @return String
     */
    function get_description()
    {
        return $this->get_default_property(self :: PROPERTY_DESCRIPTION);
    }

    /**
     * Sets the description of this CourseType object
     *
     * @param $description String           
     */
    function set_description($description)
    {
        $this->set_default_property(self :: PROPERTY_DESCRIPTION, $description);
    }

    /**
     * Returns the active status of this CourseType object
     *
     * @return boolean
     */
    function is_active()
    {
        return $this->get_default_property(self :: PROPERTY_ACTIVE);
    }

    /**
     * Sets the active status of this CourseType object
     *
     * @param $active boolean           
     */
    function set_active($active)
    {
        $this->set_default_property(self :: PROPERTY_ACTIVE, $active);
    }

    /**
     * Returns the display_order of this CourseType object
     *
     * @return int
     */
    function get_display_order()
    {
        return $this->get_default_property(self :: PROPERTY_DISPLAY_ORDER);
    }

    /**
     * Sets the display_order of this CourseType object
     *
     * @param $display_order int           
     */
    function set_display_order($display_order)
    {
        $this->set_old_display_order($this->get_display_order());
        
        $this->set_default_property(self :: PROPERTY_DISPLAY_ORDER, $display_order);
    }

    /**
     * **************************************************************************************************************
     * Display order getters and setters *
     * **************************************************************************************************************
     */
    
    /**
     * Returns the old_display_order of this CourseType object
     *
     * @return int
     */
    function get_old_display_order()
    {
        return $this->old_display_order;
    }

    /**
     * Sets the old_display_order of this CourseType object
     *
     * @param $old_display_order int           
     */
    function set_old_display_order($old_display_order)
    {
        $this->old_display_order = $old_display_order;
    }
}
?>