<?php

namespace application\weblcms\course_type;

use common\libraries\DataClass;
use common\libraries\Utilities;
use common\libraries\EqualityCondition;

/**
 * Describes a course type user order (personal ordering of the course types per
 * user)
 *
 * @package application\weblcms\course_type;
 *         
 * @author Sven Vanpoucke - Hogeschool Gent
 */
class CourseTypeUserOrder extends DataClass
{
    /**
     * **************************************************************************************************************
     * Table Properties *
     * **************************************************************************************************************
     */
    
    const PROPERTY_COURSE_TYPE_ID = 'course_type_id';
    const PROPERTY_USER_ID = 'user_id';
    const PROPERTY_DISPLAY_ORDER = 'display_order';
    
    /**
     * **************************************************************************************************************
     * Display Order Properties *
     * **************************************************************************************************************
     */
    
    /**
     * When changing the display order of this object, the old display order is
     * still available so we can fix the
     * other display orders
     *
     * @var int
     */
    private $old_display_order;

    /**
     * **************************************************************************************************************
     * Inherited Functionality *
     * **************************************************************************************************************
     */
    
    /**
     * Returns the default properties of this dataclass
     *
     * @return String[] - The property names.
     */
    static function get_default_property_names($extended_property_names = array())
    {
        return parent :: get_default_property_names(array(self :: PROPERTY_COURSE_TYPE_ID, self :: PROPERTY_USER_ID, 
                self :: PROPERTY_DISPLAY_ORDER));
    }

    /**
     * Returns the datamanager for this dataclass
     *
     * @return CourseTypeDataManager
     */
    function get_data_manager()
    {
        return CourseTypeDataManager :: get_instance();
    }

    /**
     * **************************************************************************************************************
     * CRUD Functionality *
     * **************************************************************************************************************
     */
    
    /**
     * Sets the default display order and creates this object in the database
     *
     * @return boolean
     */
    function create()
    {
        $condition = new EqualityCondition(self :: PROPERTY_USER_ID, $this->get_user_id());
        
        $this->set_display_order($this->get_data_manager()->retrieve_next_sort_value(self :: get_table_name(), self :: PROPERTY_DISPLAY_ORDER, $condition));
        
        return parent :: create();
    }

    /**
     * Updates the object and fixes the display orders if necessary
     *
     * @return boolean
     */
    function update()
    {
        $old_display_order = $this->get_old_display_order();
        if ($old_display_order)
        {
            $new_display_order = $this->get_display_order();
            
            $direction = ($new_display_order > $old_display_order) ? CourseTypeDataManager :: DISPLAY_ORDER_DIRECTION_UP : CourseTypeDataManager :: DISPLAY_ORDER_DIRECTION_DOWN;
            
            $condition = new EqualityCondition(self :: PROPERTY_USER_ID, $this->get_user_id());
            
            CourseTypeDataManager :: fix_course_type_user_order_display_order($condition, $old_display_order, $new_display_order, 1, $direction);
            
            $this->set_old_display_order();
        }
        
        return parent :: update();
    }

    /**
     * Deletes the object and fixes the display orders
     *
     * @return boolean
     */
    function delete()
    {
        $succes = parent :: delete();
        
        if (! $succes)
        {
            return false;
        }
        
        $condition = new EqualityCondition(self :: PROPERTY_USER_ID, $this->get_user_id());
        
        CourseTypeDataManager :: fix_course_type_user_order_display_order($condition, $this->get_display_order());
        
        return true;
    }

    /**
     * **************************************************************************************************************
     * Helper Functionality *
     * **************************************************************************************************************
     */
    
    /**
     * Changes the current display order added with the given value
     *
     * @param $count int           
     */
    function change_display_order_with_count($count)
    {
        $this->set_display_order($this->get_display_order() + $count);
    }

    /**
     * **************************************************************************************************************
     * Getters and Setters *
     * **************************************************************************************************************
     */
    
    /**
     * Returns the course_type_id of this CourseType object
     *
     * @return String
     */
    function get_course_type_id()
    {
        return $this->get_default_property(self :: PROPERTY_COURSE_TYPE_ID);
    }

    /**
     * Sets the course_type_id of this CourseType object
     *
     * @param $course_type_id String           
     */
    function set_course_type_id($course_type_id)
    {
        $this->set_default_property(self :: PROPERTY_COURSE_TYPE_ID, $course_type_id);
    }

    /**
     * Returns the user_id of this CourseType object
     *
     * @return String
     */
    function get_user_id()
    {
        return $this->get_default_property(self :: PROPERTY_USER_ID);
    }

    /**
     * Sets the user_id of this CourseType object
     *
     * @param $user_id String           
     */
    function set_user_id($user_id)
    {
        $this->set_default_property(self :: PROPERTY_USER_ID, $user_id);
    }

    /**
     * Returns the display_order of this CourseType object
     *
     * @return int
     */
    function get_display_order()
    {
        return $this->get_default_property(self :: PROPERTY_DISPLAY_ORDER);
    }

    /**
     * Sets the display_order of this CourseType object
     *
     * @param $display_order int           
     */
    function set_display_order($display_order)
    {
        $this->set_old_display_order($this->get_display_order());
        
        $this->set_default_property(self :: PROPERTY_DISPLAY_ORDER, $display_order);
    }

    /**
     * **************************************************************************************************************
     * Display order getters and setters *
     * **************************************************************************************************************
     */
    
    /**
     * Returns the old_display_order of this CourseType object
     *
     * @return int
     */
    function get_old_display_order()
    {
        return $this->old_display_order;
    }

    /**
     * Sets the old_display_order of this CourseType object
     *
     * @param $old_display_order int           
     */
    function set_old_display_order($old_display_order)
    {
        $this->old_display_order = $old_display_order;
    }
}
?>