<?php
namespace application\weblcms;

use common\libraries\DataClassRetrievesParameters;
use common\libraries\AjaxManager;
use common\libraries\JsonAjaxResult;
use common\libraries\Path;
use common\libraries\AdvancedElementFinderElements;
use common\libraries\AdvancedElementFinderElement;
use common\libraries\Request;
use common\libraries\PatternMatchCondition;
use common\libraries\OrCondition;
use common\libraries\EqualityCondition;
use common\libraries\InCondition;
use common\libraries\AndCondition;
use common\libraries\ObjectTableOrder;
use user\UserDataManager;
use user\User;
use common\libraries\Utilities;
use application\weblcms\course\CourseGroupRelation;
use application\weblcms\course\CourseUserRelation;

/**
 * Feed to return users of this course
 *
 * @author Sven Vanpoucke
 * @package application.weblcms
 */
class WeblcmsAjaxCourseUsersFeed extends AjaxManager
{
    const PARAM_SEARCH_QUERY = 'query';
    const PARAM_COURSE_ID = 'course_id';
    const PARAM_OFFSET = 'offset';
    const PROPERTY_ELEMENTS = 'elements';
    const PROPERTY_TOTAL_ELEMENTS = 'total_elements';

    private $user_count = 0;

    function required_parameters()
    {
        return array(self :: PARAM_COURSE_ID);
    }

    /**
     * Runs this ajax component
     */
    function run()
    {
        $result = new JsonAjaxResult();

        $elements = $this->get_elements();
        $elements = $elements->as_array();

        $result->set_property(self :: PROPERTY_ELEMENTS, $elements);
        $result->set_property(self :: PROPERTY_TOTAL_ELEMENTS, $this->user_count);

        $result->display();
    }

    /**
     * Returns all the elements for this feed
     *
     * @return Array
     */
    private function get_elements()
    {
        $elements = new AdvancedElementFinderElements();

        // Add user category
        $user_category = new AdvancedElementFinderElement('users', 'category', 'Users', 'Users');
        $elements->add_element($user_category);

        $users = $this->retrieve_users();
        if ($users)
        {
            while ($user = $users->next_result())
            {
                $user_category->add_child(
                        new AdvancedElementFinderElement(CourseUserEntity :: ENTITY_TYPE . '_' . $user->get_id(),
                                'type type_user', $user->get_fullname(), $user->get_official_code()));
            }
        }

        return $elements;
    }

    /**
     * Retrieves the users from the course (direct subscribed and group subscribed)
     *
     * @return ResultSet
     */
    private function retrieve_users()
    {
        $course_id = $this->get_parameter(self :: PARAM_COURSE_ID);

        $wdm = WeblcmsDataManager :: get_instance();
        $udm = UserDataManager :: get_instance();

        // Retrieve the users directly subscribed to the course
        $relation_condition = new EqualityCondition(CourseUserRelation :: PROPERTY_COURSE_ID, $course_id);
        $course_user_relation_result_set = $wdm->retrieve_course_user_relations($relation_condition);

        $user_ids = array();
        while ($course_user = $course_user_relation_result_set->next_result())
        {
            $user_ids[] = $course_user->get_user_id();
        }

        // Retrieve the users subscribed through platform groups
        $relation_condition = new EqualityCondition(CourseGroupRelation :: PROPERTY_COURSE_ID, $course_id);
        $course_group_relations = $wdm->retrieve_course_group_relations($relation_condition);

        $group_users = array();

        while ($group_relation = $course_group_relations->next_result())
        {
            $group = $group_relation->get_group();
            $group_user_ids = $group->get_users(true, true);

            $group_users = array_merge($group_users, $group_user_ids);
        }

        $user_ids = array_merge($user_ids, $group_users);

        if (count($user_ids) == 0)
        {
            return;
        }

        $search_query = Request :: post(self :: PARAM_SEARCH_QUERY);

        // Set the conditions for the search query
        if ($search_query && $search_query != '')
        {
            $conditions[] = Utilities :: query_to_condition($search_query,
                    array(User :: PROPERTY_USERNAME, User :: PROPERTY_FIRSTNAME, User :: PROPERTY_LASTNAME));
        }

        $conditions[] = new InCondition(User :: PROPERTY_ID, $user_ids);

        // Combine the conditions
        $count = count($conditions);
        if ($count > 1)
        {
            $condition = new AndCondition($conditions);
        }

        if ($count == 1)
        {
            $condition = $conditions[0];
        }
        $this->user_count = $udm->count_users($condition);

        $parameters = new DataClassRetrievesParameters($condition, 100, $this->get_offset(),
                array(new ObjectTableOrder(User :: PROPERTY_FIRSTNAME), new ObjectTableOrder(User :: PROPERTY_LASTNAME)));

        return \user\DataManager :: retrieves(User :: class_name(), $parameters);
    }

    protected function get_offset()
    {
        $offset = Request :: post(self :: PARAM_OFFSET);
        if (! isset($offset) || is_null($offset))
        {
            $offset = 0;
        }

        return $offset;
    }
}

?>