<?php

namespace application\weblcms;

use common\libraries\Utilities;
use common\libraries\Path;
use common\libraries\EqualityCondition;
use common\libraries\ObjectTableOrder;
use common\libraries\OptionsMenuRenderer;
use common\libraries\TreeMenuRenderer;
use HTML_Menu;
use HTML_Menu_ArrayRenderer;
use common\libraries\Request;
use common\libraries\AndCondition;
use common\libraries\GenericTree;
use common\extensions\rights_editor_manager\RightsEditorManagerBrowserComponent;
use group\GroupManager;
use group\GroupDataManager;
use common\libraries\InCondition;
use group\Group;

/**
 * $Id: group_menu.class.php 224 2009-11-13 14:40:30Z kariboe $
 * 
 * @package group.lib
 */

/**
 * This class provides a navigation menu to allow a user to browse through
 * categories of courses.
 * 
 * @author Pieterjan Broekaert
 */
class PlatformGroupsTree extends GenericTree
{
    const TREE_NAME = __CLASS__;
    const ROOT_NODE_CLASS = 'home';
    const NODE_CLASS = 'category';
    const ROOT_NODE_TITLE = 'root';
    
    private $data_manager;
    private $browser;
    private $subscribed_groups;

    /**
     * Creates a new category navigation menu.
     * 
     * @param $owner int
     *            The ID of the owner of the categories to provide in
     *            this menu.
     * @param $current_category int
     *            The ID of the current category in the menu.
     * @param $url_format string
     *            The format to use for the URL of a category.
     *            Passed to sprintf(). Defaults to the string
     *            "?category=%s".
     * @param $extra_items array
     *            An array of extra tree items, added to the
     *            root.
     */
    function __construct($browser)
    {
        $this->data_manager = WeblcmsDataManager :: get_instance();
        $this->browser = $browser;
        
        // get all subscribed groups to build the tree
        $relation_condition = new EqualityCondition(CourseGroupRelation :: PROPERTY_COURSE_ID, Request :: get(WeblcmsManager :: PARAM_COURSE));
        $relation_children = $this->data_manager->retrieve_course_group_relations($relation_condition);
        while ($child = $relation_children->next_result())
        {
            $this->subscribed_groups[] = $child->get_group_id();
        }
        
        parent :: __construct(false);
    }

    /**
     * Returns the url of a node
     * 
     * @param $node_id int           
     * @return string
     */
    public function get_node_url($node_id)
    {
        if ($node_id != 0)
        {
            $url_param[RightsEditorManagerBrowserComponent :: PARAM_TYPE] = 'group';
            $url_param[WeblcmsManager :: PARAM_COURSE] = Request :: get(WeblcmsManager :: PARAM_COURSE);
            $url_param[GroupManager :: PARAM_GROUP_ID] = $node_id;
        }
        return $this->browser->get_url($url_param);
    }

    public function get_current_node_id()
    {
        $node_id = intval(Request :: get(GroupManager :: PARAM_GROUP_ID));
        if ($node_id != 0)
            return $node_id;
        else
            return $this->subscribed_groups[0];
    }

    public function get_node($node_id)
    {
        $node = GroupDataManager :: get_instance()->retrieve_group($node_id);
        return $node;
    }

    public function get_node_id($node)
    {
        return $node->get_id();
    }

    public function get_node_children($parent_node_id)
    {
        if ($parent_node_id == 0)
        {
            $in_condition = new InCondition(Group :: PROPERTY_ID, $this->subscribed_groups);
            $children = GroupDataManager :: get_instance()->retrieve_groups($in_condition);
        }
        else
        {
            $eq_condition = new EqualityCondition(Group :: PROPERTY_PARENT, $parent_node_id);
            $children = GroupDataManager :: get_instance()->retrieve_groups($eq_condition);
        }
        return $children;
    }

    public function node_has_children($node_id)
    {
        $eq_condition = new EqualityCondition(Group :: PROPERTY_PARENT, $node_id);
        if (GroupDataManager :: get_instance()->count_groups($eq_condition) > 0)
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    public function get_search_url()
    {
        return Path :: get(WEB_PATH) . 'group/php/xml_feeds/xml_group_menu_feed.php';
    }

    public function get_url_format()
    {
        $course_id = $this->browser->get_parent()->get_course_id();
        $tool = Request :: get(WeblcmsManager :: PARAM_TOOL);
        
        $url_format = '?application=weblcms&course=' . $course_id . '&go=course_viewer&tool=' . $tool;
        
        $tool_action = Request :: get(WeblcmsManager :: PARAM_TOOL_ACTION);
        if (! is_null($tool_action))
            $url_format .= '&' . WeblcmsManager :: PARAM_TOOL_ACTION . '=' . $tool_action;
        $browser_type = Request :: get(Tool :: PARAM_BROWSER_TYPE);
        if (! is_null($browser_type))
            $url_format .= '&' . Tool :: PARAM_BROWSER_TYPE . '=' . $browser_type;
        $rights_type = CoursesRightsEditorManager :: TYPE_GROUP;
        if (! is_null($rights_type))
            $url_format .= '&rights_type=' . $rights_type;
        
        $url_format .= '&' . GroupManager :: PARAM_GROUP_ID . '=';
        
        // $url_format .= '&publication_category=';
        
        return $url_format;
    }

    public function get_root_node_class()
    {
        return self :: ROOT_NODE_CLASS;
    }

    public function get_node_class($node)
    {
        return self :: NODE_CLASS;
    }

    public function get_root_node_title()
    {
        return self :: ROOT_NODE_TITLE;
    }

    public function get_node_title($node)
    {
        return $node->get_name();
    }

    public function get_node_parent($node)
    {
        return $node->get_parent();
    }

    public function get_current_url()
    {
        if ($this->get_current_node_id() == 0)
        {
            return $this->get_node_url($this->subscribed_groups[0]);
        }
        $this->get_node_url($this->get_current_node_id());
    }

    function create_tree_recursive($node_id)
    {
        $node = $this->get_node($node_id);
        $tree_item = array();
        $tree_item['title'] = $this->get_node_title($node);
        $tree_item['url'] = $this->get_node_url($this->get_node_id($node));
        $sub = $this->retrieve_child_tree_items($this->get_node_id($node));
        if (count($sub) > 0)
        {
            $tree_item['sub'] = $sub;
            $tree_item['children'] = 'collapse';
        }
        $tree_item['class'] = $this->get_node_class();
        $tree_item['id'] = $node_id;
        
        // $this->tree[$this->current_node_id] = $tree_item;
        $this->active_tree_node = $tree_item;
        
        if ($this->get_node_parent($node) == 0 || in_array($node->get_id(), $this->subscribed_groups))
        {
            $this->create_tree_root();
        }
        else
        {
            $this->create_tree_recursive($this->get_node_parent($node));
        }
    }

}

?>