<?php

namespace application\weblcms;

use common\libraries\Utilities;
use common\libraries\Path;
use common\libraries\EqualityCondition;
use common\libraries\ObjectTableOrder;
use common\libraries\OptionsMenuRenderer;
use common\libraries\TreeMenuRenderer;
use HTML_Menu;
use HTML_Menu_ArrayRenderer;
use common\libraries\Request;
use common\libraries\AndCondition;
use common\libraries\GenericTree;
use common\libraries\Translation;

/**
 * $Id: group_menu.class.php 224 2009-11-13 14:40:30Z kariboe $
 * 
 * @package group.lib
 */

/**
 * This class provides a navigation menu to allow a user to browse through
 * categories of courses.
 * 
 * @author Pieterjan Broekaert
 */
class PublicationCategoriesTree extends GenericTree
{
    const TREE_NAME = __CLASS__;
    const ROOT_NODE_CLASS = 'category';
    
    const CATEGORY_CLASS = 'category';
    const NEW_CATEGORY_CLASS = 'new_category';
    const INVISIBLE_CATEGORY_CLASS = 'invisible_category';
    
    private $data_manager;
    private $browser;

    /**
     * Creates a new category navigation menu.
     * 
     * @param $owner int
     *            The ID of the owner of the categories to provide in
     *            this menu.
     * @param $current_category int
     *            The ID of the current category in the menu.
     * @param $url_format string
     *            The format to use for the URL of a category.
     *            Passed to sprintf(). Defaults to the string
     *            "?category=%s".
     * @param $extra_items array
     *            An array of extra tree items, added to the
     *            root.
     */
    function __construct($browser)
    {
        $this->data_manager = WeblcmsDataManager :: get_instance();
        $this->browser = $browser;
        parent :: __construct();
    }

    /**
     * Returns the url of a node
     * 
     * @param $node_id int           
     * @return string
     */
    public function get_node_url($node_id)
    {
        $url_param[WeblcmsManager :: PARAM_CATEGORY] = $node_id;
        return $this->browser->get_url($url_param);
    }

    public function get_current_node_id()
    {
        return intval(Request :: get(WeblcmsManager :: PARAM_CATEGORY));
    }

    public function get_node($node_id)
    {
        $child = $this->data_manager->retrieve_content_object_publication_category($node_id);
        return $child;
    }

    public function get_node_children($parent_node_id)
    {
        $conditions[] = new EqualityCondition(ContentObjectPublicationCategory :: PROPERTY_PARENT, $parent_node_id);
        $conditions[] = new EqualityCondition(ContentObjectPublicationCategory :: PROPERTY_COURSE, $this->browser->get_parent()->get_course_id());
        $conditions[] = new EqualityCondition(ContentObjectPublicationCategory :: PROPERTY_TOOL, $this->browser->get_parent()->get_tool_id());
        if (! $this->is_invisible_allowed())
            $conditions[] = new EqualityCondition(ContentObjectPublicationCategory :: PROPERTY_VISIBLE, true);
        $condition = new AndCondition($conditions);
        
        $children = $this->data_manager->retrieve_content_object_publication_categories($condition, null, null, new ObjectTableOrder(ContentObjectPublicationCategory :: PROPERTY_DISPLAY_ORDER));
        
        return $children;
    }

    public function node_has_children($node_id)
    {
        $conditions[] = new EqualityCondition(ContentObjectPublicationCategory :: PROPERTY_PARENT, $node_id);
        $conditions[] = new EqualityCondition(ContentObjectPublicationCategory :: PROPERTY_COURSE, $this->browser->get_parent()->get_course_id());
        $conditions[] = new EqualityCondition(ContentObjectPublicationCategory :: PROPERTY_TOOL, $this->browser->get_parent()->get_tool_id());
        if (! $this->is_invisible_allowed())
            $conditions[] = new EqualityCondition(ContentObjectPublicationCategory :: PROPERTY_VISIBLE, true);
        
        $condition = new AndCondition($conditions);
        if ($this->data_manager->count_content_object_publication_categories($condition) > 0)
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * Returns true if the current user is allowed to view invisible categories.
     */
    function is_invisible_allowed()
    {
        return $this->browser->is_allowed('view_invisible_category_right');
    }

    public function get_search_url()
    {
        return Path :: get(WEB_PATH) . 'application/weblcms/php/xml_feeds/xml_publications_tree_feed.php';
    }

    public function get_url_format()
    {
        $course_id = $this->browser->get_parent()->get_course_id();
        $tool = Request :: get(WeblcmsManager :: PARAM_TOOL);
        
        $url_format = '?application=weblcms&course=' . $course_id . '&go=course_viewer&tool=' . $tool;
        
        $tool_action = Request :: get(WeblcmsManager :: PARAM_TOOL_ACTION);
        if (! is_null($tool_action))
            $url_format .= '&tool_action=' . $tool_action;
        $browser_type = Request :: get(Tool :: PARAM_BROWSER_TYPE);
        if (! is_null($browser_type))
            $url_format .= '&browser=' . $browser_type;
        
        $url_format .= '&publication_category=%d';
        
        return $url_format;
    }

    public function get_root_node_class()
    {
        return self :: ROOT_NODE_CLASS;
    }

    public function get_node_class($node)
    {
        if ($this->get_node($node->get_id())->get_visibility())
        {
            if ($this->browser->tool_category_has_new_publications($node->get_id()))
            {
                return self :: NEW_CATEGORY_CLASS;
            }
            else
            {
                return self :: CATEGORY_CLASS;
            }
        }
        else
        {
            return self :: INVISIBLE_CATEGORY_CLASS;
        }
    }

    public function get_root_node_title()
    {
        return Translation :: get('Root');
    }

    public function get_node_title($node)
    {
        return $node->get_name();
    }

    function get_node_id($node)
    {
        return $node->get_id();
    }

    function get_node_parent($node)
    {
        return $node->get_parent();
    }

}

?>