<?php

namespace application\weblcms;

use common\libraries\Path;
use common\libraries\FormValidator;
use common\libraries\WebApplication;
use common\libraries\PlatformSetting;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\ToolbarItem;
use common\libraries\Toolbar;
use common\libraries\AdvancedElementFinderElementTypes;
use common\libraries\AdvancedElementFinderElementType;
use common\libraries\AdvancedElementFinderElements;
use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\Theme;
use common\libraries\ResourceManager;
use common\libraries\Mail;
use common\libraries\Session;
use common\libraries\Redirect;

use DOMDocument;
use repository\RepositoryDataManager;
use repository\content_object\document\Document;

use admin\AdminDataManager;
use group\GroupDataManager;
use user\UserDataManager;
use rights\RightsDataManager;
use repository\RepositoryRights;
use common\libraries\Display;
use admin\AdminManager;

/**
 * This class represents a form to allow a user to publish a learning object.
 *
 * The form allows the user to set some properties of the publication
 * (publication dates, target users, visibility, ...)
 *
 * @author Sven Vanpoucke
 */
class ContentObjectPublicationForm extends FormValidator
{
    const TYPE_CREATE = 1;
    const TYPE_UPDATE = 2;

    const PROPERTY_TARGETS = 'targets';
    const PROPERTY_FOREVER = 'forever';
    const PROPERTY_FROM_DATE = 'from_date';
    const PROPERTY_TO_DATE = 'to_date';
    const PROPERTY_PUBLISH_AND_BUILD = 'publish_and_build';

    // Rights
    const PROPERTY_INHERIT = 'inherit';
    const PROPERTY_RIGHT_OPTION = 'right_option';
    const PROPERTY_COLLABORATE = 'collaborate';

    const INHERIT_TRUE = 0;
    const INHERIT_FALSE = 1;

    const RIGHT_OPTION_ALL = 0;
    const RIGHT_OPTION_ME = 1;
    const RIGHT_OPTION_SELECT = 2;

    /**
     * The type of the form (create or edit)
     *
     * @var int
     */
    private $form_type;

    /**
     * The publications that will be created / edited
     */
    private $publications;

    /**
     * Caching of the course so that it can be used in email publication
     *
     * @var Course
     */
    private $course;

    /**
     * Available entities for the view rights
     *
     * @var Array
     */
    private $entities;

    /**
     * Is it possible to collaborate the selected objects?
     *
     * @var boolean
     */
    private $collaborate_possible;

    /**
     * Creates a new learning object publication form.
     */
    function __construct($form_type, $publications, $course, $action, $is_course_admin)
    {
        parent :: __construct('content_object_publication_form', 'post', $action);

        if (count($publications) <= 0)
        {
            Display :: error_page(Translation :: get('NoObjectsSelected', null, Utilities :: COMMON_LIBRARIES));
        }
        else
        {
            // set collaborate right for course admins if we are owner of each
            // content object to share
            $owner = true;
            foreach ($publications as $publication)
            {
                $owner_id = $publication->get_content_object()->get_owner_id();
                $owner &= ($owner_id == Session :: get_user_id());
            }
            if ($owner)
            {
                $this->collaborate_possible = true;
            }
            else
            {
                $this->collaborate_possible = false;
            }
        }

        $this->publications = $publications;
        $this->course = $course;
        $this->form_type = $form_type;
        $this->is_course_admin = $is_course_admin;

        $this->entities = array();
        $this->entities[CourseUserEntity :: ENTITY_TYPE] = new CourseUserEntity($course->get_id());
        $this->entities[CourseGroupEntity :: ENTITY_TYPE] = new CourseGroupEntity($course->get_id());
        $this->entities[CoursePlatformGroupEntity :: ENTITY_TYPE] = new CoursePlatformGroupEntity($course->get_id());

        switch ($form_type)
        {
            case self :: TYPE_CREATE :
                $this->build_create_form();
                break;
            case self :: TYPE_UPDATE :
                $this->build_update_form();
                break;
        }

        $this->setDefaults();
    }

    /**
     * Sets the default values of the form.
     *
     * By default the publication is for everybody who has access to the tool
     * and the publication will be available forever.
     */
    function setDefaults()
    {
        $defaults = array();

        $publications = $this->publications;

        $defaults[ContentObjectPublication :: PROPERTY_CATEGORY_ID] = Request :: get(WeblcmsManager :: PARAM_CATEGORY);
        $defaults[self :: PROPERTY_FOREVER] = 1;
        $defaults[self :: PROPERTY_INHERIT] = self :: INHERIT_TRUE;
        $defaults[self :: PROPERTY_RIGHT_OPTION] = self :: RIGHT_OPTION_ALL;

        if (count($publications) == 1)
        {
            $first_publication = $publications[0];

            if ($first_publication->get_id())
            {
                $defaults[ContentObjectPublication :: PROPERTY_CATEGORY_ID] = $first_publication->get_category_id();

                if ($first_publication->get_from_date() != 0)
                {
                    $defaults[self :: PROPERTY_FOREVER] = 0;
                    $defaults[ContentObjectPublication :: PROPERTY_FROM_DATE] = $first_publication->get_from_date();
                    $defaults[ContentObjectPublication :: PROPERTY_TO_DATE] = $first_publication->get_to_date();
                }

                $defaults[ContentObjectPublication :: PROPERTY_HIDDEN] = $first_publication->is_hidden();
                $defaults[ContentObjectPublication :: PROPERTY_SHOW_ON_HOMEPAGE] = $first_publication->get_show_on_homepage();
            }

            $right_defaults = $this->set_right_defaults($first_publication);
            if (! empty($right_defaults))
            {
                $defaults = array_merge($defaults, $right_defaults);
            }

        }

        if ($this->collaborate_possible)
        {
            $defaults[self :: PROPERTY_COLLABORATE] = 0;
        }

        parent :: setDefaults($defaults);
    }

    /**
     * Sets the default values for the publish for rights
     *
     * @param $publication ContentObjectPublication
     */
    function set_right_defaults(ContentObjectPublication $publication)
    {
        $right_defaults = array();

        $location = WeblcmsRights :: get_instance()->get_weblcms_location_by_identifier_from_courses_subtree(WeblcmsRights :: TYPE_PUBLICATION, $publication->get_id(), $publication->get_course_id());

        if (! $location)
        {
            return;
        }

        if ($location->inherits())
        {
            $right_defaults[self :: PROPERTY_INHERIT] = self :: INHERIT_TRUE;
            $right_defaults[self :: PROPERTY_RIGHT_OPTION] = self :: RIGHT_OPTION_ALL;
        }
        else
        {
            $right_defaults[self :: PROPERTY_INHERIT] = self :: INHERIT_FALSE;

            $selected_entities = RightsDataManager :: retrieve_rights_location_rights_for_location(WeblcmsManager :: APPLICATION_NAME, $location->get_id(), WeblcmsRights :: VIEW_RIGHT)->as_array();

            if (count($selected_entities) == 1)
            {
                $selected_entity = $selected_entities[0];
                if ($selected_entity->get_entity_type() == 0 && $selected_entity->get_entity_id() == 0)
                {
                    $right_defaults[self :: PROPERTY_RIGHT_OPTION . '_' . $right_id] = self :: RIGHT_OPTION_ALL;
                    return $right_defaults;
                }

                if ($selected_entity->get_entity_type() == 1 && $selected_entity->get_entity_id() == Session :: get_user_id())
                {
                    $right_defaults[self :: PROPERTY_RIGHT_OPTION] = self :: RIGHT_OPTION_ME;
                    return $right_defaults;
                }

            }

            $right_defaults[self :: PROPERTY_RIGHT_OPTION] = self :: RIGHT_OPTION_SELECT;

            $default_elements = new AdvancedElementFinderElements();

            foreach ($selected_entities as $selected_entity)
            {
                $entity = $this->entities[$selected_entity->get_entity_type()];

                $default_elements->add_element($entity->get_element_finder_element($selected_entity->get_entity_id()));
            }

            $element = $this->getElement(self :: PROPERTY_TARGETS);
            $element->setDefaultValues($default_elements);
        }

        return $right_defaults;
    }

    /**
     * Builds the create form with the publish button
     */
    function build_create_form()
    {
        $this->build_form();

        $this->addElement('checkbox', ContentObjectPublication :: PROPERTY_EMAIL_SENT, Translation :: get('SendByEMail'));

        // collaborate right for course admins if we are owner of each content
        // object to share
        if ($this->collaborate_possible)
        {
            $this->addElement('checkbox', self :: PROPERTY_COLLABORATE, Translation :: get('CourseAdminCollaborate'));
        }

        $buttons[] = $this->createElement('style_submit_button', self :: PARAM_SUBMIT, Translation :: get('Publish', null, Utilities :: COMMON_LIBRARIES), array(
                'class' => 'positive publish'));

        if (count($this->publications) == 1)
        {
            $first_publication = $this->publications[0];

            if ($first_publication && $first_publication->get_content_object()->is_complex_content_object() && ($first_publication->get_content_object()->get_owner_id() == Session :: get_user_id() || RepositoryRights :: get_instance()->is_allowed_in_user_subtree(RepositoryRights :: COLLABORATE_RIGHT, $first_publication->get_content_object()->get_id(), RepositoryRights :: TYPE_USER_CONTENT_OBJECT, $first_publication->get_content_object()->get_owner_id())))
            {
                $buttons[] = $this->createElement('style_submit_button', self :: PROPERTY_PUBLISH_AND_BUILD, Translation :: get('PublishAndBuild', null, Utilities :: COMMON_LIBRARIES), array(
                        'class' => 'positive build'));
            }
        }

        $buttons[] = $this->createElement('style_reset_button', self :: PARAM_RESET, Translation :: get('Reset', null, Utilities :: COMMON_LIBRARIES), array(
                'class' => 'normal empty'));

        $this->addGroup($buttons, 'buttons', null, '&nbsp;', false);
    }

    /**
     * Builds the update form with the update button
     */
    function build_update_form()
    {
        $this->build_form();

        $buttons[] = $this->createElement('style_submit_button', self :: PARAM_SUBMIT, Translation :: get('Update', null, Utilities :: COMMON_LIBRARIES), array(
                'class' => 'positive update'));
        $buttons[] = $this->createElement('style_reset_button', self :: PARAM_RESET, Translation :: get('Reset', null, Utilities :: COMMON_LIBRARIES), array(
                'class' => 'normal empty'));

        $this->addGroup($buttons, 'buttons', null, '&nbsp;', false);
    }

    /**
     * Builds the form by adding the necessary form elements.
     */
    function build_form()
    {
        $tool = WeblcmsDataManager :: retrieve_course_tool_by_name($this->get_tool());

        if ($this->is_course_admin || WeblcmsRights :: get_instance()->is_allowed_in_courses_subtree(WeblcmsRights :: ADD_RIGHT, $tool->get_id(), WeblcmsRights :: TYPE_COURSE_MODULE, $this->get_course_id()))
        {
            $this->categories[0] = Translation :: get('Root', null, Utilities :: COMMON_LIBRARIES);
        }

        $this->get_categories(0);

        if (count($this->categories) > 1 || ! $this->categories[0])
        {
            // More than one category -> let user select one
            $this->addElement('select', ContentObjectPublication :: PROPERTY_CATEGORY_ID, Translation :: get('Category', null, Utilities :: COMMON_LIBRARIES), $this->categories);
        }
        else
        {
            // Only root category -> store object in root category
            $this->addElement('hidden', ContentObjectPublication :: PROPERTY_CATEGORY_ID, 0);
        }

        $this->build_rights_form();

        $this->add_forever_or_timewindow();
        $this->addElement('checkbox', ContentObjectPublication :: PROPERTY_HIDDEN, Translation :: get('Hidden', null, Utilities :: COMMON_LIBRARIES));

        $this->addElement('checkbox', ContentObjectPublication :: PROPERTY_SHOW_ON_HOMEPAGE, Translation :: get('ShowOnHomepage'));
    }

    private $categories;
    private $level = 1;

    /**
     * Gets the categories for the current tool and course recursively
     *
     * @param $parent_id int
     */
    function get_categories($parent_id)
    {
        $conditions[] = new EqualityCondition(ContentObjectPublicationCategory :: PROPERTY_COURSE, $this->get_course_id());
        $conditions[] = new EqualityCondition(ContentObjectPublicationCategory :: PROPERTY_TOOL, $this->get_tool());
        $conditions[] = new EqualityCondition(ContentObjectPublicationCategory :: PROPERTY_PARENT, $parent_id);
        $condition = new AndCondition($conditions);

        $cats = WeblcmsDataManager :: get_instance()->retrieve_content_object_publication_categories($condition);
        while ($cat = $cats->next_result())
        {
            if ($this->is_course_admin || WeblcmsRights :: get_instance()->is_allowed_in_courses_subtree(WeblcmsRights :: ADD_RIGHT, $cat->get_id(), WeblcmsRights :: TYPE_COURSE_CATEGORY, $this->get_course_id()))
            {
                $this->categories[$cat->get_id()] = str_repeat('--', $this->level) . ' ' . $cat->get_name();
                $this->level ++;
                $this->level --;
            }

            $this->get_categories($cat->get_id());
        }
    }

    /**
     * Builds the form to set the view rights
     */
    function build_rights_form()
    {
        // Add the inheritance option
        $group = array();

        $group[] = & $this->createElement('radio', null, null, Translation :: get('InheritRights'), self :: INHERIT_TRUE, array(
                'class' => 'inherit_rights_selector'));
        // $inherit_group[] = & $this->createElement('html',;

        $group[] = & $this->createElement('radio', null, null, Translation :: get('UseSpecificRights'), self :: INHERIT_FALSE, array(
                'class' => 'specific_rights_selector'));

        $this->addGroup($group, self :: PROPERTY_INHERIT, Translation :: get('PublishFor', null, Utilities :: COMMON_LIBRARIES), '<br />');

        $this->addElement('html', '<div class="right">');

        // Add the rights options
        $group = array();

        $group[] = & $this->createElement('radio', null, null, Translation :: get('Everyone'), self :: RIGHT_OPTION_ALL, array(
                'class' => 'other_option_selected'));
        $group[] = & $this->createElement('radio', null, null, Translation :: get('OnlyForMe'), self :: RIGHT_OPTION_ME, array(
                'class' => 'other_option_selected'));
        $group[] = & $this->createElement('radio', null, null, Translation :: get('SelectSpecificEntities'), self :: RIGHT_OPTION_SELECT, array(
                'class' => 'entity_option_selected'));

        $this->addElement('html', '<div style="margin-left:25px; display:none;" class="specific_rights_selector_box">');
        $this->addGroup($group, self :: PROPERTY_RIGHT_OPTION, '', '<br />');

        // Add the advanced element finder
        $types = new AdvancedElementFinderElementTypes();

        foreach ($this->entities as $entity)
        {
            $types->add_element_type($entity->get_element_finder_type());
        }

        $this->addElement('html', '<div style="margin-left:25px; display:none;" class="entity_selector_box">');
        $this->addElement('advanced_element_finder', self :: PROPERTY_TARGETS, null, $types);

        $this->addElement('html', '</div></div></div>');

        $this->addElement('html', ResourceManager :: get_instance()->get_resource_html(Path :: get_application_web_path() . 'weblcms/resources/javascript/rights_form.js'));
    }

    // Old gradebook code
    // if ($values['evaluation'] == true)
    // {
    // $gradebook_internal_item_form = new GradebookInternalItemForm();
    // $category = 'C' . $course . '_T' . $tool;
    // $calculated = false;
    // if
    // ($gradebook_internal_item_form->is_application_result_calculated($tool))
    // $calculated = true;
    // $gradebook_internal_item_form->create_internal_item($pub->get_id(),
    // $calculated, $category);
    // }

    /**
     * Handles the submit of the form for both create and edit
     *
     * @return boolean
     */
    function handle_form_submit()
    {
        $publications = $this->publications;
        $succes = true;
        $wdm = WeblcmsDataManager :: get_instance();

        foreach ($publications as $publication)
        {
            $old_category = $publication->get_category_id();
            $this->set_publication_values($publication);

            switch ($this->form_type)
            {
                case self :: TYPE_CREATE :
                    $succes &= $publication->create();
                    $this->set_publication_rights($publication);
                    break;
                case self :: TYPE_UPDATE :
                    $category_changed = ($publication->get_category_id() != $old_category);
                    if ($category_changed)
                    {
                        $old_order = $publication->get_display_order_index();

                        $publication->set_display_order_index($wdm->get_next_content_object_publication_display_order_index($publication->get_course_id(), $publication->get_tool(), $publication->get_category_id()));
                    }

                    $succes &= $publication->update();

                    if ($category_changed)
                    {
                        $wdm->fix_publication_display_order($publication->get_course_id(), $publication->get_tool(), $old_category, $old_order);
                    }

                    $this->set_publication_rights($publication, $category_changed);
                    break;
            }

            // add collabate right for course admins
            if ($this->exportValue(self :: PROPERTY_COLLABORATE))
            {
                $this->collaborate_content_object_with_course_admins($publication->get_content_object());
            }

            // always mail publication last! we need the publication id and
            // rights created to get the targets...
            if ($this->form_type == self :: TYPE_CREATE && $this->exportValue(ContentObjectPublication :: PROPERTY_EMAIL_SENT))
            {
                $this->mail_publication($publication);
            }
        }
        return $succes;
    }

    /**
     * Sets the values for the content object publication
     *
     * @param $publication ContentObjectPublication
     */
    function set_publication_values(ContentObjectPublication $publication)
    {
        $values = $this->exportValues();

        $category = $values[ContentObjectPublication :: PROPERTY_CATEGORY_ID];
        if (! $category)
        {
            $category = 0;
        }

        if ($category > 0 && ! array_key_exists($category, $this->categories))
        {
            Display :: error_page(Translation :: get("PublicationInSelectedCategoryNotAllowed"));
            return false;
        }

        if ($values[self :: PROPERTY_FOREVER] != 0)
        {
            $from = $to = 0;
        }
        else
        {
            $from = Utilities :: time_from_datepicker($values[self :: PROPERTY_FROM_DATE]);
            $to = Utilities :: time_from_datepicker($values[self :: PROPERTY_TO_DATE]);
        }

        $publication->set_category_id($category);
        $publication->set_from_date($from);
        $publication->set_to_date($to);
        $publication->set_publication_date(time());
        $publication->set_modified_date(time());
        $publication->set_hidden($values[ContentObjectPublication :: PROPERTY_HIDDEN] ? 1 : 0);
        $publication->set_show_on_homepage($values[ContentObjectPublication :: PROPERTY_SHOW_ON_HOMEPAGE] ? 1 : 0);
    }

    /**
     * Sets the rights for the given content object publication
     *
     * @param $publication ContentObjectPublication
     * @param $category_changed boolean
     */
    function set_publication_rights(ContentObjectPublication $publication, $category_changed = false)
    {
        $values = $this->exportValues();

        $location = WeblcmsRights :: get_instance()->get_weblcms_location_by_identifier_from_courses_subtree(WeblcmsRights :: TYPE_PUBLICATION, $publication->get_id(), $publication->get_course_id());

        if ($category_changed)
        {
            $new_parent_id = WeblcmsRights :: get_instance()->get_weblcms_location_id_by_identifier_from_courses_subtree(WeblcmsRights :: TYPE_COURSE_CATEGORY, $publication->get_category_id(), $publication->get_course_id());
            $location->move($new_parent_id);
        }

        if (! $location->clear_right(WeblcmsRights :: VIEW_RIGHT))
        {
            continue;
        }

        if ($values[self :: PROPERTY_INHERIT] == self :: INHERIT_TRUE)
        {
            if (! $location->inherits())
            {
                $location->inherit();
                if (! $location->update())
                {
                    return false;
                }
            }
        }
        else
        {
            if ($location->inherits())
            {
                $location->disinherit();
                if (! $location->update())
                {
                    return false;
                }
            }

            $option = $values[self :: PROPERTY_RIGHT_OPTION];
            $location_id = $location->get_id();

            $weblcms_rights = WeblcmsRights :: get_instance();

            switch ($option)
            {
                case self :: RIGHT_OPTION_ALL :
                    if (! $weblcms_rights->invert_location_entity_right(WeblcmsRights :: VIEW_RIGHT, 0, 0, $location_id))
                    {
                        return false;
                    }
                    break;
                case self :: RIGHT_OPTION_ME :
                    if (! $weblcms_rights->invert_location_entity_right(WeblcmsRights :: VIEW_RIGHT, Session :: get_user_id(), CourseUserEntity :: ENTITY_TYPE, $location_id))
                    {
                        return false;
                    }
                    break;
                case self :: RIGHT_OPTION_SELECT :
                    foreach ($values[self :: PROPERTY_TARGETS] as $entity_type => $target_ids)
                    {
                        foreach ($target_ids as $target_id)
                        {
                            if (! $weblcms_rights->invert_location_entity_right(WeblcmsRights :: VIEW_RIGHT, $target_id, $entity_type, $location_id))
                            {
                                return false;
                            }
                        }
                    }
            }
        }

    }

    /**
     * Sets collaboration right for course admins for the given publication
     *
     * @param $publication ContentObjectPublication
     */
    function collaborate_content_object_with_course_admins(ContentObject $content_object)
    {
        $succes = false;

        if ($content_object && ($content_object->get_owner_id() == Session :: get_user_id()))
        {
            $succes = true;

            // prepare data
            $admin_users = $this->course->get_course_admin_users();
            $admin_groups = $this->course->get_course_admin_groups();

            // loop users
            foreach ($admin_users as $admin_user)
            {
                // exclude myself
                if ($admin_user->get_id() != Session :: get_user_id())
                {
                    $succes &= $content_object->share_with_user($admin_user->get_id(), RepositoryRights :: COLLABORATE_RIGHT);
                }
            }

            // loop groups
            foreach ($admin_groups as $admin_group)
            {
                $succes &= $content_object->share_with_group($admin_group->get_id(), RepositoryRights :: COLLABORATE_RIGHT);
            }
        }

        return $succes;
    }

    /**
     * Mails the given publication to the target users
     *
     * @param $publication ContentObjectPublication
     */
    function mail_publication(ContentObjectPublication $publication)
    {
        set_time_limit(3600);

        // prepare mail
        $user = $publication->get_publication_publisher();
        $content_object = $publication->get_content_object();
        $tool = $publication->get_tool();
        $link = $this->get_course_viewer_link($publication);

        $body = Translation :: get('NewPublicationMailDescription') . ' ' . $this->course->get_title() . ' : <a href="' . $link . '" target="_blank">' . $content_object->get_title() . '</a><br />--<br />';
        $body .= $content_object->get_description();
        $body .= '--<br />';
        $body .= $user->get_fullname() . ' - ' . $this->course->get_visual_code() . ' - ' . $this->course->get_title() . ' - ' . Translation :: get('TypeName', null, 'application\\weblcms\\tool\\' . $tool);

        // get targets
        $target_email = array();

        // Add the publisher to the email address
        $target_email[] = $user->get_email();

        $target_users = WeblcmsDataManager :: get_publication_target_users($publication);
        foreach ($target_users as $target_user)
        {
            $target_email[] = $target_user->get_email();
        }

        // safety check: filter any dubbles
        $unique_email = array_unique($target_email);

        $site_name = PlatformSetting :: get('site_name', AdminManager :: get_application_namespace('admin'));

        $mail = Mail :: factory('[' . $site_name . '] ' . Translation :: get('NewPublicationMailSubject', array(
                'COURSE' => $this->course->get_title(), 'CONTENTOBJECT' => $content_object->get_title())), '', '', array(
                Mail :: NAME => $user->get_fullname(), Mail :: EMAIL => $user->get_email()));

        $doc = new DOMDocument();
        $doc->loadHTML($body);
        $elements = $doc->getElementsByTagname('resource');

        // replace image document resource tags with a html img tag with base64
        // data
        // remove all other resource tags
        foreach ($elements as $i => $element)
        {
            $type = $element->attributes->getNamedItem('type')->value;
            $id = $element->attributes->getNamedItem('source')->value;
            if ($type == 'document')
            {
                $obj = RepositoryDataManager :: get_instance()->retrieve_content_object($id);
                if ($obj->is_image())
                {
                    $fn = $obj->get_full_path();
                    $index = $mail->add_embedded_image($fn);
                    $elem = $doc->createElement('img');
                    $elem->setAttribute('src', 'cid:' . $index);
                    $elem->setAttribute('alt', $obj->get_filename());
                    $element->parentNode->replaceChild($elem, $element);
                }
                else
                    $element->parentNode->removeChild($element);
            }
            else
                $element->parentNode->removeChild($element);
        }
        $body = $doc->saveHTML();

        $mail->set_message($body);
        // send mail
        foreach ($unique_email as $mail_to)
        {
            $mail->set_to($mail_to);
            $mail->send();
        }

        $publication->set_email_sent(true);
        $publication->update();
    }

    /**
     * Gets the tool parameter from the url
     *
     * @return String
     */
    function get_tool()
    {
        return Request :: get(WeblcmsManager :: PARAM_TOOL);
    }

    /**
     * Gets the course parameter from the url
     *
     * @return int
     */
    function get_course_id()
    {
        return Request :: get(WeblcmsManager :: PARAM_COURSE);
    }

    private function get_course_viewer_link($publication)
    {
        $params[WeblcmsManager :: PARAM_ACTION] = WeblcmsManager :: ACTION_VIEW_COURSE;
        $params[WeblcmsManager :: PARAM_COURSE] = $this->course->get_id();
        $params[WeblcmsManager :: PARAM_TOOL] = $publication->get_tool();
        $params[Tool :: PARAM_ACTION] = Tool :: ACTION_VIEW;
        $params[Tool :: PARAM_PUBLICATION_ID] = $publication->get_id();

        return Path :: get(WEB_PATH) . Redirect :: get_link(WeblcmsManager :: APPLICATION_NAME, $params);
    }
}
?>
