<?php
namespace application\weblcms;

use HTML_Menu_ArrayRenderer;
use common\libraries\TreeMenuRenderer;
use HTML_Menu;
use common\libraries\Application;
use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\Request;
use common\libraries\Translation;
use application\weblcms\course\Course;
use application\weblcms\course\CourseDataManager;

/**
 * $Id: course_category_menu.class.php 216 2009-11-13 14:08:06Z kariboe $
 * 
 * @package application.lib.weblcms.course
 */
/**
 * This class provides a navigation menu to allow a user to browse through
 * categories of courses.
 * 
 * @author Bart Mollet
 */
class CourseCategoryMenu extends HTML_Menu
{
    const TREE_NAME = __CLASS__;
    
    /**
     * The string passed to sprintf() to format category URLs
     */
    private $urlFmt;
    /**
     * The array renderer used to determine the breadcrumbs.
     */
    private $array_renderer;

    /**
     * Creates a new category navigation menu.
     * 
     * @param $owner int
     *            The ID of the owner of the categories to provide in
     *            this menu.
     * @param $current_category int
     *            The ID of the current category in the menu.
     * @param $url_format string
     *            The format to use for the URL of a category.
     *            Passed to sprintf(). Defaults to the string
     *            "?category=%s".
     * @param $extra_items array
     *            An array of extra tree items, added to the
     *            root.
     */
    function __construct($current_category, $url_format = '?category=%s', $extra_items = array())
    {
        $this->urlFmt = $url_format;
        $menu = $this->get_menu_items($extra_items);
        parent :: __construct($menu);
        $this->array_renderer = new HTML_Menu_ArrayRenderer();
        $this->forceCurrentUrl($this->get_category_url($current_category));
    }

    /**
     * Returns the menu items.
     * 
     * @param $extra_items array
     *            An array of extra tree items, added to the
     *            root.
     * @return array An array with all menu items. The structure of this array
     *         is the structure needed by PEAR::HTML_Menu, on which this
     *         class is based.
     */
    private function get_menu_items($extra_items)
    {
        $wdm = WeblcmsDataManager :: get_instance();
        $usercategories = $wdm->retrieve_course_categories();
        $categories = array();
        while ($category = $usercategories->next_result())
        {
            $categories[$category->get_parent()][] = $category;
        }
        $menu = $this->get_sub_menu_items($categories, 0);
        if (count($extra_items))
        {
            $menu = array_merge($menu, $extra_items);
        }
        
        $home = array();
        $home['title'] = Translation :: get('AllCourses');
        $home['url'] = $this->get_home_url();
        $home['class'] = 'home';
        $home_item[] = $home;
        $menu = array_merge($home_item, $menu);
        return $menu;
    }

    /**
     * Returns the items of the sub menu.
     * 
     * @param $categories array
     *            The categories to include in this menu.
     * @param $parent int
     *            The parent category ID.
     * @return array An array with all menu items. The structure of this array
     *         is the structure needed by PEAR::HTML_Menu, on which this
     *         class is based.
     */
    private function get_sub_menu_items($categories, $parent)
    {
        $sub_tree = array();
        foreach ($categories[$parent] as $index => $category)
        {
            $menu_item = array();
            
            // $cdm = CourseDataManager :: get_instance();
            // $count = $cdm->count_courses(new EqualityCondition(Course ::
            // PROPERTY_CATEGORY_ID, $category->get_id()));
            
            $menu_item['title'] = $category->get_name();
            if (Request :: get(Application :: PARAM_ACTION) == WeblcmsManager :: ACTION_COURSE_CATEGORY_MANAGER)
            {
                $menu_item['url'] = $this->get_category_url($category->get_id());
            }
            else
            {
                $menu_item['url'] = $this->get_category_url($category->get_id());
            }
            $sub_menu_items = $this->get_sub_menu_items($categories, $category->get_id());
            if (count($sub_menu_items) > 0)
            {
                $menu_item['sub'] = $sub_menu_items;
            }
            $menu_item['class'] = 'type_category';
            $menu_item['node_id'] = $category->get_id();
            $sub_tree[$category->get_id()] = $menu_item;
        }
        return $sub_tree;
    }

    /**
     * Gets the URL of a given category
     * 
     * @param $category int
     *            The id of the category
     * @return string The requested URL
     */
    private function get_category_url($category)
    {
        // TODO: Put another class in charge of the htmlentities() invocation
        return htmlentities(sprintf($this->urlFmt, $category));
    }

    private function get_home_url($category)
    {
        // TODO: Put another class in charge of the htmlentities() invocation
        return htmlentities(str_replace('&category_id=%s', '', $this->urlFmt));
    }

    /**
     * Get the breadcrumbs which lead to the current category.
     * 
     * @return array The breadcrumbs.
     */
    function get_breadcrumbs()
    {
        $this->render($this->array_renderer, 'urhere');
        $breadcrumbs = $this->array_renderer->toArray();
        foreach ($breadcrumbs as $crumb)
        {
            $crumb['name'] = $crumb['title'];
            unset($crumb['title']);
        }
        return $breadcrumbs;
    }

    /**
     * Renders the menu as a tree
     * 
     * @return string The HTML formatted tree
     */
    function render_as_tree()
    {
        $renderer = new TreeMenuRenderer($this->get_tree_name());
        $this->render($renderer, 'sitemap');
        return $renderer->toHTML();
    }

    static function get_tree_name()
    {
        return Utilities :: get_classname_from_namespace(self :: TREE_NAME, true);
    }
}