<?php
namespace application\weblcms;

use common\libraries\Theme;
use common\libraries\OrCondition;
use common\libraries\InCondition;
use common\libraries\EqualityCondition;
use common\libraries\Translation;
use common\libraries\Utilities;

use application\weblcms\course\Course;

/**
 * Course list renderer to render the course list (used in courses home, courses
 * sorter, courses block...)
 * 
 * @author Sven Vanpoucke
 */
class CourseListRenderer
{
    /**
     * The parent on which the course list renderer is running
     */
    private $parent;
    
    /**
     * Show the what's new icons or not
     * 
     * @var boolean
     */
    private $new_publication_icons;
    
    /**
     * Link target.
     *
     * @var string
     */
    private $target = '';

    function __construct($parent, $target = '')
    {
        $this->parent = $parent;
        $this->new_publication_icons = false;
        $this->target = $target;
    }

    function get_parent()
    {
        return $this->parent;
    }

    function set_parent($parent)
    {
        $this->parent = $parent;
    }

    function get_user()
    {
        return $this->get_parent()->get_user();
    }

    function show_new_publication_icons()
    {
        $this->new_publication_icons = true;
    }

    function get_new_publication_icons()
    {
        return $this->new_publication_icons;
    }

    function get_target()
    {
        return $target;
    }

    function set_target($value)
    {
        $this->target = $target;
    }

    /**
     * Renders the course list
     */
    function render()
    {
        echo $this->as_html();
    }

    /**
     * Returns the course list as html
     */
    function as_html()
    {
        return $this->display_courses();
    }

    /**
     * Retrieves the courses for the user
     */
    protected function retrieve_courses()
    {
        return WeblcmsDataManager :: get_instance()->retrieve_user_courses($this->get_retrieve_courses_condition());
    }

    /**
     * Returns the conditions needed to retrieve the courses
     */
    protected function get_retrieve_courses_condition()
    {
        return WeblcmsDataManager :: get_user_courses_condition($this->get_parent()->get_user());
    }

    /**
     * Displays the courses
     */
    protected function display_courses()
    {
        $html = array();
        $courses = $this->retrieve_courses();
        $target = $this->target ? ' target="' . $this->target . '" ' : '';
        
        if ($courses->size() > 0)
        {
            $html[] = '<ul style="padding: 0px; margin: 0px 0px 0px 15px;">';
            
            $course_settings_controller = CourseSettingsController :: get_instance();
            
            while ($course = $courses->next_result())
            {
                $course_access = $course_settings_controller->get_course_setting($course->get_id(), CourseSettingsConnector :: COURSE_ACCESS);
                
                $course_visible = $course_settings_controller->get_course_setting($course->get_id(), CourseSettingsConnector :: VISIBILITY);
                
                if (($course_access == CourseSettingsConnector :: COURSE_ACCESS_CLOSED || ! $course_visible) && ! $course->is_course_admin($this->get_user()))
                {
                    continue;
                }
                
                $html[] = '<li><a href="' . htmlspecialchars($this->get_course_url($course)) . '"' . $target . '>' . htmlspecialchars($course->get_title()) . '</a>';
                
                if ($this->get_new_publication_icons())
                {
                    $html[] = $this->display_new_publication_icons($course);
                }
                
                $html[] = '</li>';
            
            }
            $html[] = '</ul>';
        }
        else
        {
            $html[] = $this->get_no_courses_message_as_html();
        }

        return implode($html, "\n");
    }
    
    /**
     * Defines the display of the message when there are no courses to display.
     */
    protected function get_no_courses_message_as_html()
    {
        return '<div class="normal-message">' . Translation :: get('NoCourses') . '</div>';
    }

    /**
     * Displays the what's new icons
     * 
     * @param $course Course           
     */
    protected function display_new_publication_icons(Course $course)
    {
        $tools = WeblcmsDataManager :: get_instance()->retrieve_course_tools();
        $html = array();
        $target = $this->target ? ' target="' . $this->target . '" ' : '';
        
        $course_settings_controller = CourseSettingsController :: get_instance();
        
        while ($tool = $tools->next_result())
        {
            $active = $course_settings_controller->get_course_setting($course->get_id(), CourseSetting :: COURSE_SETTING_TOOL_ACTIVE, $tool->get_id());
            $visible = $course_settings_controller->get_course_setting($course->get_id(), CourseSetting :: COURSE_SETTING_TOOL_VISIBLE, $tool->get_id());
            
            if ($active && $visible && WeblcmsDataManager :: tool_has_new_publications($tool->get_name(), $this->get_user(), $course))
            {
                $html[] = '<a href="' . htmlspecialchars($this->get_tool_url($tool->get_name(), $course)) . '"' . $target . '><img src="' . htmlspecialchars(Theme :: get_image_path(Tool :: get_tool_type_namespace($tool->get_name()))) . 'logo/' . Theme :: ICON_MEDIUM . '_new.png" alt="' . htmlspecialchars(Translation :: get('New', null, Utilities :: COMMON_LIBRARIES)) . '"/></a>';
            }
        }
        return implode($html, "\n");
    }

    /**
     * Gets the url from the given course
     * 
     * @param $course Course           
     */
    function get_course_url(Course $course)
    {
        $parameters = array();
        $parameters[WeblcmsManager :: PARAM_APPLICATION] = WeblcmsManager :: APPLICATION_NAME;
        $parameters[WeblcmsManager :: PARAM_ACTION] = WeblcmsManager :: ACTION_VIEW_COURSE;
        $parameters[WeblcmsManager :: PARAM_COURSE] = $course->get_id();
        return $this->get_parent()->get_link($parameters);
    }

    /**
     * Gets the url from the given tool in the given course
     * 
     * @param $tool String           
     * @param $course Course           
     */
    function get_tool_url($tool, Course $course)
    {
        $parameters = array();
        $parameters[WeblcmsManager :: PARAM_APPLICATION] = WeblcmsManager :: APPLICATION_NAME;
        $parameters[WeblcmsManager :: PARAM_ACTION] = WeblcmsManager :: ACTION_VIEW_COURSE;
        $parameters[WeblcmsManager :: PARAM_COURSE] = $course->get_id();
        $parameters[WeblcmsManager :: PARAM_TOOL] = $tool;
        return $this->get_parent()->get_link($parameters);
    }
}

?>