<?php

namespace application\weblcms;

use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\InCondition;
use common\libraries\NestedTreeNode;
use common\libraries\Translation;

use user\UserDataManager;

/**
 * $Id: course_group.class.php 216 2009-11-13 14:08:06Z kariboe $
 *
 * @package application.lib.weblcms.course_group
 */

/**
 * This class represents a course_group of users in a course in the weblcms.
 *
 * To access the values of the properties, this class and its subclasses should
 * provide accessor methods. The names of the properties should be defined as
 * class constants, for standardization purposes. It is recommended that the
 * names of these constants start with the string "PROPERTY_".
 */
class CourseGroup extends NestedTreeNode
{
    const CLASS_NAME = __CLASS__;

    const PROPERTY_COURSE_CODE = 'course_id';
    const PROPERTY_NAME = 'name';
    const PROPERTY_MAX_NUMBER_OF_MEMBERS = 'max_number_of_members';
    const PROPERTY_DESCRIPTION = 'description';
    const PROPERTY_SELF_UNREG = 'self_unreg_allowed';
    const PROPERTY_SELF_REG = 'self_reg_allowed';
    const PROPERTY_DOCUMENT_CATEGORY_ID = 'document_category_id';
    const PROPERTY_FORUM_CATEGORY_ID = 'forum_category_id';
    const PROPERTY_GROUP_ID = "group_id";
    // random registration = randomly selects from the subscribed users of the
    // course and registers for the course_group
    const PROPERTY_RANDOM_REG = 'random_registration_allowed';

    /**
     * Stores the members of this course group in a cache variable for multiple
     * use
     *
     * @var <type>
     */
    private $members_cache;

    /**
     * Get the default properties of all course_groups.
     *
     * @return array The property names.
     */
    static function get_default_property_names($extended_property_names = array())
    {
        return parent :: get_default_property_names(array(self :: PROPERTY_ID, self :: PROPERTY_COURSE_CODE,
                self :: PROPERTY_NAME, self :: PROPERTY_DESCRIPTION, self :: PROPERTY_MAX_NUMBER_OF_MEMBERS,
                self :: PROPERTY_SELF_REG, self :: PROPERTY_SELF_UNREG));
    }

    /**
     * Gets the id of this course_group
     *
     * @return int
     */
    function get_id()
    {
        return $this->get_default_property(self :: PROPERTY_ID);
    }

    function set_id($id)
    {
        return $this->set_default_property(self :: PROPERTY_ID, $id);
    }

    /**
     * Gets the group_id of this course_group
     *
     * @return int
     */
    function get_group_id()
    {
        // return $this->get_default_property(self :: PROPERTY_GROUP_ID);

        return $this->get_optional_property(self :: PROPERTY_GROUP_ID);
    }

    function set_group_id($id)
    {
        return $this->set_default_property(self :: PROPERTY_GROUP_ID, $id);
    }

    /**
     * Gets the course code of the course in which this course_group was created
     *
     * @return string
     */
    function get_course_code()
    {
        return $this->get_default_property(self :: PROPERTY_COURSE_CODE);
    }

    function set_course_code($code)
    {
        $this->set_default_property(self :: PROPERTY_COURSE_CODE, $code);
    }

    /**
     * Gets the name of this course_group
     *
     * @return string
     */
    function get_name()
    {
        return $this->get_default_property(self :: PROPERTY_NAME);
    }

    /**
     * Sets the name of this course_group
     *
     * @param $name string
     */
    function set_name($name)
    {
        return $this->set_default_property(self :: PROPERTY_NAME, $name);
    }

    /**
     * Gets the description of this course_group
     *
     * @return string
     */
    function get_description()
    {
        return $this->get_default_property(self :: PROPERTY_DESCRIPTION);
    }

    /**
     * Sets the description of this course_group
     *
     * @param $description string
     */
    function set_description($description)
    {
        return $this->set_default_property(self :: PROPERTY_DESCRIPTION, $description);
    }

    /**
     * Gets the maximum number of members than can be subscribed to this
     * course_group
     *
     * @return int null null, no limit is set to the number of members
     */
    function get_max_number_of_members()
    {
        return $this->get_default_property(self :: PROPERTY_MAX_NUMBER_OF_MEMBERS);
    }

    /**
     * Sets the document_category_id for the course_group
     * If null no document category exist
     */
    function set_document_category_id($document_category_id)
    {
        if (is_null($document_category_id))
        {
            $document_category_id = null;
        }
        return $this->set_default_property(self :: PROPERTY_DOCUMENT_CATEGORY_ID, $document_category_id);
    }

    /**
     * Gets the document_category_id for the course_group
     *
     * @return int null null no document category exist
     */
    function get_document_category_id()
    {
        return $this->get_optional_property(self :: PROPERTY_DOCUMENT_CATEGORY_ID);
    }

    /**
     * Sets the forum_category_id for the course_group
     * If null no forum category exist
     */
    function set_forum_category_id($forum_category_id)
    {
        if (is_null($forum_category_id))
        {
            $forum_category_id = null;
        }
        return $this->set_default_property(self :: PROPERTY_FORUM_CATEGORY_ID, $forum_category_id);
    }

    /**
     * Gets the forum_category_id for the course_group
     *
     * @return int null null no forum categories exist
     */
    function get_forum_category_id()
    {
        return $this->get_optional_property(self :: PROPERTY_FORUM_CATEGORY_ID);
    }

    /**
     * Sets the maximum number of members of this course_group
     * If the new value is smaller than the number of members currently
     * subscribed, no changes are made.
     *
     * @param $max_number_of_members int|null
     *            If null, no limit is set to the
     *            number of members.
     */
    function set_max_number_of_members($max_number_of_members)
    {
        // Todo: Check current number of members.
        return $this->set_default_property(self :: PROPERTY_MAX_NUMBER_OF_MEMBERS, $max_number_of_members);
    }

    /**
     * Determines if self registration is allowed
     *
     * @return boolean
     */
    function is_self_registration_allowed()
    {
        return $this->get_default_property(self :: PROPERTY_SELF_REG);
    }

    /**
     * Sets if self registration is allowed
     *
     * @param $self_reg boolean
     */
    function set_self_registration_allowed($self_reg)
    {
        if (is_null($self_reg))
        {
            $self_reg = 0;
        }
        return $this->set_default_property(self :: PROPERTY_SELF_REG, $self_reg);
    }

    /**
     * Determines if self unregistration is allowed
     *
     * @return boolean
     */
    function is_self_unregistration_allowed()
    {
        return $this->get_default_property(self :: PROPERTY_SELF_UNREG);
    }

    /**
     * Determines if the course group users were randomly subscribed
     *
     * @return boolean
     */
    function is_random_registration_done()
    {
        return $this->get_default_property(self :: PROPERTY_RANDOM_REG);
    }

    /**
     * Sets if self unregistration is allowed
     *
     * @param $self_unreg boolean
     */
    function set_self_unregistration_allowed($self_unreg)
    {
        if (is_null($self_unreg))
        {
            $self_unreg = 0;
        }
        return $this->set_default_property(self :: PROPERTY_SELF_UNREG, $self_unreg);
    }

    /**
     * Retrieves the users subscribed to this course_group and/or it's children
     *
     * @param $include_subgroups boolean
     *            - Include the children of the subgroups
     * @param $recursive_subgroups boolean
     *            - Include the direct subgroups or include all the subgroups
     * @param $include_users boolean
     *            - Includes the users as real user objects
     * @return DatabaseUserResultSet
     */
    function get_members($include_subgroups = false, $recursive_subgroups = false, $include_users = false)
    {
        if (! isset($this->members_cache[(int) $this->get_id()][(int) $include_subgroups][(int) $recursive_subgroups][(int) $include_users]))
        {
            $condition = $this->get_members_condition($include_subgroups, $recursive_subgroups);

            $wdm = WeblcmsDataManager :: get_instance();
            $udm = UserDataManager :: get_instance();
            $course_group_user_relations = $wdm->retrieve_course_group_user_relations($condition);

            $users = array();

            while ($relation = $course_group_user_relations->next_result())
            {
                if ($include_users)
                {
                    $users[$relation->get_user()] = $udm->retrieve_user($relation->get_user());
                }
                else
                {
                    $users[$relation->get_user()] = $relation->get_user();
                }
            }
            if (! empty($users))
            {
                $this->members_cache[(int) $this->get_id()][(int) $include_subgroups][(int) $recursive_subgroups][(int) $include_users] = $users;
            }
        }

        return $this->members_cache[(int) $this->get_id()][(int) $include_subgroups][(int) $recursive_subgroups][(int) $include_users];
    }

    /**
     * Returns the condition to retrieve the members of the subgroups
     *
     * @param $include_subgroups boolean
     *            - Include the children of the subgroups
     * @param $recursive_subgroups boolean
     *            - Include the direct subgroups or include all the subgroups
     * @return InCondition
     */
    private function get_members_condition($include_subgroups = false, $recursive_subgroups = false)
    {
        $groups = array();
        $groups[] = $this->get_id();

        if ($include_subgroups)
        {
            $subgroups = $this->get_children($recursive_subgroups);

            while ($subgroup = $subgroups->next_result())
            {
                $groups[] = $subgroup->get_id();
            }
        }

        return new InCondition(CourseGroupUserRelation :: PROPERTY_COURSE_GROUP, $groups);
    }

    /**
     * Checks if a user is a member of this group
     *
     * @param $user User
     * @return boolean
     */
    function is_member($user)
    {
        $wdm = WeblcmsDataManager :: get_instance();
        return $wdm->is_course_group_member($this, $user);
    }

    function count_members()
    {
        $members = $this->get_members();
        return count($members);
    }

    /**
     * Subscribes users to this course_group
     *
     * @param
     *            array|User A single user or an array of users
     */
    function subscribe_users($users)
    {
        $wdm = WeblcmsDataManager :: get_instance();
        return $wdm->subscribe_users_to_course_groups($users, $this);
    }

    /**
     * Unsubscribes users from this course_group
     *
     * @param
     *            array|User A single user or an array of users
     */
    function unsubscribe_users($users)
    {
        $wdm = WeblcmsDataManager :: get_instance();
        return $wdm->unsubscribe_users_from_course_groups($users, $this);
    }

    static function get_table_name()
    {
        return Utilities :: get_classname_from_namespace(self :: CLASS_NAME, true);
    }

    function get_data_manager()
    {
        return WeblcmsDataManager :: get_instance();
    }

    function create()
    {
        if (! $this->get_parent_id())
        {
            $root_group = WeblcmsDataManager :: get_instance()->retrieve_course_group_root($this->get_course_code());
            if ($root_group)
            {
                $this->set_parent_id($root_group->get_id());
            }
        }

        return parent :: create();
    }

    function update()
    {
        if ($this->check_before_save())
        {
            return parent :: update();
        }

        return false;
    }

    function check_before_save()
    {
        $children = WeblcmsDataManager :: get_instance()->count_course_group_users($this);

        if ($children > $this->get_max_number_of_members())
        {
            $this->add_error(Translation :: get('MaximumMembersToSmall'));
            return false;
        }

        return true;
    }

    function get_parents($include_self = true)
    {
        return parent :: get_parents($include_self, $this->get_nested_tree_node_condition());
    }

    function get_nested_tree_node_condition()
    {
        return new EqualityCondition(CourseGroup :: PROPERTY_COURSE_CODE, $this->get_course_code());
    }
}

?>