<?php
namespace application\weblcms;

use common\libraries\Filesystem;

use repository\ContentObjectRendition;
use repository\ContentObjectRenditionImplementation;

use common\libraries\NotAllowedException;
use common\libraries;
use reporting\ReportingManager;
use common\libraries\SubselectCondition;
use Exception;
use repository\RepositoryDataManager;
use common\libraries\Toolbar;
use common\libraries\ToolbarItem;
use common\libraries\FormValidator;
use common\libraries\Theme;
use common\libraries\Session;
use common\libraries\Utilities;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Request;
use common\libraries\SubManager;
use common\libraries\Translation;
use repository\ContentObject;
use repository\content_object\introduction\Introduction;
use rights\RightsUtilities;
use common\libraries\Path;
use rights\TemplateEntity;
use common\extensions\category_manager\CategoryVisibilitySupported;

/**
 * This is the base class for all tools used in applications.
 *
 * @author Tim De Pauw
 * @author Hans De Bisschop - Erasmus Hogeschool Brussel - Refactoring
 * @author Sven Vanpoucke - Hogeschool Gent - Refactoring, Cleanup
 */
abstract class Tool extends SubManager
{
    /**
     * **************************************************************************************************************
     * URL Parameters *
     * **************************************************************************************************************
     */

    const PARAM_ACTION = 'tool_action';
    const PARAM_PUBLICATION_ID = 'publication';
    const PARAM_COMPLEX_ID = 'cid';
    const PARAM_MOVE = 'move';
    const PARAM_VISIBILITY = 'visible';
    const PARAM_OBJECT_ID = 'object_id';
    const PARAM_BROWSER_TYPE = 'browser';
    const PARAM_TEMPLATE_NAME = 'template_name';
    const PARAM_PUBLISH_MODE = 'publish_mode';
    const PARAM_MOVE_DIRECTION = 'move_direction';
    const PARAM_BROWSE_PUBLICATION_TYPE = 'pub_type';
    const PARAM_VIEW_AS_ID = 'view_as_id';
    const PARAM_VIEW_AS_COURSE_ID = 'view_as_course_id';

    /**
     * **************************************************************************************************************
     * Actions *
     * **************************************************************************************************************
     */

    const ACTION_BROWSE = 'browser';
    const ACTION_VIEW = 'viewer';
    const ACTION_PUBLISH = 'publisher';
    const ACTION_DELETE = 'deleter';
    const ACTION_UPDATE = 'updater';
    const ACTION_TOGGLE_VISIBILITY = 'toggle_visibility';
    const ACTION_MOVE = 'mover';
    const ACTION_MOVE_TO_CATEGORY = 'category_mover';
    const ACTION_PUBLISH_INTRODUCTION = 'introduction_publisher';
    const ACTION_MANAGE_CATEGORIES = 'category_manager';
    const ACTION_VIEW_REPORTING_TEMPLATE = 'reporting_viewer';
    const ACTION_BUILD_COMPLEX_CONTENT_OBJECT = 'complex_builder';
    const ACTION_DISPLAY_COMPLEX_CONTENT_OBJECT = 'complex_display';
    const ACTION_SHOW_PUBLICATION = 'show_publication';
    const ACTION_HIDE_PUBLICATION = 'hide_publication';
    const ACTION_EVALUATE_TOOL_PUBLICATION = 'evaluate';
    const ACTION_EDIT_RIGHTS = 'rights_editor';
    const ACTION_UPDATE_CONTENT_OBJECT = 'content_object_updater';
    const ACTION_UPDATE_PUBLICATION = 'publication_updater';
    const ACTION_SHARE_CONTENT_OBJECTS = 'share_content_objects';
    const ACTION_VIEW_ATTACHMENT = 'attachment_viewer';

    const DEFAULT_ACTION = self :: ACTION_BROWSE;

    /**
     * **************************************************************************************************************
     * Publication Types *
     * **************************************************************************************************************
     */

    const PUBLICATION_TYPE_ALL = 1;
    const PUBLICATION_TYPE_FOR_ME = 2;
    const PUBLICATION_TYPE_FROM_ME = 3;

    const PUBLISH_TYPE_FORM = 1;
    const PUBLISH_TYPE_AUTO = 2;
    const PUBLISH_TYPE_BOTH = 3;

    const PUBLISH_MODE_QUICK = 1;
    const PUBLISH_MODE_FULL = 2;

    /**
     * **************************************************************************************************************
     * Move Directions *
     * **************************************************************************************************************
     */

    const PARAM_MOVE_DIRECTION_UP = - 1;
    const PARAM_MOVE_DIRECTION_DOWN = 1;

    /**
     * **************************************************************************************************************
     * Construct Functionality *
     * **************************************************************************************************************
     */

    /**
     * Constructor.
     *
     * @param $parent Application
     *            - The component in which this tool runs
     */
    function __construct($parent)
    {
        parent :: __construct($parent);

        if($this->get_course_id() && !$this->get_parent()->is_tool_accessible())
        {
            throw new NotAllowedException();
        }
        $this->set_parameter(WeblcmsManager :: PARAM_CATEGORY, Request :: get(WeblcmsManager :: PARAM_CATEGORY));
        $this->set_parameter(Tool :: PARAM_BROWSER_TYPE, $this->get_browser_type());
    }

    /**
     * Launches the tool with a given type
     *
     * @param $type string
     * @param $application Application
     */
    static function factory_and_launch($type, $application)
    {
        $file = dirname(__FILE__) . '/../../../tool/' . $type . '/php/lib/' . $type . '_tool.class.php';
        if (! file_exists($file))
        {
            throw new Exception(Translation :: get('ToolTypeDoesNotExist', array('type' => $type)));
        }

        $class = self :: type_to_class($type);
        $class :: launch($class, $application);
    }

    /**
     * **************************************************************************************************************
     * Main Functionality *
     * **************************************************************************************************************
     */

    function get_browser_type()
    {
        $browser_type = Request :: get(Tool :: PARAM_BROWSER_TYPE);

        if ($browser_type && in_array($browser_type, $this->get_available_browser_types()))
        {
            return $browser_type;
        }
        else
        {
            $available_browser_types = $this->get_available_browser_types();
            return $available_browser_types[0];
        }
    }

    function get_available_browser_types()
    {
        $browser_types = array();
        $browser_types[] = ContentObjectPublicationListRenderer :: TYPE_LIST;
        $browser_types[] = ContentObjectPublicationListRenderer :: TYPE_TABLE;
        return $browser_types;
    }

    /**
     * Returns the name of the tool
     *
     * @return String
     */
    function get_tool_id()
    {
        return $this->get_tool_registration()->get_name();
    }

    /**
     * Returns the registration object of the tool
     *
     * @return CourseTool
     */
    function get_tool_registration()
    {
        return $this->get_parent()->get_tool_registration();
    }

    function display_header($visible_tools = null, $show_introduction_text = false)
    {
        if (! $visible_tools)
        {
            $visible_tools = $this->get_visible_tools();
        }

        parent :: display_header();

        $this->display_course_menus($visible_tools, $show_introduction_text);
    }

    function display_footer()
    {
        echo '</div>';
        parent :: display_footer();
    }

    /**
     * Returns the visible tools for this course
     *
     * @return CourseTool[]
     */
    function get_visible_tools()
    {
        $tools = array();

        $course_tools = WeblcmsDataManager :: get_instance()->retrieve_course_tools();

        $edit_right = $this->is_allowed(WeblcmsRights :: EDIT_RIGHT);

        $course_settings_controller = CourseSettingsController :: get_instance();

        while ($tool = $course_tools->next_result())
        {
            $tool_active = $course_settings_controller->get_course_setting($this->get_course_id(), CourseSetting :: COURSE_SETTING_TOOL_ACTIVE, $tool->get_id());

            $tool_visible = $course_settings_controller->get_course_setting($this->get_course_id(), CourseSetting :: COURSE_SETTING_TOOL_VISIBLE, $tool->get_id());

            if ($tool_active && ($edit_right || $tool_visible))
            {
                $tools[] = $tool;
            }
        }

        return $tools;
    }

    function display_course_menus($tools, $show_introduction_text = false)
    {
        $shortcuts_visible = false;

        $course_settings_controller = CourseSettingsController :: get_instance();

        $menu_layout = $course_settings_controller->get_course_setting($this->get_course_id(), CourseSettingsConnector :: MENU_LAYOUT);

        if ($menu_layout != CourseSettingsConnector :: MENU_LAYOUT_OFF && count($tools) > 0)
        {
            $renderer = ToolListRenderer :: factory(ToolListRenderer :: TYPE_MENU, $this, $tools);

            echo $renderer->display();
            echo '<div id="tool_browser_' . ($renderer->display_menu_icons() && ! $renderer->display_menu_text() ? 'icon_' : '') . $renderer->get_menu_style() . '">';
        }
        else
        {
            echo '<div id="tool_browser">';
        }

        $tool_shortcut = $course_settings_controller->get_course_setting($this->get_course_id(), CourseSettingsConnector :: TOOL_SHORTCUT_MENU);

        $introduction_text_allowed = $course_settings_controller->get_course_setting($this->get_course_id(), CourseSettingsConnector :: ALLOW_INTRODUCTION_TEXT);

        if (($this->get_tool_id() == 'home' && $introduction_text_allowed && ! $this->get_introduction_text()) || ($tool_shortcut && count($tools) > 0))
        {
            echo '<div style="margin-bottom: 10px;">';
            $shortcuts_visible = true;
        }

        $toolbar = new Toolbar();

        // if ($this->get_course()->get_external_url() &&
        // $show_introduction_text)
        // {
        // $link_name = $this->get_course()->get_external_name();
        // if (! $link_name)
        // {
        // $link_name = $this->get_course()->get_external_url();
        // }
        //
        // $toolbar->add_item(new ToolbarItem($link_name, Theme ::
        // get_common_image_path() . 'action_home.png',
        // $this->get_course()->get_external_url()));
        // }

        if ($show_introduction_text)
        {
            $introduction_text = $this->get_introduction_text();
            if (! $introduction_text)
            {
                if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
                {
                    $toolbar->add_item(new ToolbarItem(Translation :: get('PublishIntroductionText'), Theme :: get_common_image_path() . 'action_introduce.png', $this->get_url(array(
                            Tool :: PARAM_ACTION => Tool :: ACTION_PUBLISH_INTRODUCTION))));
                }
            }
        }

        if ($toolbar->has_items())
        {
            echo '<div class="home_actions">';
            echo $toolbar->as_html();
            echo '</div>';
        }

        if ($tool_shortcut && count($tools) > 0)
        {
            $renderer = ToolListRenderer :: factory(ToolListRenderer :: TYPE_SHORTCUT, $this, $tools);
            echo '<div style="float:right;">';
            $renderer->display();
            echo '</div>';
        }

        if ($shortcuts_visible)
        {
            echo '<div class="clear"></div>';
            echo '</div>';
        }

        echo '<div class="clear"></div>';
    }

    function get_user_info($user_id)
    {
        return $this->get_parent()->get_user_info($user_id);
    }

    /**
     *
     * @see WebApplication :: get_course_id()
     */
    function get_course()
    {
        return $this->get_parent()->get_course();
    }

    /**
     *
     * @see WebApplication :: get_course_id()
     */
    function get_course_id()
    {
        return $this->get_parent()->get_course_id();
    }

    /**
     *
     * @see WebApplication :: get_course_groups()
     */
    function get_course_groups()
    {
        return $this->get_parent()->get_course_groups();
    }

    function get_course_group()
    {
        return $this->get_parent()->get_course_group();
    }

    /**
     * Check if the current user has a given right in this tool
     *
     * @param $right int
     * @param $publication ContentObjectPublication
     * @return boolean True if the current user has the right
     */
    function is_allowed($right, $publication = null)
    {
        $studentview = Session :: retrieve('studentview');
        if ($studentview == 1)
        {
            return false;
        }
        // add check for student view/login as
        $id = Session :: get_user_id();
        $va_id = Session :: get(Tool :: PARAM_VIEW_AS_ID);
        $course_id = Session :: get(Tool :: PARAM_VIEW_AS_COURSE_ID);
        // fake the id with the set "login as id" only if we're in the right
        // course
        if (isset($va_id) && isset($course_id))
        {
            if ($course_id == $this->get_course_id())
            {
                $id = $va_id;
            }
            else
            {
                // not the course we're viewing as another user, so check
                // regular stuff
                if ($this->get_parent()->is_teacher())
                {
                    return true;
                }
            }
        }
        else // try to get rid of duplication
        {
            // no valid view as information
            if ($this->get_parent()->is_teacher())
            {
                return true;
            }
        }
        if ($publication)
        {
            $category_id = $publication->get_category_id();
            if($category_id != 0)
            {
                $category = WeblcmsDataManager :: get_instance()->retrieve_content_object_publication_category($category_id);
                if (!$category->is_recursive_visible())
                {
                    return false;
                }
            }

            if ($publication->get_publisher_id() == $id) // Session ::
            // get_user_id())
            {
                return true; // the publisher has all the rights
            }

            if ($publication->is_hidden() && $right == WeblcmsRights :: VIEW_RIGHT)
            {
                return WeblcmsRights :: get_instance()->is_allowed_in_courses_subtree(WeblcmsRights :: EDIT_RIGHT,
                        $publication->get_id(), WeblcmsRights :: TYPE_PUBLICATION, $this->get_course_id(), $id);
            }
            return WeblcmsRights :: get_instance()->is_allowed_in_courses_subtree($right, $publication->get_id(),
                    WeblcmsRights :: TYPE_PUBLICATION, $this->get_course_id(), $id);

        }
        else
        {
            $category_id = Request :: get(WeblcmsManager :: PARAM_CATEGORY);

            if ($category_id)
            {
                $category = WeblcmsDataManager :: get_instance()->retrieve_content_object_publication_category($category_id);
                if ($category->is_recursive_visible())
                {
                    return WeblcmsRights :: get_instance()->is_allowed_in_courses_subtree($right, $category_id,
                            WeblcmsRights :: TYPE_COURSE_CATEGORY, $this->get_course_id(), $id);
                }
                else
                {
                    return false;
                }
            }

            if ($this->get_tool_id() == 'home')
            {
                return WeblcmsRights :: get_instance()->is_allowed_in_courses_subtree($right, 0,
                        RightsUtilities :: TYPE_ROOT, $this->get_course_id(), $id);
            }

            $tool_registration = $this->get_tool_registration();
            $course_settings_controller = CourseSettingsController :: get_instance();

            $module_visible = $course_settings_controller->get_course_setting($this->get_course_id(),
                    CourseSetting :: COURSE_SETTING_TOOL_VISIBLE, $tool_registration->get_id());

            if (!$module_visible)
            {
                return false;
            }
            return WeblcmsRights :: get_instance()->is_allowed_in_courses_subtree($right, $tool_registration->get_id(),
                    WeblcmsRights :: TYPE_COURSE_MODULE, $this->get_course_id(), $id);
        }
    }
    
    function get_user_id()
    {
        $va_id = Session :: get(Tool :: PARAM_VIEW_AS_ID);
        $course_id = Session :: get(Tool :: PARAM_VIEW_AS_COURSE_ID);
        // fake the id with the set "login as id" only if we're in the right
        // course
        if (isset($va_id) && isset($course_id))
        {
            if ($course_id == $this->get_course_id())
            {
                return $va_id;
            }
        }
        return parent :: get_user_id();
    }


    /**
     * Converts a tool name to the corresponding class name.
     *
     * @param $tool string
     *            The tool name.
     * @return string The class name.
     */
    static function type_to_class($tool)
    {
        return __NAMESPACE__ . '\tool\\' . $tool . '\\' . Utilities :: underscores_to_camelcase($tool) . 'Tool';
    }

    /**
     * Converts a tool class name to the corresponding tool name.
     *
     * @param $class string
     *            The class name.
     * @return string The tool name.
     */
    static function class_to_type($class)
    {
        return Utilities :: camelcase_to_underscores(str_replace('Tool', '', $class));
    }

    /**
     *
     * @see WeblcmsManager :: get_last_visit_date()
     */
    function get_last_visit_date()
    {
        return $this->get_parent()->get_last_visit_date();
    }

    /**
     *
     * @see Application :: get_category()
     */
    function get_category($id)
    {
        return $this->get_parent()->get_category($id);
    }

    function display_introduction_text($introduction_text)
    {
        $html = array();

        if ($introduction_text)
        {
            if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
            {
                $toolbar = new Toolbar();

                $toolbar->add_item(new ToolbarItem(Translation :: get('Edit', null, Utilities :: COMMON_LIBRARIES), Theme :: get_common_image_path() . 'action_edit.png', $this->get_url(array(
                        Tool :: PARAM_ACTION => Tool :: ACTION_UPDATE_CONTENT_OBJECT,
                        Tool :: PARAM_PUBLICATION_ID => $introduction_text->get_id())), ToolbarItem :: DISPLAY_ICON));
            }
            if ($this->is_allowed(WeblcmsRights :: DELETE_RIGHT))
            {
                if (! $toolbar)
                {
                    $toolbar = new Toolbar();
                }
                $toolbar->add_item(new ToolbarItem(Translation :: get('Delete', null, Utilities :: COMMON_LIBRARIES), Theme :: get_common_image_path() . 'action_delete.png', $this->get_url(array(
                        Tool :: PARAM_ACTION => Tool :: ACTION_DELETE,
                        Tool :: PARAM_PUBLICATION_ID => $introduction_text->get_id())), ToolbarItem :: DISPLAY_ICON, true));
            }

            $html[] = '<div class="announcements level_1" style="background-image: url(' . Theme :: get_common_image_path() . 'content_object/introduction.png);">';
            $html[] = '<div class="title" style="border-bottom: 1px dotted #D3D3D3; width:100%;">';
            $html[] = $introduction_text->get_content_object()->get_title();
            $html[] = '</div><div class="clear">&nbsp;</div>';
            $html[] = '<div class="description">';

            $content_object = $introduction_text->get_content_object();
            $rendition_implementation = ContentObjectRenditionImplementation :: factory($content_object, ContentObjectRendition:: FORMAT_HTML, ContentObjectRendition :: VIEW_DESCRIPTION, $this);
            $html[] = $rendition_implementation->render();

            $html[] = '</div>';
            if ($toolbar)
            {
                $html[] = $toolbar->as_html() . '<div class="clear"></div>';
            }
            $html[] = '</div>';
            $html[] = '<br />';
        }

        return implode("\n", $html);
    }

    private $introduction_cache;

    function get_introduction_text()
    {
        $course_id = $this->get_course_id();
        $tool_id = $this->get_tool_id();

        if (is_null($this->introduction_cache[$course_id][$tool_id]))
        {
            $this->introduction_cache[$course_id][$tool_id] = false;
            $conditions = array();
            $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_COURSE_ID, $course_id);
            $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_TOOL, $tool_id);

            $condition = new AndCondition($conditions);

            $publication = WeblcmsDataManager :: get_instance()->retrieve_introduction($condition);
            if ($publication)
            {
                $this->introduction_cache[$course_id][$tool_id] = $publication;
            }
            else
            {
                $this->introduction_cache[$course_id][$tool_id] = false;
            }
        }
        return $this->introduction_cache[$course_id][$tool_id];
    }

    static function get_allowed_types()
    {
        return array();
    }

    function get_access_details_toolbar_item($parent)
    {
        if (Request :: get(self :: PARAM_PUBLICATION_ID))
        {
            $url = $this->get_parent()->get_url(array(Tool :: PARAM_ACTION => Tool :: ACTION_VIEW_REPORTING_TEMPLATE,
                    Tool :: PARAM_PUBLICATION_ID => Request :: get(self :: PARAM_PUBLICATION_ID),
                    ReportingManager :: PARAM_TEMPLATE_NAME => 'publication_detail_reporting_template'));
            return new ToolbarItem(Translation :: get('AccessDetails'), Theme :: get_common_image_path() . 'action_reporting.png', $url);
        }
        else
        {
            return new ToolbarItem('');
        }
    }

    function get_complex_builder_url($pid)
    {
        return $this->get_url(array(Tool :: PARAM_ACTION => Tool :: ACTION_BUILD_COMPLEX_CONTENT_OBJECT,
                Tool :: PARAM_PUBLICATION_ID => $pid));
    }

    function get_complex_display_url($pid)
    {
        return $this->get_url(array(Tool :: PARAM_ACTION => Tool :: ACTION_DISPLAY_COMPLEX_CONTENT_OBJECT,
                Tool :: PARAM_PUBLICATION_ID => $pid));
    }

    static function get_pcattree_parents($pcattree)
    {
        $parent = WebLcmsDataManager :: get_instance()->retrieve_content_object_publication_category($pcattree);
        $parents[] = $parent;

        while ($parent && $parent->get_parent() != 0)
        {
            $parent = WebLcmsDataManager :: get_instance()->retrieve_content_object_publication_category($parent->get_parent());
            $parents[] = $parent;
        }
        $parents = array_reverse($parents);

        return $parents;
    }

    function convert_content_object_publication_to_calendar_event($publication)
    {
        return $publication;
    }

    function tool_has_new_publications($tool_name, $course)
    {
        return $this->get_parent()->tool_has_new_publications($tool_name, $course);
    }

    static function get_tool_type_namespace($type)
    {
        return 'application\\weblcms\\tool\\' . $type;
    }

    function get_entities()
    {
        $entities = array();
        $entities[CourseGroupEntity :: ENTITY_TYPE] = CourseGroupEntity :: get_instance($this->get_course_id());
        $entities[CourseUserEntity :: ENTITY_TYPE] = CourseUserEntity :: get_instance();
        $entities[CoursePlatformGroupEntity :: ENTITY_TYPE] = CoursePlatformGroupEntity :: get_instance();
        $entities[TemplateEntity :: ENTITY_TYPE] = TemplateEntity :: get_instance();

        return $entities;
    }

    /**
     * Retrieves the location in the given category, or the location in the
     * current category by publication id
     *
     * @param $category_id type
     * @return type
     */
    function get_location($category_id = null)
    {
        $locations = $this->get_locations($category_id);
        return $locations[0];
    }

    /**
     * Retrieves the locations in the given category, or the locations in the
     * current category by publication id
     *
     * @param $category_id type
     * @return type array
     */
    function get_locations($category_id = null)
    {
        $course_id = $this->get_course_id();

        if (! isset($category_id))
        {
            $category_id = Request :: get(WeblcmsManager :: PARAM_CATEGORY);
        }

        $publications = Request :: get(WeblcmsManager :: PARAM_PUBLICATION);

        $rights_util = WeblcmsRights :: get_instance();

        if ($publications)
        {
            if (! is_array($publications))
            {
                $publications = array($publications);
            }

            foreach ($publications as $publication)
            {
                $locations[] = $rights_util->get_weblcms_location_by_identifier_from_courses_subtree(WeblcmsRights :: TYPE_PUBLICATION, $publication, $course_id);
            }
        }
        else
        {
            if ($category_id)
            {
                $locations[] = $rights_util->get_weblcms_location_by_identifier_from_courses_subtree(WeblcmsRights :: TYPE_COURSE_CATEGORY, $category_id, $course_id);
            }
            else
            {
                $course_tool = $this->get_parent()->get_tool_registration();
                $course_tool_name = $course_tool->get_name();

                if ($course_tool_name && $course_tool_name != 'rights')
                {
                    $locations[] = $rights_util->get_weblcms_location_by_identifier_from_courses_subtree(WeblcmsRights :: TYPE_COURSE_MODULE, $course_tool->get_id(), $course_id);

                }
                else
                {
                    $locations[] = $rights_util->get_courses_subtree_root($course_id);
                }
            }
        }

        return $locations;
    }

    function get_content_object_display_attachment_url($attachment)
    {
        return $this->get_url(array(Tool :: PARAM_ACTION => Tool :: ACTION_VIEW_ATTACHMENT,
                Tool :: PARAM_OBJECT_ID => $attachment->get_id()));
    }

    function get_packages_from_filesystem()
    {
        $types = array();

        $directories = Filesystem :: get_directory_content(Path :: namespace_to_full_path(__NAMESPACE__ . '\tool'),
                Filesystem :: LIST_DIRECTORIES, false);

        foreach ($directories as $directory)
        {
            $types[] = __NAMESPACE__ . '\tool\\' . $directory;
        }

        return $types;
    }

}

?>
