<?php

namespace application\weblcms;

use common\libraries\package\Installer;
use common\libraries\Translation;
use common\libraries\Filesystem;
use common\libraries\Utilities;

/**
 * Abstract class to define the installation of a tool
 *
 * @package application\weblcms;
 *
 * @author Sven Vanpoucke - Hogeschool Gent
 */
abstract class ToolInstaller extends Installer
{
    /**
     * The registration of the tool
     * This registration is needed to register the settings
     *
     * @var ToolRegistration
     */
    private $tool_registration;

    /**
     * Constructor
     *
     * @param $values String[]
     */
    public function __construct($values)
    {
        parent :: __construct($values, WeblcmsDataManager :: get_instance());
    }

    /**
     * Factory
     *
     * @param $tool string
     *            - The tool
     * @param $values string[]
     *            - The installation values
     *
     * @return ToolInstaller
     */
    static function factory($tool, $values)
    {
        $class = Tool :: get_tool_type_namespace($tool) . '\\Installer';

        if (! class_exists($class))
        {
            throw new \Exception(Translation :: get('CouldNotFindToolInstaller', array('TOOL' => $tool)));
        }

        return new $class($values);
    }

    /**
     * Installs the tool
     *
     * @return boolean
     */
    function install_extra()
    {
        if (! $this->register_tool() || ! CourseSettingsController :: get_instance()->install_course_settings($this, $this->tool_registration->get_id()) || ! $this->install_static_settings())
        {
            return false;
        }

        return true;
    }

    /**
     * **************************************************************************************************************
     * Helper functions *
     * **************************************************************************************************************
     */

    /**
     * Registers the tool in the tool table
     */
    private function register_tool()
    {
        $tool_name = Utilities :: get_package_name_from_object($this);

        $tool_object = new CourseTool();

        $tool_object->set_name($tool_name);
        $tool_object->set_section_type($this->retrieve_course_section_type_from_package_info());

        if ($tool_object->create())
        {
            $this->add_message(Installer :: TYPE_NORMAL, Translation :: get('RegisteredTool'));
            $this->tool_registration = $tool_object;
            return true;
        }
        else
        {
            return $this->installation_failed(Installer :: TYPE_NORMAL, Translation :: get('CouldNotRegisterTool'));
        }
    }

    /**
     * Retrieves a course section type from the package info.
     * If no package info or course section definition is found
     * the default section "tool" is selected.
     *
     * @return int
     */
    private function retrieve_course_section_type_from_package_info()
    {
        $section_type = CourseSection :: TYPE_TOOL;

        $path_to_package_info = $this->get_path() . '/php/package/package.info';
        if ($path_to_package_info)
        {
            $xml_document = new \DOMDocument();
            $xml_document->load($path_to_package_info);

            $xml_search_path = new \DOMXPath($xml_document);
            $query_result = $xml_search_path->query('//course_section');

            $first_course_section_result = $query_result->item(0);
            if ($first_course_section_result)
            {
                $section_type = CourseSection :: get_type_from_type_name($first_course_section_result->nodeValue);
            }
        }

        return $section_type;
    }

    /**
     * Installs the settings that are required for each tool (active, visible)
     *
     * @return boolean
     */
    private function install_static_settings()
    {
        foreach (CourseSetting :: get_static_tool_settings() as $static_tool_setting)
        {
            $course_setting = new CourseSetting();
            $course_setting->set_tool_id($this->tool_registration->get_id());
            $course_setting->set_name($static_tool_setting);
            $course_setting->set_global_setting(0);

            if (! $course_setting->create())
            {
                return false;
            }

            $course_setting_default_value = new CourseSettingDefaultValue();
            $course_setting_default_value->set_course_setting_id($course_setting->get_id());
            $course_setting_default_value->set_value(1);

            if (! $course_setting_default_value->create())
            {
                return false;
            }

        }

        return true;
    }

}

?>
