<?php
namespace application\weblcms;

use common\libraries\Utilities;

use common\libraries\Filesystem;
use common\libraries\EqualityCondition;
use common\libraries\Configuration;
use common\libraries\InCondition;
use common\libraries\AndCondition;
use DOMDocument;
use repository\ContentObject;
use common\libraries\SubselectCondition;
use repository\RepositoryDataManager;
use common\libraries\OrCondition;
use common\libraries\InequalityCondition;
use common\libraries\Session;
use common\libraries\Request;
use common\libraries\NotCondition;
use group\GroupDataManager;
use user\UserDataManager;
use user\User;

use application\weblcms\course\CourseUserRelation;
use application\weblcms\course\CourseGroupRelation;
use application\weblcms\course\CourseRelCourseSetting;
use application\weblcms\course_type\CourseTypeRelCourseSetting;
use application\weblcms\course\Course;

use common\libraries\DataClassCache;

use rights\RightsLocation;
use rights\RightsLocationEntityRight;

/**
 * $Id: weblcms_data_manager.class.php 218 2009-11-13 14:21:26Z kariboe $
 *
 * @package application.lib.weblcms
 */
/**
 * ============================================================================== This is a skeleton for a data manager
 * for the Weblcms application. Data managers must extend this class.
 *
 * @author Tim De Pauw ==============================================================================
 */

class WeblcmsDataManager
{
    /**
     * Instance of the class, for the singleton pattern.
     */
    private static $instance;

    /**
     * Constructor. Initializes the data manager.
     */
    public function __construct()
    {
        $this->initialize();
    }

    /**
     * Creates the shared instance of the configured data manager if necessary and returns it. Uses a factory pattern.
     *
     * @return WeblcmsDataManagerInterface The instance.
     */
    static function get_instance()
    {
        if (! isset(self :: $instance))
        {
            $type = Configuration :: get_instance()->get_parameter('general', 'data_manager');
            $class = __NAMESPACE__ . '\\' . Utilities :: underscores_to_camelcase($type) . 'WeblcmsDataManager';
            self :: $instance = new $class();
        }
        return self :: $instance;
    }

    /*
     * Gets the tool of a section
     */
    static function get_tools($requested_section = 'all')
    {
        $course_modules = Array();

        $options = array('forceEnum' => array('properties'));

        $dir = dirname(__FILE__) . '/../../tool/';
        $tools = Filesystem :: get_directory_content($dir, Filesystem :: LIST_DIRECTORIES, false);
        foreach ($tools as $tool)
        {
            $info_file = $dir . $tool . '/php/package/package.info';
            if (! file_exists($info_file))
            {
                continue;
            }

            $doc = new DOMDocument();
            $doc->load($info_file);
            $xpath = new \DOMXPath($doc);

            $course_section = $xpath->query('//course_section');
            $section = $course_section->item(0)->nodeValue;

            if ($section == $requested_section || $requested_section == 'all')
            {
                $course_modules[] = $tool;
            }
        }

        return $course_modules;
    }

    /**
     * Checks whether subscription to a specific course is allowed.
     *
     * @param $course Course
     * @param $user User
     * @return boolean
     */
    static function course_subscription_allowed($course, $user)
    {
        $already_subscribed = self :: get_instance()->is_subscribed($course, $user);

        $subscription_allowed = ($course->get_access() == 1 ? true : false);

        if (! $already_subscribed && $subscription_allowed)
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * Checks whether unsubscription from a specific course is allowed.
     *
     * @param $course Course
     * @param $user User
     * @return boolean
     */
    static function course_unsubscription_allowed($course, $user)
    {
        if ($course->is_course_admin($user))
        {
            return false;
        }

        $already_subscribed = self :: get_instance()->is_subscribed($course, $user);
        $unsubscription_allowed = ($course->get_unsubscribe_allowed() == 1 ? true : false);
        if ($already_subscribed && $unsubscription_allowed)
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    static function get_user_course_groups($user_id, $course_id)
    {
        $course_groups = self :: get_instance()->retrieve_course_groups_from_user($user_id, $course_id)->as_array();

        $course_groups_recursive = array();

        foreach ($course_groups as $course_group)
        {
            if (! array_key_exists($course_group->get_id(), $course_groups_recursive))
            {
                $course_groups_recursive[$course_group->get_id()] = $course_group;
            }

            $parents = $course_group->get_parents(false);

            while ($parent = $parents->next_result())
            {
                if (! array_key_exists($parent->get_id(), $course_groups_recursive))
                {
                    $course_groups_recursive[$parent->get_id()] = $parent;
                }
            }
        }

        return $course_groups_recursive;
    }

    private static $is_cached;
    private static $new_publications_cache;

    /**
     * Determines if a tool has new publications since the last time the current user visited the tool.
     *
     * @param $tool string
     * @param $course Course
     */
    static function tool_has_new_publications($tool, User $user, Course $course = null)
    {
        $key = $user->get_id() . $course->get_id();
        if (! isset(self :: $is_cached[$key]))
        {
            self :: $is_cached[$key] = true;

            if (! $course || $course->get_id() == 0)
            {
                return false;
            }

            $weblcms_rights = WeblcmsRights :: get_instance();

            if ($course->is_course_admin($user))
            {
                $tools_with_new_publications = self :: get_instance()->retrieve_new_publication_icon_ids($course->get_id(), $user->get_id(), true);
                while ($publication = $tools_with_new_publications->next_result(false))
                {
                    self :: $new_publications_cache[$key][$publication->get_tool()] = true;
                }
            }
            else
            {
                $publications = self :: get_instance()->retrieve_new_publication_icon_ids($course->get_id(), $user->get_id());

                while ($publication = $publications->next_result(false))
                {
                    if (! isset(self :: $new_publications_cache[$key][$publication->get_tool()]) && $weblcms_rights->is_allowed_in_courses_subtree(WeblcmsRights :: VIEW_RIGHT, $publication->get_id(), WeblcmsRights :: TYPE_PUBLICATION, $course->get_id()))
                    {
                        // check if the publication is visible
                        $visible = true;
                        if ($publication->get_category_id() != 0)
                        {
                            // categories can be made invisible
                            $category = self :: get_instance()->retrieve_content_object_publication_category($publication->get_category_id());
                            $visible = $category->is_recursive_visible();
                        }
                        if ($visible)
                        {
                            self :: $new_publications_cache[$key][$publication->get_tool()] = true;
                        }
                    }
                }
            }
        }        

        return self :: $new_publications_cache[$key][$tool];
    }

    /**
     * Returns if a category inside a tool has new publications
     *
     * @param $tool type
     * @param $user User
     * @param $course Course
     * @param $category type
     * @return type
     */

    static function tool_category_has_new_publications($tool, User $user, Course $course, $category)
    {
        if ($course->is_course_admin($user))
        {
            $tools_with_new_publications = self :: get_instance()->retrieve_new_publication_icon_ids($course->get_id(), $user->get_id(), true, $tool, $category);
            if ($tools_with_new_publications->size() > 0)
            {
                return true;
            }
            else
            {
                return false;
            }
        }
        else
        {
            $weblcms_rights = WeblcmsRights :: get_instance();

            $publications = self :: get_instance()->retrieve_new_publication_icon_ids($course->get_id(), $user->get_id(), false, $tool, $category);

            while ($publication = $publications->next_result(false))
            {
                if ($weblcms_rights->is_allowed_in_courses_subtree(WeblcmsRights :: VIEW_RIGHT, $publication->get_id(), WeblcmsRights :: TYPE_PUBLICATION, $course->get_id()))
                {
                    return true;
                }
            }
            return false;
        }
    }

    static $is_teacher_cache;

    static function is_teacher_through_platform_groups(Course $course, User $user)
    {
        if (is_null(self :: $is_teacher_cache[$course->get_id()][$user->get_id()]))
        {
            $group_ids = $user->get_groups(true);

            $conditions = array();

            if (count($group_ids) == 0)
            {
                self :: $is_teacher_cache[$course->get_id()][$user->get_id()] = false;
            }
            else
            {
                $conditions[] = new InCondition(CourseGroupRelation :: PROPERTY_GROUP_ID, $group_ids);
                $conditions[] = new EqualityCondition(CourseGroupRelation :: PROPERTY_STATUS, CourseGroupRelation :: STATUS_TEACHER);
                $conditions[] = new EqualityCondition(CourseGroupRelation :: PROPERTY_COURSE_ID, $course->get_id());
                $condition = new AndCondition($conditions);
                self :: $is_teacher_cache[$course->get_id()][$user->get_id()] = (self :: get_instance()->count_course_group_relations($condition) > 0);
            }

        }

        return self :: $is_teacher_cache[$course->get_id()][$user->get_id()];
    }

    static function get_publications_condition(Course $course, User $user, $tool = null, $content_object_type = null)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_COURSE_ID, $course->get_id());

        if ($tool)
        {
            $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_TOOL, $tool);
        }

        if (! empty($content_object_type))
        {
            if (is_array($content_object_type))
            {
                $subselect_condition = new InCondition(ContentObject :: PROPERTY_TYPE, $content_object_type);
            }
            else
            {
                $subselect_condition = new EqualityCondition(ContentObject :: PROPERTY_TYPE, $content_object_type);
            }

            $conditions[] = new SubselectCondition(ContentObjectPublication :: PROPERTY_CONTENT_OBJECT_ID, ContentObject :: PROPERTY_ID, ContentObject :: get_table_name(), $subselect_condition, null, RepositoryDataManager :: get_instance());
        }

        $user_id = $user->get_id();
        $course_groups = self :: get_instance()->retrieve_course_groups_from_user($user_id, $course->get_id());

        while ($course_group = $course_groups->next_result())
        {
            $course_group_ids[] = $course_group->get_id();
        }

        if ((! $course->is_course_admin($user)))
        {
            $time_conditions = array();
            $time_conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_HIDDEN, 0);

            $forever_condition = new AndCondition(array(
                    new EqualityCondition(ContentObjectPublication :: PROPERTY_FROM_DATE, 0),
                    new EqualityCondition(ContentObjectPublication :: PROPERTY_TO_DATE, 0)));

            $between_condition = new AndCondition(array(
                    new InequalityCondition(ContentObjectPublication :: PROPERTY_FROM_DATE, InequalityCondition :: LESS_THAN_OR_EQUAL, time()),
                    new InequalityCondition(ContentObjectPublication :: PROPERTY_TO_DATE, InequalityCondition :: GREATER_THAN_OR_EQUAL, time())));

            $time_conditions[] = new OrCondition(array($forever_condition, $between_condition));

            $conditions[] = new AndCondition($time_conditions);

            // if ($user_id)
            // {
            // $access[] = new InCondition(ContentObjectPublicationUser ::
            // PROPERTY_USER, $user_id,
            // ContentObjectPublicationUser :: get_table_name());
            // }
            //
            // if (count($course_group_ids) > 0)
            // {
            // $access[] = new InCondition(ContentObjectPublicationCourseGroup
            // :: PROPERTY_COURSE_GROUP_ID,
            // $course_group_ids, ContentObjectPublicationCourseGroup ::
            // get_table_name());
            // }
            //
            // if (!empty($user_id) || !empty($course_group_ids))
            // {
            // $access[] = new AndCondition(array(
            // new EqualityCondition(ContentObjectPublicationUser ::
            // PROPERTY_USER, null,
            // ContentObjectPublicationUser :: get_table_name()),
            // new EqualityCondition(ContentObjectPublicationCourseGroup ::
            // PROPERTY_COURSE_GROUP_ID,
            // null, ContentObjectPublicationCourseGroup :: get_table_name())));
            // }
            //
            // $conditions[] = new OrCondition($access);
        }

        $condition = new AndCondition($conditions);

        return $condition;
    }

    static function get_user_courses_condition(User $user)
    {
        $access_conditions = array();
        $access_conditions[] = new EqualityCondition(CourseUserRelation :: PROPERTY_USER_ID, $user->get_id(), CourseUserRelation :: get_table_name());
        $access_conditions[] = new InCondition(CourseGroupRelation :: PROPERTY_GROUP_ID, $user->get_groups(true), CourseGroupRelation :: get_table_name());
        $condition = new OrCondition($access_conditions);

        return $condition;
    }

    static function is_course_code_available($code)
    {
        $condition = new EqualityCondition(Course :: PROPERTY_VISUAL_CODE, $code);
        return (self :: get_instance()->count_courses($condition) == 0);
    }

    /**
     * Returns an array of users that are the targets of the given publication based on the rights structure
     *
     * @param $publication ContentObjectPublication
     * @return array
     */
    static function get_publication_target_users($publication)
    {
        $target_users = array();
        $publication_id = $publication->get_id();
        $course_id = $publication->get_course_id();

        // get the entities
        $target_entities = WeblcmsRights :: get_instance()->get_target_entities(WeblcmsRights :: VIEW_RIGHT, WeblcmsManager :: APPLICATION_NAME, $publication_id, WeblcmsRights :: TYPE_PUBLICATION, $course_id, WeblcmsRights :: TREE_TYPE_COURSE);

        // check for everybody
        if (array_key_exists(0, $target_entities[0]))
        {
            // get all course users
            $target_users = self :: get_instance()->retrieve_all_course_users($course_id)->as_array();
        }
        else
        {
            // get all users for all entities
            foreach ($target_entities as $entity_type => $entity_ids)
            {
                switch ($entity_type)
                {
                    case CoursePlatformGroupEntity :: ENTITY_TYPE :
                        foreach ($entity_ids as $group_id)
                        {
                            $group = GroupDataManager :: get_instance()->retrieve_group($group_id);
                            if ($group)
                            {
                                $group_user_ids = $group->get_users(true, true);
                                if (count($group_user_ids) > 0)
                                {
                                    $group_users = UserDataManager :: get_instance()->retrieve_users(new InCondition(User :: PROPERTY_ID, $group_user_ids))->as_array();
                                    foreach ($group_users as $group_user)
                                    {
                                        if (! array_key_exists($group_user->get_id(), $target_users))
                                        {
                                            $target_users[$group_user->get_id()] = $group_user;
                                        }
                                    }
                                }
                            }
                        }
                        break;
                    case CourseUserEntity :: ENTITY_TYPE :
                        foreach ($entity_ids as $user_id)
                        {
                            $user = UserDataManager :: get_instance()->retrieve_user($user_id);
                            if ($user && ! array_key_exists($user->get_id(), $target_users))
                            {
                                $target_users[$user->get_id()] = $user;
                            }
                        }
                        break;
                    case CourseGroupEntity :: ENTITY_TYPE :
                        foreach ($entity_ids as $course_group_id)
                        {
                            $course_group = self :: get_instance()->retrieve_course_group($course_group_id);
                            if ($course_group)
                            {
                                $course_group_users = $course_group->get_members(true, true, true);
                                foreach ($course_group_users as $course_group_user)
                                {
                                    if (! array_key_exists($course_group_user->get_id(), $target_users))
                                    {
                                        $target_users[$course_group_user->get_id()] = $course_group_user;
                                    }
                                }
                            }
                        }
                        break;
                    case TemplateEntity :: ENTITY_TYPE :
                        foreach ($entity_ids as $template_id)
                        {
                            $template_entity = RightsDataManager :: get_instance()->retrieve_template($template_id);
                            if ($template_entity)
                            {
                                $template_entity_users = $template_entity->get_users();
                                foreach ($template_entity_users as $template_entity_user)
                                {
                                    if (! array_key_exists($template_entity_user->get_id(), $target_users))
                                    {
                                        $target_users[$template_entity_user->get_id()] = $template_entity_user;
                                    }
                                }
                            }
                        }
                        break;
                }
            }
        }
        return $target_users;
    }

    /**
     * Returns an array of users that are the targets of the publication identified by the given ID based on the rights
     * structure
     *
     * @param $publication_id int
     * @return array
     */
    static function get_publication_target_users_by_publication_id($publication_id)
    {
        $publication = self :: get_instance()->retrieve_content_object_publication($publication_id);
        if (! $publication)
        {
            return array();
        }
        return self :: get_publication_target_users($publication);
    }

    /**
     * Retrieves all users that are admins in the given course.
     *
     * @param $course_id int The id of the course
     */
    static function retrieve_course_admin_users($course_id)
    {
        $wdm = self :: get_instance();
        // get subscribed users with direct subscription status 1
        $conditions = array();
        $conditions[] = new EqualityCondition(CourseUserRelation :: PROPERTY_COURSE_ID, $course_id, CourseUserRelation :: get_table_name());
        $conditions[] = new EqualityCondition(CourseUserRelation :: PROPERTY_STATUS, 1, CourseUserRelation :: get_table_name());
        $condition = new AndCondition($conditions);
        $users = $wdm->retrieve_users_directly_subscribed_to_course($condition);
        return $users->as_array();
    }

    /**
     * Retrieves all groups that are admins in the given course.
     *
     * @param $course_id type The id of the course
     */
    static function retrieve_course_admin_groups($course_id)
    {
        $wdm = self :: get_instance();
        // get subscribed groups with direct subscription status 1
        $conditions = array();
        $conditions[] = new EqualityCondition(CourseGroupRelation :: PROPERTY_COURSE_ID, $course_id, CourseGroupRelation :: get_table_name());
        $conditions[] = new EqualityCondition(CourseGroupRelation :: PROPERTY_STATUS, 1, CourseGroupRelation :: get_table_name());
        $condition = new AndCondition($conditions);
        $groups = $wdm->retrieve_groups_directly_subscribed_to_course($condition);
        return $groups->as_array();
    }

    /**
     * Retrieves the course settings with values for a given course
     *
     * @param $course_id int
     *
     * @return ResultSet<CourseSetting>
     */
    static function retrieve_course_settings_with_values_for_course($course_id)
    {
        $condition = new EqualityCondition(CourseRelCourseSetting :: PROPERTY_COURSE_ID, $course_id, CourseRelCourseSetting :: get_table_name());

        $result_set = self :: get_instance()->retrieve_course_settings_with_course_values($condition);

        DataClassCache :: truncate(CourseSetting :: class_name());

        return $result_set;
    }

    /**
     * Retrieves the course settings with values for a given course type
     *
     * @param $course_type_id int
     *
     * @return ResultSet<CourseSetting>
     */
    static function retrieve_course_settings_with_values_for_course_type($course_type_id)
    {
        $condition = new EqualityCondition(CourseTypeRelCourseSetting :: PROPERTY_COURSE_TYPE_ID, $course_type_id, CourseTypeRelCourseSetting :: get_table_name());

        $result_set = self :: get_instance()->retrieve_course_settings_with_course_type_values($condition);

        DataClassCache :: truncate(CourseSetting :: class_name());

        return $result_set;
    }

    /**
     * Retrieves the course settings with the default values
     *
     * @return ResultSet<CourseSetting>
     */
    static function retrieve_course_settings_with_default_values()
    {
        $result_set = self :: get_instance()->retrieve_course_settings_with_default_values();

        DataClassCache :: truncate(CourseSetting :: class_name());

        return $result_set;
    }

    /**
     * Retrieves a course tool by a given tool name
     *
     * @param $course_tool_name String
     *
     * @return CourseTool
     */
    static function retrieve_course_tool_by_name($course_tool_name)
    {
        $condition = new EqualityCondition(CourseTool :: PROPERTY_NAME, $course_tool_name);

        return self :: get_instance()->retrieve_course_tools($condition)->next_result();
    }

    /**
     * Fixes the course type user category rel course display orders for deleted courses for a given user Retrieves all
     * the coures type user category rel course objects for which the course does not exist anymore and removes them and
     * fixes the display orders
     *
     * @param $user_id int
     *
     * @return boolean
     */
    static function fix_course_type_user_category_rel_course_for_user($user_id)
    {
        $conditions = array();

        $conditions[] = new EqualityCondition(CourseTypeUserCategoryRelCourse :: PROPERTY_USER_ID, $user_id);
        $conditions[] = new NotCondition(new SubselectCondition(CourseTypeUserCategoryRelCourse :: PROPERTY_COURSE_ID, Course :: PROPERTY_ID, Course :: get_table_name()));

        $condition = new AndCondition($conditions);

        $instance = self :: get_instance();

        while ($course_type_user_category_rel_course = $instance->retrieve_course_type_user_category_rel_courses($condition, 0, 1)->next_result())
        {
            if (! $course_type_user_category_rel_course->delete())
            {
                return false;
            }
        }

        return true;
    }

    /**
     * Clears the locked rights for a given location id (an optionally a given right id)
     *
     * @param $location_id int
     * @param $right_id int - [OPTIONAL] default null
     * @return boolean
     */
    static function clear_locked_rights_for_location($location_id, $right_id = null)
    {
        $conditions = array();

        $conditions[] = new EqualityCondition(RightsLocationLockedRight :: PROPERTY_LOCATION_ID, $location_id);

        if (! is_null($right_id))
        {
            $conditions[] = new EqualityCondition(RightsLocationLockedRight :: PROPERTY_RIGHT_ID, $right_id);
        }

        $condition = new AndCondition($conditions);

        return self :: get_instance()->deletes(RightsLocationLockedRight :: class_name(), $condition);
    }

    /**
     * Is the given right locked for the given location
     *
     * @param $location_id int
     * @param $right_id int
     *
     * @return boolean
     */
    static function is_right_locked_for_location($location_id, $right_id)
    {
        $conditions = array();

        $conditions[] = new EqualityCondition(RightsLocationLockedRight :: PROPERTY_LOCATION_ID, $location_id);
        $conditions[] = new EqualityCondition(RightsLocationLockedRight :: PROPERTY_RIGHT_ID, $right_id);

        $condition = new AndCondition($conditions);

        return (self :: get_instance()->count_objects(RightsLocationLockedRight :: get_table_name(), $condition) > 0);
    }

    /**
     * **************************************************************************************************************
     * Helper Functionality *
     * **************************************************************************************************************
     */

    const DISPLAY_ORDER_DIRECTION_UP = - 1;
    const DISPLAY_ORDER_DIRECTION_DOWN = 1;

    /**
     * Fixes the display order for a given class and column with a given condition from start to end with a given number
     * and direction
     *
     * @param $class_name string
     * @param $column_name string
     * @param $condition Condition [OPTIONAL - default null]
     * @param $start_display_order int [OPTIONAL - default 1]
     * @param $end_display_order int [OPTIONAL - default null]
     * @param $count int [OPTIONAL - default 1]
     * @param $direction int [OPTIONAL - default self :: DISPLAY_ORDER_DIRECTION_UP]
     * @return boolean
     */
    static function fix_display_order($class_name, $column_name, $condition = null, $start_display_order = 1, $end_display_order = null, $count = 1, $direction = self :: DISPLAY_ORDER_DIRECTION_UP)
    {
        // Make sure the start value is the lowest and the end value is the
        // highest value
        $start = $end_display_order ? min($start_display_order, $end_display_order) : $start_display_order;
        $end = max($start_display_order, $end_display_order);

        $properties = array();
        $properties[$column_name] = $column_name . '+' . ($direction * $count);

        $conditions = array();

        if ($condition)
        {
            $conditions[] = $condition;
        }

        $conditions[] = new InequalityCondition($column_name, ($direction == self :: DISPLAY_ORDER_DIRECTION_DOWN) ? InequalityCondition :: GREATER_THAN_OR_EQUAL : InequalityCondition :: GREATER_THAN, $start);

        if (! is_null($end_display_order))
        {
            $conditions[] = new InequalityCondition($column_name, ($direction == self :: DISPLAY_ORDER_DIRECTION_DOWN) ? InequalityCondition :: LESS_THAN : InequalityCondition :: LESS_THAN_OR_EQUAL, $end);
        }

        $condition = new AndCondition($conditions);

        return self :: get_instance()->updates($class_name, $properties, $condition);
    }

    /**
     * **************************************************************************************************************
     * Rights Functionality *
     * **************************************************************************************************************
     */

    /**
     * Removes the entities for all the children of a given rights location, limited to the given rights
     *
     * @param $rights_location RightsLocation
     * @param $rights int[]
     *
     * @return boolean
     */
    static function remove_entities_for_children_from_rights_location(RightsLocation $rights_location, $rights = array())
    {
        $location_conditions = array();

        $location_conditions[] = new EqualityCondition(RightsLocation :: PROPERTY_TREE_TYPE, WeblcmsRights :: TREE_TYPE_COURSE);
        $location_conditions[] = new EqualityCondition(RightsLocation :: PROPERTY_TREE_IDENTIFIER, 0);
        $location_condition[] = new InequalityCondition(RightsLocation :: PROPERTY_LEFT_VALUE, InequalityCondition :: GREATER_THAN, $rights_location->get_left_value());
        $location_condition[] = new InequalityCondition(RightsLocation :: PROPERTY_RIGHT_VALUE, InequalityCondition :: LESS_THAN, $rights_location->get_right_value());

        $location_condition = new AndCondition($location_conditions);

        $conditions = array();
        $conditions[] = new SubSelectCondition(RightsLocationEntityRight :: PROPERTY_LOCATION_ID, RightsLocation :: PROPERTY_ID, RightsLocation :: get_table_name(), $location_condition);

        if (count($rights) > 0)
        {
            $conditions[] = new InCondition(RightsLocationEntityRight :: PROPERTY_RIGHT_ID, $rights);
        }

        $condition = new AndCondition($conditions);

        return self :: get_instance()->delete(RightsLocationEntityRight :: get_table_name(), $condition);

    }

    /**
     * **************************************************************************************************************
     * Request Functionality *
     * **************************************************************************************************************
     */

    /**
     * Returns whether or not there is a pending request for a given user and course
     *
     * @param $user_id int
     * @param $course_id int
     *
     * @return boolean
     */
    static function is_user_requested_for_course($user_id, $course_id)
    {
        $conditions = array();

        $conditions[] = new EqualityCondition(CourseRequest :: PROPERTY_COURSE_ID, $course_id);
        $conditions[] = new EqualityCondition(CourseRequest :: PROPERTY_USER_ID, $user_id);
        $conditions[] = new EqualityCondition(CourseRequest :: PROPERTY_DECISION, CourseRequest :: NO_DECISION);

        $condition = new AndCondition($conditions);

        return (self :: get_instance()->count_objects(CourseRequest :: get_table_name(), $condition) > 0);

    }

}
?>
