<?php
namespace application\weblcms;

use reporting\Reporting;
use user\VisitTracker;
use reporting\ReportingData;
use reporting\ReportingFormatter;
use common\libraries\PatternMatchCondition;
use reporting\ReportingManager;
use common\libraries\Redirect;
use common\libraries\Application;
use common\libraries\Theme;
use common\libraries\OrCondition;
use common\libraries\Utilities;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Path;
use common\libraries\Translation;
use application\weblcms\tool\reporting\ReportingTool;
use common\libraries\DatetimeUtilities;
use common\libraries\Request;
use repository\RepositoryDataManager;
use repository\ContentObjectRenderer;
use repository\RepositoryManager;
use common\libraries\Session;
use user\UserDataManager;
use user\UserManager;
use application\weblcms\tool\user\UserTool;
use common\libraries\ObjectTableOrder;
use common\libraries\InCondition;
use repository\content_object\assessment\Assessment;
use repository\ComplexContentObjectItem;
use common\libraries\PlatformSetting;

/**
 * @package application.weblcms.php.reporting.blocks
 *
 * Reporting block with an overview of scores of each assessment question per user
 * 
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 * @author Anthony Hurst (Hogeschool Gent)
 */
class WeblcmsAssessmentQuestionsUsersReportingBlock extends WeblcmsToolReportingBlock
{
    private static $COLUMN_NAME;
    
    /**
     * Cache of reporting data to prevent the code being run twice.
     * @var \reporting\ReportingData 
     */
    private $reporting_data;
    
    /**
     * Instatiates the column headers.
     * @param type $parent Pass-through variable. Please refer to parent class(es) for more details.
     * &param type $vertical Pass-through variable. Please refer to parent class(es) for more details.
     */
    public function __construct($parent, $vertical)
    {
        self::$COLUMN_NAME = Translation::get('Name');
        parent :: __construct($parent, $vertical);
    }
    
    /**
     * Compiles the statistics to be displayed in the reporting block.
     * @author Anthony Hurst (Hogeschool Gent)
     * @return \reporting\ReportingData The reporting data to be displayed.
     */
    public function retrieve_data()
    {
        if(!is_null($this->reporting_data))
        {
            return $this->reporting_data;
        }
        $this->reporting_data = new ReportingData();
        $publication = WeblcmsDataManager :: get_instance()->retrieve_content_object_publication($this->get_pid());
        $condition = new EqualityCondition(ComplexContentObjectItem :: PROPERTY_PARENT, $publication->get_content_object_id());
        $complex_questions = RepositoryDataManager :: get_instance()->retrieve_complex_content_object_items($condition)->as_array();
        
        /**
         * @var $questions Defines column headers against the question id.
         */
        $question_headers = array();
        $question_weights = array();
        // Defines the column headers in the reporting block.
        $question_headers[-1] = self::$COLUMN_NAME;
        $vertical_headers = count($complex_questions) > 5;
        foreach($complex_questions as $complex_question)
        {
            $question_header = $vertical_headers ? substr($complex_question->get_ref_object()->get_title(),0,14) : 
                $complex_question->get_ref_object()->get_title();
            $question_headers[$complex_question->get_id()] = '<div id="' . $complex_question->get_id() . '">' . $question_header . '</div>';
            $question_weights[$complex_question->get_id()] = $complex_question->get_weight();
        }
        
        // Defines the row categories in the reporting block.
        $this->reporting_data->set_rows($question_headers);
        $users = WeblcmsDataManager :: get_instance()->retrieve_publication_target_users($this->get_pid(), $this->get_course_id())->as_array();
        foreach($users as $user)
        {
            $this->reporting_data->add_category($user->get_id());
            $this->reporting_data->add_data_category_row($user->get_id(), self::$COLUMN_NAME, $user->get_fullname());
        }
        // Retrieve all the assessment attempts trackers for the current assessment ordered by the user id.
        $condition = new EqualityCondition(WeblcmsAssessmentAttemptsTracker :: PROPERTY_ASSESSMENT_ID, $publication->get_id());
        $order_by = array();
        $order_by[] = new ObjectTableOrder(WeblcmsAssessmentAttemptsTracker :: PROPERTY_USER_ID);
        $assessment_attempts_trackers = WeblcmsAssessmentAttemptsTracker::get_data(WeblcmsAssessmentAttemptsTracker::CLASS_NAME, WeblcmsManager::APPLICATION_NAME, $condition, null, null, $order_by)->as_array();
        
        $user_question_statistics = array();
        if(count($assessment_attempts_trackers) > 0)
        {
            $assessment_attempts_tracker_ids = array();
            $current_user_id = reset($assessment_attempts_trackers)->get_user_id();
            
            // Aggregate all the assessment attempts trackers for a single user to retrieve the user's question attempts trackers in one database transaction.
            foreach($assessment_attempts_trackers as $assessment_attempts_tracker)
            {
                // If the user id changes, calculate the user's statistics and move on to the next user.
                if($assessment_attempts_tracker->get_user_id() != $current_user_id)
                {
                    $user_question_statistics[$current_user_id] = $this->collate_question_attempts_trackers($assessment_attempts_tracker_ids);
                    $assessment_attempts_tracker_ids = array();
                    $current_user_id = $assessment_attempts_tracker->get_user_id();
                }
                $assessment_attempts_tracker_ids[] = $assessment_attempts_tracker->get_id();
            }
            // Catch any remainders in the array that won't be caught by the if. ($assessment_attempts_tracker is null, preventing a last pass through)
            $user_question_statistics[$current_user_id] = $this->collate_question_attempts_trackers($assessment_attempts_tracker_ids);
        }
        
        foreach($user_question_statistics as $user_id => $question_statistics)
        {
            foreach($question_statistics as $question_id => $score)
            {
                if(!is_null($score))
                {
                    if(($score / $question_weights[$question_id]) >= PlatformSetting :: get('passing_percentage') / 100)
                    {
                        $score_html = '<span style="color:green">' . $score . '/' . $question_weights[$question_id] . '</span>';
                    }
                    else
                    {
                        $score_html = '<span style="color:red">' . $score . '/' . $question_weights[$question_id] . '</span>';
                    }
                    $this->reporting_data->add_data_category_row($user_id, $question_headers[$question_id], $score_html);
                }
            }
        }
        $this->reporting_data->hide_categories();
        return $this->reporting_data;
    }
    
    /**
     * Obtains the score for each question.
     * @author Anthony Hurst (Hogeschool Gent)
     * @param array $assessment_attempts_tracker_ids The ids of the assessment attempts trackers whose question attempts trackers are to be retrieved
     * @return array Format $question_id => $question_score.
     */
    private function collate_question_attempts_trackers($assessment_attempts_tracker_ids)
    {
        // Retrieve all the question attempts trackers of a single user ordered by the question id.
        $condition = new InCondition(WeblcmsQuestionAttemptsTracker :: PROPERTY_ASSESSMENT_ATTEMPT_ID, $assessment_attempts_tracker_ids);
        $order_by = array();
        $order_by[] = new ObjectTableOrder(WeblcmsQuestionAttemptsTracker::PROPERTY_QUESTION_CID);
        $question_attempts_trackers = WeblcmsQuestionAttemptsTracker::get_data(WeblcmsQuestionAttemptsTracker::CLASS_NAME, WeblcmsManager::APPLICATION_NAME, $condition, null, null, $order_by)->as_array();
        
        $user_question_statistics = array();
        if(count($question_attempts_trackers) > 0)
        {
            $current_question_attempts_trackers = array();
            $current_question_id = reset($question_attempts_trackers)->get_question_cid();
            
            // Aggregate all the question attempts trackers for a single question to calculate the score to be displayed.
            foreach($question_attempts_trackers as $question_attempts_tracker)
            {
                // If the question id changes, calculate the score for the current question and move on to the next question.
                if($question_attempts_tracker->get_question_cid() != $current_question_id)
                {
                    $user_question_statistics[$current_question_id] = $this->get_score($current_question_attempts_trackers);
                    $current_question_attempts_trackers = array();
                    $current_question_id = $question_attempts_tracker->get_question_cid();
                }
                $current_question_attempts_trackers[] = $question_attempts_tracker;
            }
            // Catch any remainders in the array that won't be caught by the if. ($question_attempts_tracker is null, preventing a last pass through)
            $user_question_statistics[$current_question_id] = $this->get_score($current_question_attempts_trackers);
        }
        return $user_question_statistics;
    }
    
    /**
     * Leave in commentary for now, pending completion of testing. Original name count_data().
     * @return \reporting\ReportingData 
     */
    /*public function retrieve_data()
    {
        $this->reporting_data = new ReportingData();
        $course_id = $this->get_course_id();
        $publication_id = $this->get_pid();
        $weblcms_data_manager = WeblcmsDataManager :: get_instance();
        $users_resultset = $weblcms_data_manager->retrieve_publication_target_users($publication_id, $course_id);
        
        

        //Retrieve the questions of the assessment     
        $weblcms_data_manager = WeblcmsDataManager :: get_instance();
        $publication = WeblcmsDataManager :: get_instance()->retrieve_content_object_publication($publication_id);
        $condition = new EqualityCondition(ComplexContentObjectItem :: PROPERTY_PARENT, $publication->get_content_object_id());
        $questions_resultset = RepositoryDataManager :: get_instance()->retrieve_complex_content_object_items($condition);
        
        if($questions_resultset->size() > 5)
        {
            $vertical_th = true;
        }
        $headings = array();
        $headings[] = Translation :: get('Name');
        
        $questions = array();
        while($complex_question = $questions_resultset->next_result())
        {
            $questions[] = $complex_question;
            if(!$vertical_th)
            {
                $headings[] = $complex_question->get_ref_object()->get_title();
            }
            else
            {
                $headings[] = substr($complex_question->get_ref_object()->get_title(),0,14);
            }
        }

        $this->reporting_data->set_rows($headings);

        
        $key = 0;
        while ($user = $users_resultset->next_result())
        {
            $key++;
            $this->reporting_data->add_category($key);
            $this->reporting_data->add_data_category_row($key, Translation :: get('Name'), $user->get_fullname());
            
            foreach($questions as $question)
            {
                $score = null;
                $question_object = $question->get_ref_object();
                if(!$vertical_th)
                {
                    $title = $question_object->get_title();
                }
                else
                {
                    $title = substr($question_object->get_title(),0,14);
                }
                $attempts_tracker = new WeblcmsAssessmentAttemptsTracker();
                $conditions = array();
                $conditions[] = new EqualityCondition(WeblcmsAssessmentAttemptsTracker :: PROPERTY_ASSESSMENT_ID, $publication_id);
                $conditions[] = new EqualityCondition(WeblcmsAssessmentAttemptsTracker :: PROPERTY_USER_ID, $user->get_id());
                $condition = new AndCondition($conditions);
                $attempts_by_user = $attempts_tracker->retrieve_tracker_items($condition);
                $question_attempts = array();
                
                foreach($attempts_by_user as $attempt)
                {
                    $question_attempts_tracker = new WeblcmsQuestionAttemptsTracker();
                    $conditions = array();
                    $conditions[] = new EqualityCondition(WeblcmsQuestionAttemptsTracker :: PROPERTY_QUESTION_CID, $question->get_id());
                    $conditions[] = new EqualityCondition(WeblcmsQuestionAttemptsTracker :: PROPERTY_ASSESSMENT_ATTEMPT_ID, $attempt->get_id());
                    $condition = new AndCondition($conditions);
                    $trackers = $question_attempts_tracker->retrieve_tracker_items($condition);
                    
                    if(count($trackers) > 0)
                    {
                        $question_attempts[] = $trackers[0];
                        $score = $this->get_score($question_attempts);
                    }
                    
                }
                
                if(! is_null($score))
                {
                    if(($score / $question->get_weight()) >= PlatformSetting :: get('passing_percentage') / 100)
                    {
                        $this->reporting_data->add_data_category_row($key, $title, '<span style="color: green">' . $score 
                                . ' / ' . $question->get_weight() . '</span>');
                    }
                    else
                    {
                         $this->reporting_data->add_data_category_row($key, $title, '<span style="color: red">' . $score 
                                . ' / ' . $question->get_weight() . '</span>');                       
                    }
                     
                }
                  
            }
   
        }
        $this->reporting_data->hide_categories();
        return $this->reporting_data;
    }*/

    public function count_data()
    {
        return $this->retrieve_data();
    }

    public function get_available_displaymodes()
    {
        $modes = array();
        $modes[ReportingFormatter :: DISPLAY_TABLE] = Translation :: get('Table');
        return $modes;
    }
    
    public function display_header()
    {
        $html[] = parent :: display_header();
        $score_type = (Request :: post('sel')) ? Request :: post('sel') : Request :: get('sel');
        $types = array(1 => 'Average', 2 => 'Minimum', 3 => 'Maximum', 4 => 'First', 5 => 'Last');
        $html[] = '<script type="text/javascript" src="' . Path :: get_application_web_path() . 'weblcms/resources/javascript/reporting_filter.js"></script>';
        $html[] = '<div class="reporting_header_filter"><form id="select_form" method="post" action ="' . Redirect :: current_url() . '">';
        $html[] = Translation :: get('ScoreToDisplay') . ': ' . '<select id="sel" name="sel">';
        
        foreach($types as $key => $value)
        {
            if($key == $score_type)
            {
                $html[] = '<option value="' . $key . '" selected="selected">' . Translation :: get($value) . '</option>';
            }
            else
            {
                $html[] = '<option value="' . $key . '">' . Translation :: get($value) . '</option>';
            }
        }
        $html[] = '</select></form></div>';
        return implode("\n", $html);
    }
    
    private function get_score($attempts)
    {
        $score_type = (Request :: post('sel')) ? Request :: post('sel') : Request :: get('sel');;
        $score = null;
        
        switch($score_type)
        {
            case self :: SCORE_TYPE_AVG: 
                foreach($attempts as $attempt)
                {
                    $score += $attempt->get_score();
                }
                return round($score / count($attempts), 2);
            case self :: SCORE_TYPE_MIN: 
                foreach($attempts as $attempt)
                {
                    if(is_null($score) || $attempt->get_score() < $score)
                    {
                        $score = $attempt->get_score();
                    }  
                }
                return $score;
            case self :: SCORE_TYPE_MAX: 
                foreach($attempts as $attempt)
                {
                    if(is_null($score) || $attempt->get_score() > $score)
                    {
                        $score = $attempt->get_score();
                    }
                }
                return $score;
            case self :: SCORE_TYPE_FIRST: 
                $date = null;
                foreach($attempts as $attempt)
                {
                    return $attempt->get_score();
                }
            case self :: SCORE_TYPE_LAST: 
                $date = null;
                foreach($attempts as $attempt)
                {
                    $score = $attempt->get_score();
                }
                return $score;
            default: 
                foreach($attempts as $attempt)
                {
                    $score += $attempt->get_score();
                }
                return round($score / count($attempts), 2);
        }
    }
}
?>
