<?php
namespace application\weblcms;

use reporting\Reporting;
use user\VisitTracker;
use reporting\ReportingData;
use reporting\ReportingFormatter;
use common\libraries\PatternMatchCondition;
use reporting\ReportingManager;
use common\libraries\Redirect;
use common\libraries\Application;
use common\libraries\Theme;
use common\libraries\OrCondition;
use common\libraries\Utilities;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Path;
use common\libraries\Translation;
use application\weblcms\tool\reporting\ReportingTool;
use common\libraries\DatetimeUtilities;
use common\libraries\Request;
use repository\RepositoryDataManager;
use repository\ContentObjectRenderer;
use repository\RepositoryManager;
use common\libraries\Session;
use user\UserDataManager;
use user\UserManager;
use application\weblcms\tool\user\UserTool;
use common\libraries\ObjectTableOrder;
use common\libraries\InCondition;
use repository\content_object\assessment\Assessment;
use common\libraries\PlatformSetting;

/**
 *
 * @package application.weblcms.php.reporting.blocks
 *         
 *          Reporting block with an overview of scores of each assessment per
 *          user
 *         
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 */
class WeblcmsAssessmentUserScoresReportingBlock extends WeblcmsToolReportingBlock
{

    public function count_data()
    {
        $reporting_data = new ReportingData();
        $course_id = $this->get_course_id();
        $weblcms_data_manager = WeblcmsDataManager :: get_instance();
        $users = $weblcms_data_manager->retrieve_all_course_users($course_id)->as_array();
        
        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_COURSE_ID, $course_id);
        $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_TOOL, Assessment :: get_type_name());
        $condition = new AndCondition($conditions);
        
        $order_by = new ObjectTableOrder(ContentObjectPublication :: PROPERTY_MODIFIED_DATE, SORT_DESC);
        
        $publication_resultset = $weblcms_data_manager->retrieve_content_object_publications($condition, $order_by);
        
        $publications = array();
        $headings = array();
        $headings[] = Translation :: get('Name');
        while ($publication = $publication_resultset->next_result())
        {
            $publications[] = $publication;
            if ($publication_resultset->size() > 5)
            {
                $headings[] = substr($publication->get_content_object()->get_title(), 0, 14);
            }
            else
            {
                $headings[] = $publication->get_content_object()->get_title();
            }
        }
        
        $reporting_data->set_rows($headings);
        
        foreach ($users as $key => $user)
        {
            $reporting_data->add_category($key);
            $reporting_data->add_data_category_row($key, Translation :: get('Name'), $user->get_fullname());
            
            foreach ($publications as $publication)
            {
                if ($publication_resultset->size() > 5)
                {
                    $title = substr($publication->get_content_object()->get_title(), 0, 14);
                }
                else
                {
                    $title = $publication->get_content_object()->get_title();
                }
                $attempts_tracker = new WeblcmsAssessmentAttemptsTracker();
                $conditions = array();
                $conditions[] = new EqualityCondition(WeblcmsAssessmentAttemptsTracker :: PROPERTY_ASSESSMENT_ID, $publication->get_id());
                $conditions[] = new EqualityCondition(WeblcmsAssessmentAttemptsTracker :: PROPERTY_USER_ID, $user->get_id());
                $condition = new AndCondition($conditions);
                $attempts_by_user = $attempts_tracker->retrieve_tracker_items($condition);
                
                if (count($attempts_by_user) == 0)
                {
                    if ($weblcms_data_manager->is_publication_target_user($user->get_id(), $publication->get_id(), $course_id))
                    {
                        $reporting_data->add_data_category_row($key, $title, null);
                        continue;
                    }
                    
                    $reporting_data->add_data_category_row($key, $title, 'X');
                    continue;
                }
                
                $score = $this->get_score($attempts_by_user);
                if ($score < PlatformSetting :: get('passing_percentage'))
                {
                    $score = '<span style="color:red">' . $score . '</span>';
                }
                else
                {
                    $score = '<span style="color:green">' . $score . '</span>';
                }
                $reporting_data->add_data_category_row($key, $title, $score);
            }
        
        }
        $reporting_data->hide_categories();
        return $reporting_data;
    }

    private function get_score($attempts)
    {
        $score_type = (Request :: post('sel')) ? Request :: post('sel') : Request :: get('sel');
        if (is_null($score_type))
        {
            $score_type = Request :: get('sel');
        }
        $score = null;
        
        switch ($score_type)
        {
            case self :: SCORE_TYPE_AVG :
                foreach ($attempts as $attempt)
                {
                    $score += $attempt->get_total_score();
                }
                return number_format($score / count($attempts), 1);
            case self :: SCORE_TYPE_MIN :
                foreach ($attempts as $attempt)
                {
                    if (is_null($score) || $attempt->get_total_score() < $score)
                    {
                        $score = $attempt->get_total_score();
                    }
                }
                return $score;
            case self :: SCORE_TYPE_MAX :
                foreach ($attempts as $attempt)
                {
                    if (is_null($score) || $attempt->get_total_score() > $score)
                    {
                        $score = $attempt->get_total_score();
                    }
                }
                return $score;
            case self :: SCORE_TYPE_FIRST :
                $date = null;
                foreach ($attempts as $attempt)
                {
                    if (is_null($score) || $attempt->get_date() < $date)
                    {
                        $date = $attempt->get_date();
                        $score = $attempt->get_total_score();
                    }
                }
                return $score;
            case self :: SCORE_TYPE_LAST :
                $date = null;
                foreach ($attempts as $attempt)
                {
                    if (is_null($score) || $attempt->get_date() > $date)
                    {
                        $date = $attempt->get_date();
                        $score = $attempt->get_total_score();
                    }
                }
                return $score;
            default :
                foreach ($attempts as $attempt)
                {
                    $score += $attempt->get_total_score();
                }
                return number_format($score / count($attempts), 1);
        }
    
    }

    public function retrieve_data()
    {
        return $this->count_data();
    }

    public function get_available_displaymodes()
    {
        $modes = array();
        $modes[ReportingFormatter :: DISPLAY_TABLE] = Translation :: get('Table');
        return $modes;
    }

}
?>
