<?php
namespace application\weblcms;

use application\weblcms\tool\assignment\AssignmentTool;
use common\libraries\Application;
use common\libraries\EqualityCondition;
use common\libraries\InCondition;
use common\libraries\PlatformSetting;
use common\libraries\Translation;
use common\libraries\Path;
use common\libraries\Request;
use common\libraries\DatetimeUtilities;
use common\libraries\PatternMatchCondition;
use common\libraries\Redirect;
use common\libraries\Utilities;
use reporting\ReportingFormatter;
use reporting\ReportingData;
use repository\content_object\assignment\Assignment;
use user\VisitTracker;

/**
 *
 * @package application.weblcms.php.reporting.blocks
 *         
 *          Reporting block displaying information about the assigment and
 *          access details
 *         
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 * @author Anthony Hurst (Hogeschool Gent)
 */
class WeblcmsAssignmentInformationReportingBlock extends WeblcmsAssignmentReportingManager
{
    const STATISTICS_COUNT_SUBMITTED = 0;
    const STATISTICS_COUNT_LATE = 1;
    
    const STATISTICS_AVG_SCORE = 0;
    const STATISTICS_MIN_SCORE = 1;
    const STATISTICS_MAX_SCORE = 2;
    
    private static $COLUMN_DETAILS;
    private static $ROW_DESCRIPTION;
    private static $ROW_NUMBER_OF_SUBMITTERS_SUBMITTED;
    private static $ROW_NUMBER_OF_SUBMITTERS_LATE;
    private static $ROW_SCORE_AVERAGE;
    private static $ROW_SCORE_MAXIMUM;
    private static $ROW_SCORE_MINIMUM;
    private static $ROW_TITLE;
    
    private $assignment;

    public function __construct($parent, $vertical = false)
    {
        $this->assignment = WeblcmsDataManager :: get_instance()->retrieve_content_object_publication($this->get_publication_id())->get_content_object();
        
        self :: $COLUMN_DETAILS = Translation :: get('Details');
        self :: $ROW_TITLE = Translation :: get('Title');
        self :: $ROW_DESCRIPTION = Translation :: get('Description');
        if (! $this->assignment->get_allow_group_submissions())
        {
            self :: $ROW_NUMBER_OF_SUBMITTERS_SUBMITTED = Translation :: get('NumberOfUsersSubmitted');
            self :: $ROW_NUMBER_OF_SUBMITTERS_LATE = Translation :: get('NumberOfUsersLate');
        }
        else
        {
            self :: $ROW_NUMBER_OF_SUBMITTERS_SUBMITTED = Translation :: get('NumberOfGroupsSubmitted');
            self :: $ROW_NUMBER_OF_SUBMITTERS_LATE = Translation :: get('NumberOfGroupsLate');
        }
        self :: $ROW_SCORE_AVERAGE = Translation :: get('AverageScore');
        self :: $ROW_SCORE_MINIMUM = Translation :: get('MinimumScore');
        self :: $ROW_SCORE_MAXIMUM = Translation :: get('MaximumScore');
        parent :: __construct($parent, $vertical);
    }

    public function count_data()
    {
        $reporting_data = new ReportingData();
        $reporting_data->set_categories(array(self :: $ROW_TITLE, self :: $ROW_DESCRIPTION, 
                self :: $ROW_NUMBER_OF_SUBMITTERS_SUBMITTED, self :: $ROW_NUMBER_OF_SUBMITTERS_LATE, 
                self :: $ROW_SCORE_AVERAGE, self :: $ROW_SCORE_MINIMUM, self :: $ROW_SCORE_MAXIMUM));
        
        $params = array();
        $params[Application :: PARAM_ACTION] = WeblcmsManager :: ACTION_VIEW_COURSE;
        $params[Application :: PARAM_APPLICATION] = WeblcmsManager :: APPLICATION_NAME;
        $params[WeblcmsManager :: PARAM_COURSE] = $this->get_course_id();
        $params[WeblcmsManager :: PARAM_TOOL] = Assignment :: get_type_name();
        $params[WeblcmsManager :: PARAM_PUBLICATION] = $this->get_publication_id();
        $params[WeblcmsManager :: PARAM_TOOL_ACTION] = AssignmentTool :: ACTION_BROWSE_SUBMITTERS;
        $url_title = Redirect :: get_url($params);
        
        $number_of_submitters = $this->count_submitters($this->assignment->get_allow_group_submissions());
        $submitter_statistics = $this->compile_submitter_statistics($this->assignment);
        $score_statistics = $this->compile_score_statistics();
        
        $reporting_data->set_rows(array(self :: $COLUMN_DETAILS));
        $reporting_data->add_data_category_row(self :: $ROW_TITLE, self :: $COLUMN_DETAILS, '<a href="' . $url_title . '">' . $this->assignment->get_title() . '</a>');
        $reporting_data->add_data_category_row(self :: $ROW_DESCRIPTION, self :: $COLUMN_DETAILS, $this->assignment->get_description());
        $reporting_data->add_data_category_row(self :: $ROW_NUMBER_OF_SUBMITTERS_SUBMITTED, self :: $COLUMN_DETAILS, $submitter_statistics[self :: STATISTICS_COUNT_SUBMITTED] . '/' . $number_of_submitters);
        $reporting_data->add_data_category_row(self :: $ROW_NUMBER_OF_SUBMITTERS_LATE, self :: $COLUMN_DETAILS, $submitter_statistics[self :: STATISTICS_COUNT_LATE] . '/' . $number_of_submitters);
        $reporting_data->add_data_category_row(self :: $ROW_SCORE_AVERAGE, self :: $COLUMN_DETAILS, $this->format_score_html($score_statistics[self :: STATISTICS_AVG_SCORE]));
        $reporting_data->add_data_category_row(self :: $ROW_SCORE_MINIMUM, self :: $COLUMN_DETAILS, $this->format_score_html($score_statistics[self :: STATISTICS_MIN_SCORE]));
        $reporting_data->add_data_category_row(self :: $ROW_SCORE_MAXIMUM, self :: $ROW_DESCRIPTION, $this->format_score_html($score_statistics[self :: STATISTICS_MAX_SCORE]));
        
        return $reporting_data;
    }

    /**
     * Counts the number of submitters registered for the assignment.
     * 
     * @param $is_group_assignment boolean
     *            Whether the assignment is a group assignment.
     * @return int the number of submitters registered for the assignment.
     */
    private function count_submitters($is_group_assignment)
    {
        if (! $is_group_assignment)
        {
            return count(WeblcmsDataManager :: get_instance()->retrieve_publication_target_users(Request :: get(Tool :: PARAM_PUBLICATION_ID))->as_array());
        }
        else
        {
            $number_of_submitters = count(WeblcmsDataManager :: get_instance()->retrieve_publication_target_course_groups(Request :: get(Tool :: PARAM_PUBLICATION_ID))->as_array());
            $number_of_submitters += count(WeblcmsDataManager :: get_instance()->retrieve_publication_target_platform_groups(Request :: get(Tool :: PARAM_PUBLICATION_ID))->as_array());
            return $number_of_submitters;
        }
    }

    /**
     * Compiles and returns general submitter statistics about the current
     * publication of the assignment.
     * 
     * @param $assignment Assignment
     *            The assignment for which the statistics are needed.
     * @return array() The general statistics in the format
     *         {$number_submitters_submitted, $number_submitters_late}.
     */
    private function compile_submitter_statistics($assignment)
    {
        $submission_trackers_array = array();
        
        if (! $assignment->get_allow_group_submissions())
        {
            $submission_trackers_array[] = WeblcmsDataManager :: get_instance()->retrieve_submissions_by_submitter_type(Request :: get(Tool :: PARAM_PUBLICATION_ID), WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_USER)->as_array();
        }
        else
        {
            $submission_trackers_array[] = WeblcmsDataManager :: get_instance()->retrieve_submissions_by_submitter_type(Request :: get(Tool :: PARAM_PUBLICATION_ID), WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_COURSE_GROUP)->as_array();
            $submission_trackers_array[] = WeblcmsDataManager :: get_instance()->retrieve_submissions_by_submitter_type(Request :: get(Tool :: PARAM_PUBLICATION_ID), WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_PLATFORM_GROUP)->as_array();
        }
        
        $count_submitted = count($submission_trackers_array, 1) - count($submission_trackers_array);
        
        $count_late = 0;
        
        foreach ($submission_trackers_array as $submission_trackers)
        {
            foreach ($submission_trackers as $submission_tracker)
            {
                if ($submission_tracker->get_optional_property('last_date') > $assignment->get_end_time())
                {
                    $count_late ++;
                }
            }
        }
        
        return array(self :: STATISTICS_COUNT_SUBMITTED => $count_submitted, 
                self :: STATISTICS_COUNT_LATE => $count_late);
    }

    /**
     * Compiles and returns the general score statistics for the current
     * publication.
     * 
     * @return array() The general score statistics in the format
     *         {$minimum_score, $average_score, $maximum_score}.
     */
    private function compile_score_statistics()
    {
        $submissions_condition = new EqualityCondition(WeblcmsAssignmentSubmissionsTracker :: PROPERTY_PUBLICATION_ID, Request :: get(Tool :: PARAM_PUBLICATION_ID));
        $submission_trackers = WeblcmsAssignmentSubmissionsTracker :: get_data(WeblcmsAssignmentSubmissionsTracker :: CLASS_NAME, WeblcmsManager :: APPLICATION_NAME, $submissions_condition)->as_array();
        
        $submission_ids = array();
        foreach ($submission_trackers as $submission_tracker)
        {
            $submission_ids[] = $submission_tracker->get_id();
        }
        
        $score_condition = new InCondition(WeblcmsSubmissionScoreTracker :: PROPERTY_SUBMISSION_ID, $submission_ids);
        $score_trackers = WeblcmsSubmissionScoreTracker :: get_data(WeblcmsSubmissionScoreTracker :: CLASS_NAME, WeblcmsManager :: APPLICATION_NAME, $score_condition)->as_array();
        
        $avg_score = $this->get_avg_score($score_trackers);
        $min_score = $this->get_min_score($score_trackers);
        $max_score = $this->get_max_score($score_trackers);
        
        return array(self :: STATISTICS_AVG_SCORE => $avg_score, self :: STATISTICS_MIN_SCORE => $min_score, 
                self :: STATISTICS_MAX_SCORE => $max_score);
    }

    public function retrieve_data()
    {
        return $this->count_data();
    }

    public function get_available_displaymodes()
    {
        $modes = array();
        $modes[ReportingFormatter :: DISPLAY_TABLE] = Translation :: get('Table');
        
        return $modes;
    }
}
?>