<?php
namespace application\weblcms;

use application\weblcms\tool\assignment\AssignmentTool;
use common\libraries\Application;
use common\libraries\DataClassRetrievesParameters;
use common\libraries\EqualityCondition;
use common\libraries\Redirect;
use common\libraries\Translation;
use group\GroupDataManager;
use reporting\ReportingData;
use reporting\ReportingFormatter;
use repository\content_object\assignment\Assignment;
use user\UserDataManager;
use user\User;

/**
 * Description of weblcms_assignment_submissions_reporting_block
 *
 * @author Anthony Hurst (Hogeschool Gent)
 */
abstract class WeblcmsAssignmentSubmissionsReportingBlock extends WeblcmsAssignmentReportingManager
{
    private static $COLUMN_DATE_SUBMITTED;
    private static $COLUMN_INDIVIDUAL_SUBMITTER_ID;
    private static $COLUMN_INDIVIDUAL_SUBMITTER_NAME;
    private static $COLUMN_IP_ADDRESS;
    private static $COLUMN_NUMBER_OF_FEEDBACKS;
    private static $COLUMN_SCORE;
    private static $COLUMN_SUBMITTER_ID;
    private $COLUMN_SUBMITTER_NAME;
    private static $COLUMN_TITLE;
    
    const SUBMITTER_INFORMATION_NAME = 0;
    const SUBMITTER_INFORMATION_CODE = 1;

    /**
     * Creates a submissions overview block depending on the current assignmnet
     * publication's allow group submissions setting.
     * 
     * @param $parent type           
     * @param $vertical type           
     * @return \application\weblcms\WeblcmsAssignmentSubmissionsGroupReportingBlock \application\weblcms\WeblcmsAssignmentSubmissionsUserReportingBlock
     */
    public static function get_instance($parent, $vertical)
    {
        if (WeblcmsDataManager :: get_instance()->retrieve_content_object_publication(self :: get_publication_id())->get_content_object()->get_allow_group_submissions())
        {
            return new WeblcmsAssignmentSubmissionsGroupReportingBlock($parent, $vertical);
        }
        return new WeblcmsAssignmentSubmissionsUserReportingBlock($parent, $vertical);
    }

    public function __construct($parent, $vertical)
    {
        self :: $COLUMN_TITLE = Translation :: get('Title');
        self :: $COLUMN_SUBMITTER_ID = Translation :: get('UserCode'); // Only
                                                                     // visible
                                                                     // in
                                                                     // individual
                                                                     // assignments.
        $this->COLUMN_SUBMITTER_NAME = $this->define_column_submitter_name_title();
        self :: $COLUMN_INDIVIDUAL_SUBMITTER_ID = Translation :: get('UserCode'); // Only
                                                                                // visible
                                                                                // in
                                                                                // group
                                                                                // assignments.
        self :: $COLUMN_INDIVIDUAL_SUBMITTER_NAME = Translation :: get('SubmittedBy'); // Only
                                                                                     // visible
                                                                                     // in
                                                                                     // group
                                                                                     // assignments.
        self :: $COLUMN_DATE_SUBMITTED = Translation :: get('DateSubmitted');
        self :: $COLUMN_SCORE = Translation :: get('Score');
        self :: $COLUMN_NUMBER_OF_FEEDBACKS = Translation :: get('NumberOfFeedbacks');
        self :: $COLUMN_IP_ADDRESS = Translation :: get('IpAddress');
        parent :: __construct($parent, $vertical);
    }

    /**
     * Defines the title of the submitter name column.
     * 
     * @return string The title of the submitter name column.
     */
    protected abstract function define_column_submitter_name_title();

    public function count_data()
    {
        $assignment = WeblcmsDataManager :: get_instance()->retrieve_content_object_publication($this->get_publication_id())->get_content_object();
        $reporting_data = new ReportingData();
        $reporting_data_headers = array(self :: $COLUMN_TITLE, $this->COLUMN_SUBMITTER_NAME, 
                self :: $COLUMN_DATE_SUBMITTED, self :: $COLUMN_SCORE, self :: $COLUMN_NUMBER_OF_FEEDBACKS, 
                self :: $COLUMN_IP_ADDRESS);
        if ($assignment->get_allow_group_submissions())
        {
            array_splice($reporting_data_headers, 2, 0, array(self :: $COLUMN_INDIVIDUAL_SUBMITTER_ID, 
                    self :: $COLUMN_INDIVIDUAL_SUBMITTER_NAME));
        }
        else
        {
            array_splice($reporting_data_headers, 1, 0, array(self :: $COLUMN_SUBMITTER_ID));
        }
        $reporting_data->set_rows($reporting_data_headers);
        
        $assignment = WeblcmsDataManager :: get_instance()->retrieve_content_object_publication($this->get_publication_id())->get_content_object();
        
        $condition = new EqualityCondition(WeblcmsAssignmentSubmissionsTracker :: PROPERTY_PUBLICATION_ID, $this->get_publication_id());
        $submission_trackers = WeblcmsAssignmentSubmissionsTracker :: get_data(WeblcmsAssignmentSubmissionsTracker :: CLASS_NAME, WeblcmsManager :: APPLICATION_NAME, $condition)->as_array();
        
        foreach ($submission_trackers as $key => $submission_tracker)
        {
            // Title column.
            $submission_title_link = $this->generate_submission_title_link($submission_tracker);
            
            // Submitter code column
            $submitter_code = $this->get_submitter_code($submission_tracker->get_submitter_type(), $submission_tracker->get_submitter_id());
            
            // Submitter name column.
            $submitter_name_link = $this->generate_submitter_name_link($submission_tracker->get_submitter_type(), $submission_tracker->get_submitter_id());
            
            // Individual submitter column.
            if ($assignment->get_allow_group_submissions())
            {
                $individual_submitter_id = $this->get_submitter_code(WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_USER, $submission_tracker->get_user_id());
                $individual_submitter_name_link = $this->generate_user_name_link($submission_tracker->get_user_id());
            }
            
            // Date submitted column.
            $date_submitted = $this->format_date_html($submission_tracker->get_date_submitted(), $assignment->get_end_time());
            
            // Score column.
            $score = null;
            $condition = new EqualityCondition(WeblcmsSubmissionScoreTracker :: PROPERTY_SUBMISSION_ID, $submission_tracker->get_id());
            $score_trackers = WeblcmsSubmissionScoreTracker :: get_data(WeblcmsSubmissionScoreTracker :: CLASS_NAME, WeblcmsManager :: APPLICATION_NAME, $condition)->as_array();
            if ($score_trackers[0])
            {
                $score = $this->format_score_html($score_trackers[0]->get_score());
            }
            
            // Number of feedbacks column.
            $condition = new EqualityCondition(WeblcmsSubmissionFeedbackTracker :: PROPERTY_SUBMISSION_ID, $submission_tracker->get_id());
            $number_feedbacks = WeblcmsSubmissionFeedbackTracker :: count_data(WeblcmsSubmissionFeedbackTracker :: CLASS_NAME, WeblcmsManager :: APPLICATION_NAME, $condition);
            
            // IP address column.
            $ip_address = $submission_tracker->get_ip_address();
            
            $reporting_data->add_category($key);
            $reporting_data->add_data_category_row($key, self :: $COLUMN_TITLE, $submission_title_link);
            
            $reporting_data->add_data_category_row($key, $this->COLUMN_SUBMITTER_NAME, $submitter_name_link);
            if ($assignment->get_allow_group_submissions())
            {
                $reporting_data->add_data_category_row($key, self :: $COLUMN_INDIVIDUAL_SUBMITTER_ID, $individual_submitter_id);
                $reporting_data->add_data_category_row($key, self :: $COLUMN_INDIVIDUAL_SUBMITTER_NAME, $individual_submitter_name_link);
            }
            else
            {
                $reporting_data->add_data_category_row($key, self :: $COLUMN_SUBMITTER_ID, $submitter_code);
            }
            $reporting_data->add_data_category_row($key, self :: $COLUMN_DATE_SUBMITTED, $date_submitted);
            $reporting_data->add_data_category_row($key, self :: $COLUMN_SCORE, $score);
            $reporting_data->add_data_category_row($key, self :: $COLUMN_NUMBER_OF_FEEDBACKS, $number_feedbacks);
            $reporting_data->add_data_category_row($key, self :: $COLUMN_IP_ADDRESS, $ip_address);
        }
        $reporting_data->hide_categories();
        return $reporting_data;
    }

    /**
     * Obtains the official code of the submitter given (only users).
     * 
     * @param $submitter_type int
     *            The type of the submitter
     *            (WeblcmsAssignmentSubmissionsTracker::SUBMITTER_TYPE_...).
     * @param $submitter_id int
     *            The id of the submitter.
     * @return string The official code of the submitter (null for groups).
     */
    protected function get_submitter_code($submitter_type, $submitter_id)
    {
        switch ($submitter_type)
        {
            case WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_COURSE_GROUP :
            case WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_PLATFORM_GROUP :
                return null;
            case WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_USER :
                return UserDataManager :: get_instance()->retrieve_user($submitter_id)->get_official_code();
        }
    }

    public function get_available_displaymodes()
    {
        $modes = array();
        $modes[ReportingFormatter :: DISPLAY_TABLE] = Translation :: get('Table');
        return $modes;
    }

    public function retrieve_data()
    {
        return $this->count_data();
    }
}

?>
