<?php
namespace application\weblcms;

use application\weblcms\course\Course;
use application\weblcms\course\CourseDataManager;
use common\libraries\Application;
use common\libraries\Request;
use common\libraries\DatetimeUtilities;
use common\libraries\Utilities;
use common\libraries\Redirect;
use common\libraries\Translation;
use common\libraries\Path;
use common\libraries\PatternMatchCondition;
use reporting\ReportingFormatter;
use reporting\ReportingData;
use common\libraries\EqualityCondition;
use user\UserDataManager;
use common\libraries\AndCondition;
use application\weblcms\tool\assignment\AssignmentTool;
use repository\content_object\assignment\Assignment;
use group\GroupDataManager;
use common\libraries\InCondition;

/**
 *
 * @package application.weblcms.php.reporting.blocks
 *         
 *          Reporting block displaying information about the assignment and the
 *          user
 *         
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 * @author Anthony Hurst (Hogeschool Gent)
 */
abstract class WeblcmsAssignmentSubmitterInformationReportingBlock extends WeblcmsAssignmentReportingManager
{
    private static $COLUMN_DETAILS;
    private static $ROW_ASSIGNMENT;
    private static $ROW_NUMBER_FEEDBACK;
    private static $ROW_NUMBER_SUBMISSIONS_LATE;
    private static $ROW_SCORE_AVERAGE;
    private static $ROW_SCORE_MAXIMUM;
    private static $ROW_SCORE_MINIMUM;
    private $ROW_SUBMITTER;

    /**
     * Instatiates the column headers.
     * 
     * @param $parent type
     *            Pass-through variable. Please refer to parent class(es) for
     *            more details.
     *            &param type $vertical Pass-through variable. Please refer to
     *            parent class(es) for more details.
     */
    public function __construct($parent, $vertical)
    {
        self :: $COLUMN_DETAILS = Translation :: get('Details');
        
        $this->ROW_SUBMITTER = $this->define_row_submitter_title();
        self :: $ROW_ASSIGNMENT = Translation :: get('AssignmentTitle');
        self :: $ROW_NUMBER_SUBMISSIONS_LATE = Translation :: get('NumberOfSubmissionsLate');
        self :: $ROW_NUMBER_FEEDBACK = Translation :: get('NumberOfSubmissionsFeedbacks');
        self :: $ROW_SCORE_AVERAGE = Translation :: get('AverageScore');
        self :: $ROW_SCORE_MINIMUM = Translation :: get('MinimumScore');
        self :: $ROW_SCORE_MAXIMUM = Translation :: get('MaximumScore');
        parent :: __construct($parent, $vertical);
    }

    /**
     * Defines the title of the submitter row.
     * 
     * @return string The title of the row.
     */
    protected abstract function define_row_submitter_title();

    public function count_data()
    {
        $reporting_data = new ReportingData();
        
        $reporting_data->set_categories(array($this->ROW_SUBMITTER, self :: $ROW_ASSIGNMENT, 
                self :: $ROW_NUMBER_SUBMISSIONS_LATE, self :: $ROW_NUMBER_FEEDBACK, self :: $ROW_SCORE_MINIMUM, 
                self :: $ROW_SCORE_AVERAGE, self :: $ROW_SCORE_MAXIMUM));
        $reporting_data->set_rows(array(self :: $COLUMN_DETAILS));
        
        $assignment = WeblcmsDataManager :: get_instance()->retrieve_content_object_publication($this->get_publication_id())->get_content_object();
        
        $title = $this->generate_assignment_name_link($this->get_publication_id());
        
        $submitter = $this->generate_submitter_name_link($this->get_submitter_type(), $this->get_target_id());
        
        $conditions = array();
        $conditions[] = new EqualityCondition(WeblcmsAssignmentSubmissionsTracker :: PROPERTY_PUBLICATION_ID, $this->get_publication_id());
        $conditions[] = new EqualityCondition(WeblcmsAssignmentSubmissionsTracker :: PROPERTY_SUBMITTER_TYPE, $this->get_submitter_type());
        $conditions[] = new EqualityCondition(WeblcmsAssignmentSubmissionsTracker :: PROPERTY_SUBMITTER_ID, $this->get_target_id());
        $condition = new AndCondition($conditions);
        $submission_trackers = WeblcmsAssignmentSubmissionsTracker :: get_data(WeblcmsAssignmentSubmissionsTracker :: CLASS_NAME, WeblcmsManager :: APPLICATION_NAME, $condition)->as_array();
        
        $count_submissions_late = 0;
        $count_submissions_feedback = 0;
        $submission_ids = array();
        foreach ($submission_trackers as $submission_tracker)
        {
            $submission_ids[] = $submission_tracker->get_id();
            if ($submission_tracker->get_date_submitted() > $assignment->get_end_time())
            {
                $count_submissions_late ++;
            }
            $condition = new EqualityCondition(WeblcmsSubmissionFeedbackTracker :: PROPERTY_SUBMISSION_ID, $submission_tracker->get_id());
            if (WeblcmsSubmissionFeedbackTracker :: count_data(WeblcmsSubmissionFeedbackTracker :: CLASS_NAME, WeblcmsManager :: APPLICATION_NAME, $condition) > 0)
            {
                $count_submissions_feedback ++;
            }
        }
        
        $condition = new InCondition(WeblcmsSubmissionScoreTracker :: PROPERTY_SUBMISSION_ID, $submission_ids);
        $score_trackers = WeblcmsSubmissionScoreTracker :: get_data(WeblcmsSubmissionScoreTracker :: CLASS_NAME, WeblcmsManager :: APPLICATION_NAME, $condition)->as_array();
        
        $minimum_score = $this->format_score_html($this->get_min_score($score_trackers));
        $average_score = $this->format_score_html($this->get_avg_score($score_trackers));
        $maximum_score = $this->format_score_html($this->get_max_score($score_trackers));
        
        $reporting_data->add_data_category_row($this->ROW_SUBMITTER, self :: $COLUMN_DETAILS, $submitter);
        $reporting_data->add_data_category_row(self :: $ROW_ASSIGNMENT, self :: $COLUMN_DETAILS, $title);
        $reporting_data->add_data_category_row(self :: $ROW_NUMBER_SUBMISSIONS_LATE, self :: $COLUMN_DETAILS, $count_submissions_late . '/' . count($submission_trackers));
        $reporting_data->add_data_category_row(self :: $ROW_NUMBER_FEEDBACK, self :: $COLUMN_DETAILS, $count_submissions_feedback . '/' . count($submission_trackers));
        $reporting_data->add_data_category_row(self :: $ROW_SCORE_AVERAGE, self :: $COLUMN_DETAILS, $average_score);
        $reporting_data->add_data_category_row(self :: $ROW_SCORE_MINIMUM, self :: $COLUMN_DETAILS, $minimum_score);
        $reporting_data->add_data_category_row(self :: $ROW_SCORE_MAXIMUM, self :: $COLUMN_DETAILS, $maximum_score);
        
        return $reporting_data;
    }

    public function retrieve_data()
    {
        return $this->count_data();
    }

    public function get_available_displaymodes()
    {
        $modes = array();
        $modes[ReportingFormatter :: DISPLAY_TABLE] = Translation :: get('Table');
        return $modes;
    }
}
?>