<?php

namespace application\weblcms;

use common\libraries\Request;
use reporting\ReportingTemplate;
use application\weblcms\tool\assignment\AssignmentTool;
use common\libraries\Breadcrumb;
use common\libraries\Translation;
use user\UserDataManager;
use group\GroupDataManager;
use reporting\Reporting;
use common\libraries\Utilities;
use reporting\ReportingManager;
use common\libraries\EqualityCondition;
use repository\content_object\assignment\Assignment;
use common\libraries\AndCondition;
use common\libraries\ObjectTableOrder;
use common\libraries\Path;
use common\libraries\Redirect;

/**
 *
 * @package application.weblcms.php.reporting.templates
 *         
 *          Reporting template with an overview of scores of each assignment per
 *          user, course group and platform group
 *         
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 * @author Bert De Clercq (Hogeschool Gent)
 */
class AssignmentScoresReportingTemplate extends ReportingTemplate
{
    const LEGEND_TYPE_GROUP = 'group';

    function __construct($parent)
    {
        parent :: __construct($parent);
        
        $this->init_parameters();
        
        $custom_breadcrumbs = array();
        $custom_breadcrumbs[] = new Breadcrumb($this->get_url(), Translation :: get('AssignmentScores'));
        $this->set_custom_breadcrumb_trail($custom_breadcrumbs);
        
        // Calculate number of assignments
        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_COURSE_ID, $this->course_id);
        $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_TOOL, Assignment :: get_type_name());
        $condition = new AndCondition($conditions);
        
        $order_by = new ObjectTableOrder(ContentObjectPublication :: PROPERTY_MODIFIED_DATE, SORT_DESC);
        
        $publications = WeblcmsDataManager :: get_instance()->retrieve_content_object_publications($condition, $order_by);
        
        $this->size = $publications->size();
        
        $publications_array = $publications->as_array();
        
        foreach ($publications_array as $publication)
        {
            $this->th_titles[] = $publication->get_content_object()->get_title();
        }
        
        $this->add_reporting_block($this->get_assignment_user_scores());
        $this->add_reporting_block($this->get_assignment_course_group_scores());
        $this->add_reporting_block($this->get_assignment_group_scores());
    }

    function display_context()
    {
        // publicatie, content_object, application ...
    }

    private function init_parameters()
    {
        $this->course_id = Request :: get(WeblcmsManager :: PARAM_COURSE);
        if ($this->course_id)
        {
            $this->set_parameter(WeblcmsManager :: PARAM_COURSE, $this->course_id);
        }
        $sel = (Request :: post('sel')) ? Request :: post('sel') : Request :: get('sel');
        if ($sel)
        {
            $this->set_parameter('sel', $sel);
        }
    }

    function get_application()
    {
        return WeblcmsManager :: APPLICATION_NAME;
    }

    function get_assignment_user_scores()
    {
        $assignment_scores_block = new WeblcmsAssignmentUserScoresReportingBlock($this, true);
        $assignment_scores_block->set_sorting(0);
        $assignment_scores_block->set_legend($this->get_legend());
        return $assignment_scores_block;
    }

    function get_assignment_course_group_scores()
    {
        $assignment_scores_block = new WeblcmsAssignmentCourseGroupScoresReportingBlock($this, true);
        $assignment_scores_block->set_sorting(0);
        $assignment_scores_block->set_legend($this->get_legend(self :: LEGEND_TYPE_GROUP));
        return $assignment_scores_block;
    }

    function get_assignment_group_scores()
    {
        $assignment_scores_block = new WeblcmsAssignmentPlatformGroupScoresReportingBlock($this, true);
        $assignment_scores_block->set_sorting(0);
        $assignment_scores_block->set_legend($this->get_legend(self :: LEGEND_TYPE_GROUP));
        return $assignment_scores_block;
    }

    /**
     * Returns the legend for the table.
     * 
     * @param $type string
     *            The type for which to get the legend
     * @return array
     */
    protected function get_legend($type = null)
    {
        $html = array();
        
        $html[] = '<div style="float:left;">';
        $html[] = 'X&nbsp;<br />';
        $html[] = '-&nbsp;<br />';
        $html[] = '?&nbsp;<br />';
        $html[] = '</div>';
        
        $html[] = '<div style="float:left;">';
        if (! $type)
        {
            $html[] = ' = ' . Translation :: get('NotPublishedForUser') . '<br />';
            $html[] = ' = ' . Translation :: get('NoSubmissionsByUser') . '<br />';
        }
        else
        {
            $html[] = ' = ' . Translation :: get('NotPublishedForGroup') . '<br />';
            $html[] = ' = ' . Translation :: get('NoSubmissionsByGroup') . '<br />';
        }
        $html[] = ' = ' . Translation :: get('NoScoreGivenByCourseAdministrator') . '<br />';
        $html[] = '</div>';
        
        return $html;
    }

    public function display_header()
    {
        $html[] = parent :: display_header();
        $score_type = (Request :: post('sel')) ? Request :: post('sel') : Request :: get('sel');
        $types = array(1 => 'Average', 2 => 'Minimum', 3 => 'Maximum', 4 => 'First', 5 => 'Last');
        $html[] = '<script type="text/javascript" src="' . Path :: get_application_web_path() . 'weblcms/resources/javascript/reporting_filter.js"></script>';
        $html[] = '<form id="select_form" method="post" action ="' . Redirect :: current_url() . '">';
        $html[] = Translation :: get('ScoreToDisplay') . ': ' . '<select id="sel" name="sel">';
        
        foreach ($types as $key => $value)
        {
            if ($key == $score_type)
            {
                $html[] = '<option value="' . $key . '" selected="selected">' . Translation :: get($value) . '</option>';
            }
            else
            {
                $html[] = '<option value="' . $key . '">' . Translation :: get($value) . '</option>';
            }
        }
        
        $html[] = '</select></form><br />';
        return implode("\n", $html);
    }
}

?>