<?php
namespace application\weblcms\tool\assessment;

use application\weblcms\ContentObjectPublication;
use application\weblcms\WeblcmsAssessmentAttemptsTracker;
use application\weblcms\WeblcmsDataManager;
use application\weblcms\WeblcmsQuestionAttemptsTracker;
use application\weblcms\WeblcmsRights;
use common\libraries\AndCondition;
use common\libraries\DatetimeUtilities;
use common\libraries\EqualityCondition;
use common\libraries\Export;
use common\libraries\InCondition;
use common\libraries\ObjectTableOrder;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;
use repository\content_object\assessment_match_numeric_question\AssessmentMatchNumericQuestion;
use repository\content_object\assessment_match_text_question\AssessmentMatchTextQuestion;
use repository\content_object\assessment_matching_question\AssessmentMatchingQuestion;
use repository\content_object\assessment_matrix_question\AssessmentMatrixQuestion;
use repository\content_object\assessment_multiple_choice_question\AssessmentMultipleChoiceQuestion;
use repository\content_object\assessment_open_question\AssessmentOpenQuestion;
use repository\content_object\assessment_rating_question\AssessmentRatingQuestion;
use repository\content_object\assessment_select_question\AssessmentSelectQuestion;
use repository\content_object\fill_in_blanks_question\FillInBlanksQuestion;
use repository\content_object\hotspot_question\HotspotQuestion;
use repository\content_object\match_question\MatchQuestion;
use repository\content_object\ordering_question\OrderingQuestion;
use repository\ComplexContentObjectItem;
use repository\ContentObject;
use repository\RepositoryDataManager;
use tracking\TrackingDataManager;
use user\User;
use user\UserDataManager;

/**
 * Description of raw_export_results
 *
 * @author Anthony Hurst (Hogeschool Gent)
 */
class AssessmentToolRawExportResultsComponent extends AssessmentTool
{
    const ASCII_CHAR_NEW_LINE = 10;
    const ASCII_CHAR_CARRIAGE_RETURN = 13;
    
    const EXPORT_FORMAT_CSV = 'csv';
    
    const PROPERTY_EXPORT_TYPE = 'export';
    const PROPERTY_PUBLICATION_WEIGHT = 'publication_weight';
    const PROPERTY_QUESTION_TITLE = 'question_title';
    const PROPERTY_QUESTION_TYPE = 'question_type';
    const PROPERTY_QUESTION_WEIGHT = 'question_weight';
    const PROPERTY_USER_FULLNAME = 'user_fullname';
    
    public function run()
    {
        if (! $this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            Display :: not_allowed();
            return;
        }
        
        //Retrieves the export type from the url.
        $export_format = self::EXPORT_FORMAT_CSV;
        if(Request::get(self::PROPERTY_EXPORT_TYPE))
        {
            $export_format = Request::get(self::PROPERTY_EXPORT_TYPE);
        }
        
        $wdm = WeblcmsDataManager::get_instance();
        $rdm = RepositoryDataManager::get_instance();
        
        $condition = $this->define_conditions();
        $order_by = $this->define_order_by();
        $basic_export = $wdm->retrieve_assessment_raw_export_data($condition, null, null, $order_by)->as_array();
        
        //Keeps unique assessment publication ids.
        $publication_ids = array();
        foreach($basic_export as $basic_line)
        {
            $publication_ids[$basic_line->get_optional_property(
                WeblcmsAssessmentAttemptsTracker::PROPERTY_ASSESSMENT_ID)] = $basic_line->get_optional_property(
                WeblcmsAssessmentAttemptsTracker::PROPERTY_ASSESSMENT_ID);
        }
        
        $condition = new InCondition(ContentObjectPublication::PROPERTY_ID, $publication_ids);
        $publications = $wdm->retrieve_content_object_publications($condition)->as_array();
        
        //Keeps assessment ids. Structure $publication_id => $assessment_id.
        $assessment_ids = array();
        foreach($publications as $publication)
        {
            $assessment_ids[$publication->get_id()] = $publication->get_content_object_id();
        }
        
        $condition = new InCondition(ComplexContentObjectItem::PROPERTY_PARENT, $assessment_ids);
        $order_by = array();
        $order_by[] = new ObjectTableOrder(ComplexContentObjectItem::PROPERTY_PARENT);
        $complex_questions_results = $rdm->retrieve_complex_content_object_items($condition, $order_by)->as_array();
        
        //Keeps complex question relations. Structure $assessment_id => $complex_question_id => $question_id.
        $complex_question_ids = array();
        //Keeps question ids. Structure $question_id => $question_id.
        $question_ids = array();
        //Keeps complex questions. Structure $complex_question_id => $complex_question.
        $complex_questions = array();
        foreach($complex_questions_results as $complex_question_line)
        {
            $complex_question_ids[$complex_question_line->get_parent()][$complex_question_line->get_id()] = $complex_question_line->get_ref();
            $question_ids[$complex_question_line->get_ref()] = $complex_question_line->get_ref();
            $complex_questions[$complex_question_line->get_id()] = $complex_question_line;
        }
        
        $condition = new InCondition(ContentObject::PROPERTY_ID, $question_ids);
        $questions_set = $rdm->retrieve_content_objects($condition)->as_array();
        
        //Structure $question_id => $question. Simple questions (rco's).
        $questions = array();
        foreach($questions_set as $questions_line)
        {
            $questions[$questions_line->get_id()] = $questions_line;
        }
        
        //Structure $publication_question_ids[$publication_id][$complex_question_id] => $question_id;
        $publication_question_ids = array();
        foreach($assessment_ids as $publication_id => $assessment_id)
        {
            $publication_question_ids[$publication_id] = $complex_question_ids[$assessment_id];
        }
        
        //Adds the additional properties necessary for the export.
        foreach($basic_export as $basic_line)
        {
            $question = $questions[$publication_question_ids[$basic_line->get_optional_property(
                WeblcmsAssessmentAttemptsTracker::PROPERTY_ASSESSMENT_ID)][$basic_line->get_question_cid()]];
            $basic_line->set_answer($this->refine_answer($question, $basic_line->get_answer()));
            $basic_line->set_optional_property(self::PROPERTY_QUESTION_TITLE, $question->get_title());
            $basic_line->set_optional_property(self::PROPERTY_QUESTION_TYPE, $question->get_type());
            $basic_line->set_optional_property(WeblcmsAssessmentAttemptsTracker::PROPERTY_DATE, 
                $this->format_date($basic_line->get_optional_property(WeblcmsAssessmentAttemptsTracker::PROPERTY_DATE)));
            $basic_line->set_optional_property(WeblcmsAssessmentAttemptsTracker::PROPERTY_TOTAL_TIME,
                $this->format_total_time($basic_line->get_optional_property(WeblcmsAssessmentAttemptsTracker::PROPERTY_TOTAL_TIME)));
            $basic_line->set_optional_property(self::PROPERTY_QUESTION_WEIGHT, $complex_questions[$basic_line->get_question_cid()]->get_weight());
            $basic_line->set_optional_property(self::PROPERTY_USER_FULLNAME, $basic_line->get_optional_property(
                User::PROPERTY_FIRSTNAME) . ' ' . $basic_line->get_optional_property(User::PROPERTY_LASTNAME));
            
            //Transfers default properties to the optional properties.
            foreach($basic_line->get_default_properties() as $default_property_key => $default_property_value)
            {
                $basic_line->set_optional_property($default_property_key, $default_property_value);
            }
            $complex_question_ids = array_keys($publication_question_ids[$basic_line->get_optional_property(
                WeblcmsAssessmentAttemptsTracker::PROPERTY_ASSESSMENT_ID)]);
            $total_weight = 0;
            foreach($complex_question_ids as $complex_question_id)
            {
                $total_weight += $complex_questions[$complex_question_id]->get_weight();
            }
            $basic_line->set_optional_property(self::PROPERTY_PUBLICATION_WEIGHT, $total_weight);
        }
        $full_export = $this->filter_exportable_questions($basic_export);
        
        $this->export($full_export, $this->define_export_properties(), $export_format);
    }
    
    /**
     * Defines the inital conditions necessary to make an export.
     * @return AndCondition The conditions for the export.
     */
    private function define_conditions()
    {
        $conditions = array();
        if(Request::get(AssessmentTool::PARAM_ASSESSMENT))
        {
            $conditions[] = new EqualityCondition(WeblcmsAssessmentAttemptsTracker::PROPERTY_ASSESSMENT_ID, 
                Request::get(AssessmentTool::PARAM_ASSESSMENT), 
                TrackingDataManager::get_instance()->get_alias(WeblcmsAssessmentAttemptsTracker::get_table_name()), true);
        }
        return new AndCondition($conditions);
    }
    
    /**
     * Defines the order of the export lines.
     * @return array The definition of the order of the export lines.
     */
    private function define_order_by()
    {
        $rdm = RepositoryDataManager::get_instance();
        $tdm = TrackingDataManager::get_instance();
        $udm = UserDataManager::get_instance();
        $order_by = array();
        $order_by[] = new ObjectTableOrder(WeblcmsAssessmentAttemptsTracker::PROPERTY_ASSESSMENT_ID, SORT_ASC, $tdm->get_alias(WeblcmsAssessmentAttemptsTracker::get_table_name()));
        $order_by[] = new ObjectTableOrder(User::PROPERTY_LASTNAME, SORT_ASC, $udm->get_alias(User::get_table_name()));
        $order_by[] = new ObjectTableOrder(User::PROPERTY_FIRSTNAME, SORT_ASC, $udm->get_alias(User::get_table_name()));
        $order_by[] = new ObjectTableOrder(WeblcmsAssessmentAttemptsTracker::PROPERTY_ID, SORT_ASC, $tdm->get_alias(WeblcmsAssessmentAttemptsTracker::get_table_name()));
        $order_by[] = new ObjectTableOrder(ComplexContentObjectItem::PROPERTY_DISPLAY_ORDER, SORT_ASC, $rdm->get_alias(ComplexContentObjectItem::get_table_name()));
        return $order_by;
    }
    
    /**
     * Converts dates into a standard format.
     * @param int $date The date to be formatted as a timestamp.
     * @return string A string representation of the date.
     */
    private function format_date($date)
    {
        return DatetimeUtilities :: format_locale_date(Translation :: get('DateTimeFormatLong', null, Utilities :: COMMON_LIBRARIES), $date);
    }
    
    /**
     * Converts the total time spent on the assessment from seconds into hours, minutes and seconds.
     * @param int $total_time The time in seconds.
     * @return string The time in hours, minutes and seconds.
     */
    private function format_total_time($total_time)
    {
        $seconds = $total_time % 60;
        $total_time /= 60;
        $minutes = $total_time % 60;
        $total_time /= 60;
        $hours = $total_time % 24;
        $total_time /= 24;
        $days = (int) $total_time;
        
        $output = array();
        if ($days != 0)
        {
            $output[] = $days . ' ' . Translation :: get('Days');
        }
        if ($hours != 0 || count($output) > 0)
        {
            $output[] = $hours . ' ' . Translation :: get('Hours');
        }
        if ($minutes != 0 || count($output) > 0)
        {
            $output[] = $minutes . ' ' . Translation :: get('Minutes');
        }
        if($seconds != 0 || count($output) > 0)
        {
            $output[] = $seconds .  ' ' . Translation :: get('Seconds');
        }
        return implode(" ", $output);
    }
    
    /**
     * Redefines a question attempt's answer to a more legible format.
     * @param mixed $question The simple question to which the $question is related.
     * @param mixed $answer A serialised array representing the answer given.
     * @return string A string representation of the question attempt's answer.
     */
    private function refine_answer($question, $answer)
    {
        $answer = unserialize($answer);
        
        switch($question->get_type_name())
        {
            case AssessmentMatchNumericQuestion::get_type_name():
                $answer = $this->refine_matchnumeric_answer($answer);
                break;
            case AssessmentMatchTextQuestion::get_type_name():
                $answer = $this->refine_matchtext_answer($answer);
                break;
            case AssessmentMatchingQuestion::get_type_name():
                $answer = $this->refine_matching_answer($question, $answer);
                break;
            case AssessmentMatrixQuestion::get_type_name():
                $answer = $this->refine_matrix_answer($question, $answer);
                break;
            case AssessmentMultipleChoiceQuestion::get_type_name():
                $answer = $this->refine_multiplechoice_answer($question, $answer);
                break;
            case AssessmentOpenQuestion::get_type_name():
                $answer = $this->refine_open_answer($answer);
                break;
            case AssessmentRatingQuestion::get_type_name():
                $answer = $this->refine_rating_answer($answer);
                break;
            case AssessmentSelectQuestion::get_type_name():
                $answer = $this->refine_select_answer($question, $answer);
                break;
            case FillInBlanksQuestion::get_type_name():
                $answer = $this->refine_fillintheblanks_answer($answer);
                break;
            case HotspotQuestion::get_type_name():
                break;
            case MatchQuestion::get_type_name():
                $answer = $this->refine_match_answer($answer);
                break;
            case OrderingQuestion::get_type_name():
                $answer = $this->refine_ordering_answer($question, $answer);
                break;
        }
        return implode('<br />', $answer);
    }
    
    private function refine_fillintheblanks_answer($answer)
    {
        foreach($answer as $answer_part_id => &$answer_part_text)
        {
            $answer_part_text = '(' . $answer_part_id . ') ' . $answer_part_text;
        }
        return $answer;
    }
    
    private function refine_match_answer($answer)
    {
        return $answer;
    }
    
    private function refine_matching_answer($question, $answer)
    {
        $options = array();
        foreach($question->get_options() as $option_id => $option)
        {
            $options[$option_id] = $option->get_value();
        }
        $matches = $question->get_matches();
        
        foreach($answer as $answer_part_id => &$answer_part)
        {
            $answer_part = $options[$answer_part_id] . ' = ' . $matches[$answer_part];
        }
        return $answer;
    }
    
    private function refine_matchnumeric_answer($answer)
    {
        
        return $answer;
    }
    
    private function refine_matchtext_answer($answer)
    {
        return $answer;
    }
    
    private function refine_matrix_answer($question, $answer)
    {
        if(!is_null($answer))
        {
            $options = array();
            foreach($question->get_options() as $option_id => $option)
            {
                $options[$option_id] = $option->get_value();
            }
            $matches = $question->get_matches();
            
            switch($question->get_matrix_type())
            {
                case AssessmentMatrixQuestion::MATRIX_TYPE_CHECKBOX:
                    foreach($answer as $answer_part_id => &$answer_part_match_ids)
                    {
                        $answer_part_matches = array();
                        foreach($answer_part_match_ids as $answer_part_match_id => $answer_part_match)
                        {
                            $answer_part_matches[] = $matches[$answer_part_match_id];
                        }
                        $answer_part_match_ids = $options[$answer_part_id] . ' = ' . implode(', ', $answer_part_matches);
                    }
                    break;
                case AssessmentMatrixQuestion::MATRIX_TYPE_RADIO:
                    foreach($answer as $answer_part_id => &$answer_part)
                    {
                        $answer_part = $options[$answer_part_id] . ' = ' . $matches[$answer_part];
                    }
                    break;
            }
        }
        else
        {
            $answer = array('');
        }
        return $answer;
    }
    
    private function refine_multiplechoice_answer($question, $answer)
    {
        $new_answer = array();
        $options = array();
        foreach($question->get_options() as $option_id => $option)
        {
            $options[$option_id] = $option->get_value();
        }
        
        switch($question->get_answer_type())
        {
            case AssessmentMultipleChoiceQuestion::ANSWER_TYPE_CHECKBOX:
                foreach($answer as $answer_part_id =>$answer_part)
                {
                    $new_answer[] = $options[$answer_part_id - 1];
                }
                break;
            case AssessmentMultipleChoiceQuestion::ANSWER_TYPE_RADIO:
                foreach($answer as $answer_part_id =>$answer_part)
                {
                    $new_answer[] = $options[$answer_part];
                }
                break;
        }
        return $new_answer;
    }
    
    private function refine_open_answer($answer)
    {
        return $answer;
    }
    
    private function refine_ordering_answer($question, $answer)
    {
        $options = $question->get_options();
        $new_answer = array();
        asort($answer, SORT_NUMERIC);
        foreach($answer as $option_index => $answer_part)
        {
            if($answer_part != '-1')
            {
                $new_answer[] = '(' . $answer_part . ') ' . $options[$option_index - 1]->get_value();
            }
        }
        if(empty($new_answer))
        {
            $new_answer[] = '';
        }
        return $new_answer;
    }
    
    private function refine_rating_answer($answer)
    {
        return $answer;
    }
    
    private function refine_select_answer($question, $answer)
    {
        $options = $question->get_options();
        $new_answer = array();
        switch($question->get_answer_type())
        {
            case AssessmentSelectQuestion::ANSWER_TYPE_CHECKBOX:
                if(!is_null($answer))
                {
                    foreach($answer[0] as $answer_part)
                    {
                        $new_answer[] = $options[$answer_part]->get_value();
                    }
                }
                else
                {
                    $new_answer[] = '';
                }
                break;
            case AssessmentSelectQuestion::ANSWER_TYPE_RADIO:
                    if(!is_null($options[$answer[0]]))
                    {
                        $new_answer[] = $options[$answer[0]]->get_value();
                    }
                    else
                    {
                        $new_answer[] = '';
                    }
                break;
        }
        return array(implode(', ', $new_answer));
    }
    
    /**
     * Defines which questions may appear in the final export.
     * @param array $basic_export The export to be filtered with property self::PROPERTY_QUESTION_TYPE as optional property.
     * @return array The filtered export.
     */
    private function filter_exportable_questions($basic_export)
    {
        $full_export = array();
        foreach($basic_export as $basic_line)
        {
            switch($basic_line->get_optional_property(self::PROPERTY_QUESTION_TYPE))
            {
                case AssessmentMatchNumericQuestion::get_type_name():
                case AssessmentMatchTextQuestion::get_type_name():
                case AssessmentMatchingQuestion::get_type_name():
                case AssessmentMatrixQuestion::get_type_name():
                case AssessmentMultipleChoiceQuestion::get_type_name():
                case AssessmentOpenQuestion::get_type_name():
                case AssessmentRatingQuestion::get_type_name():
                case AssessmentSelectQuestion::get_type_name():
                case FillInBlanksQuestion::get_type_name():
                case MatchQuestion::get_type_name():
                case OrderingQuestion::get_type_name():
                    $full_export[] = $basic_line;
                    break;
                default:
                    break;
            }
        }
        return $full_export;
    }
    
    /**
     * Defines the properties to be exported.
     * @return array The properties to be exported. 
     */
    private function define_export_properties()
    {
        $headers = array();
        $headers[Translation::get('UserCode')] = User::PROPERTY_OFFICIAL_CODE;
        $headers[Translation::get('UserName')] = self::PROPERTY_USER_FULLNAME;
        $headers[Translation::get('AssessmentId')] = WeblcmsAssessmentAttemptsTracker::PROPERTY_ASSESSMENT_ID;
        $headers[Translation::get('AssessmentTitle')] = ContentObject::PROPERTY_TITLE;
        $headers[Translation::get('AssessmentDate')] = WeblcmsAssessmentAttemptsTracker::PROPERTY_DATE;
        $headers[Translation::get('AssessmentTime')] = WeblcmsAssessmentAttemptsTracker::PROPERTY_TOTAL_TIME;
        $headers[Translation::get('AssessmentScore')] = WeblcmsAssessmentAttemptsTracker::PROPERTY_TOTAL_SCORE;
        $headers[Translation::get('PublicationWeight')] = self::PROPERTY_PUBLICATION_WEIGHT;
        $headers[Translation::get('QuestionType')] = self::PROPERTY_QUESTION_TYPE;
        $headers[Translation::get('QuestionId')] = WeblcmsQuestionAttemptsTracker::PROPERTY_QUESTION_CID;
        $headers[Translation::get('QuestionNumber')] = ComplexContentObjectItem::PROPERTY_DISPLAY_ORDER;
        $headers[Translation::get('QuestionTitle')] = self::PROPERTY_QUESTION_TITLE;
        $headers[Translation::get('QuestionAnswer')] = WeblcmsQuestionAttemptsTracker::PROPERTY_ANSWER;
        $headers[Translation::get('QuestionScore')] = WeblcmsQuestionAttemptsTracker::PROPERTY_SCORE;
        $headers[Translation::get('QuestionWeight')] = self::PROPERTY_QUESTION_WEIGHT;
        return $headers;
    }
    
    /**
     * Exports the data to the given format.
     * @param array $export_lines The data to export.
     */
    private function export($export_lines, $properties, $export_format)
    {
        switch($export_format)
        {
            case self::EXPORT_FORMAT_CSV:
                $this->export_csv($export_lines, $properties);
                break;
        }
    }
    
    /**
     * Exports the data to csv format.
     * @param array $export_lines The data to be exported.
     * @param array $properties The peoperties to be exported.
     */
    private function export_csv($export_lines, $properties)
    {
        $csv_data = array();
        foreach($export_lines as $export_line)
        {
            $csv_data_line = array();
            foreach($properties as $header_key => $header)
            {
                $value = str_replace(chr(self::ASCII_CHAR_NEW_LINE), ' ', $export_line->get_optional_property($header));
                $value = str_replace(chr(self::ASCII_CHAR_CARRIAGE_RETURN), ' ', $value);
                $csv_data_line[$header_key] = '"' . $value . '"';
            }
            $csv_data[] = $csv_data_line;
        }
        $export = Export::factory('csv', $csv_data);
        $export->set_filename('raw_assessment_export' . date('_Y-m-d_H-i-s'));
        $export->send_to_browser();
    }
}

?>
