<?php

namespace application\weblcms\tool\assignment;

use application\weblcms\WeblcmsAssignmentSubmissionsTracker;
use application\weblcms\ToolComponent;
use application\weblcms\WeblcmsDataManager;
use application\weblcms\WeblcmsManager;
use application\weblcms\WeblcmsRights;
use application\weblcms\WeblcmsSubmissionFeedbackTracker;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Request;
use repository\RepositoryDataManager;

/**
 * This component allows a user to view an attachment.
 *
 * @author Bert De Clercq (Hogeschool Gent)
 */
class AssignmentToolAttachmentViewerComponent extends SubmissionsManager
{
    
    const TYPE_SUBMISSION = 'submission';
    const TYPE_AUTOMATIC_FEEDBACK = 'automatic_feedback';

    function run()
    {
        ToolComponent :: launch($this);
    }

    /**
     * Checks whether an object is really attached to a context determined by
     * the attachment type parameter form the url.
     * 
     * @return boolean True if the object is attached in the correct context for
     *         the item
     */
    function is_object_attached_in_context()
    {
        $type = Request :: get(AssignmentTool :: PARAM_ATTACHMENT_TYPE);
        
        switch ($type)
        {
            case self :: TYPE_SUBMISSION :
                return $this->is_submission_attachment();
            case self :: TYPE_AUTOMATIC_FEEDBACK :
                return $this->is_automatic_feedback_attachment();
            default :
                if($this->is_submission_feedback_attachment())
                {
                    return true;
                }
                return $this->is_publication_attachment();
        }
    }

    /**
     * Checks whether the object you want to view is really attached to the submission.
     * It will get the submission id and the object id from the url.
     * @return boolean True if the object is attached to the submission.
     */
    private function is_submission_attachment()
    {
        $submission_tracker = $this->get_submission_tracker_by_id($this->get_submission_id());

        return $submission_tracker->get_publication_id() == $this->get_publication_id() &&
            $submission_tracker->get_content_object_id() == $this->get_object_id();
    }

    /**
     * Checks whether or not the object you want to view is attached to the feedback of a submission
     *
     * @return bool
     */
    protected function is_submission_feedback_attachment()
    {
        if(is_null($this->get_submission_id()))
        {
            return false;
        }

        $submission_tracker = $this->get_submission_tracker_by_id($this->get_submission_id());

        if ($submission_tracker->get_publication_id() != $this->get_publication_id())
        {
            return false;
        }

        $feedbacks = WeblcmsSubmissionFeedbackTracker :: get_data(
            WeblcmsSubmissionFeedbackTracker :: CLASS_NAME,
            WeblcmsManager :: APPLICATION_NAME,
            new EqualityCondition(
                WeblcmsSubmissionFeedbackTracker :: PROPERTY_SUBMISSION_ID,
                $submission_tracker->get_id()
            )
        );

        while($feedback = $feedbacks->next_result())
        {
            if(RepositoryDataManager :: is_attached_or_included_to(
                $feedback->get_content_object_id(), $this->get_object_id()
            ))
            {
                return true;
            }
        }

        return false;
    }

    /**
     * Returns the submission tracker by submission id
     *
     * @param int $submission_id
     *
     * @return WeblcmsAssignmentSubmissionsTracker
     */
    protected function get_submission_tracker_by_id($submission_id)
    {
        $submission_trackers = WeblcmsAssignmentSubmissionsTracker :: get_data(
            WeblcmsAssignmentSubmissionsTracker :: CLASS_NAME,
            WeblcmsManager :: APPLICATION_NAME,
            new EqualityCondition(
                WeblcmsAssignmentSubmissionsTracker :: PROPERTY_ID,
                $submission_id
            )
        );

        return $submission_trackers->next_result();
    }

    /**
     * Checks whether the automatic feedback object you want to view is really
     * attached to the assignment.
     * It will get the assignment via the publication id from the url and the
     * object id from the url.
     * 
     * @return boolean True if the automatic feedback object is attached to the
     *         assignment
     */
    private function is_automatic_feedback_attachment()
    {
        $assignment = WeblcmsDataManager :: get_instance()->retrieve_content_object_publication($this->get_publication_id())->get_content_object();
        $automatic_feedback_co_ids_string = $assignment->get_automatic_feedback_co_ids();
        $automatic_feedback_co_ids_array = explode(",", $automatic_feedback_co_ids_string);
        $key = array_search($this->get_object_id(), $automatic_feedback_co_ids_array);
        if (! ($key === false))
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * Checks whether the object you want to view is really attached to the
     * publication.
     * It will get the publication id and object id from the url.
     * 
     * @return boolean True if the object is attached to the publication
     */
    private function is_publication_attachment()
    {
        $publication = WeblcmsDataManager :: get_instance()->retrieve_content_object_publication($this->get_publication_id());
        return RepositoryDataManager :: is_attached_or_included_to($publication->get_content_object_id(), $this->get_object_id());
    }

}

?>
