<?php

namespace application\weblcms\tool\assignment;

use application\weblcms\Tool;
use common\libraries\Toolbar;
use common\libraries\ToolbarItem;
use common\libraries\Theme;
use common\libraries\Path;
use common\libraries\Translation;
use repository\DefaultContentObjectTableCellRenderer;
use common\libraries\Request;
use application\weblcms\WeblcmsAssignmentSubmissionsTracker;
use common\libraries\DatetimeUtilities;
use common\libraries\Utilities;
use application\weblcms\WeblcmsRights;
use user\User;

/**
 * @package application.weblcms.tool.assignment.php.component.submission_browser
 * 
 * This class is a cell renderer for a submitters browser table
 *
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 * @author Bert De Clercq (Hogeschool Gent)
 * @author Anthony Hurst (Hogeschool Gent)
 */
class SubmissionUsersBrowserTableCellRenderer extends DefaultContentObjectTableCellRenderer
{

    private $table_actions;
    private $browser;

    /**
     * Constructor.
     * @param WeblcmsManagerComponent $browser
     */
    function __construct($browser)
    {
        $this->table_actions = array();
        $this->browser = $browser;
        $this->pid = Request::get(Tool :: PARAM_PUBLICATION_ID);
    }

    function render_cell($column, $user)
    {
        if ($column === SubmissionBrowserTableColumnModel :: get_action_column())
        {
            return $this->get_actions($user);
        }
        else
        {
            switch ($column->get_name())
            {
                case User :: PROPERTY_LASTNAME :
                    // Check if submissions of the user should be visible
                    if ($user->get_submitter_id() == $this->browser->get_user_id()
                            || $this->browser->get_assignment()->get_visibility_submissions() == 1
                            || $this->browser->is_allowed(WeblcmsRights::EDIT_RIGHT))
                    {
                        $url = $this->browser->get_url(array(
                            Tool :: PARAM_PUBLICATION_ID => $this->pid,
                            AssignmentTool :: PARAM_TARGET_ID => $user->get_submitter_id(),
                            AssignmentTool :: PARAM_SUBMITTER_TYPE => WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_USER,
                            Tool :: PARAM_ACTION => AssignmentTool :: ACTION_BROWSE_SUBMISSIONS));
                        return '<a href=\'' . $url . '\'>' . $user->get_optional_property(User::PROPERTY_LASTNAME) . ', ' . $user->get_optional_property(User::PROPERTY_FIRSTNAME) . '</a>';
                    }
                    return $user->get_optional_property(User::PROPERTY_LASTNAME) . ', ' . $user->get_optional_property(User::PROPERTY_FIRSTNAME);

                case AssignmentTool :: PROPERTY_FIRST_SUBMISSION :
                    if(is_null($user->get_optional_property(AssignmentTool::PROPERTY_FIRST_SUBMISSION)))
                    {
                        return '-';
                    }
                    return $this->format_date($user->get_optional_property(AssignmentTool::PROPERTY_FIRST_SUBMISSION));

                case AssignmentTool :: PROPERTY_LAST_SUBMISSION :
                    if(is_null($user->get_optional_property(AssignmentTool::PROPERTY_LAST_SUBMISSION)))
                    {
                        return '-';
                    }
                    return $this->format_date($user->get_optional_property(AssignmentTool::PROPERTY_LAST_SUBMISSION));

                case AssignmentTool :: PROPERTY_NUMBER_OF_SUBMISSIONS :
                    return $user->get_optional_property(AssignmentTool::PROPERTY_NUMBER_OF_SUBMISSIONS);

                case AssignmentTool :: PROPERTY_NUMBER_OF_FEEDBACKS :
                    $feedbacks = $this->browser->get_submitter_feedbacks
                            (WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_USER, $user->get_submitter_id());
                    if ($feedbacks != null)
                        return $feedbacks->get_optional_property('count');
                    else
                        return '0';

                default :
                    return '-';
            }
        }
    }

    /**
     * Creates a toolbar with the appropriate actions
     * @param User $user
     * @return string
     */
    function get_actions($user)
    {
        $toolbar = new Toolbar();

        if ($user->get_submitter_id() == $this->browser->get_user_id()
                || $this->browser->get_assignment()->get_visibility_submissions() == 1
                || $this->browser->is_allowed(WeblcmsRights::EDIT_RIGHT))
        {
            $toolbar->add_item($this->get_view_submissions_action($user));
        }
        
        if ($this->browser->is_allowed(WeblcmsRights::EDIT_RIGHT) && $user->get_optional_property(AssignmentTool::PROPERTY_NUMBER_OF_SUBMISSIONS) > 0)
        {
            $toolbar->add_item($this->get_download_all_submissions_action($user));
        }
        
        if ($user->get_submitter_id() == $this->browser->get_user_id())
        {
            $toolbar->add_item($this->get_submission_submit_action($user));
        }
        
        return $toolbar->as_html();
    }

    private function get_submission_submit_action($user)
    {
        return new ToolbarItem(Translation :: get('SubmissionSubmit'),
                Theme :: get_common_image_path() . 'action_add.png', $this->browser->get_url(array(
                    Tool :: PARAM_ACTION => AssignmentTool::ACTION_SUBMIT_SUBMISSION,
                    Tool :: PARAM_PUBLICATION_ID => $this->pid, AssignmentTool :: PARAM_TARGET_ID => $user->get_submitter_id(),
                    AssignmentTool :: PARAM_SUBMITTER_TYPE => WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_USER)),
                ToolbarItem :: DISPLAY_ICON);
    }

    private function get_view_submissions_action($user)
    {
        return new ToolbarItem(Translation :: get('ViewSubmissions'),
                Theme :: get_common_image_path() . 'action_browser.png', $this->browser->get_url(array(
                    Tool :: PARAM_ACTION => AssignmentTool :: ACTION_BROWSE_SUBMISSIONS,
                    AssignmentTool :: PARAM_TARGET_ID => $user->get_submitter_id(),
                    AssignmentTool::PARAM_SUBMITTER_TYPE => WeblcmsAssignmentSubmissionsTracker::SUBMITTER_TYPE_USER)),
                ToolbarItem :: DISPLAY_ICON);
    }

    private function get_download_all_submissions_action($user)
    {
        return new ToolbarItem(Translation :: get('DownloadAllSubmissions'),
                Theme :: get_common_image_path() . 'action_download.png', $this->browser->get_url(array(
                    Tool :: PARAM_ACTION => AssignmentTool :: ACTION_DOWNLOAD_SUBMISSIONS,
                    AssignmentTool :: PARAM_TARGET_ID => $user->get_submitter_id(),
                    AssignmentTool::PARAM_SUBMITTER_TYPE => WeblcmsAssignmentSubmissionsTracker::SUBMITTER_TYPE_USER)),
                ToolbarItem :: DISPLAY_ICON);
    }

    /**
     * Formats a date.
     * @param type $date the date to be formatted.
     * @return the formatted representation of the date.
     */
    private function format_date($date)
    {
        $formatted_date = DatetimeUtilities :: format_locale_date(Translation :: get('DateTimeFormatLong', null,
                                Utilities :: COMMON_LIBRARIES), $date);
        if ($date > $this->browser->get_assignment()->get_end_time())
        {
            return '<span style="color:red">' . $formatted_date . '</span>';
        }
        //$this->render_id_cell($object);
        return $formatted_date;
    }

    /**
     * #Override
     * Modifies the assignment of the user id to match the modified join call to the database. To avoid caching conflicts, the user id is stored in the variable $submitter_id.
     * @param type $object The user whose id is to be retrieved.
     */
    public function render_id_cell($object)
    {
        return $object->get_submitter_id();
    }
}

?>