<?php

namespace application\weblcms\tool\assignment;

use application\weblcms\WeblcmsAssignmentSubmissionsTracker;
use application\weblcms\WeblcmsDataManager;
use common\libraries\DatetimeUtilities;
use common\libraries\InCondition;
use common\libraries\Request;
use common\libraries\Theme;
use common\libraries\Translation;
use common\libraries\Utilities;
use group\GroupDataManager;
use user\User;
use user\UserDataManager;

/**
 * Contains all the general info from a submission and can render this on
 * another page.
 *
 * @author Bert De Clercq (Hogeschool Gent)
 */
class SubmissionDetailGeneralInfoSection
{
    
    /**
     * The main page where this section will be rendered.
     * 
     * @var AssignmentToolSubmissionViewerComponent
     */
    private $main_page;

    /**
     * Assigns the given submission details main page to the caching variable.
     * 
     * @param $main_page AssignmentToolSubmissionViewerComponent
     *            The main page
     */
    function __construct($main_page)
    {
        $this->main_page = $main_page;
    }

    /**
     * Renders the section.
     */
    function render_section()
    {
        echo $this->main_page->set_section_content(Translation :: get('General'), $this->get_section_content());
    }

    /**
     * Returns the contents of this section as an array of strings.
     * 
     * @return array The content as html code
     */
    private function get_section_content()
    {
        $html = array();
        
        // Date
        $html[] = $this->get_date_submitted_html();
        
        // Description
        $html[] = $this->get_description_html();
        
        // Submitter
        if ($this->main_page->get_assignment()->get_allow_group_submissions())
        {
            $html[] = $this->get_submitter_html();
        }
        
        // Group members
        if ($this->main_page->get_submitter_type() != WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_USER)
        {
            $html[] = $this->get_group_members_html();
        }
        
        return implode("\n", $html);
    }

    /**
     * Returns the date the submission was submitted as an array of strings of
     * html code.
     * 
     * @return array The date
     */
    private function get_date_submitted_html()
    {
        $date_submitted = $this->main_page->get_submission_tracker()->get_date_submitted();
        $html = array();
        
        $html[] = '<div style="font-weight:bold; float:left">';
        $html[] = Translation :: get('DateSubmitted') . ':&nbsp;<br />';
        $html[] = '</div>';
        
        // Content
        $html[] = '<div style="float:left">';
        $html[] = $this->format_date($date_submitted) . '<br />';
        $html[] = '</div><br />';
        
        return implode("\n", $html);
    }

    /**
     * Returns the description as an array of strings.
     * 
     * @return array The description as html code
     */
    private function get_description_html()
    {
        $html = array();
        
        // Title
        $html[] = '<div>';
        $html[] = '<h4>' . Translation :: get('Description') . ':&nbsp;</h4>';
        $html[] = '</div>';
        
        // Content
        $html[] = '<div>';
        $html[] = $this->main_page->get_submission()->get_description();
        $html[] = '</div>';
        
        return implode("\n", $html);
    }

    /**
     * Returns the submitter info as an array of strings.
     * 
     * @return array The submitter info as html code
     */
    private function get_submitter_html()
    {
        $html = array();
        
        // Title
        $html[] = '<div style="font-weight:bold; float:left">';
        $html[] = Translation :: get('Submitter') . ':&nbsp;<br />';
        $html[] = '</div>';
        
        // Content
        $html[] = '<div style="float:left">';
        $html[] = $this->get_submitter() . '<br />';
        $html[] = '</div><br />';
        
        return implode("\n", $html);
    }

    /**
     * Returns the group members as an array of strings.
     * 
     * @return array The group members as html code
     */
    private function get_group_members_html()
    {
        $html = array();
        
        // Title
        $html[] = '<div style="font-weight:bold; float:left">';
        $html[] = Translation :: get('GroupMembers') . ':&nbsp;<br />';
        $html[] = '</div>';
        
        // Content
        $html[] = '<div style="float:left">';
        $html[] = $this->get_group_members() . '<br />';
        $html[] = '</div><br />';
        
        return implode("\n", $html);
    }

    /**
     * Returns the name of the submitter of the submission, followed by the
     * group.
     * 
     * @return string The submitter name and group
     */
    private function get_submitter()
    {
        $user_name = UserDataManager :: get_instance()->retrieve_user($this->main_page->get_submission_tracker()->get_user_id());
        
        switch ($this->main_page->get_submitter_type())
        {
            case WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_USER :
                return $this->main_page->get_submitter_name();
            case WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_COURSE_GROUP :
                return $user_name->get_fullname() . ' - ' . $this->main_page->get_submitter_name();
            case WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_PLATFORM_GROUP :
                return $user_name->get_fullname() . ' - ' . $this->main_page->get_submitter_name();
        }
    }

    /**
     * Returns a string of all the group members separated by commas.
     * 
     * @return string The group members
     */
    private function get_group_members()
    {
        if ($this->main_page->get_submitter_type() == WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_COURSE_GROUP)
        {
            $group_members = WeblcmsDataManager :: get_instance()->retrieve_course_group_users(WeblcmsDataManager :: get_instance()->retrieve_course_group($this->main_page->get_target_id()))->as_array();
        }
        else
        {
            $group_members = UserDataManager :: get_instance()->retrieve_users(new InCondition(User :: PROPERTY_ID, GroupDataManager :: get_instance()->retrieve_group($this->main_page->get_target_id())->get_users()))->as_array();
        }
        
        $group_member_names = array();
        
        foreach ($group_members as $member)
        {
            $group_member_names[$member->get_lastname()] = $member->get_fullname();
        }
        
        ksort($group_member_names);
        
        return implode(", ", $group_member_names);
    }

    /**
     * Formats a date.
     * 
     * @param $date type
     *            the date to be formatted.
     * @return the formatted representation of the date.
     */
    private function format_date($date_submitted)
    {
        $formatted_date = DatetimeUtilities :: format_locale_date(Translation :: get('DateTimeFormatLong', null, Utilities :: COMMON_LIBRARIES), $date_submitted);
        
        if ($date_submitted > $this->main_page->get_assignment()->get_end_time())
        {
            return '<span style="color:red">' . $formatted_date . ' (' . $this->get_time_late($date_submitted) . ')</span>';
        }
        
        return $formatted_date;
    }

    /**
     * Returns the time the submission was too late.
     * 
     * @param $date_submitted time           
     * @return array The time late
     */
    private function get_time_late($date_submitted)
    {
        $time_late = $date_submitted - $this->main_page->get_assignment()->get_end_time();
        
        $seconds_late = $time_late % 60;
        $time_late /= 60;
        $minutes_late = $time_late % 60;
        $time_late /= 60;
        $hours_late = $time_late % 24;
        $time_late /= 24;
        $days_late = (int) $time_late;
        
        $output = array();
        
        if ($days_late != 0)
        {
            $output[] = $days_late . ' ' . Translation :: get('Days');
        }
        if ($hours_late != 0 || count($output) > 0)
        {
            $output[] = $hours_late . ' ' . Translation :: get('Hours');
        }
        if ($minutes_late != 0 || count($output) > 0)
        {
            $output[] = $minutes_late . ' ' . Translation :: get('Minutes');
        }
        if ($days_late == 0 && $hours_late == 0 && $minutes_late == 0)
        {
            $output[] = $seconds_late . ' ' . Translation :: get('Seconds');
        }
        $output[] = ' ' . Translation :: get('Late');
        
        return implode(" ", $output);
    }

}

?>
