<?php

namespace application\weblcms\tool\assignment;

use application\weblcms\WeblcmsSubmissionScoreTracker;
use common\libraries\FormValidator;
use common\libraries\Request;
use common\libraries\Theme;
use common\libraries\Translation;
use common\libraries\Utilities;
use tracking\Event;

/**
 * This form allows a user to give or update the score of a submission.
 *
 * @author Bert De Clercq (Hogeschool Gent)
 */
class SubmissionDetailScoreSection extends FormValidator
{
    
    /**
     * Caching variable for the score tracker.
     * 
     * @var WeblcmsSubmissionScoreTracker The score tracker
     */
    private $score_tracker;

    /**
     * Fills the form with all the data.
     * 
     * @param $score_tracker WeblcmsSubmissionScoreTracker           
     * @param $url string
     *            The url of the main page
     */
    function __construct($score_tracker, $url = '')
    {
        parent :: __construct('submission_score', 'post', $url);
        $this->score_tracker = $score_tracker;
        
        $choices = array();
        
        $choices[- 1] = Translation :: get('NoScore');
        for($i = 0; $i <= 100; $i ++)
        {
            $choices[$i] = $i . '%';
        }
        
        $this->addElement('select', WeblcmsSubmissionScoreTracker :: PROPERTY_SCORE, Translation :: get('Score'), $choices);
        
        $save_button = $this->createElement('style_submit_button', 'submit', Translation :: get('Save', null, Utilities :: COMMON_LIBRARIES), array(
                'class' => 'positive publish'));
        
        $this->addElement($save_button);
        
        $this->set_initial_score();
    }

    /**
     * Sets the score on the form when it is rendered.
     */
    private function set_initial_score()
    {
        if ($this->score_tracker)
        {
            $defaults = array();
            $defaults[WeblcmsSubmissionScoreTracker :: PROPERTY_SCORE] = $this->score_tracker->get_score();
            parent :: setDefaults($defaults);
        }
    }

    /**
     * Updates or creates the score tracker of the submission.
     */
    function set_score()
    {
        if ($this->score_tracker)
        {
            $this->update_tracker();
        }
        else
        {
            $this->create_tracker();
        }
    }

    /**
     * Creates a score tracker for the submission.
     * 
     * @return boolean
     */
    private function create_tracker()
    {
        $values = $this->exportValues();
        
        if ($values[WeblcmsSubmissionScoreTracker :: PROPERTY_SCORE] == - 1)
        {
            return false;
        }
        
        $arguments = array(
                WeblcmsSubmissionScoreTracker :: PROPERTY_SUBMISSION_ID => Request :: get(AssignmentTool :: PARAM_SUBMISSION), 
                WeblcmsSubmissionScoreTracker :: PROPERTY_CREATED => time(), 
                WeblcmsSubmissionScoreTracker :: PROPERTY_MODIFIED => time(), 
                WeblcmsSubmissionScoreTracker :: PROPERTY_USER_ID => Request :: get(AssignmentTool :: PARAM_TARGET_ID), 
                WeblcmsSubmissionScoreTracker :: PROPERTY_SCORE => $values[WeblcmsSubmissionScoreTracker :: PROPERTY_SCORE]);
        Event :: trigger('score_submission', 'weblcms', $arguments);
        
        return true;
    }

    /**
     * Updates the score tracker of the submission when a new score is assigned.
     * 
     * @return boolean
     */
    private function update_tracker()
    {
        $values = $this->exportValues();
        
        if ($values[WeblcmsSubmissionScoreTracker :: PROPERTY_SCORE] == - 1)
        {
            return $this->score_tracker->delete();
        }
        
        $score = $values[WeblcmsSubmissionScoreTracker :: PROPERTY_SCORE];
        $this->score_tracker->set_score($score);
        $this->score_tracker->set_modified(time());
        
        return $this->score_tracker->update();
    }

}

?>
