<?php

namespace application\weblcms\tool\assignment;

use group\GroupDataManager;

use application\weblcms\WeblcmsManager;
use repository\content_object\document\Document;
use repository\content_object\description\Description;
use application\weblcms\WeblcmsAssignmentSubmissionsTracker;
use application\weblcms\WeblcmsDataManager;
use application\weblcms\WeblcmsSubmissionFeedbackTracker;
use application\weblcms\WeblcmsRights;
use application\weblcms\Tool;
use common\libraries\Breadcrumb;
use common\libraries\Translation;
use common\libraries\EqualityCondition;
use common\libraries\Request;
use common\libraries\Utilities;
use common\extensions\repo_viewer\RepoViewer;
use common\extensions\repo_viewer\RepoViewerInterface;
use tracking\Event;
use tracking\TrackingDataManager;
use user\UserDataManager;

/**
 *
 * @package application.weblcms.tool.assignment.php.component
 *         
 *          Component for the submission feedback form for assignment
 *          publications.
 *          Uses repoviewer
 *         
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 * @author Bert De Clercq (Hogeschool Gent)
 */
class AssignmentToolSubmissionFeedbackComponent extends AssignmentTool implements RepoViewerInterface
{
    
    /**
     * ID of the submission
     *
     * @var int
     */
    private $submission_id;
    
    /**
     * The assignment content object
     *
     * @var Assignment
     */
    private $assignment;

    function run()
    {
        $publication_id = Request :: get(AssignmentTool :: PARAM_PUBLICATION_ID);
        $target_id = Request :: get(AssignmentTool :: PARAM_TARGET_ID);
        $submitter_type = Request :: get(AssignmentTool :: PARAM_SUBMITTER_TYPE);
        
        // display form when content object created in RepoViewer
        if (RepoViewer :: is_ready_to_be_published())
        {
            $this->create_tracker();
            $this->redirect(Translation :: get('FeedbackCreated'), false, array(
                    Tool :: PARAM_ACTION => AssignmentTool :: ACTION_VIEW_SUBMISSION, 
                    Tool :: PARAM_PUBLICATION_ID => $publication_id, AssignmentTool :: PARAM_TARGET_ID => $target_id, 
                    AssignmentTool :: PARAM_SUBMITTER_TYPE => $submitter_type, 
                    AssignmentTool :: PARAM_SUBMISSION => $this->submission_id));
        }
        
        // construct RepoViewer when no content object created yet
        if (! RepoViewer :: is_ready_to_be_published())
        {
            $repo_viewer = RepoViewer :: construct($this);
            $repo_viewer->set_maximum_select(RepoViewer :: SELECT_SINGLE);
            $repo_viewer->set_parameter(AssignmentTool :: PARAM_SUBMISSION, $this->submission_id);
            $repo_viewer->run();
        }
    }

    /**
     * Creates a submission feedback tracker
     *
     * @param $values array
     *            exported values of the form
     */
    function create_tracker()
    {
        $arguments = array(WeblcmsSubmissionFeedbackTracker :: PROPERTY_SUBMISSION_ID => $this->submission_id, 
                WeblcmsSubmissionFeedbackTracker :: PROPERTY_CREATED => time(), 
                WeblcmsSubmissionFeedbackTracker :: PROPERTY_MODIFIED => time(), 
                WeblcmsSubmissionFeedbackTracker :: PROPERTY_USER_ID => $this->get_user_id(), 
                WeblcmsSubmissionFeedbackTracker :: PROPERTY_CONTENT_OBJECT_ID => $this->get_repo_object());
        $tracker = Event :: trigger('feedback_submission', 'weblcms', $arguments);
    }

    function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $publication_id = Request :: get(Tool :: PARAM_PUBLICATION_ID);
        $this->submission_id = Request :: get(AssignmentTool :: PARAM_SUBMISSION);
        
        $tracker = new WeblcmsAssignmentSubmissionsTracker();
        $condition = new EqualityCondition(WeblcmsAssignmentSubmissionsTracker :: PROPERTY_ID, $this->submission_id);
        $submission_trackers = $tracker->retrieve_tracker_items($condition);
        $submission_tracker = $submission_trackers[0];
        
        $submissions = WeblcmsAssignmentSubmissionsTracker :: get_data(WeblcmsAssignmentSubmissionsTracker :: CLASS_NAME, WeblcmsManager :: APPLICATION_NAME, $condition)->as_array();
        $submission = $submissions[0]->get_content_object();
        
        $weblcms_manager = WeblcmsDataManager :: get_instance();
        $pub = $weblcms_manager->retrieve_content_object_publication($publication_id);
        if (! $this->is_allowed(WeblcmsRights :: EDIT_RIGHT, $pub))
        {
            $this->redirect(Translation :: get("NotAllowed", null, Utilities :: COMMON_LIBRARIES), true, array(), array(
                    Tool :: PARAM_ACTION, Tool :: PARAM_PUBLICATION_ID, AssignmentTool :: PARAM_TARGET_ID, 
                    AssignmentTool :: PARAM_SUBMITTER_TYPE, AssignmentTool :: PARAM_SUBMISSION));
        }
        $this->assignment = $pub->get_content_object();
        
        $breadcrumbtrail->add(new Breadcrumb($this->get_url(array(
                Tool :: PARAM_ACTION => AssignmentTool :: ACTION_BROWSE_SUBMITTERS, 
                Tool :: PARAM_PUBLICATION_ID => $publication_id)), $this->assignment->get_title()));
        $breadcrumbtrail->add(new Breadcrumb($this->get_url(array(
                Tool :: PARAM_ACTION => AssignmentTool :: ACTION_BROWSE_SUBMISSIONS, 
                Tool :: PARAM_PUBLICATION_ID => $submission_tracker->get_publication_id(), 
                AssignmentTool :: PARAM_TARGET_ID => $submission_tracker->get_submitter_id(), 
                AssignmentTool :: PARAM_SUBMITTER_TYPE => $submission_tracker->get_submitter_type())), $this->get_submitter_name($submission_tracker)));
        $breadcrumbtrail->add(new Breadcrumb($this->get_url(array(
                Tool :: PARAM_ACTION => AssignmentTool :: ACTION_VIEW_SUBMISSION, 
                Tool :: PARAM_PUBLICATION_ID => $submission_tracker->get_publication_id(), 
                AssignmentTool :: PARAM_TARGET_ID => $submission_tracker->get_submitter_id(), 
                AssignmentTool :: PARAM_SUBMITTER_TYPE => $submission_tracker->get_submitter_type(), 
                AssignmentTool :: PARAM_SUBMISSION => $this->submission_id)), $submission->get_title() . ' - ' . Translation :: get('Detail')));
    }

    function get_allowed_content_object_types()
    {
        $types = explode(',', $this->assignment->get_allowed_types());
        $types[] = Description :: get_type_name();
        return $types;
    }

    /**
     * Gets the selected object id of the RepoViewer
     *
     * @return int
     */
    function get_repo_object()
    {
        return RepoViewer :: get_selected_objects();
    }

    function get_additional_parameters()
    {
        return array(Tool :: PARAM_PUBLICATION_ID, AssignmentTool :: PARAM_TARGET_ID, 
                AssignmentTool :: PARAM_SUBMITTER_TYPE, AssignmentTool :: PARAM_SUBMISSION);
    }

    /**
     * Returns the name of the submitter as a string.
     * When submitted as a group, it will return the name of the user who
     * submitted followed by the group name.
     *
     * @return string The name of the submitter
     */
    private function get_submitter_name($submission_tracker)
    {
        // name of the user who submitted
        $user_name = UserDataManager :: get_instance()->retrieve_user($submission_tracker->get_user_id());
        
        switch ($submission_tracker->get_submitter_type())
        {
            case WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_USER :
                return $user_name->get_fullname();
            case WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_COURSE_GROUP :
                return $user_name->get_fullname() . ' - ' . WeblcmsDataManager :: get_instance()->retrieve_course_group($submission_tracker->get_submitter_id())->get_name();
            case WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_PLATFORM_GROUP :
                return $user_name->get_fullname() . ' - ' . GroupDataManager :: get_instance()->retrieve_group($submission_tracker->get_submitter_id())->get_name();
        }
    }

}

?>
