<?php

namespace application\weblcms\tool\assignment;

use application\weblcms\Tool;
use application\weblcms\WeblcmsAssignmentSubmissionsTracker;
use application\weblcms\WeblcmsDataManager;
use application\weblcms\WeblcmsManager;
use application\weblcms\WeblcmsRights;
use application\weblcms\WeblcmsSubmissionFeedbackTracker;
use application\weblcms\WeblcmsSubmissionNoteTracker;
use application\weblcms\WeblcmsSubmissionScoreTracker;
use common\libraries\ActionBarRenderer;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\DatetimeUtilities;
use common\libraries\EqualityCondition;
use common\libraries\InCondition;
use common\libraries\Path;
use common\libraries\Request;
use common\libraries\Theme;
use common\libraries\Toolbar;
use common\libraries\ToolbarItem;
use common\libraries\Translation;
use common\libraries\Utilities;
use group\GroupDataManager;
use repository\content_object\description\Description;
use repository\content_object\document\Document;
use repository\ContentObject;
use repository\ContentObjectDisplay;
use repository\ContentObjectRendition;
use repository\ContentObjectRenditionImplementation;
use repository\RepositoryDataManager;
use repository\RepositoryManager;
use user\User;
use user\UserDataManager;

/**
 * This component allows a user to view the details of a single submission.
 *
 * @author Bert De Clercq (Hogeschool Gent)
 * @author Anthony Hurst (Hogeschool Gent)
 */
class AssignmentToolSubmissionViewerComponent extends SubmissionsManager
{
    
    /**
     * The submission of which to show the details.
     * 
     * @var Document
     */
    private $submission;
    private $assignment;
    
    private $score_section;
    private $feedback_section;
    private $notes_section;

    /**
     * Runs this submission viewer component.
     */
    function run()
    {
        $this->validate_forms();
        
        $this->test_view_rights();
        
        $this->change_last_breadcrumb();
        
        $this->display_header();
        $this->display_submission_details();
        $this->display_footer();
    }

    /**
     * This function checks whether or not a user has permission to access this
     * page.
     */
    private function test_view_rights()
    {
        $publication = WeblcmsDataManager :: get_instance()->retrieve_content_object_publication($this->get_publication_id());
        $this->assignment = $publication->get_content_object();
        
        if (! $this->is_allowed(WeblcmsRights :: VIEW_RIGHT, $publication) || ! $this->is_allowed(WeblcmsRights :: VIEW_RIGHT))
        {
            $this->redirect(Translation :: get('NotAllowed', null, Utilities :: COMMON_LIBRARIES), true, array(), array(
                    Tool :: PARAM_ACTION, Tool :: PARAM_PUBLICATION_ID));
        }
        if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT) || $this->assignment->get_visibility_submissions())
        {
            return;
        }
        
        $is_member = false;
        switch ($this->get_submitter_type())
        {
            case WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_COURSE_GROUP :
                $is_member = $this->is_course_group_member($this->get_target_id(), $this->get_user_id());
                break;
            case WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_PLATFORM_GROUP :
                $is_member = $this->is_platform_group_member($this->get_target_id(), $this->get_user_id());
                break;
            case WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_USER :
                $is_member = $this->is_current_user($this->get_target_id());
                break;
        }
        if (! $is_member)
        {
            $this->redirect(Translation :: get('NotAllowed', null, Utilities :: COMMON_LIBRARIES), true, array(), array(
                    Tool :: PARAM_ACTION, Tool :: PARAM_PUBLICATION_ID));
        }
    }

    /**
     * Returns true if the user with given user id is a member of the course
     * group with the given group id.
     * 
     * @param $group_id int           
     * @param $user_id int           
     * @return boolean
     */
    private function is_course_group_member($group_id, $user_id)
    {
        $group = WeblcmsDataManager :: get_instance()->retrieve_course_group($group_id);
        $user = UserDataManager :: get_instance()->retrieve_user($user_id);
        return WeblcmsDataManager :: get_instance()->is_course_group_member($group, $user);
    }

    /**
     * Returns true if the user with the given user id is a member of the
     * platform group with the given group id.
     * 
     * @param $group_id type           
     * @param $user_id type           
     * @return boolean
     */
    private function is_platform_group_member($group_id, $user_id)
    {
        $group = GroupDataManager :: get_instance()->retrieve_group($group_id);
        if (GroupDataManager :: get_instance()->is_group_member($group_id, $user_id))
        {
            return true;
        }
        if ($group->has_children())
        {
            foreach ($group->get_subgroups() as $subgroup)
            {
                if ($this->is_platform_group_member($subgroup->get_id(), $user_id))
                {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * Returns true when the user that is logged in is the same as the user with
     * the given user id, and false otherwise.
     * 
     * @param $user_id int           
     * @return boolean
     */
    private function is_current_user($user_id)
    {
        return $this->get_user_id() == $user_id;
    }

    /**
     * Assigns a value to all the caching variables.
     */
    private function find_data()
    {
        $condition = new EqualityCondition(WeblcmsAssignmentSubmissionsTracker :: PROPERTY_ID, Request :: get(AssignmentTool :: PARAM_SUBMISSION));
        
        // Submission
        $submissions = WeblcmsAssignmentSubmissionsTracker :: get_data(WeblcmsAssignmentSubmissionsTracker :: CLASS_NAME, WeblcmsManager :: APPLICATION_NAME, $condition)->as_array();
        $this->submission = $submissions[0]->get_content_object();
    }

    /**
     * Returns the submission.
     * 
     * @return Document The submission
     */
    function get_submission()
    {
        return $this->submission;
    }

    /**
     * Returns the assignment for the submission.
     * 
     * @return Assignment The assignment
     */
    function get_assignment()
    {
        return WeblcmsDataManager :: get_instance()->retrieve_content_object_publication($this->get_publication_id())->get_content_object();
    }

    /**
     * Returns a submission tracker of the submission currently being viewed.
     * 
     * @return WeblcmsAssignmentSubmissionsTracker The submission tracker
     */
    function get_submission_tracker()
    {
        $condition = new EqualityCondition(WeblcmsAssignmentSubmissionsTracker :: PROPERTY_ID, Request :: get(AssignmentTool :: PARAM_SUBMISSION));
        
        $submission_tracker = new WeblcmsAssignmentSubmissionsTracker();
        $submission_trackers = $submission_tracker->retrieve_tracker_items_result_set($condition)->as_array();
        return $submission_trackers[0];
    }

    /**
     * Returns a score tracker of the submission currently being viewed.
     * 
     * @return WeblcmsSubmissionScoreTracker The score tracker
     */
    function get_score_tracker()
    {
        $condition = new EqualityCondition(WeblcmsSubmissionScoreTracker :: PROPERTY_SUBMISSION_ID, Request :: get(AssignmentTool :: PARAM_SUBMISSION));
        
        $score_tracker = new WeblcmsSubmissionScoreTracker();
        $score_trackers = $score_tracker->retrieve_tracker_items_result_set($condition)->as_array();
        return $score_trackers[0];
    }

    /**
     * Returns feedback trackers of the submission currently being viewed.
     * 
     * @return array The feedback trackers
     */
    function get_feedback_trackers()
    {
        $condition = new EqualityCondition(WeblcmsSubmissionFeedbackTracker :: PROPERTY_SUBMISSION_ID, Request :: get(AssignmentTool :: PARAM_SUBMISSION));
        
        $feedback_tracker = new WeblcmsSubmissionFeedbackTracker();
        return $feedback_tracker->retrieve_tracker_items_result_set($condition)->as_array();
    }

    /**
     * Returns a note tracker of the submission currently being viewed.
     * 
     * @return WeblcmsSubmissionNoteTracker The note tracker
     */
    function get_note_tracker()
    {
        $condition = new EqualityCondition(WeblcmsSubmissionNoteTracker :: PROPERTY_SUBMISSION_ID, Request :: get(AssignmentTool :: PARAM_SUBMISSION));
        
        $note_tracker = new WeblcmsSubmissionNoteTracker();
        $note_trackers = $note_tracker->retrieve_tracker_items_result_set($condition)->as_array();
        return $note_trackers[0];
    }

    /**
     * Returns the name of the submitter as a string.
     * When submitted as a group, it will return the name of the user who
     * submitted followed by the group name.
     * 
     * @return string The name of the submitter
     */
    function get_submitter_name()
    {
        // name of the user who submitted
        $user_name = UserDataManager :: get_instance()->retrieve_user($this->get_submission_tracker()->get_user_id());
        
        switch ($this->get_submitter_type())
        {
            case WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_USER :
                return $user_name->get_fullname();
            case WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_COURSE_GROUP :
                return WeblcmsDataManager :: get_instance()->retrieve_course_group($this->get_target_id())->get_name();
            case WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_PLATFORM_GROUP :
                return GroupDataManager :: get_instance()->retrieve_group($this->get_target_id())->get_name();
        }
    }

    /**
     * Returns additional parameters as an array.
     * 
     * @return array The additional parameters
     */
    function get_additional_parameters()
    {
        return array(Tool :: PARAM_PUBLICATION_ID, AssignmentTool :: PARAM_TARGET_ID, 
                AssignmentTool :: PARAM_SUBMITTER_TYPE, AssignmentTool :: PARAM_SUBMISSION);
    }

    /**
     * Validates the different forms on the page.
     */
    private function validate_forms()
    {
        $this->score_section = new SubmissionDetailScoreSection($this->get_score_tracker(), $this->get_url());
        $this->feedback_section = new SubmissionDetailFeedbackSection($this->get_submission_tracker(), $this, $this->get_url());
        $this->notes_section = new SubmissionDetailNotesSection($this->get_note_tracker(), $this->get_url());
        
        // Score section
        if ($this->score_section->validate())
        {
            
            $this->score_section->set_score();
            $this->redirect(Translation :: get('ScoreUpdated'), null, $this->get_parameters());
        }
        
        // Feedback section
        if ($this->feedback_section->validate())
        {
            $this->feedback_section->set_feedback();
            $this->redirect(Translation :: get('FeedbackCreated'), null, $this->get_parameters());
        }
        
        // Note section
        if ($this->notes_section->validate())
        {
            
            $this->notes_section->set_note();
            $this->redirect(Translation :: get('NoteUpdated'), null, $this->get_parameters());
        }
    }

    /**
     * Displays the details of the submission.
     */
    private function display_submission_details()
    {
        $this->display_navigation_bar();
        $general_info_section = new SubmissionDetailGeneralInfoSection($this);
        
        $general_info_section->render_section();
        $this->display_section(Translation :: get('Score'), $this->get_score_section_content());
        $this->display_section(Translation :: get('Feedback'), $this->get_feedback_section_content());
        
        if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $this->display_section(Translation :: get('PersonalNote'), $this->get_notes_section_content());
        }
    }

    /**
     * Displays the navigation bar.
     */
    private function display_navigation_bar()
    {
        $html = array();
        $html[] = '<div class="announcements level_2" style="background-image:url(' . Theme :: get_common_image_path() . 'content_object/introduction.png;width=100%;">';
        if ($this->assignment->get_visibility_submissions() || $this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $html[] = $this->generate_submitters_navigator();
        }
        $html[] = $this->generate_submissions_navigator();
        $html[] = '</div>';
        $html[] = '<div class="clear">&nbsp;</div><br/>';
        echo implode("\n", $html);
    }

    /**
     * Generates the HTML associated with the navigation bar for the submissions
     * of the current submitter.
     * 
     * @return type the HTML associated with the submissions navigation bar.
     */
    private function generate_submissions_navigator()
    {
        $html = array();
        $html[] = '<div style="text-align:center;">';
        $earlier_submission_url = $this->get_earlier_submission_url();
        if ($earlier_submission_url)
        {
            $html[] = '<a href="' . $earlier_submission_url . '">';
            $html[] = '<img src="' . Theme :: get_common_image_path() . 'action_prev.png"/>';
            $html[] = Translation :: get('EarlierSubmission');
            $html[] = '</a>';
        }
        else
        {
            $html[] = '<img src="' . Theme :: get_common_image_path() . 'action_prev_na.png"/>';
            $html[] = Translation :: get('EarlierSubmission');
        }
        $html[] = ' [' . $this->get_position_submissions($this->get_submission_id()) . '/' . $this->get_count_submissions() . '] ';
        $later_submission_url = $this->get_later_submission_url();
        if ($later_submission_url)
        {
            $html[] = '<a href="' . $later_submission_url . '">';
            $html[] = Translation :: get('LaterSubmission');
            $html[] = '<img src="' . Theme :: get_common_image_path() . 'action_next.png"/>';
            $html[] = '</a>';
        }
        else
        {
            $html[] = Translation :: get('LaterSubmission');
            $html[] = '<img src="' . Theme :: get_common_image_path() . 'action_next_na.png"/>';
        }
        $html[] = '</div>';
        return implode("\n", $html);
    }

    /**
     * Gets the url associated with the previous submission submitted earlier
     * (when ordered chronologically).
     * 
     * @return type the url associated with the submission submitted earlier or
     *         null if none found.
     */
    private function get_earlier_submission_url()
    {
        $earlier_submission_information = $this->get_earlier_submission_information($this->get_submitter_type(), $this->get_target_id(), $this->get_submission_id());
        if ($earlier_submission_information)
        {
            return $this->get_url(array(AssignmentTool :: PARAM_SUBMISSION => $earlier_submission_information));
        }
        return null;
    }

    /**
     * Gets the url associated with the next submission submitted later (when
     * ordered chronologically).
     * 
     * @return type the url associated with the submission submitted later or
     *         null if none found.
     */
    private function get_later_submission_url()
    {
        $later_submission_information = $this->get_later_submission_information($this->get_submitter_type(), $this->get_target_id(), $this->get_submission_id());
        if ($later_submission_information)
        {
            return $this->get_url(array(AssignmentTool :: PARAM_SUBMISSION => $later_submission_information));
        }
        return null;
    }

    /**
     * Creates the navigation bar for navigating between submitters.
     * 
     * @return type the HTML associated with the submitters navigation bar.
     */
    private function generate_submitters_navigator()
    {
        $html = array();
        $html[] = '<div style="text-align:center;">';
        $previous_submitter_url = $this->get_previous_submitter_url();
        if ($previous_submitter_url)
        {
            $html[] = '<a href="' . $previous_submitter_url . '">';
            $html[] = '<img src="' . Theme :: get_common_image_path() . 'action_prev.png"/>';
            $html[] = Translation :: get('PreviousSubmitter');
            $html[] = '</a>';
        }
        else
        {
            $html[] = '<img src="' . Theme :: get_common_image_path() . 'action_prev_na.png"/>';
            $html[] = Translation :: get('PreviousSubmitter');
        }
        $html[] = ' [' . $this->get_position_submitter_with_submissions($this->get_submitter_type(), $this->get_target_id()) . '/' . $this->get_count_submitters($this->get_submitter_type()) . '] ';
        $next_submitter_url = $this->get_next_submitter_url();
        if ($next_submitter_url)
        {
            $html[] = '<a href="' . $next_submitter_url . '">';
            $html[] = Translation :: get('NextSubmitter');
            $html[] = '<img src="' . Theme :: get_common_image_path() . 'action_next.png"/>';
            $html[] = '</a>';
        }
        else
        {
            $html[] = Translation :: get('NextSubmitter');
            $html[] = '<img src="' . Theme :: get_common_image_path() . 'action_next_na.png"/>';
        }
        $html[] = '</div>';
        return implode("\n", $html);
    }

    /**
     * Gets the url for the previous submitter with submissions.
     * 
     * @return type the url of the previous submitter's latest submission or
     *         null if none found.
     */
    private function get_previous_submitter_url()
    {
        $previous_submitter_information = $this->get_previous_submitter_information($this->get_submitter_type(), $this->get_target_id());
        if (! $previous_submitter_information)
        {
            return null;
        }
        while (! $previous_submitter_information[WeblcmsAssignmentSubmissionsTracker :: PROPERTY_ID])
        {
            $previous_submitter_information = $this->get_previous_submitter_information($previous_submitter_information[WeblcmsAssignmentSubmissionsTracker :: PROPERTY_SUBMITTER_TYPE], $previous_submitter_information[WeblcmsAssignmentSubmissionsTracker :: PROPERTY_SUBMITTER_ID]);
            if (! $previous_submitter_information)
            {
                return null;
            }
        }
        return $this->get_url(array(
                AssignmentTool :: PARAM_TARGET_ID => $previous_submitter_information[WeblcmsAssignmentSubmissionsTracker :: PROPERTY_SUBMITTER_ID], 
                AssignmentTool :: PARAM_SUBMITTER_TYPE => $previous_submitter_information[WeblcmsAssignmentSubmissionsTracker :: PROPERTY_SUBMITTER_TYPE], 
                AssignmentTool :: PARAM_SUBMISSION => $previous_submitter_information[WeblcmsAssignmentSubmissionsTracker :: PROPERTY_ID]));
    }

    /**
     * Gets the url for the next submitter with submissions.
     * 
     * @return type the url of the next submitter's latest submission or null if
     *         none found.
     */
    private function get_next_submitter_url()
    {
        $next_submitter_information = $this->get_next_submitter_information($this->get_submitter_type(), $this->get_target_id());
        if (! $next_submitter_information)
        {
            return null;
        }
        while (! $next_submitter_information[WeblcmsAssignmentSubmissionsTracker :: PROPERTY_ID])
        {
            $next_submitter_information = $this->get_next_submitter_information($next_submitter_information[WeblcmsAssignmentSubmissionsTracker :: PROPERTY_SUBMITTER_TYPE], $next_submitter_information[WeblcmsAssignmentSubmissionsTracker :: PROPERTY_SUBMITTER_ID]);
            if (! $next_submitter_information)
            {
                return null;
            }
        }
        return $this->get_url(array(
                AssignmentTool :: PARAM_TARGET_ID => $next_submitter_information[WeblcmsAssignmentSubmissionsTracker :: PROPERTY_SUBMITTER_ID], 
                AssignmentTool :: PARAM_SUBMITTER_TYPE => $next_submitter_information[WeblcmsAssignmentSubmissionsTracker :: PROPERTY_SUBMITTER_TYPE], 
                AssignmentTool :: PARAM_SUBMISSION => $next_submitter_information[WeblcmsAssignmentSubmissionsTracker :: PROPERTY_ID]));
    }

    /**
     * Returns the content of the score section.
     * 
     * @return array The score content
     */
    private function get_score_section_content()
    {
        $html = array();
        
        // Course admin can edit score
        if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            // Show the score form
            $html[] = $this->score_section->toHtml();
        }
        // Students can only see their score and not edit
        else
        {
            $html[] = '<div style="font-weight:bold;float:left;">';
            $html[] = Translation :: get('Score') . ':&nbsp;<br />';
            $html[] = '</div>';
            
            $score_tracker = $this->get_score_tracker();
            if ($score_tracker)
            {
                $html[] = $this->get_score_tracker()->get_score() . '%<br />';
            }
            else
            {
                $html[] = '-<br />';
            }
            $html[] = '<div style="float:left;">';
            $html[] = '</div>';
        }
        
        return implode("\n", $html);
    }

    /**
     * Returns the content of the feedback section.
     * 
     * @return array The feedback content
     */
    private function get_feedback_section_content()
    {
        $html = array();
        
        // Show the feedbacks
        $html[] = $this->get_feedbacks_html();
        
        // Show the feedback form (only for course admins)
        if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $html[] = $this->feedback_section->toHtml();
        }
        
        return implode("\n", $html);
    }

    /**
     * Returns the feedbacks with their creation date as html code.
     * 
     * @return array The feedbacks in html code
     */
    private function get_feedbacks_html()
    {
        $html = array();
        
        $feedback_trackers = $this->get_feedback_trackers();
        
        if (count($feedback_trackers) == 0 && ! $this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $html[] = Translation :: get('NoFeedbacks');
        }
        else
        {
            foreach ($feedback_trackers as $feedback_tracker)
            {
                $html[] = $this->get_feedback_html($feedback_tracker);
            }
        }
        
        return implode("\n", $html);
    }

    /**
     * Returns the display as html code of the given feedback tracker.
     * 
     * @param $feedback_tracker WeblcmsSubmissionFeedbackTracker           
     * @return array The display of the feedback tracker
     */
    private function get_feedback_html($feedback_tracker)
    {
        $submission_tracker = $this->get_submission_tracker();
        $feedback = $feedback_tracker->get_content_object();
        
        $html = array();
        
        $html[] = '<div class="announcements level_1" style="border-style:solid;border-width:1px;border-color:#CCC;">';
        
        // Show the content object
        $html[] = '<div>';
        $display = ContentObjectRenditionImplementation :: factory($feedback, ContentObjectRendition :: FORMAT_HTML, ContentObjectRendition :: VIEW_FULL, $this);
        $html[] = $display->render();
        $html[] = '</div>';
        
        // Publication info
        $html[] = '<div class="publication_info">';
        $html[] = Translation :: get('CreatedOn');
        $html[] = $this->format_date($feedback_tracker->get_created());
        $html[] = Translation :: get('By', null, Utilities :: COMMON_LIBRARIES);
        $html[] = UserDataManager :: get_instance()->retrieve_user($feedback_tracker->get_user_id())->get_fullname();
        $html[] = '</div>';
        
        // Toolbar (only for course admins)
        if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $toolbar = new Toolbar();
            // Edit button
            $toolbar->add_item(new ToolbarItem(Translation :: get('EditFeedback'), Theme :: get_common_image_path() . 'action_edit.png', $this->get_url(array(
                    Tool :: PARAM_ACTION => AssignmentTool :: ACTION_EDIT_FEEDBACK, 
                    AssignmentTool :: PARAM_FEEDBACK_ID => $feedback_tracker->get_id(), 
                    Tool :: PARAM_PUBLICATION_ID => $submission_tracker->get_publication_id(), 
                    AssignmentTool :: PARAM_SUBMISSION => $submission_tracker->get_id())), ToolbarItem :: DISPLAY_ICON));
            // Remove button
            $toolbar->add_item(new ToolbarItem(Translation :: get('DeleteFeedback'), Theme :: get_common_image_path() . 'action_delete.png', $this->get_url(array(
                    Tool :: PARAM_ACTION => AssignmentTool :: ACTION_DELETE_FEEDBACK, 
                    AssignmentTool :: PARAM_FEEDBACK_ID => $feedback_tracker->get_id(), 
                    Tool :: PARAM_PUBLICATION_ID => $submission_tracker->get_publication_id())), ToolbarItem :: DISPLAY_ICON, true));
            
            $html[] = '<div class="submission_toolbar">' . $toolbar->as_html() . '</div>';
        }
        $html[] = '</div><div class="clear">&nbsp;</div><br />';
        
        return implode("\n", $html);
    }

    /**
     * Returns the content of the notes section.
     * 
     * @return array The notes content
     */
    private function get_notes_section_content()
    {
        $html = array();
        
        $html[] = $this->notes_section->toHtml();
        
        return implode("\n", $html);
    }

    /**
     * Displays the title of a section and the content.
     * 
     * @param $title string
     *            The title for the section
     * @param $content array
     *            The content as an array of string in html code
     */
    private function display_section($title, $content)
    {
        $html = array();
        
        $html[] = $this->set_section_content($title, $content);
        
        echo implode("\n", $html);
    }

    /**
     * Sets the title and content of a section.
     * 
     * @param $title string
     *            The title for the section
     * @param $html_content array
     *            The content of the section as an array of html code
     * @return array The content
     */
    function set_section_content($title, $html_content)
    {
        $html = array();
        
        $html[] = '<div class="announcements level_1">';
        
        // Title
        $html[] = '<div class="title" style="width:100%;">';
        $html[] = Translation :: get($title);
        $html[] = '</div><div class="clear">&nbsp;</div><br />';
        
        // Content
        $html[] = $html_content;
        
        // Closes the section
        $html[] = '</div><br />';
        
        return implode("\n", $html);
    }

    /**
     * Returns a formatted string of the given date.
     * 
     * @param $date type           
     * @return string The formatted date
     */
    private function format_date($date)
    {
        $formatted_date = DatetimeUtilities :: format_locale_date(Translation :: get('DateTimeFormatLong', null, Utilities :: COMMON_LIBRARIES), $date);
        
        return $formatted_date;
    }

    /**
     * Displays the header.
     */
    function display_header()
    {
        parent :: display_header();
        
        $this->action_bar = $this->get_action_bar();
        
        if ($this->action_bar)
        {
            echo $this->action_bar->as_html();
        }
    }

    /**
     * Returns the action bar with actions.
     * 
     * @return ActionBarRenderer The action bar
     */
    private function get_action_bar()
    {
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);
        
        $action_bar->add_common_action(new ToolbarItem(Translation :: get('ViewSubmission'), Theme :: get_common_image_path() . 'action_browser.png', 'javascript:openPopup(\'' . $this->generate_attachment_viewer_url($this->submission, AssignmentToolAttachmentViewerComponent :: TYPE_SUBMISSION) . '\');void(0);', ToolbarItem :: DISPLAY_ICON_AND_LABEL));
        
        if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            if (self :: is_downloadable($this->submission))
            {
                $action_bar->add_common_action(new ToolbarItem(Translation :: get('DownloadSubmission'), Theme :: get_common_image_path() . 'action_download.png', $this->get_url(array(
                        Tool :: PARAM_ACTION => AssignmentTool :: ACTION_DOWNLOAD_SUBMISSIONS, 
                        AssignmentTool :: PARAM_SUBMISSION => $this->get_submission_tracker()->get_id())), ToolbarItem :: DISPLAY_ICON_AND_LABEL));
            }
            
            $action_bar->add_common_action(new ToolbarItem(Translation :: get('DeleteSubmission'), Theme :: get_common_image_path() . 'action_delete.png', $this->get_url(array(
                    Tool :: PARAM_ACTION => AssignmentTool :: ACTION_DELETE_SUBMISSION, 
                    AssignmentTool :: PARAM_SUBMISSION => $this->get_submission_tracker()->get_id())), ToolbarItem :: DISPLAY_ICON_AND_LABEL, true));
            
            $action_bar->add_common_action(new ToolbarItem(Translation :: get('AddFeedback'), Theme :: get_image_path() . 'give_feedback.png', $this->get_url(array(
                    Tool :: PARAM_ACTION => AssignmentTool :: ACTION_GIVE_FEEDBACK, 
                    AssignmentTool :: PARAM_SUBMISSION => $this->get_submission_tracker()->get_id(), 
                    Tool :: PARAM_PUBLICATION_ID => $this->get_publication_id(), 
                    AssignmentTool :: PARAM_TARGET_ID => $this->get_target_id(), 
                    AssignmentTool :: PARAM_SUBMITTER_TYPE => $this->get_submitter_type())), ToolbarItem :: DISPLAY_ICON_AND_LABEL));
        }
        
        return $action_bar;
    }

    /**
     * Adds additional breadcrumbs.
     * 
     * @param $breadcrumbtrail BreadcrumbTrail
     *            The current trail
     */
    function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $this->find_data();
        
//         $breadcrumbtrail->add(new Breadcrumb($this->get_url(array(Tool :: PARAM_ACTION => Tool :: ACTION_BROWSE)), Translation :: get('AssignmentToolBrowserComponent')));
        
        $breadcrumbtrail->add(new Breadcrumb($this->define_browse_submitters_action_url(), WeblcmsDataManager :: get_instance()->retrieve_content_object_publication($this->get_publication_id())->get_content_object()->get_title()));
        
        $breadcrumbtrail->add(new Breadcrumb($this->get_url(array(
                Tool :: PARAM_ACTION => AssignmentTool :: ACTION_BROWSE_SUBMISSIONS, 
                Tool :: PARAM_PUBLICATION_ID => $this->get_publication_id(), 
                AssignmentTool :: PARAM_TARGET_ID => $this->get_target_id(), 
                AssignmentTool :: PARAM_SUBMITTER_TYPE => $this->get_submitter_type())), $this->get_submitter_name()));
    }

    /**
     * Returns an url for the submitters browser based on who clicked on the
     * link.
     * 
     * @return string The url
     */
    private function define_browse_submitters_action_url()
    {
        if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            return $this->get_url(array(Tool :: PARAM_ACTION => AssignmentTool :: ACTION_BROWSE_SUBMITTERS, 
                    Tool :: PARAM_PUBLICATION_ID => $this->get_publication_id()));
        }
        return $this->get_url(array(Tool :: PARAM_ACTION => AssignmentTool :: ACTION_STUDENT_BROWSE_SUBMISSIONS, 
                Tool :: PARAM_PUBLICATION_ID => $this->get_publication_id()));
    }

    /**
     * Changes the last breadcrumb to the name of the submission.
     */
    private function change_last_breadcrumb()
    {
        $breadcrumb_trail = BreadcrumbTrail :: get_instance();
        $breadcrumbs = $breadcrumb_trail->get_breadcrumbs();
        
        $breadcrumbs[$breadcrumb_trail->size() - 1] = new Breadcrumb($this->get_url(array(
                Tool :: PARAM_ACTION => AssignmentTool :: ACTION_VIEW_SUBMISSION, 
                Tool :: PARAM_PUBLICATION_ID => $this->get_publication_id(), 
                AssignmentTool :: PARAM_TARGET_ID => $this->get_target_id(), 
                AssignmentTool :: PARAM_SUBMITTER_TYPE => $this->get_submitter_type())), $this->submission->get_title() . ' - ' . Translation :: get('Detail'));
        
        $breadcrumb_trail->set_breadcrumbtrail($breadcrumbs);
    }

}

?>
