<?php
namespace application\weblcms\tool\assignment;

use application\weblcms\Tool;
use application\weblcms\WeblcmsAssignmentSubmissionsTracker;
use application\weblcms\WeblcmsDataManager;
use application\weblcms\WeblcmsRights;
use application\weblcms\WeblcmsSubmissionFeedbackTracker;
use application\weblcms\WeblcmsSubmissionScoreTracker;
use common\libraries\DataClassRetrievesParameters;
use common\libraries\DatetimeUtilities;
use common\libraries\EqualityCondition;
use common\libraries\InCondition;
use common\libraries\ObjectTableOrder;
use common\libraries\Theme;
use common\libraries\Toolbar;
use common\libraries\ToolbarItem;
use common\libraries\Translation;
use common\libraries\Utilities;
use group\GroupDataManager;
use repository\DefaultContentObjectTableCellRenderer;
use user\User;
use user\DataManager;

/**
 * Description of student_submissions_browser_own_groups_table_cell_renderer
 *
 * @author Anthony Hurst (Hogeschool Gent)
 */
class StudentSubmissionsBrowserOwnGroupsTableCellRenderer extends DefaultContentObjectTableCellRenderer
{
    private $browser;

    public function __construct($browser)
    {
        $this->browser = $browser;
    }

    public function render_cell($column, $submission)
    {
        if ($column === StudentSubmissionsBrowserOwnGroupsTableColumnModel :: get_action_column())
            return $this->get_actions($submission);
        switch ($column->get_name())
        {
            case StudentSubmissionsBrowserOwnGroupsTableColumnModel :: PROPERTY_PUBLICATION_TITLE :
                return $this->construct_title_link($submission);
            case StudentSubmissionsBrowserOwnGroupsTableColumnModel :: PROPERTY_CONTENT_OBJECT_DESCRIPTION :
                return $submission->get_content_object()->get_description();
            case WeblcmsAssignmentSubmissionsTracker :: PROPERTY_DATE_SUBMITTED :
                return $this->format_date($submission->get_date_submitted());
            case WeblcmsAssignmentSubmissionsTracker :: PROPERTY_SUBMITTER_ID :
                return $this->get_submitter_name($submission);
            case AssignmentTool :: PROPERTY_NAME :
                return $this->get_group_name($submission);
            case AssignmentTool :: PROPERTY_GROUP_MEMBERS :
                return $this->get_group_members($submission);
            case WeblcmsSubmissionScoreTracker :: PROPERTY_SCORE :
                return $this->get_score_from_submission($submission->get_id());
            case AssignmentTool :: PROPERTY_NUMBER_OF_FEEDBACKS :
                return $this->get_number_of_feedback($submission->get_id());
        }
    }

    /**
     * Returns the submitter name of the user who submitted the given
     * submission.
     * 
     * @param $submission type           
     * @return string The submitter name
     */
    private function get_submitter_name($submission)
    {
        return DataManager :: get_instance()->retrieve_user($submission->get_user_id())->get_fullname();
    }

    /**
     * Returns the group members that are part of the submission.
     * 
     * @param $submission type           
     * @return array
     */
    private function get_group_members($submission)
    {
        $retrieve_limit = 21;
        $order_properties = array();
        $order_properties[] = new ObjectTableOrder(User :: PROPERTY_LASTNAME);
        $order_properties[] = new ObjectTableOrder(User :: PROPERTY_FIRSTNAME);
        $users = array();
        
        switch ($submission->get_submitter_type())
        {
            case WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_PLATFORM_GROUP :
                $user_ids = GroupDataManager :: get_instance()->retrieve_group($submission->get_submitter_id())->get_users(true, true);
                $condition = new InCondition(User :: PROPERTY_ID, $user_ids);
                $users = DataManager :: retrieves(User :: CLASS_NAME, new DataClassRetrievesParameters($condition, $retrieve_limit, null, $order_properties))->as_array();
                break;
            case WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_COURSE_GROUP :
                $group = WeblcmsDataManager :: get_instance()->retrieve_course_group($submission->get_submitter_id());
                $users = WeblcmsDataManager :: get_instance()->retrieve_course_group_users($group, null, null, $retrieve_limit, $order_properties)->as_array();
                break;
        }
        
        if (count($users) == 0)
        {
            return null;
        }
        
        $display_limit_breached = false;
        if (count($users) == $retrieve_limit)
        {
            $display_limit_breached = true;
            array_pop($users);
        }
        
        $html = array();
        $html[] = '<select style="width:180px">';
        foreach ($users as $user)
        {
            $html[] = '<option>' . $user->get_fullname() . '</option>';
        }
        if ($display_limit_breached)
        {
            $html[] = '<option>...</option>';
        }
        $html[] = '</select>';
        return implode("\n", $html);
    }

    /**
     * Retrieves the group name of the group that submitted the given
     * submission.
     * 
     * @param $submission type           
     * @return string The group name
     */
    private function get_group_name($submission)
    {
        switch ($submission->get_submitter_type())
        {
            case WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_COURSE_GROUP :
                $group = WeblcmsDataManager :: get_instance()->retrieve_course_group($submission->get_submitter_id());
                break;
            case WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_PLATFORM_GROUP :
                $group = GroupDataManager :: get_instance()->retrieve_group($submission->get_submitter_id());
                break;
        }
        return $group->get_name();
    }

    /**
     * Retrieves the score from the submission with the given submission id.
     * 
     * @param $submission_id int           
     * @return mixed The score or null
     */
    private function get_score_from_submission($submission_id)
    {
        $tracker = new WeblcmsSubmissionScoreTracker();
        $condition = new EqualityCondition(WeblcmsSubmissionScoreTracker :: PROPERTY_SUBMISSION_ID, $submission_id);
        $trackers = $tracker->retrieve_tracker_items($condition);
        
        if (count($trackers) > 0)
        {
            $score_tracker = $trackers[0];
            return $score_tracker->get_score() . '%';
        }
        return null;
    }

    /**
     * Returns the number of feedback the submission with the given submission
     * id has.
     * 
     * @param $submission_id int           
     * @return int The number of feedback
     */
    private function get_number_of_feedback($submission_id)
    {
        $tracker = new WeblcmsSubmissionFeedbackTracker();
        $condition = new EqualityCondition(WeblcmsSubmissionFeedbackTracker :: PROPERTY_SUBMISSION_ID, $submission_id);
        return $tracker->count_tracker_items($condition);
    }

    /**
     * Makes a hyperlink of the given submission.
     * 
     * @param $submission type           
     * @return string The hyperlink
     */
    private function construct_title_link($submission)
    {
        return '<a href="' . $this->construct_title_url($submission) . '">' . $submission->get_content_object()->get_title() . '</a>';
    }

    /**
     * Returns the url that links to the submission viewer for the given
     * submission.
     * 
     * @param $submission type           
     * @return string The url
     */
    private function construct_title_url($submission)
    {
        return $this->browser->get_url(array(AssignmentTool :: PARAM_SUBMISSION => $submission->get_id(), 
                Tool :: PARAM_ACTION => AssignmentTool :: ACTION_VIEW_SUBMISSION, 
                AssignmentTool :: PARAM_TARGET_ID => $submission->get_submitter_id(), 
                AssignmentTool :: PARAM_SUBMITTER_TYPE => $submission->get_submitter_type()));
    }

    /**
     * Creates a toolbar with the appropriate actions
     * 
     * @param $submission type           
     * @return string The HTML code that represents the actions.
     */
    private function get_actions($submission)
    {
        $toolbar = new Toolbar();
        if ($submission->get_user_id() == $this->browser->get_user_id() || $this->browser->get_assignment()->get_visibility_submissions() || $this->browser->is_allowed(WeblcmsRights :: VIEW_RIGHT))
        {
            $toolbar->add_item(new ToolbarItem(Translation :: get('ViewSubmission'), Theme :: get_common_image_path() . 'action_browser.png', $this->browser->get_url(array(
                    AssignmentTool :: PARAM_SUBMISSION => $submission->get_id(), 
                    Tool :: PARAM_ACTION => AssignmentTool :: ACTION_VIEW_SUBMISSION)), ToolbarItem :: DISPLAY_ICON));
        }
        if ($this->browser->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $toolbar->add_item(new ToolbarItem(Translation :: get('DownloadSubmission'), Theme :: get_common_image_path() . 'action_download.png', $this->browser->get_url(), ToolbarItem :: DISPLAY_ICON));
            $toolbar->add_item(new ToolbarItem(Translation :: get('DeleteSubmission'), Theme :: get_common_image_path() . 'action_delete.png', $this->browser->get_url(), ToolbarItem :: DISPLAY_ICON));
        }
        return $toolbar->as_html();
    }

    /**
     * Formats the given date so it display red when it's greater than the end
     * time of the assignment.
     * 
     * @param $date type           
     * @return string The formatted date
     */
    private function format_date($date)
    {
        $formatted_date = DatetimeUtilities :: format_locale_date(Translation :: get('DateTimeFormatLong', null, Utilities :: COMMON_LIBRARIES), $date);
        
        if ($date > $this->browser->get_assignment()->get_end_time())
        {
            return '<span style="color:red">' . $formatted_date . '</span>';
        }
        return $formatted_date;
    }
}

?>
