<?php

namespace application\weblcms\tool\assignment;

use common\libraries\DelegateComponent;

use application\weblcms\CourseGroup;
use application\weblcms\tool\ephorus\EphorusTool;
use application\weblcms\Tool;
use application\weblcms\tool\reporting\ReportingTool;
use application\weblcms\WeblcmsAssignmentSubmissionsTracker;
use application\weblcms\WeblcmsDataManager;
use application\weblcms\WeblcmsManager;
use application\weblcms\WeblcmsRights;
use common\libraries\ActionBarRenderer;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\DatetimeUtilities;
use common\libraries\DynamicVisualTab;
use common\libraries\DynamicVisualTabsRenderer;
use common\libraries\OrCondition;
use common\libraries\Path;
use common\libraries\PatternMatchCondition;
use common\libraries\Redirect;
use common\libraries\Request;
use common\libraries\Theme;
use common\libraries\ToolbarItem;
use common\libraries\Translation;
use common\libraries\Utilities;
use group\Group;
use reporting\Reporting;
use reporting\ReportingManager;
use repository\ContentObject;
use repository\RepositoryManager;
use user\User;

/**
 *
 * @package application.weblcms.tool.assignment.php.component
 *         
 *          Browser of the users and groups the assignment was published for,
 *          with their submission details
 *         
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 * @author Anthony Hurst (Hogeschool Gent)
 * @author Bert De Clercq (Hogeschool Gent)
 */
class AssignmentToolSubmittersBrowserComponent extends SubmissionsManager implements DelegateComponent
{
    const TYPE_INDIVIDUAL = 'individual';
    const TYPE_COURSE_GROUP = 'course_group';
    const TYPE_GROUP = 'group';
    
    /**
     * The assignment content object
     *
     * @var Assignment
     */
    public $assignment;
    
    /**
     * All users that need to be shown in the individual submitters list.
     *
     * @var array WeblcmsAssignmentSubmissionsTracker
     */
    private $users;
    
    /**
     * All course groups that need to be shown on the course groups tab.
     *
     * @var array WeblcmsAssignmentSubmissionsTracker
     */
    private $course_groups;
    
    /**
     * All platform groups that need to be shown on the platform groups tab.
     *
     * @var array WeblcmsAssignmentSubmissionsTracker
     */
    private $platform_groups;

    function run()
    {
        $weblcms_manager = WeblcmsDataManager :: get_instance();
        $publication = $weblcms_manager->retrieve_content_object_publication($this->get_publication_id());
        
        if (! $this->is_allowed(WeblcmsRights :: VIEW_RIGHT, $publication) || ! $this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            if ($this->is_allowed(WeblcmsRights :: VIEW_RIGHT, $publication) && $this->is_allowed(WeblcmsRights :: VIEW_RIGHT))
            {
                $this->redirect(null, false, array(
                        Tool :: PARAM_ACTION => AssignmentTool :: ACTION_STUDENT_BROWSE_SUBMISSIONS, 
                        Tool :: PARAM_PUBLICATION_ID => $publication->get_id()));
            }
            $this->redirect(Translation :: get("NotAllowed", null, Utilities :: COMMON_LIBRARIES), true, array(), array(
                    Tool :: PARAM_ACTION, Tool :: PARAM_PUBLICATION_ID));
        }
        $this->assignment = $publication->get_content_object();
        
        $breadcrumb_trail = BreadcrumbTrail :: get_instance();
        $breadcrumbs = $breadcrumb_trail->get_breadcrumbs();
        $breadcrumbs[$breadcrumb_trail->size() - 1] = new Breadcrumb($this->get_url(array(
                Tool :: PARAM_ACTION => AssignmentTool :: ACTION_BROWSE_SUBMITTERS, 
                Tool :: PARAM_PUBLICATION_ID => $this->get_publication_id())), $this->assignment->get_title());
        $breadcrumb_trail->set_breadcrumbtrail($breadcrumbs);
        
        $this->get_submitters_data();
        $this->browse_submitters();
    }

    /**
     * Returns the assignment.
     *
     * @return Assignment
     */
    function get_assignment()
    {
        return $this->assignment;
    }

    /**
     * Gets the users, course groups and platform groups the assignment was
     * published for
     * plus the number of their submissions and feedbacks
     */
    private function get_submitters_data()
    {
        $manager = WeblcmsDataManager :: get_instance();
        
        $publication = $manager->retrieve_content_object_publication($this->get_publication_id());
        
        // Users
        $this->users = WeblcmsDataManager :: get_publication_target_users_by_publication_id($this->get_publication_id());
        // Course groups
        $this->course_groups = $manager->retrieve_publication_target_course_groups($this->get_publication_id(), $publication->get_course_id())->as_array();
        // Platform groups
        $this->platform_groups = $manager->retrieve_publication_target_platform_groups($this->get_publication_id(), $publication->get_course_id())->as_array();
    }

    /**
     * Displays assignment details and tabs with a submitters browser table per
     * submitter type
     */
    function browse_submitters()
    {
        $visible = $this->display_header();
        if (! $visible)
        {
            return;
        }
        
        echo '<div class="announcements level_1" style="background-image: url(' . Theme :: get_common_image_path() . 'content_object/introduction.png);">';
        echo $this->generate_assignment_details_html();
        echo $this->get_reporting_html();
        echo '</div>';
        
        // retrieve group submissions allowed
        $publication = WeblcmsDataManager :: get_instance()->retrieve_content_object_publication($this->get_publication_id());
        $assignment = $publication->get_content_object();
        $allow_group_submissions = $assignment->get_allow_group_submissions();
        
        if ($allow_group_submissions == 0)
        {
            $users_table = new SubmissionUsersBrowserTable($this, $this->get_search_condition());
            $users_html = $users_table->as_html();
            echo $users_html;
        }
        else
        {
            // Create tabs per submitter type
            $type = Request :: get(AssignmentTool :: PARAM_TYPE);
            if ($type == null)
            {
                $type = self :: TYPE_COURSE_GROUP;
            }
            switch ($type)
            {
                case self :: TYPE_COURSE_GROUP :
                    $course_groups_table = new SubmissionGroupsBrowserTable($this, WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_COURSE_GROUP, $this->get_search_condition($type));
                    $content_table = $course_groups_table->as_html();
                    $selected_course_group = true;
                    break;
                case self :: TYPE_GROUP :
                    $groups_table = new SubmissionGroupsBrowserTable($this, WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_PLATFORM_GROUP, $this->get_search_condition($type));
                    $content_table = $groups_table->as_html();
                    $parameters[] = $parameters[AssignmentTool :: PARAM_TYPE] = self :: TYPE_GROUP;
                    $selected_group = true;
                    break;
            }
            $parameters_course_group = $this->get_parameters();
            $parameters_course_group[AssignmentTool :: PARAM_TYPE] = self :: TYPE_COURSE_GROUP;
            $link_course_group = $this->get_url($parameters_course_group);
            $parameters_group = $this->get_parameters();
            $parameters_group[AssignmentTool :: PARAM_TYPE] = self :: TYPE_GROUP;
            $link_group = $this->get_url($parameters_group);
            
            $tabs = new DynamicVisualTabsRenderer('submissions', $content_table);
            $tab_course_group = new DynamicVisualTab('tab_course_group', Translation :: get('CourseGroups'), null, $link_course_group, $selected_course_group);
            $tabs->add_tab($tab_course_group);
            $tab_group = new DynamicVisualTab('tab_group', Translation :: get('Groups'), null, $link_group, $selected_group);
            $tabs->add_tab($tab_group);
            echo $tabs->render();
        }
        
        $this->display_footer();
    }

    /**
     * Returns the reporting as an array of strings.
     *
     * @return array The reporting
     */
    private function get_reporting_html()
    {
        $html = array();
        
        // Reporting title
        $html[] = '<div class="title" style="border-bottom:1px dotted #D3D3D3;width:100%;border-top:1px dotted #D3D3D3;
            padding-top:5px">';
        $html[] = Translation :: get('Reporting');
        $html[] = '</div><div class="clear">&nbsp;</div><br />';
        
        $html[] = '<div style="font-weight:bold;float:left">';
        
        if (! $this->assignment->get_allow_group_submissions())
        {
            $html[] = Translation :: get('UsersSubmissions') . ':&nbsp;<br />';
            $html[] = Translation :: get('UsersFeedback') . ':&nbsp;<br />';
            
            if ($this->assignment->get_allow_late_submissions())
            {
                $html[] = Translation :: get('UsersLateSubmissions') . ':&nbsp;<br />';
            }
        }
        else
        {
            $type = Request :: get(AssignmentTool :: PARAM_TYPE);
            
            if ($type == null)
            {
                $type = self :: TYPE_COURSE_GROUP;
            }
            switch ($type)
            {
                case self :: TYPE_COURSE_GROUP :
                    $html[] = Translation :: get('CourseGroupsSubmissions') . ':&nbsp;<br />';
                    $html[] = Translation :: get('CourseGroupsFeedback') . ':&nbsp;<br />';
                    if ($this->assignment->get_allow_late_submissions())
                    {
                        $html[] = Translation :: get('CourseGroupsLateSubmissions') . ':&nbsp;<br />';
                    }
                    break;
                
                case self :: TYPE_GROUP :
                    $html[] = Translation :: get('PlatformGroupsSubmissions') . ':&nbsp;<br />';
                    $html[] = Translation :: get('PlatformGroupsFeedback') . ':&nbsp;<br />';
                    if ($this->assignment->get_allow_late_submissions())
                    {
                        $html[] = Translation :: get('PlatformGroupsLateSubmissions') . ':&nbsp;<br />';
                    }
                    break;
            }
        }
        $html[] = '</div>';
        
        // Reporting data
        $html[] = '<div style="float:left">';
        $html[] = $this->get_reporting_block_html();
        $html[] = '</div>';
        
        return implode("\n", $html);
    }

    /**
     * Returns the reporting data block as an array of strings.
     *
     * @return array The reporting data block
     */
    private function get_reporting_block_html()
    {
        if (! $this->assignment->get_allow_group_submissions())
        {
            return $this->get_reporting_data_html(WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_USER);
        }
        else
        {
            $type = Request :: get(AssignmentTool :: PARAM_TYPE);
            
            if ($type == null)
            {
                $type = self :: TYPE_COURSE_GROUP;
            }
            switch ($type)
            {
                case self :: TYPE_COURSE_GROUP :
                    return $this->get_reporting_data_html(WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_COURSE_GROUP);
                case self :: TYPE_GROUP :
                    return $this->get_reporting_data_html(WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_PLATFORM_GROUP);
            }
        }
    }

    /**
     * Returns the reporting data from the specified type as an array of
     * strings.
     *
     * @param $submitter_type int
     *            The type of submitter
     * @return array The reporting data
     */
    private function get_reporting_data_html($submitter_type)
    {
        $count_submissions = 0;
        $count_feedbacks = 0;
        $count_late_submissions = 0;
        
        $submitters = array();
        
        $html = array();
        
        switch ($submitter_type)
        {
            case WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_USER :
                $submitters = $this->users;
                break;
            case WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_COURSE_GROUP :
                $submitters = $this->course_groups;
                break;
            case WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_PLATFORM_GROUP :
                $submitters = $this->platform_groups;
                break;
        }
        
        foreach ($submitters as $submitter)
        {
            $submissions_tracker = $this->get_submissions_tracker($submitter_type, $submitter->get_id());
            $feedbacks_tracker = $this->get_submitter_feedbacks($submitter_type, $submitter->get_id());
            
            if ($submissions_tracker)
            {
                $count_submissions ++;
                
                if ($submissions_tracker->get_optional_property('last_date') > $this->assignment->get_end_time())
                {
                    $count_late_submissions ++;
                }
            }
            
            if ($feedbacks_tracker)
            {
                $count_feedbacks ++;
            }
        }
        
        $html[] = $count_submissions . '/' . count($submitters) . '<br />';
        $html[] = $count_feedbacks . '/' . count($submitters) . '<br />';
        if ($this->assignment->get_allow_late_submissions())
        {
            $html[] = $count_late_submissions . '/' . count($submitters) . '<br />';
        }
        
        return implode("\n", $html);
    }

    /**
     * Gets a toolbar with common actions
     *
     * @return ActionBarRenderer
     */
    function get_action_bar()
    {
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);
        if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            if ($this->get_submission_trackers_by_publication($this->get_publication_id()))
            {
                $action_bar->add_common_action(new ToolbarItem(Translation :: get('DownloadAllSubmissions'), Theme :: get_common_image_path() . 'action_download.png', $this->get_url(array(
                        Tool :: PARAM_ACTION => AssignmentTool :: ACTION_DOWNLOAD_SUBMISSIONS)), ToolbarItem :: DISPLAY_ICON_AND_LABEL));
            }
            
            $action_bar->add_tool_action(new ToolbarItem(Translation :: get('ScoresOverview'), Theme :: get_common_image_path() . 'action_statistics.png', $this->get_url(array(
                    WeblcmsManager :: PARAM_TOOL => WeblcmsManager :: ACTION_REPORTING, 
                    ReportingManager :: PARAM_TEMPLATE_ID => Reporting :: get_name_registration(Utilities :: camelcase_to_underscores('CourseAssignmentSubmittersReportingTemplate'), WeblcmsManager :: APPLICATION_NAME)->get_id(), 
                    WeblcmsManager :: PARAM_TOOL_ACTION => ReportingTool :: ACTION_VIEW)), ToolbarItem :: DISPLAY_ICON_AND_LABEL));
            
            $action_bar->add_tool_action(new ToolbarItem(Translation :: get('SubmissionsOverview'), Theme :: get_common_image_path() . 'action_statistics.png', $this->get_url(array(
                    WeblcmsManager :: PARAM_TOOL => WeblcmsManager :: ACTION_REPORTING, 
                    ReportingManager :: PARAM_TEMPLATE_ID => Reporting :: get_name_registration(Utilities :: camelcase_to_underscores('AssignmentSubmissionsReportingTemplate'), WeblcmsManager :: APPLICATION_NAME)->get_id(), 
                    WeblcmsManager :: PARAM_TOOL_ACTION => ReportingTool :: ACTION_VIEW)), ToolbarItem :: DISPLAY_ICON_AND_LABEL));
        }

        $course = $this->get_course();
        $ephorus_tool = WeblcmsDataManager :: get_instance()->retrieve_course_tool_by_name('ephorus');
        if (!is_null($ephorus_tool) && $course->get_course_setting(\application\weblcms\CourseSetting ::
            COURSE_SETTING_TOOL_ACTIVE,
            $ephorus_tool->get_id())
        )
        {
            $action_bar->add_tool_action(
                new ToolbarItem(
                    Translation :: get('EphorusOverview'),
                    Theme :: get_image_path(Tool :: get_tool_type_namespace('ephorus')) . 'logo/16.png',
                    $this->get_url(
                        array(
                            WeblcmsManager :: PARAM_TOOL        => 'ephorus',
                            WeblcmsManager :: PARAM_PUBLICATION => $this->get_publication_id(),
                            WeblcmsManager :: PARAM_TOOL_ACTION => EphorusTool :: ACTION_ASSIGNMENT_BROWSER
                        )
                    ), ToolbarItem :: DISPLAY_ICON_AND_LABEL
                )
            );
        }
        
        $action_bar->add_common_action(new ToolbarItem(Translation :: get('SubmissionSubmit'), Theme :: get_common_image_path() . 'action_add.png', $this->get_url($this->generate_add_submission_url()), ToolbarItem :: DISPLAY_ICON_AND_LABEL));
        
        $type = Request :: get(AssignmentTool :: PARAM_TYPE);
        $url = $this->get_url(array(AssignmentTool :: PARAM_TYPE => $type));
        $action_bar->set_search_url($url);
        $action_bar->add_common_action(new ToolbarItem(Translation :: get('ShowAll', null, Utilities :: COMMON_LIBRARIES), Theme :: get_common_image_path() . 'action_browser.png', $url, ToolbarItem :: DISPLAY_ICON_AND_LABEL, false));
        
        return $action_bar;
    }

    /**
     * Returns the url for the add submission button.
     * When it's not a group assignment the url will have the extra parameters
     * target id and submitter type.
     *
     * @return array The url
     */
    private function generate_add_submission_url()
    {
        if ($this->assignment->get_allow_group_submissions())
        {
            return array(Tool :: PARAM_ACTION => AssignmentTool :: ACTION_SUBMIT_SUBMISSION, 
                    Tool :: PARAM_PUBLICATION_ID => $this->get_publication_id());
        }
        else
        {
            return array(Tool :: PARAM_ACTION => AssignmentTool :: ACTION_SUBMIT_SUBMISSION, 
                    Tool :: PARAM_PUBLICATION_ID => $this->get_publication_id(), 
                    AssignmentTool :: PARAM_TARGET_ID => $this->get_user_id(), 
                    AssignmentTool :: PARAM_SUBMITTER_TYPE => WeblcmsAssignmentSubmissionsTracker :: SUBMITTER_TYPE_USER);
        }
    }

    /**
     * Gets the search condition from the search bar in the action bar so it can
     * be used to update the table.
     *
     * @param $type int
     *            (optional) The submitter type
     * @return mixed
     */
    function get_search_condition($type = null)
    {
        $query = $this->action_bar->get_query();
        if (isset($query) && $query != '')
        {
            if ($type == null)
            {
                $type = self :: TYPE_INDIVIDUAL;
            }
            switch ($type)
            {
                case self :: TYPE_INDIVIDUAL :
                    $conditions = array();
                    $conditions[] = new PatternMatchCondition(User :: PROPERTY_FIRSTNAME, '*' . $query . '*');
                    $conditions[] = new PatternMatchCondition(User :: PROPERTY_LASTNAME, '*' . $query . '*');
                    return new OrCondition($conditions);
                case self :: TYPE_COURSE_GROUP :
                    return new PatternMatchCondition(CourseGroup :: PROPERTY_NAME, '*' . $query . '*');
                case self :: TYPE_GROUP :
                    return new PatternMatchCondition(Group :: PROPERTY_NAME, '*' . $query . '*');
            }
        }
        return null;
    }

    function display_header()
    {
        parent :: display_header();
        
        $this->action_bar = $this->get_action_bar();
        
        if ($this->action_bar)
        {
            echo $this->action_bar->as_html();
        }
        
        return true;
    }

    function get_additional_parameters()
    {
        return array(Tool :: PARAM_PUBLICATION_ID, AssignmentTool :: PARAM_TARGET_ID, 
                AssignmentTool :: PARAM_SUBMITTER_TYPE);
    }
}

?>
