<?php

namespace application\weblcms\tool\learning_path;

use application\weblcms\Tool;
use application\weblcms\WeblcmsDataManager;
use application\weblcms\WeblcmsLearningPathQuestionAttemptsTracker;
use application\weblcms\WeblcmsLpiAttemptTracker;
use application\weblcms\WeblcmsManager;
use application\weblcms\WeblcmsRights;
use common\libraries\ActionBarRenderer;
use common\libraries\AndCondition;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\EqualityCondition;
use common\libraries\Request;
use common\libraries\Session;
use common\libraries\Theme;
use common\libraries\ToolbarItem;
use common\libraries\Translation;
use common\libraries\Utilities;
use repository\content_object\assessment\AssessmentComplexDisplay;
use repository\ComplexBuilder;
use repository\ComplexDisplay;
use repository\RepositoryDataManager;
use user\UserDataManager;

/**
 * This component renders the assessment attempts from the learning path.
 * Depending on the parameters in the URL, it will show an overview of all the assessment attempts
 * or a detail from one attempt.
 *
 * @author Bert De Clercq (Hogeschool Gent)
 */
class LearningPathToolAssessmentResultsViewerComponent extends LearningPathTool
{
    
    const PARAM_DELETE_ID = 'delete_id';
    
    private $assessment;
    
    /**
     * Launches this component.
     */
    function run()
    {
        $this->check_view_rights();
        
        $this->modify_last_breadcrumb();
        
        if (Request :: get(self :: PARAM_DELETE_ID))
        {
            $this->delete_assessment_attempt(Request :: get(self :: PARAM_DELETE_ID));
        }
        
        if (Request :: get(LearningPathTool :: PARAM_LEARNING_PATH_ITEM_ATTEMPT_ID))
        {
            $this->view_single_result();
        }
        else
        {
            $this->view_assessment_results();
        }
    }
    
    /**
     * Checks if the user is allowed to view this page.
     */
    protected function check_view_rights()
    {
        $publication = WeblcmsDataManager :: get_instance()->retrieve_content_object_publication(
                $this->get_publication_id());

        if (!$this->is_allowed(WeblcmsRights :: VIEW_RIGHT, $publication)
                || (!$this->is_allowed(WeblcmsRights :: EDIT_RIGHT) && Session :: get_user_id() != Request :: get(WeblcmsManager :: PARAM_USERS)))
        {
            $this->redirect(Translation :: get('NotAllowed', null, Utilities :: COMMON_LIBRARIES), true, array(),
                    array(Tool :: PARAM_ACTION, Tool :: PARAM_PUBLICATION_ID));
        }
    }
    
    /**
     * Returns the publication id from the URL.
     * 
     * @return int The publication id
     */
    function get_publication_id()
    {
        return Request :: get(Tool :: PARAM_PUBLICATION_ID);
    }
    
    /**
     * Returns the user id from the URL.
     * 
     * @return int The user id
     */
    function get_user_id()
    {
        return Request :: get(WeblcmsManager :: PARAM_USERS);
    }
    
    /**
     * Returns the learning path attempt id from the URL.
     * 
     * @return int The learning path attempt id
     */
    function get_attempt_id()
    {
        return Request :: get(LearningPathTool :: PARAM_ATTEMPT_ID);
    }
    
    /**
     * Returns the complex content object item id from the URL.
     * 
     * @return int The complex content object item id
     */
    function get_ccoi_id()
    {
        return Request :: get(ComplexBuilder :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID);
    }
    
    /**
     * Returns the assessment id from the URL.
     * 
     * @return int The assessment id
     */
    function get_assessment_id()
    {
        return Request :: get(LearningPathTool :: PARAM_ASSESSMENT_ID);
    }
    
    /**
     * Renders an overview of the assessment attempt in the learning path.
     * On top there's a description of the assessment with some info about the attempts such as the average score
     * of all the attempts.
     */
    function view_assessment_results()
    {
        $this->display_header();
            
        echo '<div class="content_object" style="background-image: url(' .
                Theme :: get_common_image_path() . 'content_object/assessment.png);">';
        echo '<div class="title">';
        echo $this->assessment->get_title();
        echo '</div>';
        echo $this->assessment->get_description();
        echo '<div class="title">';
        echo Translation :: get('Statistics');
        echo '</div>';
        
        $conditions = array();
        $conditions[] = new EqualityCondition(WeblcmsLpiAttemptTracker :: PROPERTY_LP_ITEM_ID,
                Request :: get(ComplexBuilder :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID));
        $conditions[] = new EqualityCondition(WeblcmsLpiAttemptTracker :: PROPERTY_LP_VIEW_ID,
                Request :: get(LearningPathTool :: PARAM_ATTEMPT_ID));
        $condition = new AndCondition($conditions);
        
        $attempts = WeblcmsLpiAttemptTracker :: get_data(WeblcmsLpiAttemptTracker :: CLASS_NAME,
                WeblcmsManager :: APPLICATION_NAME, $condition)->as_array();
        
        if ($attempts)
        {
            $sum_score = 0;
            
            foreach ($attempts as $attempt)
            {
                $sum_score += $attempt->get_score();
            }
            
            $avg = round($sum_score / count($attempts)) . '%';
            $times_taken = count($attempts);
        }
        else
        {
            $avg = '-';
            $times_taken = '-';
        }
        
        echo Translation :: get('AverageScore') . ': ' . $avg;
        echo '<br/>' . Translation :: get('TimesTaken') . ': ' . $times_taken;
        echo '</div>';
        
        $table = new AssessmentResultsTable($this, $condition);
        echo $table->as_html();

        $this->display_footer();
    }
    
    /**
     * Renders a detailed overview of a single assessment attempt. A course admin can change scores and add feedback.
     */
    function view_single_result()
    {
        Request :: set_get('display_action', AssessmentComplexDisplay :: ACTION_VIEW_ASSESSMENT_RESULT);
        
        $this->set_parameter(LearningPathTool :: PARAM_LEARNING_PATH_ITEM_ATTEMPT_ID,
                Request :: get(LearningPathTool :: PARAM_LEARNING_PATH_ITEM_ATTEMPT_ID));

        ComplexDisplay :: launch($this->assessment->get_type(), $this);
    }
    
    /**
     * Retrieves the results for the assessment attempt.
     * 
     * @return array The assessment attempt results
     */
    function retrieve_assessment_results()
    {
        $condition = new EqualityCondition(WeblcmsLearningPathQuestionAttemptsTracker :: PROPERTY_LPI_ATTEMPT_ID,
                Request :: get(LearningPathTool :: PARAM_LEARNING_PATH_ITEM_ATTEMPT_ID));
        
        $question_attempts_trackers = WeblcmsLearningPathQuestionAttemptsTracker :: get_data(
                WeblcmsLearningPathQuestionAttemptsTracker :: CLASS_NAME,
                WeblcmsManager :: APPLICATION_NAME, $condition)->as_array();
        
        $results = array();

        foreach ($question_attempts_trackers as $question_attempts)
        {
            $results[$question_attempts->get_question_cid()] = array(
                'answer' => $question_attempts->get_answer(),
                'feedback' => $question_attempts->get_feedback(),
                'score' => $question_attempts->get_score());
        }

        return $results;
    }
    
    /**
     * Deletes the assessment attempt with the given id.
     * 
     * @param int $assessment_attempt_id The assessment attempt id
     */
    protected function delete_assessment_attempt($assessment_attempt_id)
    {
        $condition = new EqualityCondition(WeblcmsLpiAttemptTracker :: PROPERTY_ID, $assessment_attempt_id);
        $assessment_attempt_trackers = WeblcmsLpiAttemptTracker :: get_data(WeblcmsLpiAttemptTracker :: CLASS_NAME,
                WeblcmsManager :: APPLICATION_NAME, $condition)->as_array();
        
        foreach ($assessment_attempt_trackers as $assessment_attempt_tracker)
        {
            $assessment_attempt_tracker->delete();
        }

        $params = array(Tool :: PARAM_ACTION => LearningPathTool :: ACTION_VIEW_ASSESSMENT_RESULTS,
            Tool :: PARAM_PUBLICATION_ID => Request :: get(Tool :: PARAM_PUBLICATION_ID),
            WeblcmsManager :: PARAM_USERS => $this->get_user_id(),
            LearningPathTool :: PARAM_ATTEMPT_ID => $this->get_attempt_id(),
            ComplexBuilder :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID => $this->get_ccoi_id(),
            LearningPathTool :: PARAM_ASSESSMENT_ID => $this->get_assessment_id());

        $this->redirect(Translation :: get('LpiAttemptDeleted'), false, $params, array());
    }
    
    /**
     * Display the header and an action bar.
     */
    function display_header()
    {
        parent :: display_header();

        $action_bar = $this->get_toolbar();
        
        if ($action_bar)
        {
            echo $action_bar->as_html();
        }
    }
    
    /**
     * Returns an action bar if the user has edit rights.
     * The action bar includes a button to download the assessment attempts documents
     * and a button to delete the assessment attempts.
     * 
     * @return ActionBarRenderer 
     */
    function get_toolbar()
    {
        if (Request :: get(LearningPathTool :: PARAM_ASSESSMENT_ID) && $this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);

            $action_bar->add_common_action(new ToolbarItem(Translation :: get('DownloadDocuments'),
                    Theme :: get_common_image_path() . 'action_save.png',
                    $this->get_url(array(Tool :: PARAM_ACTION => LearningPathTool :: ACTION_DOWNLOAD_DOCUMENTS,
                        WeblcmsManager :: PARAM_PUBLICATION => $this->get_publication_id(),
                        WeblcmsManager :: PARAM_USERS => $this->get_user_id(),
                        LearningPathTool :: PARAM_ATTEMPT_ID => $this->get_attempt_id(),
                        ComplexBuilder :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID => $this->get_ccoi_id(),
                        LearningPathTool :: PARAM_ASSESSMENT_ID => $this->get_assessment_id())),
                    ToolbarItem :: DISPLAY_ICON_AND_LABEL));
            
            if (Request :: get(LearningPathTool :: PARAM_LEARNING_PATH_ITEM_ATTEMPT_ID))
            {
                $action_bar->add_common_action(new ToolbarItem(Translation :: get('DeleteResult'),
                        Theme :: get_common_image_path() . 'action_delete.png',
                        $this->get_url(array(Tool :: PARAM_ACTION => LearningPathTool :: ACTION_VIEW_ASSESSMENT_RESULTS,
                            WeblcmsManager :: PARAM_PUBLICATION => $this->get_publication_id(),
                            WeblcmsManager :: PARAM_USERS => $this->get_user_id(),
                            LearningPathTool :: PARAM_ATTEMPT_ID => $this->get_attempt_id(),
                            LearningPathToolStatisticsViewerComponent :: PARAM_ITEM_ID => $this->get_ccoi_id(),
                            LearningPathToolStatisticsViewerComponent :: PARAM_DELETE_ID => 
                        Request :: get(LearningPathTool :: PARAM_LEARNING_PATH_ITEM_ATTEMPT_ID))),
                        ToolbarItem :: DISPLAY_ICON_AND_LABEL, true));
            }
            else
            {
                $action_bar->add_common_action(new ToolbarItem(Translation :: get('DeleteAllResults'),
                        Theme :: get_common_image_path() . 'action_delete.png',
                        $this->get_url(array(Tool :: PARAM_ACTION => LearningPathTool :: ACTION_VIEW_STATISTICS,
                            WeblcmsManager :: PARAM_PUBLICATION => $this->get_publication_id(),
                            WeblcmsManager :: PARAM_USERS => $this->get_user_id(),
                            LearningPathTool :: PARAM_ATTEMPT_ID => $this->get_attempt_id(),
                            LearningPathToolStatisticsViewerComponent :: PARAM_STAT =>
                                LearningPathToolStatisticsViewerComponent :: ACTION_DELETE_LPI_ATTEMPTS,
                            LearningPathToolStatisticsViewerComponent :: PARAM_ITEM_ID => $this->get_ccoi_id())),
                        ToolbarItem :: DISPLAY_ICON_AND_LABEL, true));
            }

            return $action_bar;
        }
    }
    
    /**
     * Returns true if the user had edit rights and false otherwise.
     * 
     * @return boolean True if the user has edit rights
     */
    function can_change_answer_data()
    {
    
        return $this->is_allowed(WeblcmsRights :: EDIT_RIGHT);
    }
    
    /**
     * Updates the question attempts of the assessment.
     * 
     * @param int $question_cid The complex question id
     * @param int $score The score
     * @param string $feedback The feedback
     */
    function change_answer_data($question_cid, $score, $feedback)
    {
        $conditions[] = new EqualityCondition(WeblcmsLearningPathQuestionAttemptsTracker :: PROPERTY_LPI_ATTEMPT_ID,
                Request :: get(LearningPathTool :: PARAM_LEARNING_PATH_ITEM_ATTEMPT_ID));
        $conditions[] = new EqualityCondition(WeblcmsLearningPathQuestionAttemptsTracker :: PROPERTY_QUESTION_CID,
                $question_cid);
        $condition = new AndCondition($conditions);
        
        $question_attempts_trackers = WeblcmsLearningPathQuestionAttemptsTracker :: get_data(
                WeblcmsLearningPathQuestionAttemptsTracker :: CLASS_NAME,
                WeblcmsManager :: APPLICATION_NAME, $condition)->as_array();
        
        $question_attempts = $question_attempts_trackers[0];
        
        $question_attempts->set_score($score);
        $question_attempts->set_feedback($feedback);
        $question_attempts->update();
    }
    
    /**
     * Updates the score of the assessment attempt in this learning path.
     */
    function change_total_score($score)
    {
        $condition = new EqualityCondition(WeblcmsLpiAttemptTracker :: PROPERTY_ID,
                Request :: get(LearningPathTool :: PARAM_LEARNING_PATH_ITEM_ATTEMPT_ID));
        
        $assessment_attempts_tracker = WeblcmsLpiAttemptTracker :: get_data(WeblcmsLpiAttemptTracker :: CLASS_NAME,
                WeblcmsManager :: APPLICATION_NAME, $condition)->as_array();
        
        $assessment_attempt = $assessment_attempts_tracker[0];
        
        $assessment_attempt->set_score($score);
        $assessment_attempt->update();
    }
    
    /**
     * Returns the root content object.
     * 
     * @return type 
     */
    function get_root_content_object()
    {
        return $this->assessment;
    }
    
    /**
     * Returns an empty array.
     * 
     * @return array 
     */
    function get_assessment_parameters()
    {
        return array();
    }
    
    /**
     * Returns the additional parameters.
     * 
     * @return array 
     */
    function get_additional_parameters()
    {
        return array(Tool :: PARAM_PUBLICATION_ID, WeblcmsManager :: PARAM_USERS, LearningPathTool :: PARAM_ATTEMPT_ID,
            ComplexBuilder :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID, LearningPathTool :: PARAM_ASSESSMENT_ID);
    }
    
    /**
     * Updates the last breadcrumb so it displays the name of the assessment.
     * 
     * @return BreadcrumbTrail The modified breadcrumb trail
     */
    protected function modify_last_breadcrumb()
    {
        $breadcrumb_trail = BreadcrumbTrail :: get_instance();        
        $breadcrumbs = $breadcrumb_trail->get_breadcrumbs();

        $breadcrumbs[$breadcrumb_trail->size() - 1] = new Breadcrumb($this->get_url(array(
            Tool :: PARAM_ACTION => LearningPathTool :: ACTION_VIEW_ASSESSMENT_RESULTS,
            Tool :: PARAM_PUBLICATION_ID => $this->get_publication_id(),
            WeblcmsManager :: PARAM_USERS => $this->get_user_id(),
            LearningPathTool :: PARAM_ATTEMPT_ID => $this->get_attempt_id(),
            ComplexBuilder :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID => $this->get_ccoi_id(),
            LearningPathTool :: PARAM_ASSESSMENT_ID => $this->get_assessment_id())),
                $this->assessment->get_title());
        
        $breadcrumb_trail->set_breadcrumbtrail($breadcrumbs);
        
        return $breadcrumb_trail;
    }
    
    /**
     * Adds additional breadcrumbs to the trail.
     */
    function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $this->assessment = RepositoryDataManager :: get_instance()->retrieve_content_object(
                Request :: get(LearningPathTool :: PARAM_ASSESSMENT_ID));
        
        $breadcrumbtrail->add(new Breadcrumb($this->get_url(array(Tool :: PARAM_ACTION => Tool :: ACTION_BROWSE)),
                Translation :: get('LearningPathToolBrowserComponent')));
        
        $lp = WeblcmsDataManager :: get_instance()->retrieve_content_object_publication(
                Request :: get(Tool :: PARAM_PUBLICATION_ID))->get_content_object();
        
        if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $breadcrumbtrail->add(new Breadcrumb($this->get_url(array(
                Tool :: PARAM_ACTION => LearningPathTool :: ACTION_VIEW_STATISTICS,
                Tool :: PARAM_PUBLICATION_ID => $this->get_publication_id())),
                    $lp->get_title()));

            if ($this->get_user_id())
            {
                $user = UserDataManager :: get_instance()->retrieve_user($this->get_user_id());

                $breadcrumbtrail->add(new Breadcrumb($this->get_url(array(
                    Tool :: PARAM_ACTION => LearningPathTool :: ACTION_VIEW_STATISTICS,
                    Tool :: PARAM_PUBLICATION_ID => $this->get_publication_id(),
                    LearningPathTool :: PARAM_ATTEMPT_ID => $this->get_attempt_id(),
                    WeblcmsManager :: PARAM_USERS => $this->get_user_id())),
                        $user->get_fullname()));
            }
        }
        else
        {
            $breadcrumbtrail->add(new Breadcrumb($this->get_url(array(
                Tool :: PARAM_ACTION => LearningPathTool :: ACTION_VIEW_STATISTICS,
                Tool :: PARAM_PUBLICATION_ID => $this->get_publication_id(),
                LearningPathTool :: PARAM_ATTEMPT_ID => $this->get_attempt_id(),
                WeblcmsManager :: PARAM_USERS => $this->get_user_id())),
                    $lp->get_title()));
        }
    }
    
}

?>
