<?php
namespace application\weblcms\tool\user;

use user\DefaultUserTableCellRenderer;
use application\weblcms\WeblcmsRights;
use application\weblcms\WeblcmsManager;
use application\weblcms\WeblcmsDataManager;
use application\weblcms\Tool;
use common\libraries\Toolbar;
use common\libraries\ToolbarItem;
use user\User;
use group\Group;
use group\GroupDataManager;
use common\libraries\PlatformSetting;
use common\libraries\Theme;
use common\libraries\Request;
use common\libraries\Path;
use common\libraries\Translation;
use common\libraries\WebApplication;
use application\weblcms\course\CourseUserRelation;
use application\weblcms\CourseManagementRights;

/**
 * * ***************************************************************************
 * Cell renderer for an all subscribed course user browser table.
 *
 * @author Stijn Van Hoecke
 *        
 *         ****************************************************************************
 */
class AllSubscribedUserBrowserTableCellRenderer extends DefaultUserTableCellRenderer
{
    // **************************************************************************
    // VARIABLES
    // **************************************************************************
    
    /**
     * Cache to store the status of unknown groups after retrieval.
     */
    private $unknown_status_cache;
    
    // **************************************************************************
    // CONSTRUCTOR
    // **************************************************************************
    /**
     * Constructor
     * 
     * @param $browser WeblcmsBrowserComponent           
     */
    function __construct($object_table)
    {
        parent :: __construct($object_table);
        $this->unknown_status_cache = array();
    }
    
    // **************************************************************************
    // GENERAL FUNCTIONS
    // **************************************************************************
    // Inherited
    /**
     * Renders a given cell.
     * 
     * @param $column type           
     * @param $user_with_subscription_status type
     *            User from the advanced join query in weblcms database class
     *            that includes his subscription status.
     * @return type
     */
    function render_cell($column, $user_with_subscription_status_and_type)
    {
        
        // Add special features here
        switch ($column->get_name())
        {
            // Exceptions that need post-processing go here ...
            case AllSubscribedUserBrowserTableColumnModel :: SUBSCRIPTION_TYPE :
                $type = $user_with_subscription_status_and_type->get_optional_property(AllSubscribedUserBrowserTableColumnModel :: SUBSCRIPTION_TYPE);
                switch ($type)
                {
                    case 1 :
                        return Translation :: get('SubscribedDireclty');
                    case 2 :
                        return Translation :: get('SubscribedGroup');
                    default :
                        return ($type % 2 == 0) ? Translation :: get('SubscribedGroup') : Translation :: get('SubscribedDirecltyAndGroup');
                }
            case AllSubscribedUserBrowserTableColumnModel :: SUBSCRIPTION_STATUS :
                switch ($user_with_subscription_status_and_type->get_optional_property(AllSubscribedUserBrowserTableColumnModel :: SUBSCRIPTION_STATUS))
                {
                    case CourseUserRelation :: STATUS_TEACHER :
                        return Translation :: get('CourseAdmin');
                    case CourseUserRelation :: STATUS_STUDENT :
                        return Translation :: get('Student');
                    default :
                        return Translation :: get('Unknown');
                }
            case User :: PROPERTY_PLATFORMADMIN :
                if ($user->get_platformadmin() == '1')
                {
                    return Translation :: get('PlatformAdministrator');
                }
                else
                {
                    return '';
                }
            case User :: PROPERTY_EMAIL :
                if (PlatformSetting :: get('active_online_email_editor'))
                {
                    $parameters = array();
                    $parameters[Tool :: PARAM_ACTION] = UserTool :: ACTION_EMAIL;
                    $parameters[UserTool :: PARAM_OBJECTS] = $user_with_subscription_status_and_type->get_id();
                    $email_url = $this->get_component()->get_url($parameters);
                }
                else
                {
                    $email_url = 'mailto:' . $user_with_subscription_status_and_type->get_email();
                }
                return '<a href="' . $email_url . '">' . $user_with_subscription_status_and_type->get_email() . '</a>';
        }
        return parent :: render_cell($column, $user_with_subscription_status_and_type);
    }

    /**
     * Gets the action links to display
     * 
     * @param $user_with_subscription_status User
     *            The user for which the
     *            action links should be returned
     * @return string A HTML representation of the action links
     */
    public function get_object_actions($user_with_subscription_status_and_type)
    {
        // construct the toolbar
        $toolbar = new Toolbar(Toolbar :: TYPE_HORIZONTAL);
        
        $parameters = array();
        $parameters[Tool :: PARAM_ACTION] = UserTool :: ACTION_USER_DETAILS;
        $parameters[UserTool :: PARAM_TAB] = Request :: get(UserTool :: PARAM_TAB);
        $parameters[WeblcmsManager :: PARAM_USERS] = $user_with_subscription_status_and_type->get_id();
        $details_url = $this->get_component()->get_url($parameters);
        
        // always show details
        $toolbar->add_item(new ToolbarItem(Translation :: get('Details'), Theme :: get_common_image_path() . 'action_details.png', $details_url, ToolbarItem :: DISPLAY_ICON));
        
        // display the actions to change the individual status and unsubscribe
        // if:
        // (1) the user has edit rights
        // AND
        // (2) the row is not the current user
        // AND
        // (3) the row is not a group-only subscription
        if ($this->get_component()->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            if ($user_with_subscription_status_and_type->get_id() != $this->get_component()->get_user()->get_id() && $user_with_subscription_status_and_type->get_optional_property(AllSubscribedUserBrowserTableColumnModel :: SUBSCRIPTION_TYPE) % 2)
            {
                if($this->get_component()->get_user()->is_platform_admin() ||
                            CourseManagementRights :: get_instance()->is_allowed(
                                 CourseManagementRights :: TEACHER_UNSUBSCRIBE_RIGHT, $this->get_component()->
                                    get_course_id(),
                                 CourseManagementRights :: TYPE_COURSE, $user_with_subscription_status_and_type
                                    ->get_id()))

                {
                    $parameters = array();
                    $parameters[Tool :: PARAM_ACTION] = UserTool :: ACTION_UNSUBSCRIBE;
                    $parameters[UserTool :: PARAM_TAB] = Request :: get(UserTool :: PARAM_TAB);
                    $parameters[UserTool :: PARAM_OBJECTS] = $user_with_subscription_status_and_type->get_id();
                    $unsubscribe_url = $this->get_component()->get_url($parameters);
                
                    $toolbar->add_item(new ToolbarItem(Translation :: get('DirectUnsubscribe'),
                            Theme :: get_common_image_path() . 'action_unsubscribe.png',
                            $unsubscribe_url, ToolbarItem :: DISPLAY_ICON));
                }
                else
                {
                    $toolbar->add_item(new ToolbarItem(Translation :: get('UnsubscribeNotAvailable'), 
                            Theme :: get_common_image_path() . 'action_unsubscribe_na.png', null,
                            ToolbarItem :: DISPLAY_ICON));
                }
                
                switch ($user_with_subscription_status_and_type->get_optional_property(AllSubscribedUserBrowserTableColumnModel :: SUBSCRIPTION_STATUS))
                {
                    case CourseUserRelation :: STATUS_TEACHER :
                        $status_change_url = $this->get_component()->get_status_changer_url($user_with_subscription_status_and_type->get_id(), CourseUserRelation :: STATUS_STUDENT);
                        $toolbar->add_item(new ToolbarItem(Translation :: get('MakeStudent'), Theme :: get_image_path(WebApplication :: determine_namespace(WeblcmsManager :: APPLICATION_NAME)) . 'action_subscribe_student.png', $status_change_url, ToolbarItem :: DISPLAY_ICON));
                        break;
                    case CourseUserRelation :: STATUS_STUDENT :
                        $status_change_url = $this->get_component()->get_status_changer_url($user_with_subscription_status_and_type->get_id(), CourseUserRelation :: STATUS_TEACHER);
                        $toolbar->add_item(new ToolbarItem(Translation :: get('MakeTeacher'), Theme :: get_image_path(WebApplication :: determine_namespace(WeblcmsManager :: APPLICATION_NAME)) . 'action_subscribe_teacher.png', $status_change_url, ToolbarItem :: DISPLAY_ICON));
                        break;
                }
            }
            else
            {
                $toolbar->add_item(new ToolbarItem(Translation :: get('UnsubscribeNotAvailable'), Theme :: get_common_image_path() . 'action_unsubscribe_na.png', null, ToolbarItem :: DISPLAY_ICON));
            }
            
            // if we have editing rights, display the reporting action
            $params = array();
            $params[WeblcmsManager :: PARAM_USERS] = $user_with_subscription_status_and_type->get_id();
            $params[Tool :: PARAM_ACTION] = UserTool :: ACTION_REPORTING;
            $parameters[UserTool :: PARAM_TAB] = Request :: get(UserTool :: PARAM_TAB);
            $reporting_url = $this->get_component()->get_url($params);
            
            $toolbar->add_item(new ToolbarItem(Translation :: get('Report'), Theme :: get_common_image_path() . 'action_reporting.png', $reporting_url, ToolbarItem :: DISPLAY_ICON));
        }
        
        // add action for view as user
        if ($this->get_component()->is_allowed(WeblcmsRights :: EDIT_RIGHT)) // ->get_parent()->is_teacher())
        {
            if ($user_with_subscription_status_and_type->get_id() != $this->get_component()->get_user()->get_id())
            {
                $parameters = array();
                $parameters[Tool :: PARAM_ACTION] = UserTool :: ACTION_VIEW_AS;
                $parameters[WeblcmsManager :: PARAM_USERS] = $user_with_subscription_status_and_type->get_id();
                $view_as_url = $this->get_component()->get_url($parameters);
                $toolbar->add_item(new ToolbarItem(Translation :: get('ViewAsUser'), Theme :: get_common_image_path() . 'action_login.png', $view_as_url, ToolbarItem :: DISPLAY_ICON));
            }
        }
        
        // return
        return $toolbar->as_html();
    }

}
?>