<?php
namespace common\extensions\category_manager;

use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\InCondition;
use common\libraries\ObjectNotExistException;
use common\libraries\NoObjectSelectedException;
use common\libraries\EqualityCondition;

/**
 * Component to view the impact of a delete command
 *
 * @author Sven Vanpoucke - Hogeschool Gent
 */
class CategoryManagerImpactViewComponent extends CategoryManagerComponent
{
    /****************************************************************************************************************
    * Inherited Functionality											    *
    ****************************************************************************************************************/

    /**
     * Runs this component and displays its output.
     */
    function run()
    {
        if(!$this->supports_impact_view())
        {
            throw new \Exception(Translation :: get('ImpactViewNotSupported'));
        }

        $category_ids = $this->get_selected_category_ids();

        $form = new ImpactViewForm($this->get_url(array(CategoryManager :: PARAM_CATEGORY_ID
            => $category_ids)));

        if($form->validate())
        {
            $failures = 0;
            
            foreach($category_ids as $category_id)
            {
                $condition = new EqualityCondition(PlatformCategory :: PROPERTY_ID, $category_id);
                $category = $this->get_parent()->retrieve_categories($condition)->next_result();

                if(is_null($category))
                {
                    $failures++;
                    continue;
                }

                if(!$category->delete())
                {
                    $failures++;
                }
            }

            $result = $this->get_result($failures, count($category_ids), 'CategoryNotDeleted', 'CategoriesNotDeleted',
                    'CategoryDeleted', 'CategoryNotDeleted');

             $this->redirect($result, $failures > 0, array(CategoryManager :: PARAM_ACTION =>
                 CategoryManager :: ACTION_BROWSE_CATEGORIES));
        }
        else
        {
            $view = $this->render_impact_view();

            $this->display_header();
            echo $view;
            $form->display();
            $this->display_footer();
        }
    }

    /****************************************************************************************************************
    * Helper Functionality											    *
    ****************************************************************************************************************/

    /**
     * Renders the impact view on the screen
     *
     * @return string
     */
    protected function render_impact_view()
    {
        $impact_view = $this->get_parent()->render_impact_view($this->get_selected_category_ids());

        if(is_null($impact_view))
        {
            $impact_view = '<div class="normal-message">' . Translation :: get('NoImpact') . '</div>';
        }

        return $impact_view;
    }

    /**
     * Returns the selected category ids as an array
     *
     * @return string[]
     */
    protected function get_selected_category_ids()
    {
	$category_ids = Request :: get(CategoryManager :: PARAM_CATEGORY_ID);

	if(!isset($category_ids))
	{
	    throw new NoObjectSelectedException(Translation :: get('Category'));
	}

        $category_ids = (array) $category_ids;

	return $category_ids;
    }

    /**
     * Builds a result message with given parameters
     *
     * @param int $failures
     * @param int $count
     * @param string $fail_message_single
     * @param string $fail_message_multiple
     * @param string $succes_message_single
     * @param string $succes_message_multiple
     *
     * @return string
     */
    function get_result($failures, $count, $fail_message_single, $fail_message_multiple, $succes_message_single,
            $succes_message_multiple)
    {
        if ($failures)
        {
            if ($count == 1)
            {
                $message = $fail_message_single;
            }
            else
            {
                $message = $fail_message_multiple;
            }
        }
        else
        {
            if ($count == 1)
            {
                $message = $succes_message_single;
            }
            else
            {
                $message = $succes_message_multiple;
            }
        }

        return Translation :: get($message);
    }

}
?>