<?php
namespace common\extensions\external_repository_manager\implementation\dropbox;

use Dropbox\API;

use common\libraries\Configuration;
use Dropbox\OAuth\Storage\Encrypter;
use Dropbox\OAuth\Consumer\Curl;

use common\libraries\Redirect;
use common\libraries\Request;
use common\libraries\Path;
use common\libraries\Session;
use common\libraries\Utilities;
use common\libraries\ArrayResultSet;
use common\libraries\ActionBarSearchForm;
use common\libraries\Filesystem;

use repository\ExternalUserSetting;
use repository\ExternalSetting;
use repository\RepositoryDataManager;

require_once Path :: get_plugin_path(__NAMESPACE__) . 'Dropbox/Autoloader.class.php';
class DataConnector extends \common\extensions\external_repository_manager\DataConnector
{
    private $dropbox;
    private $consumer;
    private $key;
    private $secret;
    private $tokens;
    private $oauth;
    
    const SORT_DATE_CREATED = 'date-created';

    function __construct($external_repository_instance)
    {
        parent :: __construct($external_repository_instance);
        spl_autoload_register('Dropbox\Autoloader::load');
        $this->key = ExternalSetting :: get('key', $this->get_external_repository_instance_id());
        $this->secret = ExternalSetting :: get('secret', $this->get_external_repository_instance_id());
        
        $callback = Redirect :: current_url();
        $encrypter = new Encrypter(Configuration :: get_instance()->get_parameter('general', 'security_key'));
        $storage = new \Dropbox\OAuth\Storage\Session($encrypter);
        $curl = new Curl($this->key, $this->secret, $storage, $callback);
        
        if ($_SESSION['dropbox_api']['access_token'])
        {
            $setting = RepositoryDataManager :: get_instance()->retrieve_external_setting_from_variable_name('access_token', $this->get_external_repository_instance_id());
            $user_setting = new ExternalUserSetting();
            $user_setting->set_setting_id($setting->get_id());
            $user_setting->set_user_id(Session :: get_user_id());
            $token = $_SESSION['dropbox_api']['access_token'];
            $user_setting->set_value($token);
            if ($user_setting->create())
            {
                Session :: unregister('dropbox_api');
                $storage->set($encrypter->decrypt($token), 'access_token');
            }
        }
        
        else
        {
            $token = ExternalUserSetting :: get('access_token', $this->get_external_repository_instance_id());
            $storage->set($encrypter->decrypt($token), 'access_token');
        }
        
        $this->dropbox = new API($curl, 'dropbox');
    
    }

    /**
     *
     * @param $condition mixed           
     * @param $order_property ObjectTableOrder           
     * @param $offset int           
     * @param $count int           
     * @return array
     */
    function retrieve_files($condition = null, $order_property, $offset, $count)
    {
        $folder = Request :: get('folder');
        if (! is_null($folder))
        {
            $folder = urldecode($folder);
        }
        else
            $folder = '/';
        
        $files = $this->dropbox->metaData($folder);
        return $files;
    }

    function retrieve_folder($path, $condition = null, $order_property, $offset, $count)
    {
        $folders = $this->dropbox->metaData($path);
        return $folders;
    }

    /**
     *
     * @param $condition mixed           
     * @param $order_property ObjectTableOrder           
     * @param $offset int           
     * @param $count int           
     * @return ArrayResultSet
     */
    function retrieve_external_repository_objects($condition = null, $order_property, $offset, $count)
    {
        $files = $this->retrieve_files($condition, $order_property, $offset, $count);
        $file_count = 0;
        
        $objects = array();
        foreach ($files['body']->contents as $file)
        {
            if ($file->is_dir != 1)
            {
                $object = new ExternalObject();
                $object->set_id((string) substr($file->path, 1));
                $object->set_external_repository_id($this->get_external_repository_instance_id());
                $object->set_title((string) substr($file->path, strripos($file->path, '/') + 1));
                $object->set_created($file->modified);
                $object->set_modified($file->modified);
                $object->set_type($file->icon);
                $object->set_description($file->size);
                $object->set_rights($this->determine_rights());
                
                $file_count ++;
                if ($file_count > $offset && $file_count <= ($count + $offset))
                {
                    $objects[] = $object;
                }
            }
        }
        return new ArrayResultSet($objects);
    }

    function retrieve_folders($folder_url)
    {
        $folders = array();
        $files = $this->retrieve_folder('/', $condition, $order_property, $offset, $count);
        
        foreach ($files['body']->contents as $file)
        {
            if ($file->is_dir == 1)
            {
                $folder[] = array();
                $folder['title'] = substr($file->path, strripos($file->path, '/') + 1);
                $folder['url'] = str_replace('__PLACEHOLDER__', substr($file->path, 1), $folder_url);
                $folder['class'] = 'category';
                $sub = $this->get_folder_tree($folder_url, $file->path);
                if (count($sub) > 0)
                {
                    $folder['sub'] = $sub;
                }
                $folders[] = $folder;
            }
        }
        return $folders;
    }

    function get_folder_tree($folder_url, $folder_path)
    {
        
        $folders = $this->retrieve_folder($folder_path, $condition, $order_property, $offset, $count);
        $items = array();
        foreach ($folders['body']->contents as $child)
        {
            if ($child->is_dir == 1)
            {
                $sub_folder = array();
                $sub_folder['title'] = substr($child->path, strripos($child->path, '/') + 1);
                $sub_folder['url'] = str_replace('__PLACEHOLDER__', $child->path, $folder_url);
                $sub_folder['class'] = 'category';
                
                $sub = $this->get_folder_tree($folder_url, $child->path);
                if (count($sub) > 0)
                {
                    $sub_folder['sub'] = $sub;
                }
                $items[] = $sub_folder;
            }
        }
        return $items;
    }

    /**
     *
     * @param $condition mixed           
     * @return int
     */
    function count_external_repository_objects($condition)
    {
        $files = $this->retrieve_files($condition, $order_property, $offset, $count);
        
        $objects = array();
        $count = 0;
        foreach ($files['body']->contents as $file)
        {
            if ($file->is_dir != 1)
            {
                $count ++;
            }
        }
        return $count;
    }

    /**
     *
     * @param $query string           
     * @return string
     */
    static function translate_search_query($query)
    {
        return $query;
    }

    /**
     *
     * @param $order_properties ObjectTableOrder           
     * @return string null
     */
    function convert_order_property($order_properties)
    {
        if (count($order_properties) > 0)
        {
            $order_property = $order_properties[0]->get_property();
            if ($order_property == self :: SORT_RELEVANCE)
            {
                return $order_property;
            }
            else
            {
                $sorting_direction = $order_properties[0]->get_direction();
                
                if ($sorting_direction == SORT_ASC)
                {
                    return $order_property . '-asc';
                }
                elseif ($sorting_direction == SORT_DESC)
                {
                    return $order_property . '-desc';
                }
            }
        }
        return null;
    }

    /**
     *
     * @return array
     */
    static function get_sort_properties()
    {
        $feed_type = Request :: get(Manager :: PARAM_FEED_TYPE);
        $query = ActionBarSearchForm :: get_query();
        
        if (($feed_type == Manager :: FEED_TYPE_GENERAL && $query))
        {
            return array(self :: SORT_DATE_CREATED);
        }
        else
        {
            return array();
        }
    }
    
    /*
     * (non-PHPdoc) @see
     * application/common/external_repository_manager/ExternalRepositoryConnector#retrieve_external_repository_object()
     */
    function retrieve_external_repository_object($id)
    {
        $file = $this->dropbox->metaData($id);
        
        $object = new ExternalObject();
        $object->set_external_repository_id($this->get_external_repository_instance_id());
        $object->set_id($id);
        $object->set_title(str_replace('/', '', substr($id, strripos($id, '/'))));
        $object->set_created($file->modified);
        $object->set_modified($file->modified);
        $object->set_type($file->icon);
        $object->set_description($file->size);
        
        $object->set_rights($this->determine_rights());
        return $object;
    }

    function determine_rights()
    {
        $rights = array();
        $rights[ExternalObject :: RIGHT_USE] = true;
        $rights[ExternalObject :: RIGHT_EDIT] = false;
        $rights[ExternalObject :: RIGHT_DELETE] = true;
        $rights[ExternalObject :: RIGHT_DOWNLOAD] = true;
        return $rights;
    }

    /**
     *
     * @param $values array           
     * @param $file_path string           
     * @return mixed
     */
    function create_external_repository_object($file, $file_path)
    {
        $file = str_replace(' ', '', $file);
        return $this->dropbox->putFile($file_path, $file);
    }

    /**
     *
     * @param $content_object ContentObject           
     * @return mixed
     */
    function export_external_repository_object($content_object)
    {
        $file = str_replace(' ', '', $content_object->get_title());
        return $this->dropbox->putFile($content_object->get_full_path(), $file);
    }

    /**
     *
     * @param $id string           
     * @return mixed
     */
    function delete_external_repository_object($id)
    {
        return $this->dropbox->delete($id);
    }

    function download_external_repository_object($id)
    {
        $file = $this->dropbox->getFile($id);
        return $file['data'];
    }

    function update_external_repository_object($values)
    {
    
    }

    function encode($path)
    {
        $file = explode('/', $path);
        $newpath = array();
        foreach ($file as $f)
        {
            $newpath[] = rawurlencode($f);
        }
        return implode('/', $newpath);
    }

    function create_external_repository_folder($folder)
    {
        return $this->dropbox->createFolder($folder);
    }
}
?>