<?php
namespace common\extensions\external_repository_manager;

use repository;

use user\UserSetting;
use user\UserDataManager;

use admin\AdminDataManager;

use common\libraries\Session;
use common\libraries\Utilities;
use common\libraries\FormValidator;
use common\libraries\Path;
use common\libraries\Translation;
use common\libraries\DynamicFormTabsRenderer;
use common\libraries\DynamicFormTab;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;

use DOMDocument;

use repository\RepositoryDataManager;
use repository\ExternalSetting;
use repository\ExternalUserSetting;

/**
 * A form to configure external repository settings.
 *
 * @package application.common
 * @author Hans De Bisschop
 */

class SettingsForm extends FormValidator
{

    private $external_repository;

    private $configuration;

    private $is_user_setting_form;

    private $configurer;

    const TAB_USER = 'user';
    const TAB_ADMIN = 'admin';

    /**
     * Constructor.
     *
     * @param $application string The name of the application.
     * @param $form_name string The name to use in the form tag.
     * @param $method string The method to use ('post' or 'get').
     * @param $action string The URL to which the form should be submitted.
     */

    function __construct($configurer, $external_repository_id, $form_name, $method = 'post', $action = null, $is_user_setting_form = false)
    {
        parent :: __construct($form_name, $method, $action);

        $this->configurer = $configurer;
        $this->is_user_setting_form = $is_user_setting_form;
        $this->external_repository = RepositoryDataManager :: get_instance()->retrieve_external_instance($external_repository_id);
        $this->configuration = $this->parse_settings();
        $this->build_form();
        $this->setDefaults();
    }

    /**
     * Builds a form to create or edit a learning object. Creates fields for default learning object properties. The
     * result of this function is equal to build_creation_form()'s, but that one may be overridden to extend the form.
     */
    private function build_form()
    {
        $this->tabs_generator = new DynamicFormTabsRenderer($this->getAttribute('name'), $this);
        $has_settings = $this->external_repository->has_settings();
        $has_user_settings = $this->external_repository->has_user_settings();
        $is_platform = $this->configurer->get_user()->is_platform_admin();

        if ($has_settings)
        {
            if ($has_user_settings)
            {
                if ($is_platform)
                {
                    $tab_admin = new DynamicFormTab(self :: TAB_ADMIN, Translation :: get(self :: TAB_ADMIN), null, 'build_admin_form');
                    $this->tabs_generator->add_tab($tab_admin);
                    $tab_user = new DynamicFormTab(self :: TAB_USER, Translation :: get(self :: TAB_USER), null, 'build_user_form');
                    $this->tabs_generator->add_tab($tab_user);
                }
                else
                {
                    $tab_user = new DynamicFormTab(self :: TAB_USER, Translation :: get(self :: TAB_USER), null, 'build_user_form');
                    $this->tabs_generator->add_tab($tab_user);
                }
            }
            else
            {
                if ($is_platform)
                {
                    $tab_admin = new DynamicFormTab(self :: TAB_ADMIN, Translation :: get(self :: TAB_ADMIN), null, 'build_admin_form');
                    $this->tabs_generator->add_tab($tab_admin);
                }
            }
        }
        $this->tabs_generator->render();
    }

    function build_user_form()
    {
        $this->build_configure_form(true);
    }

    function build_admin_form()
    {
        $this->build_configure_form(false);
    }

    function build_configure_form($is_user_setting_form)
    {
        $type = ($is_user_setting_form ? 'user' : 'admin');
        $external_repository = $this->external_repository;
        $configuration = $this->configuration;

        if (count($configuration['settings']) > 0)
        {
            $categories = count($configuration['settings']);

            foreach ($configuration['settings'] as $category_name => $settings)
            {
                $has_settings = false;

                foreach ($settings as $name => $setting)
                {
                    if ($is_user_setting_form && ! $setting['user_setting'])
                    {
                        continue;
                    }

                    if (! $has_settings && $categories > 1)
                    {
                        $this->addElement('html', '<div class="configuration_form">');
                        $this->addElement('html', '<span class="category">' . Translation :: get(Utilities :: underscores_to_camelcase($category_name), null, Manager :: get_namespace($configuration['name'])) . '</span>');
                        $has_settings = true;
                    }

                    if ($setting['locked'] == 'true')
                    {
                        $this->addElement('static', $type . '[' . $name . ']', Translation :: get(Utilities :: underscores_to_camelcase($name)), null, Manager :: get_namespace($configuration['name']));
                    }
                    elseif ($setting['field'] == 'text')
                    {
                        $this->add_textfield($type . '[' . $name . ']', Translation :: get(Utilities :: underscores_to_camelcase($name), null, Manager :: get_namespace($configuration['name'])), ($setting['required'] == 'true'));

                        $validations = $setting['validations'];
                        if ($validations)
                        {
                            foreach ($validations as $validation)
                            {
                                if ($this->is_valid_validation_method($validation['rule']))
                                {
                                    if ($validation['rule'] != 'regex')
                                    {
                                        $validation['format'] = NULL;
                                    }

                                    $this->addRule($type . '[' . $name . ']', Translation :: get($validation['message'], null, Manager :: get_namespace($configuration['name'])), $validation['rule'], $validation['format']);
                                }
                            }
                        }

                    }
                    elseif ($setting['field'] == 'html_editor')
                    {
                        $this->add_html_editor($type . '[' . $name . ']', Translation :: get(Utilities :: underscores_to_camelcase($name), Manager :: get_namespace($configuration['name'])), ($setting['required'] == 'true'));
                    }
                    elseif ($setting['field'] == 'password')
                    {
                        $this->addElement('password', $type . '[' . $name . ']', Translation :: get(Utilities :: underscores_to_camelcase($name), Manager :: get_namespace($configuration['name'])));
                        $validations = $setting['validations'];
                        if ($validations)
                        {
                            foreach ($validations as $validation)
                            {
                                if ($this->is_valid_validation_method($validation['rule']))
                                {
                                    if ($validation['rule'] != 'regex')
                                    {
                                        $validation['format'] = NULL;
                                    }

                                    $this->addRule($type . '[' . $name . ']', Translation :: get($validation['message'], null, Manager :: get_namespace($configuration['name'])), $validation['rule'], $validation['format']);
                                }
                            }
                        }
                    }
                    else
                    {
                        $options_type = $setting['options']['type'];
                        if ($options_type == 'dynamic')
                        {
                            $options_source = $setting['options']['source'];
                            $class = $this->external_repository->get_type() . '\SettingsConnector';
                            $options = call_user_func(array($class, $options_source));
                        }
                        else
                        {
                            $options = $setting['options']['values'];
                        }

                        if ($setting['field'] == 'radio' || $setting['field'] == 'checkbox')
                        {
                            $group = array();
                            foreach ($options as $option_value => $option_name)
                            {
                                if ($setting['field'] == 'checkbox')
                                {
                                    $group[] = & $this->createElement($setting['field'], $type . '[' . $name . ']', null, null, $option_value);
                                }
                                else
                                {
                                    $group[] = & $this->createElement($setting['field'], $type . '[' . $name . ']', null, Translation :: get(Utilities :: underscores_to_camelcase($option_name)), $option_value);
                                }
                            }
                            $this->addGroup($group, $type . '[' . $name . ']', Translation :: get(Utilities :: underscores_to_camelcase($name), null, Manager :: get_namespace($configuration['name'])), '<br/>', false);
                        }
                        elseif ($setting['field'] == 'select')
                        {
                            $this->addElement('select', $type . '[' . $name . ']', Translation :: get(Utilities :: underscores_to_camelcase($name), null, Manager :: get_namespace($configuration['name'])), $options);
                        }
                    }
                }

                if ($has_settings && $categories > 1)
                {
                    $this->addElement('html', '<div style="clear: both;"></div>');
                    $this->addElement('html', '</div>');
                }
            }

            $buttons = array();
            $buttons[] = $this->createElement('style_submit_button', 'submit', Translation :: get('Save', null, Utilities :: COMMON_LIBRARIES), array(
                    'class' => 'positive'));
            $buttons[] = $this->createElement('style_reset_button', 'reset', Translation :: get('Reset', null, Utilities :: COMMON_LIBRARIES), array(
                    'class' => 'normal empty'));
            $this->addGroup($buttons, 'buttons', null, '&nbsp;', false);
        }
        else
        {
            $this->addElement('html', Translation :: get('NoConfigurableSettings', null, Utilities :: COMMON_LIBRARIES));
        }
    }

    function parse_settings()
    {
        $external_repository = $this->external_repository;

        $file = Path :: namespace_to_full_path($external_repository->get_type()) . 'php/settings/settings.xml';
        $result = array();

        if (file_exists($file))
        {
            $doc = new DOMDocument();
            $doc->load($file);
            $object = $doc->getElementsByTagname('package')->item(0);
            $name = $object->getAttribute('context');

            // Get categories
            $categories = $doc->getElementsByTagname('category');
            $settings = array();

            foreach ($categories as $index => $category)
            {
                $category_name = $category->getAttribute('name');
                $category_properties = array();

                // Get settings in category
                $properties = $category->getElementsByTagname('setting');
                $attributes = array('field', 'default', 'locked', 'user_setting');

                foreach ($properties as $index => $property)
                {
                    $property_info = array();

                    foreach ($attributes as $index => $attribute)
                    {
                        if ($property->hasAttribute($attribute))
                        {
                            $property_info[$attribute] = $property->getAttribute($attribute);
                        }
                    }

                    if ($property->hasChildNodes())
                    {
                        $property_options = $property->getElementsByTagname('options')->item(0);

                        if ($property_options)
                        {
                            $property_options_attributes = array('type', 'source');

                            foreach ($property_options_attributes as $index => $options_attribute)
                            {
                                if ($property_options->hasAttribute($options_attribute))
                                {
                                    $property_info['options'][$options_attribute] = $property_options->getAttribute($options_attribute);
                                }
                            }

                            if ($property_options->getAttribute('type') == 'static' && $property_options->hasChildNodes())
                            {
                                $options = $property_options->getElementsByTagname('option');
                                $options_info = array();
                                foreach ($options as $option)
                                {
                                    $options_info[$option->getAttribute('value')] = $option->getAttribute('name');
                                }
                                $property_info['options']['values'] = $options_info;
                            }
                        }

                        $property_validations = $property->getElementsByTagname('validations')->item(0);

                        if ($property_validations)
                        {
                            if ($property_validations->hasChildNodes())
                            {
                                $validations = $property_validations->getElementsByTagname('validation');
                                $validation_info = array();
                                foreach ($validations as $validation)
                                {
                                    $validation_info[] = array(
                                            'rule' => $validation->getAttribute('rule'),
                                            'message' => $validation->getAttribute('message'),
                                            'format' => $validation->getAttribute('format'));
                                }
                                $property_info['validations'] = $validation_info;
                            }
                        }
                    }
                    $category_properties[$property->getAttribute('name')] = $property_info;
                }

                $settings[$category_name] = $category_properties;
            }

            $result['context'] = $name;
            $result['settings'] = $settings;
        }

        return $result;
    }

    /**
     *
     * @param $defaults array Default values for this form's parameters.
     */
    function setDefaults($defaults = array ())
    {
        $has_settings = $this->external_repository->has_settings();
        $has_user_settings = $this->external_repository->has_user_settings();
        $is_platform = $this->configurer->get_user()->is_platform_admin();
        $external_repository = $this->external_repository;
        $configuration = $this->configuration;

        foreach ($configuration['settings'] as $category_name => $settings)
        {
            foreach ($settings as $name => $setting)
            {
                if ($has_settings)
                {
                    if ($has_user_settings)
                    {
                        if ($is_platform)
                        {
                            $user_value = ExternalUserSetting :: get($name, $this->configurer->get_external_repository()->get_id());
                            $defaults['user[' . $name . ']'] = (isset($user_value) ? $user_value : $setting['default']);

                            $admin_value = ExternalSetting :: get($name, $this->configurer->get_external_repository()->get_id());
                            $defaults['admin[' . $name . ']'] = (isset($admin_value) ? $admin_value : $setting['default']);
                        }
                        else
                        {
                            $user_value = ExternalUserSetting :: get($name, $this->configurer->get_external_repository()->get_id());
                            $defaults['user[' . $name . ']'] = (isset($user_value) ? $user_value : $setting['default']);
                        }
                    }
                    else
                    {
                        if ($is_platform)
                        {
                            $admin_value = ExternalSetting :: get($name, $this->configurer->get_external_repository()->get_id());
                            $defaults['admin[' . $name . ']'] = (isset($admin_value) ? $admin_value : $setting['default']);
                        }
                    }
                }
            }
        }

        parent :: setDefaults($defaults);
    }

    /**
     * Updates the configuration.
     *
     * @return boolean True if the update succeeded, false otherwise.
     */
    function update_configuration()
    {
        $values = $this->exportValues();
        $configuration = $this->configuration;
        $external_repository = $this->external_repository;
        $is_platform = $this->configurer->get_user()->is_platform_admin();
        $problems = 0;

        foreach ($configuration['settings'] as $category_name => $settings)
        {
            foreach ($settings as $name => $setting)
            {
                if ($is_platform)
                {
                    if ($setting['locked'] != 'true')
                    {
                        $setting = RepositoryDataManager :: get_instance()->retrieve_external_setting_from_variable_name($name, $external_repository->get_id());
                        if ($setting instanceof ExternalSetting)
                        {
                            if (isset($values['admin'][$name]))
                            {
                                $setting->set_value($values['admin'][$name]);
                            }
                            else
                            {
                                $setting->set_value(0);
                            }
                            if (! $setting->update())
                            {
                                $problems ++;
                            }
                        }
                        else
                        {
                            $setting = new ExternalSetting();
                            $setting->set_external_id($external_repository->get_id());
                            $setting->set_variable($name);

                            if (isset($values['admin'][$name]))
                            {
                                $setting->set_value($values['admin'][$name]);
                            }
                            else
                            {
                                $setting->set_value(0);
                            }

                            if (! $setting->create())
                            {
                                $problems ++;
                            }
                        }
                    }
                }
                if (isset($values['user'][$name]))
                {
                    $conditions = array();
                    $conditions[] = new EqualityCondition(ExternalSetting :: PROPERTY_EXTERNAL_ID, $external_repository->get_id());
                    $conditions[] = new EqualityCondition(ExternalSetting :: PROPERTY_VARIABLE, $name);
                    $conditions[] = new EqualityCondition(ExternalSetting :: PROPERTY_USER_SETTING, 1);
                    $condition = new AndCondition($conditions);

                    $settings = RepositoryDataManager :: get_instance()->retrieve_external_settings($condition, array(), null, 1);
                    if ($settings->size() > 0)
                    {
                        $setting = $settings->next_result();

                        $conditions = array();
                        $conditions[] = new EqualityCondition(ExternalUserSetting :: PROPERTY_SETTING_ID, $setting->get_id());
                        $conditions[] = new EqualityCondition(ExternalUserSetting :: PROPERTY_USER_ID, $this->configurer->get_user_id());
                        $condition = new AndCondition($conditions);
                        $user_settings = RepositoryDataManager :: get_instance()->retrieve_external_user_settings($condition, array(), null, 1);

                        if ($user_settings->size() > 0)
                        {
                            $user_setting = $user_settings->next_result();
                            $user_setting->set_value($values['user'][$name]);
                            $user_setting->update();
                        }
                        else
                        {
                            $user_setting = new ExternalUserSetting();
                            $user_setting->set_value($values['user'][$name]);
                            $user_setting->set_user_id($this->configurer->get_user_id());
                            $user_setting->set_setting_id($setting->get_id());
                            $user_setting->create();
                        }
                    }
                    else
                    {
                        $problems ++;
                    }
                }
            }
        }

        if ($problems > 0)
        {
            return false;
        }
        else
        {
            return true;
        }
    }

    private function is_valid_validation_method($validation_method)
    {
        $available_validation_methods = array('regex', 'email', 'lettersonly', 'alphanumeric', 'numeric', 'required');
        return in_array($validation_method, $available_validation_methods);
    }
}
?>
