<?php
namespace common\extensions\new_rights_editor_manager;

use common\libraries\SubManager;
use common\libraries\Translation;
use Exception;

/**
 * Rights editor manager for unlimited amount of entities. With simple and advanced interface.
 * 
 * @package application.common.rights_editor_manager
 * @author Sven Vanpoucke
 */
class RightsEditorManager extends SubManager
{
    // Parameters
    const PARAM_ACTION = 'rights_action';
    
    const PARAM_ENTITY_TYPE = 'entity_type';
    const PARAM_ENTITY_ID = 'entity_id';
    const PARAM_RIGHT_ID = 'right_id';
    
    // Actions
    const ACTION_EDIT_SIMPLE_RIGHTS = 'simple_rights_editor';
    const ACTION_EDIT_ADVANCED_RIGHTS = 'advanced_rights_editor';
    const ACTION_MANAGE = 'manager';
    const ACTION_SET_ENTITY_RIGHTS = 'entity_rights_setter';
    const ACTION_CHANGE_INHERIT = 'inherit_changer';
    
    // Additional properties
    private $context;
    private $locations;
    private $entities;
    
    /**
     * Cached selected entity
     */
    private $selected_entity;

    // Default functions
    

    /**
     * Launches the rights editor with the additional properties
     * @param Application $application
     * @param String context
     * @param Array $locations
     * @param Array $entities
     */
    static function launch($application, $context, $locations = array(), $entities = array())
    {
        $component = self :: construct($application);
        if (! $context)
        {
            throw new Exception(Translation :: get('NoContextGiven'));
        }
        
        $component->set_context($context);
        
        if (count($locations) == 0)
        {
            throw new Exception(Translation :: get('NoLocationsSelected'));
        }
        
        $component->set_locations($locations);
        
        if (count($entities) == 0)
        {
            throw new Exception(Translation :: get('NoEntitiesSelected'));
        }
        
        $component->set_entities($entities);
        $component->run();
    }

    /**
     * Returns the parameter with which this submanager's actions are described
     * @return String
     */
    static function get_action_parameter()
    {
        return self :: PARAM_ACTION;
    }

    /**
     * Returns the default action for this submannager
     * @return String
     */
    static function get_default_action()
    {
        return self :: ACTION_EDIT_SIMPLE_RIGHTS;
    }

    /**
     * Returns the path to the components folder
     * @return String
     */
    function get_application_component_path()
    {
        return dirname(__FILE__) . '/component/';
    }

    // Getters and setters
    function get_context()
    {
        return $this->context;
    }

    function set_context($context)
    {
        $this->context = $context;
    }

    function get_locations()
    {
        return $this->locations;
    }

    function set_locations($locations)
    {
        $this->locations = $locations;
    }

    function get_entities()
    {
        return $this->entities;
    }

    function set_entities($entities)
    {
        $this->entities = $entities;
    }

    // Url building
    

    /**
     * Builds the url to browse an entity
     * @param int $entity_type
     * @return String
     */
    function get_entity_url($entity_type)
    {
        return $this->get_url(array(self :: PARAM_ENTITY_TYPE => $entity_type));
    }

    // Delegation
    

    /**
     * Retrieves the available rights
     * @return Array
     */
    function get_available_rights()
    {
        $locations = $this->get_locations();
        return $this->get_parent()->get_available_rights($locations[0]);
    }

    /**
     * Retrieves additional information from the parent application
     * @return String
     */
    function get_additional_information()
    {
        if (method_exists($this->get_parent(), 'get_additional_information'))
        {
            return $this->get_parent()->get_additional_information();
        }
    }

    // Helper functions
    

    /**
     * Gets the selected entity type and if no type selected, uses the first available entity
     * @return String
     */
    function get_selected_entity()
    {
        if (! $this->selected_entity)
        {
            $selected_entity = $this->get_parameter(self :: PARAM_ENTITY_TYPE);
            if ($selected_entity)
            {
                $this->selected_entity = $this->entities[$selected_entity];
            }
            else
            {
                $first_entity = array_shift($this->get_entities());
                if ($first_entity)
                {
                    array_unshift($this->get_entities(), $first_entity);
                    $this->selected_entity = $first_entity;
                }
            }
        }
        
        return $this->selected_entity;
    }

}

?>