<?php
namespace common\extensions\repo_viewer;

use repository\RepositoryRights;
use repository\RepositoryManager;
use repository\ContentObjectCategoryMenu;
use common\libraries\Path;
use common\libraries\Application;
use common\libraries\ComplexDisplayPreviewLauncher;
use common\libraries\ComplexContentObjectSupport;
use common\libraries\Toolbar;
use common\libraries\ToolbarItem;
use common\libraries\Translation;
use common\libraries\Theme;
use common\libraries\Request;
use common\libraries\FormValidator;
use common\libraries\OptionsMenuRenderer;
use common\libraries\Utilities;
use repository\SharedRepositoryCategoryTreeMenu;

class RepoViewerBrowserComponent extends RepoViewer
{
    const SHARED_BROWSER = 'shared';
    const SHARED_BROWSER_ALLOWED = 'allow_shared_browser';
    const PROPERTY_CATEGORY = 'category';

    /**
     * The search form
     */
    private $form;

    /**
     * The renderer for the search form
     */
    private $renderer;

    /**
     * The browser actions
     */
    private $browser_actions;

    function __construct($parent)
    {
        parent :: __construct($parent);
        // $this->set_browser_actions($this->get_default_browser_actions());

        $form_parameters = $this->get_parameter();
        $form_parameters[RepoViewer :: PARAM_ACTION] = RepoViewer :: ACTION_BROWSER;
        if ($this->is_shared_object_browser())
        {
            $form_parameters[self :: SHARED_BROWSER] = 1;
        }

        $this->set_form(
                new FormValidator('search', 'post', $this->get_url($form_parameters), '', array('id' => 'search'), false));
        $this->get_form()->addElement('text', RepoViewer :: PARAM_QUERY,
                Translation :: get('Search', null, Utilities :: COMMON_LIBRARIES), 'size="30" class="search_query"');
        $this->get_form()->addElement('style_submit_button', 'submit', null, array('class' => 'search'));
        $this->get_form()->setDefaults(array(RepoViewer :: PARAM_QUERY => $this->get_query()));
    }

    /*
     * Inherited
     */
    function run()
    {
        $this->renderer = clone $this->form->defaultRenderer();
        $this->renderer->setElementTemplate('<span>{element}</span> ');
        $this->form->accept($this->renderer);

        $html = array();
        $html[] = '<div class="search_form" style="float: right; margin: 0px 0px 5px 0px;">';
        $html[] = '<div class="simple_search">';
        $html[] = $this->renderer->toHTML();
        $html[] = '</div>';
        $html[] = '</div>';

        if ($this->get_maximum_select() > RepoViewer :: SELECT_SINGLE)
        {
            $html[] = '<b>' . sprintf(Translation :: get('SelectMaximumNumberOfContentObjects'),
                    $this->get_maximum_select()) . '</b><br />';
        }
        $menu = $this->get_menu();

        if ($this->is_allowed_shared_object_browsing())
        {
            $shared_menu = $this->get_shared_menu();
        }

        $table = $this->get_object_table();

        $html[] = '<br />';

        $html[] = '<div style="width: 15%; overflow: auto; float:left">';
        $html[] = $menu->render_as_tree();
        $html[] = '<br />';
        $html[] = $shared_menu ? $shared_menu->render_as_tree() : '';
        $html[] = '</div>';

        $html[] = '<div style="width: 83%; float: right;">';
        $html[] = $table->as_html();
        $html[] = '</div>';
        $html[] = '<div class="clear">&nbsp;</div>';

        $this->display_header();
        echo implode("\n", $html);
        $this->display_footer();
    }

    protected function get_object_table()
    {
        return new ContentObjectTable($this, $this->get_user(), $this->get_types(), $this->get_query());
    }

    /**
     * Returns the search query.
     *
     * @return string null query, or null if none.
     */
    protected function get_query()
    {
        if ($this->get_form()->validate())
        {

            return $this->get_form()->exportValue(RepoViewer :: PARAM_QUERY);
        }

        if (Request :: get(RepoViewer :: PARAM_QUERY))
        {
            return Request :: get(RepoViewer :: PARAM_QUERY);
        }

        return null;
    }

    function get_browser_actions()
    {
        return $this->browser_actions;
    }

    function set_browser_actions($browser_actions)
    {
        $this->browser_actions = $browser_actions;
    }

    function get_form()
    {
        return $this->form;
    }

    function set_form($form)
    {
        $this->form = $form;
    }

    function get_menu($allow_shared = true)
    {
        $url = $this->get_url($this->get_parameters()) . '&' . self :: PROPERTY_CATEGORY . '=%s';

        $extra = array();

        if ($this->get_query())
        {
            $search_url = '#';
            $search = array();

            if ($this->is_shared_object_browser())
            {
                $search['title'] = Translation :: get('SharedSearchResults');
            }
            else
            {
                $search['title'] = Translation :: get('SearchResults', null, Utilities :: COMMON_LIBRARIES);
            }

            $search['url'] = $search_url;
            $search['class'] = 'search_results';
            $extra[] = $search;
        }
        else
        {
            $search_url = null;
        }

        $menu = new RepositoryCategoryMenu($this, $this->get_user_id(),
                Request :: get(self :: PROPERTY_CATEGORY) ? Request :: get(self :: PROPERTY_CATEGORY) : 0, $url, $extra,
                $this->get_types());

        if ($search_url)
        {
            $menu->forceCurrentUrl($search_url);
        }
        elseif ($this->is_shared_object_browser())
        {
            $menu->forceCurrentUrl($shared['url']);
        }

        return $menu;
    }
    
    function get_category_url($category_id)
    {
        return $this->get_url(array(self :: PROPERTY_CATEGORY => $category_id));
    }

    /**
     * Returns the shared repository category tree menu
     *
     * @return SharedRepositoryCategoryTreeMenu
     */
    function get_shared_menu()
    {
        return new RepositorySharedCategoryMenu($this);
    }

    function get_default_browser_actions($content_object)
    {
        $toolbar = new Toolbar(Toolbar :: TYPE_HORIZONTAL);

        if ($content_object->has_right(RepositoryRights :: USE_RIGHT, $this->get_user_id()))
        {
            $toolbar->add_item(
                    new ToolbarItem(Translation :: get('Publish', null, Utilities :: COMMON_LIBRARIES),
                            Theme :: get_common_image_path() . 'action_publish.png',
                            $this->get_url(
                                    array_merge($this->get_parameters(),
                                            array(RepoViewer :: PARAM_ACTION => RepoViewer :: ACTION_PUBLISHER,
                                                    RepoViewer :: PARAM_ID => $content_object->get_id())), false),
                            ToolbarItem :: DISPLAY_ICON));
        }

        if ($content_object->has_right(RepositoryRights :: VIEW_RIGHT, $this->get_user_id()))
        {
            $toolbar->add_item(
                    new ToolbarItem(Translation :: get('Preview'),
                            Theme :: get_common_image_path() . 'action_browser.png',
                            $this->get_url(
                                    array_merge($this->get_parameters(),
                                            array(RepoViewer :: PARAM_ACTION => RepoViewer :: ACTION_VIEWER,
                                                    RepoViewer :: PARAM_ID => $content_object->get_id())), false),
                            ToolbarItem :: DISPLAY_ICON));
        }

        if ($content_object->has_right(RepositoryRights :: COLLABORATE_RIGHT, $this->get_user_id()))
        {
            $toolbar->add_item(
                    new ToolbarItem(Translation :: get('EditAndPublish'),
                            Theme :: get_common_image_path() . 'action_editpublish.png',
                            $this->get_url(
                                    array_merge($this->get_parameters(),
                                            array(RepoViewer :: PARAM_ACTION => RepoViewer :: ACTION_CREATOR,
                                                    RepoViewer :: PARAM_EDIT_ID => $content_object->get_id())), false),
                            ToolbarItem :: DISPLAY_ICON));
        }

        if ($content_object instanceof ComplexContentObjectSupport)
        {

            $preview_url = Path :: get_launcher_application_path(true) . 'index.php?' . Application :: PARAM_APPLICATION . '=' . ComplexDisplayPreviewLauncher :: APPLICATION_NAME . '&' . RepositoryManager :: PARAM_CONTENT_OBJECT_ID . '=' . $content_object->get_id();
            $onclick = '" onclick="javascript:openPopup(\'' . $preview_url . '\'); return false;';
            $toolbar->add_item(
                    new ToolbarItem(Translation :: get('Preview', null, Utilities :: COMMON_LIBRARIES),
                            Theme :: get_common_image_path() . 'action_preview.png', $preview_url,
                            ToolbarItem :: DISPLAY_ICON, false, $onclick, '_blank'));
        }

        return $toolbar;
    }

    function is_shared_object_browser()
    {
        return (Request :: get(self :: SHARED_BROWSER) == 1);
    }

    function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add_help('repo_viewer_browser');
    }

    function is_allowed_shared_object_browsing()
    {
        $parameters = $this->get_parameters();
        $allowed = $parameters[self :: SHARED_BROWSER_ALLOWED];
        if ($allowed === false)
        {
            return false;
        }
        else
        {
            return true;
        }
    }
}
?>