<?php
namespace common\libraries\package;

use common\libraries\MessageLogger;

use core\lynx\package\Package;
use core\lynx\package\DependencyVerifier;

use common\libraries\Path;
use common\libraries\NoAuthenticationWebservice;
use common\libraries\Utilities;
use common\libraries\Translation;
use common\libraries\Filesystem;
use common\libraries\Registration;
use common\libraries\Setting;

use DOMDocument;

use home\HomeBlockRegistration;
use home\HomeManager;

use reporting\ReportingManager;
use reporting\ReportingBlockRegistration;
use reporting\ReportingTemplateRegistration;
use reporting\Reporting;

use rights\RightsManager;
use rights\RightsUtilities;

use tracking\TrackingManager;
use tracking\TrackingDataManager;
use tracking\Event;
use tracking\TrackerRegistration;
use tracking\EventRelTracker;

use webservice\WebserviceManager;
use webservice\WebserviceCategory;
use webservice\WebserviceRegistration;

abstract class Installer
{
    const TYPE_NORMAL = '1';
    const TYPE_CONFIRM = '2';
    const TYPE_WARNING = '3';
    const TYPE_ERROR = '4';
    const INSTALL_SUCCESS = 'success';
    const INSTALL_MESSAGE = 'message';

    /**
     * The datamanager which can be used by the installer of the application
     */
    private $data_manager;

    /**
     * Message to be displayed upon completion of the installation procedure
     */
    private $message;

    /**
     * Form values passed on from the installation wizard
     */
    private $form_values;

    /**
     * Constructor
     */
    function __construct($form_values, $data_manager = null)
    {
        $this->form_values = $form_values;
        $this->data_manager = $data_manager;
        $this->message = array();
    }

    /**
     * Installs and configures the application
     *
     * @return boolean
     */
    function install()
    {
        if (! $this->verify_dependencies())
        {
            return false;
        }

        if (! $this->install_storage_units())
        {
            return false;
        }

        if (! $this->configure_package())
        {
            return false;
        }

        return $this->installation_successful();
    }

    /**
     * Verifies the package dependencies
     *
     * @param $package_attributes
     * @return boolean
     */
    function verify_dependencies()
    {
        $verifier = new DependencyVerifier(Package :: get(static :: context()));
        $success = $verifier->is_installable();

        $this->add_message(self :: TYPE_NORMAL, $verifier->get_logger()->render());

        if (! $success)
        {
            return $this->installation_failed(Translation :: get('PackageDependenciesFailed'));
        }
        else
        {
            $this->add_message(self :: TYPE_NORMAL, Translation :: get('PackageDependenciesVerified'));
            return true;
        }
    }

    /**
     * Scans for the available storage units in the install folder and creates them
     *
     * @return boolean
     */
    function install_storage_units()
    {
        $dir = $this->get_path() . 'php/package/install/';
        $files = Filesystem :: get_directory_content($dir, Filesystem :: LIST_FILES);

        foreach ($files as $file)
        {
            if ((substr($file, - 3) == 'xml'))
            {
                if (! $this->create_storage_unit($file))
                {
                    return false;
                }
            }
        }

        return true;
    }

    /**
     * Parses an XML file describing a storage unit. For defining the 'type' of the field, the same definition is used
     * as the PEAR::MDB2 package. See http://pear.php.net/manual/en/package.database. mdb2.datatypes.php
     *
     * @param $file string The complete path to the XML-file from which the storage unit definition should be read.
     * @return array An with values for the keys 'name','properties' and 'indexes'
     */
    public static function parse_xml_file($file)
    {
        $name = '';
        $properties = array();
        $indexes = array();

        $doc = new DOMDocument();
        $doc->load($file);
        $object = $doc->getElementsByTagname('object')->item(0);
        $name = $object->getAttribute('name');
        $xml_properties = $doc->getElementsByTagname('property');
        $attributes = array('type', 'length', 'unsigned', 'notnull', 'default', 'autoincrement', 'fixed');
        foreach ($xml_properties as $index => $property)
        {
            $property_info = array();
            foreach ($attributes as $index => $attribute)
            {
                if ($property->hasAttribute($attribute))
                {
                    $property_info[$attribute] = $property->getAttribute($attribute);
                }
            }
            $properties[$property->getAttribute('name')] = $property_info;
        }
        $xml_indexes = $doc->getElementsByTagname('index');
        foreach ($xml_indexes as $key => $index)
        {
            $index_info = array();
            $index_info['type'] = $index->getAttribute('type');
            $index_properties = $index->getElementsByTagname('indexproperty');
            foreach ($index_properties as $subkey => $index_property)
            {
                $index_info['fields'][$index_property->getAttribute('name')] = array(
                        'length' => $index_property->getAttribute('length'));
            }
            $indexes[$index->getAttribute('name')] = $index_info;
        }
        $result = array();
        $result['name'] = $name;
        $result['properties'] = $properties;
        $result['indexes'] = $indexes;

        return $result;
    }

    function add_message($type = self :: TYPE_NORMAL, $message)
    {
        switch ($type)
        {
            case self :: TYPE_NORMAL :
                $this->message[] = $message;
                break;
            case self :: TYPE_CONFIRM :
                $this->message[] = '<span style="color: green; font-weight: bold;">' . $message . '</span>';
                break;
            case self :: TYPE_WARNING :
                $this->message[] = '<span style="color: orange; font-weight: bold;">' . $message . '</span>';
                break;
            case self :: TYPE_ERROR :
                $this->message[] = '<span style="color: red; font-weight: bold;">' . $message . '</span>';
                break;
            default :
                $this->message[] = $message;
                break;
        }
    }

    function set_message($message)
    {
        $this->message = $message;
    }

    /**
     *
     * @return multitype:string
     */
    function get_message()
    {
        return $this->message;
    }

    function set_form_values($form_values)
    {
        $this->form_values = $form_values;
    }

    function get_form_values()
    {
        return $this->form_values;
    }

    function set_data_manager($data_manager)
    {
        $this->data_manager = $data_manager;
    }

    function get_data_manager()
    {
        return $this->data_manager;
    }

    /**
     *
     * @return string
     */
    function retrieve_message()
    {
        return implode('<br />' . "\n", $this->get_message());
    }

    /**
     * Parses an XML file and sends the request to the database manager
     *
     * @param $path String
     */
    function create_storage_unit($path)
    {
        $storage_unit_info = self :: parse_xml_file($path);

        $this->add_message(self :: TYPE_NORMAL,
                Translation :: get_instance()->get('StorageUnitCreation', null, 'install') . ': <em>' . $storage_unit_info['name'] . '</em>');
        if (! $this->data_manager->create_storage_unit($storage_unit_info['name'], $storage_unit_info['properties'],
                $storage_unit_info['indexes']))
        {
            return $this->installation_failed(
                    Translation :: get_instance()->get('StorageUnitCreationFailed', null, 'install') . ': <em>' . $storage_unit_info['name'] . '</em>');
        }
        else
        {
            return true;
        }
    }

    /**
     * Parses an XML file and sends the request to the tracking database manager
     *
     * @param $path String
     */
    function create_tracking_storage_unit($path)
    {
        $tdm = TrackingDataManager :: get_instance();
        $storage_unit_info = self :: parse_xml_file($path);
        $this->add_message(self :: TYPE_NORMAL,
                Translation :: get('StorageUnitCreation', null, 'install') . ': <em>' . $storage_unit_info['name'] . '</em>');
        if (! $tdm->create_storage_unit($storage_unit_info['name'], $storage_unit_info['properties'],
                $storage_unit_info['indexes']))
        {
            return $this->installation_failed(
                    Translation :: get('TrackingStorageUnitCreationFailed', null, 'install') . ': <em>' . $storage_unit_info['name'] . '</em>');
        }
        else
        {
            return true;
        }
    }

    // TODO: It's probably a good idea to write some kind of XML-parsing class
    // that automatically converts the entire thing to a uniform array or
    // object.

    function parse_application_events($file)
    {
        $doc = new DOMDocument();
        $result = array();

        $doc->load($file);
        $object = $doc->getElementsByTagname('package')->item(0);
        $result['context'] = $object->getAttribute('context');

        // Get events
        $events = $doc->getElementsByTagname('event');
        $trackers = array();

        foreach ($events as $index => $event)
        {
            $event_name = $event->getAttribute('name');
            $trackers = array();

            // Get trackers in event
            $event_trackers = $event->getElementsByTagname('tracker');
            $attributes = array('name', 'active');

            foreach ($event_trackers as $index => $event_tracker)
            {
                $property_info = array();

                foreach ($attributes as $index => $attribute)
                {
                    if ($event_tracker->hasAttribute($attribute))
                    {
                        $property_info[$attribute] = $event_tracker->getAttribute($attribute);
                    }
                }
                $trackers[$event_tracker->getAttribute('name')] = $property_info;
            }

            $result['events'][$event_name]['name'] = $event_name;
            $result['events'][$event_name]['trackers'] = $trackers;
        }

        return $result;
    }

    function parse_application_settings($file)
    {
        $doc = new DOMDocument();

        $doc->load($file);
        $object = $doc->getElementsByTagname('package')->item(0);

        $setting_elements = $doc->getElementsByTagname('setting');
        $settings = array();

        foreach ($setting_elements as $index => $setting_element)
        {
            $settings[$setting_element->getAttribute('name')] = array(
                    'default' => $setting_element->getAttribute('default'),
                    'user_setting' => $setting_element->getAttribute('user_setting'));
        }

        return $settings;
    }

    /**
     * Function used to register a block.
     */
    function register_reporting_block($array)
    {
        return Reporting :: create_reporting_block_registration($array);
    }

    function register_reporting_template(&$props)
    {
        return Reporting :: create_reporting_template_registration($props);
    }

    function register_reporting()
    {
        $dirblock = static :: get_path() . 'php/reporting/blocks';
        $dir = static :: get_path() . 'php/reporting/templates';

        if (is_dir($dirblock) || is_dir($dir))
        {
            $this->add_message(self :: TYPE_NORMAL,
                    '<span class="subtitle">' . Translation :: get('TypeName', null,
                            ReportingManager :: APPLICATION_NAME) . '</span>');

            if (is_dir($dirblock))
            {
                $files = Filesystem :: get_directory_content($dirblock, Filesystem :: LIST_FILES);
                if (count($files) > 0)
                {
                    foreach ($files as $file)
                    {
                        if ((substr($file, - 16) == '_block.class.php'))
                        {
                            $bla = explode('.', basename($file));
                            $classname = static :: context() . Utilities :: underscores_to_camelcase($bla[0]);
                            $block = $bla[0];

                            $props = array();
                            $props[ReportingBlockRegistration :: PROPERTY_APPLICATION] = Utilities :: get_package_name_from_object(
                                    $this);
                            $props[ReportingBlockRegistration :: PROPERTY_BLOCK] = $block;
                            if ($this->register_reporting_block($props))
                            {
                                $this->add_message(self :: TYPE_NORMAL,
                                        Translation :: get('RegisteredBlock', null,
                                                ReportingManager :: APPLICATION_NAME) . ': <em>' . $props[ReportingBlockRegistration :: PROPERTY_BLOCK] . '</em>');
                            }
                            else
                            {
                                $this->add_message(self :: TYPE_ERROR,
                                        Translation :: get('ReportingBlockRegistrationFailed', null, 'install') . ': <em>' . $props[ReportingBlockRegistration :: PROPERTY_BLOCK] . '</em>');
                                return false;
                            }
                        }
                    }
                }
            }

            if (is_dir($dir))
            {
                $files = Filesystem :: get_directory_content($dir, Filesystem :: LIST_FILES);

                if (count($files) > 0)
                {
                    foreach ($files as $file)
                    {
                        if ((substr($file, - 19) == '_template.class.php'))
                        {
                            $bla = explode('.', basename($file));
                            $classname = static :: context() . '\\' . Utilities :: underscores_to_camelcase($bla[0]);
                            $template = $bla[0];
                            $props = array();
                            $props[ReportingTemplateRegistration :: PROPERTY_PLATFORM] = $classname :: is_platform();
                            $props[ReportingTemplateRegistration :: PROPERTY_APPLICATION] = Utilities :: get_package_name_from_object(
                                    $this);
                            $props[ReportingTemplateRegistration :: PROPERTY_TEMPLATE] = $template;
                            if ($this->register_reporting_template($props))
                            {
                                $this->add_message(self :: TYPE_NORMAL,
                                        Translation :: get('RegisteredTemplate', null,
                                                ReportingManager :: APPLICATION_NAME) . ': <em>' . $props[ReportingTemplateRegistration :: PROPERTY_TEMPLATE] . '</em>');
                            }
                            else
                            {
                                $this->add_message(self :: TYPE_ERROR,
                                        Translation :: get('ReportingTemplateRegistrationFailed', null, 'install') . ': <em>' . $props[ReportingTemplateRegistration :: PROPERTY_TEMPLATE] . '</em>');
                                return false;
                            }
                        }
                    }
                }
            }

            $this->add_message(self :: TYPE_NORMAL,
                    Translation :: get('ReportingAdded', null, ReportingManager :: APPLICATION_NAME));
            $this->add_message(self :: TYPE_NORMAL, '');
        }

        return true;
    }

    /**
     *
     * @return boolean
     */
    function register_home()
    {
        $dirblock = static :: get_path() . 'php/blocks/type';

        if (is_dir($dirblock))
        {
            $files = Filesystem :: get_directory_content($dirblock, Filesystem :: LIST_FILES);
            if (count($files) > 0)
            {
                $this->add_message(self :: TYPE_NORMAL,
                        '<span class="subtitle">' . Translation :: get('TypeName', null,
                                HomeManager :: APPLICATION_NAME) . '</span>');

                foreach ($files as $file)
                {
                    $file_path_info = pathinfo($file);

                    if ($file_path_info['extension'] === 'php')
                    {
                        $file_info = pathinfo($file_path_info['filename']);
                        $block_name = $file_info['filename'];

                        $home_block_registration = new HomeBlockRegistration();
                        $home_block_registration->set_context(static :: context());
                        $home_block_registration->set_block($block_name);

                        if ($home_block_registration->create())
                        {
                            $this->add_message(self :: TYPE_NORMAL,
                                    Translation :: get('RegisteredBlock', null, ReportingManager :: APPLICATION_NAME) . ': <em>' . $block_name . '</em>');
                        }
                        else
                        {
                            $this->add_message(self :: TYPE_ERROR,
                                    Translation :: get('HomeBlockRegistrationFailed', null,
                                            ReportingManager :: APPLICATION_NAME) . ': <em>' . $block_name . '</em>');
                            return false;
                        }
                    }
                }

                $this->add_message(self :: TYPE_NORMAL,
                        Translation :: get('HomeBlocksAdded', null, HomeManager :: APPLICATION_NAME));
                $this->add_message(self :: TYPE_NORMAL, '');
            }
        }

        return true;
    }

    /**
     * Registers the trackers, events and creates the storage units for the trackers
     */
    function register_trackers()
    {
        $dir = static :: get_path() . 'php/trackers/tracker_tables/';
        $trackers_file = static :: get_path() . 'php/trackers/trackers.xml';

        if (is_dir($dir) || file_exists($trackers_file))
        {
            $this->add_message(self :: TYPE_NORMAL,
                    '<span class="subtitle">' . Translation :: get('TypeName', null,
                            TrackingManager :: APPLICATION_NAME) . '</span>');

            $files = array();

            if (is_dir($dir))
            {
                $files = Filesystem :: get_directory_content($dir, Filesystem :: LIST_FILES);

                if (count($files) > 0)
                {
                    foreach ($files as $file)
                    {
                        if ((substr($file, - 3) == 'xml'))
                        {
                            $this->create_tracking_storage_unit($file);
                        }
                    }
                }
            }

            if (! file_exists($trackers_file))
            {
                if (count($files) > 0)
                {
                    // $warning_message = Translation :: get('UnlinkedTrackers',
                    // null, 'install') . ': <em>' . Translation ::
                    // get('CheckUnlinkedTrackers', null, TrackingManager ::
                    // APPLICATION_NAME) . ' ' . $path . '</em>';
                    // $this->add_message(self :: TYPE_WARNING, $warning_message);
                }
                return true;
            }

            $xml = $this->parse_application_events($trackers_file);
            if (! isset($xml['events']))
            {
                if (count($files) > 0)
                {
                    // $warning_message = Translation :: get('UnlinkedTrackers',
                    // null, 'install') . ': <em>' . Translation ::
                    // get('CheckUnlinkedTrackers', null, TrackingManager ::
                    // APPLICATION_NAME) . ' ' . $path . '</em>';
                    // $this->add_message(self :: TYPE_WARNING, $warning_message);
                }
                return true;
            }

            $registered_trackers = array();
            foreach ($xml['events'] as $event_name => $event_properties)
            {

                $event_block = isset($event_properties['block']) ? $event_properties['block'] : Utilities :: get_package_name_from_object(
                        $this);

                $store = TrackingDataManager :: get_instance();
                $event = $store->retrieve_event_by_name($event_name, $event_block);

                if (! $event)
                {
                    $event_block = $event_block ? $event_block : Utilities :: get_package_name_from_namespace(
                            $xml['context']);
                    $event = new Event();
                    $event->set_name($event_name);
                    $event->set_active(true);
                    $event->set_block($event_block);

                    if (! $event->create())
                    {
                        $this->add_message(self :: TYPE_ERROR,
                                Translation :: get('EventCreationFailed', null, TrackingManager :: APPLICATION_NAME) . ': <em>' . $event_properties['name'] . '</em>');
                        return false;
                    }
                }

                foreach ($event_properties['trackers'] as $tracker_name => $tracker_properties)
                {
                    if (! array_key_exists($tracker_properties['name'], $registered_trackers))
                    {
                        $the_tracker = new TrackerRegistration();
                        $the_tracker->set_tracker($tracker_properties['name'] . '_tracker');
                        $the_tracker->set_application(Utilities :: get_package_name_from_namespace($xml['context']));

                        if (! $the_tracker->create())
                        {
                            $this->add_message(self :: TYPE_ERROR,
                                    Translation :: get('TrackerRegistrationFailed', null,
                                            TrackingManager :: APPLICATION_NAME) . ': <em>' . $tracker_properties['name'] . '</em>');
                            return false;
                        }

                        $registered_trackers[$tracker_properties['name']] = $the_tracker;
                    }

                    $rel = new EventRelTracker();
                    $rel->set_tracker_id($registered_trackers[$tracker_properties['name']]->get_id());
                    $rel->set_event_id($event->get_id());
                    $rel->set_active(true);
                    if ($rel->create())
                    {
                        $this->add_message(self :: TYPE_NORMAL,
                                Translation :: get('TrackersRegisteredToEvent', null,
                                        TrackingManager :: APPLICATION_NAME) . ': <em>' . $event_properties['name'] . ' + ' . $tracker_properties['name'] . '</em>');
                    }
                    else
                    {
                        $this->add_message(self :: TYPE_ERROR,
                                Translation :: get('TrackerRegistrationToEventFailed', null, 'install') . ': <em>' . $event_properties['name'] . '</em>');
                        return false;
                    }
                }
            }

            $this->add_message(self :: TYPE_NORMAL,
                    Translation :: get('TrackingAdded', null, TrackingManager :: APPLICATION_NAME));
            $this->add_message(self :: TYPE_NORMAL, '');
        }

        return true;
    }

    function configure_package()
    {
        $settings_file = static :: get_path() . 'php/settings/settings.xml';

        if (file_exists($settings_file))
        {
            $xml = $this->parse_application_settings($settings_file);

            foreach ($xml as $name => $parameters)
            {
                $setting = new Setting();
                $setting->set_context(static :: context());
                $setting->set_variable($name);
                $setting->set_value($parameters['default']);

                $user_setting = $parameters['user_setting'];
                if ($user_setting)
                {
                    $setting->set_user_setting($user_setting);
                }
                else
                {
                    $setting->set_user_setting(0);
                }

                if (! $setting->create())
                {
                    $message = Translation :: get('PackageConfigurationFailed', null, 'install');
                    return $this->installation_failed($message);
                }
            }

            $this->add_message(self :: TYPE_NORMAL,
                    Translation :: get('PackageSettingsAdded', null, 'install'));
        }

        return true;
    }

    function register_package()
    {
        $namespace = Utilities :: get_namespace_from_object($this);
        $package_name = Utilities :: get_package_name_from_object($this);

        $application = $package_name;

        $this->add_message(self :: TYPE_NORMAL, Translation :: get('RegisteringPackage', null, 'install'));

        $package_info = Package :: get($namespace);

        $application_registration = new Registration();

        $parent_namespace = Utilities :: get_namespace_parent($namespace);
        $application_registration->set_type((! $parent_namespace ? 'core' : $parent_namespace));

        $application_registration->set_name($package_info->get_code());
        $application_registration->set_category($package_info->get_category());
        $application_registration->set_version($package_info->get_version());
        $application_registration->set_status(Registration :: STATUS_ACTIVE);
        $application_registration->set_context($namespace);

        if (! $application_registration->create())
        {
            return $this->installation_failed(Translation :: get('PackageRegistrationFailed', null, 'install'));
        }
        else
        {
            return true;
        }
    }

    /**
     * deprecated Registers the webservices
     */
    function register_webservices()
    {
        $methods = array('get', 'get_list', 'create', 'update', 'delete');

        $path = static :: get_path() . 'php/webservices/';

        if (is_dir($path))
        {
            $folders = Filesystem :: get_directory_content($path, Filesystem :: LIST_DIRECTORIES, false);
            if (count($folders) > 0)
            {
                $this->add_message(self :: TYPE_NORMAL,
                        '<span class="subtitle">' . Translation :: get('TypeName', null,
                                WebserviceManager :: APPLICATION_NAME) . '</span>');

                $application_webservice_category = new WebserviceCategory();
                $application_webservice_category->set_name(Translation :: get('TypeName', null, static :: context()));
                if (! $application_webservice_category->create())
                {
                    $this->add_message(self :: TYPE_ERROR,
                            Translation :: get('WebserviceCategoryCreationFailed', null,
                                    WebserviceManager :: APPLICATION_NAME) . ' : <em>' . $application_webservice_category->get_name() . '</em>');
                    return false;
                }
                else
                {
                    $this->add_message(self :: TYPE_NORMAL,
                            Translation :: get('WebserviceCategoryCreated', null, WebserviceManager :: APPLICATION_NAME) . ': <em>' . $application_webservice_category->get_name() . '</em>');
                }

                foreach ($folders as $folder)
                {
                    $camelcase_folder = Utilities :: underscores_to_camelcase($folder);

                    $object_webservice_category = new WebserviceCategory();
                    $object_webservice_category->set_name(
                            Translation :: get($camelcase_folder, null, static :: context()));
                    $object_webservice_category->set_parent($application_webservice_category->get_id());
                    if (! $object_webservice_category->create())
                    {
                        $this->add_message(self :: TYPE_ERROR,
                                Translation :: get('WebserviceCategoryCreationFailed', null,
                                        WebserviceManager :: APPLICATION_NAME) . ' : <em>' . $object_webservice_category->get_name() . '</em>');
                        return false;
                    }
                    else
                    {
                        $this->add_message(self :: TYPE_NORMAL,
                                Translation :: get('WebserviceCategoryCreated', null,
                                        WebserviceManager :: APPLICATION_NAME) . ': <em>' . $object_webservice_category->get_name() . '</em>');
                    }

                    foreach ($methods as $method)
                    {
                        $file = $path . $folder . '/' . $method . '.class.php';
                        if (file_exists($file))
                        {
                            $camelcase_method = Utilities :: underscores_to_camelcase($method);

                            $webservice = new WebserviceRegistration();

                            $webservice->set_name(
                                    Translation :: get($camelcase_method, null, WebserviceManager :: APPLICATION_NAME));
                            $webservice->set_description(
                                    Translation :: get($camelcase_folder . $camelcase_method . 'Description', null,
                                            static :: context()));

                            $webservice->set_active(1);
                            $webservice->set_category($object_webservice_category->get_id());
                            $webservice->set_code(static :: context() . '_' . $folder . '_' . $method);

                            $class = static :: context() . '\\' . Utilities :: underscores_to_camelcase(
                                    $folder . '_' . $method . '_webservice_handler');
                            $object = new $class();
                            if ($object instanceof NoAuthenticationWebservice)
                            {
                                $webservice->set_authentication(0);
                            }
                            else
                            {
                                $webservice->set_authentication(1);
                            }

                            if (! $webservice->create())
                            {
                                $this->add_message(self :: TYPE_ERROR,
                                        Translation :: get('WebserviceCreationFailed', null,
                                                WebserviceManager :: APPLICATION_NAME) . ' : <em>' . $webservice->get_name() . '</em>');
                                return false;
                            }
                            else
                            {
                                $this->add_message(self :: TYPE_NORMAL,
                                        Translation :: get('WebserviceCreated', null,
                                                WebserviceManager :: APPLICATION_NAME) . ': <em>' . $webservice->get_name() . '</em>');
                            }
                        }
                    }
                }

                $this->add_message(self :: TYPE_NORMAL,
                        Translation :: get('WebserviceSucces', null, WebserviceManager :: APPLICATION_NAME));
                $this->add_message(self :: TYPE_NORMAL, '');
            }
        }

        return true;
    }

    function post_process()
    {
        if (! $this->register_package())
        {
            return false;
        }

        $this->add_message(self :: TYPE_NORMAL, '');

        // Parse the Locations XML of the application
        // $this->add_message(self :: TYPE_NORMAL,
        // '<span class="subtitle">' . Translation :: get('TypeName', null, RightsManager :: APPLICATION_NAME) .
        // '</span>');
        if (! RightsUtilities :: create_application_root_location(Utilities :: get_package_name_from_object($this)))
        {
            return $this->installation_failed(
                    Translation :: get('LocationsFailed', null, RightsManager :: APPLICATION_NAME));
        }
        else
        {
            // $this->add_message(self :: TYPE_NORMAL,
            // Translation :: get('LocationsAdded', null, RightsManager :: APPLICATION_NAME));
        }
        // $this->add_message(self :: TYPE_NORMAL, '');

        // Handle any and every other thing that needs to happen after
        // the entire kernel was installed
        // VARIOUS #1: Tracking
        if (! $this->register_trackers())
        {
            return $this->installation_failed(
                    Translation :: get('TrackingFailed', null, TrackingManager :: APPLICATION_NAME));
        }

        // VARIOUS #2: Reporting
        if (! $this->register_reporting())
        {
            return $this->installation_failed(
                    Translation :: get('ReportingFailed', null, ReportingManager :: APPLICATION_NAME));
        }

        // VARIOUS #3: Home blocks / widgets
        if (! $this->register_home())
        {
            return $this->installation_failed(Translation :: get('HomeFailed', null, HomeManager :: APPLICATION_NAME));
        }

        // VARIOUS #4: Webservices
        if (! $this->register_webservices())
        {
            return $this->installation_failed(
                    Translation :: get('WebserviceFailed', null, WebserviceManager :: APPLICATION_NAME));
        }

        // VARIOUS #5: The rest
        if (method_exists($this, 'install_extra'))
        {
            $this->add_message(self :: TYPE_NORMAL,
                    '<span class="subtitle">' . Translation :: get('Various', null, 'install') . '</span>');
            if (! $this->install_extra())
            {
                return $this->installation_failed(Translation :: get('VariousFailed', null, 'install'));
            }
            else
            {
                $this->add_message(self :: TYPE_NORMAL, Translation :: get('VariousFinished', null, 'install'));
            }
            $this->add_message(self :: TYPE_NORMAL, '');
        }
        return $this->installation_successful(Translation :: get('ConfigurationSuccess', null, 'install'));
    }

    function installation_failed($error_message)
    {
        $this->add_message(self :: TYPE_ERROR, $error_message);
        $this->add_message(self :: TYPE_ERROR, Translation :: get('ApplicationInstallFailed', null, 'install'));
        $this->add_message(self :: TYPE_ERROR, Translation :: get('PlatformInstallFailed', null, 'install'));
        return false;
    }

    function installation_successful($message = null)
    {
        if (! $message)
        {
            $message = Translation :: get('InstallSuccess', null, 'install');
        }

        $this->add_message(self :: TYPE_CONFIRM, $message);
        return true;
    }

    /**
     * Creates an application-specific installer.
     *
     * @param $context string The namespace of the package for which we want to start the installer.
     * @param $values string The form values passed on by the wizard.
     */
    static function factory($context, $values)
    {
        $class = $context . '\Installer';
        $base_path = Path :: namespace_to_full_path($context);
        if (! class_exists($class))
        {
            $class = $context . '\\' . Utilities :: get_package_name_from_namespace($context, true) . 'Installer';
        }
        return new $class($values);
    }

    /**
     * Returns the list with extra installable packages that are connected to this package Example: tool package in
     * weblcms optional application
     *
     * @return String[]
     */
    function get_additional_installers()
    {
        return array();
    }

    function extract_xml_file($file)
    {
        return Utilities :: extract_xml_file($file);
    }

    function get_path()
    {
        return Path :: namespace_to_full_path(static :: context());
    }

    /**
     * Get the fully qualified class name of the DataClass object
     *
     * @return string
     */
    static function class_name()
    {
        return get_called_class();
    }

    /**
     * Get the namespace of the DataClass object
     *
     * @return string
     */
    static function context()
    {
        return Utilities :: get_namespace_from_classname(get_called_class());
    }

}
?>