<?php
namespace common\libraries;

/**
 * @package common.authentication.shibboleth
 */
use group\GroupRelUser;
use user\User;

/**
 * This class allow to login into Chamilo by using the Shibboleth authentication system.
 *
 * To use this class, the folder 'shibboleth' must be configured in your webserver settings
 * to be protected by Shibboleth. In this way, user attributes can be retrieved from the code.
 *
 * Shibboleth attributes can be mapped to Chamilo user atributes by setting the correct mappings
 * in the 'shibboleth.xml' file. Role / Group mapping can be configured in this file as well.
 *
 * The configuration file allow to define :
 *
 * - the mapping between Shibboleth attributes and Chamilo user attributes
 * - the mapping between the 'affiliation' attribute and Chamilo user roles / groups
 * - default values for Chamilo user attributes
 * - what Chamilo user attributes must be updated when a user logs in again
 *
 * About the Role / Group mapping
 * *******************************
 * The role / group mapping can retrieved from an attribute mapped to 'affiliation'
 * This attribute may contain more than one value, separated by a character (typically a semi-colon).
 *
 * The config file allow to add possible values that may be given by Shibboleth and allow for each of them
 * to specify to which group and / or role it must be mapped.
 * If a specific value must be mapped to more than one group or role, you can add many times the same value.
 *
 * Each value has a precedence. It allows to look for values in priority. All values with the same precedence
 * are searched in the Shibboleth attributes at the same time. If the user 'affiliation' attribute contains one
 * of the searched values, the role / group links are created and the next values with the next precedence are ignored.
 * If no value match the user 'affiliation' attribute for a precedence, then the values with next precedence
 * are looked for.
 *
 * It allows for instance in weblcms to check first if a user is a teacher. If he is a teacher, he can be created
 * with "teacher" rights, meaning with specific group(s) and / or role(s) links. In the case the user was not recognized
 * as a teacher on the first step (with the first precedence value), the system can then (and only then) check
 * if the user is a "student", and create the corresponding role / group links for him.
 *
 */
class ShibbolethAuthentication extends ExternalAuthentication
{

    const PARAM_SOURCE = 'source_urls';
    const SHIBBOLETH_AUTHENTICATION = 'shibboleth';
    const RIGHTS_SEPARATOR = 'RIGHTS_SEPARATOR';

    private $config_xml_document = null;
    private $user_status_is_unclear = false;

    /*
     * Constructor
     */
    function __construct()
    {
        $this->initialize();
    }

    /**
     * If true and it is needed then check_login redirect the browser to the Shibboleth Index page. If false do not redirect.
     *
     * @return boolean
     */
    public function get_enable_forwarding()
    {
        return PlatformSetting :: get('enable_shibboleth_forwarding');
    }

    /**
     * The path to the Shibboleth Index Page. That is the one that must be secured at the web server leve by Shibboleth.
     *
     * @return string
     */
    public function get_shibboleth_index_path()
    {
        return Path :: get(WEB_PATH) . 'common/libraries/php/authentication/shibboleth/index.php';
    }

    /**
     * Return true if user is already loged in.
     *
     * @return boolean
     */
    public function is_logged_in()
    {
        return (bool) Session :: get_user_id();
    }

    /**
     * Returns true if Shibboleth is enabled. False otherwise.
     *
     * @return bool
     */
    public function is_enabled()
    {
        return PlatformSetting :: get('enable_shibboleth_authentication') && PlatformSetting :: get('enable_external_authentication');
    }

    /**
     * Forward/Redirect if needed to the Shibboleth - index - protected page in order to extract Shibboleth attributes.
     *
     * @return bool|exit
     */
    public function forward()
    {
        if (! $this->get_enable_forwarding())
        {
            return false;
        }

        $index_url = $this->get_shibboleth_index_path();

        //we don't forward the main page - that can be served to anonymous users
        if ((strpos(Redirect :: current_url(), 'index.php') !== false) || (strpos(Redirect :: current_url(), Path :: get(WEB_PATH)) !== false))
        {
            //messages are OK we accept that
            $parameters = $_GET;
            unset($parameters[Application :: PARAM_MESSAGE]);
            unset($parameters[Application :: PARAM_WARNING_MESSAGE]);
            unset($parameters[Application :: PARAM_ERROR_MESSAGE]);
            if (count($parameters) == 0)
            {
                return false;
            }
        }
        //we don't forward if we are already on the target page
        if (strpos(Redirect :: current_url(), 'shibboleth') !== false)
        {
            return false;
        }

        //if the page we are on is already protected by Shibboleth we don't need to forward the call to another page protected by Shibboleth
        if ($this->get_shibboleth_user_id())
        {
            return false;
        }

        $index_url .= '?' . self :: PARAM_SOURCE . '=' . urlencode(Redirect :: current_url());
        Redirect :: write_header($index_url);
        exit();
    }

    /**
     * If loged in redirect to the original url. If not url is provided go to home.
     */
    public function backward()
    {
        //we backward only on success to avoid infine loops
        if (! $this->is_logged_in())
        {
            return false;
        }

        if ($url = Request :: get(self :: PARAM_SOURCE))
        {
            //if we have a source url we get back to it
            Redirect :: write_header($url);
        }
        else
        {
            //otherwise we go to homepage
            $this->redirect_to_home_page();
        }
        exit();
    }

    /*
     * Initialize the Shibboleth authentication configuration
     */
    protected function initialize()
    {
        $doc = simplexml_load_file(dirname(__FILE__) . '/shibboleth.xml');
        if ($doc !== false)
        {
            $this->config_xml_document = $doc;

            parent :: initialize();

            $this->set_authentication_source_name(self :: SHIBBOLETH_AUTHENTICATION);
            $this->set_automatic_registration(true);
            $this->set_register_user_without_role(false);
        }
        else
        {
            $this->error('unable to load configuration');
        }
    }

    protected function error($message)
    {
        $parameters[Application :: PARAM_ERROR_MESSAGE] = $message;
        $url = Path :: get(WEB_PATH) . 'index.php';
        Redirect :: web_link($url, $parameters);
        exit();
    }

    /**
     * Inherited. Note that with Shibboleth $user, $username and $password are not used
     *
     * @param object|null $user
     * @param string|null $username
     * @param string|null $password
     */
    function check_login($user = null, $username = null, $password = null)
    {
        //we need to check because Shibboleth is called directly from the index page.
        if (! $this->is_enabled())
        {
            return $this->error('ShibbolethIsDisabled');
        }

        // With Shibboleth, we don't use $user, $username and $password
        unset($user);
        unset($username);
        unset($password);

        $this->forward();

        //we don't have Shibboelth's values so we can't log the user.
        if (! $shibboleth_id = $this->get_shibboleth_user_id())
        {
            return false;
        }

        // Check if user already exists


        $is_new_user = false;
        $user = $this->get_existing_user();

        if (isset($user))
        {
            //user already exists


            if ($this->has_fields_to_update_at_login())
            {
                /*
                 * Update user's fields that must be updated when an existing user logs in
                 */

                $user = $this->set_user_attributes($user, $this->get_fields_to_update_at_login());

                //save user in datasource
                if (! $user->update())
                {
                    echo 'An error occured while updating your informations';
                }
                else
                {
                    if (in_array(self :: PARAM_MAPPING_AFFILIATION, $this->get_fields_to_update_at_login()))
                    {
                        if (! $this->set_user_rights($user))
                        {
                            $this->error('Unable to set the user rights');
                        }
                    }
                }
            }
        }
        else
        {
            //user does not exist yet


            $is_new_user = true;

            if ($this->is_automatic_registration_enabled())
            {
                $user = $this->get_new_user();

                if (isset($user))
                {
                    if ($user->create())
                    {
                        if (array_key_exists(self :: PARAM_MAPPING_AFFILIATION, $this->get_user_attribute_mapping()))
                        {
                            if (! $this->set_user_rights($user))
                            {
                                $this->error('Unable to set the user rights');
                            }
                        }
                    }
                }
            }
            else
            {
                $this->error('You don\'t have the right to access this platform. Please contact the platform administrator');
            }
        }

        /*
         * Login user
         */
        if (isset($user))
        {
            if ($this->user_status_is_unclear && $this->get_display_rights_form_when_unclear_status())
            {
                $this->login($user, false);
                $this->redirect_to_rights_request_form($is_new_user);
            }
            else
            {
                $this->login($user, false);
                $this->backward();
            }
        }
        else
        {
            $this->error('Unable to perform login');
        }
    }

    /**
     * override
     */
    protected function initialize_user_attributes_mapping()
    {
        if (isset($this->config_xml_document))
        {
            $attributes_mapping = $this->config_xml_document->xpath('/authentication/attribute_mapping/attribute');

            $user_attribute_mapping = array();
            $fields_to_update_at_login = array();

            foreach ($attributes_mapping as $attribute)
            {
                $user_attribute_mapping[(string) $attribute->attributes()->mapped_to] = array(
                        'name' => (string) $attribute->attributes()->name,
                        'default' => (string) $attribute->attributes()->default_value);

                if (isset($attribute->attributes()->update_at_login) && $attribute->attributes()->update_at_login == 'true')
                {
                    $fields_to_update_at_login[] = (string) $attribute->attributes()->mapped_to;
                }
            }

            $this->set_user_attribute_mapping($user_attribute_mapping);
            $this->set_fields_to_update_at_login($fields_to_update_at_login);
        }
        else
        {
            echo '<p>Attribute mapping initialization error</p>';
        }
    }

    /**
     * override
     */
    protected function initialize_fields_to_update_at_login()
    {
        /*
         * Nothing is implemented here, as the initialization is already done when
         * 'initialize_user_attributes_mapping()' is called;
         */
    }

    /**
     * override
     */
    protected function initialize_role_attributes_mapping()
    {
        if (isset($this->config_xml_document))
        {
            $separator = $this->config_xml_document->xpath('/authentication/role_mapping/@name_separator');
            $this->set_config_parameter(self :: RIGHTS_SEPARATOR, $separator[0]);

            $display_form = $this->config_xml_document->xpath('/authentication/role_mapping/@display_request_form_when_unclear');
            if (isset($display_form[0]) && $display_form[0] == 'true')
            {
                $this->set_display_rights_form_when_unclear_status(true);
            }
            else
            {
                $this->set_display_rights_form_when_unclear_status(false);
            }

            $role_mapping = array();

            $value_nodes = $this->config_xml_document->xpath('/authentication/role_mapping/value');

            foreach ($value_nodes as $value_node)
            {
                $name = (string) $value_node->attributes()->name;
                $precedence = (string) $value_node->attributes()->precedence;
                $group_id = (string) $value_node->attributes()->group_id;
                $role_id = (string) $value_node->attributes()->role_id;

                if (! array_key_exists($precedence, $role_mapping))
                {
                    $role_mapping[$precedence] = array();
                }

                if (! array_key_exists($name, $role_mapping[$precedence]))
                {
                    $role_mapping[$precedence][$name] = array();
                    $role_mapping[$precedence][$name]['group_id'] = array();
                    $role_mapping[$precedence][$name]['role_id'] = array();
                }

                if (isset($group_id) && strlen($group_id) > 0)
                {
                    $role_mapping[$precedence][$name]['group_id'][] = $group_id;
                }
                if (isset($role_id) && strlen($role_id) > 0)
                {
                    $role_mapping[$precedence][$name]['role_id'][] = $role_id;
                }
            }

            $this->set_user_role_attribute_mapping($role_mapping);
        }
        else
        {
            echo '<p>Role mapping initialization error</p>';
        }
    }

    /**
     * Returns the Shibboleth User Id.
     *
     * @return string|null
     */
    protected function get_shibboleth_user_id()
    {
        $user_mapping = $this->get_user_attribute_mapping();
        return $this->get_shibboleth_value($user_mapping[self :: PARAM_MAPPING_EXTERNAL_UID]['name']);
    }

    /**
     * Returns the current user logged through Shibboleth if he already
     * exists in the datasource. Otherwise return null.
     *
     * @return User The current user if he already exists in the datasource
     */
    protected function get_existing_user()
    {
        $shibboleth_id = $this->get_shibboleth_user_id();

        $user = $this->retrieve_user_by_external_uid($shibboleth_id);

        if (isset($user))
        {
            return $user;
        }
        else
        {
            return null;
        }
    }

    /**
     * Creates and returns a new User based on the Shibboleth attributes
     *
     * @return User A new user
     */
    protected function get_new_user()
    {
        $user = new User();
        $user = $this->set_user_attributes($user);
        return $user;
    }

    /**
     * override
     *
     * @param User $user
     * @param array $fields_to_set
     * @return User
     */
    protected function set_user_attributes($user, $fields_to_set = null)
    {
        foreach ($this->get_user_attribute_mapping() as $field_name => $field_source)
        {
            $must_set = true;
            if (isset($fields_to_set) && is_array($fields_to_set) && count($fields_to_set) > 0 && ! in_array($field_name, $fields_to_set))
            {
                $must_set = false;
            }

            if ($must_set)
            {
                $value = $this->get_shibboleth_value($field_source['name']);

                /*
                 * Default value if no value is found
                 */
                if (! isset($value))
                {
                    if (isset($field_source['default']) && strlen($field_source['default']) > 0)
                    {
                        $value = $field_source['default'];
                    }
                }

                if ($field_name == self :: PARAM_MAPPING_LANG)
                {
                    $lang = $this->get_language_code($value);
                    $user->set_language($lang);
                }
                elseif (method_exists($user, 'set_' . $field_name))
                {
                    call_user_func_array(array($user, 'set_' . $field_name), array($value));
                }
            }
        }

        /*
         * case of new user -> init some more default values
         */
        $user_id = $user->get_id();
        if (! isset($user_id))
        {
            $user->set_username($this->generate_username($user->get_firstname(), $user->get_lastname()));
            $user->set_password(md5($this->generate_random_password())); //generate random password to prevent security hole if the user's auth_source is modified
            $user->set_auth_source($this->get_authentication_source_name());
            $user->set_platformadmin(0);
        }

        return $user;
    }

    /**
     * Add the given user in groups and / or grant the user roles.
     * The user must already have an id (as it is used to create groups and /or role links)
     *
     * @param User $user
     * @return Boolean role(s) / group(s) affiliation result
     */
    protected function set_user_rights($user)
    {
        if ($this->has_user_role_attribute_mapping() && array_key_exists(self :: PARAM_MAPPING_AFFILIATION, $this->get_user_attribute_mapping()))
        {
            $user_id = $user->get_id();
            if (isset($user) && isset($user_id))
            {
                /*
                 * Deletes all user_role and user_group relations for the user
                 * before recreating them with his affiliation attribute
                 */
                $user_groups = $user->get_user_groups();

                if (isset($user_groups))
                {
                    while ($user_group = $user_groups->next_result())
                    {
                        if (! $user_group->delete())
                        {
                            echo 'Unable to clear user groups';
                        }
                    }
                }

                $user_mapping = $this->get_user_attribute_mapping();
                $affiliation = $this->get_shibboleth_value($user_mapping[self :: PARAM_MAPPING_AFFILIATION]['name']);

                /*
                 * Default value if no value is found
                 */
                if (! isset($affiliation))
                {
                    if (isset($user_mapping[self :: PARAM_MAPPING_AFFILIATION]['default']) && strlen($user_mapping[self :: PARAM_MAPPING_AFFILIATION]['default']) > 0)
                    {
                        $affiliation = $user_mapping[self :: PARAM_MAPPING_AFFILIATION]['default'];
                    }
                }

                $affiliation_values = explode($this->get_config_parameter(self :: RIGHTS_SEPARATOR), $affiliation);

                $role_mapping = $this->get_user_role_attribute_mapping();

                /*
                 * If the user has an affiliation value that the config file doesn't know,
                 * his status is considered as unclear
                 */
                $configured_affiliation_values = array();

                foreach ($role_mapping as $mapping_for_precedence)
                {
                    foreach ($mapping_for_precedence as $attribute_value => $rights)
                    {
                        $configured_affiliation_values[] = $attribute_value;
                    }
                }
                foreach ($affiliation_values as $affiliation_value)
                {
                    if (! in_array($affiliation_value, $configured_affiliation_values))
                    {
                        $this->user_status_is_unclear = true;
                    }
                }

                /*
                 * Loops on each level of precedence and checks if a corresponding
                 * user affiliation value is found.
                 *
                 * If a value is found in the user attributes, the link between
                 * the user and group(s) and / or role(s) are created. The process then stops,
                 * meaning we don't look for other precedence levels
                 *
                 */
                $mapping_found = false;
                foreach ($role_mapping as $mapping_for_precedence)
                {
                    if ($mapping_found)
                    {
                        break;
                    }

                    $groups = array();

                    foreach ($mapping_for_precedence as $attribute_value => $rights)
                    {
                        if (in_array($attribute_value, $affiliation_values))
                        {
                            /*
                             * Find group mappings
                             */
                            foreach ($rights['group_id'] as $group_id)
                            {
                                if (! in_array($group_id, $groups))
                                {
                                    $groups[] = $group_id;
                                }
                            }

                            $mapping_found = true;
                        }
                    }
                }

                if (count($groups) > 0)
                {
                    /*
                     * Create group_user records
                     */
                    foreach ($groups as $group_id)
                    {
                        $group_user = new GroupRelUser();
                        $group_user->set_user_id($user_id);
                        $group_user->set_group_id($group_id);

                        if (! $group_user->create())
                        {
                            echo 'Unable to add user in group ' . $group_id;
                        }
                    }
                }
                return true;
            }
            else
            {
                return false;
            }
        }
        else
        {
            return false;
        }
    }

    /**
     * Return the shibboleth attribute value for the given attribute name
     *
     * So far Shibboleth gives values to PHP through HTTP headers (Shibboleth 1.x) or
     * environment variables (Shibboleth 2) that can be read through the $_SERVER variable
     *
     * @param String $shibboleth_param_name the shibboleth attribute name
     * @return String the shibboleth attribute value
     */
    protected function get_shibboleth_value($shibboleth_param_name)
    {
        return Request :: server($shibboleth_param_name);
    }

    /**
     * Print a table in the page with the current Shibboleth attributes.
     * Mainly useful for debugging the Shibboleth system.
     *
     * Note: see file 'show_my_infos.php' to see how this method can be called
     *
     */
    function print_shibboleth_attributes()
    {
        echo '<table cellpadding="5" cellspacing="0">';
        echo '<tr>';
        echo '<th>User property</th><th></th><th>Shibboleth parameter name</th><th></th><th>User value</th><th></th><th>Default value</th>';
        echo '</tr>';

        foreach ($this->get_user_attribute_mapping() as $field_name => $field_source)
        {

            echo '<tr>';
            echo '  <td>' . $field_name . '</td>';
            echo '  <td>&rarr;</td>';
            echo '  <td>' . $field_source['name'] . '</td>';
            echo '  <td>&rarr;</td>';
            echo '  <td>' . $this->get_shibboleth_value($field_source['name']) . '</td>';
            echo '  <td>&rarr;</td>';
            echo '  <td>' . $this->get_shibboleth_value($field_source['default']) . '</td>';
            echo '</tr>';
        }

        echo '</table>';
    }

}

?>