<?php
namespace common\libraries;

use admin\AdminDataManager;
use user\UserManager;

/**
 * This is a display library for Chamilo. Include/require it in your code to use its functionality. There are also
 * several display functions in the main api library. All functions static functions inside a class called Display, so
 * you use them like this: e.g. Display :: normal_message($message)
 */
class Display
{
    const MESSAGE_TYPE_CONFIRM = 'confirm';
    const MESSAGE_TYPE_NORMAL = 'normal';
    const MESSAGE_TYPE_WARNING = 'warning';
    const MESSAGE_TYPE_ERROR = 'error';

    /**
     * Displays a normal message. It is recommended to use this function to display any normal information messages.
     *
     * @author Roan Embrechts
     * @author Tim De Pauw
     * @author Hans De Bisschop
     * @param $message string - include any additional html tags if you need them
     * @param $return boolean
     * @return mixed
     */
    public static function normal_message($message, $return = false)
    {
        return self :: message(self :: MESSAGE_TYPE_NORMAL, $message, $return);
    }

    /**
     * Display a page which contains a normal message
     *
     * @param $message string
     */
    public static function normal_message_page($message)
    {
        self :: header();
        self :: normal_message($message);
        self :: footer();
        exit();
    }

    /**
     * Displays a message. It is recommended to use this function to display any confirmation or error messages.
     *
     * @author Hugues Peeters
     * @author Roan Embrechts
     * @author Tim De Pauw
     * @author Hans De Bisschop
     * @param $message string - include any additional html tags if you need them
     * @param $return boolean
     * @return mixed
     */
    public static function error_message($message, $return = false)
    {
        return self :: message(self :: MESSAGE_TYPE_ERROR, $message, $return);
    }

    /**
     * Display a page which contains an error message
     *
     * @param $message string
     */
    public static function error_page($message)
    {
        self :: header();
        self :: error_message($message);
        self :: footer();
        exit();
    }

    /**
     * Displays a message. It is recommended to use this function to display any warning messages.
     *
     * @author Hugues Peeters
     * @author Roan Embrechts
     * @author Tim De Pauw
     * @author Hans De Bisschop
     * @param $message string - include any additional html tags if you need them
     * @param $return boolean
     * @return mixed
     */
    public static function warning_message($message, $return = false)
    {
        return self :: message(self :: MESSAGE_TYPE_WARNING, $message, $return);
    }

    /**
     * Renders and/or displays a message.
     *
     * @author Hans De Bisschop
     * @param $type string
     * @param $message string
     * @param $return boolean
     * @return string void
     */
    public static function message($type = self :: MESSAGE_TYPE_NORMAL, $message, $return = false)
    {
        $html = array();
        $html[] = '<div class="' . $type . '-message">';
        $html[] = $message;
        $html[] = '<div class="close_message" id="closeMessage"></div>';
        $html[] = '</div>';

        if ($return)
        {
            return implode("\n", $html);
        }

        echo implode("\n", $html);
    }

    /**
     * Return an encrypted mailto hyperlink
     *
     * @param $email - (string) - e-mail
     * @param $text - (string) - clickable text
     * @param $style_class - (string) - optional, class from stylesheet
     * @return - encrypted mailto hyperlink
     */
    public static function encrypted_mailto_link($email, $clickable_text = null, $style_class = '')
    {
        if (is_null($clickable_text))
        {
            $clickable_text = $email;
        }
        // mailto already present?
        if (substr($email, 0, 7) != 'mailto:')
            $email = 'mailto:' . $email;

            // class (stylesheet) defined?
        if ($style_class != '')
        {
            $style_class = ' class="full_url_print ' . $style_class . '"';
        }
        else
        {
            $style_class = ' class="full_url_print"';
        }

        // encrypt email
        $hmail = '';
        for($i = 0; $i < strlen($email); $i ++)
            $hmail .= '&#' . ord($email{$i}) . ';';

            // encrypt clickable text if @ is present
        if (strpos($clickable_text, '@'))
        {
            for($i = 0; $i < strlen($clickable_text); $i ++)
                $hclickable_text .= '&#' . ord($clickable_text{$i}) . ';';
        }
        else
        {
            $hclickable_text = htmlspecialchars($clickable_text);
        }

        // return encrypted mailto hyperlink
        return '<a href="' . $hmail . '"' . $style_class . '>' . $hclickable_text . '</a>';
    }

    /**
     * Display the page header
     */
    public static function header($breadcrumbtrail)
    {
        $document_language = Translation :: get_instance()->get_language();
        if (empty($document_language))
        {
            // if there was no valid iso-code, use the english one
            $document_language = 'en';
        }

        $header = Header :: get_instance();
        $header->set_language_code($document_language);
        $header->set_page_title(PlatformSetting :: get('institution') . ' - ' . PlatformSetting :: get('site_name'));
        $header->add_default_headers();
        $header->display();

        if (! isset($text_dir))
        {
            $text_dir = 'ltr';
        }

        echo '<body dir="' . $text_dir . '">' . "\n";

        echo '<!-- #outerframe container to control some general layout of all pages -->' . "\n";
        echo '<div id="outerframe">' . "\n";

        // Banner
        $banner = new Banner();
        $banner->display();

        echo '<div id="main"> <!-- start of #main wrapper for #content and #menu divs -->';
        echo '<!--   Begin Of script Output   -->';
        echo '<div id="helpbox" class="helpdialog"></div>';
    }

    /**
     *
     * @param $page_title string Optional, defaults to site name if empty
     * @param $body_style string Optional
     */
    public static function small_header($page_title = null, $style = null, $html_headers = array())
    {
        $page_title = is_null($page_title) ? $page_title = PlatformSetting :: get('site_name') : $page_title;
        $style = is_null($style) ? 'body {background-color:white; padding: 10px;}' : $style;
        $html_headers = is_array($html_headers) ? $html_headers : array($html_headers);

        $document_language = Translation :: get_language();
        if (empty($document_language))
        {
            // if there was no valid iso-code, use the english one
            $document_language = 'en'; // @todo: shouldn't we put that in
                                           // Translation::get_languate()?
        }

        if (! isset($text_dir))
        {
            $text_dir = 'ltr';
        }

        $header = new Header($document_language);
        $header->add_default_headers();
        $header->set_page_title(PlatformSetting :: get('site_name'));
        // @todo: shouldn't we put that somewhere else? or add a class instead?
        if ($style)
        {
            $header->add_html_header('<style type="text/css">' . $style . '</style>');
        }
        $header->add_html_header(
                '<script type="text/javascript">var rootWebPath="' . Path :: get(WEB_PATH) . '"</script>');
        foreach ($html_headers as $html_header)
        {
            $header->add_html_header($html_header);
        }
        $header->display();

        echo '<body dir="' . $text_dir . '">' . "\n";
    }

    /**
     * Display the page footer
     */
    public static function footer()
    {
        $footer = new Footer();
        $footer->display();
    }

    public static function small_footer()
    {
        echo '</body>' . "\n";
        echo '</html>' . "\n";
    }

    public static function not_allowed($trail = null, $show_login_form = true)
    {
        if (is_null($trail))
        {
            $trail = BreadcrumbTrail :: get_instance();
        }
        self :: header($trail);
        $home_url = Path :: get(WEB_PATH);

        $html[] = Translation :: get('NotAllowed', null, Utilities :: COMMON_LIBRARIES);

        if ($show_login_form)
        {
            $html[] = self :: display_login_form();
        }

        self :: error_message(implode("\n", $html));
        $_SESSION['request_uri'] = $_SERVER['REQUEST_URI'];
        self :: footer();
        exit();
    }

    static function display_login_form()
    {
        $form = new FormValidator('formLogin', 'post',
                Redirect :: get_link(UserManager :: APPLICATION_NAME,
                        array(Application :: PARAM_ACTION => UserManager :: ACTION_LOGIN), array(), false,
                        Redirect :: TYPE_CORE));
        $renderer = & $form->defaultRenderer();
        $renderer->setElementTemplate('<div class="row">{element}</div>');
        // $renderer->setElementTemplate('<div>{element}</div>','submitAuth');
        $form->setRequiredNote(null);
        $form->addElement('text', 'login', Translation :: get('UserName'),
                array('size' => 20, 'onclick' => 'this.value=\'\';'));
        $form->addRule('login', Translation :: get('ThisFieldIsRequired', null, Utilities :: COMMON_LIBRARIES),
                'required');
        $form->addElement('password', 'password', Translation :: get('Pass'),
                array('size' => 20, 'onclick' => 'this.value=\'\';'));
        $form->addRule('password', Translation :: get('ThisFieldIsRequired'), 'required');
        $form->addElement('style_submit_button', 'submitAuth', Translation :: get('Login'),
                array('class' => 'positive login'));
        $form->setDefaults(array('login' => Translation :: get('Username'), 'password' => '*******'));
        return $form->toHtml();
    }

    public static function tool_title($titleElement)
    {
        if (is_string($titleElement))
        {
            $tit = $titleElement;
            unset($titleElement);
            $titleElement['mainTitle'] = $tit;
        }
        echo '<h3>';
        if ($titleElement['supraTitle'])
        {
            echo '<small>' . $titleElement['supraTitle'] . '</small><br>';
        }
        if ($titleElement['mainTitle'])
        {
            echo $titleElement['mainTitle'];
        }
        if ($titleElement['subTitle'])
        {
            echo '<br /><small>' . $titleElement['subTitle'] . '</small>';
        }
        echo '</h3>';
    }

    public static function get_progress_bar($percent, $step = 2)
    {
        $done = (int) ($percent / $step);
        $rest = (int) (100.0 / $step) - $done;
        return '<div class="progress_information"><div class="progress_bar">' . str_repeat('<div class="done"></div>',
                $done) . str_repeat('<div class=""></div>', $rest) . '</div><div class="progress_status">' . round(
                $percent, 2) . ' %</div></div>';
    }

    public static function get_rating_bar($percent, $show_text = true, $step = 2)
    {
        $done = (int) ($percent / $step);
        $rest = (int) (100.0 / $step) - $done;
        return '<div class="rating_information"><div class="rating_bar">' . str_repeat(
                '<div class="' . ($percent <= 50 ? 'bad' : $percent <= 75 ? 'average' : 'good') . '"></div>', $done) . str_repeat(
                '<div class=""></div>', $rest) . '</div>' . ($show_text ? '<div class="rating_status">' . round($percent,
                2) . ' %</div>' : '') . '</div>';
    }

    static function form_category($title = null, $extra_classes = null)
    {
        $html = array();

        if ($title != null)
        {
            $html[] = '<div class="configuration_form' . ($extra_classes ? ' ' . $extra_classes : '') . '" >';
            $html[] = '<span class="category">' . $title . '</span>';
        }
        else
        {
            $html[] = '<div style="clear: both;"></div>';
            $html[] = '</div>';
        }

        return implode("\n", $html);
    }

    static function form_row($label = null, $value = null)
    {
        $html = array();

        $html[] = '<div class="row">';
        $html[] = '<div class="label">' . $label . '</div>';
        $html[] = '<div class="formw">';
        $html[] = '<div class="element">' . $value . '</div>';
        $html[] = '</div>';
        $html[] = '<div class="clear">&nbsp;</div>';
        $html[] = '</div>';

        return implode("\n", $html);
    }
}
?>
