<?php
namespace common\libraries;

/**
 * This class describes a gallery table property model
 *
 * @package \common\libraries;
 * @author Hans De Bisschop
 */
abstract class NewGalleryObjectTablePropertyModel
{
    /**
     * **************************************************************************************************************
     * Default constants *
     * **************************************************************************************************************
     */
    const DEFAULT_ORDER_PROPERTY_INDEX = 0;
    const DEFAULT_ORDER_PROPERTY_DIRECTION = SORT_ASC;

    /**
     * **************************************************************************************************************
     * Properties *
     * **************************************************************************************************************
     */

    /**
     * The object table
     *
     * @var NewObjectTable
     */
    private $object_table;

    /**
     * The properties in the table.
     */
    private $properties;
    /**
     * The property by which the table is currently sorted.
     */
    private $order_property;
    /**
     * The direction in which the table is currently sorted.
     */
    private $order_direction;

    /**
     * **************************************************************************************************************
     * Main Functionality *
     * **************************************************************************************************************
     */

    /**
     * Constructor.
     */
    function __construct($object_table)
    {
        $this->object_table = $object_table;

        $this->initialize_properties();

        $this->set_default_order_property(static :: DEFAULT_ORDER_PROPERTY_INDEX);
        $this->set_default_order_direction(static :: DEFAULT_ORDER_PROPERTY_DIRECTION);
    }

    /**
     *
     * @return \common\libraries\NewObjectTable
     */
    function get_object_table()
    {
        return $this->object_table;
    }

    /**
     * Returns the component of the object table
     *
     * @param Application
     */
    function get_component()
    {
        return $this->get_object_table()->get_component();
    }

    /**
     * Returns an ObjectTableOrder with the given order column index. If the selected index is not sortable, the default
     * column index will be used. If the default column is not sortable then no order will be given
     *
     * @param $property_number int
     * @param $order_direction int
     *
     * @return \common\libraries\ObjectTableOrder null
     */
    function get_order_property($property_number, $order_direction)
    {
        $property = $this->get_property($property_number);

        // If it's an ObjectTableProperty, then return the property
        if ($property instanceof GalleryObjectTableProperty)
        {
            return new ObjectTableOrder($property->get_property(), $order_direction, $property->get_storage_unit_alias());
        }
        // If not, return the default order property
        else
        {
            $default_property = $this->get_property($this->get_default_order_property());

            // Make sure the default order property is actually an ObjectTableProperty AND sortabele
            if ($default_property instanceof GalleryObjectTableProperty)
            {
                return new ObjectTableOrder($default_property->get_property(), $order_direction, $default_property->get_storage_unit_alias());
            }
            // If not, just don't sort (probably a table with display orders)
            else
            {
                return null;
            }
        }
    }

    /**
     * **************************************************************************************************************
     * Getters and Setters *
     * **************************************************************************************************************
     */

    /**
     * Returns the properties
     *
     * @return multitype:\common\libraries\GalleryObjectTableProperty
     */
    function get_properties()
    {
        return $this->properties;
    }

    /**
     * Sets the properties
     *
     * @param $properties multitype:\common\libraries\GalleryObjectTableProperty
     */
    function set_properties($properties)
    {
        $this->properties = $properties;
    }

    /**
     * Returns the index of the default property to order objects by
     *
     * @return int
     */
    function get_default_order_property()
    {
        return $this->order_property;
    }

    /**
     * Sets the index of the default property to order objects by
     *
     * @param $column int
     */
    function set_default_order_property($property_index)
    {
        $this->order_property = $property_index;
    }

    /**
     * Gets the default order direction.
     *
     * @return int - The direction. Either the PHP constant SORT_ASC or SORT_DESC.
     */
    function get_default_order_direction()
    {
        return $this->order_direction;
    }

    /**
     * Sets the default order direction.
     *
     * @param $direction int - The direction. Either the PHP constant SORT_ASC or SORT_DESC.
     */
    function set_default_order_direction($direction)
    {
        $this->order_direction = $direction;
    }

    /**
     * **************************************************************************************************************
     * List Functionality *
     * **************************************************************************************************************
     */

    /**
     * Returns the number of properties in the model.
     *
     * @return int
     */
    function get_property_count()
    {
        return count($this->properties);
    }

    /**
     * Gets the property at the given index in the model.
     *
     * @param $index int
     *
     * @return \common\libraries\GalleryObjectTableProperty
     */
    function get_property($index)
    {
        return $this->properties[$index];
    }

    /**
     * Adds the given property at the end of the table.
     *
     * @param $property \common\libraries\GalleryObjectTableProperty
     */
    function add_property($property)
    {
        $this->properties[] = $property;
    }

    /**
     * Delete a property at a given index
     *
     * @param $property_index int
     */
    function delete_property($property_index)
    {
        unset($this->properties[$column_index]);

        $this->properties = array_values($this->properties);
    }

    /**
     * **************************************************************************************************************
     * Abstract Functionality *
     * **************************************************************************************************************
     */

    /**
     * Initializes the properties for the table
     */
    abstract function initialize_properties();
}
?>