<?php
namespace common\libraries;

use Exception;

/**
 *
 * @package common.libraries.storage
 * @author Hans De Bisschop
 */

abstract class ConditionTranslator
{
    private $data_manager;

    /**
     *
     * @var string
     */
    private $storage_unit;

    function __construct($data_manager, $storage_unit = null)
    {
        $this->data_manager = $data_manager;
        $this->storage_unit = $storage_unit;
    }

    function get_data_manager()
    {
        return $this->data_manager;
    }

    function set_data_manager($data_manager)
    {
        $this->data_manager = $data_manager;
    }

    /**
     *
     * @return string
     */
    function get_storage_unit()
    {
        return $this->storage_unit;
    }

    /**
     *
     * @param $storage_unit string
     */
    function set_storage_unit($storage_unit)
    {
        $this->storage_unit = $storage_unit;
    }

    function translate($condition)
    {
        if ($condition instanceof AggregateCondition)
        {
            $string = $this->translate_aggregate_condition($condition);
        }
        elseif ($condition instanceof InCondition)
        {
            $string = $this->translate_in_condition($condition);
        }
        elseif ($condition instanceof SubselectCondition)
        {
            $string = $this->translate_subselect_condition($condition);
        }
        elseif ($condition instanceof Condition)
        {
            $string = $this->translate_simple_condition($condition);
        }
        else
        {
            throw new Exception('Need a Condition instance');
        }

        return $string;
    }

    /**
     * Translates an aggregate condition to a SQL WHERE clause.
     *
     * @param $condition AggregateCondition The AggregateCondition object.
     * @return string The WHERE clause.
     */
    abstract function translate_aggregate_condition($aggregate_condition);

    /**
     * Translates an in condition to a SQL WHERE clause.
     *
     * @param $condition InCondition The InCondition object.
     * @return string The WHERE clause.
     */
    abstract function translate_in_condition($condition);

    abstract function translate_subselect_condition($condition);

    /**
     * Translates a simple condition to a SQL WHERE clause.
     *
     * @param $condition Condition The Condition object.
     * @return string The WHERE clause.
     */
    abstract function translate_simple_condition($condition);

    /**
     * Translates a string with wildcard characters "?" (single character) and "*" (any character sequence) to a SQL
     * pattern for use in a LIKE condition. Should be suitable for any SQL flavor.
     *
     * @param $string string The string that contains wildcard characters.
     * @return string The escaped string.
     */
    abstract function translate_search_string($string);

    abstract function render_query($condition, $add_where = true);

    /**
     *
     * @param $type string
     * @param $data_manager unknown_type
     * @param $storage_unit string
     * @return ConditionTranslator
     */
    static function factory($data_manager, $storage_unit = null)
    {
        $type = $data_manager->get_storage_type();

        $condition_translator_class = __NAMESPACE__ . '\\' . Utilities :: underscores_to_camelcase($type) . 'ConditionTranslator';

        return new $condition_translator_class($data_manager, $storage_unit);
    }

}
?>