<?php
namespace common\libraries;

use Exception;

/**
 * A class to allow caching of DataClass objects
 *
 * @author Hans De Bisschop
 * @package common.libraries
 */
class DataClassResultCache extends DataClassCache
{

    /**
     * Add a DataClass object to the cache
     *
     * @param $object \common\libraries\DataClass
     * @param $parameters \common\libraries\DataClassRetrieveParameters
     * @throws Exception
     * @return boolean
     */
    static function add(DataClass $object, DataClassRetrieveParameters $parameters = null)
    {
        if (! $object instanceof DataClass)
        {
            $type = is_object($object) ? get_class($object) : gettype($object);
            throw new Exception('The DataClass cache only allows for caching of DataClass objects. Currently trying to add: ' . $type . '.');
        }

        $class_name = ($object instanceof CompositeDataClass ? $object :: parent_class_name() : $object :: class_name());

        foreach ($object->get_cacheable_property_names() as $cacheable_property)
        {
            $value = $object->get_default_property($cacheable_property);
            if (isset($value) && ! is_null($value))
            {
                $cacheable_property_parameters = new DataClassRetrieveParameters(new EqualityCondition($cacheable_property, $value));
                self :: set_cache($class_name, $cacheable_property_parameters->hash(), $object);
            }
        }

        if ($parameters instanceof DataClassRetrieveParameters)
        {
            self :: set_cache($class_name, $parameters->hash(), $object);
        }

        return true;
    }

    /**
     * Process a DataClassNoResultException for the DataClassCache, preventing continued access to the storage layer for
     * empty results
     *
     * @param $exception DataClassNoResultException
     * @return boolean
     */
    static function no_result(DataClassNoResultException $exception)
    {
        self :: set_cache($exception->get_class_name(), $exception->get_parameters()->hash(), false);
        return true;
    }

    /**
     * Delete a specific DataClass object from the cache
     *
     * @param $object \common\libraries\DataClass
     * @throws Exception
     * @return boolean
     */
    static function delete(DataClass $object)
    {
        if (! $object instanceof DataClass)
        {
            throw new Exception('Not a DataClass');
        }

        $class_name = ($object instanceof CompositeDataClass ? $object :: parent_class_name() : $object :: class_name());

        foreach ($object->get_cacheable_property_names() as $cacheable_property)
        {
            $cacheable_property_parameters = new DataClassRetrieveParameters(new EqualityCondition($cacheable_property, $object->get_default_property($cacheable_property)));
            self :: set_cache($class_name, $cacheable_property_parameters->hash(), null);
        }

        return true;
    }
}
?>