<?php
namespace common\libraries;

abstract class CompositeDataClass extends DataClass
{
    const PROPERTY_TYPE = 'type';

    const PROPERTIES_ADDITIONAL = 'additional_properties';

    function __construct($default_properties = array(), $additional_properties = null)
    {
        parent :: __construct($default_properties);
        $this->set_additional_properties($additional_properties);
    }

    /**
     *
     * @param $class string
     * @param $record multitype:string
     * @throws Exception
     * @return \common\libraries\CompositeDataClass
     */
    static function factory($class, $record)
    {
        $object = parent :: factory($record[self :: PROPERTY_TYPE], $record);

        foreach ($object->get_additional_property_names() as $property)
        {
            if (array_key_exists($property, $record))
            {
                $object->set_additional_property($property, $record[$property]);
                unset($record[$property]);
            }
        }

        return $object;
    }

    /**
     * Gets the additional property names Should be overridden when needed
     *
     * @return array the aditional property names
     */
    static function get_additional_property_names()
    {
        return array();
    }

    /**
     * Gets an additional (type-specific) property of this object by name.
     *
     * @param $name string The name of the property.
     */
    function get_additional_property($name)
    {
        $this->check_for_additional_properties();
        return $this->get_specific_property(self :: PROPERTIES_ADDITIONAL, $name);
    }

    /**
     * Sets an additional (type-specific) property of this object by name.
     *
     * @param $name string The name of the property.
     * @param $value mixed The new value for the property.
     */
    function set_additional_property($name, $value)
    {
        $this->set_specific_property(self :: PROPERTIES_ADDITIONAL, $name, $value);
    }

    /**
     * Gets the additional (type-specific) properties of this object.
     *
     * @return array An associative array containing the properties.
     */
    function get_additional_properties()
    {
        $this->check_for_additional_properties();
        return $this->get_specific_properties(self :: PROPERTIES_ADDITIONAL);
    }

    /**
     * Sets the additional (type-specific) properties of this object.
     *
     * @param array An associative array containing the properties.
     */
    function set_additional_properties($additional_properties)
    {
        $this->set_specific_properties(self :: PROPERTIES_ADDITIONAL, $additional_properties);
    }

    static function is_extended()
    {
        return count(static :: get_additional_property_names()) > 0;
    }

    function get_type()
    {
        return $this->get_default_property(self :: PROPERTY_TYPE);
    }

    function set_type($type)
    {
        $this->set_default_property(self :: PROPERTY_TYPE, $type);
    }

    function check_for_additional_properties()
    {
        $additional_properties = $this->get_specific_properties(self :: PROPERTIES_ADDITIONAL);

        if (isset($additional_properties) && ! empty($additional_properties))
        {
            return;
        }

        $data_manager = Utilities :: get_namespace_from_object($this) . '\DataManager';

        $this->set_additional_properties($data_manager :: retrieve_composite_data_class_additional_properties($this));
    }

    /**
     * Get the fully qualified class name of the DataClass object
     *
     * @return string
     */
    static function parent_class_name()
    {
        return get_parent_class(static :: class_name());
    }

}
?>