<?php

namespace common\libraries;

use Exception;

/**
 * Abstract class that describes a dataclass
 *
 * @package common.libraries
 * @author Hans De Bisschop - Erasmus Hogeschool Brussel
 * @author Sven Vanpoucke - Hogeschool Gent
 */
abstract class DataClass
{

    const CLASS_NAME = __CLASS__;
    const PROPERTY_ID = 'id';
    const NO_UID = - 1;

    /**
     * Properties of the data class object, stored in an associative array. Combination of different types of
     * properties. Default properties => properties that are a mapping of dataclass and data table Optional properties
     * => other properties that were added in join queries Foreign properties => objects of other dataclasses
     */
    private $properties;

    /**
     * A list of errors that this dataclass has
     *
     * @var Array
     */
    private $errors;

    /**
     * **************************************************************************************************************
     * Property Types
     * **************************************************************************************************************
     */

    const PROPERTIES_DEFAULT = 'default_properties';
    const PROPERTIES_OPTIONAL = 'optional_properties';
    const PROPERTIES_FOREIGN = 'foreign_properties';

    /**
     * **************************************************************************************************************
     * Main functionality
     * **************************************************************************************************************
     */

    /**
     * Creates a new data class object.
     *
     * @param $default_properties array The default properties of the data class object. Associative array.
     * @param $optional_properties array The optional properties of the data class object. Associative array.
     */
    function __construct($default_properties = array(), $optional_properties = array())
    {
        $this->set_default_properties($default_properties);
        $this->set_optional_properties($optional_properties);
    }

    /**
     *
     * @param $class string
     * @param $record multitype:string
     * @throws Exception
     * @return \common\libraries\DataClass
     */
    static function factory($class, $record)
    {
        if (! is_array($record) || ! count($record))
        {
            throw new Exception(Translation :: get('InvalidDataRetrievedFromStorageLayer'));
        }

        $object = new $class();

        foreach ($object->get_default_property_names() as $property)
        {
            if (array_key_exists($property, $record))
            {
                $object->set_default_property($property, $record[$property]);
                unset($record[$property]);
            }
        }

        if (count($record) > 0 && $object instanceof DataClass)
        {
            foreach ($record as $optional_property_name => $optional_property_value)
            {
                $object->set_optional_property($optional_property_name, $optional_property_value);
            }
        }

        return $object;
    }

    /**
     * *************************************************************************************************************
     * Properties functionality
     * *************************************************************************************************************
     */

    /**
     * Retrieves all the properties
     *
     * @return Array
     */
    function get_properties()
    {
        return $this->properties;
    }

    /**
     * Sets all the properties
     *
     * @param $properties Array
     */
    function set_properties($properties)
    {
        $this->properties = $properties;
    }

    /**
     * *************************************************************************************************************
     * Specific properties functionality
     * *************************************************************************************************************
     */

    /**
     * Sets the properties for a specific type
     *
     * @param $properties_type string
     * @param $properties Array
     */
    function set_specific_properties($properties_type, $properties)
    {
        $this->properties[$properties_type] = $properties;
    }

    /**
     * Returns the properties for a specific type
     *
     * @param $properties_type string
     *
     * @return Array
     */
    function get_specific_properties($properties_type)
    {
        return $this->properties[$properties_type];
    }

    /**
     * Get a property from a property type
     *
     * @param $properties_type string
     * @param $property_name string
     *
     * @return string
     */
    function get_specific_property($properties_type, $property_name)
    {
        $properties = $this->get_specific_properties($properties_type);

        return (isset($properties) && array_key_exists($property_name, $properties)) ? $properties[$property_name] : null;
    }

    /**
     * Sets a property for a specific property type
     *
     * @param $properties_type string
     * @param $property_name string
     * @param $property_value string
     */
    function set_specific_property($properties_type, $property_name, $property_value)
    {
        $this->properties[$properties_type][$property_name] = $property_value;
    }

    /**
     * *************************************************************************************************************
     * Default properties functionality
     * *************************************************************************************************************
     */

    /**
     * Gets a default property of this data class object by name.
     *
     * @param $name string The name of the property.
     */
    function get_default_property($name)
    {
        return $this->get_specific_property(self :: PROPERTIES_DEFAULT, $name);
    }

    /**
     * Sets a default property of this data class by name.
     *
     * @param $name string The name of the property.
     * @param $value mixed The new value for the property.
     */
    function set_default_property($name, $value)
    {
        $this->set_specific_property(self :: PROPERTIES_DEFAULT, $name, $value);
    }

    /**
     * Gets the default properties of this data class.
     *
     * @return array An associative array containing the properties.
     */
    function get_default_properties()
    {
        return $this->get_specific_properties(self :: PROPERTIES_DEFAULT);
    }

    /**
     * Sets the default properties of this dataclass
     *
     * @param $default_properties array
     */
    function set_default_properties($default_properties)
    {
        $this->set_specific_properties(self :: PROPERTIES_DEFAULT, $default_properties);
    }

    /**
     * Get the default properties of all data classes.
     *
     * @return array The property names.
     */
    static function get_default_property_names($extended_property_names = array())
    {
        $extended_property_names[] = self :: PROPERTY_ID;
        return $extended_property_names;
    }

    /**
     * Checks if the given identifier is the name of a default data class property.
     *
     * @param $name string The identifier.
     * @return boolean True if the identifier is a property name, false otherwise.
     */
    static function is_default_property_name($name)
    {
        return in_array($name, self :: get_default_property_names());
    }

    /**
     * **************************************************************************************************************
     * Optional properties functionality
     * *************************************************************************************************************
     */

    /**
     * Gets the optional properties of this data class.
     *
     * @return array An associative array containing the properties.
     */
    function get_optional_properties()
    {
        return $this->get_specific_properties(self :: PROPERTIES_OPTIONAL);
    }

    /**
     * Sets the optional properties of this dataclass
     *
     * @param $optional_properties Array
     */
    function set_optional_properties($optional_properties)
    {
        $this->set_specific_properties(self :: PROPERTIES_OPTIONAL, $optional_properties);
    }

    /**
     * Gets a optional property of this data class object by name.
     *
     * @param $name string The name of the property.
     */
    function get_optional_property($name)
    {
        return $this->get_specific_property(self :: PROPERTIES_OPTIONAL, $name);
    }

    /**
     * Sets a optional property of this data class by name.
     *
     * @param $name string The name of the property.
     * @param $value mixed The new value for the property.
     */
    function set_optional_property($name, $value)
    {
        $this->set_specific_property(self :: PROPERTIES_OPTIONAL, $name, $value);
    }

    /**
     * *************************************************************************************************************
     * Foreign properties functionality
     * *************************************************************************************************************
     */

    /**
     * Gets the foreign properties of this data class.
     *
     * @return array An associative array containing the properties.
     */
    function get_foreign_properties()
    {
        return $this->get_specific_properties(self :: PROPERTIES_FOREIGN);
    }

    /**
     * Sets the foreign properties of this dataclass
     *
     * @param $foreign_properties Array
     */
    function set_foreign_properties($foreign_properties)
    {
        $this->set_specific_properties(self :: PROPERTIES_FOREIGN, $foreign_properties);
    }

    /**
     * Gets a foreign property of this data class object by name and retrieves it with lazy loading if the property does
     * not yet exists in this dataclass
     *
     * @param $name string The name of the property.
     * @param $datamanager DataManager - [OPTIONAL] The datamanager, default retrieved via get_data_manager()
     */
    function get_foreign_property($name, $datamanager = null)
    {
        $foreign_property = $this->get_specific_property(self :: PROPERTIES_FOREIGN, $name);
        if (is_null($foreign_property))
        {
            $retrieve_function = 'retrieve_' . $name;
            $foreign_property_id = $this->get_default_property($name . '_id');

            if (is_null($datamanager))
            {
                $datamanager = $this->get_data_manager();
            }

            $foreign_property = call_user_func(array($datamanager, $retrieve_function), $foreign_property_id);

            $this->set_foreign_property($name, $foreign_property);
        }

        return $foreign_property;
    }

    /**
     * Sets a foreign property of this data class by name and pushes the id of this
     *
     * @param $name string The name of the property.
     * @param $value mixed The new value for the property.
     */
    function set_foreign_property($name, $value)
    {
        if (is_null($value) || ! $value instanceof DataClass)
        {
            throw new \Exception(Translation :: get('ForeignObjectPropertyCanNotBeNull',
		    array('OBJECT' => $this->class_name(), 'FOREIGN_OBJECT' => $name)));
        }

        $this->set_specific_property(self :: PROPERTIES_FOREIGN, $name, $value);
        $this->set_default_property($name . '_id', $value->get_id());
    }

    /**
     * **************************************************************************************************************
     * Properties Metadata Functionality
     * **************************************************************************************************************
     */

    /**
     * Returns all (unique) properties by which a DataClass object can be cached
     *
     * @param $extended_property_names multitype:string
     * @return multitype:string
     */
    static function get_cacheable_property_names($extended_property_names = array())
    {
        $extended_property_names[] = self :: PROPERTY_ID;
        return $extended_property_names;
    }

    /**
     * *************************************************************************************************************
     * Getters and setters
     * *************************************************************************************************************
     */

    /**
     * Returns the id of this object
     *
     * @return int The id.
     */
    function get_id()
    {
        return $this->get_default_property(self :: PROPERTY_ID);
    }

    /**
     * Sets id of the object
     *
     * @param $id int
     */
    function set_id($id)
    {
        if (isset($id) && strlen($id) > 0)
        {
            $this->set_default_property(self :: PROPERTY_ID, $id);
        }
    }

    /**
     * *************************************************************************************************************
     * CRUD functionality
     * *************************************************************************************************************
     */

    /**
     * Saves the object
     *
     * @return boolean
     */
    function save()
    {
        if ($this->is_identified())
        {
            return $this->update();
        }
        else
        {
            return $this->create();
        }
    }

    /**
     * Check if the data class has an id or not (and therefore exists in the database)
     *
     * @return boolean
     */
    function is_identified()
    {
        $id = $this->get_id();
        return isset($id) && strlen($id) > 0 && $id != self :: NO_UID;
    }

    /**
     * Check wether the object contains all mandatory properties to be saved in datasource This method should be
     * overriden in classes inheriting from DataClass
     *
     * @return boolean Return true if the object can be saved, false otherwise
     */
    protected function check_before_save()
    {
        /*
         * Example: object with mandatory title if(stringUtilities :: is_null_or_empty($this->get_title())) {
         * $this->add_error(Translation :: get('TitleIsRequired')); }
         */

        return ! $this->has_errors();
    }

    /**
     * Creates the object
     *
     * @return boolean
     */
    function create()
    {
        if ($this->check_before_save())
        {
            // Using the generalized method to call a DataManager instance
            $namespace = Utilities :: get_namespace_from_object($this);
            $class_name = $namespace . '\DataManager';

            while (! class_exists($class_name) && strlen($namespace) > 0)
            {
                $namespace = Utilities :: get_namespace_parent($namespace);
                $class_name = $namespace . '\DataManager';
            }

            if (strlen($namespace) > 0)
            {
                return $class_name :: create($this);
            }

            // Code for backwards compatibility
            $data_manager = $this->get_data_manager();
            $class_name = $this->get_object_name();

            $method = 'create_' . $class_name;

            if (method_exists($data_manager, $method))
            {
                return $data_manager->$method($this);
            }
            else
            {
                return $data_manager->create($this);
            }
        }
        return false;
    }

    /**
     * Updates the object
     *
     * @return boolean
     */
    function update()
    {
        if ($this->check_before_save())
        {
            // Using the generalized method to call a DataManager instance
            $namespace = Utilities :: get_namespace_from_object($this);
            $class_name = $namespace . '\DataManager';

            while (! class_exists($class_name) && strlen($namespace) > 0)
            {
                $namespace = Utilities :: get_namespace_parent($namespace);
                $class_name = $namespace . '\DataManager';
            }

            if (strlen($namespace) > 0 && method_exists($class_name, 'update'))
            {
                return $class_name :: update($this);
            }

            // Code for backwards compatibility
            $data_manager = $this->get_data_manager();
            $class_name = $this->get_object_name();

            $method = 'update_' . $class_name;

            if (method_exists($data_manager, $method))
            {
                return $data_manager->$method($this);
            }
            else
            {
                $condition = new EqualityCondition(self :: PROPERTY_ID, $this->get_id());
                return $data_manager->update($this, $condition);
            }
        }
        return false;
    }

    /**
     * Deletes the object
     *
     * @return boolean
     */
    function delete()
    {
        // Using the generalized method to call a DataManager instance
        $namespace = Utilities :: get_namespace_from_object($this);
        $class_name = $namespace . '\DataManager';

        while (! class_exists($class_name) && strlen($namespace) > 0)
        {
            $namespace = Utilities :: get_namespace_parent($namespace);
            $class_name = $namespace . '\DataManager';
        }

        if (strlen($namespace) > 0)
        {
            return $class_name :: delete($this);
        }

        // Code for backwards compatibility
        $data_manager = $this->get_data_manager();
        $class_name = $this->get_object_name();

        $method = 'delete_' . $class_name;

        if (method_exists($data_manager, $method))
        {
            return $data_manager->$method($this);
        }
        else
        {
            $condition = new EqualityCondition(self :: PROPERTY_ID, $this->get_id());
            return $data_manager->delete($this->get_table_name(), $condition);
        }
    }

    /**
     * *************************************************************************************************************
     * Error handling
     * *************************************************************************************************************
     */

    /**
     * Adds an error to the error list
     *
     * @param $error_msg string
     */
    public function add_error($error_msg)
    {
        if (! isset($this->errors))
        {
            $this->errors = array();
        }

        $this->errors[] = $error_msg;
    }

    /**
     * Checks wether the object has errors
     *
     * @return booleans
     */
    public function has_errors()
    {
        return isset($this->errors) && count($this->errors) > 0;
    }

    /**
     * Retrieves the list of errors
     *
     * @return Array
     */
    public function get_errors()
    {
        return isset($this->errors) ? $this->errors : array();
    }

    /**
     * Clears the errors
     */
    public function clear_errors()
    {
        unset($this->errors);
    }

    /**
     * *************************************************************************************************************
     * Helper functionality
     * *************************************************************************************************************
     */

    /**
     * Returns the name of the object
     *
     * @return string
     * @deprecated Only used for legacy calls to CUD method implementations, should no longer be necessary
     */
    function get_object_name()
    {
        return Utilities :: get_classname_from_object($this, true);
    }

    /**
     * *************************************************************************************************************
     * Static functionality
     * *************************************************************************************************************
     */

    /**
     * Returns whether or not this dataclass is an extended type
     *
     * @return boolean
     */
    static function is_extended_type()
    {
        return false;
    }

    /**
     * Returns the table name for this dataclass
     *
     * @return string
     */
    static function get_table_name()
    {
        return Utilities :: get_classname_from_namespace(get_called_class(), true);
    }

    /**
     * Get the fully qualified class name of the DataClass object
     *
     * @return string
     */
    static function class_name()
    {
        return get_called_class();
    }

    /**
     * Get the namespace of the DataClass object
     *
     * @return string
     */
    static function context()
    {
        return Utilities :: get_namespace_from_classname(get_called_class());
    }

    /**
     * *************************************************************************************************************
     * Abstract functionality
     * *************************************************************************************************************
     */

    /**
     * Returns the data manager
     * @deprecated DataManager class names can now be automatically built from the namespace / context of the object
     */
    abstract function get_data_manager();
}

?>