<?php
namespace common\libraries;

use Exception;

/**
 * General and basic DataManager, providing basic functionality for all other DataManager objects
 *
 * @author Hans De Bisschop
 * @package common.libraries
 */
class DataManager
{
    /**
     * Instance of this class for the singleton pattern
     *
     * @var \common\libraries\DataManager
     */
    static $instance;

    /**
     * Uses a singleton pattern and a factory pattern to return the data manager. The configuration determines which
     * data manager class is to be instantiated
     *
     * @return \common\libraries\DataManager
     */
    static function get_instance()
    {
        if (! isset(self :: $instance[static :: context()]))
        {
            $type = Utilities :: underscores_to_camelcase(static :: get_type());
            $class = static :: context() . '\\' . $type . 'DataManager';
            self :: $instance[static :: context()] = new $class();
        }
        return self :: $instance[static :: context()];
    }

    /**
     * Set the instance of the DataManager
     *
     * @param $instance \common\libraries\DataManager
     */
    static function set_instance($instance)
    {
        self :: $instance[Utilities :: get_namespace_from_object($instance)] = $instance;
    }

    /**
     * Gets the type of DataManager to be instantiated, by default configured in the main Chamilo configuration file
     *
     * @return string
     */
    static function get_type()
    {
        return Configuration :: get_instance()->get_parameter('general', 'data_manager');
    }

    /**
     * Magic method to handle calls to non-existing methods and providing *some* backward compatibility
     *
     * @param $name string
     * @param $arguments multitype:mixed
     * @throws Exception
     * @return boolean
     */
    function __callStatic($name, $arguments)
    {
        $name_parts = explode('_', $name, 2);
        $class = static :: context() . '\\' . Utilities :: underscores_to_camelcase($name_parts[1]);

        switch ($name_parts[0])
        {
            case 'retrieve' :
                return static :: retrieve_by_id($class, $arguments[0]);
                break;
            default :
                throw new Exception('Your call to the method <i>' . $name . '</i> in <i>' . $class . 'DataManager</i> could not be resolved.');
        }
    }

    /**
     * Get the fully qualified class name of the DataManager object
     *
     * @return string
     */
    static function class_name()
    {
        return get_called_class();
    }

    /**
     * Get the namespace of the DataManager object
     *
     * @return string
     */
    static function context()
    {
        return Utilities :: get_namespace_from_classname(get_called_class());
    }

    /**
     * Write an instance of a DataClass object to the storage layer
     *
     * @param $object \common\libraries\DataClass
     * @return boolean
     */
    static function create(DataClass $object)
    {
        if (! self :: get_instance()->create($object))
        {
            return false;
        }
        return DataClassResultCache :: add($object);
    }

    /**
     * Retrieve an instance of a DataClass object from the storage layer by a set of parameters
     *
     * @param $class string
     * @param $parameters \common\libraries\DataClassRetrieveParameters
     * @return \common\libraries\DataClass
     */
    static function retrieve($class, $parameters = null)
    {
        if (! $parameters instanceof DataClassRetrieveParameters)
        {
            $parameters = DataClassRetrieveParameters :: generate($parameters);
        }

        if (! DataClassCache :: exists($class, $parameters))
        {
            try
            {
                $base = (is_subclass_of($class, CompositeDataClass :: class_name()) ? CompositeDataClass :: class_name() : DataClass :: class_name());
                DataClassResultCache :: add($base :: factory($class, self :: get_instance()->retrieve($class, $parameters)), $parameters);
            }
            catch (DataClassNoResultException $exception)
            {
                DataClassResultCache :: no_result($exception);
            }
        }
        return DataClassCache :: get($class, $parameters);
    }

    /**
     * Retrieve an instance of a DataClass object from the storage layer by it's unique identifier
     *
     * @param $class string
     * @param $id int
     * @return \common\libraries\DataClass
     */
    static function retrieve_by_id($class, $id)
    {
        return self :: retrieve($class, new DataClassRetrieveParameters(new EqualityCondition($class :: PROPERTY_ID, $id)));
    }

    /**
     * Retrieve a ResultSet of DataClass object instances from the storage layer
     *
     * @param $class string
     * @param $parameters \common\libraries\DataClassRetrievesParameters
     * @return \common\libraries\ResultSet
     */
    static function retrieves($class, $parameters)
    {
        if (! $parameters instanceof DataClassRetrievesParameters)
        {
            $parameters = DataClassRetrievesParameters :: generate($parameters);
        }

        if (! DataClassResultSetCache :: exists($class, $parameters))
        {
            DataClassResultSetCache :: add(self :: get_instance()->retrieves($class, $parameters), $parameters);
        }

        $result_set = DataClassResultSetCache :: get($class, $parameters);
        $result_set->reset();
        return $result_set;
    }

    /**
     * Retrieve all distinct values of a specific DataClass' property from the storage layer
     *
     * @param $class string
     * @param $property string
     * @param $condition \common\libraries\Condition
     * @return multitype:string
     */
    static function distinct($class, $property, Condition $condition = null)
    {
        return self :: get_instance()->distinct($class, $property, $condition);
    }

    /**
     * Update an instance of a DataClass object in the storage layer
     *
     * @param $object \common\libraries\DataClass
     * @return boolean
     */
    static function update(DataClass $object)
    {
        $condition = new EqualityCondition($object :: PROPERTY_ID, $object->get_id());
        return self :: get_instance()->update($object, $condition);
    }

    /**
     * Updates any given number of properties for a specific DataClass property in the storage layer
     *
     * @param $class string
     * @param $properties string
     * @param $condition \common\libraries\Condition
     * @param $offset int
     * @param $count int
     * @param $order_by multitype:\common\libraries\ObjectTableOrder
     * @return boolean
     */
    static function updates($class, array $properties, Condition $condition, $offset = null, $count = null, $order_by = array())
    {
        if (! self :: get_instance()->updates($class, $properties, $condition, $offset, $count, $order_by))
        {
            return false;
        }

        return DataClassCache :: truncate($class);
    }

    /**
     * Delete an instance of a DataClass object from the storage layer
     *
     * @param $object \common\libraries\DataClass
     * @return boolean
     */
    static function delete(DataClass $object)
    {
        $class_name = ($object instanceof CompositeDataClass ? $object :: parent_class_name() : $object :: class_name());

        $condition = new EqualityCondition($class_name :: PROPERTY_ID, $object->get_id());
        if (! self :: get_instance()->new_delete($class_name, $condition))
        {
            return false;
        }

        if ($object instanceof CompositeDataClass && $object :: is_extended())
        {
            $condition = new EqualityCondition($object :: PROPERTY_ID, $object->get_id());
            if (! self :: get_instance()->new_delete($object :: class_name(), $condition))
            {
                return false;
            }
        }

        return DataClassResultCache :: delete($object);
    }

    /**
     * Deletes any given number of instance of the DataClass object from the storage layer
     *
     * @param $class string
     * @param $condition \common\libraries\Condition
     * @return boolean
     */
    static function deletes($class, Condition $condition)
    {
        if (! self :: get_instance()->deletes($class, $condition))
        {
            return false;
        }
        return DataClassCache :: truncate($class);
    }

    /**
     * Count the number of instances of a DataClass object in the storage layer
     *
     * @param $class string
     * @param $parameters \common\libraries\DataClassCountParameters
     * @return int
     */
    static function count($class, $parameters = null)
    {
        if (! $parameters instanceof DataClassCountParameters)
        {
            $parameters = DataClassCountParameters :: generate($parameters);
        }

        if (! DataClassCountCache :: exists($class, $parameters))
        {
            DataClassCountCache :: add($class, $parameters, self :: get_instance()->count($class, $parameters));
        }
        return DataClassCountCache :: get($class, $parameters);
    }

    /**
     * Count the number of instances of a DataClass object in the storage layer, based on a specific property ad grouped
     * by another property in the storage layer
     *
     * @param $class string
     * @param $parameters \common\libraries\DataClassCountGroupedParameters
     * @return multitype:int
     */
    static function count_grouped($class, $parameters)
    {
        if (! $parameters instanceof DataClassCountGroupedParameters)
        {
            $parameters = DataClassCountGroupedParameters :: generate($parameters);
        }

        if (! DataClassCountGroupedCache :: exists($class, $parameters))
        {
            DataClassCountGroupedCache :: add($class, $parameters, self :: get_instance()->count_grouped($class, $parameters));
        }
        return DataClassCountGroupedCache :: get($class, $parameters);
    }

    /**
     * Count the number of distinct values of a specific property of the DataClass in the storage layer
     *
     * @param $class string
     * @param $parameters \common\libraries\DataClassCountDistinctParameters
     * @return int
     */
    static function count_distinct($class, $parameters)
    {
        if (! $parameters instanceof DataClassCountDistinctParameters)
        {
            $parameters = DataClassCountDistinctParameters :: generate($parameters);
        }

        if (! DataClassCountCache :: exists($class, $parameters))
        {
            DataClassCountCache :: add($class, $parameters, self :: get_instance()->new_count_distinct($class, $parameters));
        }
        return DataClassCountCache :: get($class, $parameters);
    }

    /**
     * Get the highest value of a specific property of a DataClass in the storage layer
     *
     * @param $class string
     * @param $property string
     * @param $condition \common\libraries\Condition
     * @return int
     */
    static function retrieve_maximum_value($class, $property, Condition $condition = null)
    {
        return self :: get_instance()->retrieve_maximum_value($class, $property, $condition);
    }

    /**
     * Return the next value - based on the highest value - of a specific property of a DataClass in the storage layer
     *
     * @param $class string
     * @param $property string
     * @param $condition \common\libraries\Condition
     * @return int
     */
    static function retrieve_next_value($class, $property, Condition $condition = null)
    {
        return static :: retrieve_maximum_value($class, $property, $condition) + 1;
    }

    /**
     * Create a storage unit in the storage layer
     *
     * @param $name string
     * @param $properties multitype:mixed
     * @param $indexes multitype:mixed
     * @return boolean
     */
    static function create_storage_unit($name, $properties, $indexes)
    {
        return self :: get_instance()->create_storage_unit($name, $properties, $indexes);
    }

    /**
     * Determine whether a storage unit exists in the storage layer
     *
     * @param $name string
     * @return boolean
     */
    static function storage_unit_exists($name)
    {
        return self :: get_instance()->storage_unit_exists($name);
    }

    /**
     * Drop a storage unit from the storage layer
     *
     * @param $name string
     * @return boolean
     */
    static function drop_storage_unit($name)
    {
        return self :: get_instance()->drop_storage_unit($name);
    }

    /**
     * Truncate a storage unit in the storage layer and optionally optimize it afterwards
     *
     * @param $name string
     * @param $optimize boolean
     * @return boolean
     */
    static function truncate_storage_unit($name, $optimize = true)
    {
        if (! self :: get_instance()->truncate_storage_unit($name))
        {
            return false;
        }

        if ($optimize && ! static :: optimize_storage_unit($name))
        {
            return false;
        }

        return true;
    }

    /**
     * Optimize a storage unit in the storage layer
     *
     * @param $name string
     * @return boolean
     */
    static function optimize_storage_unit($name)
    {
        return self :: get_instance()->optimize_storage_unit($name);
    }

    /**
     * Get the alias of a storage unit in the storage layer
     *
     * @param $storage_unit_name string
     * @return string
     */
    static function get_alias($storage_unit_name)
    {
        return self :: get_instance()->get_alias($storage_unit_name);
    }

    /**
     * Retrieve the additional properties for a specific CompositeDataClass instance
     *
     * @param $object \common\libraries\CompositeDataClass
     * @return multitype:string
     */
    static function retrieve_composite_data_class_additional_properties(CompositeDataClass $object)
    {
        return self :: get_instance()->retrieve_composite_data_class_additional_properties($object);
    }
}
?>