<?php
namespace common\libraries;

use Exception;
use MDB2;

/**
 * $Id: database.class.php 229 2009-11-16 09:02:34Z scaramanga $
 *
 * @package common.database
 */

/**
 * This class provides basic functionality for database connections Create Table, Get next id, Insert, Update, Delete,
 * Select(with use of conditions), Count(with use of conditions)
 *
 * @author Sven Vanpoucke
 */
class DoctrineDatabase
{
    const STORAGE_TYPE = 'doctrine';
    const ALIAS_MAX_SORT = 'max_sort';

    private $connection;

    private $prefix;

    private $aliases;

    /**
     * Static error log so we don't open the error log every time an error is written
     *
     * @var FileLogger
     */
    private static $error_log;

    /**
     * Used for debug
     *
     * @var int
     */
    private static $query_counter;

    function __call($name, $arguments)
    {
        $class = static :: context() . '\\' . Utilities :: get_package_name_from_namespace(static :: context(), true) . 'DataManager';
        return call_user_func_array($class . '::' . $name, $arguments);
    }

    /**
     * Constructor
     */
    function __construct($aliases = array())
    {
        $this->aliases = $aliases;
        $this->initialize();
    }

    /**
     * Initialiser, creates the connection and sets the database to UTF8
     */
    function initialize()
    {
        $this->connection = DoctrineConnection :: get_instance()->get_connection();
        $this->connection->setCharset('utf8');

        $package = Utilities :: get_package_name_from_namespace(static :: context());
        $this->set_prefix($package . '_');
    }

    function set_aliases($aliases = array())
    {
        $this->aliases = $aliases;
    }

    function get_aliases()
    {
        return $this->aliases;
    }

    function get_storage_type()
    {
        return self :: STORAGE_TYPE;
    }

    /**
     * Returns the prefix
     *
     * @return String the prefix
     */
    function get_prefix()
    {
        return $this->prefix;
    }

    /**
     * Sets the prefix
     *
     * @param $prefix String
     */
    function set_prefix($prefix)
    {
        $this->prefix = $prefix;
    }

    /**
     * Returns the connection
     *
     * @return Doctrine\DBAL\Connection
     */
    function get_connection()
    {
        return $this->connection;
    }

    /**
     * Sets the connection
     *
     * @param $connection Doctrine\DBAL\Connection
     */
    function set_connection($connection)
    {
        $this->connection = $connection;
    }

    /**
     * Debug function Uncomment the lines if you want to debug
     */
    static function debug()
    {
        $args = func_get_args();
        // Do something with the arguments
        if ($args[1] == 'query' || $args[1] == 'prepare')
        {
            // echo '<pre>';
            // echo($args[2]);
            // echo self :: $query_counter;
            // echo '</pre>';
            // self :: $query_counter++;
        }
    }

    /*
     * A workaround to the fact that MDB2 hardcodes a default PEAR_Error error mode of PEAR_ERROR_RETURN
     */
    function error_handling($error)
    {
        // I don't care about performance, this should not happen more than once per request
        if (! self :: $error_log)
        {
            $logfile = Path :: get_log_path() . '/doctrine_errors.log';
            self :: $error_log = new FileLogger($logfile, true);
        }

        $message = "[Message: {$error->getMessage()}] [Information: { USER INFO GOES HERE}]";
        self :: $error_log->log_message($message);

        // throw new Exception("[Message: ".$err->getMessage()."] [Information: ".$err->getUserInfo()."]",
        // $err->getCode());
    }

    /**
     * Escapes a column name in accordance with the database type.
     *
     * @param $name string The column name.
     * @param $table_alias String The alias of the table the coloumn is in
     * @return string The escaped column name.
     */
    function escape_column_name($name, $table_alias = null)
    {
        $quoted_name = $this->connection->quoteIdentifier($name);
        if (! is_null($table_alias))
        {
            return $this->connection->quoteIdentifier($table_alias) . '.' . $quoted_name;
        }
        else
        {
            return $quoted_name;
        }
    }

    /**
     * Expands a table identifier to the real table name. Currently, this method prefixes the given table name with the
     * user-defined prefix, if any.
     *
     * @param $name string The table identifier.
     * @return string The actual table name.
     */
    function get_table_name($name)
    {
        return $this->prefix . $name;
    }

    /**
     * Escapes a table name in accordance with the database type.
     *
     * @param $name string The table identifier as provided by the data class.
     * @return string The escaped table name INCLUDING the application prefix.
     */
    function escape_table_name($name)
    {
        return $this->connection->quoteIdentifier($this->get_table_name($name));
    }

    /**
     * Maps a record to an object
     *
     * @param $record Record a record from the database
     * @param $class String Class to create new object
     * @return new object from type Class
     * @deprecated Use DataClass :: factory()
     */
    function record_to_object($record, $class_name)
    {
        return DataClass :: factory($class_name, $record);
    }

    function storage_unit_exist($name)
    {
        return $this->get_connection()->getSchemaManager()->tablesExist(array($this->get_table_name($name)));
    }

    function database_exists($name)
    {
        return in_array($name, $this->get_connection()->getSchemaManager()->listDatabases());
    }

    /**
     * Creates a storage unit in the system
     *
     * @param $name String the table name
     * @param $properties Array the table properties
     * @param $indexes Array the table indexes
     * @return true if the storage unit is succesfully created
     */
    function create_storage_unit($name, $properties, $indexes)
    {
        $table_name = $this->get_table_name($name);

        if ($this->get_connection()->getSchemaManager()->tablesExist(array($table_name)))
        {
            $this->drop_storage_unit($table_name);
        }

        $schema = new \Doctrine\DBAL\Schema\Schema();
        $table = $schema->createTable($table_name);

        foreach ($properties as $property => $attributes)
        {
            switch ($attributes['type'])
            {
                case 'text' :
                    if ($attributes['length'] && $attributes['length'] <= 255)
                    {
                        $attributes['type'] = 'string';
                    }
                    break;
            }

            $options = self :: parse_storage_unit_attributes($attributes);
            $table->addColumn($property, $attributes['type'], $options);

            if ($options['autoincrement'] == true)
            {
                $name = StorageAliasGenerator :: get_instance()->get_constraint_name($table_name, $name,
                        StorageAliasGenerator :: TYPE_CONSTRAINT);
                $table->setPrimaryKey(array($property), $name);
            }
        }

        foreach ($indexes as $index => $attributes)
        {
            $name = StorageAliasGenerator :: get_instance()->get_constraint_name($table_name, $index,
                    StorageAliasGenerator :: TYPE_CONSTRAINT);

            switch ($attributes['type'])
            {
                case 'primary' :
                    $table->setPrimaryKey(array_keys($attributes['fields']), $name);
                    break;
                case 'unique' :
                    $table->addUniqueIndex(array_keys($attributes['fields']), $name);
                    break;
                default :
                    $table->addIndex(array_keys($attributes['fields']), $name);
                    break;
            }
        }

        foreach ($schema->toSql($this->get_connection()->getDatabasePlatform()) as $query)
        {
            $statement = $this->query($query);

            if ($statement instanceof \PDOException)
            {
                $this->error_handling($statement);
                return false;
            }
        }

        return true;
    }

    static function parse_storage_unit_attributes($attributes)
    {
        $options = array();

        foreach ($attributes as $attribute => $value)
        {
            switch ($attribute)
            {
                case 'length' :
                    $options[$attribute] = (is_numeric($value) ? (int) $value : null);
                    break;
                case 'unsigned' :
                    $options[$attribute] = ($value == 1 ? true : false);
                    break;
                case 'fixed' :
                    if ($attributes['type'] != 'string')
                    {
                        $options[$attribute] = ($value == 1 ? true : false);
                    }
                    break;
                case 'notnull' :
                    $options[$attribute] = ($value == 1 ? true : false);
                    break;
                case 'default' :
                    $options[$attribute] = (! is_numeric($value) && empty($value) ? null : $value);
                    break;
                case 'autoincrement' :
                    $options[$attribute] = ($value == 'true' ? true : false);
                    break;
            }
        }

        if (! isset($options['notnull']))
        {
            $options['notnull'] = false;
        }

        return $options;
    }

    /**
     * Retrieves the next id for a given table
     *
     * @param $table_name String
     * @return Int the id
     */
    function get_next_id($table_name)
    {
        $id = $this->connection->nextID($this->get_table_name($table_name));
        return $id;
    }

    function get_better_next_id($table_name, $column)
    {
        return null;
    }

    /**
     *
     * @return True if creation is successfull or false
     */
    function create($object, $auto_id = true)
    {
        if ($object instanceof CompositeDataClass)
        {
            $parent_class = $object :: parent_class_name();
            $object_table = $parent_class :: get_table_name();
        }
        else
        {
            $object_table = $object->get_table_name();
        }

        $props = array();
        foreach ($object->get_default_properties() as $key => $value)
        {
            $props[$this->escape_column_name($key)] = $value;
        }

        if ($auto_id && in_array('id', $object->get_default_property_names()))
        {
            $props[$this->escape_column_name('id')] = $this->get_better_next_id($object_table, 'id');
        }

        try
        {
            $result = $this->connection->insert($this->get_table_name($object_table), $props);
            if ($auto_id && in_array('id', $object->get_default_property_names()))
            {
                $object->set_id($this->connection->lastInsertId($this->get_table_name($object_table)));
            }

            if ($object instanceof CompositeDataClass && $object :: is_extended())
            {
                $props = array();
                foreach ($object->get_additional_properties() as $key => $value)
                {
                    $props[$this->escape_column_name($key)] = $value;
                }
                $props[$this->escape_column_name('id')] = $object->get_id();

                try
                {
                    $result = $this->connection->insert($this->get_table_name($object->get_table_name()), $props);
                }
                catch (\Exception $exception)
                {
                    var_dump($exception);
                    $this->error_handling($exception);
                    return false;
                }
            }

            return true;
        }
        catch (\Exception $exception)
        {
            var_dump($exception);
            $this->error_handling($exception);
            return false;
        }
    }

    /**
     * Update functionality (can only be used when table has an ID)
     *
     * @param $object Object the object that has to be updated
     * @param $table_name String the table name
     * @param $condition Condition The condition for the item that has to be updated
     * @return True if update is successfull
     */
    function update($object, $condition = false)
    {
        if ($object instanceof CompositeDataClass)
        {
            $parent_class = $object :: parent_class_name();
            $object_table = $parent_class :: get_table_name();
        }
        else
        {
            $object_table = $object->get_table_name();
        }

        $query_builder = $this->connection->createQueryBuilder();
        $query_builder->update($this->get_table_name($object_table));

        foreach ($object->get_default_properties() as $key => $value)
        {
            $query_builder->set($this->escape_column_name($key), $this->escape($value));
        }

        if ($condition)
        {
            $translator = ConditionTranslator :: factory($this);
            $query_builder->where($translator->render_query($condition, false));
        }
        else
        {
            throw new Exception('Cannot update records without a condition');
        }

        $statement = $this->query($query_builder->getSQL());

        if ($statement instanceof \PDOException)
        {
            $this->error_handling($statement);
            return false;
        }

        if ($object instanceof CompositeDataClass && $object :: is_extended())
        {
            $query_builder = $this->connection->createQueryBuilder();
            $query_builder->update($this->get_table_name($object->get_table_name()));

            $props = array();
            foreach ($object->get_additional_properties() as $key => $value)
            {
                $query_builder->set($this->escape_column_name($key), $this->escape($value));
            }

            if ($condition)
            {
                $translator = ConditionTranslator :: factory($this);
                $query_builder->where($translator->render_query($condition, false));
            }
            else
            {
                throw new Exception('Cannot update records without a condition');
            }

            $statement = $this->query($query_builder->getSQL());

            if ($statement instanceof \PDOException)
            {
                $this->error_handling($statement);
                return false;
            }
        }

        return true;
    }

    function update_objects($table_name, $properties = array(), $condition, $offset = null, $max_objects = null, $order_by = array())
    {
        if (count($properties) > 0)
        {
            $query_builder = $this->connection->createQueryBuilder();
            $query_builder->update($this->get_table_name($table_name));

            foreach ($properties as $column => $property)
            {
                $query_builder->set($this->escape_column_name($column), $this->escape($property));
            }

            if ($condition)
            {
                $translator = ConditionTranslator :: factory($this);
                $query_builder->where($translator->render_query($condition, false));
            }
            else
            {
                throw new Exception('Cannot update records without a condition');
            }

            $statement = $this->query($query_builder->getSQL());

            if (! $statement instanceof \PDOException)
            {
                return true;
            }
            else
            {
                $this->error_handling($statement);
                return false;
            }
        }
        return true;
    }

    function updates($class, $properties, $condition, $offset = null, $count = null, $order_by = array())
    {
        return $this->update_objects($class :: get_table_name(), $properties, $condition, $offset, $count, $order_by);
    }

    /**
     * Deletes an object from a table with a given condition
     *
     * @param $table_name String
     * @param $condition Condition
     * @return true if deletion is successfull
     */
    function delete($table_name, $condition)
    {
        $query = 'DELETE FROM ' . $this->escape_table_name($table_name);

        if (isset($condition))
        {
            $translator = ConditionTranslator :: factory($this, $this->get_table_name($table_name));
            $query .= $translator->render_query($condition);
        }

        $res = $this->connection->exec($query);

        if (MDB2 :: isError($res))
        {
            $this->mdb2_error_handling($res);
            return false;
        }
        return true;
    }

    /**
     * Deletes an object from a storage unit with a given condition
     *
     * @param $class string
     * @param $condition Condition
     * @return boolean
     */
    function new_delete($class, $condition)
    {
        return $this->delete($class :: get_table_name(), $condition);
    }

    /**
     * Deletes the objects of a given table
     *
     * @param $table_name String
     * @param $condition Condition the condition
     * @return boolean
     */
    function delete_objects($table_name, $condition = null)
    {
        $query = 'DELETE FROM ' . $this->escape_table_name($table_name);

        if (isset($condition))
        {
            $translator = ConditionTranslator :: factory($this, $this->get_table_name($table_name));
            $query .= $translator->render_query($condition);
        }

        $statement = $this->query($query);

        if (! $statement instanceof \PDOException)
        {
            return true;
        }
        else
        {
            $this->error_handling($statement);
            return false;
        }
    }

    function deletes($class, $condition = null)
    {
        return $this->delete_objects($class :: get_table_name(), $condition);
    }

    /**
     * Drop a given storage unit
     *
     * @param $table_name String
     * @return boolean
     */
    function drop_storage_unit($table_name)
    {
        $schema = new \Doctrine\DBAL\Schema\Schema();
        $schema->dropTable($table_name);

        foreach ($schema->toSql($this->get_connection()->getDatabasePlatform()) as $query)
        {
            $statement = $this->query($query);

            if ($statement instanceof \PDOException)
            {
                $this->error_handling($statement);
                return false;
            }
        }

        return true;
    }

    /**
     * Counts the objects in a storage unit with a given condition
     *
     * @param $table_name string
     * @param $condition Condition
     * @return int
     *
     * @deprecated Use count() in implementations, use the PackageDataManager :: count() in applications
     */
    function count_objects($table_name, $condition = null)
    {
        $query_builder = $this->connection->createQueryBuilder();
        $query_builder->addSelect(' COUNT(1)');
        $query_builder->from($this->escape_table_name($table_name), $this->get_alias($table_name));

        return $this->count_result_set($query_builder, $table_name, $condition);
    }

    /**
     * Counts the objects in a storage unit with a given condition
     *
     * @param $class string
     * @param $parameters \common\libraries\DataClassCountParameters
     */
    function count($class, $parameters)
    {
        return $this->count_objects($class :: get_table_name(), $parameters->get_condition());
    }

    /**
     *
     * @param $class string
     * @param $parameters \common\libraries\DataClassCountGroupedParameters
     * @return multitype:int
     */
    function count_grouped($class, $parameters)
    {
        return $this->count_grouped_object($class :: get_table_name(), $parameters->get_property(),
                $parameters->get_property(), $parameters->get_condition());
    }

    /**
     *
     * @param $table_name string
     * @param $group string
     * @param $count string
     * @param $condition \common\libraries\Condition
     * @return multitype:int
     * @deprecated Use count_grouped() in implementations, use the PackageDataManager :: count_grouped() in applications
     */
    function count_grouped_object($table_name, $group, $count, $condition = null)
    {
        $query = 'SELECT ' . $this->escape_column_name($group, $this->get_alias($table_name)) . ', COUNT(' . $this->escape_column_name(
                $count, $this->get_alias($table_name)) . ') FROM ' . $this->escape_table_name($table_name) . ' AS ' . $this->get_alias(
                $table_name);

        return $this->count_grouped_result_set($query, $table_name, $group, $condition);
    }

    function count_grouped_result_set($query, $table_name, $group, $condition)
    {
        if (isset($condition))
        {
            $translator = ConditionTranslator :: factory($this, $this->get_alias($table_name));
            $query .= $translator->render_query($condition);
        }

        $query .= ' GROUP BY' . $this->escape_column_name($group, $this->get_alias($table_name));

        $res = $this->query($query);

        if (MDB2 :: isError($res))
        {
            $this->mdb2_error_handling($res);
            return false;
        }
        else
        {
            $counts = array();
            while ($record = $res->fetchRow(MDB2_FETCHMODE_ORDERED))
            {
                $counts[$record[0]] = $record[1];
            }
            $res->free();

            return $counts;
        }
    }

    function count_result_set($query_builder, $table_name, $condition = null)
    {
        if (isset($condition))
        {
            $translator = ConditionTranslator :: factory($this, $this->get_alias($table_name));
            $query_builder->where($translator->render_query($condition, false));
        }

        $statement = $this->query($query_builder->getSQL());

        if (! $statement instanceof \PDOException)
        {
            $record = $statement->fetch(\PDO :: FETCH_NUM);
            return (int) $record[0];
        }
        else
        {
            $this->error_handling($statement);
            return false;
        }
    }

    /**
     * Retrieves the objects of a given table
     *
     * @param $table_name String
     * @param $classname String The name of the class where the object has to be mapped to
     * @param $condition Condition the condition
     * @param $offset Int the starting offset
     * @param $max_objects Int the max amount of objects to be retrieved
     * @param $order_by Array(String) the list of column names that the objects have to be ordered by
     * @param $resultset String - Optional, the resultset to map the items to
     * @return ResultSet
     */
    function retrieve_objects($table_name, $condition = null, $offset = null, $max_objects = null, $order_by = array(), $class_name = null)
    {
        $query_builder = $this->connection->createQueryBuilder();
        $query_builder->addSelect('*');
        $query_builder->from($this->escape_table_name($table_name), $this->get_alias($table_name));

        return $this->retrieve_object_set($query_builder, $table_name, $condition, $offset, $max_objects, $order_by,
                $class_name);
    }

    /**
     *
     * @param $class string
     * @param $parameters \common\libraries\DataClassResultSetParameters
     * @return \common\libraries\ResultSet
     */
    function retrieves($class, $parameters)
    {
        return $this->retrieve_objects($class :: get_table_name(), $parameters->get_condition(),
                $parameters->get_offset(), $parameters->get_count(), $parameters->get_order_by(), $class);
    }

    /**
     *
     * @param $query_builder Doctrine\DBAL\Query\QueryBuilder
     * @param $table_name string
     * @param $condition Condition
     * @param $offset int
     * @param $max_objects int
     * @param $order_by multi:ObjectTableOrder
     * @return Ambigous <boolean, unknown>
     */
    function retrieve_result($query_builder, $table_name, $condition = null, $offset = null, $max_objects = null,
            $order_by = array())
    {
        if (isset($condition))
        {
            $translator = ConditionTranslator :: factory($this, $this->get_alias($table_name));
            $query_builder->where($translator->render_query($condition, false));
        }

        if (is_null($order_by))
        {
            $order_by = array();
        }
        elseif (! is_array($order_by))
        {
            $order_by = array($order_by);
        }

        foreach ($order_by as $order)
        {
            $query_builder->addOrderBy(
                    $this->escape_column_name($order->get_property(),
                            ($order->alias_is_set() ? $order->get_alias() : $this->get_alias($table_name))),
                    ($order->get_direction() == SORT_DESC ? 'DESC' : 'ASC'));
        }

        if (intval($max_objects) > 0)
        {
            $query_builder->setMaxResults(intval($max_objects));
        }

        if (intval($offset) > 0)
        {
            $query_builder->setFirstResult(intval($offset));
        }
        return $this->query($query_builder->getSQL());
    }

    function retrieve_object_set($query_builder, $table_name, $condition = null, $offset = null, $max_objects = null,
            $order_by = array(), $class_name = null)
    {
        $statement = $this->retrieve_result($query_builder, $table_name, $condition, $offset, $max_objects, $order_by);
        if (is_null($class_name))
        {
            $called_class = get_called_class();
            $namespace = Utilities :: get_namespace_from_classname($called_class);

            $class_name = $namespace . '\\' . Utilities :: underscores_to_camelcase($table_name);
        }

        return new DoctrineResultSet($this, $statement, $class_name);
    }

    /*
     * broken function, does anyone care ? function retrieve_record_set($query, $table_name, $condition = null, $offset
     * = null, $max_objects = null, $order_by = array()) { return new RecordResultSet($this->retrieve_result($query,
     * $table_name, $condition, $offset, $max_objects, $order_by)); }
     */

    /**
     *
     * @param $table_name string
     * @param $column string
     * @param $condition \common\libraries\Condition
     * @return int
     * @deprecated Use retrieve_maximum_value() in implementations, use the PackageDataManager ::
     *             retrieve_maximum_value() in applications
     */
    function retrieve_max_sort_value($table_name, $column, $condition = null)
    {
        $query = 'SELECT MAX(' . $this->escape_column_name($column) . ') as ' . self :: ALIAS_MAX_SORT . ' FROM ' . $this->escape_table_name(
                $table_name) . ' AS ' . $this->get_alias($table_name);

        if (isset($condition))
        {
            $translator = ConditionTranslator :: factory($this, $this->get_alias($table_name));
            $query .= $translator->render_query($condition);
        }

        $statement = $this->query($query);

        if (! $statement instanceof \PDOException)
        {
            $record = $statement->fetch(\PDO :: FETCH_NUM);
            return (int) $record[0];
        }
        else
        {
            $this->error_handling($statement);
            return false;
        }
    }

    /**
     *
     * @param $class string
     * @param $property string
     * @param $condition \common\libraries\Condition
     * @return int
     */
    function retrieve_maximum_value($class, $property, $condition = null)
    {
        return $this->retrieve_max_sort_value($class :: get_table_name(), $property, $condition);
    }

    /**
     *
     * @param $table_name string
     * @param $column string
     * @param $condition \common\libraries\Condition
     * @return int
     * @deprecated Use PackageDataManager :: retrieve_next_value()
     */
    function retrieve_next_sort_value($table_name, $column, $condition = null)
    {
        return $this->retrieve_max_sort_value($table_name, $column, $condition) + 1;
    }

    function truncate_storage_unit($table_name, $optimize = true)
    {
        $this->connection->loadModule('Manager');
        $manager = $this->connection->manager;
        $result = $manager->truncateTable($this->escape_table_name($table_name));
        if (! MDB2 :: isError($result))
        {
            if ($optimize)
            {
                return $this->optimize_storage_unit($table_name);
            }
            else
            {
                return true;
            }
        }
        else
        {
            $this->mdb2_error_handling($result);
            return false;
        }
    }

    function optimize_storage_unit($table_name)
    {
        $this->connection->loadModule('Manager');
        $manager = $this->connection->manager;
        if ($manager->vacuum($this->escape_table_name($table_name)))
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    function retrieve_record($table_name, $condition = null, $order_by = array())
    {
        $query_builder = $this->connection->createQueryBuilder();
        $query_builder->addSelect('*');
        $query_builder->from($this->escape_table_name($table_name), $this->get_alias($table_name));

        return $this->retrieve_row($query_builder, $table_name, $condition, $order_by);
    }

    /**
     *
     * @param $query_builder Doctrine\DBAL\Query\QueryBuilder
     * @param $table_name string
     * @param $condition Condition
     * @param $order_by multi:ObjectTableOrder
     * @return boolean \PDOException
     */
    function retrieve_row($query_builder, $table_name, $condition = null, $order_by = array())
    {
        if (isset($condition))
        {
            $translator = ConditionTranslator :: factory($this, $this->get_alias($table_name));
            $query_builder->where($translator->render_query($condition, false));
        }

        foreach ($order_by as $order)
        {
            $query_builder->addOrderBy(
                    $this->escape_column_name($order->get_property(),
                            ($order->alias_is_set() ? $order->get_alias() : $this->get_alias($table_name))),
                    ($order->get_direction() == SORT_DESC ? 'DESC' : 'ASC'));
        }

        $query_builder->setFirstResult(0);
        $query_builder->setMaxResults(1);

        $statement = $this->query($query_builder->getSQL());

        if (! $statement instanceof \PDOException)
        {
            $record = $statement->fetch(\PDO :: FETCH_ASSOC);
        }
        else
        {
            $this->error_handling($statement);
            return false;
        }

        if ($record instanceof \PDOException)
        {
            $this->error_handling($record);
            return false;
        }

        if (is_null($record))
        {
            return false;
        }

        foreach ($record as &$field)
        {
            if (is_resource($field))
            {
                $data = '';
                while (! feof($field))
                    $data .= fread($field, 1024);
                $field = $data;
            }
        }

        if ($record)
        {
            return $record;
        }
        else
        {
            return false;
        }
    }

    /**
     *
     * @param $class string
     * @param $parameters \common\libraries\DataClassResultParameters
     * @throws DataClassNoResultException
     * @return multitype:string
     */
    function retrieve($class, $parameters = null)
    {
        $record = $this->retrieve_record($class :: get_table_name(), $parameters->get_condition(),
                $parameters->get_order_by());

        if (! is_array($record) || count($record) == 0)
        {
            throw new DataClassNoResultException($class, $parameters);
        }
        else
        {
            return $record;
        }
    }

    /**
     *
     * @param $table_name string
     * @param $condition Condition
     * @param $order_by array
     * @param $class_name string
     * @return Ambigous <\common\libraries\DataClass|boolean>
     * @deprecated Use PackageDataManager :: retrieve()
     */
    function retrieve_object($table_name, $condition = null, $order_by = array(), $class_name = null)
    {
        $record = $this->retrieve_record($table_name, $condition, $order_by);
        if (is_null($class_name))
        {
            $called_class = get_called_class();
            $namespace = Utilities :: get_namespace_from_classname($called_class);

            $class_name = $namespace . '\\' . Utilities :: underscores_to_camelcase($table_name);
        }

        if ($record)
        {
            return self :: record_to_object($record, $class_name);
        }
        else
        {
            return false;
        }
    }

    function retrieve_distinct($table_name, $column_name, $condition = null)
    {
        $query_builder = $this->connection->createQueryBuilder();
        $query_builder->addSelect('DISTINCT(' . $this->escape_column_name($column_name) . ')');
        $query_builder->from($this->escape_table_name($table_name), $this->get_alias($table_name));

        if (isset($condition))
        {
            $translator = ConditionTranslator :: factory($this, $this->get_alias($table_name));
            $query_builder->where($translator->render_query($condition, false));
        }

        $statement = $this->query($query_builder->getSQL());

        if (! $statement instanceof \PDOException)
        {
            $distinct_elements = array();
            while ($record = $statement->fetch(\PDO :: FETCH_ASSOC))
            {
                $distinct_elements[] = $record[$column_name];
            }
            return $distinct_elements;
        }
        else
        {
            $this->error_handling($statement);
            return false;
        }
    }

    /**
     *
     * @param $class string
     * @param $property string
     * @param $condition \common\libraries\Condition
     * @return multitype:string
     */
    function distinct($class, $property, $condition = null)
    {
        return $this->retrieve_distinct($class :: get_table_name(), $property, $condition);
    }

    /**
     *
     * @param $class string
     * @param $parameters \common\libraries\DataClassCountDistinctParameters
     * @return int
     */
    function new_count_distinct($class, DataClassCountDistinctParameters $parameters)
    {
        return $this->count_distinct($class :: get_table_name(), $parameters->get_property(),
                $parameters->get_condition());
    }

    /**
     *
     * @param $table_name string
     * @param $column_name string
     * @param $condition \common\libraries\Condition
     * @return int
     * @deprecated Use new_count_distinct() in implementations, use the PackageDataManager :: count_distinct() in
     *             applications
     */
    function count_distinct($table_name, $column_name, $condition = null)
    {
        $query = 'SELECT COUNT(DISTINCT(' . $this->escape_column_name($column_name) . ')) FROM ' . $this->escape_table_name(
                $table_name) . ' AS ' . $this->get_alias($table_name);

        if (isset($condition))
        {
            $translator = ConditionTranslator :: factory($this, $this->get_alias($table_name));
            $query .= $translator->render_query($condition);
        }

        $res = $this->query($query);
        if (MDB2 :: isError($res))
        {
            $this->mdb2_error_handling($res);
            return false;
        }

        $record = $res->fetchRow(MDB2_FETCHMODE_ORDERED);
        $res->free();
        return $record[0];
    }

    /**
     * Returns the alias of the table name Please note that this table name should NOT be escaped using the
     * escape_table_name method of the Database class
     *
     * @param $table_name String
     * @return String the alias
     */
    function get_alias($table_name)
    {
        return StorageAliasGenerator :: get_instance()->get_table_alias($table_name, $this->get_prefix());

        // if (!array_key_exists($table_name, $this->aliases))
        // {
        // $possible_name = substr($table_name, 0, 2) . substr($table_name, - 2);
        // $index = 0;
        // while (array_key_exists($possible_name, $this->aliases))
        // {
        // $possible_name = $possible_name . $index;
        // $index = $index ++;
        // }
        // $this->aliases[$table_name] = $possible_name;
        // }
        //
        // return $this->aliases[$table_name];
    }

    function get_constraint_name($name)
    {
        $possible_name = '';
        $parts = explode('_', $name);
        foreach ($parts as & $part)
        {
            $possible_name .= $part{0};
        }

        return $possible_name;
    }

    /**
     * Function to check whether a column is a date column or not
     *
     * @param $name String the column name
     * @return false (default value)
     */
    static function is_date_column($name)
    {
        return false;
    }

    function quote($value, $type = null, $quote = true, $escape_wildcards = false)
    {
        return $this->connection->quote($value, $type, $quote, $escape_wildcards);
    }

    function escape($text, $escape_wildcards = false)
    {
        return $this->connection->quote($text);
    }

    function query($query, $types = true, $result_class = true, $result_wrap_class = false)
    {
        try
        {
            $result = $this->connection->query($query);
            return $result;
        }
        catch (\PDOException $exception)
        {
            $this->error_handling($exception);
            return false;
        }
    }

    /**
     * Delegate method for the exec function Exec is used when the query does not expect a resultset but only true /
     * false
     *
     * @param $query String
     * @return boolean
     */
    function exec($query)
    {
        $result = $this->connection->exec($query);
        if (MDB2 :: isError($result))
        {
            $this->mdb2_error_handling($result);
            return false;
        }
        return $result;
    }

    function set_limit($limit, $offset = 0)
    {
        if ($offset && ! $limit)
            throw new Exception('Supplying an offset without a limit is not supported by MDB2!');

        $result = $this->connection->setLimit($limit, $offset);
        if (MDB2 :: isError($result))
        {
            $this->mdb2_error_handling($result);
            return false;
        }
    }

    /**
     * ************************************************************************ FUNCTIONALITY THAT ENABLES NESTED TREES
     * VIA NESTED_TREE_NODE.CLASS.PHP * ************************************************************************
     */

    /**
     * Counts the children of a tree node
     *
     * @param $node NestedTreeNode - the node
     * @param $condition Condition - additional conditions
     */
    function nested_tree_count_children($node, $recursive = false, $condition = null)
    {
        $condition = $this->nested_tree_build_children_condition($node, $recursive, $condition);
        return $this->count_objects($node->get_table_name(), $condition);
    }

    /**
     * Retrieves the children of a tree node
     *
     * @param $node NestedTreeNode - the node
     * @param $recursive boolean - if put on true, every child will be retrieved, even those who are not directly
     *            connected with parent_id
     * @param $condition Condition - additional conditions
     */
    function nested_tree_get_children($node, $recursive = false, $condition = null)
    {
        $condition = $this->nested_tree_build_children_condition($node, $recursive, $condition);
        return $this->retrieve_objects($node->get_table_name(), $condition, null, null, array(), get_class($node));
    }

    /**
     * Build the conditions for the get / count children methods
     *
     * @param $node NestedTreeNode - the node
     * @param $recursive boolean - use recursive checks with left / right value or not recursive checks with parent_id
     * @param $condition Condition - additional conditions
     */
    private function nested_tree_build_children_condition($node, $recursive = false, $condition = null)
    {
        $children_conditions = array();

        if ($recursive)
        {
            $children_conditions[] = new InequalityCondition(NestedTreeNode :: PROPERTY_LEFT_VALUE,
                    InequalityCondition :: GREATER_THAN, $node->get_left_value());
            $children_conditions[] = new InequalityCondition(NestedTreeNode :: PROPERTY_RIGHT_VALUE,
                    InequalityCondition :: LESS_THAN, $node->get_right_value());
        }
        else
        {
            $children_conditions[] = new EqualityCondition(NestedTreeNode :: PROPERTY_PARENT_ID, $node->get_id());
        }

        if ($condition)
        {
            $children_conditions[] = $condition;
        }

        return new AndCondition($children_conditions);
    }

    /**
     * Counts the parents of a tree node
     *
     * @param $node NestedTreeNode - the node
     * @param $include_object boolean - if set to true the current node will be added to the count
     * @param $condition Condition - additional conditions
     */
    function nested_tree_count_parents($node, $include_object = false, $condition = null)
    {
        $condition = $this->nested_tree_build_parents_condition($node, true, $include_object, $condition);
        return $this->count_objects($node->get_table_name(), $condition);
    }

    /**
     * Get the parents of a tree node
     *
     * @param $node NestedTreeNode - the node
     * @param $recursive boolean - if set to true every parent will be retrieved recursivly, even those not connected
     *            with parent_id directly
     * @param $include_object boolean - if set to true the current node will be added to the parents list
     * @param $condition Condition - additional conditions
     */
    function nested_tree_get_parents($node, $recursive = false, $include_object = false, $condition = null)
    {
        $condition = $this->nested_tree_build_parents_condition($node, $recursive, $include_object, $condition);
        $order = new ObjectTableOrder(NestedTreeNode :: PROPERTY_LEFT_VALUE, SORT_DESC);
        return $this->retrieve_objects($node->get_table_name(), $condition, null, null, $order, get_class($node));
    }

    /**
     * Build the conditions for the get / count parents methods
     *
     * @param $node NestedTreeNode - the node
     * @param $recursive boolean - use recursive checks with left / right value or not recursive checks with parent_id
     * @param $include_object boolean - if set to true the current node will be added to the parents list
     * @param $condition Condition - additional conditions
     */
    private function nested_tree_build_parents_condition($node, $recursive = false, $include_object = false, $condition = null)
    {
        $parent_conditions = array();

        if ($recursive)
        {
            if ($include_object)
            {
                $parent_conditions[] = new InequalityCondition(NestedTreeNode :: PROPERTY_LEFT_VALUE,
                        InequalityCondition :: LESS_THAN_OR_EQUAL, $node->get_left_value());
                $parent_conditions[] = new InequalityCondition(NestedTreeNode :: PROPERTY_RIGHT_VALUE,
                        InequalityCondition :: GREATER_THAN_OR_EQUAL, $node->get_right_value());
            }
            else
            {
                $parent_conditions[] = new InequalityCondition(NestedTreeNode :: PROPERTY_LEFT_VALUE,
                        InequalityCondition :: LESS_THAN, $node->get_left_value());
                $parent_conditions[] = new InequalityCondition(NestedTreeNode :: PROPERTY_RIGHT_VALUE,
                        InequalityCondition :: GREATER_THAN, $node->get_right_value());
            }
        }
        else
        {
            $parent_conditions[] = new EqualityCondition(NestedTreeNode :: PROPERTY_ID, $node->get_parent_id());
        }

        if ($condition)
        {
            $parent_conditions[] = $condition;
        }

        return new AndCondition($parent_conditions);
    }

    /**
     * Counts the sibblings of a tree node
     *
     * @param $node NestedTreeNode - the node
     * @param $include_object boolean - if set to true the current node will be added to the count
     * @param $condition Condition - additional conditions
     */
    function nested_tree_count_sibblings($node, $include_object = false, $condition = null)
    {
        $condition = $this->nested_tree_build_sibblings_condition($node, $include_object, $condition);
        return $this->count_objects($node->get_table_name(), $condition);
    }

    /**
     * Gets the sibblings of a tree node
     *
     * @param $node NestedTreeNode - the node
     * @param $include_object boolean - if set to true the current node will be added to the sibblings list
     * @param $condition Condition - additional conditions
     */
    function nested_tree_get_sibblings($node, $include_object = false, $condition = null)
    {
        $condition = $this->nested_tree_build_sibblings_condition($node, $include_object, $condition);
        return $this->retrieve_objects($node->get_table_name(), $condition, null, null, array(), get_class($node));
    }

    /**
     * Build the conditions for the get / count sibblings methods
     *
     * @param $node NestedTreeNode - the node
     * @param $include_object boolean - if set to true the current node will be added to the sibblings list
     * @param $condition Condition - additional conditions
     */
    private function nested_tree_build_sibblings_condition($node, $include_object = false, $condition = null)
    {
        $siblings_conditions = array();

        $siblings_conditions[] = new EqualityCondition(NestedTreeNode :: PROPERTY_PARENT_ID, $node->get_parent());

        if (! $include_object)
        {
            $siblings_conditions[] = new NotCondition(
                    new EqualityCondition(NestedTreeNode :: PROPERTY_ID, $node->get_id()));
        }

        if ($condition)
        {
            $siblings_conditions[] = $condition;
        }

        return new AndCondition($siblings_conditions);
    }

    /**
     * Retrieve a node from the database
     *
     * @param $table_name String - the table name
     * @param $id int - the id of the node
     */
    public function nested_tree_retrieve_node($node, $id)
    {
        if (! isset($id))
        {
            return false;
        }

        $condition = new EqualityCondition(NestedTreeNode :: PROPERTY_ID, $id);
        return $this->retrieve_object($node->get_table_name(), $condition, array(), get_class($node));
    }

    /**
     * Retrieve a parent node from the database
     *
     * @param $table_name String - the table name
     */
    public function nested_tree_retrieve_parent_from_node($node)
    {
        $condition = new EqualityCondition(NestedTreeNode :: PROPERTY_ID, $node->get_parent_id());
        return $this->retrieve_object($node->get_table_name(), $condition, array(), get_class($node));
    }

    /**
     * Change the left/right values in the tree of every node that comes after the given node
     *
     * @param $node NestedTreeNode - the node
     * @param $previous_visited int - the previous node
     * @param $number_of_elements int - the number of elements which have to be inserted
     * @param $condition Condition - additional condition
     */
    function nested_tree_add_nested_values($node, $previous_visited, $number_of_elements = 1, $condition = null)
    {
        // Update all necessary left-values
        $conditions = array();
        $conditions[] = new InequalityCondition(NestedTreeNode :: PROPERTY_LEFT_VALUE,
                InequalityCondition :: GREATER_THAN, $previous_visited);

        if ($condition)
        {
            $conditions[] = $condition;
        }

        $update_condition = new AndCondition($conditions);

        $properties = array(
                NestedTreeNode :: PROPERTY_LEFT_VALUE => $this->escape_column_name(
                        NestedTreeNode :: PROPERTY_LEFT_VALUE) . ' + ' . $this->quote($number_of_elements * 2));
        $res = $this->update_objects($node->get_table_name(), $properties, $update_condition);

        if (! $res)
        {
            return false;
        }

        // Update all necessary right-values
        $conditions = array();
        $conditions[] = new InequalityCondition(NestedTreeNode :: PROPERTY_RIGHT_VALUE,
                InequalityCondition :: GREATER_THAN, $previous_visited);

        if ($condition)
        {
            $conditions[] = $condition;
        }

        $update_condition = new AndCondition($conditions);

        $properties = array(
                NestedTreeNode :: PROPERTY_RIGHT_VALUE => $this->escape_column_name(
                        NestedTreeNode :: PROPERTY_RIGHT_VALUE) . ' + ' . $this->quote($number_of_elements * 2));
        $res = $this->update_objects($node->get_table_name(), $properties, $update_condition);

        if (! $res)
        {
            return false;
        }

        return true;
    }

    /**
     * Change the left/right values in the tree of every node that is infected due to a delete of the given node
     *
     * @param $node NestedTreeNode - the node
     * @param $condition Condition - additional condition
     */
    function nested_tree_delete_nested_values($node, $condition)
    {
        $delta = $node->get_right_value() - $node->get_left_value() + 1;

        // Update all necessary nested-values
        $conditions = array();
        $conditions[] = new InequalityCondition(NestedTreeNode :: PROPERTY_LEFT_VALUE,
                InequalityCondition :: GREATER_THAN, $node->get_left_value());

        if ($condition)
        {
            $conditions[] = $condition;
        }

        $update_condition = new AndCondition($conditions);

        $properties = array();
        $properties[NestedTreeNode :: PROPERTY_LEFT_VALUE] = $this->escape_column_name(
                NestedTreeNode :: PROPERTY_LEFT_VALUE) . ' - ' . $this->quote($delta);
        $properties[NestedTreeNode :: PROPERTY_RIGHT_VALUE] = $this->escape_column_name(
                NestedTreeNode :: PROPERTY_RIGHT_VALUE) . ' - ' . $this->quote($delta);
        $res = $this->update_objects($node->get_table_name(), $properties, $update_condition);

        if (! $res)
        {
            return false;
        }

        // Update some more nested-values
        $conditions = array();
        $conditions[] = new InequalityCondition(NestedTreeNode :: PROPERTY_LEFT_VALUE, InequalityCondition :: LESS_THAN,
                $node->get_left_value());
        $conditions[] = new InequalityCondition(NestedTreeNode :: PROPERTY_RIGHT_VALUE,
                InequalityCondition :: GREATER_THAN, $node->get_right_value());

        if ($condition)
        {
            $conditions[] = $condition;
        }

        $update_condition = new AndCondition($conditions);

        $properties = array(
                NestedTreeNode :: PROPERTY_RIGHT_VALUE => $this->escape_column_name(
                        NestedTreeNode :: PROPERTY_RIGHT_VALUE) . ' - ' . $this->quote($delta));
        $res = $this->update_objects($node->get_table_name(), $properties, $update_condition);

        if (! $res)
        {
            return false;
        }

        return true;
    }

    public function __sleep()
    {
        throw new Exception("MDB2Database Cannot be Serialized !");
    }

    /**
     * Get the fully qualified class name of the Mdb2Database object
     *
     * @return string
     */
    static function class_name()
    {
        return get_called_class();
    }

    /**
     * Get the namespace of the Mdb2Database object
     *
     * @return string
     */
    static function context()
    {
        return Utilities :: get_namespace_from_classname(get_called_class());
    }

    function retrieve_composite_data_class_additional_properties(CompositeDataClass $object)
    {
        if (! $object->is_extended())
        {
            return array();
        }
        $array = array_map(array($this, 'escape_column_name'), $object->get_additional_property_names());

        if (count($array) == 0)
        {
            $array = array("*");
        }

        $query = 'SELECT ' . implode(',', $array) . ' FROM ' . $this->escape_table_name($object :: get_table_name()) . ' WHERE ' . $this->escape_column_name(
                $object :: PROPERTY_ID) . '=' . $this->quote($object->get_id());

        $statement = $this->query($query);

        if (! $statement instanceof \PDOException)
        {
            $distinct_elements = array();
            $record = $statement->fetch(\PDO :: FETCH_ASSOC);

            $properties = $object->get_additional_property_names();

            $additional_properties = array();
            if (count($properties) > 0)
            {
                foreach ($properties as $prop)
                {
                    if (is_resource($record[$prop]))
                    {
                        $data = '';
                        while (! feof($record[$prop]))
                            $data .= fread($record[$prop], 1024);
                        $record[$prop] = $data;
                    }
                    $additional_properties[$prop] = $record[$prop];
                }
            }

            return $additional_properties;
        }
        else
        {
            $this->error_handling($statement);
            return false;
        }
    }
}
