<?php
namespace common\libraries;
/**
 * This class represents a result set. It allows you to create an abstract representation of a remote set of data, e.g.
 * a database record set.
 *
 * @author Tim De Pauw
 * @author Hans De Bisschop
 * @package common.libraries
 */
abstract class ResultSet
{
    const POSITION_FIRST = 1;
    const POSITION_LAST = 2;
    const POSITION_SINGLE = 3;
    const POSITION_MIDDLE = 4;
    const POSITION_INVALID = 5;

    /**
     * Retrieves next item from this result set
     *
     * @param $map_to_object boolean Whether or not to map the record to an object
     * @return mixed The item, or null if none.
     */
    abstract function next_result($map_to_object = true);

    /**
     * Retrieves the number of items in this result set.
     *
     * @return int The number of items.
     */
    abstract function size();

    /**
     * Checks whether this result set is empty. The default implementation of this method checks whether the size()
     * function returns 0.
     *
     * @return boolean True if empty, false otherwise.
     */
    function is_empty()
    {
        return ($this->size() == 0);
    }

    /**
     * Skips a number of items. The default implementation of this method merely discards the output of the
     * next_result() function $count times.
     *
     * @param $count int The number of items to skip.
     */
    function skip($count)
    {
        for($i = 0; $i < $count; $i ++)
        {
            $this->next_result();
        }
    }

    /**
     * Returns an array representation of this result set.
     *
     * @return array An array containing all the items in the set.
     */
    function as_array()
    {
        $array = array();
        while ($result = $this->next_result())
        {
            $array[] = $result;
        }
        return $array;
    }

    /**
     * Get the current position in the ResultSet
     *
     * @return int
     */
    function position()
    {
        if ($this->size() == 1)
        {
            return self :: POSITION_SINGLE;
        }
        elseif ($this->size() > 1 && $this->current() == $this->size())
        {
            return self :: POSITION_LAST;
        }
        elseif ($this->size() > 1 && $this->current() == 1)
        {
            return self :: POSITION_FIRST;
        }
        elseif ($this->current() == 0 || $this->current() > $this->size())
        {
            return self :: POSITION_INVALID;
        }
        else
        {
            return self :: POSITION_MIDDLE;
        }
    }

    /**
     * Is the result on the given position in the ResultSet?
     * @param $position int
     * @return boolean
     */
    function is_position($position)
    {
        return ($this->position() == $position || $this->is_single());
    }

    /**
     * Is the result the first one in the ResultSet?
     *
     * @return boolean
     */
    function is_first()
    {
        return $this->is_position(self :: POSITION_FIRST);
    }

    /**
     * Is the result the last one in the ResultSet?
     *
     * @return boolean
     */
    function is_last()
    {
        return $this->is_position(self :: POSITION_LAST);
    }

    /**
     * Is the result in the middle of the ResultSet. That is not the first, not the last and not the only result?
     *
     * @return boolean
     */
    function is_middle()
    {
        return $this->is_position(self :: POSITION_MIDDLE);
    }

    /**
     * Is the result the only one?
     *
     * @return boolean
     */
    function is_single()
    {
        return $this->size() == 1;
    }

//     abstract function current();
}
?>