<?php

namespace common\libraries;

/**
 * Cache data. Lazy loaded and saved as needed to/from the cache.
 * Everything is loaded/stored in one go.
 * 
 * Usage
 * 
 * $data = SystemCache::system();  //retrieve the system cache
 * $data->key = value;             //save data to the cache
 * $value = $data->key             //retrieve data from the cache
 * unset($data->key)               //remove data from the cache 
 *
 * @author laurent
 */
class CacheData
{
    private $_save = null;
    private $_load = null;
    private $_stale = null;
    private $_notify = null;
    private $_is_dirty = false;
    private $_data = false;

    /**
     *
     * @param callback $load    the load from cache function
     * @param callback $save    the save to cache function
     * @param callback $stale   the stale function. If it returns true the cache is invalid. False otherwise.
     * @param callback $notify  the notify function. Called when data are stored to the cache
     */
    public function __construct($load, $save, $stale, $notify = null)
    {
        $this->_load = $load;
        $this->_save = $save;
        $this->_stale = $stale;
        $this->_notify = $notify;
    }

    /**
     * Returns true if the object has been modified since it was fetched from
     * the cache. False otherwise.
     * 
     * @return bool 
     */
    public function is_dirty()
    {
        return $this->_is_dirty;
    }

    /**
     * Converse of is_dirty. True if object does not need to be store in the db.
     * 
     * @return bool
     */
    public function is_clean()
    {
        return ! $this->_is_dirty;
    }

    /**
     * Returns true if data have been fetched from the cache. False otherwise.
     * 
     * @return bool
     */
    public function is_loaded()
    {
        return $this->_data !== false;
    }

    public function is_stale()
    {
        if (empty($this->_stale))
        {
            return false;
        }
        $f = $this->_stale;
        return $f();
    }

    /**
     * Remove all data. 
     */
    public function clear()
    {
        $this->load();
        $this->_data = (object) array();
        $this->_is_dirty = true;
    }

    /**
     * Load data from the cache - if not already done - and returns true.
     * If data have already been loaded from the cache do nothing and returns false.
     * 
     * @return bool
     */
    protected function load()
    {
        if ($this->is_loaded())
        {
            return false;
        }
        if ($this->is_stale())
        {
            $this->_data = (object) array();
            $this->_is_dirty = true;
            return true;
        }
        $f = $this->_load;
        $this->_data = $f();
        if (empty($this->_data))
        {
            $this->_data = (object) array();
        }
        $this->_is_dirty = false;
        return true;
    }

    /**
     * Save data to the cache if the object is dirty and returns true.
     * If object is clean do nothing and returns false.
     * @return bool
     */
    protected function save()
    {
        if ($this->is_clean())
        {
            return false;
        }
        $f = $this->_save;
        $f($this->_data);
        $this->_is_dirty = false;
        if ($this->_notify)
        {
            $f = $this->_notify;
            $f($this->_data);
        }
        return true;
    }

    /**
     * Magical method. Set the value and mark the object as dirty.
     * 
     * @param string $name
     * @param any $value 
     */
    public function __set($name, $value)
    {
        $this->load();
        $this->_data->$name = $value;
        $this->_is_dirty = true;
    }

    /**
     * Magical method. Fetch data from the cache if needed and returns the value.
     * If the 
     * 
     * @param string $name
     * @return any
     */
    public function __get($name)
    {
        $this->load();
        return isset($this->_data->$name) ? $this->_data->$name : null;
    }

    /**
     * Magical method. Returns true if the $name is set.
     * 
     * @param string $name
     * @return bool 
     */
    public function __isset($name)
    {
        $this->load();
        return isset($this->_data->$name);
    }

    /**
     * Magical method. If needed unset $name and set the object to dirty.
     * 
     * @param string $name 
     */
    public function __unset($name)
    {
        $this->load();
        if (! isset($this->_data->$name))
        {
            return;
        }
        unset($this->_data->$name);
        $this->_is_dirty = true;
    }

    /**
     * Update the cache if needed.
     */
    public function __destruct()
    {
        $this->save();
    }
}

?>
