<?php

namespace common\libraries;

/**
 * System caches. File based. Synch through the DB with CacheSynch.
 *
 * - system()  shared by everybody
 * - user()    per user cache
 *
 * Those two caches are loaded nearly for all pages as they are used in global.
 * If you need another
 *
 */
class SystemCache
{
    const SYSTEM_ROOT = 'system';
    const SYSTEM_KEY = 'system';

    const USER_ROOT = 'user';
    const USER_KEY = 'user';

    const APPLICATION_ROOT = 'application';
    const APPLICATION_KEY = 'application';

    private static $system_store = null;
    private static $user_store = null;
    private static $system_data = null;
    private static $user_data = null;
    private static $application_data = null;

    private static $_is_active = null;

    public static function is_active()
    {
        if (is_null(self :: $_is_active))
        {
            self :: $_is_active = strpos(Request :: server('HTTP_REFERER', ''), 'install') === false;
        }
        return self :: $_is_active;
    }

    /**
     * @return FileCache
     */
    public static function get_system_store()
    {
        if (empty(self :: $system_store))
        {
            self :: $system_store = new FileCache(self :: SYSTEM_ROOT, true);
        }
        return self :: $system_store;
    }

    /**
     * @return FileCache
     */
    public static function get_user_store()
    {
        if (empty(self :: $user_store))
        {
            self :: $user_store = new FileCache(self :: USER_ROOT, true);
        }
        return self :: $user_store;
    }

    /**
     * @return FileCache
     */
    public static function get_application_store()
    {
        if (empty(self :: $user_store))
        {
            self :: $user_store = new FileCache(self :: APPLICATION_ROOT, true);
        }
        return self :: $user_store;
    }

    public static function reset_client($delayed = false)
    {
        $f = function ()
        {
            ResourceCache :: reset();
            ClientCache :: reset();
        };
        if ($delayed)
        {
            $key = __CLASS__ . __FUNCTION__;
            if (! ExitAction :: has($key))
            {
                ExitAction :: register($f, $key);
            }
        }
        else
        {
            $f();
        }
    }

    public static function reset_system($delayed = false)
    {
        $f = function ()
        {
            CacheSynch :: reset(SystemCache :: SYSTEM_KEY);
            SystemCache :: get_system_store()->reset();
        };
        if ($delayed)
        {
            $key = __CLASS__ . __FUNCTION__;
            if (! ExitAction :: has($key))
            {
                ExitAction :: register($f, $key);
            }
        }
        else
        {
            $f();
        }
    }

    public static function reset_user($delayed = false)
    {
        $f = function ()
        {
            CacheSynch :: reset(SystemCache :: USER_KEY);
            SystemCache :: get_user_store()->reset();
        };
        if ($delayed)
        {
            $key = __CLASS__ . __FUNCTION__;
            if (! ExitAction :: has($key))
            {
                ExitAction :: register($f, $key);
            }
        }
        else
        {
            $f();
        }
    }

    public static function reset_application($delayed = false)
    {
        $f = function ()
        {
            CacheSynch :: reset(SystemCache :: APPLICATION_KEY);
            SystemCache :: get_application_store()->reset();
        };
        if ($delayed)
        {
            $key = __CLASS__ . __FUNCTION__;
            if (! ExitAction :: has($key))
            {
                ExitAction :: register($f, $key);
            }
        }
        else
        {
            $f();
        }
    }

    /**
     * Generic cache. Shared by everybody.
     *
     * @return CacheData
     */
    public static function system()
    {
        if (empty(self :: $system_data))
        {
            if (! self :: is_active())
            {
                self :: $system_data = new CacheDataTransient();
            }
            else
            {
                $load = function ()
                {
                    $key = SystemCache :: SYSTEM_KEY;
                    $store = SystemCache :: get_system_store();
                    return $store->get($key);
                };
                $save = function ($data)
                {
                    $key = SystemCache :: SYSTEM_KEY;
                    $store = SystemCache :: get_system_store();
                    return $store->set($key, $data);
                };
                $stale = function ()
                {
                    $key = SystemCache :: SYSTEM_KEY;
                    $store = SystemCache :: get_system_store();
                    $modification_time = CacheSynch :: get($key);
                    return $store->is_stale($key, $modification_time);
                };
                self :: $system_data = new CacheData($load, $save, $stale);
            }
        }
        return self :: $system_data;
    }

    /**
     * User specific cache.
     *
     * @return CacheData
     */
    public static function user()
    {
        if (empty(self :: $user_data))
        {
            if (! self :: is_active())
            {
                self :: $user_data = new CacheDataTransient();
            }
            else
            {
                $load = function ()
                {
                    $user_id = Session :: get_user_id();
                    $key = SystemCache :: USER_KEY . '_' . $user_id;
                    $store = SystemCache :: get_user_store();
                    return $store->get($key);
                };
                $save = function ($data)
                {
                    $user_id = Session :: get_user_id();
                    $key = SystemCache :: USER_KEY . '_' . $user_id;
                    $store = SystemCache :: get_user_store();
                    return $store->set($key, $data);
                };
                $stale = function ()
                {
                    //$user_id = Session::get_user_id();
                    $key = SystemCache :: USER_KEY; //no user specific key would be too much would need special treatment to work.
                    $store = SystemCache :: get_system_store();
                    $modification_time = CacheSynch :: get($key);
                    return $store->is_stale($key, $modification_time);
                };
                self :: $user_data = new CacheData($load, $save, $stale);
            }
        }

        return self :: $user_data;
    }

    /**
     * Application specific cache.
     *
     * @return CacheData
     */
    public static function application()
    {
        if (empty(self :: $application_data))
        {
            if (! self :: is_active())
            {
                self :: $application_data = new CacheDataTransient();
            }
            else
            {
                $load = function ()
                {
                    $application = Request :: get(Application :: PARAM_APPLICATION, 'common');
                    $key = SystemCache :: APPLICATION_KEY . '_' . $application;
                    $store = SystemCache :: get_application_store();
                    return $store->get($key);
                };
                $save = function ($data)
                {
                    $application = Request :: get(Application :: PARAM_APPLICATION, 'common');
                    $key = SystemCache :: APPLICATION_KEY . '_' . $application;
                    $store = SystemCache :: get_application_store();
                    return $store->set($key, $data);
                };
                $stale = function ()
                {
                    $application = Request :: get(Application :: PARAM_APPLICATION, 'common');
                    $key = SystemCache :: APPLICATION_KEY . '_' . $application;
                    $store = SystemCache :: get_application_store();
                    $modification_time = CacheSynch :: get($key);
                    return $store->is_stale($key, $modification_time);
                };
                self :: $application_data = new CacheData($load, $save, $stale);
            }
        }

        return self :: $application_data;
    }

}

?>