<?php
namespace common\libraries;

//require_once(__DIR__ . '/../../../lib/data_manager.class.php');
//require_once("C:\wamp\www\chamilo-stable\common\libraries\php/lib/data_manager.class.php");

/**
 * Provides a basic framework for creating integration tests.
 * @author Anthony Hurst (Hogeschool Gent)
 */
abstract class DataClassIntegrationTestCase extends \PHPUnit_Framework_TestCase
{
    /**
     * @var DataClass[] A collection of entities persisted to the database during testing, and which need deleting afterwards.
     */
    private $persisted_entities;

    /**
     * Final to guarantee execution. Override setUpTest() instead.
     */
    protected final function setUp()
    {
        $dm = new CommonDataManager();
        $this->persisted_entities = array();
        $this->setUpTest();
    }

    protected function setUpTest()
    {}

    /**
     * Creates a standardised error message for inclusion as message in an assertion.
     * @static
     * @param string $value An identifier for the value that failed the assertion.
     * @param mixed $expected The expected value used in the assertion.
     * @param mixed $actual The actual value used in the assertion.
     * @return string The composed message.
     */
    protected static function createErrorMessage($value, $expected, $actual)
    {
        return "Assertion failed on $value. Expected $expected, got $actual.";
    }

    /**
     * Retrieves an entity from the database by its class name and id.
     * @static
     * @param string|DataClass $class_name The fully qualified name of the class being retrieved.
     * @param int $id The id of the entity being retrieved.
     * @return $class_name|null The retrieved entity from the database or null if not found.
     */
    protected static function retrieveEntity($class_name, $id)
    {
        $condition = new EqualityCondition(DataClass :: PROPERTY_ID, $id);

        /**
         * @var CommonDataManager $data_manager
         */
        $data_manager = $class_name :: get_data_manager();
        $method = 'retrieve_object';
        if(method_exists($data_manager, $method))
        {
            return $data_manager->$method($class_name :: get_table_name(), $condition, array(), $class_name);
        }
        $parameters = new DataClassRetrieveParameters($condition);
        return $data_manager->retrieve($class_name, $parameters);
    }

    /**
     * @static
     * @param DataClass $entity
     */
    protected static function deleteEntity($entity)
    {
        $namespace = Utilities :: get_namespace_from_object($entity);
        $data_manager = $namespace . '\DataManager';

        while (!class_exists($data_manager) && strlen($namespace) > 0)
        {
            $namespace = Utilities :: get_namespace_parent($namespace);
            $data_manager = $namespace . '\DataManager';
        }

        if (strlen($namespace) > 0)
        {
            return $data_manager :: delete($entity);
        }

        // Code for backwards compatibility
        $data_manager = $entity->get_data_manager();
        $class_name = $entity->get_object_name();

        $method = 'delete_' . $class_name;

        if (method_exists($data_manager, $method))
        {
            return $data_manager->$method($entity);
        }
        else
        {
            $condition = new EqualityCondition(DataClass :: PROPERTY_ID, $entity->get_id());
            return $data_manager->delete($entity->get_table_name(), $condition);
        }

    }

    /**
     * Modifies the properties of an entity.
     * @static
     * @param DataClass $entity The entity whose properties are to be modified.
     * @param boolean $alter_id Whether the id must be modified or not.
     */
    protected abstract static function modifyEntity(&$entity, $alter_id);

    /**
     * Adds an entity to the collection of entities persisted to the database.
     * @param DataClass $entity The entity to be added to the collection of entities persisted to the database.
     */
    protected function registerPersistedEntity($entity)
    {
        $this->persisted_entities[] = $entity;
    }

    /**
     * Final to guarantee execution. Override tearDownTest() instead.
     */
    protected final function tearDown()
    {
        foreach($this->persisted_entities as $persisted_entity)
        {
            self :: deleteEntity($persisted_entity);
        }
        unset($this->persisted_entities);
        $this->tearDownTest();
    }

    protected function tearDownTest()
    {}
}
?>