<?php
namespace common\libraries;

use webservice\WebserviceDataManager;
use webservice\WebserviceRights;
use webservice\WebserviceManager;

/**
 * Extension to the rest server to automatically determine the context - the object and check for authentication +
 * rights
 */
class ChamiloRestServer extends RestServer
{
    const PARAM_ID = 'id';
    const PARAM_CONTEXT = 'context';
    const PARAM_OBJECT = 'object';

    const ACTION_GET = 'get';
    const ACTION_GET_LIST = 'get_list';
    const ACTION_CREATE = 'create';
    const ACTION_UPDATE = 'update';
    const ACTION_DELETE = 'delete';

    /**
     * The action that is connected to the given parameters
     *
     * @var String
     */
    private $action;
    /**
     * The webservice handler
     *
     * @var WebserviceHandler
     */
    private $webservice_handler;
    /**
     * The result from the webservice handler
     *
     * @var String
     */
    private $result;
    /**
     * Keeps track of the logged in user
     *
     * @var User
     */
    private $user;

    /**
     * Handles the webservice
     */
    function handle()
    {
        $this->process_request();
        $this->manipulate_retrieved_data();

        $this->determine_action();

        if ($this->determine_webservice_handler($action))
        {
            $this->call_webservice_handler();
        }

        $this->handle_result();
    }

    /**
     * Manipulates the retrieved data (removes context, object and id from parameters)
     */
    private function manipulate_retrieved_data()
    {
        unset($this->data[self :: PARAM_CONTEXT]);
        unset($this->data[self :: PARAM_OBJECT]);
        unset($this->data[self :: PARAM_ID]);
    }

    /**
     * Returns the name of the action that will be executed for the given method
     *
     * @return String
     */
    private function determine_action()
    {
        $id = Request :: get(self :: PARAM_ID);

        switch ($this->get_method())
        {
            case self :: METHOD_GET :
                if ($id)
                {
                    $this->action = self :: ACTION_GET;
                    break;
                }
                else
                {
                    $this->action = self :: ACTION_GET_LIST;
                    break;
                }
                break;
            case self :: METHOD_POST :
                $this->action = self :: ACTION_CREATE;
                break;
            case self :: METHOD_PUT :
                $this->action = self :: ACTION_UPDATE;
                break;
            case self :: METHOD_DELETE :
                $this->action = self :: ACTION_DELETE;
                break;
        }
    }

    /**
     * Determine the webservice handler
     *
     * @return boolean (if the handler can be determined or not)
     */
    private function determine_webservice_handler()
    {
        $context = $this->get_context();
        $object = $this->get_object();

        if ($context && $object)
        {
            $action = $this->action;
            $path = Path :: namespace_to_full_path($context) . 'php/webservices/' . $object . '/' . $action . '.class.php';
            if (! file_exists($path))
            {
                $this->result = new SuccessRestMessage(false,
                        Translation :: get('WebserviceHandlerNotImplemented', null,
                                WebserviceManager :: APPLICATION_NAME));
                return false;
            }

            $class = $context . '\\' . Utilities :: underscores_to_camelcase($object) . Utilities :: underscores_to_camelcase(
                    $action) . 'WebserviceHandler';

            $this->webservice_handler = new $class();
            return true;
        }
        else
        {
            $this->result = new SuccessRestMessage(false,
                    Translation :: get('ContextAndObjectShouldNotBeEmpty', null, WebserviceManager :: APPLICATION_NAME));

            return false;
        }
    }

    /**
     * Calls the webservice handler with the given parameters
     */
    private function call_webservice_handler()
    {
        if ($this->is_allowed())
        {
            $this->webservice_handler->set_id($this->get_id());
            $this->webservice_handler->set_data($this->data);
            $this->webservice_handler->set_user($this->user);

            $this->result = $this->webservice_handler->run();
        }
    }

    /**
     * Check if the user is allowed to execute the webservice The user is retrieved with the webservice authentication
     * framework.
     *
     * @param $context
     * @param $object <type>
     * @param $function <type>
     * @return <type>
     */
    private function is_allowed()
    {
        $context = $this->get_context();
        $object = $this->get_object();
        $action = $this->action;

        // Check the registration
        $registration = WebserviceDataManager :: retrieve_webservice_registration_by_code(
                $context . '_' . $object . '_' . $action);

        if (! $registration)
        {
            $this->result = new SuccessRestMessage(false,
                    Translation :: get('WebserviceNotRegistered', null, WebserviceManager :: APPLICATION_NAME));
            return false;
        }

        // Check the active status
        if (! $registration->get_active())
        {
            $this->result = new SuccessRestMessage(false,
                    Translation :: get('WebserviceNotActive', null, WebserviceManager :: APPLICATION_NAME));
            return false;
        }

        // Check whether the webservice needs authentication
        if (! $registration->get_authentication())
        {
            return true;
        }

        // Check the authentication
        $authentication = WebserviceAuthentication :: factory();

        if (! $authentication->is_valid())
        {
            $this->result = new SuccessRestMessage(false,
                    Translation :: get('NotAuthorized', null, WebserviceManager :: APPLICATION_NAME) . ': ' . implode(
                            ', ', $authentication->get_messages()));
            return false;
        }

        $this->user = $authentication->get_user();

        // Check the rights
        if (! WebserviceRights :: is_allowed_in_webservices_subtree(WebserviceRights :: USE_RIGHT,
                $registration->get_id(), WebserviceRights :: TYPE_WEBSERVICE, $this->user->get_id()))
        {
            $this->result = new SuccessRestMessage(false,
                    Translation :: get('NoRightsToExecuteWebservice', null, WebserviceManager :: APPLICATION_NAME));
            return false;
        }

        return true;
    }

    /**
     * Handles the result
     */
    private function handle_result()
    {
        $renderer = RestMessageRenderer :: factory($this->format);
        $renderer->render($this->result);
    }

    /**
     * Returns the selected id
     *
     * @return int
     */
    private function get_id()
    {
        return Request :: get(self :: PARAM_ID);
    }

    /**
     * Returns the selected context
     *
     * @return String
     */
    private function get_context()
    {
        return Request :: get(self :: PARAM_CONTEXT);
    }

    /**
     * Returns the selected object
     *
     * @return String
     */
    private function get_object()
    {
        return Request :: get(self :: PARAM_OBJECT);
    }

}

?>
