<?php
namespace core\lynx\manager;

use common\libraries\Theme;

use common\libraries\Utilities;

use core\lynx\Action;
use core\lynx\package\Package;
use common\libraries\CommonDataManager;
use core\lynx\DependencyVerifier;
use admin\AdminDataManager;
use core\lynx\SourceAction;
use common\libraries\Translation;
use common\libraries\Request;

set_time_limit(0);

/**
 * Package installation
 *
 * @author Hans De Bisschop - Erasmus Hogeschool Brussel
 * @author Magali Gillard - Erasmus Hogeschool Brussel
 * @author Sven Vanpoucke - Hogeschool Gent - Cleanup, code refactoring and bugfixes, comments
 * @package admin.lib.package_installer
 */
class PackageInstaller extends Action
{
	private $additional_installers = array();

    /**
     * Runs the package installer
     *
     * @return boolean
     */
    function run()
    {
        if ($this->initialize() && $this->process())
        {
        	$title = Translation :: get('Finished', null, Utilities:: get_namespace_parent(__NAMESPACE__));
        	$image = Theme :: get_image_path(Utilities:: get_namespace_parent(__NAMESPACE__)) . 'package_action/finished.png';
            return $this->action_successful($title, $image, Translation :: get('PackageCompletelyInstalled'));
        }
        else
        {
        	$title = Translation :: get('Failed', null, Utilities:: get_namespace_parent(__NAMESPACE__));
        	$image = Theme :: get_image_path(Utilities:: get_namespace_parent(__NAMESPACE__)) . 'package_action/failed.png';
            return $this->action_failed($title, $image, Translation :: get('PackageInstallFailed'));
        }
    }

    /**
     * Initializes the package installer
     *
     * @return boolean
     */
    function initialize()
    {
    	$title = Translation :: get('Initialization', null, Utilities:: get_namespace_parent(__NAMESPACE__));
    	$image = Theme :: get_image_path(Utilities:: get_namespace_parent(__NAMESPACE__)) . 'package_action/initialization.png';
    	
        if (! $this->get_package() instanceof Package)
        {
            return $this->action_failed($title, $image, Translation :: get('PackageAttributesNotFound'));
        }
        else
        {
            $this->add_message(Translation :: get('PackageAttributesFound'));
        }

        // Check registration
        if ($this->is_package_registered())
        {
            return $this->action_failed($title, $image, Translation :: get('PackageIsAlreadyRegistered'));
        }
        else
        {
            $this->add_message(Translation :: get('PackageNotYetRegistered'));
        }

        return $this->action_successful($title, $image, Translation :: get('PackageInstallInitialized'));
    }

    /**
     * Checks if the package is registered
     *
     * @return boolean
     */
    function is_package_registered()
    {
        return CommonDataManager :: is_registered($this->get_package()->get_context());
    }

    /**
     * Installs the package
     *
     * @return boolean
     */
    function process()
    {
    	$title = Translation :: get('Installation', array('PACKAGE' => Translation :: get('TypeName', null, $this->get_package()->get_context())), Utilities:: get_namespace_parent(__NAMESPACE__));
    	$image = Theme :: get_image_path(Utilities:: get_namespace_parent(__NAMESPACE__)) . 'package_action/installation.png';
    	
        $installer = \common\libraries\package\Installer :: factory($this->get_package()->get_context(), array());
        if (! $installer->install())
        {
        	$this->add_message($installer->retrieve_message());
            return $this->action_failed($title, $image, Translation :: get('InitializationFailed'));
        }
        else
        {
            $this->add_message($installer->retrieve_message());
            $this->action_successful($title, $image);
        }

        $installer->set_message(array());
        
        $title = Translation :: get('Configuration', array('PACKAGE' => Translation :: get('TypeName', null, $this->get_package()->get_context())), Utilities:: get_namespace_parent(__NAMESPACE__));
        $image = Theme :: get_image_path(Utilities:: get_namespace_parent(__NAMESPACE__)) . 'package_action/configuration.png';

        if (! $installer->post_process())
        {
        	$this->add_message($installer->retrieve_message());
            return $this->action_failed($title, $image, Translation :: get('ConfigurationFailed'));
        }
        else
        {
            $this->add_message($installer->retrieve_message());
            $this->action_successful($title, $image);
        }
        
        $this->add_additional_installers($installer->get_additional_installers());
        
        $title = Translation :: get('AdditionalPackages', null, Utilities:: get_namespace_parent(__NAMESPACE__));
        $image = Theme :: get_image_path(Utilities:: get_namespace_parent(__NAMESPACE__)) . 'package_action/additional_packages.png';
        
        while (($additional_installer = $this->get_next_additional_installer()) != null)
        {
        	if (! $this->install_additional_package($additional_installer))
        	{
        		return $this->action_failed($title, $image, Translation :: get('AdditionalPackagesFailed'));
        	}
        }        	
       
        return true;
    }
    
    /**
     * Installs an additional package
     *
     * @param $package_installer \common\libraries\package\Installer
     */
    private function install_additional_package(\common\libraries\package\Installer $package_installer)
    {
    	$context = Utilities :: get_namespace_from_object($package_installer);
    	
    	$title = Translation :: get('Installation', array('PACKAGE' => Translation :: get('TypeName', null, $context)), Utilities:: get_namespace_parent(__NAMESPACE__));
    	$image = Theme :: get_image_path($context) . 'logo/48.png';
    	
    	if (! $package_installer->install())
    	{
    		$this->add_message($package_installer->retrieve_message());
    		return $this->action_failed($title, $image, Translation :: get('InitializationFailed'));
    	}
    	else
    	{
    		$this->add_message($package_installer->retrieve_message());
    		$this->action_successful($title, $image);
    	}
    	
    	$package_installer->set_message(array());
    	
    	$title = Translation :: get('Configuration', array('PACKAGE' => Translation :: get('TypeName', null, $context)), Utilities:: get_namespace_parent(__NAMESPACE__));
    	
    	if (! $package_installer->post_process())
    	{
    		$this->add_message($package_installer->retrieve_message());
    		return $this->action_failed($title, $image, Translation :: get('ConfigurationFailed'));
    	}
    	else
    	{
    		$this->add_message($package_installer->retrieve_message());
    		$this->action_successful($title, $image);
    	}
    
    	$this->add_additional_installers($package_installer->get_additional_installers());
    	
    	return true;
    }
    
    /**
     * Returns the additional installers
     *
     * @return \common\libraries\package\Installer[]
     */
    public function get_additional_installers()
    {
    	return $this->additional_installers;
    }
    
    /**
     * Sets the additional installers
     *
     * @param $additional_installers \common\libraries\package\Installer[]
     */
    public function set_additional_installers($additional_installers)
    {
    	$this->additional_installers = $additional_installers;
    }
    
    /**
     * Adds an additional installer to the list of additional installers
     *
     * @param $additional_installer common\libraries\Installer
     */
    public function add_additional_installer(\common\libraries\package\Installer $additional_installer)
    {
    	array_push($this->additional_installers, $additional_installer);
    }
    
    /**
     * Adds multiple additional installers to the list of additional installers
     *
     * @param $additional_installers common\libraries\Installer[]
     */
    public function add_additional_installers($additional_installers)
    {
    	foreach ($additional_installers as $additional_installer)
    	{
    		$this->add_additional_installer($additional_installer);
    	}
    }
    
    /**
     * Removes and returns the first installer from the list of additional installers
     *
     * @return common\libraries\Installer
     */
    public function get_next_additional_installer()
    {
    	return array_shift($this->additional_installers);
    }
}
?>