<?php

namespace group;

use common\libraries\InequalityCondition;
use common\libraries\EqualityCondition;
use common\libraries\ConditionTranslator;
use common\libraries\Mdb2Database;
use common\libraries\NotCondition;
use common\libraries\InCondition;
use common\libraries\OrCondition;
use common\libraries\AndCondition;
use user\UserDataManager;
use user\User;

/**
 * $Id: database_group_data_manager.class.php 232 2009-11-16 10:11:48Z vanpouckesven $
 * @package group.lib.datamanager
 */

/**
  ==============================================================================
 * This is a data manager that uses a database for storage. It was written
 * for MySQL, but should be compatible with most SQL flavors.
 *
 * @author Tim De Pauw
 * @author Bart Mollet
 * @author Sven Vanpoucke
 * @author Hans De Bisschop
  ==============================================================================
 */
class Mdb2GroupDataManager extends Mdb2Database implements GroupDataManagerInterface
{
    private $group_cache;

    function initialize()
    {
        parent :: initialize();
        $this->set_prefix('group_');
    }

    function update_group($group)
    {
        $condition = new EqualityCondition(Group :: PROPERTY_ID, $group->get_id());
        return $this->update($group, $condition);
    }

    function delete_group($group)
    {
        $condition = new EqualityCondition(Group :: PROPERTY_ID, $group->get_id());
        $bool = $this->delete($group->get_table_name(), $condition);

        $condition_subgroups = new EqualityCondition(Group :: PROPERTY_PARENT, $group->get_id());
        $groups = $this->retrieve_groups($condition_subgroups);
        while ($gr = $groups->next_result())
        {
            $bool = $bool & $this->delete_group($gr);
        }

        $this->truncate_group($group);

        return $bool;
    }

    function truncate_group($group)
    {
        $condition = new EqualityCondition(GroupRelUser :: PROPERTY_GROUP_ID, $group->get_id());
        return $this->delete(GroupRelUser :: get_table_name(), $condition);
    }

    function delete_group_rel_user($groupreluser)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(GroupRelUser :: PROPERTY_GROUP_ID, $groupreluser->get_group_id());
        $conditions[] = new EqualityCondition(GroupRelUser :: PROPERTY_USER_ID, $groupreluser->get_user_id());
        $condition = new AndCondition($conditions);

        return $this->delete($groupreluser->get_table_name(), $condition);
    }

    function create_group($group)
    {
        return $this->create($group);
    }

    function create_group_rel_user($groupreluser)
    {
        return $this->create($groupreluser);
    }

    function count_groups($condition = null)
    {
        return $this->count_objects(Group :: get_table_name(), $condition);
    }

    function count_group_rel_users($condition = null)
    {
        return $this->count_objects(GroupRelUser :: get_table_name(), $condition);
    }

    /**
     * Counts a resultset of entire users
     */
    function count_group_rel_user_users($condition = null)
    {
        $user_dm = UserDataManager :: get_instance();
        $user_alias = $user_dm->get_alias(User :: get_table_name());
        $group_rel_user_alias = $this->get_alias(GroupRelUser :: get_table_name());

        $query = 'SELECT COUNT(1)';
        $query .= ' FROM ' . $this->escape_table_name(GroupRelUser :: get_table_name());
        $query .= ' AS ' . $group_rel_user_alias;

        $query .= ' JOIN ' . $user_dm->escape_table_name(User :: get_table_name()) . ' AS ' . $user_alias;
        $query .= ' ON ' . $this->escape_column_name(GroupRelUser :: PROPERTY_USER_ID, $group_rel_user_alias);
        $query .= ' = ' . $user_dm->escape_column_name(User :: PROPERTY_ID, $user_alias);

        return $this->count_result_set($query, User :: get_table_name(), $condition);
    }

    function retrieve_groups($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        return $this->retrieve_objects(Group :: get_table_name(), $condition, $offset, $max_objects, $order_by, Group :: CLASS_NAME);
    }

    function retrieve_group_rel_users($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        return $this->retrieve_objects(GroupRelUser :: get_table_name(), $condition, $offset, $max_objects, $order_by, GroupRelUser :: CLASS_NAME);
    }

    /**
     * Returns a resultset of entire users
     */
    function retrieve_group_rel_user_users($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        $user_dm = UserDataManager :: get_instance();
        $user_alias = $user_dm->get_alias(User :: get_table_name());
        $group_rel_user_alias = $this->get_alias(GroupRelUser :: get_table_name());

        $query = 'SELECT ' . $user_alias . '.*';
        $query .= ' FROM ' . $this->escape_table_name(GroupRelUser :: get_table_name());
        $query .= ' AS ' . $group_rel_user_alias;

        $query .= ' JOIN ' . $user_dm->escape_table_name(User :: get_table_name()) . ' AS ' . $user_alias;
        $query .= ' ON ' . $this->escape_column_name(GroupRelUser :: PROPERTY_USER_ID, $group_rel_user_alias);
        $query .= ' = ' . $user_dm->escape_column_name(User :: PROPERTY_ID, $user_alias);

        return $this->retrieve_object_set($query, User :: get_table_name(), $condition, $offset, $max_objects, $order_by, User :: CLASS_NAME);
    }

    function retrieve_group_rel_user($user_id, $group_id)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(GroupRelUser :: PROPERTY_USER_ID, $user_id);
        $conditions[] = new EqualityCondition(GroupRelUser :: PROPERTY_GROUP_ID, $group_id);
        $condition = new AndCondition($conditions);

        return $this->retrieve_object(GroupRelUser :: get_table_name(), $condition, array(), GroupRelUser :: CLASS_NAME);
    }

    function retrieve_group_rel_user_by_official_code_and_code($official_code, $group_code)
    {
        $user_dm = UserDataManager :: get_instance();

        $user_table = $user_dm->escape_table_name(User :: get_table_name());
        $user_alias = $user_dm->get_alias(User :: get_table_name());
        $group_table = $this->escape_table_name(Group :: get_table_name());
        $group_alias = $this->get_alias(Group :: get_table_name());
        $group_rel_user_table = $this->escape_table_name(GroupRelUser :: get_table_name());
        $group_rel_user_alias = $this->get_alias(GroupRelUser :: get_table_name());

        $conditions = array();
        $conditions[] = new EqualityCondition(User :: PROPERTY_OFFICIAL_CODE, $official_code, $user_alias, true);
        $conditions[] = new EqualityCondition(Group :: PROPERTY_CODE, $group_code, Group :: get_table_name());
        $condition = new AndCondition($conditions);

        $query = 'SELECT ' . $group_rel_user_alias . '.* FROM ' . $group_rel_user_table . ' AS ' . $group_rel_user_alias;
        $query .= ' JOIN ' . $group_table . ' AS ' . $group_alias;
        $query .= ' ON ' . $this->escape_column_name(GroupRelUser :: PROPERTY_GROUP_ID, $group_rel_user_alias);
        $query .= ' = ' . $this->escape_column_name(Group :: PROPERTY_ID, $group_alias);
        $query .= ' JOIN ' . $user_table . ' AS ' . $user_alias;
        $query .= ' ON ' . $this->escape_column_name(GroupRelUser :: PROPERTY_USER_ID, $group_rel_user_alias);
        $query .= ' = ' . $user_dm->escape_column_name(User :: PROPERTY_ID, $user_alias);

        return $this->retrieve_object_set($query, GroupRelUser :: get_table_name(), $condition)->next_result();
    }

    function retrieve_user_groups($user_id)
    {
        $condition = new EqualityCondition(GroupRelUser :: PROPERTY_USER_ID, $user_id);
        return $this->retrieve_objects(GroupRelUser :: get_table_name(), $condition, array(), array(), array(), GroupRelUser :: CLASS_NAME);
    }

    /**
     *
     * @param int $id
     * @param bool $cache enables cache
     * @return Group
     */
    function retrieve_group($id, $cache = true)
    {
        $condition = new EqualityCondition(Group :: PROPERTY_ID, $id);

        if ($cache)
        {
            if (is_null($this->group_cache[$id]))
            {
                $this->group_cache[$id] = $this->retrieve_object(Group :: get_table_name(), $condition, array(), Group :: CLASS_NAME);
            }
            return $this->group_cache[$id];
        }
        return $this->retrieve_object(Group :: get_table_name(), $condition, array(), Group :: CLASS_NAME);
    }

    function retrieve_group_by_name($name)
    {
        $condition = new EqualityCondition(Group :: PROPERTY_NAME, $name);
        return $this->retrieve_object(Group :: get_table_name(), $condition, array(), Group :: CLASS_NAME);
    }

    function retrieve_group_rights_templates($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        return $this->retrieve_objects(GroupRightsTemplate :: get_table_name(), $condition, $offset, $max_objects, $order_by, GroupRightsTemplate :: CLASS_NAME);
    }

    function delete_group_rights_templates($condition)
    {
        return $this->delete(GroupRightsTemplate :: get_table_name(), $condition);
    }

    function add_rights_template_link($group, $rights_template_id)
    {
        $props = array();
        $props[GroupRightsTemplate :: PROPERTY_GROUP_ID] = $group->get_id();
        $props[GroupRightsTemplate :: PROPERTY_RIGHTS_TEMPLATE_ID] = $rights_template_id;
        $this->get_connection()->loadModule('Extended');
        return $this->get_connection()->extended->autoExecute($this->get_table_name(GroupRightsTemplate :: get_table_name()), $props, MDB2_AUTOQUERY_INSERT);
    }

    function delete_rights_template_link($group, $rights_template_id)
    {
        $conditions = array();
        $conditions = new EqualityCondition(GroupRightsTemplate :: PROPERTY_GROUP_ID, $group->get_id());
        $conditions = new EqualityCondition(GroupRightsTemplate :: PROPERTY_RIGHTS_TEMPLATE_ID, $rights_template_id);
        $condition = new AndCondition($conditions);

        return $this->delete(GroupRightsTemplate :: get_table_name(), $condition);
    }

    function update_rights_template_links($group, $rights_templates)
    {
        // Delete the no longer existing links
        $conditions = array();
        $conditions = new NotCondition(new InCondition(GroupRightsTemplate :: PROPERTY_RIGHTS_TEMPLATE_ID, $rights_templates));
        $conditions = new EqualityCondition(GroupRightsTemplate :: PROPERTY_GROUP_ID, $group->get_id());
        $condition = new AndCondition($conditions);

        $success = $this->delete(GroupRightsTemplate :: get_table_name(), $condition);
        if (! $success)
        {
            return false;
        }

        // Get the group's rights_templates
        $condition = new EqualityCondition(GroupRightsTemplate :: PROPERTY_GROUP_ID, $group->get_id());
        $group_rights_templates = $this->retrieve_group_rights_templates($condition);
        $existing_rights_templates = array();

        while ($group_rights_template = $group_rights_templates->next_result())
        {
            $existing_rights_templates[] = $group_rights_template->get_rights_template_id();
        }

        // Add the new links
        foreach ($rights_templates as $rights_template)
        {
            if (! in_array($rights_template, $existing_rights_templates))
            {
                if (! $this->add_rights_template_link($group, $rights_template))
                {
                    return false;
                }
            }
        }

        return true;
    }

    function is_groupname_available($groupname, $group_id = null)
    {
        $condition = new EqualityCondition(Group :: PROPERTY_NAME, $groupname);

        if ($group_id)
        {
            $conditions = array();
            $conditions[] = new EqualityCondition(Group :: PROPERTY_NAME, $groupname);
            $conditions = new EqualityCondition(Group :: PROPERTY_ID, $group_id);
            $condition = new AndCondition($conditions);
        }

        return ! ($this->count_objects(Group :: get_table_name(), $condition) == 1);
    }

    function add_nested_values($previous_visited, $number_of_elements = 1)
    {
        // Update all necessary left-values
        $condition = new InequalityCondition(Group :: PROPERTY_LEFT_VALUE, InequalityCondition :: GREATER_THAN, $previous_visited);

        $query = 'UPDATE ' . $this->escape_table_name('group') . ' SET ' . $this->escape_column_name(Group :: PROPERTY_LEFT_VALUE) . '=' . $this->escape_column_name(Group :: PROPERTY_LEFT_VALUE) . ' + ' . $this->quote($number_of_elements * 2);
        if (isset($condition))
        {
            $translator = ConditionTranslator :: factory($this);
            $query .= $translator->render_query($condition);
        }

        $res = $this->get_connection()->exec($query);

        // Update all necessary right-values
        $condition = new InequalityCondition(Group :: PROPERTY_RIGHT_VALUE, InequalityCondition :: GREATER_THAN, $previous_visited);
        $query = 'UPDATE ' . $this->escape_table_name('group') . ' SET ' . $this->escape_column_name(Group :: PROPERTY_RIGHT_VALUE) . '=' . $this->escape_column_name(Group :: PROPERTY_RIGHT_VALUE) . ' + ' . $this->quote($number_of_elements * 2);

        $translator = ConditionTranslator :: factory($this);
        $query .= $translator->render_query($condition);

        $res = $this->get_connection()->exec($query);
        // TODO: For now we just return true ...
        return true;
    }

    function delete_nested_values($group)
    {
        $delta = $group->get_right_value() - $group->get_left_value() + 1;

        // Update all necessary nested-values
        $condition = new InequalityCondition(Group :: PROPERTY_LEFT_VALUE, InequalityCondition :: GREATER_THAN, $group->get_left_value());

        $query = 'UPDATE ' . $this->escape_table_name('group');
        $query .= ' SET ' . $this->escape_column_name(Group :: PROPERTY_LEFT_VALUE) . '=' . $this->escape_column_name(Group :: PROPERTY_LEFT_VALUE) . ' - ' . $this->quote($delta) . ', ';
        $query .= $this->escape_column_name(Group :: PROPERTY_RIGHT_VALUE) . '=' . $this->escape_column_name(Group :: PROPERTY_RIGHT_VALUE) . ' - ' . $this->quote($delta);

        $translator = ConditionTranslator :: factory($this);
        $query .= $translator->render_query($condition);

        $res = $this->query($query);

        // Update some more nested-values
        $conditions = array();
        $conditions[] = new InequalityCondition(Group :: PROPERTY_LEFT_VALUE, InequalityCondition :: LESS_THAN, $group->get_left_value());
        $conditions[] = new InequalityCondition(Group :: PROPERTY_RIGHT_VALUE, InequalityCondition :: GREATER_THAN, $group->get_right_value());
        $condition = new AndCondition($conditions);

        $query = 'UPDATE ' . $this->escape_table_name('group');
        $query .= ' SET ' . $this->escape_column_name(Group :: PROPERTY_RIGHT_VALUE) . '=' . $this->escape_column_name(Group :: PROPERTY_RIGHT_VALUE) . ' - ' . $this->quote($delta);

        $translator = ConditionTranslator :: factory($this);
        $query .= $translator->render_query($condition);

        $res = $this->query($query);

        return true;
    }

    function move_group($group, $new_parent_id, $new_previous_id = 0)
    {
        // Check some things first to avoid trouble
        if ($new_previous_id)
        {
            // Don't let people move an element behind itself
            // No need to spawn an error, since we're just not doing anything
            if ($new_previous_id == $group->get_id())
            {
                return true;
            }

            $new_previous = $this->retrieve_group($new_previous_id);
            // TODO: What if group $new_previous_id doesn't exist ? Return error.
            $new_parent_id = $new_previous->get_parent();
        }
        else
        {
            // No parent ID was set ... problem !
            if ($new_parent_id == 0)
            {
                return false;
            }
            // Move the group underneath one of it's children ?
            // I think not ... Return error
            if ($group->is_parent_of($new_parent_id))
            {
                return false;
            }
            // Move an element underneath itself ?
            // No can do ... just ignore and return true
            if ($new_parent_id == $group->get_id())
            {
                return true;
            }
            // Try to retrieve the data of the parent element
            $new_parent = $this->retrieve_group($new_parent_id);

     // TODO: What if this is an invalid group ? Return error.
        }

        $number_of_elements = ($group->get_right_value() - $group->get_left_value() + 1) / 2;
        $previous_visited = $new_previous_id ? $new_previous->get_right_value() : $new_parent->get_left_value();

        // Update the nested values so we can actually add the element
        // Return false if this failed
        if (! $this->add_nested_values($previous_visited, $number_of_elements))
        {
            return false;
        }

        // Now we can update the actual parent_id
        // Return false if this failed
        $group = $this->retrieve_group($group->get_id());
        $group->set_parent($new_parent_id);
        if (! $group->update())
        {
            return false;
        }

        // Update the left/right values of those elements that are being moved
        // First get the offset we need to add to the left/right values
        // if $newPrevId is given we need to get the right value,
        // otherwise the left since the left/right has changed
        // because we already updated it up there. We need to get them again.
        // We have to do that anyway, to have the proper new left/right values
        if ($new_previous_id)
        {
            $temp = $this->retrieve_group($new_previous_id);
            // TODO: What if $temp doesn't exist ? Return error.
            $calculate_width = $temp->get_right_value();
        }
        else
        {
            $temp = $this->retrieve_group($new_parent_id);
            // TODO: What if $temp doesn't exist ? Return error.
            $calculate_width = $temp->get_left_value();
        }

        // Get the element that is being moved again, since the left and
        // right might have changed by the add-call


        $group = $this->retrieve_group($group->get_id());
        // TODO: What if $group doesn't exist ? Return error.
        // Calculate the offset of the element to to the spot where it should go
        // correct the offset by one, since it needs to go inbetween!
        $offset = $calculate_width - $group->get_left_value() + 1;

        // Do the actual update
        $conditions = array();
        $conditions[] = new InequalityCondition(Group :: PROPERTY_LEFT_VALUE, InequalityCondition :: GREATER_THAN, ($group->get_left_value() - 1));
        $conditions[] = new InequalityCondition(Group :: PROPERTY_RIGHT_VALUE, InequalityCondition :: LESS_THAN, ($group->get_right_value() + 1));
        $condition = new AndCondition($conditions);

        $query = 'UPDATE ' . $this->escape_table_name('group');
        $query .= ' SET ' . $this->escape_column_name(Group :: PROPERTY_LEFT_VALUE) . '=' . $this->escape_column_name(Group :: PROPERTY_LEFT_VALUE) . ' + ' . $this->quote($offset) . ', ';
        $query .= $this->escape_column_name(Group :: PROPERTY_RIGHT_VALUE) . '=' . $this->escape_column_name(Group :: PROPERTY_RIGHT_VALUE) . ' + ' . $this->quote($offset);

        $translator = ConditionTranslator :: factory($this);
        $query .= $translator->render_query($condition);

        $res = $this->query($query);

        // Remove the subtree where the group was before
        if (! $this->delete_nested_values($group))
        {
            return false;
        }

        return true;
    }

    /**
     * Checks whether the given column name is the name of a column that
     * contains a date value, and hence should be formatted as such.
     * @param string $name The column name.
     * @return boolean True if the column is a date column, false otherwise.
     */
    static function is_date_column($name)
    {
        return false;
    }

    function retrieve_usable_groups($condition = null, $offset = null, $count = null, $order_property = null)
    {
        $group_alias = $this->get_alias(Group :: get_table_name());
        $group_use_group_alias = $this->get_alias(GroupUseGroup :: get_table_name());

        $query = 'SELECT DISTINCT ' . $group_alias . '.*';
        $query .= ' FROM ' . $this->escape_table_name(Group :: get_table_name()) . ' AS ' . $group_alias;
        $query .= ' JOIN ' . $this->escape_table_name(GroupUseGroup :: get_table_name()) . ' AS ' . $group_use_group_alias . ' ON ' . $this->escape_column_name(Group :: PROPERTY_ID, $group_alias) . ' = ' . $this->escape_column_name(GroupUseGroup :: PROPERTY_USE_GROUP_ID, $group_use_group_alias);

        return $this->retrieve_object_set($query, Group :: get_table_name(), $condition, $offset, $max_objects, $order_by, Group :: CLASS_NAME);
    }

    function delete_group_use_group($group_use_group)
    {
        $condition = new EqualityCondition(GroupUseGroup :: PROPERTY_ID, $group_use_group->get_id());
        return $this->delete(GroupUseGroup :: get_table_name(), $condition);
    }

    function update_group_use_group($group_use_group)
    {
        $condition = new EqualityCondition(GroupUseGroup :: PROPERTY_ID, $group_use_group->get_id());
        return $this->update(GroupUseGroup :: get_table_name(), $condition);
    }

    function create_group_use_group($group_use_group)
    {
        return $this->create($group_use_group);
    }

    function count_group_use_group($conditions = null)
    {
        return $this->count_objects(GroupUseGroup :: get_table_name(), $conditions);
    }

    function retrieve_group_use_groups($condition = null, $offset = null, $count = null, $order_property = null)
    {
        return $this->retrieve_objects(GroupUseGroup :: get_table_name(), $condition, $offset, $count, $order_property);
    }

    function retrieve_group_use_group($id)
    {
        $condition = new EqualityCondition(GroupUseGroup :: PROPERTY_ID, $id);
        return $this->retrieve_object(GroupUseGroup :: get_table_name(), $condition);
    }

    function retrieve_all_groups_and_templates($user_id)
    {
        $condition = new EqualityCondition(GroupRelUser :: PROPERTY_USER_ID, $user_id);
        $group_rel_user_alias = $this->get_alias(GroupRelUser :: get_table_name());
        $group_rights_template_alias = $this->get_alias(GroupRightsTemplate :: get_table_name());

        $query = 'SELECT ' . $this->escape_column_name(GroupRelUser :: PROPERTY_GROUP_ID, $group_rel_user_alias) . ', c.' . $this->escape_column_name(Group :: PROPERTY_ID) . ' as ' . $this->escape_column_name(Group :: PROPERTY_PARENT) . ', ' . $this->escape_column_name(GroupRightsTemplate :: PROPERTY_RIGHTS_TEMPLATE_ID, $group_rights_template_alias);
        $query .= ' FROM ' . $this->escape_table_name(GroupRelUser :: get_table_name()) . ' as ' . $group_rel_user_alias;
        $query .= ' JOIN ' . $this->escape_table_name(Group :: get_table_name()) . ' as b on ' . $this->escape_column_name(GroupRelUser :: PROPERTY_GROUP_ID, $group_rel_user_alias) . ' = ' . 'b.' . $this->escape_column_name(Group :: PROPERTY_ID);
        $query .= ' JOIN ' . $this->escape_table_name(Group :: get_table_name()) . ' as c on c.' . $this->escape_column_name(Group :: PROPERTY_LEFT_VALUE) . ' < b. ' . $this->escape_column_name(Group :: PROPERTY_RIGHT_VALUE) . ' and c.' . $this->escape_column_name(Group :: PROPERTY_RIGHT_VALUE) . ' > b.' . $this->escape_column_name(Group :: PROPERTY_RIGHT_VALUE);
        $query .= ' LEFT JOIN ' . $this->escape_table_name(GroupRightsTemplate :: get_table_name()) . ' as ' . $group_rights_template_alias . ' on ' . $this->escape_column_name(GroupRightsTemplate :: PROPERTY_GROUP_ID, $group_rights_template_alias) . ' = ' . $this->escape_column_name(GroupRelUser :: PROPERTY_GROUP_ID, $group_rel_user_alias) . ' or c. ' . $this->escape_column_name(Group :: PROPERTY_ID) . ' = ' . $this->escape_column_name(GroupRightsTemplate :: PROPERTY_GROUP_ID, $group_rights_template_alias);

        return $this->retrieve_object_set($query, GroupRelUser :: get_table_name(), $condition, null, null, null, Group :: CLASS_NAME);

     //   $groups_and_templates = new ObjectResultSet($user->get_data_manager(), $user->get_data_manager()->query($query), $class_name = GroupRightsTemplate :: CLASS_NAME);
    }

    /**
     * Retrieves the groups a user is linked to
     * Use the condition to limit or exclude certain groups
     * @param <type> $user_id
     * @return Mdb2ResultSet
     */
    function retrieve_direct_subscribed_groups($user_id)
    {

        $condition = new EqualityCondition(GroupRelUser :: PROPERTY_USER_ID, $user_id);

        $group_rel_user_alias = $this->get_alias(GroupRelUser :: get_table_name());
        $group_alias = $this->get_alias(Group :: get_table_name());

        $query = 'SELECT * ';
        $query .= ' FROM ' . $this->escape_table_name(GroupRelUser :: get_table_name()) . ' as ' . $group_rel_user_alias;
        $query .= ' JOIN ' . $this->escape_table_name(Group :: get_table_name()) . ' as ' . $group_alias . ' on ' . $this->escape_column_name(GroupRelUser :: PROPERTY_GROUP_ID, $group_rel_user_alias) . ' = ' . $this->escape_column_name(Group :: PROPERTY_ID, $group_alias);

        return $this->retrieve_object_set($query, GroupRelUser :: get_table_name(), $condition, null, null, null, Group :: CLASS_NAME);
    }

    private $direct_subscribed_groups_cache;
    private $only_retrieve_ids;

    /**
     * Retrieves the groups a user is linked to
     * Use the condition to limit or exclude certain groups
     * @param <type> $user_id
     * @param <type> $condition
     * @param <type> $only_retrieve_ids
     * @return <array>
     */
    function retrieve_direct_subscribed_groups_array($user_id, $only_retrieve_ids = false)
    {
        $cache_hash = md5($user_id . ':' . (int) $only_retrieve_ids);
        
        if (is_null($this->direct_subscribed_groups_cache[$cache_hash]))
        {
            $direct_subscribed_groups = $this->retrieve_direct_subscribed_groups($user_id);
            if (! $direct_subscribed_groups->is_empty())
            {
                if ($only_retrieve_ids)
                {
                    while ($direct_subscribed_group = $direct_subscribed_groups->next_result())
                    {
                        $this->direct_subscribed_groups_cache[$cache_hash][] = $direct_subscribed_group->get_id();
                    }
                }
                else
                {
                    while ($direct_subscribed_group = $direct_subscribed_groups->next_result())
                    {
                        $this->direct_subscribed_groups_cache[$cache_hash][] = $direct_subscribed_group;
                    }
                }
            }
        }
        return $this->direct_subscribed_groups_cache[$cache_hash];
    }

    private $all_subscribed_groups_cache;

    /**
     * Retrieves all the groups a user is subscribed in, including their parents
     * Use the condition to limit or exclude certain groups
     * @param <type> $user_id
     * @return array
     */
    function retrieve_all_subscribed_groups_array($user_id, $only_retrieve_ids = false)
    {
        $cache_hash = md5($user_id . ':' . (int) $only_retrieve_ids);
        
        if (is_null($this->all_subscribed_groups_cache[$cache_hash]))
        {
            $direct_subscribed_groups = $this->retrieve_direct_subscribed_groups_array($user_id, false);

            if (! empty($direct_subscribed_groups))
            {
                $conditions = array();
                foreach ($direct_subscribed_groups as $direct_subscribed_group)
                {
                    $and_conditions = array();
                    $and_conditions[] = new InequalityCondition(Group :: PROPERTY_LEFT_VALUE, InequalityCondition :: LESS_THAN_OR_EQUAL, $direct_subscribed_group->get_left_value());

                    $and_conditions[] = new InequalityCondition(Group :: PROPERTY_RIGHT_VALUE, InequalityCondition :: GREATER_THAN_OR_EQUAL, $direct_subscribed_group->get_right_value());

                    $sub_conditions[] = new AndCondition($and_conditions);
                }
                $conditions[] = new OrCondition($sub_conditions);

                $final_condition = new AndCondition($conditions);

                $group_alias = $this->get_alias(Group :: get_table_name());
                $query = 'SELECT ' . $this->escape_column_name(Group :: PROPERTY_ID);
                $query .= ' FROM ' . $this->escape_table_name(Group :: get_table_name()) . ' as ' . $group_alias;

                $result_set = $this->retrieve_object_set($query, Group :: get_table_name(), $final_condition, null, null, null, Group :: CLASS_NAME);

                $this->all_subscribed_groups_cache[$cache_hash] = array();

                if ($only_retrieve_ids)
                {
                    while ($group = $result_set->next_result())
                    {
                        $this->all_subscribed_groups_cache[$cache_hash][] = $group->get_id();
                    }
                }
                else
                {
                    while ($group = $result_set->next_result())
                    {
                        $this->all_subscribed_groups_cache[$cache_hash][] = $group;
                    }
                }
            }
            else
            {
                $this->all_subscribed_groups_cache[$cache_hash] = $direct_subscribed_groups;
            }
        }
        return $this->all_subscribed_groups_cache[$cache_hash];
    }

    function unsubscribe_user_from_all_groups($user)
    {
        $condition = new EqualityCondition(GroupRelUser :: PROPERTY_USER_ID, $user->get_id());
        return $this->delete(GroupRelUser :: get_table_name(), $condition);
    }

    function is_group_member($group_id, $user_id)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(GroupRelUser :: PROPERTY_GROUP_ID, $group_id);
        $conditions[] = new EqualityCondition(GroupRelUser :: PROPERTY_USER_ID, $user_id);
        $condition = new AndCondition($conditions);

        return $this->count_objects(GroupRelUser :: get_table_name(), $condition) > 0;
    }

    function retrieve_groups_and_subgroups($group_ids, $offset = null, $count = null, $order_by = null, $condition = null)
    {
        if (count($group_ids) == 0)
        {
            $group_ids[] = - 1;
        }
        $dg_condition = new InCondition(Group :: PROPERTY_ID, $group_ids);
        $direct_groups = $this->retrieve_groups($dg_condition);

        $direct_group_conditions = array();
        while ($group = $direct_groups->next_result())
        {
            $and_conditions = array();
            $and_conditions[] = new InequalityCondition(Group :: PROPERTY_LEFT_VALUE, InequalityCondition :: GREATER_THAN_OR_EQUAL, $group->get_left_value());
            $and_conditions[] = new InequalityCondition(Group :: PROPERTY_RIGHT_VALUE, InequalityCondition :: LESS_THAN_OR_EQUAL, $group->get_right_value());
            $direct_group_conditions[] = new AndCondition($and_conditions);
        }

        if (count($direct_group_conditions) > 0)
        {
            $group_conditions = array();
            if ($condition)
            {
                $group_conditions[] = $condition;
            }
            $group_conditions[] = new OrCondition($direct_group_conditions);
            $group_condition = new AndCondition($group_conditions);
        }
        else
        {
            $group_condition = new EqualityCondition(Group :: PROPERTY_ID, - 1);
        }

        return $this->retrieve_groups($group_condition, $offset, $count, $order_by);
    }

    function retrieve_group_by_code_and_parent_id($code, $parent_id)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(Group :: PROPERTY_CODE, $code);
        $conditions[] = new EqualityCondition(Group :: PROPERTY_PARENT, $parent_id);
        $condition = new AndCondition($conditions);
        return $this->retrieve_object(Group :: get_table_name(), $condition, array(), Group :: CLASS_NAME);
    }

}
