<?php
namespace install;

use common\extensions\video_conferencing_manager\VideoConferencingManagerInstaller;
use common\extensions\external_repository_manager\ExternalRepositoryManagerInstaller;

use repository\ContentObjectInstaller;
use repository\ContentObject;

use common\libraries\SystemCache;
use common\libraries\CoreApplication;
use common\libraries\Path;
use common\libraries\Filesystem;
use common\libraries\Translation;
use common\libraries\Connection;
use common\libraries\WebApplication;
use common\libraries\PlatformSetting;
use common\libraries\Utilities;

use MDB2;
use Exception;

class Installer
{
    /**
     *
     * @var InstallerConfig $installer_config
     */
    private $installer_config;

    /**
     *
     * @var string where to write the new config file
     */
    private $config_file_destination;

    /**
     *
     * @var install\InstallerObservers list of the observers to be notified when performing the install
     */
    private $observers;

    /**
     *
     * @var install\DataManagerInterface the storage manager used to perform install
     */
    private $data_manager;

    /**
     * A list of additional installers that are provided by another installer. This can be used for optional installers
     * / applications / packages.
     *
     * @var \common\libraries\package\Installer[]
     */
    private $additional_installers;

    public function __construct(InstallerConfig $installer_config, $data_manager)
    {
        $this->set_installer_config($installer_config);
        $default_config_file = Path :: get_common_path() . 'configuration/configuration.php';
        $this->set_config_file_destination($default_config_file);

        $this->set_data_manager($data_manager);

        $this->additional_installers = array();

        $this->observers = new InstallerObservers();
    }

    public function set_data_manager($data_manager)
    {
        $this->data_manager = $data_manager;
    }

    public function set_installer_config($installer_config)
    {
        $this->installer_config = $installer_config;
    }

    public function set_config_file_destination($config_file_destination)
    {
        $this->config_file_destination = $config_file_destination;
    }

    public function add_observer(InstallerObserver $observer)
    {
        $this->observers->add_observer($observer);
    }

    /**
     * Returns the additional installers
     *
     * @return \common\libraries\package\Installer[]
     */
    public function get_additional_installers()
    {
        return $this->additional_installers;
    }

    /**
     * Sets the additional installers
     *
     * @param $additional_installers \common\libraries\package\Installer[]
     */
    public function set_additional_installers($additional_installers)
    {
        $this->additional_installers = $additional_installers;
    }

    /**
     * Adds an additional installer to the list of additional installers
     *
     * @param $additional_installer common\libraries\Installer
     */
    public function add_additional_installer(\common\libraries\package\Installer $additional_installer)
    {
        array_push($this->additional_installers, $additional_installer);
    }

    /**
     * Adds multiple additional installers to the list of additional installers
     *
     * @param $additional_installers common\libraries\Installer[]
     */
    public function add_additional_installers($additional_installers)
    {
        foreach ($additional_installers as $additional_installer)
        {
            $this->add_additional_installer($additional_installer);
        }
    }

    /**
     * Removes and returns the first installer from the list of additional installers
     *
     * @return common\libraries\Installer
     */
    public function get_next_additional_installer()
    {
        return array_shift($this->additional_installers);
    }

    public function perform_install()
    {

        $this->observers->before_install();
        $this->initialize_install();

        $this->perform_preprod();

        $this->install_core_applications();
        $this->install_extensions();
        $this->install_content_objects();
        $this->install_extra_applications();
        $this->install_external_repository_managers();
        $this->install_video_conferencing_managers();
        $this->install_additional_packages();

        $this->create_folders();

        $this->observers->after_install();

        SystemCache :: reset_system();
        // not so good but good enough for now
        SystemCache :: reset_client();
    }

    private function initialize_install()
    {
        PlatformSetting :: set_instance(new InstallationPlatformSettings());
        Translation :: get_instance()->set_language($this->installer_config->get_platform_language());
    }

    private function perform_preprod()
    {
        $this->observers->before_preprod();
        $result_config = $this->write_config_file();
        $this->observers->preprod_config_file_written($result_config);

        $this->data_manager->init_storage_access();
        $this->data_manager->init_storage_structure();

        $result = new InstallerStepResult(true, Translation :: get('DatabaseCreated'));
        $this->observers->preprod_db_created($result);

        $this->install_common_libraries();

        $this->observers->after_preprod();
    }

    private function write_config_file()
    {
        $config_template_file = Path :: get_common_path() . 'configuration/configuration.dist.php';
        $template = file_get_contents($config_template_file);

        if ($template === false)
        {
            throw new \Exception(Translation :: get('ConfigWriteFailed'));
        }
        $config_string = $this->template_to_config($template);

        $write_status = Filesystem :: write_to_file($this->config_file_destination, $config_string);

        if ($write_status === false)
        {
            throw new \Exception(Translation :: get('ConfigWriteFailed'));
        }

        return new InstallerStepResult(true, Translation :: get('ConfigWriteSuccess'));
    }

    private function template_to_config($template)
    {
        $config = array();
        $config['{DATABASE_DRIVER}'] = $this->installer_config->get_db_driver();
        $config['{DATABASE_HOST}'] = $this->installer_config->get_db_host();
        $config['{DATABASE_USER}'] = $this->installer_config->get_db_username();
        $config['{DATABASE_PASSWORD}'] = $this->installer_config->get_db_password();
        $config['{DATABASE_NAME}'] = $this->installer_config->get_db_name();
        $config['{ROOT_WEB}'] = $this->installer_config->get_base_url();
        $config['{ROOT_SYS}'] = str_replace('\\', '/', realpath($config['{ROOT_WEB}']) . '/');
        $config['{SECURITY_KEY}'] = md5(uniqid(rand() . time()));
        $config['{URL_APPEND}'] = $this->installer_config->get_url_append();
        $config['{HASHING_ALGORITHM}'] = $this->installer_config->get_crypt_algorithm();
        $config['{INSTALL_DATE}'] = time();

        $result = $template;
        foreach ($config as $key => $value)
        {
            $result = str_replace($key, $value, $result);
        }
        return $result;
    }

    private function install_core_applications()
    {
        $core_apps = CoreApplication :: get_list();

        $this->observers->before_core_applications_install();
        foreach ($core_apps as $core_application)
        {
            $this->install_application($core_application);
        }
        $this->observers->after_core_applications_install();

        $this->observers->before_core_applications_configuration();
        foreach ($core_apps as $core_application)
        {
            $this->post_process_application($core_application);
        }
        $this->observers->after_core_applications_configuration();
    }

    private function install_extensions()
    {
        $extensions = Utilities :: get_common_extensions_packages_from_filesystem();

        $this->observers->before_extensions_install();

        foreach ($extensions as $extension)
        {
            $installer = \common\libraries\Installer :: factory($extension);
            if ($installer)
            {
                $this->observers->before_extension_install($extension);

                $success = $installer->install();
                $installer->add_message();
                $success = $success && $installer->post_process();
                $step_result = new InstallerStepResult($success, $installer->get_message());

                $this->observers->after_extension_install($step_result);
            }
        }

        $this->observers->after_extensions_install();
    }

    private function install_common_libraries()
    {
        $this->observers->before_application_install('common\libraries');
        $values = $this->installer_config->as_values_array();
        $installer = \common\libraries\package\Installer :: factory('common\libraries', $values);

        $success = $installer->install();
        $success = $success && $installer->post_process();
        $step_result = new InstallerStepResult($success, $installer->get_message());

        $this->observers->after_application_install($step_result);
    }

    private function install_application($application)
    {
        $this->observers->before_application_install($application);
        $values = $this->installer_config->as_values_array();
        $installer = \common\libraries\package\Installer :: factory($application, $values);

        $success = $installer->install();

        if ($success !== true)
        {
            throw new \Exception($installer->retrieve_message());
        }

        $this->add_additional_installers($installer->get_additional_installers());

        $step_result = new InstallerStepResult($success, $installer->get_message());
        $this->observers->after_application_install($step_result);

    }

    private function post_process_application($application)
    {
        $this->observers->before_post_process_for_application($application);
        $values = $this->installer_config->as_values_array();
        $installer = \common\libraries\package\Installer :: factory($application, $values);

        $success = $installer->post_process();

        if ($success !== true)
        {
            throw new \Exception($installer->retrieve_message());
        }

        $step_result = new InstallerStepResult($success, $installer->get_message());
        $this->observers->after_post_process_for_application($step_result);

    }

    private function install_extra_applications()
    {
        $applications = $this->installer_config->get_applications();

        $this->observers->before_optional_applications_install();
        foreach ($applications as $application)
        {
            $this->install_extra_application($application);
            $this->observers->after_optional_application_install();
        }
        $this->observers->after_optional_applications_install();

        $this->observers->before_optional_applications_configuration();
        foreach ($applications as $application)
        {
            $this->post_process_application($application);
            $this->observers->after_optional_application_install();
        }
        $this->observers->after_optional_applications_configuration();
    }

    private function install_extra_application($application)
    {
        $this->install_application($application);
    }

    private function install_content_objects()
    {
        $this->observers->before_content_objects_install();

        $content_objects = $this->installer_config->get_content_objects();

        foreach ($content_objects as $content_object)
        {
            $installer = ContentObjectInstaller :: factory($content_object);
            if ($installer)
            {
                $this->observers->before_content_object_install($content_object);

                $success = $installer->install();
                $installer->add_message();
                $success = $success && $installer->post_process();
                $step_result = new InstallerStepResult($success, $installer->get_message());

                $this->observers->after_content_object_install($step_result);
            }
        }
        $this->observers->after_content_objects_install();
    }

    private function install_external_repository_managers()
    {
        $this->observers->before_external_repository_managers_install();
        $external_instances = $this->installer_config->get_external_instances();

        foreach ($external_instances as $external_instance)
        {
            $installer = ExternalRepositoryManagerInstaller :: factory($external_instance);
            if ($installer)
            {
                $this->observers->before_external_repository_manager_install($external_instance);

                $success = $installer->install();
                $installer->add_message();
                $success = $success && $installer->post_process();
                $step_result = new InstallerStepResult($success, $installer->get_message());

                $this->observers->after_external_repository_manager_install($step_result);
            }
        }
        $this->observers->after_external_repository_managers_install();
    }

    private function install_video_conferencing_managers()
    {
        $this->observers->before_video_conferencing_managers_install();
        $dir = Path :: get_common_extensions_path() . 'video_conferencing_manager/implementation/';
        $folders = Filesystem :: get_directory_content($dir, Filesystem :: LIST_DIRECTORIES, false);

        foreach ($folders as $folder)
        {
            $installer = VideoConferencingManagerInstaller :: factory(\common\extensions\video_conferencing_manager\Manager :: get_namespace($folder));
            if ($installer)
            {
                $this->observers->before_video_conferencing_manager_install($folder);

                $success = $installer->install();
                $installer->add_message();
                $success = $success && $installer->post_process();
                $step_result = new InstallerStepResult($success, $installer->get_message());

                $this->observers->after_video_conferencing_manager_install($step_result);
            }
        }
        $this->observers->after_video_conferencing_managers_install();
    }

    /**
     * Installs the additional packages
     */
    private function install_additional_packages()
    {
        $this->observers->before_additional_packages_install();

        while (($installer = $this->get_next_additional_installer()) != null)
        {
            $this->install_additional_package($installer);
        }

        $this->observers->after_additional_packages_install();
    }

    /**
     * Installs an additional package
     *
     * @param $package_installer \common\libraries\package\Installer
     */
    private function install_additional_package(\common\libraries\package\Installer $package_installer)
    {
        $context = Utilities :: get_namespace_from_object($package_installer);

        $this->observers->before_additional_package_install($context);

        $success = $package_installer->install();

        if ($success !== true)
        {
            throw new \Exception($package_installer->retrieve_message());
        }

        $success = $package_installer->post_process();

        if ($success !== true)
        {
            throw new \Exception($package_installer->retrieve_message());
        }

        $this->add_additional_installers($package_installer->get_additional_installers());

        $step_result = new InstallerStepResult($success, $package_installer->get_message(), $context);
        $this->observers->after_additional_package_install($step_result);
    }

    private function create_folders()
    {
        $this->observers->before_filesystem_prepared();
        $files_path = Path :: get(SYS_FILE_PATH);
        $directories = array('archive', 'garbage', 'repository', 'temp', 'userpictures', 'scorm', 'logs', 'hotpotatoes');

        foreach ($directories as $directory)
        {
            $path = $files_path . $directory;

            if (file_exists($path) && is_dir($path))
            {
                Filesystem :: remove($path);
            }

            if (! Filesystem :: create_dir($path))
            {
                throw new \Exception(Translation :: get('FoldersCreatedFailed'));
            }
        }

        $step_result = new InstallerStepResult(true, Translation :: get('FoldersCreatedSuccess'));
        $this->observers->after_filesystem_prepared($step_result);
    }

}
