<?php
namespace metadata;

use common\libraries\DataClass;
use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\Translation;
use common\libraries\AndCondition;

use user\UserDataManager;

use repository\ContentObject;

/**
 * This class describes a ContentObjectPropertyMetadata data object
 * @author Sven Vanpoucke
 * @author Jens Vanderheyden
 */
class ContentObjectPropertyMetadata extends DataClass
{
    const CLASS_NAME = __CLASS__;
    
    /**
     * ContentObjectPropertyMetadata properties
     */
    const PROPERTY_ELEMENT_ID = 'element_id';
    const PROPERTY_CONTENT_OBJECT_PROPERTY = 'content_object_property';
    const PROPERTY_SOURCE = 'source';
    
    const SOURCE_TEXT = '1';
    const SOURCE_CHAMILO_USER = '2';
    const SOURCE_TIMESTAMP = '3';

    /**
     * Get the default properties
     * @return array The property names.
     */
    static function get_default_property_names($extended_property_names = array())
    {
        return array(self :: PROPERTY_ID, self :: PROPERTY_ELEMENT_ID, self :: PROPERTY_CONTENT_OBJECT_PROPERTY, 
                self :: PROPERTY_SOURCE);
    }

    function get_data_manager()
    {
        return MetadataDataManager :: get_instance();
    }

    /**
     * Returns the element_id of this ContentObjectPropertyMetadata.
     * @return the element_id.
     */
    function get_element_id()
    {
        return $this->get_default_property(self :: PROPERTY_ELEMENT_ID);
    }

    /**
     * Sets the element_id of this ContentObjectPropertyMetadata.
     * @param element_id
     */
    function set_element_id($element_id)
    {
        $this->set_default_property(self :: PROPERTY_ELEMENT_ID, $element_id);
    }

    /**
     * Returns the content_object_property of this ContentObjectPropertyMetadata.
     * @return the content_object_property.
     */
    function get_content_object_property()
    {
        return $this->get_default_property(self :: PROPERTY_CONTENT_OBJECT_PROPERTY);
    }

    /**
     * Sets the content_object_property of this ContentObjectPropertyMetadata.
     * @param content_object_property
     */
    function set_content_object_property($content_object_property)
    {
        $this->set_default_property(self :: PROPERTY_CONTENT_OBJECT_PROPERTY, $content_object_property);
    }

    function get_source()
    {
        return $this->get_default_property(self :: PROPERTY_SOURCE);
    }

    function set_source($source)
    {
        $this->set_default_property(self :: PROPERTY_SOURCE, $source);
    }

    static function get_table_name()
    {
        return Utilities :: camelcase_to_underscores(Utilities :: get_classname_from_namespace(self :: CLASS_NAME));
    }

    /**
     * @param ContentObject $content_object
     * @return string
     */
    function format_content_object_property_according_to_source(ContentObject $content_object)
    {
        $function_name = 'get_' . $this->get_content_object_property();
        if (method_exists(ContentObject :: CLASS_NAME, $function_name))
        {
            $content_object_property = $content_object->$function_name();
            switch ($this->get_source())
            {
                case self :: SOURCE_TEXT :
                    return $content_object_property;
                    break;
                
                case self :: SOURCE_CHAMILO_USER :
                    $udm = UserDataManager :: get_instance();
                    $user = $udm->retrieve_user($content_object_property);
                    return $user->get_firstname() . ' ' . $user->get_lastname();
                    break;
                
                case self :: SOURCE_TIMESTAMP :
                    return date('Ymd', $content_object_property);
                    break;
            }
        }
    }

    /*
     * creates one only if content object property is unique
     */
    function create()
    {
        $condition = new EqualityCondition(ContentObjectPropertyMetadata :: PROPERTY_CONTENT_OBJECT_PROPERTY, $this->get_content_object_property());
        $count = $this->get_data_manager()->count_content_object_property_metadatas($condition);
        
        if ($count >= 1)
        {
            $this->add_error(Translation :: get('ObjectAlreadyExists', array(
                    'OBJECT' => Translation :: get('ContentObjectPropertyMetadata')), Utilities :: COMMON_LIBRARIES));
            return false;
        }
        return parent :: create();
    }

    /*
     * also deletes linked property attribute values
     */
    function delete()
    {
        $condition1 = new EqualityCondition(MetadataAttributeValue :: PROPERTY_PARENT_ID, $this->get_id());
        $condition2 = new EqualityCondition(MetadataAttributeValue :: PROPERTY_RELATION, MetadataAttributeValue :: RELATION_CONTENT_OBJECT_PROPERTY);
        
        $condition = new AndCondition($condition1, $condition2);
        
        $mdm = MetadataManager :: get_instance();
        $metadata_attribute_values = $mdm->retrieve_metadata_attribute_values();
        
        $fails = 0;
        while ($metadata_attribute_value = $metadata_attribute_values->next_result())
        {
            if ($metadata_attribute_value->delete())
            {
                $fails ++;
            }
        }
        
        if ($fails == 0)
            return parent :: delete();
        
        $this->add_error(Translation :: get('ObjectsNotDeleted', array(
                'OBJECT' => Translation :: get('MetadataAttributeValue')), Utilities :: COMMON_LIBRARIES));
        return false;
    }
}

?>