<?php
/**
 * This class describes the form for a MetadataElementValue object.
 * @author Sven Vanpoucke
 * @author Jens Vanderheyden
 **/
namespace metadata;

use common\libraries\FormValidator;
use common\libraries\Translation;
use common\libraries\ResourceManager;
use common\libraries\Path;
use common\libraries\Utilities;
use common\libraries\Request;

class ContentObjectMetadataEditorForm extends MetadataForm
{
    
    const TYPE = 'content_object';
    const OPTION_BLANK = 'blank';
    const PARAM_FIXED = 'fixed';
    
    private $metadata_element_values = array();
    private $user;
    private $content_object_property_metadata_values = array();
    private $attributes = array();
    private $application;
    private $content_object;
    private $metadata_attribute_values = array();
    private $allowed_metadata_attributes = array();

    function __construct($content_object, $metadata_element_values, $content_object_property_metadata_values, $metadata_attribute_values, $allowed_metadata_attributes, $action, $user, $application)
    {
        $this->set_parent_type(self :: TYPE);
        
        parent :: __construct('content_object_metadata_element_value_settings', 'post', $action);
        
        $this->content_object = $content_object;
        $this->content_object_property_metadata_values = $content_object_property_metadata_values;
        $this->user = $user;
        $this->form_type = $form_type;
        $this->application = $application;
        $this->metadata_element_values = $metadata_element_values;
        $this->metadata_attribute_values = $metadata_attribute_values;
        $this->allowed_metadata_attributes = $allowed_metadata_attributes;
        
        $this->get_attributes();
        
        $this->build_creation_form();
        
        $this->setDefaults();
    }

    function build_basic_form()
    {
        
        $this->addElement('hidden', parent :: PARENT_ID);
        
        if (count($this->content_object_property_metadata_values))
        {
            $this->addElement('html', '<h4>' . Translation :: get('Predefined') . '</h4>');
            $this->build_content_object_property_metadata_values();
        }
        
        $this->addElement('html', '<h4>' . Translation :: get('Additional') . '</h4>');
        
        $this->build_empty_element_value();
        
        $this->build_metadata_element_values();
    }

    function build_creation_form()
    {
        $this->build_basic_form();
        
        $buttons[] = $this->createElement('style_submit_button', 'submit', Translation :: get('Create', null, Utilities :: COMMON_LIBRARIES), array(
                'class' => 'positive'));
        $buttons[] = $this->createElement('style_reset_button', 'reset', Translation :: get('Reset', null, Utilities :: COMMON_LIBRARIES), array(
                'class' => 'normal empty'));
        
        $this->addGroup($buttons, 'buttons', null, '&nbsp;', false);
    }

    /**
     * handles creation and updating of metadata elements
     * multiple elements can be updated or created at once
     * creation or update is triggered when form elements are changed
     * 
     * creation of element_values is handled by MetadataForm
     * 
     * triggers
     * --------
     * element values updates   =>  MetadataElementValue :: PROPERTY_VALUE . '_' . $metadata_element_value->get_id() has changed
     * attribute creation       =>  $values[MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE . '_' . MetadataAttributeValue :: PROPERTY_ATTRIBUTE_ID . '_' . $metadata_element_value->get_id() is not left blank
     * attribute update         =>  MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE . '_' . MetadataAttributeValue :: PROPERTY_VALUE . '_' . $metadata_element_value->get_id() . '_' . $metadata_attribute_value->get_id() has changed
     * @return bool succes 
     */
    function edit_metadata()
    {
        $values = $this->exportValues();
        
        $fails = 0;
        
        if (! parent :: edit_metadata())
            $fails ++;
        
     //update existing property values
        foreach ($this->metadata_element_values as $metadata_element_value)
        {
            $cond = $values[MetadataElementValue :: PROPERTY_VALUE . '_' . $metadata_element_value->get_id()] != $this->_defaults[MetadataElementValue :: PROPERTY_VALUE . '_' . $metadata_element_value->get_id()];
            if ($cond)
            {
                $metadata_element_value->set_value($values[MetadataElementValue :: PROPERTY_VALUE . '_' . $metadata_element_value->get_id()]);
                
                if (! $metadata_element_value->update())
                    $fails ++;
            }
            
            // new property attribute values
            if ($values[MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE . '_' . MetadataAttributeValue :: PROPERTY_ATTRIBUTE_ID . '_' . $metadata_element_value->get_id()] != self :: OPTION_BLANK)
            {
                $metadata_attribute_value = new MetadataAttributeValue();
                
                $metadata_attribute_value->set_parent_id($metadata_element_value->get_id(MetadataAttributeValue :: PROPERTY_ATTRIBUTE_ID . '_' . $metadata_element_value->get_id()));
                $metadata_attribute_value->set_attribute_id($values[MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE . '_' . MetadataAttributeValue :: PROPERTY_ATTRIBUTE_ID . '_' . $metadata_element_value->get_id()]);
                $metadata_attribute_value->set_relation(MetadataAttributeValue :: RELATION_PROPERTY_VALUE);
                $metadata_attribute_value->set_value_type($values[MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE . '_' . MetadataAttributeValue :: PROPERTY_VALUE_TYPE . '_' . $metadata_element_value->get_id()]);
                
                $metadata_attribute_value->set_value(Request :: post(MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE . '_' . MetadataAttributeValue :: PROPERTY_VALUE . '_' . $metadata_element_value->get_id()));
                
                if (! $metadata_attribute_value->create())
                    $fails ++;
            }
            
            //update existing attribute values
            foreach ($this->metadata_attribute_values[MetadataAttributeValue :: RELATION_PROPERTY_VALUE][$metadata_element_value->get_id()] as $id => $metadata_attribute_value)
            {
                //check if changes occurred
                if ($metadata_attribute_value->get_value() != $values[MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE . '_' . MetadataAttributeValue :: PROPERTY_VALUE . '_' . $metadata_element_value->get_id() . '_' . $metadata_attribute_value->get_id()])
                {
                    $metadata_attribute_value->set_value($values[MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE . '_' . MetadataAttributeValue :: PROPERTY_VALUE . '_' . $metadata_element_value->get_id() . '_' . $metadata_attribute_value->get_id()]);
                    $metadata_attribute_value->update();
                }
            }
        }
        
        //update existing content object property metadata attributes
        foreach ($this->content_object_property_metadata_values as $id => $content_object_property_metadata)
        {
            foreach ($this->metadata_attribute_values[MetadataAttributeValue :: RELATION_CONTENT_OBJECT_PROPERTY][$content_object_property_metadata->get_id()] as $id => $metadata_attribute_value)
            {
                $prop = $content_object_property_metadata->get_content_object_property();
                $val = $metadata_attribute_value->get_value();
                
                //check if changes occurred
                if ($metadata_attribute_value->get_value() != $values[$content_object_property_metadata->get_content_object_property() . '_' . MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE . '_' . MetadataAttributeValue :: PROPERTY_VALUE . '_' . $metadata_attribute_value->get_id()])
                {
                    $metadata_attribute_value->set_value($values[$content_object_property_metadata->get_id() . '_' . MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE . '_' . MetadataAttributeValue :: PROPERTY_VALUE . '_' . $metadata_element_value->get_id() . '_' . $metadata_attribute_value->get_id()]);
                    $metadata_attribute_value->update();
                }
            }
        }
        
        //create content object metadata_attribute_values
        foreach ($this->content_object_property_metadata_values as $id => $content_object_property)
        {
            //create new one if an attribute type is selected
            if ($values[$content_object_property->get_content_object_property() . '_' . MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE . '_' . MetadataAttributeValue :: PROPERTY_ATTRIBUTE_ID] != self :: OPTION_BLANK)
            {
                //do not continue if no value type is selected
                if ($values[$content_object_property->get_content_object_property() . '_' . MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE . '_' . MetadataAttributeValue :: PROPERTY_VALUE_TYPE] != self :: OPTION_BLANK)
                {
                    $metadata_attribute_value = new MetadataAttributeValue();
                    
                    //id of the content object property metadata
                    $metadata_attribute_value->set_parent_id($content_object_property->get_id());
                    //content_object id
                    $metadata_attribute_value->set_content_object_id($this->content_object->get_id());
                    
                    $metadata_attribute_value->set_attribute_id($values[$content_object_property->get_content_object_property() . '_' . MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE . '_' . MetadataAttributeValue :: PROPERTY_ATTRIBUTE_ID]);
                    $metadata_attribute_value->set_relation(MetadataAttributeValue :: RELATION_CONTENT_OBJECT_PROPERTY);
                    $metadata_attribute_value->set_value_type($values[$content_object_property->get_content_object_property() . '_' . MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE . '_' . MetadataAttributeValue :: PROPERTY_VALUE_TYPE]);
                    
                    $appendix = ($metadata_attribute_value->get_value_type() == MetadataAttributeValue :: VALUE_TYPE_ID) ? '_2' : '';
                    $metadata_attribute_value->set_value($values[$content_object_property->get_content_object_property() . '_' . MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE . '_' . MetadataAttributeValue :: PROPERTY_VALUE . $appendix]);
                    
                    if (! $metadata_attribute_value->create())
                        $fails ++;
                }
            }
        }
        return ($fails) ? false : true;
    }

    /**
     * Sets default values.
     * @param array $defaults Default values for this form's parameters.
     */
    function setDefaults($defaults = array ())
    {
        $defaults[parent :: PARENT_ID] = $this->content_object->get_id();
        
        //content object attribute values
        foreach ($this->content_object_property_metadata_values as $id => $content_object_property)
        {
            foreach ($this->metadata_attribute_values[MetadataAttributeValue :: RELATION_CONTENT_OBJECT_PROPERTY][$content_object_property->get_id()] as $id => $metadata_attribute_value)
            {
                $name = $this->content_object_property_metadata_values[$metadata_attribute_value->get_parent_id()]->get_content_object_property() . '_' . MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE . '_' . MetadataAttributeValue :: PROPERTY_VALUE . '_' . $id;
                $defaults[$this->content_object_property_metadata_values[$metadata_attribute_value->get_parent_id()]->get_content_object_property() . '_' . MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE . '_' . MetadataAttributeValue :: PROPERTY_VALUE . '_' . $id] = $metadata_attribute_value->get_value();
            }
        }
        
        //metadata element values
        foreach ($this->metadata_element_values as $metadata_element_value)
        {
            $defaults[MetadataElementValue :: PROPERTY_ELEMENT_ID . '_' . $metadata_element_value->get_id()] = $metadata_element_value->get_element_id();
            $defaults[MetadataElementValue :: PROPERTY_VALUE . '_' . $metadata_element_value->get_id()] = $metadata_element_value->get_value();
            
            foreach ($this->metadata_attribute_values[MetadataAttributeValue :: RELATION_PROPERTY_VALUE][$metadata_element_value->get_id()] as $id => $metadata_attribute_value)
            {
                $defaults[MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE . '_' . MetadataAttributeValue :: PROPERTY_VALUE . '_' . $metadata_element_value->get_id() . '_' . $metadata_attribute_value->get_id()] = $metadata_attribute_value->get_value();
            }
        }
        parent :: setDefaults($defaults);
    }

    //creates output of content_object properties that are assigned to metadata elements by default
    function build_content_object_property_metadata_values()
    {
        foreach ($this->content_object_property_metadata_values as $id => $content_object_property)
        {
            $function_name = 'get_' . $content_object_property->get_content_object_property();
            
            $elements = $this->get_elements();
            
            $this->addElement('category', Translation :: get('ElementValue'));
            $this->addElement('static', '', $elements[$content_object_property->get_element_id()], $content_object_property->format_content_object_property_according_to_source($this->content_object));
            
            $allowed_elements = $this->filter_allowed_attributes($content_object_property->get_element_id());
            
            //if there are any allowed attributes for this element value
            if (isset($this->allowed_metadata_attributes[$content_object_property->get_element_id()]))
            {
                //new empty attribute_value
                $group = array();
                
                $group[] = $this->createElement('select', $content_object_property->get_content_object_property() . '_' . MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE . '_' . MetadataAttributeValue :: PROPERTY_ATTRIBUTE_ID, Translation :: get('NsPrefix'), $allowed_elements);
                $group[] = $this->createElement('select', $content_object_property->get_content_object_property() . '_' . MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE . '_' . MetadataAttributeValue :: PROPERTY_VALUE_TYPE, Translation :: get('ValueType'), $this->get_value_types());
                $group[] = $this->createElement('text', $content_object_property->get_content_object_property() . '_' . MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE . '_' . MetadataAttributeValue :: PROPERTY_VALUE, Translation :: get('Value'));
                $group[] = $this->createElement('select', $content_object_property->get_content_object_property() . '_' . MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE . '_' . MetadataAttributeValue :: PROPERTY_VALUE . '_2', Translation :: get('Value'), $this->attributes);
                
                $this->addGroup($group, '', Translation :: get('Add') . ' ' . Translation :: get('AttributeValue'));
            }
            //existing attribute_values
            $this->build_content_object_property_metadata_attribute_values($content_object_property);
            
            $this->addElement('category');
        
        }
    }

    function get_value_types()
    {
        $value_types = array();
        $value_types[MetadataAttributeValue :: VALUE_TYPE_NONE] = '--';
        $value_types[MetadataAttributeValue :: VALUE_TYPE_ID] = Translation :: get('id');
        $value_types[MetadataAttributeValue :: VALUE_TYPE_VALUE] = Translation :: get('value');
        
        return $value_types;
    }

    function build_metadata_element_values()
    {
        
        foreach ($this->metadata_element_values as $metadata_element_value)
        {
            $this->addElement('category', Translation :: get('ElementValue'));
            
            $group = array();
            
            $group[] = $this->createElement('text', MetadataElementValue :: PROPERTY_VALUE . '_' . $metadata_element_value->get_id(), Translation :: get('ElementValue'), array(
                    'size' => '50'));
            
            $group[] = $this->createElement('static', null, null, '<a href="' . $this->application->get_url(array(
                    MetadataManager :: PARAM_ACTION => MetadataManager :: ACTION_DELETE_CONTENT_OBJECT_METADATA_PROPERTY_VALUE, 
                    MetadataManager :: PARAM_METADATA_PROPERTY_VALUE => $metadata_element_value->get_id(), 
                    MetadataManager :: PARAM_CONTENT_OBJECT => $this->content_object->get_id())) . '">delete</a>');
            
            $elements = $this->get_elements();
            
            $this->addGroup($group, '', $elements[$metadata_element_value->get_element_id()]);
            
            if (isset($this->allowed_metadata_attributes[$metadata_element_value->get_element_id()]))
            {
                $group = array();
                
                $group[] = $this->createElement('select', MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE . '_' . MetadataAttributeValue :: PROPERTY_ATTRIBUTE_ID . '_' . $metadata_element_value->get_id(), Translation :: get('Attribute'), $this->filter_allowed_attributes($metadata_element_value->get_element_id()));
                $group[] = $this->createElement('select', MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE . '_' . MetadataAttributeValue :: PROPERTY_VALUE_TYPE . '_' . $metadata_element_value->get_id(), Translation :: get('ValueType'), $this->get_value_types(), array(
                        'class' => 'attribute_value', 'id' => $metadata_element_value->get_id()));
                
                $this->addGroup($group, '', Translation :: get('Add') . ' ' . Translation :: get('AttributeValue'));
                
                $this->addElement('html', ResourceManager :: get_instance()->get_resource_html(Path :: get(WEB_PATH) . 'metadata/resources/javascript/format_metadata_attribute_value.js'));
            }
            
            $this->build_metadata_attribute_values($metadata_element_value);
            
            $this->addElement('category');
        }
    }

    // builds elements for attributes linked to default content object properties
    function build_content_object_property_metadata_attribute_values(ContentObjectPropertyMetadata $content_object_property)
    {
        foreach ($this->metadata_attribute_values[MetadataAttributeValue :: RELATION_CONTENT_OBJECT_PROPERTY][$content_object_property->get_id()] as $id => $metadata_attribute_value)
        {
            $group = array();
            
            //displays a dropdown or text field depanding on value type
            if ($metadata_attribute_value->get_value_type() == MetadataAttributeValue :: VALUE_TYPE_VALUE)
            {
                $group[] = $this->createElement('text', $content_object_property->get_content_object_property() . '_' . MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE . '_' . MetadataAttributeValue :: PROPERTY_VALUE . '_' . $id, Translation :: get('Attribute'));
            }
            else
            {
                $group[] = $this->createElement('select', $content_object_property->get_content_object_property() . '_' . MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE . '_' . MetadataAttributeValue :: PROPERTY_VALUE . '_' . $id, Translation :: get('Attribute'), $this->attributes);
            }
            $group[] = $this->createElement('static', null, null, '<a href="' . $this->application->get_url(array(
                    MetadataManager :: PARAM_ACTION => MetadataManager :: ACTION_DELETE_CONTENT_OBJECT_METADATA_ATTRIBUTE_VALUE, 
                    MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE => $metadata_attribute_value->get_id(), 
                    MetadataManager :: PARAM_CONTENT_OBJECT => $this->content_object->get_id())) . '">delete</a>');
            
            $this->addGroup($group, '', $this->attributes[$metadata_attribute_value->get_attribute_id()]);
        
        }
    }

    function build_metadata_attribute_values($metadata_element_value)
    {
        foreach ($this->metadata_attribute_values[MetadataAttributeValue :: RELATION_PROPERTY_VALUE][$metadata_element_value->get_id()] as $n => $metadata_attribute_value)
        {
            if ($metadata_attribute_value->get_relation() == MetadataAttributeValue :: RELATION_PROPERTY_VALUE)
            {
                $group = array();
                
                //displays a dropdown or text field depanding on value type
                if ($metadata_attribute_value->get_value_type() == MetadataAttributeValue :: VALUE_TYPE_VALUE)
                {
                    $group[] = $this->createElement('text', MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE . '_' . MetadataAttributeValue :: PROPERTY_VALUE . '_' . $metadata_element_value->get_id() . '_' . $metadata_attribute_value->get_id(), Translation :: get('Value'));
                }
                else
                {
                    $group[] = $this->createElement('select', MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE . '_' . MetadataAttributeValue :: PROPERTY_VALUE . '_' . $metadata_element_value->get_id() . '_' . $metadata_attribute_value->get_id(), Translation :: get('Value'), $this->attributes);
                }
                $group[] = $this->createElement('static', null, null, '<a href="' . $this->application->get_url(array(
                        MetadataManager :: PARAM_ACTION => MetadataManager :: ACTION_DELETE_CONTENT_OBJECT_METADATA_ATTRIBUTE_VALUE, 
                        MetadataManager :: PARAM_METADATA_ATTRIBUTE_VALUE => $metadata_attribute_value->get_id(), 
                        MetadataManager :: PARAM_CONTENT_OBJECT => $this->content_object->get_id())) . '">delete</a>');
                
                $this->addGroup($group, '', $this->attributes[$metadata_attribute_value->get_attribute_id()]);
            }
        }
    }

    function get_attributes()
    {
        $attributes = $this->application->retrieve_metadata_attributes();
        
        while ($attribute = $attributes->next_result())
        {
            $this->attributes[$attribute->get_id()] = $attribute->render_name();
        }
    }

    /**
     * determines which attributes belong to a certain element
     * @param int $element_id
     * @return array allowedTypes
     */
    function filter_allowed_attributes($element_id)
    {
        $allowedTypes = array();
        $allowedTypes[self :: OPTION_BLANK] = '--';
        
        foreach ($this->allowed_metadata_attributes[$element_id] as $n => $allowed)
        {
            $allowedTypes[$allowed] = $this->attributes[$allowed];
        }
        return $allowedTypes;
    }
}
?>