<?php
namespace metadata;
use common\libraries\DataClass;
use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\OrCondition;
use common\libraries\Translation;
/**
 * This class describes a MetadataElement data object
 * @author Jens Vanderheyden
 */
class MetadataElement extends DataClass
{
    const CLASS_NAME = __CLASS__;
    
    /**
     * MetadataElement properties
     */
    const PROPERTY_ID = 'id';
    const PROPERTY_NAMESPACE = 'namespace';
    const PROPERTY_NS_PREFIX = 'ns_prefix';
    const PROPERTY_NAME = 'name';
    const PROPERTY_ORDER = 'display_order';
    
    private $ns_prefix = false;

    /**
     * Get the default properties
     * @return array The property names.
     */
    static function get_default_property_names()
    {
        return array(self :: PROPERTY_ID, self :: PROPERTY_NAMESPACE, self :: PROPERTY_NAME, self :: PROPERTY_ORDER);
    }

    function get_data_manager()
    {
        return MetadataDataManager :: get_instance();
    }

    /**
     * Returns the id of this MetadataElement.
     * @return the id.
     */
    function get_id()
    {
        return $this->get_default_property(self :: PROPERTY_ID);
    }

    /**
     * Sets the id of this MetadataElement.
     * @param id
     */
    function set_id($id)
    {
        $this->set_default_property(self :: PROPERTY_ID, $id);
    }

    /**
     * Returns the ns_prefix of this MetadataElement.
     * @return the namespace_id.
     */
    function get_namespace()
    {
        return $this->get_default_property(self :: PROPERTY_NAMESPACE);
    }

    /**
     * Sets the ns_prefix of this MetadataElement.
     * @param namespace
     */
    function set_namespace($namespace)
    {
        $this->set_default_property(self :: PROPERTY_NAMESPACE, $namespace);
    }

    /**
     * Returns the ns_prefix of this MetadataElement.
     * @return the ns_prefix.
     */
    function get_ns_prefix()
    {
        if (! $this->ns_prefix)
        {
            $namespace = $this->get_data_manager()->retrieve_metadata_namespace($this->get_namespace());
            
            if (! $namespace)
            {
                return false;
            }
            $this->set_ns_prefix($namespace->get_ns_prefix());
        }
        return $this->ns_prefix;
    }

    /**
     * Sets the ns_prefix of this MetadataElement.
     * @param ns_prefix
     */
    function set_ns_prefix($ns_prefix)
    {
        $this->ns_prefix = $ns_prefix;
    }

    /**
     * Returns the value of this MetadataElement.
     * @return the value.
     */
    function get_name()
    {
        return $this->get_default_property(self :: PROPERTY_NAME);
    }

    /**
     * Sets the value of this MetadataElement.
     * @param value
     */
    function set_name($name)
    {
        $this->set_default_property(self :: PROPERTY_NAME, $name);
    }

    /**
     * Returns the order of this MetadataElement.
     * @return the order.
     */
    function get_order()
    {
        return $this->get_default_property(self :: PROPERTY_ORDER);
    }

    /**
     * Sets the order of this MetadataElement.
     * @param order
     */
    function set_order($order)
    {
        $this->set_default_property(self :: PROPERTY_ORDER, $order);
    }

    static function get_table_name()
    {
        return Utilities :: camelcase_to_underscores(Utilities :: get_classname_from_namespace(self :: CLASS_NAME));
    }

    /*
     * object creation if name + ns_prfix are unique -> unique key is on namespace and name in db
     * order is set according to no of element with same namespace
     * return MetadaElement or false
     */
    function create()
    {
        //set order
        $condition = new EqualityCondition(MetadataElement :: PROPERTY_NAMESPACE, $this->get_namespace());
        $count = $this->get_data_manager()->count_metadata_elements($condition);
        $this->set_order($count + 1);
        
        return parent :: create();
    }

    function delete()
    {
        //check dependencies before deleting
        //if any element_values delete is not possible
        $mdm = $this->get_data_manager();
        
        $condition = new EqualityCondition(MetadataElementValue :: PROPERTY_ELEMENT_ID, $this->get_id());
        
        $count = $mdm->count_content_object_metadata_element_values($condition);
        $count += $mdm->count_user_metadata_element_values($condition);
        
        if ($count === 0)
        {
            if (parent :: delete())
            {
                //recount order of all elements in namespace
                $condition = new EqualityCondition(self :: PROPERTY_NAMESPACE, $this->get_namespace());
                
                $elements_retrieved = $this->get_data_manager()->retrieve_metadata_elements($condition);
                
                $i = 0;
                while ($element_retrieved = $elements_retrieved->next_result())
                {
                    $i ++;
                    $element_retrieved->set_order($i);
                    $element_retrieved->update();
                }
                
                //delete associations
                $condition1 = new EqualityCondition(MetadataElementNesting :: PROPERTY_PARENT_ID, $this->get_id());
                
                $condition2 = new EqualityCondition(MetadataElementNesting :: PROPERTY_CHILD_ID, $this->get_id());
                $condition3 = new EqualityCondition(MetadataElementNesting :: PROPERTY_CHILD_TYPE, Utilities :: camelcase_to_underscores(self :: CLASS_NAME));
                $condition4 = new AndCondition($condition2, $condition3);
                
                $condition = new OrCondition($condition1, $condition4);
                
                $metadata_element_nestings = $mdm->retrieve_metadata_element_nestings($condition);
                
                while ($metadata_element_nesting = $metadata_element_nestings->next_result())
                {
                    $metadata_element_nesting->delete();
                }
                
                $condition1 = new EqualityCondition(MetadataAttributeAssociation :: PROPERTY_PARENT_ID, $this->get_id());
                
                $condition2 = new EqualityCondition(MetadataAttributeAssociation :: PROPERTY_CHILD_ID, $this->get_id());
                $condition3 = new EqualityCondition(MetadataAttributeAssociation :: PROPERTY_CHILD_TYPE, Utilities :: camelcase_to_underscores(self :: CLASS_NAME));
                $condition4 = new AndCondition($condition2, $condition3);
                
                $condition = new OrCondition($condition1, $condition4);
                
                $metadata_attribute_associations = $mdm->retrieve_metadata_attribute_associations($condition);
                
                while ($metadata_attribute_association = $metadata_attribute_associations->next_result())
                {
                    $metadata_attribute_association->delete();
                }
                
                //content_object_property_metadata
                $condition = new EqualityCondition(ContentObjectPropertyMetadata :: PROPERTY_ELEMENT_ID, $this->get_id());
                $content_object_property_metadatas = $mdm->retrieve_content_object_property_metadatas($condition);
                
                while ($content_object_property_metadata = $content_object_property_metadatas->next_result())
                {
                    $content_object_property_metadata->delete();
                }
                
                //default_values
                $condition = new EqualityCondition(MetadataDefaultValue :: PROPERTY_ELEMENT_ID, $this->get_id());
                $metadata_default_values = $mdm->retrieve_metadata_default_values($condition);
                
                while ($metadata_default_value = $metadata_default_values->next_result())
                {
                    $metadata_default_value->delete();
                }
            }
        }
        else
        {
            $this->add_error(Translation :: get('ChildrenExist'));
            return false;
        }
        return true;
    }

    /*
     * @return string ns_prefix + name
     */
    function render_name()
    {
        $pref = $this->get_ns_prefix();
        $prefix = (empty($pref)) ? '' : $this->get_ns_prefix() . ':';
        return $prefix . $this->get_name();
    }

    function move($direction)
    {
        $new_order = $this->get_order() + $direction;
        
        //get item that changes
        $condition = new EqualityCondition(self :: PROPERTY_ORDER, $new_order);
        $elements_result = $this->get_data_manager()->retrieve_metadata_elements($condition);
        
        //there should be exactly one result
        if (! $elements_result->size() != 1)
        {
            //give order of ou element
            $the_element = $elements_result->next_result();
            $the_element->set_order($this->get_order());
            if ($the_element->update())
            {
                $this->set_order($new_order);
                
                if ($this->update())
                    return true;
            }
        }
        return false;
    }
}

?>