<?php

namespace migration;

use common\libraries\Translation;
use common\libraries\Filesystem;
use common\libraries\Utilities;
use common\libraries\Path;
use common\libraries\MessageLogger;
use Exception;

/**
 * Classes can extend this class to create functionality that should run after the migration of the selected blocks
 * This can be used to fix data in batch afterwards due to performance issues.
 */
abstract class MigrationPostProcess extends MigrationElement
{
    /**
     * Processes the functionality
     */
    function process()
    {
	$this->prepare();
	$this->run();
	$this->finish();
    }

    /**
     * Prepares the process
     * Logging, Timing
     */
    function prepare()
    {
	$this->get_timer()->start();
        $logger = $this->get_file_logger();

        $this->log_message_and_file(Translation :: get('StartPostProcessing', array(
                'NAME' => $this->get_name())), MessageLogger :: TYPE_CONFIRM);
    }

    /**
     * Finishes the process
     * Logging, Timing
     */
    function finish()
    {
	$logger = $this->get_file_logger();
        $this->get_timer()->stop();

        $this->log_message_and_file(Translation :: get('PostProcessingComplete', array(
                'NAME' => $this->get_name(), 'TIME' => $this->get_timer()->get_time_in_hours())),
		MessageLogger :: TYPE_CONFIRM);

        $logger->close_file();
    }

    // Messaging

    function get_file_logger_path()
    {
	return Path :: get(SYS_FILE_PATH) . '/logs/migration/post_process/';
    }

    function get_file_logger_name()
    {
	return $this->get_name();
    }

    /**
     * Factory method for migration post process
     * @param String $platform - the selected platform
     * @param String $name - the name
     * @return MigrationPostProcess
     */
    function factory($platform, $name)
    {
        $file = dirname(__FILE__) . '/platform/' . $platform . '/post_process/' . $name . '_post_process.class.php';
        if (! file_exists($file))
        {
            throw new Exception(Translation :: get('CanNotFindPostProcess', array(
                    'PLATFORM' => $platform,
                    'NAME' => $name)));
        }

        require_once ($file);

        $class = __NAMESPACE__ . '\\' . Utilities :: underscores_to_camelcase($name) . 'PostProcess';

        return new $class();
    }

    static function get_post_processors($platform)
    {
	$dir = dirname(__FILE__) . '/platform/' . $platform . '/post_process/';
	if(!is_dir($dir))
	{
	    return array();
	}

	$list = array();

	$files = Filesystem :: get_directory_content($dir, Filesystem :: LIST_FILES, false);
	foreach($files as $file)
	{
	    $file_name = basename($file);
	    $list[] = substr($file_name, 0, -23);
	}

	return $list;
	
    }

    /**
     * Runs the post processing script
     */
    abstract function run();

    /**
     * Returns the name for the post processing script
     */
    abstract function get_name();
}

?>
