<?php

namespace migration;

use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\Text;
use common\libraries\Translation;
use repository\RepositoryDataManager;
use common\libraries\Utilities;
use application\weblcms\WeblcmsDataManager;
use application\weblcms\ContentObjectPublicationCategory;
use common\libraries\ObjectTableOrder;

/**
 * $Id: dokeos185_document.class.php 221 2009-11-13 14:36:41Z vanpouckesven $
 * @package migration.platform.dokeos185
 */
require_once dirname(__FILE__) . '/../dokeos185_course_data_migration_data_class.class.php';
require_once dirname(__FILE__) . '/dokeos185_group.class.php';

/**
 * This class represents an old Dokeos 1.8.5 document
 *
 * @author David Van Wayenbergh
 */
class Dokeos185Document extends Dokeos185CourseDataMigrationDataClass
{
    /**
     * document properties
     */
    const CLASS_NAME = __CLASS__;
    const TABLE_NAME = 'document';

    const PROPERTY_ID = 'id';
    const PROPERTY_PATH = 'path';
    const PROPERTY_TITLE = 'title';
    const PROPERTY_SIZE = 'size';
    const PROPERTY_COMMENT = 'comment';
    const PROPERTY_FILETYPE = 'filetype';

    const CACHE_PROPERTY_ID = 'id';
    const CACHE_PROPERTY_VISIBLE = 'visible';

    static $i = 0;
    private static $category_cache; // holds the migrated categories for a single course
    private static $course_code;
    private $directory;
    /**
     * Default properties of the document object, stored in an associative
     * array.
     */
    private $defaultProperties;

    /**
     * Creates a new document object.
     * @param array $defaultProperties The default properties of the document
     *                                 object. Associative array.
     */
    function __construct($defaultProperties = array())
    {
	$this->defaultProperties = $defaultProperties;
    }

    /**
     * Gets a default property of this document object by name.
     * @param string $name The name of the property.
     */
    function get_default_property($name)
    {
	return $this->defaultProperties[$name];
    }

    /**
     * Gets the default properties of this document.
     * @return array An associative array containing the properties.
     */
    function get_default_properties()
    {
	return $this->defaultProperties;
    }

    /**
     * Sets the default properties of this class
     */
    function set_default_properties($defaultProperties)
    {
	$this->defaultProperties = $defaultProperties;
    }

    /**
     * Get the default properties of all documents.
     * @return array The property names.
     */
    static function get_default_property_names()
    {
	return array(self :: PROPERTY_ID, self :: PROPERTY_PATH, self :: PROPERTY_TITLE, self :: PROPERTY_SIZE, self :: PROPERTY_COMMENT, self :: PROPERTY_FILETYPE);
    }

    /**
     * Sets a default property of this document by name.
     * @param string $name The name of the property.
     * @param mixed $value The new value for the property.
     */
    function set_default_property($name, $value)
    {
	$this->defaultProperties[$name] = $value;
    }

    /**
     * Checks if the given identifier is the name of a default document
     * property.
     * @param string $name The identifier.
     * @return boolean True if the identifier is a property name, false
     *                 otherwise.
     */
    static function is_default_property_name($name)
    {
	return in_array($name, self :: get_default_property_names());
    }

    /**
     * Returns the id of this document.
     * @return int The id.
     */
    function get_id()
    {
	return $this->get_default_property(self :: PROPERTY_ID);
    }

    /**
     * Returns the path of this document.
     * @return String The path.
     */
    function get_path()
    {
	return $this->get_default_property(self :: PROPERTY_PATH);
    }

    /**
     * Returns the title of this document.
     * @return String The title.
     */
    function get_title()
    {
	return $this->get_default_property(self :: PROPERTY_TITLE);
    }

    /**
     * Returns the size of this document.
     * @return int The size.
     */
    function get_size()
    {
	return $this->get_default_property(self :: PROPERTY_SIZE);
    }

    /**
     * Returns the comment of this document.
     * @return String The comment.
     */
    function get_comment()
    {
	return $this->get_default_property(self :: PROPERTY_COMMENT);
    }

    /**
     * Returns the filetype of this document.
     * @return String The filetype.
     */
    function get_filetype()
    {
	return $this->get_default_property(self :: PROPERTY_FILETYPE);
    }

    /**
     * Check if the document is valid
     * @param Course $course the course of the document
     * @return true if the dropbox category is valid
     */
    function is_valid()
    {
	$course = $this->get_course();
	$this->set_item_property($this->get_data_manager()->get_item_property($course, 'document', $this->get_id()));

	//due to problems with the character encoding:
	//$this->set_default_property(self :: PROPERTY_PATH, utf8_decode($this->get_path()));


	$pos = strrpos($this->get_path(), '/');

	$filename = substr($this->get_path(), $pos);
	$old_path = substr($this->get_path(), 0, $pos);
	unset($pos);

	$old_rel_path = 'courses/' . $course->get_directory() . '/document/' . $old_path . '/';
	unset($old_path);

	$this->directory = $this->get_data_manager()->get_sys_path() . $old_rel_path;
	$path = $this->directory . $filename;
	$path_exists = file_exists($path) || is_dir($path);

	if (!$path_exists)
	{
	    $path = mb_convert_encoding($this->directory . $filename, 'UTF-8', 'Windows-1252');
	    $path_exists = file_exists($path) || is_dir($path);

	    if (!$path_exists)
	    {
		$path = utf8_encode($this->directory . $filename);
		$path_exists = file_exists($path) || is_dir($path);

		if ($path_exists)
		{
		    $this->set_default_property(self :: PROPERTY_PATH, utf8_encode($this->get_path()));
		    $this->directory = utf8_encode($this->directory);
		}
	    }
	    else
	    {

		$this->set_default_property(self :: PROPERTY_PATH, mb_convert_encoding(
				$this->get_path(), 'UTF-8', 'Windows-1252'));
		$this->directory = mb_convert_encoding(
				$this->directory, 'UTF-8', 'Windows-1252');
	    }
	}

	if (!$this->get_id() || !$this->get_path() || !$this->get_filetype() || !$this->get_item_property() || !$this->get_item_property()->get_ref() || !
		$this->get_item_property()->get_insert_date() || !$path_exists)
	{
	    if (!$path_exists)
	    {
		$this->set_message(Translation :: get('PathDoesNotExistMessage', array('TYPE' => 'document', 'ID' => $this->get_id(), 'ORIGINALPATH' => $this->get_path(), 'CONVERTEDPATH' => $path)));
	    }
	    else
	    {
		$this->set_message(Translation :: get('GeneralInvalidMessage', array('TYPE' => 'document', 'ID' => $this->get_id())));
	    }
	    $this->create_failed_element($this->get_id());
	    return false;
	}

	unset($old_rel_path);
	unset($filename);
	unset($course);
	return true;
    }

    /**
     * Convert to new document/folder
     * @return the new document/folder
     */
    function convert_data()
    {
	if ($this->get_filetype() == 'folder')
	{
	    return $this->process_folder();
	}
	else
	{
	    return $this->process_file();
	}
    }

    private function process_folder()
    {
	//check the course for folder caching
	if ($this->get_course()->get_code() != self :: $course_code)
	{
	    self :: $category_cache = array(); // reset the cache
	    self :: $course_code = $this->get_course()->get_code();
	}

	$new_course_code = $this->get_id_reference(self :: $course_code, 'main_database.course');

	$pos = strrpos($this->get_path(), '/');

	//needed for retrieval through category_cache
	$new_folder = substr($this->get_path(), $pos);
	$parent_path = substr($this->get_path(), 0, $pos);

	if (empty($parent_path))
	{
	    $parent_id = 0;
	    $parent_visible = 1;
	}
	else
	{
	    if (array_key_exists($parent_path, self :: $category_cache))
	    {
		$parent_id = (self :: $category_cache[$parent_path][self :: CACHE_PROPERTY_ID]);
		$parent_visible = self :: $category_cache[$parent_path][self :: CACHE_PROPERTY_VISIBLE];
	    }
	    else
	    {
		$this->create_failed_element($this->get_id());
		$this->set_message(Translation :: get('CouldNotFindParentPath', array('ID' => $this->get_id(), 'PATH' => $parent_path)));
		return false;
	    }
	}

	//Check for groupdocs and put them in seperate category

	$title = $this->get_title() ? $this->get_title() : $new_folder;
	if (strpos($title, 'groupdocs') !== false)
	{
	    //Retrieve groupdocs from category cache
	    if (array_key_exists('groupdocs', self :: $category_cache))
	    {
		$parent_id = (self :: $category_cache['groupdocs'][self :: CACHE_PROPERTY_ID]);
		$parent_visible = 1;
	    }
	    else
	    {
		//Create groupdocs category
		$category = new ContentObjectPublicationCategory();

		$category->set_name(Translation :: get('GroupDocs'));
		$category->set_course($new_course_code);
		$category->set_tool('document');
		$category->set_parent(0);

		//Create category in database
		$category->create();

		$parent_id = $category->get_id();
		$parent_visible = 1;

		self :: $category_cache['groupdocs'][self :: CACHE_PROPERTY_ID] = $parent_id;
		self :: $category_cache['groupdocs'][self :: CACHE_PROPERTY_VISIBLE] = $parent_visible;
	    }
	}

	$category = new ContentObjectPublicationCategory();

	$category->set_name($title);
	$category->set_course($new_course_code);
	$category->set_tool('document');

	if (!$parent_id)
	{
	    $parent_id = 0;
	}

	$category->set_parent($parent_id);

	//Create category in database
	if (!$category->create())
	{
	    $this->create_failed_element($this->get_id());
	    $this->set_message(Translation :: get('CouldNotCreateCategory', array('ID' => $this->get_id())));
	    return false;
	}

	$category_id = $category->get_id();
	self :: $category_cache[$this->get_path()][self :: CACHE_PROPERTY_ID] = $category_id;

	if ($parent_visible == 0)
	{
	    $visible = 0;
	}
	else
	{
	    if ($this->get_item_property())
	    {
		$visible = ($this->get_item_property()->get_visibility() == 1);
	    }
	    else
	    {
		$visible = 1;
	    }
	}

	self :: $category_cache[$this->get_path()][self :: CACHE_PROPERTY_ID] = $category_id;
	self :: $category_cache[$this->get_path()][self :: CACHE_PROPERTY_VISIBLE] = $visible;

	$this->set_message(Translation :: get('GeneralConvertedMessage', array('TYPE' => 'category', 'OLD_ID' => $this->get_id(), 'NEW_ID' => $category_id)));
	return true;
    }

    private function process_file()
    {
	$course = $this->get_course();

	$new_user_id = $this->get_id_reference($this->get_item_property()->get_insert_user_id(), 'main_database.user');
	$new_course_code = $this->get_id_reference($course->get_code(), 'main_database.course');

	$all_item_properties = $this->get_data_manager()->get_item_properties($this->get_course(), 'document', $this->get_id());

	while ($item_property = $all_item_properties->next_result())
	{
	    $to_group_id = $this->get_id_reference($item_property->get_to_group_id(), $this->get_database_name() . '.' . Dokeos185Group::get_table_name());
	    if ($to_group_id)
	    {
		$new_to_group_id[] = $to_group_id;
	    }
	    $to_user_id = $this->get_id_reference($item_property->get_to_user_id(), 'main_database.user');
	    if ($to_user_id)
	    {
		$new_to_user_id[] = $to_user_id;
	    }
	}

	if (!$new_user_id)
	{
	    $new_user_id = $this->get_data_manager()->get_owner_id($new_course_code);
	}

	$filename_split = split('/', $this->get_path());
	$original_filename = $filename_split[count($filename_split) - 1];

	$converted_filename = ($original_filename);
	$converted_directory = ($this->directory);

	$chamilo_repository_document = $this->migrate_file_and_create_document($converted_directory, $converted_filename, $new_user_id);

	if ($chamilo_repository_document)
	{
	    if (!$chamilo_repository_document->get_id())
	    {
		if ($this->get_title())
		{
		    $chamilo_repository_document->set_title($this->get_title());
		}
		else
		{
		    $chamilo_repository_document->set_title($original_filename);
		}

		if ($this->get_comment())
		{
		    $chamilo_repository_document->set_description($this->get_comment());
		}
		else
		{
		    $chamilo_repository_document->set_description($chamilo_repository_document->get_title());
		}

		$chamilo_repository_document->set_creation_date(strtotime($this->get_item_property()->get_insert_date()));
		$chamilo_repository_document->set_modification_date(strtotime($this->get_item_property()->get_lastedit_date()));

		$repository_category_course_id = $this->get_repository_migration_course_category_id($new_user_id);
		$documents_category_id = RepositoryDataManager :: get_repository_category_by_name_or_create_new($new_user_id, Translation :: get('Documents'), $repository_category_course_id);

		$chamilo_repository_document->set_parent_id($documents_category_id);

		if ($this->get_item_property()->get_visibility() == 2)
		{
		    $chamilo_repository_document->set_state(1);
		}

		//Create document in db
		if (!$chamilo_repository_document->create())
		{
		    $this->create_failed_element($this->get_id());
		    $this->set_message(Translation :: get('ContentObjectCreationFailedMessage', array('TYPE' => 'document', 'ID' => $this->get_id())));
		    return false;
		}
	    }

	    $this->create_id_reference($this->get_id(), $chamilo_repository_document->get_id());
	    $this->set_message(Translation :: get('GeneralConvertedMessage', array('TYPE' => 'document', 'OLD_ID' => $this->get_id(), 'NEW_ID' => $chamilo_repository_document->get_id())));

	    $pos = strrpos($this->get_path(), '/');
	    $parent_path = substr($this->get_path(), 0, $pos);

	    $publication_category_id = self :: $category_cache[$parent_path][self :: CACHE_PROPERTY_ID];
	    $publication_visible = self :: $category_cache[$parent_path][self :: CACHE_PROPERTY_VISIBLE];
	    $publication_hidden = $publication_visible ? null : 1;

	    if (!$publication_category_id)
	    {
		$publication_category_id = 0;
	    }

	    if (!$this->create_publication($chamilo_repository_document, $new_course_code, $new_user_id, 'document', $publication_category_id, $new_to_user_id, $new_to_group_id, 0, $publication_hidden))
	    {
		$this->set_message(Translation :: get('CouldNotCreateContentObjectPublication', array('TYPE' => 'document', 'ID' => $this->get_id())));
	    }

	    return true;
	}
	else
	{
	    $this->create_failed_element($this->get_id());
	    $this->set_message(Translation :: get('CreationOnFileSystemFailedMessage', array('TYPE' => 'document', 'ID' => $this->get_id())));
	    return false;
	}
    }

    static function get_table_name()
    {
	return Utilities :: camelcase_to_underscores(substr(Utilities :: get_classname_from_namespace(__CLASS__), 9));
	;
    }

    static function get_class_name()
    {
	return self :: CLASS_NAME;
    }

    function get_retrieve_order_property()
    {
	$order_properties[] = new ObjectTableOrder(self :: PROPERTY_FILETYPE, SORT_DESC);
	$order_properties[] = new ObjectTableOrder(self :: PROPERTY_PATH, SORT_ASC);

	return $order_properties;
    }

}

?>