<?php
namespace repository\content_object\forum;

use common\libraries\Configuration;
use common\libraries\Utilities;
use common\libraries\Mdb2Database;
use common\libraries\ObjectTableOrder;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;

use repository\content_object\forum\ForumSubscribe;
use repository\content_object\forum_topic\ForumPost;
use repository\content_object\forum_topic\ForumTopic;
use repository\ComplexContentObjectItem;

use repository\content_object\forum\ForumUserViewed;

use user\UserDataManager;

/**
 * Class for the specific Forum data management, it receives data calls and delegates it to the mdb2 class.
 *
 * @author Mattias De Pauw - Hogeschool Gent
 * @author Maarten Volckaert - Hogeschool Gent
 */
class ForumDataManager
{
    /**
     * **************************************************************************************************************
     * Variables *
     * **************************************************************************************************************
     */

    /**
     * Instance of the ForumDataManager
     *
     * @var <ForumDataManager
     */
    private static $instance;

    /**
     * Uses a singleton pattern and a factory pattern to return the data manager. The configuration determines which
     * data manager class is to be instantiated.
     *
     * @return ForumDataManagerInterface The data manager.
     */
    static function get_instance()
    {
        if (! isset(self :: $instance))
        {
            $type = Configuration :: get_instance()->get_parameter('general', 'data_manager');
            $class = __NAMESPACE__ . '\\' . Utilities :: underscores_to_camelcase($type) . 'ForumDataManager';
            self :: $instance = new $class();
        }
        return self :: $instance;
    }

    /**
     * This function retrieves a ForumSubscribe object based on its forum id and user id.
     *
     * @param $forum_id type The forum id.
     * @param $user_id type The user id.
     * @return ForumSubscribe
     *
     */
    static function retrieve_subscribe($forum_id, $user_id)
    {
        $conditions[] = new EqualityCondition(ForumSubscribe :: PROPERTY_FORUM_ID, $forum_id);
        $conditions[] = new EqualityCondition(ForumSubscribe :: PROPERTY_USER_ID, $user_id);
        $condition = new AndCondition($conditions);

        return self :: get_instance()->retrieve_subscribe(null, $condition);
    }

    /**
     * Retrieve all the subscribers of a forum based on the forum id.
     *
     * @param $forum_id int The id of the forum of which this function retrieves all subscribers.
     * @return ResultSet Returns a result set of forum subscribers.
     */
    static function retrieve_forum_subscribers($forum_id)
    {
        $condition = new EqualityCondition(ForumSubscribe :: PROPERTY_FORUM_ID, $forum_id);

        return self :: get_instance()->retrieve_forum_subscribers($condition);
    }

    /**
     * Gets the list of users who are subscribed to a forum.
     *
     * @param $forum_id int
     *
     * @return List of subscribed users
     */
    static function retrieve_subscribed_forum_users($forum_id)
    {
        $subscribeds = ForumDataManager :: retrieve_forum_subscribers($forum_id)->as_array();

        $users = array();

        foreach ($subscribeds as $subscribe)
        {
            $user = UserDataManager :: get_instance()->retrieve_user($subscribe->get_user_id());
            $users[$user->get_id()] = $user;
        }

        return $users;
    }

    /**
     * Gets the number of subscribers a forum has
     *
     * @param $forum_id int The id of a forum of which this function counts the subscribers.
     * @return int Returns the number of subscribers of a forum.
     */
    static function count_forum_subscribers($forum_id)
    {
        $condition = new EqualityCondition(ForumSubscribe :: PROPERTY_FORUM_ID, $forum_id);

        return self :: get_instance()->count_forum_subscribes($condition);
    }

    /**
     * get the last post of a (sub)forum his topics
     *
     * @param $forum_id int
     *
     * @return ForumPost
     */
    function retrieve_last_post_forum_topics($forum_id)
    {

        return self :: get_instance()->retrieve_last_post_forum_topics($forum_id);
    }

    /**
     * get the last post of a (sub)forum his subforum(s)
     *
     * @param $forum_id int
     *
     * @return Forum
     */
    function retrieve_forum_last_post_forum_subforums($forum_id)
    {
        return self :: get_instance()->retrieve_forum_last_post_forum_subforums($forum_id);
    }

    /**
     * Creates a new subscribe based on a user id and a forum id.
     *
     * @param $user_id int
     * @param $forum_id int
     *
     * @return boolean Returns whether the creation was succesfull.
     */
    function create_subscribe($user_id, $forum_id)
    {
        $forum_subscribe = new ForumSubscribe();
        $forum_subscribe->set_forum_id($forum_id);
        $forum_subscribe->set_user_id($user_id);

        return $forum_subscribe->create();
    }
}
?>