<?php
namespace repository\content_object\forum_topic;

use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\Versionable;
use common\libraries\AttachmentSupport;
use common\libraries\ObjectTableOrder;
use common\libraries\ComplexContentObjectSupport;
use common\libraries\Path;
use common\libraries\Translation;
use common\libraries\Session;

use user\UserDataManager;

use repository\ContentObject;
use repository\RepositoryDataManager;
use repository\ComplexContentObjectItem;
use repository\content_object\forum_topic\ForumPost;
use repository\content_object\forum\EmailNotificator;
use repository\content_object\forum\TopicEmailNotificator;


/**
 * $Id: forum_topic.class.php 200 2009-11-13 12:30:04Z kariboe $
 * @package repository.lib.content_object.forum_topic
 */
/**
 * This class represents a topic in a discussion forum.
 * 
 * @author Mattias De Pauw - Hogeschool Gent
 * @author Maarten Volckaert - Hogeschool Gent
 */
class ForumTopic extends ContentObject implements Versionable, AttachmentSupport
{
    /****************************************************************************************************************
    * Variables     												    *
    ****************************************************************************************************************/
    const PROPERTY_LOCKED = 'locked';
    const PROPERTY_TOTAL_POSTS = 'total_posts';
    const PROPERTY_LAST_POST = 'last_post_id';
    
    const CLASS_NAME = __CLASS__;
    
    /**
     * Variable that contains the first post of a forum topic.
     * @var ForumPost The first post of a forum topic. 
     */
    private $first_post;
    
    /****************************************************************************************************************
    * Getters   												    *
    ****************************************************************************************************************/
    
    /**
     * Gets the additional properties of a forum topic.
     * 
     * @return Array Returns an array of additional property names.
     */
    static function get_additional_property_names()
    {
        return array(self :: PROPERTY_LOCKED, self :: PROPERTY_TOTAL_POSTS, self :: PROPERTY_LAST_POST);
    }

    /**
     * Gets the allowed types for a forum topic.
     * 
     * @return Array Returns an array with allowed types.
     */
    function get_allowed_types()
    {
        return array(ForumPost :: get_type_name());
    }
    
    /**
     * Gets the type name by using an utility namespace function.
     * 
     * @return string
     */
    static function get_type_name()
    {
        return Utilities :: camelcase_to_underscores(Utilities :: get_classname_from_namespace(self :: CLASS_NAME));
    }
    
    /**
     * Gets the table name of this object in the database.
     * 
     * @return string Returns the table name in the database of this object.
     */
    static function get_table_name()
    {
        return Utilities :: camelcase_to_underscores(array_pop(explode('\\', self :: CLASS_NAME)));
    }
    
    /**
     * Gets whether this object is locked.
     * 
     * @return int
     */
    function get_locked()
    {
        return $this->get_additional_property(self :: PROPERTY_LOCKED);
    }
    
    /**
     * Gets the total number of posts of this topic.
     * 
     * @return int The number of posts of this topic.
     */
    function get_total_posts()
    {
        return $this->get_additional_property(self :: PROPERTY_TOTAL_POSTS);
    }
    
    /**
     * Gets the id of the last post in this topic.
     * 
     * @return int The id of the last post.
     */
    function get_last_post()
    {
        return $this->get_additional_property(self :: PROPERTY_LAST_POST);
    }
    
    /****************************************************************************************************************
    * Setters           											    *
    ****************************************************************************************************************/
    
    /**
     * Sets whether this topic is locked.
     * 
     * @param int $locked
     */
    function set_locked($locked)
    {
        $this->set_additional_property(self :: PROPERTY_LOCKED, $locked);
    }
    
    /**
     * Sets the total number of posts of this topic.
     * @param int $total_posts The total number of posts.
     */
    function set_total_posts($total_posts)
    {
        $this->set_additional_property(self :: PROPERTY_TOTAL_POSTS, $total_posts);
    }
    
    /**
     * Sets the last post of this topic by giving the id of the new last post.
     * @param int $last_post The id of the last post.
     */
    function set_last_post($last_post)
    {
        $this->set_additional_property(self :: PROPERTY_LAST_POST, $last_post);
    }
    
    /****************************************************************************************************************
    * CRUD      												    *
    ****************************************************************************************************************/
    
    /*
     * This function creates a forum topic object and if succesfull the first post in this topic aswell.
     * 
     * @return boolean $succes Returns whether the create was succesfull or not.
     * 
     */
    function create()
    {
        $succes = parent :: create();
     
        if($succes)
        {
            $forum_post = new ForumPost();
            
            $forum_post->set_title($this->get_title());
            $forum_post->set_content($this->get_description());
            $forum_post->set_user_id($this->get_owner_id());
            
            $forum_post->set_forum_topic_id($this->get_id());
            $forum_post->set_creation_date($this ->get_creation_date());
            $forum_post->set_modification_date($this->get_modification_date());
            $forum_post->create(true);       
            
            $this->first_post = $forum_post;
            $this->set_total_posts(1);
            $this->set_last_post(0);
            $this->update();
              
        }
             
        return $succes;
    }
       
    /**
     * Delete a Forum Topic and all its posts.
     * @return boolean Returns whether the delete was succesfull.
     */
    function delete()
    {
        $posts = ForumTopicDataManager :: retrieve_forum_posts($this->get_id()) -> as_array();
        $subscribes = ForumTopicDatamanager :: retrieve_subscribes($this->get_id())->as_array();
        foreach($posts as $post)
        {
            $post->delete(true);
        }
        foreach($subscribes as $subscribe)
        {
            $subscribe->delete();
        }
        return parent :: delete();
    }
    
    /*
     * This function attaches a content object to a forum topic.
     * 
     * @param int $aid  The id of the content object that needs to be attached to the forum topic.
     * @param const $type
     * 
     * @return boolean $success  Returns true if content object is attached succesful
     * 
     */
    function attach_content_object($aid, $type = self :: ATTACHMENT_NORMAL)
    {
        $success = parent :: attach_content_object($aid, $type);

        if ($this->first_post)
        {
            $success &= $this->first_post->attach_content_object($aid, $type);
        }
        
        return $success;
    }
    
    /**
     * Add the last post to a topic and his parents
     * 
     * @param int $last_post The id of the last post.
     */
    function add_last_post($last_post)
    {
        $this->set_last_post($last_post);
       
        $this->update();
        
        $rdm = RepositoryDataManager :: get_instance();
        
        $condition = new EqualityCondition(ComplexContentObjectItem :: PROPERTY_REF, $this->get_id());
        $wrappers = $rdm->retrieve_complex_content_object_items($condition);
        
        while ($item = $wrappers->next_result())
        {
            $lo = $rdm->retrieve_content_object($item->get_parent());
            $lo->add_last_post($last_post);
        }
    }

    /**
     * This function is used to recalculate the last post of a specific forum topic.
     * 
     * @param int $forum_topic_id  The id of the forum topic of which this method will recalculate the last post. 
     * 
     */
    function recalculate_last_post()
    {
        $rdm = RepositoryDataManager :: get_instance();
          
        $next_last = ForumTopicDataManager :: retrieve_last_post($this->get_id());
       
        $next_last_post = $next_last->next_result();
        
        //after set id to O never update the forum post!!!!
        
       
 
        $first_post = $this->is_first_post($next_last_post);
       
        
        if ($this->get_last_post() != $next_last_post->get_id())
        {
            if(!$first_post)
            {
            
                $this->set_last_post($next_last_post->get_id());
            }
            else
            {
                $this->set_last_post(0);
            }
            $this->update();
            
           
            $condition = new EqualityCondition(ComplexContentObjectItem :: PROPERTY_REF, $this->get_id());
            $wrappers = $rdm->retrieve_complex_content_object_items($condition);
            
            while ($item = $wrappers->next_result())
            {
                $lo = $rdm->retrieve_content_object($item->get_parent());
                $lo->recalculate_last_post();
            }
        }
    }
  
    /**
     * This function adds a number of posts to the property total posts of a topic.
     * 
     * @param int $posts  Number of posts that needs to be added to a topic.
     * @param int $last_post_id  The id of the new last post.
     * 
     */
    function add_post($posts, $last_post_id, $emailnotificator)
    {
        $this->set_total_posts($this->get_total_posts() + $posts);
       
        $this->set_last_post($last_post_id);
        $this->update();
        $rdm = RepositoryDataManager :: get_instance();
        
        $condition = new EqualityCondition(ComplexContentObjectItem :: PROPERTY_REF, $this->get_id());
        $wrappers = $rdm->retrieve_complex_content_object_items($condition);
       if($emailnotificator)
       {
        $emailnotificator->add_users(ForumTopicDataManager :: retrieve_subscribed_forum_topic_users($this->get_id()));
        $emailnotificator->set_topic($this);
       }
        while ($item = $wrappers->next_result())
        {
            $lo = $rdm->retrieve_content_object($item->get_parent());
            $lo->add_post($posts,$emailnotificator,$item->get_id(),$last_post_id);
            
        }
        
    }
    /** 
     * The subscribed users to this topic and his parents notifieren
     * 
     * @param type $emailnotificator 
     */
     function notify_subscribed_users_edited_post_topic($emailnotificator )
    {
       
        $rdm = RepositoryDataManager :: get_instance();
        
        $condition = new EqualityCondition(ComplexContentObjectItem :: PROPERTY_REF, $this->get_id());
        $wrappers = $rdm->retrieve_complex_content_object_items($condition);
        
        if($emailnotificator)
        {
            $emailnotificator->add_users(ForumTopicDataManager :: retrieve_subscribed_forum_topic_users($this->get_id()));
            $emailnotificator->set_topic($this);
        }
        
        while ($item = $wrappers->next_result())
        {
            $lo = $rdm->retrieve_content_object($item->get_parent());
            $lo->notify_subscribed_users($emailnotificator);
        }
        
    }
    
    /*
     * This function subtracts a number of posts from the total post property of a topic and does this aswell for all its parents.
     * 
     * @param int $posts  Number of posts that needs to be subtracted.
     * @param int $forum_topic_id The id of the topic of which we subtract a number of posts.
     * 
     */
    function remove_post($posts = 1)
    {
        $this->set_total_posts($this->get_total_posts() - $posts);
        $this->update();
        
        $rdm = RepositoryDataManager :: get_instance();
        
        $condition = new EqualityCondition(ComplexContentObjectItem :: PROPERTY_REF, $this->get_id());
        $wrappers = $rdm->retrieve_complex_content_object_items($condition);
        
        while ($item = $wrappers->next_result())
        {
            $lo = $rdm->retrieve_content_object($item->get_parent());
            $lo->remove_post($posts);
        }
        $this->recalculate_last_post();
    }

    function is_locked()
    {
        if ($this->get_locked())
        {
            return true;
        }
        $rdm = RepositoryDataManager :: get_instance();
        $condition = new EqualityCondition(ComplexContentObjectItem :: PROPERTY_REF, $this->get_id());
        $parents = $rdm->retrieve_complex_content_object_items($condition);
        
        while ($parent = $parents->next_result())
        {
            $content_object = $rdm->retrieve_content_object($parent->get_parent());
            if ($content_object->is_locked())
            {
                return true;
            }
        }        
        return false;
    }
    
    function invert_locked()
    {
        $this->set_locked(! $this->get_locked());
        return $this->update();
    }

    /**
     * This function checks whether a given post object is the first post of its parent
     * 
     * @param ForumPost $object
     * 
     * @return boolean Returns true when a post is the first post of a topic.
     */
    function is_first_post($object)
    {
        $first_post = ForumTopicDataManager :: retrieve_first_post($this->get_id())->as_array();
        if($first_post[0]->get_creation_date() == $object->get_creation_date())
        {
            return true;
        }
        return false;
    }
    
    function get_full_path()
    {
        // return realpath(Configuration ::
        // get_instance()->get_parameter('general',
        // 'upload_path').'/'.$this->get_path());
        return Path :: get(SYS_REPO_PATH) . $this->get_path();
    }
    /**
     * update if the firstpost is al ready updated then only do an update else
     * make a new topicnotificator and send the emails
     * 
     * @param type $firstpostupdated
     *
     * @return succes 
     */
    function update($request_from_forum_postpost)
    {
   
          if(!$request_from_forum_postpost)
            {
               $first_post = ForumTopicDataManager :: retrieve_first_post($this->get_id())->as_array();
                if($this->get_title() == $first_post[0]->get_title() && $this->get_description() == $first_post[0]->get_content() )
                {
                    return parent :: update();
                }else
                {
                    $email_notificator = new TopicEmailNotificator();    

                    $text = Translation :: get("TopicEditedEmailTitle", null, ContentObject :: get_content_object_type_namespace('forum'));
                    $email_notificator->set_action_title($text);
                    $text = Translation :: get("TopicEditedEmailBody", null, ContentObject :: get_content_object_type_namespace('forum'));
                    $email_notificator->set_action_body($text);
                    $email_notificator->set_action_user(UserDataManager:: get_instance()->retrieve_user(Session :: get_user_id()));
                    $email_notificator->set_is_topic_edited(true);

                    $now = time();
                    $this->set_modification_date($now);
                    $first_post = ForumTopicDataManager :: retrieve_first_post($this->get_id())->as_array();
                    $first_post[0]->set_modification_date($now);

                    $email_notificator->set_previous_title($first_post[0]->get_title());

                    $first_post[0]->set_title($this->get_title());
                    $first_post[0]->set_content($this->get_description());


                    parent :: update();

                    $email_notificator->set_topic($this); 
                    $this->notify_subscribed_users_edited_post_topic($email_notificator);
                    $email_notificator->send_emails();     

                    return $first_post[0]->update(true);
                }
            }
            else
            {
                 return parent :: update();
            }
             
        }
    
}
?>
