<?php
namespace repository\content_object\forum_topic;

use common\libraries\Configuration;
use common\libraries\Utilities;
use repository\content_object\forum_topic\ForumPost;

use common\libraries\Mdb2Database;

use common\libraries\ObjectTableOrder;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;

use user\UserDataManager;

/**
 * Head class of the datamanager, it receives data calls and delegates it to the mdb2 class.
 *
 * @author Mattias De Pauw - Hogeschool Gent
 * @author Maarten Volckaert - Hogeschool Gent
 */
class ForumTopicDataManager
{
    /**
     * ***************************************************************************************************************
     * Variables *
     * **************************************************************************************************************
     */
    /**
     * Instance of the ForumTopicDataManager
     *
     * @var <ForumTopicDataManager
     */
    private static $instance;

    /**
     * Uses a singleton pattern and a factory pattern to return the data manager. The configuration determines which
     * data manager class is to be instantiated.
     *
     * @return SurveyContextDataManagerInterface The data manager.
     */
    static function get_instance()
    {
        if (! isset(self :: $instance))
        {
            $type = Configuration :: get_instance()->get_parameter('general', 'data_manager');
            $class = __NAMESPACE__ . '\\' . Utilities :: underscores_to_camelcase($type) . 'ForumTopicDataManager';
            self :: $instance = new $class();
        }
        return self :: $instance;
    }

    /**
     * **************************************************************************************************************
     * Posts *
     * **************************************************************************************************************
     */

    /**
     * Retrieve the last post of a particular topic.
     *
     * @param $forum_topic_id int The id of the topic of which this function retrieves the last post.
     * @return DataClass
     */
    static function retrieve_last_post($forum_topic_id)
    {
        $condition = new EqualityCondition(ForumPost :: PROPERTY_FORUM_TOPIC_ID, $forum_topic_id);
        $order_by = array();
        $order_by[] = new ObjectTableOrder(ForumPost :: PROPERTY_CREATION_DATE, SORT_DESC);

        return self :: get_instance()->retrieve_forum_posts($condition, 0, 1, $order_by);
    }

    /**
     * Retrieves modification date of a particular post.
     *
     * @param $condition Condition
     *
     * @return objects Date
     */
    function retrieve_forum_post_date($forum_post_id)
    {
        $post = self :: get_instance()->retrieve_forum_post($forum_post_id);

        return $post->get_modification_date();
    }

    /**
     * Retrieve the first post of a topic.
     *
     * @param $forum_topic_id int The id of the topic of which this function retrieves the first post.
     * @return DataClass
     */
    static function retrieve_first_post($forum_topic_id)
    {
        $condition = new EqualityCondition(ForumPost :: PROPERTY_FORUM_TOPIC_ID, $forum_topic_id);
        $order_by = array();
        $order_by[] = new ObjectTableOrder(ForumPost :: PROPERTY_CREATION_DATE, SORT_ASC);

        return self :: get_instance()->retrieve_forum_posts($condition, 0, 1, $order_by);
    }

    /**
     * Retrieve all the posts of a topic based on the topic id.
     *
     * @param $forum_topic_id int The id of the topic of which this function retrieves all posts.
     * @return ResultSet Returns a result set of posts.
     */
    static function retrieve_forum_posts($forum_topic_id, $patterncondition)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(ForumPost :: PROPERTY_FORUM_TOPIC_ID, $forum_topic_id);

        if ($patterncondition)
        {
            $conditions[] = $patterncondition;
        }

        $condition = new AndCondition($conditions);

        return self :: get_instance()->retrieve_forum_posts($condition);
    }

    /**
     * Gets the number of post a topic has.
     *
     * @param $forum_topic_id int The id of a forum topic of which this function counts the posts.
     * @return int Returns the number of post of a topic.
     */
    static function count_forum_topic_posts($forum_topic_id)
    {
        $condition = new EqualityCondition(ForumPost :: PROPERTY_FORUM_TOPIC_ID, $forum_topic_id);

        return self :: get_instance()->count_forum_posts($condition);
    }

    static function retrieve_forum_post_of_topic($topic_id, $post_id)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(ForumPost :: PROPERTY_FORUM_TOPIC_ID, $topic_id);
        $conditions[] = new EqualityCondition(ForumPost :: PROPERTY_ID, $post_id);
        $condition = new AndCondition($conditions);

        return self :: get_instance()->retrieve_forum_post_of_topic($condition);
    }

    /**
     * **************************************************************************************************************
     * Attachments *
     * **************************************************************************************************************
     */

    /**
     * This method is used to detach a content object from a particular forum post.
     *
     * @param $object ForumPost
     * @param $attachment_id int
     * @param $type int
     *
     * @return boolean returns true when successive delete
     */
    static function detach_content_object($object, $attachment_id, $type)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(ForumPostAttachment :: PROPERTY_FORUM_POST_ID, $object->get_id());
        $conditions[] = new EqualityCondition(ForumPostAttachment :: PROPERTY_ATTACHMENT_ID, $attachment_id);
        $condition = new AndCondition($conditions);

        return self :: get_instance()->detach_content_object($condition);
    }

    /*
     * This function returns all attachment objects of a particular forum post. @param Condition $condition @return
     * Dataclass
     */
    static function retrieve_attached_objects($forum_post_id)
    {
        $condition = new EqualityCondition(ForumPostAttachment :: PROPERTY_FORUM_POST_ID, $forum_post_id);

        return self :: get_instance()->retrieve_attached_objects($condition);
    }

    /*
     * This function counts the number of attachments objects of particular forum post. @Param Condition $condition
     * @Return int
     */
    function count_forum_post_attachments($forum_post_id)
    {
        $condition = new EqualityCondition(ForumPostAttachment :: PROPERTY_FORUM_POST_ID, $forum_post_id);

        return self :: get_instance()->count_forum_post_attachments($condition);
    }

    function retrieve_attachment_of_post($post_id, $attachment_id)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(ForumPostAttachment :: PROPERTY_FORUM_POST_ID, $post_id);
        $conditions[] = new EqualityCondition(ForumPostAttachment :: PROPERTY_ATTACHMENT_ID, $attachment_id);
        $condition = new AndCondition($conditions);

        return self :: get_instance()->retrieve_attachment_of_post($condition);
    }

    /**
     * **************************************************************************************************************
     * Subscribes *
     * **************************************************************************************************************
     */

    /**
     * This function is used to retrieve a single subscribe based on its Forum Topic id and its User ID.
     *
     * @param $forum_topic_id int The id of the forum topic.
     * @param $user_id int The id of the user.
     * @return ForumTopicSubscribe
     */
    function retrieve_subscribe($forum_topic_id, $user_id)
    {
        $conditions[] = new EqualityCondition(ForumTopicSubscribe :: PROPERTY_FORUM_TOPIC_ID, $forum_topic_id);
        $conditions[] = new EqualityCondition(ForumTopicSubscribe :: PROPERTY_USER_ID, $user_id);
        $condition = new AndCondition($conditions);

        return self :: get_instance()->retrieve_subscribe(null, $condition);
    }

    /**
     * Retrieve a subscribtion by its user id.
     *
     * @param $forum_topic_id int The id of the forum topic of which the function will retrieve the subscribtions.
     * @return ResultSet
     *
     */
    static function retrieve_subscribes($forum_topic_id)
    {
        $condition = new EqualityCondition(ForumTopicSubscribe :: PROPERTY_FORUM_TOPIC_ID, $forum_topic_id);

        return self :: get_instance()->retrieve_subscribes($condition);
    }

    /**
     * Gets the list of users who are subscribed to a forum topic.
     *
     * @param $forum_topic_id int
     *
     * @return List of subscribed users
     */
    static function retrieve_subscribed_forum_topic_users($forum_topic_id)
    {
        $subscribeds = ForumTopicDataManager :: retrieve_subscribes($forum_topic_id)->as_array();
        $users = array();
        foreach ($subscribeds as $subscribe)
        {
            $user = UserDataManager :: get_instance()->retrieve_user($subscribe->get_user_id());
            $users[$user->get_id()] = $user;
        }

        return $users;
    }

    /**
     * Create a subscribe based on a user id and topic id.
     *
     * @param $user_id int
     * @param $forum_topic_id int
     *
     * @return boolean Returns whether the create was succesfull.
     */
    static function create_subscribe($user_id, $forum_topic_id)
    {
        $forum_subscribe = new ForumTopicSubscribe();
        $forum_subscribe->set_forum_topic_id($forum_topic_id);
        $forum_subscribe->set_user_id($user_id);

        return $forum_subscribe->create();
    }
}
?>