<?php
namespace repository\content_object\forum_topic;

use user\UserDataManager;
use user\User;

use group\GroupDataManager;
use group\Group;

use common\libraries\Path;
use common\libraries\EqualityCondition;
use common\libraries\DataClass;
use common\libraries\Utilities;
use common\libraries\AndCondition;
use common\libraries\InCondition;
use common\libraries\Mdb2Database;
use common\libraries\Translation;
use common\libraries\Condition;

use repository\ContentObject;
use repository\content_object\forum_topic\ForumPost;
use repository\content_object\forum_topic\ForumPostAttachment;
use repository\content_object\forum_topic\ForumTopic;
use repository\RepositoryDataManager;

use MDB2;
use MDB2_Error;
use Exception;
     
/*
 * This class does all the necessary communication with the database layer.
 * 
 * @author Mattias De Pauw - Hogeschool Gent
 * @author Maarten Volckaert - Hogeschool Gent
 */
class Mdb2ForumTopicDataManager extends Mdb2Database implements ForumTopicDataManagerInterface
{
    /**
     *  Inherited.
     *  Sets the right prefix for this mdb2 class.
     */
    function initialize()
    {
        parent :: initialize();
        $this->set_prefix('repository_');
    }
    
    /****************************************************************************************************************
    * Post      											            *
    ****************************************************************************************************************/  
    
    /**
     * Return 1 forum post by his id.
     * 
     * @param int $forum_post_id
     * 
     * @return dataclass
     * 
     */
    function retrieve_forum_post($forum_post_id)
    {   
        $condition = new EqualityCondition(ForumPost :: PROPERTY_ID, $forum_post_id);
        
        return $this->retrieve_object(ForumPost :: get_table_name(), $condition, array(), ForumPost :: CLASS_NAME);
    }
    
    /**
     * Retrieves all the posts of a topic.
     * 
     * @param Condition $condition
     * 
     * @Return objects DataClass
     */
    function retrieve_forum_posts($condition,$offset,$max_objects,$order_by)
    {
        return $this->retrieve_objects(ForumPost :: get_table_name(), $condition, $offset, $max_objects, $order_by, ForumPost :: CLASS_NAME);
    }
    
    function retrieve_forum_post_of_topic($condition)
    {
        return $this->retrieve_object(ForumPost :: get_table_name(), $condition, array(), ForumPost :: CLASS_NAME);
    }
    
    /****************************************************************************************************************
    * Topic											                    *
    ****************************************************************************************************************/  
    
     /*
      * This function is used for retrieving a particular forum topic based on its id.
      * 
      * @param int $forum_topic_id  The id of the forum topic that needs to be retrieved
      * 
      * @return ForumTopic
      * 
      */
     function retrieve_forum_topic($forum_topic_id)
     {
        $condition = new EqualityCondition(ForumTopic :: PROPERTY_ID, $forum_topic_id);
        
        return $this->retrieve_object(ForumTopic :: get_table_name(), $condition, array(), ForumTopic :: CLASS_NAME);
     }
    /**
     *Count the number of posts a topic has.
     * 
     *@param Condition $condition
     * 
     *@return int The number of Forum posts based on a condition.   
     * 
     */
    public function count_forum_posts($condition) 
    {       
        return $this->count_objects(ForumPost :: get_table_name(), $condition);  
    }
    
    
    /****************************************************************************************************************
    * Attachments											            *
    ****************************************************************************************************************/ 
    
     /*
      * This function returns a ContentObject that is attached to a forum post.
      * 
      * @param int $forum_post_id  The id of a forum post that has an attached ContentObject.
      * @param int $attachment_id  The id of a ContentObject that is attached to a forum post.
      * 
      * @return ContentObject
      * 
      */
    function retrieve_attached_object($forum_post_id, $attachment_id)
    {
        $conditions[] = new EqualityCondition(ForumPostAttachment :: PROPERTY_FORUM_POST_ID, $forum_post_id);
        $conditions[] = new EqualityCondition(ForumPostAttachment :: PROPERTY_ATTACHMENT_ID, $attachment_id);
        $condition = new AndCondition($conditions);
        
        return $this->retrieve_object(ForumPostAttachment :: get_table_name(), $condition, array(), ForumPostAttachment :: CLASS_NAME);
    }
    
    /*
     * This method returns all attachment objects of a particular forum post.
     * 
     * @param Condition $condition
     *
     * @return ResultSet
     * 
     */
    function retrieve_attached_objects($condition)
    {
        return $this->retrieve_objects(ForumPostAttachment :: get_table_name(), $condition, array(), ForumPostAttachment :: CLASS_NAME);
    }

    /*
     * This method retrieves an array of the attached content objects of a particular post.
     * 
     * @param int $forum_post_id the id of the forum post.
     * @param int $offset
     * @param int $max_objects
     * @param int $order_by
     * 
     * @return ResultSet A resultset of content objects.
     * 
     */
    function retrieve_attached_object_from_forum_post($forum_post_id, $offset = null, $max_objects = null, $order_by = null)
    {
        $forum_post_attachment_alias = $this->get_alias(ForumPostAttachment :: get_table_name());
	$forum_post_attachment_table = $this->get_table_name(ForumPostAttachment :: get_table_name());
        $content_object_alias = $this->get_alias(ContentObject :: get_table_name());
        $content_object_table = $this->get_table_name(ContentObject :: get_table_name());        
        $content_object_id_column = $this->escape_column_name(ContentObject :: PROPERTY_ID, $content_object_alias);
        $forum_post_attachment_id_column = $this->escape_column_name(ForumPostAttachment :: PROPERTY_ATTACHMENT_ID,
                                           $forum_post_attachment_alias);
        
        $query = 'SELECT ' . $content_object_alias . '.* ';
        $query .= 'FROM ' . $content_object_table . ' AS ' . $content_object_alias;
        $query .= ' JOIN ' . $forum_post_attachment_table . ' AS ' . $forum_post_attachment_alias;
        $query .= ' ON ' . $forum_post_attachment_id_column . ' = ' . $content_object_id_column;
        $condition = new EqualityCondition(ForumPostAttachment :: PROPERTY_FORUM_POST_ID, $forum_post_id,
            $forum_post_attachment_alias, true);
        
        return $this->retrieve_object_set($query, ContentObject :: get_table_name(), $condition, $offset, 
	$max_objects, $order_by, ContentObject :: CLASS_NAME);        
    }
    
    /*
     * This method counts the number of attachments objects of  particular forum post.
     * 
     * @param Condition $condition
     * 
     * @return int The number of Forum Post Attachments.
     * 
     */
    function count_forum_post_attachments($condition)
    {
        return $this->count_objects(ForumPostAttachment :: get_table_name(), $condition);
    }
    
    /**
     * Inherited.
     * 
     * This method attaches a content object to a particular forum post.
     * 
     * @param ForumPostAttachment $object the forumpostattachment that will be created.
     * 
     * @return boolean Returns whether the detaching was succesfull.
     * 
     */
    function detach_content_object($condition)
    {        
        return $this->delete_content_object_attachments($condition);
    }
    
    /**
     * Inherited.
     * 
     * Delete ForumPostAttachments based on a condition.
     * 
     * @param Condition $condition The condition of the delete.
     * 
     * @return boolean Returns whether the delete was succesfull.
     *  
     */
    function delete_content_object_attachments(Condition $condition)
    {
        return $this->delete_objects(ForumPostAttachment :: get_table_name(), $condition);
    }
    
    function retrieve_attachment_of_post(Condition $condition)
    {
        return $this->retrieve_object(ForumPostAttachment :: get_table_name(), $condition, array(), ForumPostAttachment :: CLASS_NAME);
    }
    
    /****************************************************************************************************************
    * Subscribes											            *
    ****************************************************************************************************************/  
    
    /**
     * This function is used to retrieve a single subscribe based on a condition.
     * 
     * @param int $subscribe_id
     * @param Condition $condition
     * 
     * @return ForumTopicSubscribe 
     * 
     */
    function retrieve_subscribe($subscribe_id, $condition = null)
    {
        if(!$condition)
            $condition = new EqualityCondition(ForumTopicSubscribe :: PROPERTY_ID, $subscribe_id);

        return $this->retrieve_object(ForumTopicSubscribe :: get_table_name(), $condition, array(), ForumTopicSubscribe :: CLASS_NAME);
    }
    
    /**
     * This function is used to retrieve multiple subscribes by a condition.
     * 
     * @param Condition $condition
     * 
     * @return ResultSet A ResultSet of ForumTopicSubscribes.
     * 
     */
    function retrieve_subscribes($condition)
    {
         return $this->retrieve_objects(ForumTopicSubscribe :: get_table_name(), $condition, array(), ForumTopicSubscribe :: CLASS_NAME);
    }
    
    /**
     * This function is used to delete a single subscribe based on a condition.
     * 
     * @param Condition $condition
     * 
     * @return boolean Returns whether the delete is succesfull.
     * 
     */    
    function delete_subscribe($condition)
    {        
        return $this->delete_objects(ForumTopicSubscribe :: get_table_name(), $condition);
    }
    
    /**
     * This function is used to delete subscribes by condition.
     * 
     * @param Condition $condition
     * 
     * @return boolean Returns whether the delete was succesfull. 
     * 
     */
    function delete_subscribes($condition)
    {
        return $this->delete_objects(ForumTopicSubscribe :: get_table_name(), $condition);
    }  
}
?>
