<?php
namespace repository;

use common\extensions\category_manager\PlatformCategory;
use common\libraries\Utilities;
use common\libraries\Session;
use common\libraries\StringUtilities;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\Translation;
use common\libraries\NotCondition;

/**
 * This class describes a category for content objects in the repository
 *
 * @author Sven Vanpoucke
 */

class RepositoryCategory extends PlatformCategory
{
    const CLASS_NAME = __CLASS__;
    
    /**
     * **************************************************************************************************************
     * Properties *
     * **************************************************************************************************************
     */
    
    const PROPERTY_USER_ID = 'user_id';
    const PROPERTY_TYPE = 'type';
    
    /**
     * **************************************************************************************************************
     * Type Definition *
     * **************************************************************************************************************
     */
    
    const TYPE_NORMAL = 1;
    const TYPE_SHARED = 2;

    /**
     * **************************************************************************************************************
     * Inherited Functionality *
     * **************************************************************************************************************
     */
    
    /**
     * Creates this category
     *
     * @param $create_in_batch boolean
     *            - Creates objects in batch without fixing the right / left
     *            values (faster)
     *           
     * @return boolean
     */
    function create($create_in_batch = false)
    {
        $user_id = $this->get_user_id();
        
        if (! $user_id)
        {
            $user_id = Session :: get_user_id();
        }
        
        if (! $this->check_before_save())
        {
            return false;
        }
        
        if (! RepositoryDataManager :: get_instance()->create_category($this))
        {
            $this->add_error(Translation :: get('CouldNotCreateObjectInDatabase', array(
                    'OBJECT' => Translation :: get('Category'), Utilities :: COMMON_LIBRARIES)));
            
            return false;
        }
        
        $parent = $this->get_parent();
        if ($parent == 0)
        {
            $parent_id = RepositoryRights :: get_instance()->get_user_root_id($user_id);
        }
        else
        {
            $parent_id = RepositoryRights :: get_instance()->get_location_id_by_identifier_from_user_subtree(RepositoryRights :: TYPE_USER_CATEGORY, $this->get_parent(), $user_id);
        }
        
        if (! RepositoryRights :: get_instance()->create_location_in_user_tree(RepositoryRights :: TYPE_USER_CATEGORY, $this->get_id(), $parent_id, $user_id, $create_in_batch))
        {
            $this->add_error(Translation :: get('CouldNotCreateLocation'), array(), Utilities :: COMMON_LIBRARIES);
            return false;
        }
        
        return true;
    }

    /**
     * Checks if the data of this object is valid + adds some default values if
     * some data is not available
     *
     * @return boolean
     */
    protected function check_before_save()
    {
        if (StringUtilities :: is_null_or_empty($this->get_name()))
        {
            $this->add_error(Translation :: get('TitleIsRequired'));
        }
        
        if (! $this->get_user_id())
        {
            $user_id = Session :: get_user_id();
            if ($user_id)
            {
                $this->set_user_id($user_id);
            }
            else
            {
                $this->add_error(Translation :: get('UserIdIsRequired'));
            }
        }
        
        $rdm = RepositoryDataManager :: get_instance();
        
        if (! $this->get_parent())
        {
            $this->set_parent(0);
        }
        else
        {
            $condition = new EqualityCondition(PlatformCategory :: PROPERTY_ID, $this->get_parent());
            $count = $rdm->count_categories($condition);
            if ($count == 0)
            {
                $this->add_error(Translation :: get('ParentDoesNotExist'));
            }
        }
        
        if (! $this->get_display_order())
        {
            $this->set_display_order($rdm->select_next_category_display_order($this->get_parent(), $this->get_user_id(), $this->get_type()));
        }
        
        $conditions = array();
        
        if ($this->get_id())
        {
            $conditions[] = new NotCondition(new EqualityCondition(PlatformCategory :: PROPERTY_ID, $this->get_id()));
        }
        
        $conditions[] = new EqualityCondition(RepositoryCategory :: PROPERTY_NAME, $this->get_name());
        $conditions[] = new EqualityCondition(RepositoryCategory :: PROPERTY_PARENT, $this->get_parent());
        $conditions[] = new EqualityCondition(RepositoryCategory :: PROPERTY_USER_ID, $this->get_user_id());
        $conditions[] = new EqualityCondition(RepositoryCategory :: PROPERTY_TYPE, $this->get_type());
        
        $condition = new AndCondition($conditions);
        $count = $rdm->count_categories($condition);
        
        if ($count > 0)
        {
            $this->add_error('CategoryWithSameNameExists');
        }
        
        return ! $this->has_errors();
    }

    /**
     * Updates this object
     *
     * @param $move boolean           
     *
     * @return boolean
     */
    function update($move = false)
    {
        if (! $this->check_before_save())
        {
            return false;
        }
        
        if (! RepositoryDataManager :: get_instance()->update_category($this))
        {
            $this->add_error(Translation :: get('CouldNotUpdateObjectInDatabase', array(
                    'OBJECT' => Translation :: get('Category'), Utilities :: COMMON_LIBRARIES)));
        }
        
        if ($move)
        {
            if ($this->get_parent())
            {
                $new_parent_id = RepositoryRights :: get_instance()->get_location_id_by_identifier_from_user_subtree(RepositoryRights :: TYPE_USER_CATEGORY, $this->get_parent(), $this->get_user_id());
            }
            else
            {
                $new_parent_id = RepositoryRights :: get_instance()->get_user_root_id(Session :: get_user_id());
            }
            
            $location = RepositoryRights :: get_instance()->get_location_by_identifier_from_users_subtree(RepositoryRights :: TYPE_USER_CATEGORY, $this->get_id(), $this->get_user_id());
            
            if (! $location->move($new_parent_id))
            {
                $this->add_error(Translation :: get('CouldNotMoveLocation'));
                return false;
            }
        }
        
        return true;
    }

    /**
     * Deletes this object
     *
     * @return boolean
     */
    function delete()
    {
        if ($this->get_type() == self :: TYPE_SHARED)
        {
            if (! RepositoryDataManager :: delete_share_category_recursive($this))
            {
                $this->add_error(Translation :: get('CouldNotDeleteCategoryInDatabase'));
                return false;
            }
        }
        else
        {
            $deleted_content_objects = RepositoryDataManager :: retrieve_recycled_content_objects_from_category($this->get_id());
            
            while ($deleted_content_object = $deleted_content_objects->next_result())
            {
                $deleted_content_object->move(0);
            }
            
            if (! RepositoryDataManager :: delete_category_recursive($this))
            {
                $this->add_error(Translation :: get('CouldNotDeleteCategoryInDatabase'));
                return false;
            }
        }
        
        $location = RepositoryRights :: get_instance()->get_location_by_identifier_from_users_subtree(RepositoryRights :: TYPE_USER_CATEGORY, $this->get_id(), $this->get_user_id());
        if ($location)
        {
            if (! $location->delete())
            {
                $this->add_error(Translation :: get('CouldNotDeleteLocation'));
                return false;
            }
        }
        
        return true;
    }

    /**
     * Returns the available property names
     *
     * @return string[]
     */
    static function get_default_property_names()
    {
        return array(self :: PROPERTY_USER_ID, self :: PROPERTY_TYPE, self :: PROPERTY_ID, self :: PROPERTY_NAME, 
                self :: PROPERTY_PARENT, self :: PROPERTY_DISPLAY_ORDER);
    }

    /**
     * **************************************************************************************************************
     * Getters & Setters *
     * **************************************************************************************************************
     */
    
    /**
     * Returns the user id of this object
     *
     * @return int
     */
    function get_user_id()
    {
        return $this->get_default_property(self :: PROPERTY_USER_ID);
    }

    /**
     * Sets the user id of this object
     *
     * @param $user_id int           
     */
    function set_user_id($user_id)
    {
        $this->set_default_property(self :: PROPERTY_USER_ID, $user_id);
    }

    /**
     * Returns the type of this object
     *
     * @return int
     */
    function get_type()
    {
        return $this->get_default_property(self :: PROPERTY_TYPE);
    }

    /**
     * Sets the type of this object
     *
     * @param $type int           
     */
    function set_type($type)
    {
        $this->set_default_property(self :: PROPERTY_TYPE, $type);
    }

    /**
     * **************************************************************************************************************
     * Helper functionality *
     * **************************************************************************************************************
     */
    
    function has_children()
    {
        $condition = new EqualityCondition(self :: PROPERTY_PARENT, $this->get_id());
        return (RepositoryDataManager :: get_instance()->count_categories($condition) > 0);
    }

    function get_children_ids($recursive = true)
    {
        $condition = new EqualityCondition(self :: PROPERTY_PARENT, $this->get_id());
        if (! $recursive)
        {
            return (RepositoryDataManager :: get_instance()->retrieve_distinct(self :: get_table_name(), self :: PROPERTY_ID, $condition));
        }
        else
        {
            $children_ids = array();
            $children = RepositoryDataManager :: get_instance()->retrieve_categories($condition);
            while ($child = $children->next_result())
            {
                $children_ids[] = $child->get_id();
                $children_ids = array_merge($children_ids, $child->get_children_ids($recursive));
            }
            return $children_ids;
        }
    }

    function get_parent_ids()
    {
        if ($this->get_parent() == 0)
        {
            return array(0);
        }
        else
        {
            $parent = RepositoryDataManager :: get_instance()->retrieve_category($this->get_parent());
            
            $parent_ids = array();
            $parent_ids[] = $parent->get_id();
            $parent_ids = array_merge($parent->get_parent_ids(), $parent_ids);
            return $parent_ids;
        }
    }

}
