<?php
namespace repository;

use common\libraries\Registration;
use common\libraries\CommonDataManager;
use common\libraries\ObjectTableOrder;
use common\libraries\Filesystem;
use common\libraries\SubselectCondition;
use common\libraries\EqualityCondition;
use common\libraries\DataClass;
use common\libraries\Utilities;
use common\libraries\Path;
use common\libraries\Versionable;
use common\libraries\Session;
use common\libraries\Theme;
use common\libraries\Translation;
use common\libraries\InCondition;
use common\libraries\AndCondition;
use common\libraries\OrCondition;
use common\libraries\AttachmentSupport;
use common\libraries\ComplexContentObjectSupport;
use admin\AdminDataManager;
use user\UserDataManager;
use repository\RepositoryRights;
use context_linker\ContextLinkerManager;
use metadata\MetadataDataManager;
use metadata\ContentObjectMetadataElementValue;
use rights\RightsLocationEntityRight;
use rights\PlatformGroupEntity;
use rights\UserEntity;

/**
 *
 * @author Tim De Pauw
 * @author Hans De Bisschop
 * @author Dieter De Neef
 */
class ContentObject extends DataClass
{
    const CLASS_NAME = __CLASS__;

    /**
     * A state constant for an inactive object
     *
     * @var int
     */
    const STATE_INACTIVE = 1;

    /**
     * A state constant for a normal object
     *
     * @var int
     */
    const STATE_NORMAL = 2;

    /**
     * A state constant for an object in the recycling bin
     *
     * @var int
     */
    const STATE_RECYCLED = 4;

    /**
     * A state constant for an autosaved object
     *
     * @var int
     */
    const STATE_AUTOSAVE = 8;

    /**
     * A state constant for an object that has been backuped
     *
     * @var int
     */
    const STATE_BACKUP = 16;

    /**
     * Property name of this object
     */
    const PROPERTY_TYPE = 'type';
    const PROPERTY_OWNER_ID = 'owner_id';
    const PROPERTY_TITLE = 'title';
    const PROPERTY_DESCRIPTION = 'description';
    const PROPERTY_PARENT_ID = 'parent_id';
    const PROPERTY_CREATION_DATE = 'created';
    const PROPERTY_MODIFICATION_DATE = 'modified';
    const PROPERTY_OBJECT_NUMBER = 'object_number';
    const PROPERTY_STATE = 'state';
    const PROPERTY_COMMENT = 'comment';
    const PROPERTY_CONTENT_HASH = 'content_hash';
    const PROPERTY_CURRENT = 'current';
    const PARAM_SECURITY_CODE = 'security_code';
    const ATTACHMENT_ALL = 'all';
    const ATTACHMENT_NORMAL = 'normal';
    const PROPERTIES_ADDITIONAL = 'additional_properties';
    const CURRENT_OLD = 0;
    const CURRENT_SINGLE = 1;
    const CURRENT_MULTIPLE = 2;

    /**
     * objects attached to this object.
     */
    private $attachments = array();

    private $attachment_ids = array();

    /**
     * Objects included into this object.
     */
    private $includes;

    /**
     * The state that this object had when it was retrieved. Used to determine if the state of its children should be
     * updated upon updating the object.
     */
    private $oldState;

    /**
     *
     * @var ObjectResultSet
     */
    private $synchronization_data;

    /**
     *
     * @var ComplexContentObjectPath
     */
    private $complex_content_object_path;

    /**
     * Creates a new object.
     *
     * @param $id int The numeric ID of the object. May be omitted if creating a new object.
     * @param $defaultProperties array The default properties of the object. Associative array.
     * @param $additionalProperties array The properties specific for this type of object. Associative array. Null if
     *            they are unknown at construction of the object; in this case, they will be retrieved when needed.
     */
    function __construct($default_properties = array(), $additional_properties = null)
    {
        parent :: __construct($default_properties);
        $this->set_additional_properties($additional_properties);
        $this->oldState = $default_properties[self :: PROPERTY_STATE];
    }

    /**
     * inherited
     */
    function get_data_manager()
    {
        return RepositoryDataManager :: get_instance();
    }

    /**
     * Returns a string representation of the type of this object.
     *
     * @return string The type.
     */
    function get_type()
    {
        $type = $this->get_default_property(self :: PROPERTY_TYPE);
        if ($type)
        {
            return $type;
        }

        return self :: class_to_type(get_class($this));
    }

    function get_type_string()
    {
        return Translation :: get('TypeName', null,
                ContentObject :: get_content_object_type_namespace($this->get_type()));
    }

    /**
     * Returns the state of this object.
     *
     * @return int The state.
     */
    function get_state()
    {
        return $this->get_default_property(self :: PROPERTY_STATE);
    }

    function get_current()
    {
        return $this->get_default_property(self :: PROPERTY_CURRENT);
    }

    function set_current($current)
    {
        $this->set_default_property(self :: PROPERTY_CURRENT, $current);
    }

    function is_current()
    {
        return ($this->get_current() != 0);
    }

    /**
     * Returns the ID of this object's owner.
     *
     * @return int The ID.
     */
    function get_owner_id()
    {
        return $this->get_default_property(self :: PROPERTY_OWNER_ID);
    }

    function get_owner()
    {
        if (! isset($this->owner))
        {
            $this->owner = UserDataManager :: get_instance()->retrieve_user($this->get_owner_id());
        }

        return $this->owner;
    }

    function get_owner_fullname()
    {
        return $this->get_owner()->get_fullname();
    }

    /**
     * Returns the title of this object.
     *
     * @return string The title.
     */
    function get_title()
    {
        return $this->get_default_property(self :: PROPERTY_TITLE);
    }

    /**
     * Returns the description of this object.
     *
     * @return string The description.
     */
    function get_description()
    {
        return $this->get_default_property(self :: PROPERTY_DESCRIPTION);
    }

    function has_description()
    {
        $description = $this->get_description();
        return ($description != '<p>&#160;</p>' && count($description) > 0);
    }

    /**
     * Returns the difference of this object with a given object based on it's id.
     *
     * @param $id int The ID of the object to compare with.
     * @return Array The difference.
     */
    function get_difference($id)
    {
        $dm = RepositoryDataManager :: get_instance();
        $version = $dm->retrieve_content_object($id);

        $lod = ContentObjectDifference :: factory($this, $version);

        return $lod;
    }

    /**
     * Returns the comment of this object version.
     *
     * @return string The version.
     */
    function get_comment()
    {
        return $this->get_default_property(self :: PROPERTY_COMMENT);
    }

    /**
     * Returns the numeric identifier of the object's parent object.
     *
     * @return int The identifier.
     */
    function get_parent_id()
    {
        return $this->get_default_property(self :: PROPERTY_PARENT_ID);
    }

    /**
     * Returns the date when this object was created, as returned by PHP's time() function.
     *
     * @return int The creation date.
     */
    function get_creation_date()
    {
        return $this->get_default_property(self :: PROPERTY_CREATION_DATE);
    }

    /**
     * Returns the date when this object was last modified, as returned by PHP's time() function.
     *
     * @return int The modification time.
     */
    function get_modification_date()
    {
        return $this->get_default_property(self :: PROPERTY_MODIFICATION_DATE);
    }

    /**
     * Returns the version number.
     *
     * @return int The version number.
     */
    function get_object_number()
    {
        return $this->get_default_property(self :: PROPERTY_OBJECT_NUMBER);
    }

    /**
     * Returns the objects attached to this object.
     *
     * @return array The objects.
     */
    function get_attached_content_objects($type = self :: ATTACHMENT_NORMAL)
    {
        if (! is_array($this->attachments[$type]))
        {
            $attachment_ids = $this->get_attached_content_object_ids($type);
            // Add non-existing element to avoid problems with empty InCondition
            if (count($attachment_ids) == 0)
            {
                $attachment_ids[] = - 1;
            }

            $condition = new InCondition(ContentObject :: PROPERTY_ID, $attachment_ids,
                    ContentObject :: get_table_name());
            $this->attachments[$type] = $this->get_data_manager()->retrieve_content_objects($condition)->as_array();
        }

        return $this->attachments[$type];
    }

    /**
     * Returns the object ids attached to this object.
     *
     * @return array The objects.
     */
    function get_attached_content_object_ids($type = self :: ATTACHMENT_NORMAL)
    {
        if (! is_array($this->attachment_ids[$type]))
        {
            $conditions = array();
            $conditions[] = new EqualityCondition(ContentObjectAttachment :: PROPERTY_CONTENT_OBJECT_ID, $this->get_id());
            if ($type != self :: ATTACHMENT_ALL)
            {
                $conditions[] = new EqualityCondition(ContentObjectAttachment :: PROPERTY_TYPE, $type);
            }
            $condition = new AndCondition($conditions);
            $this->attachment_ids[$type] = $this->get_data_manager()->retrieve_attached_content_object_ids($condition);
        }
        return $this->attachment_ids[$type];
    }

    /**
     * Returns the objects included into this object.
     *
     * @return array The objects.
     */
    function get_included_content_objects()
    {
        if (! is_array($this->includes))
        {
            $dm = RepositoryDataManager :: get_instance();
            $this->includes = $dm->retrieve_included_content_objects($this);
        }
        return $this->includes;
    }

    function get_content_object_versions($include_last = true)
    {
        if (! is_array($this->versions))
        {
            $dm = RepositoryDataManager :: get_instance();
            $this->versions = $dm->retrieve_content_object_versions($this, $include_last);
        }
        return $this->versions;
    }

    function get_latest_version_id()
    {
        $dm = RepositoryDataManager :: get_instance();
        return $dm->get_latest_version_id($this);
    }

    function get_latest_version()
    {
        $dm = RepositoryDataManager :: get_instance();
        return $dm->retrieve_content_object($dm->get_latest_version_id($this));
    }

    /**
     * Returns the edition of this object
     *
     * @return an int; the number of the version.
     */
    function get_content_object_edition()
    {
        $dm = RepositoryDataManager :: get_instance();
        return array_search($this->id, $dm->get_version_ids($this)) + 1;
    }

    /**
     * Returns the full URL where this object may be viewed.
     *
     * @return string The URL.
     */
    function get_view_url()
    {
        return $this->get_path(WEB_PATH) . 'index_repository_manager.php?go=view&category=' . $this->get_parent_id() . '&object=' . $this->get_id();
    }

    /**
     * Sets this object's state to any of the STATE_* constants.
     *
     * @param $state int The state.
     * @return boolean True upon success, false upon failure.
     */
    function set_state($state)
    {
        return $this->set_default_property(self :: PROPERTY_STATE, $state);
    }

    /**
     * Sets the ID of this object's owner.
     *
     * @param $id int The ID.
     */
    function set_owner_id($owner)
    {
        $this->set_default_property(self :: PROPERTY_OWNER_ID, $owner);
    }

    /**
     * Sets the object number of this object.
     *
     * @param $object_number int The Object Number.
     */
    function set_object_number($object_number)
    {
        $this->set_default_property(self :: PROPERTY_OBJECT_NUMBER, $object_number);
    }

    /**
     * Sets the title of this object.
     *
     * @param $title string The title.
     */
    function set_title($title)
    {
        $this->set_default_property(self :: PROPERTY_TITLE, $title);
    }

    /**
     * Sets the description of this object.
     *
     * @param $description string The description.
     */
    function set_description($description)
    {
        $this->set_default_property(self :: PROPERTY_DESCRIPTION, $description);
    }

    /**
     * Sets the comment of this object version.
     *
     * @param $comment string The comment.
     */
    function set_comment($comment)
    {
        $this->set_default_property(self :: PROPERTY_COMMENT, $comment);
    }

    /**
     * Sets the ID of this object's parent object.
     *
     * @param $parent int The ID.
     */
    function set_parent_id($parent)
    {
        $this->set_default_property(self :: PROPERTY_PARENT_ID, $parent);
    }

    /**
     * Sets the date when this object was created.
     *
     * @param $created int The creation date, as returned by time().
     */
    function set_creation_date($created)
    {
        $this->set_default_property(self :: PROPERTY_CREATION_DATE, $created);
    }

    /**
     * Sets the date when this object was modified.
     *
     * @param $modified int The modification date, as returned by time().
     */
    function set_modification_date($modified)
    {
        $this->set_default_property(self :: PROPERTY_MODIFICATION_DATE, $modified);
    }

    function get_content_hash()
    {
        return $this->get_default_property(self :: PROPERTY_CONTENT_HASH);
    }

    function set_content_hash($content_hash)
    {
        $this->set_default_property(self :: PROPERTY_CONTENT_HASH, $content_hash);
    }

    /**
     * Returns whether or not this object is extended, i.e. whether its type defines additional properties.
     *
     * @return boolean True if the object is extended, false otherwise.
     */
    function is_extended()
    {
        return self :: is_extended_type($this->get_type());
    }

    /**
     * Determines whether this object can have versions.
     *
     * @return boolean True if the object is versionable, false otherwise.
     * @deprecated Use instanceof Versionable directly from now on
     */
    function is_versionable()
    {
        return $this instanceof Versionable;
    }

    /**
     * Checks whether the current version of the object is the latest version
     *
     * @deprecated Use is_current() now
     */
    function is_latest_version()
    {
        return $this->is_current();
    }

    /**
     * Returns the number of versions of the object
     */
    function get_version_count()
    {
        $rdm = RepositoryDataManager :: get_instance();
        return count($rdm->get_version_ids($this));
    }

    /**
     * Checks if this object has versions
     *
     * @return boolean
     */
    function has_versions()
    {
        return ($this->get_current() != 1);
    }

    /**
     * Attaches the object with the given ID to this object.
     *
     * @param $id int The ID of the object to attach.
     */
    function attach_content_object($id, $type = self :: ATTACHMENT_NORMAL)
    {
        $dm = RepositoryDataManager :: get_instance();
        return $dm->attach_content_object($this, $id, $type);
    }

    function attach_content_objects($ids = array(), $type = self :: ATTACHMENT_NORMAL)
    {
        if (! is_array($ids))
        {
            $ids = array($ids);
        }

        foreach ($ids as $id)
        {
            if (! $this->attach_content_object($id, $type))
            {
                return false;
            }
        }
        return true;
    }

    /**
     * Is the object attached to object with the identifier as passed on
     *
     * @param $object_id int
     * @return boolean
     */
    function is_attached_to($object_id)
    {
        return RepositoryDataManager :: get_instance()->is_content_object_already_attached($object_id, $this->get_id());
    }

    /**
     * Includes the object with the given ID in this object.
     *
     * @param $id int The ID of the object to include.
     */
    function include_content_object($id)
    {
        $rdm = RepositoryDataManager :: get_instance();

        $is_already_included = $rdm->is_content_object_already_included($this, $id);

        if ($is_already_included)
        {
            return true;
        }
        else
        {
            return $rdm->include_content_object($this, $id);
        }
    }

    /**
     * Removes the object with the given ID from this object's attachment list.
     *
     * @param $id int The ID of the object to remove from the attachment list.
     * @return boolean True if the attachment was removed, false if it did not exist.
     */
    function detach_content_object($id, $type = self :: ATTACHMENT_NORMAL)
    {
        $dm = RepositoryDataManager :: get_instance();
        return $dm->detach_content_object($this, $id, $type);
    }

    function detach_content_objects($ids = array(), $type = self :: ATTACHMENT_NORMAL)
    {
        if (! is_array($ids))
        {
            $ids = array($ids);
        }

        foreach ($ids as $id)
        {
            if (! $this->detach_content_object($id, $type))
            {
                return false;
            }
        }
        return true;
    }

    function truncate_attachments($type = self :: ATTACHMENT_NORMAL)
    {
        // Reset the cache
        $this->truncate_attachment_cache($type);

        // Delete all types of attachments from persistent storage (only the
        // links, not the actual objects)
        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObjectAttachment :: PROPERTY_CONTENT_OBJECT_ID, $this->get_id());
        $conditions[] = new EqualityCondition(ContentObjectAttachment :: PROPERTY_TYPE, $type);
        $condition = new AndCondition($conditions);
        return $this->get_data_manager()->delete_content_object_attachments($condition);
    }

    /**
     * Empty the lazy load variables to trigger a new retrieve
     *
     * @param $type String
     */
    function truncate_attachment_cache($type = self :: ATTACHMENT_NORMAL)
    {
        unset($this->attachment_ids[$type]);
        unset($this->attachments[$type]);
    }

    /**
     * Removes the object with the given ID from this object's include list.
     *
     * @param $id int The ID of the object to remove from the include list.
     * @return boolean True if the include was removed, false if it did not exist.
     */
    function exclude_content_object($id)
    {
        $dm = RepositoryDataManager :: get_instance();
        return $dm->exclude_content_object($this, $id);
    }

    /**
     * Gets an additional (type-specific) property of this object by name.
     *
     * @param $name string The name of the property.
     */
    function get_additional_property($name)
    {
        $this->check_for_additional_properties();
        return $this->get_specific_property(self :: PROPERTIES_ADDITIONAL, $name);
    }

    /**
     * Sets an additional (type-specific) property of this object by name.
     *
     * @param $name string The name of the property.
     * @param $value mixed The new value for the property.
     */
    function set_additional_property($name, $value)
    {
        $this->set_specific_property(self :: PROPERTIES_ADDITIONAL, $name, $value);
    }

    /**
     * Gets the additional (type-specific) properties of this object.
     *
     * @return array An associative array containing the properties.
     */
    function get_additional_properties()
    {
        $this->check_for_additional_properties();
        return $this->get_specific_properties(self :: PROPERTIES_ADDITIONAL);
    }

    /**
     * Sets the additional (type-specific) properties of this object.
     *
     * @param array An associative array containing the properties.
     */
    function set_additional_properties($additional_properties)
    {
        $this->set_specific_properties(self :: PROPERTIES_ADDITIONAL, $additional_properties);
    }

    function create($create_in_batch = false)
    {
        // checks wether to create a new content object or version:
        // if the ID is set, we create a new version, otherwise a new CO.
        $orig_id = $this->get_id();
        $version = isset($orig_id);

        $now = time();
        $this->set_creation_date($now);
        $this->set_modification_date($now);

        $dm = RepositoryDataManager :: get_instance();
        if ($version)
        { // id changes in create new version, so location needs to be fetched
          // now
            $original_location = $this->get_rights_location();
            $this->set_current(self :: CURRENT_MULTIPLE);
        }
        else
        {
            $this->set_object_number(
                    $dm->retrieve_next_sort_value(ContentObject :: get_table_name(),
                            ContentObject :: PROPERTY_OBJECT_NUMBER));
            $this->set_current(self :: CURRENT_SINGLE);
        }

        if (! $dm->create_content_object($this, ($version ? 'version' : 'new')))
        {
            return false;
        }

        if ($this->get_owner_id() == 0)
            return true;

        $parent = $this->get_parent_id();
        if (! $parent)
        {
            $parent_id = RepositoryRights :: get_instance()->get_user_root_id($this->get_owner_id());
        }
        else
        {
            $parent_id = RepositoryRights :: get_instance()->get_location_id_by_identifier_from_user_subtree(
                    RepositoryRights :: TYPE_USER_CATEGORY, $this->get_parent_id(), $this->get_owner_id());
        }

        $new_location = RepositoryRights :: get_instance()->create_location_in_user_tree(
                RepositoryRights :: TYPE_USER_CONTENT_OBJECT, $this->get_id(), $parent_id, $this->get_owner_id(),
                $create_in_batch);

        if (! $new_location)
        {
            return false;
        }

        if ($version) // copy the granted rights
        {
            $rights_entities = $original_location->get_rights_entities();
            foreach ($rights_entities as $rights_entity)
            {
                $rights_entity_object = new RightsLocationEntityRight();
                $rights_entity_object->set_context($rights_entity->get_context());
                $rights_entity_object->set_entity_id($rights_entity->get_entity_id());
                $rights_entity_object->set_entity_type($rights_entity->get_entity_type());
                $rights_entity_object->set_right_id($rights_entity->get_right_id());
                $rights_entity_object->set_location_id($new_location->get_id());

                if (! $rights_entity_object->create())
                {
                    return false;
                }
            }
        }

        return true;
    }

    function get_rights_location()
    {
        return RepositoryRights :: get_instance()->get_location_by_identifier_from_users_subtree(
                RepositoryRights :: TYPE_USER_CONTENT_OBJECT, $this->get_id(), $this->get_owner_id());
    }

    function create_all()
    {
        $now = time();

        if (! $this->get_creation_date())
        {
            $this->set_creation_date($now);
        }

        if (! $this->get_modification_date())
        {
            $this->set_modification_date($now);
        }

        $dm = RepositoryDataManager :: get_instance();
        $object_number = $dm->get_next_content_object_number();
        $this->set_object_number($object_number);

        if (! $dm->create_content_object($this, 'new'))
        {
            return false;
        }

        if ($this->get_owner_id() == 0)
            return true;

        $parent = $this->get_parent_id();
        if (! $parent)
        {
            $parent_id = RepositoryRights :: get_instance()->get_user_root_id($this->get_owner_id());
        }
        else
        {
            $parent_id = RepositoryRights :: get_instance()->get_location_id_by_identifier_from_user_subtree(
                    RepositoryRights :: TYPE_USER_CATEGORY, $this->get_parent_id(), $this->get_owner_id());
        }

        if (! RepositoryRights :: get_instance()->create_location_in_user_tree(
                RepositoryRights :: TYPE_USER_CONTENT_OBJECT, $this->get_id(), $parent_id, $this->get_owner_id()))
        {
            return false;
        }

        return true;
    }

    /**
     * Instructs the data manager to update the object, making any modifications permanent. Also sets the object's
     * modification date to the current time if the update is a true update. A true update is an update that implicates
     * a change to a property that affects the object itself; changing the object's category, for instance, should not
     * change the last modification date.
     *
     * @param $trueUpdate boolean True if the update is a true update (default), false otherwise.
     * @return boolean True if the update succeeded, false otherwise.
     */
    function update($trueUpdate = true)
    {
        if ($trueUpdate)
        {
            $this->set_modification_date(time());
        }
        $dm = RepositoryDataManager :: get_instance();
        $success = $dm->update_content_object($this);
        if (! $success)
        {
            return false;
        }
        /*
         * $state = $this->get_state(); if ($state == $this->oldState) { return true; } $child_ids = self ::
         * get_child_ids($this->get_id()); $dm->set_content_object_states($child_ids, $state);
         */
        /*
         * We return true here regardless of the result of the child update,
         * since the object itself did get updated.
         */
        return true;
    }

    function recycle()
    {
        $this->set_modification_date(time());
        $this->set_state(self :: STATE_RECYCLED);

        $dm = RepositoryDataManager :: get_instance();
        return $dm->update_content_object($this);
    }

    function move($new_parent_id)
    {
        $this->set_parent_id($new_parent_id);
        $dm = RepositoryDataManager :: get_instance();
        $succes = $dm->update_content_object($this);
        if (! $succes)
        {
            return false;
        }

        if ($new_parent_id == 0)
        {
            $new_parent = RepositoryRights :: get_instance()->get_user_root_id($this->get_owner_id());
        }
        else
        {
            $new_parent = RepositoryRights :: get_instance()->get_location_id_by_identifier_from_user_subtree(
                    RepositoryRights :: TYPE_USER_CATEGORY, $new_parent_id, $this->get_owner_id());
        }

        $location = RepositoryRights :: get_instance()->get_location_by_identifier_from_users_subtree(
                RepositoryRights :: TYPE_USER_CONTENT_OBJECT, $this->get_id(), $this->get_owner_id());

        return $location->move($new_parent);
    }

    function version()
    {
        // create version
        if (! $this->create())
        {
            return false;
        }

        // update state
        $state = $this->get_state();
        if ($state == $this->oldState)
        {
            return true;
        }
        $child_ids = self :: get_child_ids($this->get_id());
        RepositoryDataManager :: get_instance()->set_content_object_states($child_ids, $state);
        /*
         * We return true here regardless of the result of the child update, since the object itself did get updated.
         */
        return true;
    }

    private static function get_child_ids($id)
    {
        $cond = new EqualityCondition(self :: PROPERTY_PARENT_ID, $id);
        $children = RepositoryDataManager :: get_instance()->retrieve_content_objects($cond, array(), 0, - 1, - 1);
        $ids = array();
        while ($child = $children->next_result())
        {
            $child_id = $child->get_id();
            $ids[] = $child_id;
            $child_ids = self :: get_child_ids($child_id);
            if (count($child_ids))
            {
                $ids = array_merge($ids, $child_ids);
            }
        }
        return $ids;
    }

    /**
     * Instructs the data manager to delete the object.
     *
     * @return boolean True if deletion succeeded, false otherwise.
     */
    function delete()
    {
        $location = RepositoryRights :: get_instance()->get_location_by_identifier_from_users_subtree(
                RepositoryRights :: TYPE_USER_CONTENT_OBJECT, $this->get_id(), $this->get_owner_id());
        if ($location)
        {
            if (! $location->delete())
            {
                return false;
            }
        }
        $success = RepositoryDataManager :: get_instance()->delete_content_object($this);

        // remove metadata
        $condition = new EqualityCondition(ContentObjectMetadataElementValue :: PROPERTY_CONTENT_OBJECT_ID,
                $this->get_id());
        $metadata_values = MetadataDataManager :: get_instance()->retrieve_content_object_metadata_element_values(
                $condition);

        if ($metadata_values->size())
        {
            while ($metadata_value = $metadata_values->next_result())
            {
                $metadata_value->delete();
            }
        }
        // remove context links
        ContextLinkerManager :: delete_content_links_for_object($this->get_id());

        return $success;
    }

    function delete_version()
    {
        if (RepositoryDataManager :: get_instance()->delete_content_object_version($this))
        {
            if ($this->is_latest_version())
            {
                $new_latest_content_object = RepositoryDataManager :: get_instance()->retrieve_most_recent_content_object_version($this);
                $count = RepositoryDataManager :: get_instance()->count_content_object_versions($this);

                $new_latest_content_object->set_current(
                        ($count > 1 ? self :: CURRENT_MULTIPLE : self :: CURRENT_SINGLE));

                return $new_latest_content_object->update();
            }

            return true;
        }
        else
        {
            return false;
        }
    }

    function delete_links()
    {
        $rdm = RepositoryDataManager :: get_instance();

        // Delete attachment links of the object
        $condition = new EqualityCondition(ContentObjectAttachment :: PROPERTY_ATTACHMENT_ID, $this->get_id());
        if (! $rdm->delete_content_object_attachments($condition))
        {
            return false;
        }

        if (RepositoryDataManager :: delete_content_object_publications($this) && $rdm->delete_content_object_includes(
                $this) && RepositoryDataManager :: delete_clois_for_content_object($this) && $rdm->delete_assisting_content_objects(
                $this))
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * Retrieves this object's ancestors.
     *
     * @return array The ancestors, all objects.
     */
    function get_ancestors()
    {
        $ancestors = array();
        $aid = $this->get_parent_id();
        while ($aid > 0)
        {
            $ancestor = RepositoryDataManager :: get_instance()->retrieve_content_object($aid);
            $ancestors[] = $ancestor;
            $aid = $ancestor->get_parent_id();
        }
        return $ancestors;
    }

    /**
     * Checks if the given ID is the ID of one of this object's ancestors.
     *
     * @param $ancestor_id int
     * @return boolean True if the ID belongs to an ancestor, false otherwise.
     */
    function has_ancestor($ancestor_id)
    {
        $aid = $this->get_parent_id();
        while ($aid > 0)
        {
            if ($aid == $ancestor_id)
            {
                return true;
            }
            $ancestor = RepositoryDataManager :: get_instance()->retrieve_content_object($aid);
            $aid = $ancestor->get_parent_id();
        }
        return false;
    }

    /**
     * Determines whether this object may be moved to the object with the given ID. By default, a object may be moved to
     * another object if the other object is not the object itself, the object is not an ancestor of the other object,
     * and the other object is a category.
     *
     * @param $target int The ID of the target object.
     * @return boolean True if the move is allowed, false otherwise.
     */
    function move_allowed($target)
    {
        /*
         * if ($target == $this->get_id()) { return false; } $target_object = RepositoryDataManager ::
         * get_instance()->retrieve_content_object($target); if ($target_object->get_type() != 'category') { return
         * false; } return !$target_object->has_ancestor($this->get_id());
         */
        return true;
    }

    // XXX: Keep this around? Override? Make useful?
    function __tostring()
    {
        return get_class($this) . '#' . $this->get_id() . ' (' . $this->get_title() . ')';
    }

    /**
     * Determines whether this object supports attachments, i.e. whether other objects may be attached to it.
     *
     * @return boolean True if attachments are supported, false otherwise.
     * @deprecated Use instanceof AttachmentSupport directly from now on
     */
    function supports_attachments()
    {
        return $this instanceof AttachmentSupport;
    }

    /**
     * Determines whether this object is a complex object
     *
     * @return boolean True if the LO is a CLO
     * @deprecated Use instanceof ComplexContentObjectSupport directly from now on
     */
    function is_complex_content_object()
    {
        return $this instanceof ComplexContentObjectSupport;
    }

    /**
     *
     * @return ComplexContentObjectPath
     */
    function get_complex_content_object_path()
    {
        if (! isset($this->complex_content_object_path))
        {
            $this->complex_content_object_path = new ComplexContentObjectPath($this);
        }

        return $this->complex_content_object_path;
    }

    /**
     * Gets the name of the icon corresponding to this object.
     */
    function get_icon_name($size = Theme :: ICON_SMALL)
    {
        return $size;
    }

    function get_icon_image($size = Theme :: ICON_SMALL)
    {
        return '<img src="' . $this->get_icon_path($size) . '" alt="' . $this->get_type() . '" />';
    }

    function get_icon_path($size = Theme :: ICON_SMALL)
    {
        return Theme :: get_image_path(ContentObject :: get_content_object_type_namespace($this->get_type())) . 'logo/' . $this->get_icon_name(
                $size) . ($this->is_current() ? '' : '_na') . '.png';
    }

    /**
     * Checks if the object's additional properties have already been loaded, and requests them from the data manager if
     * they have not.
     */
    private function check_for_additional_properties()
    {
        $additional_properties = $this->get_specific_properties(self :: PROPERTIES_ADDITIONAL);

        if (isset($additional_properties) && ! empty($additional_properties))
        {
            return;
        }

        $dm = RepositoryDataManager :: get_instance();
        $this->set_additional_properties($dm->retrieve_additional_content_object_properties($this));
    }

    /**
     * Get the default properties of all objects.
     *
     * @return array The property names.
     */
    static function get_default_property_names($extended_property_names = array())
    {
        return parent :: get_default_property_names(
                array(self :: PROPERTY_OWNER_ID, self :: PROPERTY_TYPE, self :: PROPERTY_TITLE,
                        self :: PROPERTY_DESCRIPTION, self :: PROPERTY_PARENT_ID, self :: PROPERTY_CREATION_DATE,
                        self :: PROPERTY_MODIFICATION_DATE, self :: PROPERTY_OBJECT_NUMBER, self :: PROPERTY_STATE,
                        self :: PROPERTY_COMMENT, self :: PROPERTY_CONTENT_HASH, self :: PROPERTY_CURRENT));
    }

    static function get_additional_property_names()
    {
        return array();
    }

    static function get_searchable_property_names()
    {
        return array();
    }

    /**
     * Checks if the given identifier is the name of a default object property.
     *
     * @param $name string The identifier.
     * @return boolean True if the identifier is a property name, false otherwise.
     */
    static function is_default_property_name($name)
    {
        return in_array($name, self :: get_default_property_names());
    }

    static function is_additional_property_name($name)
    {
        return in_array($name, self :: get_additional_property_names());
    }

    /**
     * Get all properties of this type of object that should be taken into account to calculate the used disk space.
     *
     * @return mixed The property names. Either a string, an array of strings, or null if no properties affect disk
     *         quota.
     */
    static function get_disk_space_properties()
    {
        return null;
    }

    /**
     * Converts a object type name to the corresponding class name.
     *
     * @param $type string The type name.
     * @return string The class name.
     */
    static function type_to_class($type)
    {
        return self :: get_content_object_type_namespace($type) . '\\' . Utilities :: underscores_to_camelcase($type);
    }

    /**
     * Converts a class name to the corresponding object type name.
     *
     * @param $class string The class name.
     * @return string The type name.
     */
    static function class_to_type($class)
    {
        return Utilities :: get_classname_from_namespace($class, true);
    }

    /**
     * Invokes the constructor of the class that corresponds to the specified type of object.
     *
     * @param $type string The object type.
     * @param $id int The ID of the object.
     * @param $defaultProperties array An associative array containing the default properties of the object.
     * @param $additionalProperties array An associative array containing the additional (type-specific) properties of
     *            the object. Null if unknown; this implies JIT retrieval.
     * @return ContentObject The newly instantiated object.
     */
    static function factory($type, $defaultProperties = array(), $additionalProperties = array())
    {
        if (! CommonDataManager :: is_registered('repository\content_object\\' . $type))
        {
            return null; // problem with the here is the prepository
        }
        $class = self :: type_to_class($type);
        return new $class($defaultProperties, $additionalProperties);
    }

    /**
     * If a registered content object "accept" the properties passed as argument it is returned. Otherwise returns
     * false. Typical use is to create an object based on a url.
     *
     * @param $properties array An array of properties used to create an object
     * @return ContentObject | false The object created or false.
     */
    static function factory_from_resource($properties = array())
    {
        $args = func_get_args();

        $condition = new EqualityCondition(\common\libraries\Registration :: PROPERTY_TYPE, 'content_object');
        $registrations = CommonDataManager :: retrieves(Registration :: class_name(), $condition);

        $result = array();
        while ($registration = $registrations->next_result())
        {
            $class = self :: type_to_class($registration->get_name());
            if ($call = call_user_func_array(array($class, 'accept'), $args))
            {
                $object = is_array($call) ? reset($call) : $call;

                // if not already set defaults basic properties
                $title = $object->get_title();
                if (empty($title))
                {
                    $title = $properties[self :: PROPERTY_TITLE] ? $properties[self :: PROPERTY_TITLE] : '';
                    $object->set_title($title);
                }
                $owner_id = $object->get_owner_id();
                if (empty($owner_id))
                {
                    $owner_id = Session :: get_user_id();
                    $object->set_owner_id($owner_id);
                }
                $description = $object->get_description();
                if (empty($description))
                {
                    $description = $properties[self :: PROPERTY_DESCRIPTION] ? $properties[self :: PROPERTY_DESCRIPTION] : '';
                    $object->set_description($description);
                }

                if (is_array($call))
                {
                    foreach ($call as $key => $val)
                    {
                        $result[$key] = $val;
                    }
                }
                else
                {
                    $result[0] = $call;
                }
            }
        }
        ksort($result);
        $result = reset($result);
        return $result;
    }

    /**
     * If the content object "accept" the properties passed as argument it returns a new instance of itself based on
     * properties passed as argument. Otherwise returns false.
     *
     * @param $properties array
     * @return ContentObject | array | false
     */
    static function accept($properties)
    {
        return false;
    }

    /**
     *
     * @param $type string
     * @return boolean
     */
    static function is_extended_type($type)
    {
        $class = self :: type_to_class($type);

        $properties = call_user_func(array($class, 'get_additional_property_names'));
        return ! empty($properties);
    }

    /**
     *
     * @param $path_type string
     */
    function get_path($path_type)
    {
        return Path :: get($path_type);
    }

    /**
     * Returns the names of the properties which are UI-wise filled by the integrated html editor
     * @return multitype:string
     */
    static function get_html_editors($html_editors = array())
    {
        $html_editors[] = self :: PROPERTY_DESCRIPTION;
        return $html_editors;
    }

    /**
     *
     * @return string
     */
    static function get_table_name()
    {
        return Utilities :: camelcase_to_underscores(array_pop(explode('\\', self :: CLASS_NAME)));

        // return Utilities :: camelcase_to_underscores(self :: CLASS_NAME);
    }

    /**
     *
     * @param $content_object_id integer
     * @return ContentObject An object inheriting from ContentObject
     */
    public static function get_by_id($content_object_id)
    {
        $rdm = RepositoryDataManager :: get_instance();
        return $rdm->retrieve_content_object($content_object_id);
    }

    /**
     *
     * @return array:
     */
    static function get_managers()
    {
        return array();
    }

    /**
     *
     * @return ExternalSync false
     */
    function get_synchronization_data()
    {
        if (! isset($this->synchronization_data))
        {
            $sync_conditions = array();
            $sync_conditions[] = new EqualityCondition(ExternalSync :: PROPERTY_CONTENT_OBJECT_ID, $this->get_id());
            // $sync_conditions[] = new EqualityCondition(self ::
            // PROPERTY_OWNER_ID, Session :: get_user_id(), self ::
            // get_table_name());
            $sync_condition = new AndCondition($sync_conditions);

            $this->synchronization_data = RepositoryDataManager :: get_instance()->retrieve_external_syncs(
                    $sync_condition)->next_result();
        }

        return $this->synchronization_data;
    }

    function set_synchronization_data($external_sync)
    {
        $this->synchronization_data = $external_sync;
    }

    function get_shared_users()
    {
        $condition = new EqualityCondition(ContentObjectUserShare :: PROPERTY_CONTENT_OBJECT_ID, $this->get_id());
        $shared_users = $this->get_data_manager()->retrieve_content_object_user_shares($condition);

        $users = array();

        while ($shared_user = $shared_users->next_result())
        {
            $users[$shared_user->get_user_id()] = $shared_user->get_user()->get_fullname();
        }

        asort($users);
        return $users;
    }

    function get_shared_groups()
    {
        $condition = new EqualityCondition(ContentObjectGroupShare :: PROPERTY_CONTENT_OBJECT_ID, $this->get_id());
        $shared_groups = $this->get_data_manager()->retrieve_content_object_group_shares($condition);

        $groups = array();

        while ($shared_group = $shared_groups->next_result())
        {
            $groups[$shared_group->get_group_id()] = $shared_group->get_group()->get_name();
        }

        asort($groups);
        return $groups;
    }

    function is_external()
    {
        $is_external = $this->get_synchronization_data();

        return isset($is_external);
    }

    static function get_content_object_type_namespace($type)
    {
        return 'repository\content_object\\' . $type;
    }

    static function type_exists($type)
    {
        $path = Path :: get_repository_content_object_path() . $type . '/';
        if (file_exists($path) && is_dir($path))
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    static function get_version_header()
    {
        return '<img src="' . Theme :: get_image_path() . 'versions_header.png" alt="' . Translation :: get('Versions') . '" title="' . Translation :: get(
                'Versions') . '" />';
    }

    function is_not_allowed_shared_objects()
    {
        return false;
    }

    /**
     * Checks if a user has the given right on this content object If user equals the owner then all rights are
     * available
     */
    function has_right($right, $user_id)
    {
        if (! $user_id)
        {
            $user_id = Session :: get_user_id();
        }

        if ($this->get_owner_id() == $user_id)
        {
            return true;
        }

        return RepositoryRights :: get_instance()->is_allowed_in_user_subtree($right, $this->get_id(),
                RepositoryRights :: TYPE_USER_CONTENT_OBJECT, $this->get_owner_id(), $user_id);
    }

    function share_with_user($user_id, $rights = array())
    {
        $location = ShareManager :: get_current_user_tree_location(Session :: get_user_id(), $this->get_id());
        if ($location)
        {
            $succes = true;
            if (! is_array($rights))
            {
                $rights = array($rights);
            }
            foreach ($rights as $right)
            {
                $succes &= RepositoryRights :: get_instance()->set_location_entity_right($right, $user_id,
                        UserEntity :: ENTITY_TYPE, $location->get_id());
            }
        }
        else
        {
            $succes = false;
        }
        return $succes;
    }

    function share_with_group($group_id, $rights = array())
    {
        $location = ShareManager :: get_current_user_tree_location(Session :: get_user_id(), $this->get_id());
        if ($location)
        {
            $succes = true;
            if (! is_array($rights))
            {
                $rights = array($rights);
            }
            foreach ($rights as $right)
            {
                $succes &= RepositoryRights :: get_instance()->set_location_entity_right($right, $group_id,
                        PlatformGroupEntity :: ENTITY_TYPE, $location->get_id());
            }
        }
        else
        {
            $succes = false;
        }
        return $succes;
    }

    /**
     * Creates a security code. The following values are used because they only change in special cases (copy, import).
     * This is important because it is hardcoded into some fields with included content e.g. description was used: If a
     * change was made to the description of an included object, the security code in the including object wouldn't
     * match anymore unless replaced.
     *
     * @return type
     */
    function calculate_security_code()
    {
        return sha1($this->get_id() . ':' . $this->get_creation_date());
    }

    /**
     * Function that updates the embedded links in fields like description
     *
     * @param $mapping array Each key(old_id) is mapped to its new object (an object is needed to calculate the security
     *            code)
     */
    function update_include_links(array $mapping)
    {
        if (count($this->get_included_content_objects()) == 0)
        {
            return;
        }

        $fields = static :: get_html_editors();

        foreach ($mapping as $old_id => $new_object)
        {
            $pattern[0] = '/(index\.php\?go=document_downloader&object=)' . $old_id . '(&security_code=)[^\&]+/';
            $pattern[1] = '/(<resource[^>]+source=)("' . $old_id . '")/';

            $security_code = $new_object->calculate_security_code();
            $replacement_string[0] = '${1}' . $new_object->get_id() . '${2}' . $security_code;
            $replacement_string[1] = '${1}"' . $new_object->get_id() . '"';

            foreach ($fields as $field)
            {
                $value = $this->get_default_property($field);
                $value = preg_replace($pattern, $replacement_string, $value);
                $this->set_default_property($field, $value);
            }

            $this->process_additional_include_links($pattern, $replacement_string);
  
        }

        $this->update();
    }

    /**
     * placeholder: if a specific type of content object stores include links outside the fields, this function
     * processes them e.g: documents: in the html file
     */
    function process_additional_include_links($pattern, $replacement_string)
    {
        return true;
    }

    function has_publications()
    {
        return $this->count_publications() > 0;
    }

    function has_parents()
    {
        return $this->count_parents() > 0;
    }

    function has_children()
    {
        return $this->count_children() > 0;
    }

    function has_attachments()
    {
        return $this->count_attachments() > 0;
    }

    function has_includes()
    {
        return $this->count_includes() > 0;
    }

    function has_attachers()
    {
        return $this->count_attachers() > 0;
    }

    function has_includers()
    {
        return $this->count_includers() > 0;
    }

    function count_publications($user)
    {
        return RepositoryDataManager :: count_publication_attributes($user, $this->get_id());
    }

    function count_parents()
    {
        $conditions[] = new EqualityCondition(ComplexContentObjectItem :: PROPERTY_REF, $this->get_id(),
                ComplexContentObjectItem :: get_table_name());

        $helper_types = RepositoryDataManager :: get_active_helper_types();

        foreach ($helper_types as $helper_type)
        {
            $subselect_condition = new EqualityCondition('reference_id', $this->get_id());
            $conditions[] = new SubselectCondition(ComplexContentObjectItem :: PROPERTY_REF,
                    ContentObject :: PROPERTY_ID, $helper_type, $subselect_condition);
        }

        $condition = new OrCondition($conditions);
        return RepositoryDataManager :: get_instance()->count_complex_content_object_items($condition);
    }

    function count_children()
    {
        $condition = new EqualityCondition(ComplexContentObjectItem :: PROPERTY_PARENT, $this->get_id(),
                ComplexContentObjectItem :: get_table_name());
        return RepositoryDataManager :: get_instance()->count_complex_content_object_items($condition);
    }

    function count_attachments()
    {
        return RepositoryDataManager :: get_instance()->count_objects_which_object_attaches($this->get_id());
    }

    function count_includes()
    {
        return RepositoryDataManager :: get_instance()->count_objects_which_object_includes($this->get_id());
    }

    function count_attachers()
    {
        return RepositoryDataManager :: get_instance()->count_objects_to_which_object_is_attached($this->get_id());
    }

    function count_includers()
    {
        return RepositoryDataManager :: get_instance()->count_objects_in_which_object_is_included($this->get_id());
    }

    function get_publications($user, $offset, $count, $order_by)
    {
        $publication_attributes = RepositoryDataManager :: get_content_object_publication_attributes($user,
                $this->get_id());
        return $publication_attributes = array_splice($publication_attributes, $offset, $count);
    }

    function get_parents($order_by = array(), $offset = null, $count = null)
    {
        $conditions[] = new EqualityCondition(ComplexContentObjectItem :: PROPERTY_REF, $this->get_id(),
                ComplexContentObjectItem :: get_table_name());

        $helper_types = RepositoryDataManager :: get_active_helper_types();

        foreach ($helper_types as $helper_type)
        {
            $subselect_condition = new EqualityCondition('reference_id', $this->get_id());
            $conditions[] = new SubselectCondition(ComplexContentObjectItem :: PROPERTY_REF,
                    ContentObject :: PROPERTY_ID, $helper_type, $subselect_condition);
        }

        $condition = new OrCondition($conditions);
        return RepositoryDataManager :: get_instance()->retrieve_complex_content_object_items($condition, $order_by,
                $offset, $count);
    }

    function get_children($order_by = array(), $offset = null, $count = null)
    {
        $condition = new EqualityCondition(ComplexContentObjectItem :: PROPERTY_PARENT, $this->get_id(),
                ComplexContentObjectItem :: get_table_name());
        return RepositoryDataManager :: get_instance()->retrieve_complex_content_object_items($condition, $order_by,
                $offset, $count);
    }

    function get_attachments($order_by = array(), $offset = null, $count = null)
    {
        return RepositoryDataManager :: get_instance()->retrieve_objects_which_object_attaches($this->get_id(),
                $order_by, $offset, $count);
    }

    function get_attachers($order_by = array(), $offset = null, $count = null)
    {
        return RepositoryDataManager :: get_instance()->retrieve_objects_to_which_object_is_attached($this->get_id(),
                $order_by, $offset, $count);
    }

    function get_includes($order_by = array(), $offset = null, $count = null)
    {
        return RepositoryDataManager :: get_instance()->retrieve_objects_which_object_includes($this->get_id(),
                $order_by, $offset, $count);
    }

    function get_includers($order_by = array(), $offset = null, $count = null)
    {
        return RepositoryDataManager :: get_instance()->retrieve_objects_in_which_object_is_included($this->get_id(),
                $order_by, $offset, $count);
    }

    function get_virtual_path()
    {
        $category_id = $this->get_parent_id();
        $virtual_path = array();

        while ($category_id != 0)
        {
            $category = $this->get_data_manager()->retrieve_category($category_id);
            $category_id = $category->get_parent();
            array_unshift($virtual_path, Filesystem :: create_safe_name($category->get_name()));
        }

        array_unshift($virtual_path, Filesystem :: create_safe_name($this->get_owner_fullname()));

        return implode(DIRECTORY_SEPARATOR, $virtual_path) . DIRECTORY_SEPARATOR;
    }

    function get_packages_from_filesystem()
    {
        $types = array();

        $directories = Filesystem :: get_directory_content(Path :: get_repository_content_object_path(),
                Filesystem :: LIST_DIRECTORIES, false);

        foreach ($directories as $directory)
        {
            $types[] = self :: get_content_object_type_namespace(basename($directory));
        }

        return $types;
    }

    static function get_active_status_types()
    {
        return array(self :: STATE_NORMAL, self :: STATE_RECYCLED, self :: STATE_AUTOSAVE, self :: STATE_BACKUP);
    }

    static function get_inactive_status_types()
    {
        return array(self :: STATE_NORMAL + self :: STATE_INACTIVE, self :: STATE_RECYCLED + self :: STATE_INACTIVE,
                self :: STATE_AUTOSAVE + self :: STATE_INACTIVE, self :: STATE_BACKUP + self :: STATE_INACTIVE);
    }
}
?>