<?php
namespace repository;

use common\libraries\ComplexContentObjectSupport;
use common\libraries\EqualityCondition;
use common\libraries\Path;
use common\libraries\Filesystem;
use common\libraries\Text;

use repository\content_object\document\Document;
use repository\content_object\hotpotatoes\Hotpotatoes;
use repository\content_object\learning_path\LearningPath;

/**
 * $Id: content_object_copier.class.php 204 2009-11-13 12:51:30Z kariboe $
 * @package repository.lib
 */
/**
 * Class that makes it possible to move a content object to another user repository.
 * This class takes the following things in account:
 * Simple content object
 * The children of a complex object
 * The included objects
 * The attached objects
 * The physical files (documents, hotpotatoes, scorm)
 * The references of LearningPathItem / PortfolioItem
 * The LearningPath prerequisites (only for chamilo learning paths)
 * Links to other files in a description field
 *
 * @author Sven Vanpoucke
 */
class ContentObjectOwnerChanger
{
    /**
     * The repository data manager
     * @var RepositoryDataManager
     */
    private $rdm;
    
    /**
     * The target repository
     * @var Int
     */
    private $target_repository;
    
    /**
     * Counter to count the items that failed while copying
     * @var Int
     */
    private $failed;
    
    /**
     * The target category id
     * @var int
     */
    private $category_id;

    /**
     * Constructor
     * Initialize the repository data manager
     * Set the target repository
     *
     * @param Int $target_repository
     */
    function __construct()
    {
        $this->rdm = RepositoryDataManager :: get_instance();
    }

    /**
     * Move a content object to the target repository
     *
     * @param Int $co
     * @param Int $category_id
     * @return Int ID of the new content object
     */
    function move_content_object($co, $target_repository = 0, $category_id = 0)
    {
        $this->failed = 0;
        $this->target_repository = $target_repository;
        $this->category_id = $category_id;
        
        return $this->change_content_object_owner($co);
    }

    /**
     * Returns how many items have failed
     * @return int
     */
    function get_failed()
    {
        return $this->failed;
    }

    /**
     * Create a content object in the target repository
     *
     * @param ContentObject $co
     * @return ContentObject the moved content object
     */
    private function change_content_object_owner($co)
    {
        $old_user_id = $co->get_owner_id();
        $old_location = $co->get_rights_location();
        
        if ($old_user_id == $this->target_repository)
        {
            return $co;
        }
        
        $co->set_owner_id($this->target_repository);
        
        // Set the category
        if (RepositoryDataManager :: is_helper_type($co->get_type()))
        {
            $co->set_parent_id(0);
        }
        else
        {
            $co->set_parent_id($this->category_id);
        }
        
        if (! $co->update())
        {
            $this->failed ++;
            return false;
        }
        
        // Process the versions
        $this->move_versions($co);
        
        // Process the children
        $this->move_complex_children($co);
        
        // Process the included items, attachments and sync data
        $this->move_includes($co);
        $this->move_attachments($co);
        
        // Process the physical files
        $this->move_files($co, $old_user_id);
        
        // Process the locations
        $this->process_locations($co, $old_location);
        
        if (in_array($co->get_type(), RepositoryDataManager :: get_active_helper_types()))
        {
            $co = $this->rdm->retrieve_content_object($co->get_reference());
            if ($co)
            {
                $this->change_content_object_owner($co);
            }
        }
        
        return $co;
    }

    private function move_versions($co)
    {
        if ($co->is_latest_version())
        {
            $versions = $this->rdm->retrieve_content_object_versions($co, false);
            foreach ($versions as $version)
            {
                $this->change_content_object_owner($version);
            }
        }
    }

    /**
     * Move the children of a content object (both items and wrappers)
     *
     * @param ContentObject $co
     */
    private function move_complex_children($co)
    {
        if (! $co instanceof ComplexContentObjectSupport)
        {
            return;
        }
        
        $item_references = array();
        
        $condition = new EqualityCondition(ComplexContentObjectItem :: PROPERTY_PARENT, $co->get_id(), ComplexContentObjectItem :: get_table_name());
        
        $items = $this->rdm->retrieve_complex_content_object_items($condition);
        while ($item = $items->next_result())
        {
            $co = $this->rdm->retrieve_content_object($item->get_ref());
            $this->change_content_object_owner($co);
            
            $item->set_user_id($this->target_repository);
            $item->update();
        }
    }

    /**
     * Move the included content objects
     *
     * @param ContentObject $co
     */
    private function move_includes($co)
    {
        $includes = $co->get_included_content_objects();
        
        foreach ($includes as $include)
        {
            $include = $this->rdm->retrieve_content_object($include->get_id());
            $this->change_content_object_owner($include);
        }
    }

    /**
     * Move the attached content objects
     *
     * @param ContentObject $co
     */
    private function move_attachments($co)
    {
        $attachments = $co->get_attached_content_objects();
        
        foreach ($attachments as $attachment)
        {
            $attachment = $this->rdm->retrieve_content_object($attachment->get_id());
            $this->change_content_object_owner($attachment);
        }
    }

    /**
     * Moves the physical files
     * @param ContentObject $co;
     */
    private function move_files($co, $old_user_id)
    {
        $type = $co->get_type();
        switch ($type)
        {
            case Document :: get_type_name() :
                return $this->move_document_files($co);
            case Hotpotatoes :: get_type_name() :
                return $this->move_hotpotatoes_files($co, $old_user_id);
            case LearningPath :: get_type_name() :
                if ($co->get_version() == 'SCORM1.2' || $co->get_version() == 'SCORM2004')
                {
                    return $this->move_scorm_files($co, $old_user_id);
                }
            default :
                return;
        }
    }

    /**
     * Move the files from the content object type document
     *
     * @param Document $co
     */
    private function move_document_files($co)
    {
        $base_path = Path :: get(SYS_REPO_PATH);
        $new_path = $this->target_repository . '/' . Text :: char_at($co->get_hash(), 0);
        $new_full_path = $base_path . $new_path;
        Filesystem :: create_dir($new_full_path);
        
        $new_hash = Filesystem :: create_unique_name($new_full_path, $co->get_hash());
        $new_full_path .= '/' . $new_hash;
        
        Filesystem :: move_file($co->get_full_path(), $new_full_path);
        
        $co->set_hash($new_hash);
        $co->set_path($new_path . '/' . $new_hash);
        $co->update();
    }

    /**
     * Move the files from the content object type hotpotatoes
     *
     * @param Hotpotatoes $co
     */
    private function move_hotpotatoes_files($co, $old_user_id)
    {
        $filename = basename($co->get_path());
        $base_path = Path :: get(SYS_HOTPOTATOES_PATH) . $this->target_repository . '/';
        
        $new_path = Filesystem :: create_unique_name($base_path, dirname($co->get_path()));
        $new_full_path = $base_path . $new_path;
        Filesystem :: create_dir($new_full_path);
        
        Filesystem :: recurse_move(Path :: get(SYS_HOTPOTATOES_PATH) . $old_user_id . '/' . dirname($co->get_path()), $new_full_path, false);
        
        $co->set_path($new_path . '/' . $filename);
        $co->update();
    }

    /**
     * Move the files from the content object type learning path
     *
     * @param LearningPath $co
     */
    private function move_scorm_files($co, $old_user_id)
    {
        $base_path = Path :: get(SYS_SCORM_PATH) . $this->target_repository . '/';
        
        $new_folder = Filesystem :: create_unique_name($base_path, $co->get_path());
        $new_full_path = $base_path . $new_folder;
        Filesystem :: create_dir($new_full_path);
        
        Filesystem :: recurse_move(Path :: get(SYS_SCORM_PATH) . $old_user_id . '/' . $co->get_path(), $new_full_path, false);
        
        $co->set_path($new_folder);
        $co->update();
    }

    /**
     * Delete the old rights location and create a new one in the target repository
     *
     * @param ContentObject $co
     */
    private function process_locations($co, $old_location)
    {
        if ($old_location)
        {
            $old_location->delete();
        }
        
        $parent = $this->category_id;
        if (! $parent)
        {
            $parent_id = RepositoryRights :: get_instance()->get_user_root_id($co->get_owner_id());
        }
        else
        {
            $parent_id = RepositoryRights :: get_instance()->get_location_id_by_identifier_from_user_subtree(RepositoryRights :: TYPE_USER_CATEGORY, $this->category_id, $co->get_owner_id());
        }
        
        RepositoryRights :: get_instance()->create_location_in_user_tree(RepositoryRights :: TYPE_USER_CONTENT_OBJECT, $co->get_id(), $parent_id, $co->get_owner_id());
    }
}

?>