<?php
namespace repository;

use common\libraries\Condition;
use common\libraries\Translation;
use common\libraries\Mdb2Database;
use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\InequalityCondition;
use common\libraries\NotCondition;
use common\libraries\InCondition;
use common\libraries\AndCondition;
use common\libraries\ConditionTranslator;
use common\libraries\ObjectTableOrder;
use common\libraries\OrCondition;
use common\libraries\DataClass;
use common\libraries\SubselectCondition;
use common\libraries\Session;
use repository\content_object\forum\Forum;
use repository\content_object\forum_topic\ForumTopic;
use repository\content_object\forum_topic\ForumPost;
use repository\content_object\document\Document;
use admin\AdminDataManager;
use common\libraries\Registration;
use rights\UserEntity;
use rights\PlatformGroupEntity;
use MDB2;
use Exception;

/**
 * This is a data manager that uses a database for storage. It was written for MySQL, but should be compatible with most
 * SQL flavors.
 *
 * @author Tim De Pauw
 * @author Bart Mollet
 * @author Hans De Bisschop
 * @author Dieter De Neef
 * @package repository.lib.data_manager
 */
class Mdb2RepositoryDataManager extends Mdb2Database implements RepositoryDataManagerInterface
{
    const ALIAS_CONTENT_OBJECT_VERSION_TABLE = 'lov';
    const ALIAS_CONTENT_OBJECT_ATTACHMENT_TABLE = 'loa';
    const ALIAS_TYPE_TABLE = 'tt';
    const ALIAS_ADMIN_REGISTRATION_TABLE = 'reg';

    // Inherited.
    function initialize()
    {
        parent :: initialize();
        // PEAR :: setErrorHandling(PEAR_ERROR_CALLBACK, array(get_class(),
        // 'handle_error'));
        $this->set_prefix('repository_');
    }

    // Inherited.
    function determine_content_object_type($id)
    {
        $content_object_alias = $this->get_alias(ContentObject :: get_table_name());
        $condition = new EqualityCondition(ContentObject :: PROPERTY_ID, $id);

        $query = 'SELECT ' . $this->escape_column_name(ContentObject :: PROPERTY_TYPE, $content_object_alias) . ' FROM ' . $this->escape_table_name(
                ContentObject :: get_table_name()) . ' AS ' . $content_object_alias;
        $record = $this->retrieve_row($query, ContentObject :: get_table_name(), $condition);
        return $record[ContentObject :: PROPERTY_TYPE];
    }

    // Inherited.
    function retrieve_content_object($id, $type = null)
    {
        if (! isset($id) || strlen($id) == 0 || $id == DataClass :: NO_UID)
        {
            return null;
        }

        if (is_null($type))
        {
            $type = $this->determine_content_object_type($id);
        }

        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObject :: PROPERTY_ID, $id);
        $conditions[] = new InCondition(ContentObject :: PROPERTY_STATE, ContentObject :: get_active_status_types());
        $condition = new AndCondition($conditions);

        if (RepositoryDataManager :: is_extended_type($type))
        {
            $content_object_alias = $this->get_alias(ContentObject :: get_table_name());

            $query = 'SELECT * FROM ' . $this->escape_table_name(ContentObject :: get_table_name()) . ' AS ' . $content_object_alias;
            $query .= ' JOIN ' . $this->escape_table_name($type) . ' AS ' . self :: ALIAS_TYPE_TABLE . ' ON ' . $this->escape_column_name(
                    ContentObject :: PROPERTY_ID, $content_object_alias) . '=' . $this->escape_column_name(
                    ContentObject :: PROPERTY_ID, self :: ALIAS_TYPE_TABLE);

            $record = $this->retrieve_row($query, ContentObject :: get_table_name(), $condition);
        }
        else
        {
            $record = $this->retrieve_record(ContentObject :: get_table_name(), $condition);
        }

        if ($record)
        {
            return self :: record_to_content_object($record, isset($type));
        }
        else
        {
            return false;
        }
    }

    function retrieve_content_object_by_condition($condition, $type)
    {
        if (RepositoryDataManager :: is_extended_type($type))
        {
            $content_object_alias = $this->get_alias(ContentObject :: get_table_name());
            $type_alias = $this->get_alias($type);

            $query = 'SELECT * FROM ' . $this->escape_table_name(ContentObject :: get_table_name()) . ' AS ' . $content_object_alias;
            $query .= ' JOIN ' . $this->escape_table_name($type) . ' AS ' . $type_alias . ' ON ' . $this->escape_column_name(
                    ContentObject :: PROPERTY_ID, $content_object_alias) . '=' . $this->escape_column_name(
                    ContentObject :: PROPERTY_ID, $type_alias);

            $record = $this->retrieve_row($query, ContentObject :: get_table_name(), $condition);
        }
        else
        {
            $record = $this->retrieve_record(ContentObject :: get_table_name(), $condition);
        }

        if ($record)
            return self :: record_to_content_object($record, true);
    }

    // Inherited.
    // TODO: Extract methods.
    function retrieve_content_objects($condition = null, $order_by = array(), $offset = 0, $max_objects = -1, $query = null)
    {
        if (is_null($query))
        {
            $query = 'SELECT * FROM ';
            $query .= $this->escape_table_name(ContentObject :: get_table_name()) . ' AS ' . $this->get_alias(
                    ContentObject :: get_table_name());
        }

        $version_condition = new NotCondition(
                new EqualityCondition(ContentObject :: PROPERTY_CURRENT, ContentObject :: CURRENT_OLD));
        $condition = ! isset($condition) ? $version_condition : new AndCondition(array($condition, $version_condition));

        if (isset($condition))
        {
            $translator = ConditionTranslator :: factory($this);
            $query .= $translator->render_query($condition);
        }

        $orders = array();
        foreach ($order_by as $order)
        {
            $orders[] = $this->escape_column_name($order->get_property(), $order->get_alias()) . ' ' . ($order->get_direction() == SORT_DESC ? 'DESC' : 'ASC');
        }
        if (count($orders))
        {
            $query .= ' ORDER BY ' . implode(', ', $orders);
        }

        if ($max_objects < 0)
        {
            $max_objects = null;
        }

        $this->set_limit(intval($max_objects), intval($offset));
        $res = $this->query($query);
        return new DatabaseContentObjectResultSet($this, $res, false);
    }

    function retrieve_active_content_objects($condition = null, $order_by = array(), $offset = 0, $max_objects = -1, $query = null,
            $type = false)
    {
        $content_object_table_alias = $this->get_alias(ContentObject :: get_table_name());

        if (is_null($query))
        {
            $query = 'SELECT ' . $content_object_table_alias . '.* FROM ';
            $query .= $this->escape_table_name(ContentObject :: get_table_name()) . ' AS ' . $content_object_table_alias;
            if (RepositoryDataManager :: is_extended_type($type))
            {
                $query .= ' JOIN ' . $this->escape_table_name($type) . ' AS ' . self :: ALIAS_TYPE_TABLE . ' ON ' . $this->escape_column_name(
                        ContentObject :: PROPERTY_ID, $content_object_table_alias) . '=' . $this->escape_column_name(
                        ContentObject :: PROPERTY_ID, self :: ALIAS_TYPE_TABLE);
            }
        }

        $conditions = array();

        if (isset($condition))
        {
            $conditions[] = $condition;
        }

        $conditions[] = new NotCondition(
                new EqualityCondition(ContentObject :: PROPERTY_CURRENT, ContentObject :: CURRENT_OLD));

        $condition = new AndCondition($conditions);

        $translator = ConditionTranslator :: factory($this);
        $query .= $translator->render_query($condition);

        $orders = array();
        foreach ($order_by as $order)
        {
            $order->set_alias($content_object_table_alias);
            $orders[] = $this->escape_column_name($order->get_property()) . ' ' . ($order->get_direction() == SORT_DESC ? 'DESC' : 'ASC');
        }

        if (count($orders))
        {
            $query .= ' ORDER BY ' . implode(', ', $orders);
        }

        if ($max_objects < 0)
        {
            $max_objects = null;
        }

        $this->set_limit(intval($max_objects), intval($offset));
        $res = $this->query($query);

        return new DatabaseContentObjectResultSet($this, $res, false);
    }

    function retrieve_type_content_objects($type, $condition = null, $order_by = array(), $offset = 0, $max_objects = -1)
    {
        $content_object_alias = $this->get_alias(ContentObject :: get_table_name());

        $query = 'SELECT * FROM ' . $this->escape_table_name(ContentObject :: get_table_name()) . ' AS ' . $content_object_alias;

        if (ContentObject :: is_extended_type($type))
        {
            $type_alias = $this->get_alias($type);
            $query .= ' JOIN ' . $this->escape_table_name($type) . ' AS ' . $type_alias . ' ON ' . $this->escape_column_name(
                    ContentObject :: PROPERTY_ID, $content_object_alias) . ' = ' . $this->escape_column_name(
                    ContentObject :: PROPERTY_ID, $type_alias);
        }
        else
        {
            $type_condition = new EqualityCondition(ContentObject :: PROPERTY_TYPE, $type);
            $condition = isset($condition) ? new AndCondition($type_condition, $condition) : $type_condition;
        }

        $version_condition = new NotCondition(
                new EqualityCondition(ContentObject :: PROPERTY_CURRENT, ContentObject :: CURRENT_OLD));
        $condition = ! isset($condition) ? $version_condition : new AndCondition(array($condition, $version_condition));

        if (isset($condition))
        {
            $translator = ConditionTranslator :: factory($this, $content_object_alias);
            $query .= $translator->render_query($condition);
        }

        $orders = array();
        foreach ($order_by as $order)
        {
            $orders[] = $this->escape_column_name($order->get_property()) . ' ' . ($order->get_direction() == SORT_DESC ? 'DESC' : 'ASC');
        }

        if (count($orders))
        {
            $query .= ' ORDER BY ' . implode(', ', $orders);
        }

        if ($max_objects < 0)
        {
            $max_objects = null;
        }

        $this->set_limit(intval($max_objects), intval($offset));
        $res = $this->query($query);
        return new DatabaseContentObjectResultSet($this, $res, true);
    }

    // Inherited.
    function retrieve_additional_content_object_properties($content_object)
    {
        $type = $content_object->get_type();
        if (! RepositoryDataManager :: is_extended_type($type))
        {
            return array();
        }
        $array = array_map(array($this, 'escape_column_name'), $content_object->get_additional_property_names());

        if (count($array) == 0)
        {
            $array = array("*");
        }

        $query = 'SELECT ' . implode(',', $array) . ' FROM ' . $this->escape_table_name($type) . ' WHERE ' . $this->escape_column_name(
                ContentObject :: PROPERTY_ID) . '=' . $this->quote($content_object->get_id());

        $this->set_limit(1, 0);
        $res = $this->query($query);
        $row = $res->fetchRow(MDB2_FETCHMODE_ASSOC);

        $properties = $content_object->get_additional_property_names();

        $additional_properties = array();
        if (count($properties) > 0)
        {
            foreach ($properties as $prop)
            {
                if (is_resource($row[$prop]))
                {
                    $data = '';
                    while (! feof($row[$prop]))
                        $data .= fread($row[$prop], 1024);
                    $row[$prop] = $data;
                }
                $additional_properties[$prop] = $row[$prop];
            }
        }

        $res->free();

        return $additional_properties;
    }

    // Inherited.
    // TODO: Extract methods; share stuff with retrieve_content_objects.
    function count_content_objects($condition = null, $query = null)
    {
        if (is_null($query))
        {
            $query = 'SELECT COUNT(' . $this->get_alias(ContentObject :: get_table_name()) . '.' . $this->escape_column_name(
                    ContentObject :: PROPERTY_OBJECT_NUMBER) . ') FROM ' . $this->escape_table_name('content_object') . ' AS ' . $this->get_alias(
                    ContentObject :: get_table_name());
        }

        $version_condition = new NotCondition(
                new EqualityCondition(ContentObject :: PROPERTY_CURRENT, ContentObject :: CURRENT_OLD));
        $condition = ! isset($condition) ? $version_condition : new AndCondition(array($condition, $version_condition));

        return $this->count_result_set($query, ContentObject :: get_table_name(), $condition);
    }

    function count_type_content_objects($type, $condition = null)
    {
        $content_object_alias = $this->get_alias(ContentObject :: get_table_name());
        $type_alias = $this->get_alias($type);

        if (ContentObject :: is_extended_type($type))
        {
            $query = 'SELECT COUNT(' . $this->escape_column_name(ContentObject :: PROPERTY_OBJECT_NUMBER,
                    $content_object_alias) . ') FROM ' . $this->escape_table_name(ContentObject :: get_table_name()) . ' AS ' . $content_object_alias . ' JOIN ' . $this->escape_table_name(
                    $type) . ' AS ' . $type_alias . ' ON ' . $this->escape_column_name(ContentObject :: PROPERTY_ID,
                    $content_object_alias) . ' = ' . $this->escape_column_name(ContentObject :: PROPERTY_ID, $type_alias);
        }
        else
        {
            $query = 'SELECT COUNT(' . $this->escape_column_name(ContentObject :: PROPERTY_OBJECT_NUMBER,
                    $content_object_alias) . ') FROM ' . $this->escape_table_name(ContentObject :: get_table_name()) . ' AS ' . $content_object_alias;
            $match = new EqualityCondition(ContentObject :: PROPERTY_TYPE, $type);
            $condition = isset($condition) ? new AndCondition(array($match, $condition)) : $match;
        }

        $version_condition = new NotCondition(
                new EqualityCondition(ContentObject :: PROPERTY_CURRENT, ContentObject :: CURRENT_OLD));
        $condition = ! isset($condition) ? $version_condition : new AndCondition(array($condition, $version_condition));

        return $this->count_result_set($query, ContentObject :: get_table_name(), $condition);
    }

    // Inherited
    function count_content_object_versions($object)
    {
        $condition = new EqualityCondition(ContentObject :: PROPERTY_OBJECT_NUMBER, $object->get_object_number());
        return $this->count_objects(ContentObject :: get_table_name(), $condition);
    }

    function get_next_content_object_number()
    {
        return $this->get_next_id(ContentObject :: get_table_name() . '_number');
    }

    function count_active_content_objects($condition = null, $query = null, $type = false)
    {
        if (is_null($query))
        {
            $content_object_table_alias = $this->get_alias(ContentObject :: get_table_name());

            $query = 'SELECT count(1) FROM ';
            $query .= $this->escape_table_name(ContentObject :: get_table_name()) . ' AS ' . $content_object_table_alias;

            if (RepositoryDataManager :: is_extended_type($type))
            {
                $query .= ' JOIN ' . $this->escape_table_name($type) . ' AS ' . self :: ALIAS_TYPE_TABLE . ' ON ' . $this->escape_column_name(
                        ContentObject :: PROPERTY_ID, $content_object_table_alias) . '=' . $this->escape_column_name(
                        ContentObject :: PROPERTY_ID, self :: ALIAS_TYPE_TABLE);
            }

            $conditions = array();

            if (isset($condition))
            {
                $conditions[] = $condition;
            }

            $conditions[] = new NotCondition(
                    new EqualityCondition(ContentObject :: PROPERTY_CURRENT, ContentObject :: CURRENT_OLD));

            $condition = new AndCondition($conditions);
        }
        return $this->count_result_set($query, ContentObject :: get_table_name(), $condition);
    }

    // Inherited.
    function create_content_object($object, $type)
    {
        $props = array();
        foreach ($object->get_default_properties() as $key => $value)
        {
            $props[$this->escape_column_name($key)] = $value;
        }
        $props[$this->escape_column_name(ContentObject :: PROPERTY_ID)] = $object->get_id();
        $props[$this->escape_column_name(ContentObject :: PROPERTY_TYPE)] = $object->get_type();
        $props[$this->escape_column_name(ContentObject :: PROPERTY_CREATION_DATE)] = $object->get_creation_date();
        $props[$this->escape_column_name(ContentObject :: PROPERTY_MODIFICATION_DATE)] = $object->get_modification_date();
        $props[$this->escape_column_name(ContentObject :: PROPERTY_ID)] = $this->get_better_next_id('content_object',
                'id');
        $this->get_connection()->loadModule('Extended');
        $this->get_connection()->extended->autoExecute($this->get_table_name('content_object'), $props,
                MDB2_AUTOQUERY_INSERT);
        $object->set_id(
                $this->get_connection()->extended->getAfterID(
                        $props[$this->escape_column_name(ContentObject :: PROPERTY_ID)], 'content_object'));
        if ($object->is_extended())
        {
            $props = array();
            foreach ($object->get_additional_properties() as $key => $value)
            {
                $props[$this->escape_column_name($key)] = $value;
            }
            $props[$this->escape_column_name(ContentObject :: PROPERTY_ID)] = $object->get_id();
            $this->get_connection()->extended->autoExecute($this->get_table_name($object->get_type()), $props,
                    MDB2_AUTOQUERY_INSERT);
        }

        if ($type == 'version')
        {
            $conditions = array();
            $conditions[] = new EqualityCondition(ContentObject :: PROPERTY_OBJECT_NUMBER, $object->get_object_number());
            $conditions[] = new NotCondition(new EqualityCondition(ContentObject :: PROPERTY_ID, $object->get_id()));
            $condition = new AndCondition($conditions);
            $this->update_objects(ContentObject :: get_table_name(), array(ContentObject :: PROPERTY_CURRENT => 0),
                    $condition);
        }

        return true;
    }

    // Inherited.
    function update_content_object($object)
    {
        $where = $this->escape_column_name(ContentObject :: PROPERTY_ID) . '=' . $object->get_id();
        $props = array();
        foreach ($object->get_default_properties() as $key => $value)
        {
            $props[$this->escape_column_name($key)] = $value;
        }
        $props[$this->escape_column_name(ContentObject :: PROPERTY_CREATION_DATE)] = $object->get_creation_date();
        $props[$this->escape_column_name(ContentObject :: PROPERTY_MODIFICATION_DATE)] = $object->get_modification_date();
        $this->get_connection()->loadModule('Extended');
        $this->get_connection()->extended->autoExecute($this->get_table_name('content_object'), $props,
                MDB2_AUTOQUERY_UPDATE, $where);
        if ($object->is_extended())
        {
            $props = array();
            foreach ($object->get_additional_properties() as $key => $value)
            {
                $props[$this->escape_column_name($key)] = $value;
            }
            $this->get_connection()->extended->autoExecute($this->get_table_name($object->get_type()), $props,
                    MDB2_AUTOQUERY_UPDATE, $where);
        }
        return true;
    }

    // Inherited.
    function retrieve_content_object_by_user($user_id)
    {
        $condition = new EqualityCondition(ContentObject :: PROPERTY_OWNER_ID, $user_id);
        return $this->retrieve_objects(ContentObject :: get_table_name(), $condition, array(), ContentObject :: CLASS_NAME);
    }

    function delete_content_object_by_id($object_id)
    {
        $object = $this->retrieve_content_object($object_id);
        return $this->delete_content_object($object);
    }

    // Inherited.
    function delete_content_object($object)
    {
        if (! RepositoryDataManager :: content_object_deletion_allowed($object))
        {
            return false;
        }
        // Delete children
        // Delete all types of attachments (only the links, not the actual
        // objects)
        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObjectAttachment :: PROPERTY_CONTENT_OBJECT_ID, $object->get_id());
        // $conditions[] = new EqualityCondition(ContentObjectAttachment ::
        // PROPERTY_ATTACHMENT_ID, $object->get_id());
        $condition = new OrCondition($conditions);
        $this->delete_content_object_attachments($condition);

        // Delete all includes (only the links, not the actual objects)
        $conditions = array();
        $conditions[] = new EqualityCondition('content_object_id', $object->get_id());
        $conditions[] = new EqualityCondition('include_id', $object->get_id());
        $condition = new OrCondition($conditions);
        $this->delete_objects('content_object_include', $condition);

        // Delete extended properties record
        if (RepositoryDataManager :: is_extended_type(Utilities :: get_classname_from_object($object, true)))
        {
            $condition = new EqualityCondition(ContentObject :: PROPERTY_ID, $object->get_id());
            $this->delete_objects(Utilities :: get_classname_from_object($object, true), $condition);
        }

        // Delete synchronization with external repositories infos
        $condition = new EqualityCondition(ExternalSync :: PROPERTY_CONTENT_OBJECT_ID, $object->get_id());
        $this->delete_objects(ExternalSync :: get_table_name(), $condition);

        // Delete object
        $condition = new EqualityCondition(ContentObject :: PROPERTY_ID, $object->get_id());
        $this->delete_objects(ContentObject :: get_table_name(), $condition);

        return true;
    }

    // Inherited.
    function delete_content_object_version($object)
    {
        if (! RepositoryDataManager :: content_object_deletion_allowed($object, 'version'))
        {
            return false;
        }

        // Delete object
        $query = 'DELETE FROM ' . $this->escape_table_name(ContentObject :: get_table_name()) . ' WHERE ' . $this->escape_column_name(
                ContentObject :: PROPERTY_ID) . '=' . $this->quote($object->get_id());
        $res = $this->exec($query);

        // Delete location
        $location = RepositoryRights :: get_instance()->get_location_by_identifier_from_users_subtree(RepositoryRights :: TYPE_USER_CONTENT_OBJECT, $object->get_id(), $object->get_owner_id());
        if ($location)
        {
            if (! $location->delete())
            {
                return false;
            }
        }
        
        if ($object->is_extended())
        {
            $query = 'DELETE FROM ' . $this->escape_table_name($object->get_type()) . ' WHERE ' . $this->escape_column_name(
                    ContentObject :: PROPERTY_ID) . '=' . $this->quote($object->get_id());
            $res = $this->exec($query);
        }

        return true;
    }

    // Inherited.
    function delete_content_object_attachments(Condition $condition)
    {
        return $this->delete_objects(ContentObjectAttachment :: get_table_name(), $condition);
    }

    // Inherited.
    function delete_all_content_objects()
    {
        foreach (RepositoryDataManager :: get_registered_types() as $type)
        {
            if (RepositoryDataManager :: is_extended_type($type))
            {
                $this->delete_objects($this->get_table_name($type));
            }
        }

        $this->delete_objects($this->get_table_name(ContentObject :: get_table_name()));
    }

    function is_only_document_occurence($path)
    {
        $condition = new EqualityCondition(Document :: PROPERTY_PATH, $path);
        $count = $this->count_objects(Document :: get_type_name(), $condition);

        return ($count == 1 ? true : false);
    }

    /**
     *
     * @param $condition Condition
     * @return Array
     */
    function retrieve_attached_content_object_ids(Condition $condition)
    {
        return $this->retrieve_distinct(ContentObjectAttachment :: get_table_name(),
                ContentObjectAttachment :: PROPERTY_ATTACHMENT_ID, $condition, array(), ContentObjectAttachment :: CLASS_NAME);
    }

    /**
     *
     * @param $condition Condition
     * @return boolean ContentObject
     */
    function retrieve_attached_content_object(Condition $condition)
    {
        $attachment_ids = $this->retrieve_attached_content_object_ids($condition);

        if (count($attachment_ids) != 1)
        {
            return false;
        }
        else
        {
            return $this->retrieve_content_object($attachment_ids[0]);
        }
    }

    // Inherited.
    function retrieve_attached_content_objects(Condition $condition)
    {
        $attachment_ids = $this->retrieve_attached_content_object_ids($condition);

        // Add non-existing element to avoid problems with
        if (count($attachment_ids) == 0)
        {
            $attachment_ids[] = - 1;
        }

        $object_condition = new InCondition(ContentObject :: PROPERTY_ID, $attachment_ids,
                ContentObject :: get_table_name());
        return $this->retrieve_content_objects($object_condition)->as_array();
    }

    function count_objects_to_which_object_is_attached($id)
    {
        $repository_content_object_alias = $this->get_alias(ContentObject :: get_table_name());
        $repository_content_object_attachment_alias = $this->get_alias(ContentObjectAttachment :: get_table_name());

        $query = 'SELECT count(*) ';
        $query .= ' FROM ' . $this->escape_table_name(ContentObjectAttachment :: get_table_name()) . ' as ' . $repository_content_object_attachment_alias;
        $query .= ' JOIN ' . $this->escape_table_name(ContentObject :: get_table_name()) . ' as ' . $repository_content_object_alias;
        $query .= ' ON ' . $this->escape_column_name(ContentObjectAttachment :: PROPERTY_CONTENT_OBJECT_ID,
                $repository_content_object_attachment_alias) . ' = ' . $this->escape_column_name(
                ContentObject :: PROPERTY_ID, $repository_content_object_alias);

        $condition = new EqualityCondition(ContentObjectAttachment :: PROPERTY_ATTACHMENT_ID, $id,
                $repository_content_object_attachment_alias, true);
        $version_condition = new NotCondition(
                new EqualityCondition(ContentObject :: PROPERTY_CURRENT, ContentObject :: CURRENT_OLD));
        $condition = new AndCondition(array($condition, $version_condition));

        return $this->count_result_set($query, ContentObject :: get_table_name(), $condition);
    }

    function retrieve_objects_to_which_object_is_attached($id)
    {
        $repository_content_object_alias = $this->get_alias(ContentObject :: get_table_name());
        $repository_content_object_attachment_alias = $this->get_alias(ContentObjectAttachment :: get_table_name());

        $query = 'SELECT * ';
        $query .= ' FROM ' . $this->escape_table_name(ContentObjectAttachment :: get_table_name()) . ' as ' . $repository_content_object_attachment_alias;
        $query .= ' JOIN ' . $this->escape_table_name(ContentObject :: get_table_name()) . ' as ' . $repository_content_object_alias;
        $query .= ' ON ' . $this->escape_column_name(ContentObjectAttachment :: PROPERTY_CONTENT_OBJECT_ID,
                $repository_content_object_attachment_alias) . ' = ' . $this->escape_column_name(
                ContentObject :: PROPERTY_ID, $repository_content_object_alias);

        $condition = new EqualityCondition(ContentObjectAttachment :: PROPERTY_ATTACHMENT_ID, $id,
                $repository_content_object_attachment_alias, true);
        $version_condition = new NotCondition(
                new EqualityCondition(ContentObject :: PROPERTY_CURRENT, ContentObject :: CURRENT_OLD));
        $condition = new AndCondition(array($condition, $version_condition));

        return $this->retrieve_object_set($query, ContentObject :: get_table_name(), $condition);
    }

    function count_objects_which_object_attaches($id)
    {
        $repository_content_object_alias = $this->get_alias(ContentObject :: get_table_name());
        $repository_content_object_attachment_alias = $this->get_alias(ContentObjectAttachment :: get_table_name());

        $query = 'SELECT count(*) ';
        $query .= ' FROM ' . $this->escape_table_name(ContentObjectAttachment :: get_table_name()) . ' as ' . $repository_content_object_attachment_alias;
        $query .= ' JOIN ' . $this->escape_table_name(ContentObject :: get_table_name()) . ' as ' . $repository_content_object_alias;
        $query .= ' ON ' . $this->escape_column_name(ContentObjectAttachment :: PROPERTY_ATTACHMENT_ID,
                $repository_content_object_attachment_alias) . ' = ' . $this->escape_column_name(
                ContentObject :: PROPERTY_ID, $repository_content_object_alias);

        $condition = new EqualityCondition(ContentObjectAttachment :: PROPERTY_CONTENT_OBJECT_ID, $id,
                $repository_content_object_attachment_alias, true);
        $version_condition = new NotCondition(
                new EqualityCondition(ContentObject :: PROPERTY_CURRENT, ContentObject :: CURRENT_OLD));
        $condition = new AndCondition(array($condition, $version_condition));

        return $this->count_result_set($query, ContentObject :: get_table_name(), $condition);
    }

    function retrieve_objects_which_object_attaches($id)
    {
        $repository_content_object_alias = $this->get_alias(ContentObject :: get_table_name());
        $repository_content_object_attachment_alias = $this->get_alias(ContentObjectAttachment :: get_table_name());

        $query = 'SELECT * ';
        $query .= ' FROM ' . $this->escape_table_name(ContentObjectAttachment :: get_table_name()) . ' as ' . $repository_content_object_attachment_alias;
        $query .= ' JOIN ' . $this->escape_table_name(ContentObject :: get_table_name()) . ' as ' . $repository_content_object_alias;
        $query .= ' ON ' . $this->escape_column_name(ContentObjectAttachment :: PROPERTY_ATTACHMENT_ID,
                $repository_content_object_attachment_alias) . ' = ' . $this->escape_column_name(
                ContentObject :: PROPERTY_ID, $repository_content_object_alias);

        $condition = new EqualityCondition(ContentObjectAttachment :: PROPERTY_CONTENT_OBJECT_ID, $id,
                $repository_content_object_attachment_alias, true);
        $version_condition = new NotCondition(
                new EqualityCondition(ContentObject :: PROPERTY_CURRENT, ContentObject :: CURRENT_OLD));
        $condition = new AndCondition(array($condition, $version_condition));

        return $this->retrieve_object_set($query, ContentObject :: get_table_name(), $condition);
    }

    function retrieve_content_object_attachments($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        return $this->retrieve_objects(ContentObjectAttachment :: get_table_name(), $condition, $offset, $max_objects,
                $order_by, ContentObjectAttachment :: CLASS_NAME);
    }

    function retrieve_content_object_includes($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        return $this->retrieve_objects(ContentObjectInclude :: get_table_name(), $condition, $offset, $max_objects,
                $order_by, ContentObjectInclude :: CLASS_NAME);
    }

    /**
     * Counts the ContentObjectAttachments objects
     *
     * @param $condition type
     * @return type
     */
    function count_content_object_attachments($condition = null)
    {
        return $this->count_objects(ContentObjectAttachment :: get_table_name(), $condition);
    }

    function count_objects_in_which_object_is_included($id)
    {
        $repository_content_object_alias = $this->get_alias(ContentObject :: get_table_name());
        $repository_content_object_include_alias = $this->get_alias(ContentObjectInclude :: get_table_name());

        $query = 'SELECT count(*) ';
        $query .= ' FROM ' . $this->escape_table_name(ContentObjectInclude :: get_table_name()) . ' as ' . $repository_content_object_include_alias;
        $query .= ' JOIN ' . $this->escape_table_name(ContentObject :: get_table_name()) . ' as ' . $repository_content_object_alias;

        $query .= ' ON ' . $this->escape_column_name(ContentObjectInclude :: PROPERTY_CONTENT_OBJECT_ID,
                $repository_content_object_include_alias) . ' = ' . $this->escape_column_name(
                ContentObject :: PROPERTY_ID, $repository_content_object_alias);

        $condition = new EqualityCondition(ContentObjectInclude :: PROPERTY_INCLUDE_ID, $id,
                $repository_content_object_include_alias, true);
        $version_condition = new NotCondition(
                new EqualityCondition(ContentObject :: PROPERTY_CURRENT, ContentObject :: CURRENT_OLD));
        $condition = new AndCondition(array($condition, $version_condition));

        return $this->count_result_set($query, ContentObject :: get_table_name(), $condition);
    }

    function retrieve_objects_in_which_object_is_included($id)
    {
        $repository_content_object_alias = $this->get_alias(ContentObject :: get_table_name());
        $repository_content_object_include_alias = $this->get_alias(ContentObjectInclude :: get_table_name());

        $query = 'SELECT * ';
        $query .= ' FROM ' . $this->escape_table_name(ContentObjectInclude :: get_table_name()) . ' as ' . $repository_content_object_include_alias;
        $query .= ' JOIN ' . $this->escape_table_name(ContentObject :: get_table_name()) . ' as ' . $repository_content_object_alias;
        $query .= ' ON ' . $this->escape_column_name(ContentObjectInclude :: PROPERTY_CONTENT_OBJECT_ID,
                $repository_content_object_include_alias) . ' = ' . $this->escape_column_name(
                ContentObject :: PROPERTY_ID, $repository_content_object_alias);

        $condition = new EqualityCondition(ContentObjectInclude :: PROPERTY_INCLUDE_ID, $id,
                $repository_content_object_include_alias, true);
        $version_condition = new NotCondition(
                new EqualityCondition(ContentObject :: PROPERTY_CURRENT, ContentObject :: CURRENT_OLD));
        $condition = new AndCondition(array($condition, $version_condition));

        return $this->retrieve_object_set($query, ContentObject :: get_table_name(), $condition);
    }

    function count_objects_which_object_includes($id)
    {
        $repository_content_object_alias = $this->get_alias(ContentObject :: get_table_name());
        $repository_content_object_include_alias = $this->get_alias(ContentObjectInclude :: get_table_name());

        $query = 'SELECT count(*) ';
        $query .= ' FROM ' . $this->escape_table_name(ContentObjectInclude :: get_table_name()) . ' as ' . $repository_content_object_include_alias;
        $query .= ' JOIN ' . $this->escape_table_name(ContentObject :: get_table_name()) . ' as ' . $repository_content_object_alias;

        $query .= ' ON ' . $this->escape_column_name(ContentObjectInclude :: PROPERTY_INCLUDE_ID,
                $repository_content_object_include_alias) . ' = ' . $this->escape_column_name(
                ContentObject :: PROPERTY_ID, $repository_content_object_alias);

        $condition = new EqualityCondition(ContentObjectInclude :: PROPERTY_CONTENT_OBJECT_ID, $id,
                $repository_content_object_include_alias, true);
        $version_condition = new NotCondition(
                new EqualityCondition(ContentObject :: PROPERTY_CURRENT, ContentObject :: CURRENT_OLD));
        $condition = new AndCondition(array($condition, $version_condition));

        return $this->count_result_set($query, ContentObject :: get_table_name(), $condition);
    }

    function retrieve_objects_which_object_includes($id)
    {
        $repository_content_object_alias = $this->get_alias(ContentObject :: get_table_name());
        $repository_content_object_include_alias = $this->get_alias(ContentObjectInclude :: get_table_name());

        $query = 'SELECT * ';
        $query .= ' FROM ' . $this->escape_table_name(ContentObjectInclude :: get_table_name()) . ' as ' . $repository_content_object_include_alias;
        $query .= ' JOIN ' . $this->escape_table_name(ContentObject :: get_table_name()) . ' as ' . $repository_content_object_alias;
        $query .= ' ON ' . $this->escape_column_name(ContentObjectInclude :: PROPERTY_INCLUDE_ID,
                $repository_content_object_include_alias) . ' = ' . $this->escape_column_name(
                ContentObject :: PROPERTY_ID, $repository_content_object_alias);

        $condition = new EqualityCondition(ContentObjectInclude :: PROPERTY_CONTENT_OBJECT_ID, $id,
                $repository_content_object_include_alias, true);
        $version_condition = new NotCondition(
                new EqualityCondition(ContentObject :: PROPERTY_CURRENT, ContentObject :: CURRENT_OLD));
        $condition = new AndCondition(array($condition, $version_condition));

        return $this->retrieve_object_set($query, ContentObject :: get_table_name(), $condition);
    }

    // Inherited.
    function retrieve_included_content_objects($object)
    {
        $repository_content_object_alias = $this->get_alias(ContentObject :: get_table_name());
        $repository_content_object_include_alias = $this->get_alias('content_object_include');

        $query = 'SELECT * ';
        $query .= ' FROM ' . $this->escape_table_name('content_object_include') . ' as ' . $repository_content_object_include_alias;
        $query .= ' JOIN ' . $this->escape_table_name(ContentObject :: get_table_name()) . ' as ' . $repository_content_object_alias;
        $query .= ' ON ' . $this->escape_column_name('include_id', $repository_content_object_include_alias) . ' = ' . $this->escape_column_name(
                ContentObject :: PROPERTY_ID, $repository_content_object_alias);

        $condition = new EqualityCondition('content_object_id', $object->get_id(),
                $repository_content_object_include_alias, true);

        return $this->retrieve_object_set($query, ContentObject :: get_table_name(), $condition)->as_array();
    }

    function is_content_object_included($object)
    {
        $condition = new EqualityCondition('include_id', $object->get_id());
        $count = $this->count_objects('content_object_include', $condition);
        return ($count > 0);
    }

    function is_content_object_already_included($content_object, $include_object_id)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition('include_id', $include_object_id);
        $conditions[] = new EqualityCondition('content_object_id', $content_object->get_id());
        $condition = new AndCondition($conditions);

        $count = $this->count_objects('content_object_include', $condition);
        return ($count > 0);
    }

    function is_content_object_already_attached($content_object, $attachment_id)
    {
        if ($content_object instanceof ContentObject)
        {
            $content_object = $content_object->get_id();
        }

        $conditions = array();
        $conditions[] = new EqualityCondition('attachment_id', $attachment_id);
        $conditions[] = new EqualityCondition('content_object_id', $content_object);
        $condition = new AndCondition($conditions);

        $count = $this->count_objects('content_object_attachment', $condition);
        return ($count > 0);
    }

    function retrieve_content_object_versions($object, $include_last = true, $include_self = true)
    {
        $object_number = $object->get_object_number();

        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObject :: PROPERTY_OBJECT_NUMBER, $object_number);
        $conditions[] = new EqualityCondition(ContentObject :: PROPERTY_STATE, $object->get_state());

        if (! $include_last)
        {
            $conditions[] = new NotCondition(
                    new InCondition(ContentObject :: PROPERTY_CURRENT,
                            array(ContentObject :: CURRENT_MULTIPLE, ContentObject :: CURRENT_SINGLE)));
        }

        if (! $include_self)
        {
            $conditions[] = new NotCondition(new EqualityCondition(ContentObject :: PROPERTY_ID, $object->get_id()));
        }

        $condition = new AndCondition($conditions);

        $version_ids = $this->retrieve_distinct(ContentObject :: get_table_name(), ContentObject :: PROPERTY_ID,
                $condition);

        foreach ($version_ids as $version_id)
        {
            $versions[] = $this->retrieve_content_object($version_id);
        }

        return $versions;
    }

    function retrieve_most_recent_content_object_version($object)
    {
        $type = $object->get_type();

        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObject :: PROPERTY_OBJECT_NUMBER, $object->get_object_number());
        $conditions[] = new InCondition(ContentObject :: PROPERTY_STATE, ContentObject :: get_active_status_types());
        $condition = new AndCondition($conditions);

        if (RepositoryDataManager :: is_extended_type($type))
        {
            $content_object_alias = $this->get_alias(ContentObject :: get_table_name());

            $query = 'SELECT * FROM ' . $this->escape_table_name(ContentObject :: get_table_name()) . ' AS ' . $content_object_alias;
            $query .= ' JOIN ' . $this->escape_table_name($type) . ' AS ' . self :: ALIAS_TYPE_TABLE . ' ON ' . $this->escape_column_name(
                    ContentObject :: PROPERTY_ID, $content_object_alias) . '=' . $this->escape_column_name(
                    ContentObject :: PROPERTY_ID, self :: ALIAS_TYPE_TABLE);

            $record = $this->retrieve_row($query, ContentObject :: get_table_name(), $condition, array(new ObjectTableOrder(ContentObject :: PROPERTY_ID, SORT_DESC)));
        }
        else
        {
            $record = $this->retrieve_record(ContentObject :: get_table_name(), $condition, array(new ObjectTableOrder(ContentObject :: PROPERTY_ID, SORT_DESC)));
        }

        if ($record)
        {
            return self :: record_to_content_object($record, isset($type));
        }
        else
        {
            return false;
        }
    }

    function retrieve_content_object_versions_resultset($condition = null, $order_by = array(), $offset = 0, $max_objects = -1)
    {
        return $this->retrieve_objects(ContentObject :: get_table_name(), $condition, $offset, $max_objects, $order_by,
                ContentObject :: CLASS_NAME);
    }

    function count_content_object_versions_resultset($condition = null)
    {
        return $this->count_objects(ContentObject :: get_table_name(), $condition);
    }

    function get_latest_version_id($object)
    {
        $conditions = array();
        $old_version_condition = new EqualityCondition(ContentObject :: PROPERTY_CURRENT, ContentObject :: CURRENT_OLD);
        $conditions[] = new NotCondition($old_version_condition);
        $conditions[] = new EqualityCondition(ContentObject :: PROPERTY_OBJECT_NUMBER, $object->get_object_number());
        $condition = new AndCondition($conditions);

        $record = $this->retrieve_record(ContentObject :: get_table_name(), $condition);

        return $record['id'];
    }

    // Inherited.
    function attach_content_object($object, $attachment_id, $type)
    {
        $props = array();
        $props[ContentObjectAttachment :: PROPERTY_CONTENT_OBJECT_ID] = $object->get_id();
        $props[ContentObjectAttachment :: PROPERTY_ATTACHMENT_ID] = $attachment_id;
        $props[ContentObjectAttachment :: PROPERTY_TYPE] = $type;
        $this->get_connection()->loadModule('Extended');
        if ($this->get_connection()->extended->autoExecute(
                $this->get_table_name(ContentObjectAttachment :: get_table_name()), $props, MDB2_AUTOQUERY_INSERT))
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    // Inherited.
    function detach_content_object($object, $attachment_id, $type)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObjectAttachment :: PROPERTY_CONTENT_OBJECT_ID, $object->get_id());
        $conditions[] = new EqualityCondition(ContentObjectAttachment :: PROPERTY_ATTACHMENT_ID, $attachment_id);
        $conditions[] = new EqualityCondition(ContentObjectAttachment :: PROPERTY_TYPE, $type);
        $condition = new AndCondition($conditions);

        return $this->delete_content_object_attachments($condition);
    }

    // Inherited.
    function detach_content_objects($object, $type)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObjectAttachment :: PROPERTY_CONTENT_OBJECT_ID, $object->get_id());
        $conditions[] = new EqualityCondition(ContentObjectAttachment :: PROPERTY_TYPE, $type);
        $condition = new AndCondition($conditions);

        return $this->delete_content_object_attachments($condition);
    }

    // Inherited.
    function include_content_object($object, $include_id)
    {
        $props = array();
        $props['content_object_id'] = $object->get_id();
        $props['include_id'] = $include_id;
        $this->get_connection()->loadModule('Extended');
        $this->get_connection()->extended->autoExecute($this->get_table_name('content_object_include'), $props,
                MDB2_AUTOQUERY_INSERT);
    }

    // Inherited.
    function exclude_content_object($object, $include_id)
    {
        $query = 'DELETE FROM ' . $this->escape_table_name('content_object_include') . ' WHERE ' . $this->escape_column_name(
                'content_object_id') . '=' . $this->quote($object->get_id()) . ' AND ' . $this->escape_column_name(
                'include_id') . '=' . $this->quote($include_id);
        $affectedRows = $this->exec($query);
        return ($affectedRows > 0);
    }

    // Inherited.
    function set_content_object_states($object_ids, $state)
    {
        if (! count($object_ids))
        {
            return true;
        }
        else
        {
            $condition = new InCondition(ContentObject :: PROPERTY_ID, $object_ids);
            $properties = array(ContentObject :: PROPERTY_STATE => $state);
            return $this->update_objects(ContentObject :: get_table_name(), $properties, $condition);
        }
    }

    function get_alias_type_table()
    {
        return self :: ALIAS_TYPE_TABLE;
    }

    // Inherited.
    function get_children_ids($object)
    {
        $children_ids = array();
        $parent_ids = array($object->get_id());
        do
        {
            $query = 'SELECT ' . $this->escape_column_name(ContentObject :: PROPERTY_ID) . ' FROM ' . $this->escape_table_name(
                    ContentObject :: get_table_name()) . ' WHERE ' . $this->escape_column_name(
                    ContentObject :: PROPERTY_PARENT_ID) . ' IN (?' . implode(',', $parent_ids) . ')';
            $res = $this->query($query);
            if ($res->numRows() == 0)
            {
                return $children_ids;
            }
            else
            {
                $parent_ids = array();
                while ($record = $res->fetchRow(MDB2_FETCHMODE_ASSOC))
                {
                    $parent_ids[] = $record[ContentObject :: PROPERTY_ID];
                    $children_ids[] = $record[ContentObject :: PROPERTY_ID];
                }
            }
            $res->free();
        }
        while (true);
    }

    function get_version_ids($object)
    {
        $condition = new EqualityCondition(ContentObject :: PROPERTY_OBJECT_NUMBER, $object->get_object_number());
        $version_ids = $this->retrieve_distinct(ContentObject :: get_table_name(), ContentObject :: PROPERTY_ID,
                $condition);
        sort($version_ids);
        return $version_ids;
    }

    /**
     * Handles PEAR errors. If an error is encountered, the program dies with a descriptive error message.
     *
     * @param $error DB_Error The error object.
     */
    static function handle_error($error)
    {
        print_r($error);
        // For debugging only. May create a security hazard.
        die(__FILE__ . ':' . __LINE__ . ': ' . $error->getMessage() . ' (' . $error->getDebugInfo() . ')');
    }

    /**
     * Converts a datetime value (as retrieved from the database) to a UNIX timestamp (as returned by time()).
     *
     * @param $date string The date as a UNIX timestamp.
     * @return int The date as a UNIX timestamp.
     */
    static function from_db_date($date)
    {
        if (isset($date))
        {
            return strtotime($date);
        }
        return null;
    }

    /**
     * Parses a database record fetched as an associative array into a learning object.
     *
     * @param $record array The associative array.
     * @param $additional_properties_known boolean True if the additional properties of the oject were fetched.
     * @return ContentObject The object.
     */
    function record_to_content_object($record, $additional_properties_known = false)
    {
        if (! is_array($record) || ! count($record))
        {
            throw new Exception(
                    Translation :: get('InvalidDataRetrievedFromDatabase', null, Utilities :: COMMON_LIBRARIES));
        }

        foreach ($record as &$field)
        {
            if (is_resource($field))
            {
                $data = '';
                while (! feof($field))
                    $data .= fread($field, 1024);
                $field = $data;
            }
        }

        $default_properties = array();
        foreach (ContentObject :: get_default_property_names() as $prop)
        {
            $default_properties[$prop] = $record[$prop];
        }

        /*
         * $defaultProp[ContentObject :: PROPERTY_CREATION_DATE] = self :: from_db_date($defaultProp[ContentObject ::
         * PROPERTY_CREATION_DATE]); $defaultProp[ContentObject :: PROPERTY_MODIFICATION_DATE] = self ::
         * from_db_date($defaultProp[ContentObject :: PROPERTY_MODIFICATION_DATE]);
         */
        $content_object = ContentObject :: factory($record[ContentObject :: PROPERTY_TYPE], $default_properties);

        if ($additional_properties_known)
        {
            $properties = $content_object->get_additional_property_names();

            $additional_properties = array();
            if (count($properties) > 0)
            {
                foreach ($properties as $prop)
                {
                    if (is_resource($record[$prop]))
                    {
                        $data = '';
                        while (! feof($record[$prop]))
                            $data .= fread($record[$prop], 1024);
                        $record[$prop] = $data;
                    }
                    $additional_properties[$prop] = $record[$prop];
                }
            }
            $content_object->set_additional_properties($additional_properties);
        }

        return $content_object;
    }

    /**
     * Translates a string with wildcard characters "?" (single character) and "*" (any character sequence) to a SQL
     * pattern for use in a LIKE condition. Should be suitable for any SQL flavor.
     *
     * @param $string string The string that contains wildcard characters.
     * @return string The escaped string.
     */
    static function translate_search_string($string)
    {
        /*
         * ====================================================================== A brief explanation of these regexps:
         * - The first one escapes SQL wildcard characters, thus prefixing %, ', \ and _ with a backslash. - The second
         * one replaces asterisks that are not prefixed with a backslash (which escapes them) with the SQL equivalent,
         * namely a percent sign. - The third one is similar to the second: it replaces question marks that are not
         * escaped with the SQL equivalent _. ======================================================================
         */
        return preg_replace(array('/([%\'\\\\_])/e', '/(?<!\\\\)\*/', '/(?<!\\\\)\?/'),
                array("'\\\\\\\\' . '\\1'", '%', '_'), $string);
    }

    /**
     * Checks whether the given column name is the name of a column that contains a date value, and hence should be
     * formatted as such.
     *
     * @param $name string The column name.
     * @return boolean True if the column is a date column, false otherwise.
     */
    static function is_date_column($name)
    {
        return ($name == ContentObject :: PROPERTY_CREATION_DATE || $name == ContentObject :: PROPERTY_MODIFICATION_DATE);
    }

    // Inherited.
    function get_used_disk_space($owner = null)
    {
        if ($owner)
        {
            $condition_owner = new EqualityCondition(ContentObject :: PROPERTY_OWNER_ID, $owner);
        }

        $types = RepositoryDataManager :: get_registered_types();
        $co_alias = $this->get_alias(ContentObject :: get_table_name());

        $disk_space = 0;

        foreach ($types as $index => $type)
        {
            $class = ContentObject :: type_to_class($type);
            $properties = call_user_func(array($class, 'get_disk_space_properties'));

            if (is_null($properties))
            {
                continue;
            }

            if (! is_array($properties))
            {
                $properties = array($properties);
            }

            $sum = array();

            foreach ($properties as $index => $property)
            {
                $sum[] = 'SUM(' . $this->escape_column_name($property) . ')';
            }

            if (RepositoryDataManager :: is_extended_type($type))
            {
                $query = 'SELECT ' . implode('+', $sum) . ' AS disk_space FROM ' . $this->escape_table_name(
                        ContentObject :: get_table_name()) . ' AS ' . $co_alias . ' JOIN ' . $this->escape_table_name(
                        $type) . ' AS ' . self :: ALIAS_TYPE_TABLE . ' ON ' . $co_alias . '.' . $this->escape_column_name(
                        ContentObject :: PROPERTY_ID) . ' = ' . self :: ALIAS_TYPE_TABLE . '.' . $this->escape_column_name(
                        ContentObject :: PROPERTY_ID);
                if (isset($condition_owner))
                {
                    $condition = $condition_owner;
                }
            }
            else
            {
                $query = 'SELECT ' . implode('+', $sum) . ' AS disk_space FROM ' . $this->escape_table_name(
                        ContentObject :: get_table_name());
                $match = new EqualityCondition(ContentObject :: PROPERTY_TYPE, $type);

                if (isset($condition_owner))
                {
                    $condition = new AndCondition(array($match, $condition_owner));
                }
                else
                {
                    $condition = $match;
                }
            }

            if (isset($condition))
            {
                $translator = ConditionTranslator :: factory($this, $this->get_alias(ContentObject :: get_table_name()));
                $query .= $translator->render_query($condition);
            }

            $res = $this->query($query);
            $record = $res->fetchRow(MDB2_FETCHMODE_OBJECT);
            $disk_space += $record->disk_space;
            $res->free();
        }

        return $disk_space;
    }

    private static function is_content_object_column($name)
    {
        return ContentObject :: is_default_property_name($name) || $name == ContentObject :: PROPERTY_TYPE || $name == ContentObject :: PROPERTY_ID;
    }

    function ExecuteQuery($sql)
    {
        $this->get_connection()->query($sql);
    }

    function is_attached($object, $type = null)
    {
        if (isset($type))
        {
            $condition = new EqualityCondition('attachment_id', $object->get_id());
        }
        else
        {
            $condition = new InCondition('attachment_id', $this->get_version_ids($object));
        }

        $count = $this->count_objects(ContentObjectAttachment :: get_table_name(), $condition);

        // $query = 'SELECT COUNT(' .
        // $this->escape_column_name('content_object_id',
        // $this->get_alias('content_object_attachment')) . ') FROM ' .
        // $this->escape_table_name('content_object_attachment') . ' AS ' .
        // $this->get_alias('content_object_attachment');
        // $count = $this->count_result_set($query, 'content_object_attachment',
        // $condition);

        return $count > 0;

        // $query = 'SELECT COUNT(' .
        // $this->escape_column_name("content_object_id") . ') FROM ' .
        // $this->escape_table_name('content_object_attachment') . ' AS ' . self
        // ::
        // ALIAS_CONTENT_OBJECT_ATTACHMENT_TABLE . ' WHERE ' . self ::
        // ALIAS_CONTENT_OBJECT_ATTACHMENT_TABLE . '.attachment_id';
        // if (isset($type))
        // {
        // $query .= '=?';
        // $params = $object->get_id();
        // }
        // else
        // {
        // $query .= ' IN (?' . str_repeat(',?',
        // count($this->get_version_ids($object)) - 1) . ')';
        // $params = $this->get_version_ids($object);
        // }
        // $sth = $this->get_connection()->prepare($query);
        // $res = $sth->execute($params);
        // $sth->free();
        // $record = $res->fetchRow(MDB2_FETCHMODE_ORDERED);
        // $res->free();
        // if ($record[0] > 0)
        // {
        // return true;
        // }
        // else
        // {
        // return false;
        // }
    }

    /**
     * Creates a new complex object in the database
     *
     * @param $clo ComplexContentObject - The complex object
     * @return True if success
     */
    function create_complex_content_object_item($clo_item)
    {
        $props = array();
        foreach ($clo_item->get_default_properties() as $key => $value)
        {
            $props[$this->escape_column_name($key)] = $value;
        }
        $props[$this->escape_column_name(ComplexContentObjectItem :: PROPERTY_ID)] = $this->get_better_next_id(
                ComplexContentObjectItem :: get_table_name(), ComplexContentObjectItem :: PROPERTY_ID);
        $this->get_connection()->loadModule('Extended');
        $this->get_connection()->extended->autoExecute(
                $this->get_table_name(ComplexContentObjectItem :: get_table_name()), $props, MDB2_AUTOQUERY_INSERT);
        $clo_item->set_id(
                $this->get_connection()->extended->getAfterID(
                        $props[$this->escape_column_name(ComplexContentObjectItem :: PROPERTY_ID)],
                        ComplexContentObjectItem :: get_table_name()));

        if ($clo_item->is_extended())
        {
            $ref = $clo_item->get_ref();

            $props = array();
            foreach ($clo_item->get_additional_properties() as $key => $value)
            {
                $props[$this->escape_column_name($key)] = $value;
            }
            $props[$this->escape_column_name(ComplexContentObjectItem :: PROPERTY_ID)] = $clo_item->get_id();
            $type = $this->determine_content_object_type($ref);
            $this->get_connection()->extended->autoExecute($this->get_table_name('complex_' . $type), $props,
                    MDB2_AUTOQUERY_INSERT);
        }

        return true;
    }

    /**
     * Updates a complex object in the database
     *
     * @param $clo ComplexContentObject - The complex object
     * @return True if success
     */
    function update_complex_content_object_item($clo_item)
    {
        $translator = ConditionTranslator :: factory($this);
        $condition = $translator->render_query(
                new EqualityCondition(ComplexContentObjectItem :: PROPERTY_ID, $clo_item->get_id()), false);

        $props = array();
        foreach ($clo_item->get_default_properties() as $key => $value)
        {
            if ($key == ComplexContentObjectItem :: PROPERTY_ID)
                continue;
            $props[$this->escape_column_name($key)] = $value;
        }

        $this->get_connection()->loadModule('Extended');
        $this->get_connection()->extended->autoExecute(
                $this->get_table_name(ComplexContentObjectItem :: get_table_name()), $props, MDB2_AUTOQUERY_UPDATE,
                $condition);
        if ($clo_item->is_extended())
        {
            $ref = $clo_item->get_ref();

            $props = array();
            foreach ($clo_item->get_additional_properties() as $key => $value)
            {
                $props[$this->escape_column_name($key)] = $value;
            }
            $type = $this->determine_content_object_type($ref);
            $this->get_connection()->extended->autoExecute($this->get_table_name('complex_' . $type), $props,
                    MDB2_AUTOQUERY_UPDATE, $condition);
        }
        return true;
    }

    /**
     * Deletes a complex object in the database
     *
     * @param $clo ComplexContentObject - The complex object
     * @return True if success
     */
    function delete_complex_content_object_item($clo_item)
    {
        $condition = new EqualityCondition(ComplexContentObjectItem :: PROPERTY_ID, $clo_item->get_id());

        $query = 'DELETE FROM ' . $this->escape_table_name(ComplexContentObjectItem :: get_table_name());

        if (isset($condition))
        {
            $translator = ConditionTranslator :: factory($this);
            $query .= $translator->render_query($condition);
        }

        // $this->set_limit(1);
        $res = $this->exec($query);

        if ($clo_item->is_extended())
        {
            $ref = $clo_item->get_ref();

            $type = $this->determine_content_object_type($ref);
            $query = 'DELETE FROM ' . $this->get_table_name('complex_' . $type);

            if (isset($condition))
            {
                $translator = ConditionTranslator :: factory($this);
                $query .= $translator->render_query($condition);
            }

            // $this->set_limit(1);
            $this->exec($query);
        }

        $conditions = array();
        $conditions[] = new InequalityCondition(ComplexContentObjectItem :: PROPERTY_DISPLAY_ORDER,
                InequalityCondition :: GREATER_THAN, $clo_item->get_display_order());
        $conditions[] = new EqualityCondition(ComplexContentObjectItem :: PROPERTY_PARENT, $clo_item->get_parent());
        $condition = new AndCondition($conditions);
        $properties[ComplexContentObjectItem :: PROPERTY_DISPLAY_ORDER] = $this->escape_column_name(
                ComplexContentObjectItem :: PROPERTY_DISPLAY_ORDER) . '-1';

        $this->update_objects(ComplexContentObjectItem :: get_table_name(), $properties, $condition);

        return true;
    }

    function delete_complex_content_object_items($condition)
    {
        $items = $this->retrieve_complex_content_object_items($condition);
        while ($item = $items->next_result())
        {
            if (! $item->delete())
            {
                return false;
            }
        }
        return true;
    }

    /**
     * Retrieves a complex object from the database with a given id
     *
     * @param $clo_id Int
     * @return The complex object
     */
    function retrieve_complex_content_object_item($clo_item_id)
    {
        // Retrieve main table
        $query = 'SELECT * FROM ' . $this->escape_table_name(ComplexContentObjectItem :: get_table_name()) . ' AS ' . $this->get_alias(
                ComplexContentObjectItem :: get_table_name());

        $condition = new EqualityCondition(ComplexContentObjectItem :: PROPERTY_ID, $clo_item_id,
                ComplexContentObjectItem :: get_table_name());

        if (isset($condition))
        {
            $translator = ConditionTranslator :: factory($this);
            $query .= $translator->render_query($condition);
        }

        $this->set_limit(1, 0);
        $res = $this->query($query);
        $record = $res->fetchRow(MDB2_FETCHMODE_ASSOC);
        if (! $record)
            return null;

            // Determine type

        $res->free();

        $ref = $record[ComplexContentObjectItem :: PROPERTY_REF];

        $type = $this->determine_content_object_type($ref);
        $cloi = ComplexContentObjectItem :: factory($type, array(), array());

        $bool = false;

        if ($cloi->is_extended())
            $bool = true;

        return self :: record_to_complex_content_object_item($record, $type, $bool);
    }

    /**
     * Mapper for a record to a complex object item
     *
     * @param $record Record
     * @return ComplexContentObjectItem
     */
    function record_to_complex_content_object_item($record, $type = null, $additional_properties_known = false)
    {
        if (! is_array($record) || ! count($record))
        {
            throw new Exception(
                    Translation :: get('InvalidDataRetrievedFromDatabase', null, Utilities :: COMMON_LIBRARIES));
        }

        foreach ($record as &$field)
        {
            if (is_resource($field))
            {
                $data = '';
                while (! feof($field))
                    $data .= fread($field, 1024);
                $field = $data;
            }
        }

        $cloi = ComplexContentObjectItem :: factory($type, array(), array());

        $defaultProp = array();
        foreach ($cloi->get_default_property_names() as $prop)
        {
            $defaultProp[$prop] = $record[$prop];
        }
        $cloi->set_default_properties($defaultProp);

        if ($additional_properties_known && $type && $cloi->is_extended())
        {
            $additionalProp = array();

            $query = 'SELECT * FROM ' . $this->escape_table_name('complex_' . $type) . ' AS ' . self :: ALIAS_TYPE_TABLE;

            $condition = new EqualityCondition(ComplexContentObjectItem :: PROPERTY_ID, $record['id']);

            if (isset($condition))
            {
                $translator = ConditionTranslator :: factory($this);
                $query .= $translator->render_query($condition);
            }

            // dump($query);

            $this->set_limit(1, 0);
            $res = $this->query($query);
            $rec2 = $res->fetchRow(MDB2_FETCHMODE_ASSOC);
            $res->free();

            foreach ($rec2 as &$field)
            {
                if (is_resource($field))
                {
                    $data = '';
                    while (! feof($field))
                        $data .= fread($field, 1024);
                    $field = $data;
                }
            }

            foreach ($cloi->get_additional_property_names() as $prop)
            {
                $additionalProp[$prop] = $rec2[$prop];
            }

            $cloi->set_additional_properties($additionalProp);
        }
        else
        {
            $additionalProp = null;
        }

        return $cloi;
    }

    /**
     * Counts the available complex objects with the given condition
     *
     * @param $condition Condition
     * @return Int the amount of complex objects
     */
    function count_complex_content_object_items($condition)
    {
        $alias = $this->get_alias(ComplexContentObjectItem :: get_table_name());
        $query = 'SELECT COUNT(*) FROM ' . $this->escape_table_name(ComplexContentObjectItem :: get_table_name()) . ' AS ' . $alias;
        $lo_alias = $this->get_alias(ContentObject :: get_table_name());
        $query .= ' JOIN ' . $this->escape_table_name(ContentObject :: get_table_name()) . ' AS ' . $lo_alias . ' ON ' . $alias . '.ref_id=' . $lo_alias . '.id';
        if (isset($condition))
        {
            $translator = ConditionTranslator :: factory($this, $alias);
            $query .= $translator->render_query($condition);
        }
        $res = $this->query($query);
        if (MDB2 :: isError($res))
        {
            return false;
        }
        else
        {
            $record = $res->fetchRow(MDB2_FETCHMODE_ORDERED);
            $res->free();
            return $record[0];
        }
    }

    /**
     * Retrieves the complex object items with the given condition
     */
    function retrieve_complex_content_object_items($condition = null, $order_by = array(), $offset = 0, $max_objects = -1,
            $type = null)
    {
        $alias = $this->get_alias(ComplexContentObjectItem :: get_table_name());

        $query = 'SELECT ' . $alias . '.* FROM ' . $this->escape_table_name(
                ComplexContentObjectItem :: get_table_name()) . ' AS ' . $alias;

        if (isset($type))
        {
            $alias_type_table = $this->get_alias($type);

            switch ($type)
            {
                case 'complex_wiki_page' :
                    $query .= ' JOIN ' . $this->escape_table_name($type) . ' AS ' . $alias_type_table . ' ON ' . $this->escape_column_name(
                            ContentObject :: PROPERTY_ID, $alias) . ' = ' . $this->escape_column_name(
                            ComplexContentObjectItem :: PROPERTY_ID, $alias_type_table);
            }
        }
        $lo_alias = $this->get_alias(ContentObject :: get_table_name());

        $query .= ' JOIN ' . $this->escape_table_name(ContentObject :: get_table_name()) . ' AS ' . $lo_alias;
        $query .= ' ON ' . $this->escape_column_name(ComplexContentObjectItem :: PROPERTY_REF, $alias);
        $query .= ' = ' . $this->escape_column_name(ContentObject :: PROPERTY_ID, $lo_alias);

        if (isset($condition))
        {
            $translator = ConditionTranslator :: factory($this, $alias);
            $query .= $translator->render_query($condition);
        }

        $order_by[] = new ObjectTableOrder(ComplexContentObjectItem :: PROPERTY_DISPLAY_ORDER, SORT_ASC, $alias);

        $orders = array();
        foreach ($order_by as $order)
        {
            $alias = $order->get_alias() ? $order->get_alias() . '.' : '';
            $orders[] = $this->escape_column_name($alias . $order->get_property()) . ' ' . ($order->get_direction() == SORT_DESC ? 'DESC' : 'ASC');
        }
        if (count($orders))
        {
            $query .= ' ORDER BY ' . implode(', ', $orders);
        }

        if ($max_objects < 0)
        {
            $max_objects = null;
        }

        $this->set_limit(intval($max_objects), intval($offset));
        $res = $this->query($query);

        return new DatabaseComplexContentObjectItemResultSet($this, $res, true);
    }

    function select_next_display_order_forum($parent_id)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(ComplexContentObjectItem :: PROPERTY_PARENT, $parent_id);
        $subcondition = new EqualityCondition(ContentObject :: PROPERTY_TYPE, Forum :: get_type_name());
        $conditions[] = new SubSelectcondition(ComplexContentObjectItem :: PROPERTY_REF, ContentObject :: PROPERTY_ID,
                'content_object', $subcondition);
        $condition = new AndCondition($conditions);
        return $this->retrieve_next_sort_value(ComplexContentObjectItem :: get_table_name(),
                ComplexContentObjectItem :: PROPERTY_DISPLAY_ORDER, $condition);
    }

    function select_next_display_order($parent_id)
    {
        $condition = new EqualityCondition(ComplexContentObjectItem :: PROPERTY_PARENT, $parent_id);

        return $this->retrieve_next_sort_value(ComplexContentObjectItem :: get_table_name(),
                ComplexContentObjectItem :: PROPERTY_DISPLAY_ORDER, $condition);
    }

    /**
     * Deprecated, use the function in the data manager class instead which handles unlinks correctly
     *
     * @param $category type
     * @return type
     */
    function delete_category($category)
    {
        $condition = new EqualityCondition(RepositoryCategory :: PROPERTY_ID, $category->get_id());
        $succes = $this->delete(RepositoryCategory :: get_table_name(), $condition);

        // Correct the display order of the remaining categories
        $conditions = array();
        $conditions[] = new InequalityCondition(RepositoryCategory :: PROPERTY_DISPLAY_ORDER,
                InequalityCondition :: GREATER_THAN, $category->get_display_order());
        $conditions[] = new EqualityCondition(RepositoryCategory :: PROPERTY_PARENT, $category->get_parent());
        $conditions[] = new EqualityCondition(RepositoryCategory :: PROPERTY_USER_ID, $category->get_user_id());
        $conditions[] = new EqualityCondition(RepositoryCategory :: PROPERTY_TYPE, $category->get_type());
        $condition = new AndCondition($conditions);
        $properties = array(
                RepositoryCategory :: PROPERTY_DISPLAY_ORDER => ($this->escape_column_name(
                        RepositoryCategory :: PROPERTY_DISPLAY_ORDER) . '-1'));
        $this->update_objects(RepositoryCategory :: get_table_name(), $properties, $condition);

        // Move the objecs in the category to the garbage bin
        $condition = new EqualityCondition(ContentObject :: PROPERTY_PARENT_ID, $category->get_id());
        $properties = array(ContentObject :: PROPERTY_STATE => ContentObject :: STATE_RECYCLED);
        $this->update_objects(ContentObject :: get_table_name(), $properties, $condition);

        // Delete all subcategories by recursively repeating the entire process
        $categories = $this->retrieve_categories(
                new EqualityCondition(RepositoryCategory :: PROPERTY_PARENT, $category->get_id()));
        while ($category = $categories->next_result())
        {
            $this->delete_category($category);
        }

        return $succes;
    }

    function update_category($category)
    {
        $condition = new EqualityCondition(RepositoryCategory :: PROPERTY_ID, $category->get_id());
        return $this->update($category, $condition);
    }

    function create_category($category)
    {
        return $this->create($category);
    }

    function create_external_instance($external_instance)
    {
        return $this->create($external_instance);
    }

    function count_categories($conditions = null)
    {
        return $this->count_objects(RepositoryCategory :: get_table_name(), $conditions);
    }

    function retrieve_category($category_id)
    {
        $condition = new EqualityCondition(RepositoryCategory :: PROPERTY_ID, $category_id);
        return $this->retrieve_object(RepositoryCategory :: get_table_name(), $condition, array(),
                RepositoryCategory :: CLASS_NAME);
    }

    function retrieve_categories($condition = null, $offset = null, $count = null, $order_property = null)
    {
        if ($order_property instanceof ObjectTableOrder)
        {
            $order_property = array($order_property);
        }

        $order_property[] = new ObjectTableOrder(RepositoryCategory :: PROPERTY_PARENT);
        $order_property[] = new ObjectTableOrder(RepositoryCategory :: PROPERTY_DISPLAY_ORDER);
        return $this->retrieve_objects(RepositoryCategory :: get_table_name(), $condition, $offset, $count,
                $order_property, RepositoryCategory :: CLASS_NAME);
    }

    function select_next_category_display_order($parent_category_id, $user_id, $type)
    {
        $conditions[] = new EqualityCondition(RepositoryCategory :: PROPERTY_PARENT, $parent_category_id);
        $conditions[] = new EqualityCondition(RepositoryCategory :: PROPERTY_USER_ID, $user_id);
        $conditions[] = new EqualityCondition(RepositoryCategory :: PROPERTY_TYPE, $type);
        $condition = new AndCondition($conditions);

        return $this->retrieve_next_sort_value(RepositoryCategory :: get_table_name(),
                RepositoryCategory :: PROPERTY_DISPLAY_ORDER, $condition, RepositoryCategory :: CLASS_NAME);
    }

    function delete_user_view($user_view)
    {
        $condition = new EqualityCondition(UserView :: PROPERTY_ID, $user_view->get_id());
        $success = $this->delete(UserView :: get_table_name(), $condition);

        $condition = new EqualityCondition(UserViewRelContentObject :: PROPERTY_VIEW_ID, $user_view->get_id());
        $success &= $this->delete(UserViewRelContentObject :: get_table_name(), $condition);

        return $success;
    }

    function update_user_view($user_view)
    {
        $condition = new EqualityCondition(UserView :: PROPERTY_ID, $user_view->get_id());
        return $this->update($user_view, $condition);
    }

    function create_user_view($user_view)
    {
        return $this->create($user_view);
    }

    function count_user_views($conditions = null)
    {
        return $this->count_objects(UserView :: get_table_name(), $conditions);
    }

    function count_external_settings($conditions = null)
    {
        return $this->count_objects(ExternalSetting :: get_table_name(), $conditions);
    }

    function retrieve_user_views($condition = null, $offset = null, $count = null, $order_property = null)
    {
        return $this->retrieve_objects(UserView :: get_table_name(), $condition, $offset, $count, $order_property,
                UserView :: CLASS_NAME);
    }

    function update_user_view_rel_content_object($user_view_rel_content_object)
    {
        $conditions[] = new EqualityCondition(UserViewRelContentObject :: PROPERTY_VIEW_ID,
                $user_view_rel_content_object->get_view_id());
        $conditions[] = new EqualityCondition(UserViewRelContentObject :: PROPERTY_CONTENT_OBJECT_TYPE,
                $user_view_rel_content_object->get_content_object_type());

        $condition = new AndCondition($conditions);

        return $this->update($user_view_rel_content_object, $condition);
    }

    function update_content_object_pub_feedback($content_object_pub_feedback)
    {
        $conditions[] = new EqualityCondition(ContentObjectPubFeedback :: PROPERTY_PUBLICATION_ID,
                $content_object_pub_feedback->get_publication_id());
        $conditions[] = new EqualityCondition(ContentObjectPubFeedback :: PROPERTY_CLOI_ID,
                $content_object_pub_feedback->get_cloi_id());
        $conditions[] = new EqualityCondition(ContentObjectPubFeedback :: PROPERTY_FEEDBACK_ID,
                $content_object_pub_feedback->get_feedback_id());
        $condition = new AndCondition($conditions);

        return $this->update($content_object_pub_feedback, $condition);
    }

    function create_user_view_rel_content_object($user_view_rel_content_object)
    {
        return $this->create($user_view_rel_content_object);
    }

    function create_content_object_pub_feedback($content_object_pub_feedback)
    {
        return $this->create($content_object_pub_feedback);
    }

    function retrieve_user_view_rel_content_objects($condition = null, $offset = null, $count = null, $order_property = null)
    {
        return $this->retrieve_objects(UserViewRelContentObject :: get_table_name(), $condition, $offset, $count,
                $order_property, UserViewRelContentObject :: CLASS_NAME);
    }

    function retrieve_content_object_pub_feedback($condition = null, $offset = null, $count = null, $order_property = null)
    {
        return $this->retrieve_objects(ContentObjectPubFeedback :: get_table_name(), $condition, $offset, $count,
                $order_property, ContentObjectPubFeedback :: CLASS_NAME);
    }

    function delete_content_object_pub_feedback($content_object_pub_feedback)
    {
        $condition = new EqualityCondition(ContentObjectPubFeedback :: PROPERTY_FEEDBACK_ID,
                $content_object_pub_feedback->get_feedback_id());

        $success = $this->delete(ContentObjectPubFeedback :: get_table_name(), $condition);

        return $success;
    }

    function reset_user_view($user_view)
    {
        $condition = new EqualityCondition(UserViewRelContentObject :: PROPERTY_VIEW_ID, $user_view->get_id());
        $properties[UserViewRelContentObject :: PROPERTY_VISIBILITY] = '0';

        return $this->update_objects(UserViewRelContentObject :: get_table_name(), $properties, $condition);
    }

    /**
     * get the last post of a (sub)forum his topics
     *
     * @param $forum_id int
     *
     * @return ForumPost
     */
    function retrieve_last_post_forum_topics($forum_id)
    {
        $complex_item_alias = $this->get_alias(ComplexContentObjectItem :: get_table_name());
        $forum_post_alias = $this->get_alias(ForumPost :: get_table_name());
        $forum_alias = $this->get_alias(Forum :: get_table_name());
        $forum_topic_alias = $this->get_alias(ForumTopic :: get_table_name());

        $query = 'SELECT *';
        $query .= ' JOIN ' . $this->escape_table_name(ForumPost :: get_table_name()) . ' AS ' . $forum_post_alias . ' ON ' . $this->escape_column_name(
                ForumTopic :: PROPERTY_ID, $forum_topic_alias) . ' = ' . $this->escape_column_name(
                ForumPost :: PROPERTY_FORUM_TOPIC_ID, $forum_post_alias);
        $query .= ' WHERE' . $this->escape_column_name(ComplexContentObjectItem :: PROPERTY_PARENT, $complex_item_alias) . ' = ' . $forum_id;

        return $this->retrieve_object_set($query, ForumPost :: get_table_name(), null, 0, 1, array(),
                ForumPost :: CLASS_NAME);
        ' AS ' . $complex_item_alias;

        $query .= ' JOIN ' . $this->escape_table_name(Forum :: get_table_name()) . ' AS ' . $forum_alias . ' ON ' . $this->escape_column_name(
                ComplexContentObjectItem :: PROPERTY_PARENT, $complex_item_alias) . ' = ' . $this->escape_column_name(
                Forum :: PROPERTY_ID, $forum_alias);

        $query .= ' JOIN ' . $this->escape_table_name(ForumTopic :: get_table_name()) . ' AS ' . $forum_topic_alias . ' ON ' . $this->escape_column_name(
                ComplexContentObjectItem :: PROPERTY_REF, $complex_item_alias) . ' = ' . $this->escape_column_name(
                ForumTopic :: PROPERTY_ID, $forum_topic_alias);

        $query .= ' JOIN ' . $this->escape_table_name(ForumPost :: get_table_name()) . ' AS ' . $forum_post_alias . ' ON ' . $this->escape_column_name(
                ForumTopic :: PROPERTY_ID, $forum_topic_alias) . ' = ' . $this->escape_column_name(
                ForumPost :: PROPERTY_FORUM_TOPIC_ID, $forum_post_alias);

        $condition = new EqualityCondition(ComplexContentObjectItem :: PROPERTY_PARENT, $forum_id, $complex_item_alias,
                true);

        $order_by = array();
        $order_by[] = new ObjectTableOrder(ForumPost :: PROPERTY_CREATION_DATE, SORT_DESC, $forum_post_alias);

        return $this->retrieve_object_set($query, ForumPost :: get_table_name(), $condition, 0, 1, $order_by,
                ForumPost :: CLASS_NAME);
    }

    /**
     * get the last post of a (sub)forum his subforum(s)
     *
     * @param $forum_id int
     *
     * @return ForumPost
     */
    function retrieve_last_post_forum_subforums($forum_id)
    {
        $complex_item_alias = $this->get_alias(ComplexContentObjectItem :: get_table_name());
        $forum_post_alias = $this->get_alias(ForumPost :: get_table_name());
        $forum_alias = $this->get_alias(Forum :: get_table_name());

        $help_table_repo_forum = 'forum';

        $forum_alias_bis = $this->get_alias($help_table_repo_forum);

        $query = 'SELECT *';
        $query .= ' JOIN ' . $this->escape_table_name('forum') . ' AS ' . $forum_alias_bis . ' ON ' . $this->escape_column_name(
                ComplexContentObjectItem :: PROPERTY_REF, $complex_item_alias) . ' = ' . $this->escape_column_name(
                Forum :: PROPERTY_ID, $forum_alias_bis);
        $query .= ' JOIN ' . $this->escape_table_name(ForumPost :: get_table_name()) . ' AS ' . $forum_post_alias . ' ON ' . $this->escape_column_name(
                Forum :: PROPERTY_LAST_POST, $forum_alias_bis) . ' = ' . $this->escape_column_name(
                ForumPost :: PROPERTY_ID, $forum_post_alias);
        $query .= ' WHERE' . $this->escape_column_name(ComplexContentObjectItem :: PROPERTY_PARENT, $complex_item_alias) . ' = ' . $forum_id;

        return $this->retrieve_object_set($query, ForumPost :: get_table_name(), null, 0, 1, array(),
                ForumPost :: CLASS_NAME);
        $complex_item_alias;

        $query .= ' JOIN ' . $this->escape_table_name(Forum :: get_table_name()) . ' AS ' . $forum_alias . ' ON ' . $this->escape_column_name(
                ComplexContentObjectItem :: PROPERTY_PARENT, $complex_item_alias) . ' = ' . $this->escape_column_name(
                Forum :: PROPERTY_ID, $forum_alias);

        $query .= ' JOIN ' . $this->escape_table_name($help_table_repo_forum) . ' AS ' . $forum_alias_bis . ' ON ' . $this->escape_column_name(
                ComplexContentObjectItem :: PROPERTY_REF, $complex_item_alias) . ' = ' . $this->escape_column_name(
                Forum :: PROPERTY_ID, $forum_alias_bis);

        $query .= ' JOIN ' . $this->escape_table_name(ForumPost :: get_table_name()) . ' AS ' . $forum_post_alias . ' ON ' . $this->escape_column_name(
                Forum :: PROPERTY_LAST_POST, $forum_alias_bis) . ' = ' . $this->escape_column_name(
                ForumPost :: PROPERTY_ID, $forum_post_alias);

        $condition = new EqualityCondition(ComplexContentObjectItem :: PROPERTY_PARENT, $forum_id, $complex_item_alias,
                true);

        $order_by = array();
        $order_by[] = new ObjectTableOrder(ForumPost :: PROPERTY_CREATION_DATE, SORT_DESC, $forum_post_alias);

        return $this->retrieve_object_set($query, ForumPost :: get_table_name(), $condition, 0, 1, $order_by,
                ForumPost :: CLASS_NAME);
    }

    function set_new_clo_version($lo_id, $new_lo_id)
    {
        $translator = ConditionTranslator :: factory($this);
        $condition = $translator->render_query(
                new EqualityCondition(ComplexContentObjectItem :: PROPERTY_PARENT, $lo_id), false);

        $props = array();
        $props[$this->escape_column_name(ComplexContentObjectItem :: PROPERTY_PARENT)] = $new_lo_id;
        $this->get_connection()->loadModule('Extended');
        return $this->get_connection()->extended->autoExecute(
                $this->get_table_name(ComplexContentObjectItem :: get_table_name()), $props, MDB2_AUTOQUERY_UPDATE,
                $condition);
    }

    function retrieve_external_instance_condition($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        return $this->retrieve_objects(ExternalInstance :: get_table_name(), $condition, $offset, $max_objects,
                $order_by, ExternalInstance :: CLASS_NAME);
    }

    function retrieve_external_instance($external_instance_id)
    {
        $condition = new EqualityCondition(ExternalInstance :: PROPERTY_ID, $external_instance_id);
        return $this->retrieve_object(ExternalInstance :: get_table_name(), $condition, array(),
                ExternalInstance :: CLASS_NAME);
    }

    function retrieve_external_instances($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        return $this->retrieve_objects(ExternalInstance :: get_table_name(), $condition, $offset, $max_objects,
                $order_by, ExternalInstance :: CLASS_NAME);
    }

    function count_external_instances($condition = null)
    {
        return $this->count_objects(ExternalInstance :: get_table_name(), $condition);
    }

    function retrieve_active_external_instance_types($instance_type)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(ExternalInstance :: PROPERTY_ENABLED, 1);
        $conditions[] = new EqualityCondition(ExternalInstance :: PROPERTY_TYPE, $instance_type);
        $condition = new AndCondition($conditions);
        return $this->retrieve_distinct(ExternalInstance :: get_table_name(), ExternalInstance :: PROPERTY_TYPE,
                $condition, array(), ExternalInstance :: CLASS_NAME);
    }

    function retrieve_external_repository_user_quotum($user_id, $external_repository_id)
    {
        $condition2 = new EqualityCondition(ExternalRepositoryUserQuotum :: PROPERTY_USER_ID, $user_id);
        $condition1 = new EqualityCondition(ExternalRepositoryUserQuotum :: PROPERTY_EXTERNAL_REPOSITORY_ID,
                $external_repository_id);

        $condition = new AndCondition($condition1, $condition2);
        return $this->retrieve_object(ExternalRepositoryUserQuotum :: get_table_name(), $condition, array(),
                ExternalRepositoryUserQuotum :: CLASS_NAME);
    }

    function retrieve_catalog($query, $table_name, $condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        /*
         * Get 'catalog' alias and add it to the query in order to support WHERE and ORDER BY clause
         */
        $after_from_position = stripos($query, 'from') + 4;
        $sub_query = trim(substr($query, $after_from_position));

        if (stripos($sub_query, ' ') !== false)
        {
            $real_table_name = trim(substr($sub_query, 0, stripos($query, ' ')));
        }
        else
        {
            $real_table_name = $sub_query;
        }

        $after_table_position = stripos($query, $real_table_name) + strlen($real_table_name);
        $alias = $this->get_alias('Catalog');
        $query = substr($query, 0, $after_table_position) . ' AS ' . $alias . ' ' . substr($query,
                $after_table_position);

        if (isset($condition))
        {
            $condition->set_storage_unit('Catalog');
        }

        if (isset($order_by))
        {
            $order_by->set_alias($alias);
        }

        return $this->retrieve_object_set($query, $table_name, $condition, $offset, $max_objects, $order_by);
    }

    function create_external_sync($external_sync)
    {
        return $this->create($external_sync);
    }

    function update_external_sync($external_sync)
    {
        $condition = new EqualityCondition(ExternalSync :: PROPERTY_ID, $external_sync->get_id());
        return $this->update($external_sync, $condition);
    }

    function delete_external_sync($external_sync)
    {
        $condition = new EqualityCondition(ExternalSync :: PROPERTY_ID, $external_sync->get_id());
        return $this->delete($external_sync->get_table_name(), $condition);
    }

    function retrieve_external_sync($condition)
    {
        $content_object_alias = $this->get_alias(ContentObject :: get_table_name());
        $synchronization_alias = $this->get_alias(ExternalSync :: get_table_name());

        $query = 'SELECT ' . $synchronization_alias . '.* FROM ' . $this->escape_table_name(
                ExternalSync :: get_table_name()) . ' AS ' . $synchronization_alias . ' JOIN ' . $this->escape_table_name(
                ContentObject :: get_table_name()) . ' AS ' . $content_object_alias . ' ON ' . $this->escape_column_name(
                ExternalSync :: PROPERTY_CONTENT_OBJECT_ID, $synchronization_alias) . ' = ' . $this->escape_column_name(
                ContentObject :: PROPERTY_ID, $content_object_alias);
        $record = $this->retrieve_row($query, ExternalSync :: get_table_name(), $condition);

        if ($record)
        {
            return self :: record_to_object($record, ExternalSync :: CLASS_NAME);
        }
        else
        {
            return false;
        }
    }

    function retrieve_external_syncs($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        $content_object_alias = $this->get_alias(ContentObject :: get_table_name());
        $synchronization_alias = $this->get_alias(ExternalSync :: get_table_name());

        $query = 'SELECT ' . $synchronization_alias . '.* FROM ' . $this->escape_table_name(
                ExternalSync :: get_table_name()) . ' AS ' . $synchronization_alias . ' JOIN ' . $this->escape_table_name(
                ContentObject :: get_table_name()) . ' AS ' . $content_object_alias . ' ON ' . $this->escape_column_name(
                ExternalSync :: PROPERTY_CONTENT_OBJECT_ID, $synchronization_alias) . ' = ' . $this->escape_column_name(
                ContentObject :: PROPERTY_ID, $content_object_alias);
        return $this->retrieve_object_set($query, ExternalSync :: get_table_name(), $condition, $offset, $max_objects,
                $order_by, ExternalSync :: CLASS_NAME);
    }

    function delete_content_object_includes($object, $delete_from_parent = false)
    {
        $column = ($delete_from_parent) ? 'content_object_id' : 'include_id';
        $query = 'DELETE FROM ' . $this->escape_table_name('content_object_include') . ' WHERE ' . $this->escape_column_name(
                $column) . '=' . $this->quote($object->get_id());
        $affectedRows = $this->exec($query);

        return ($affectedRows !== false);
    }

    function delete_assisting_content_objects($object)
    {
        $assisting_types = RepositoryDataManager :: get_active_helper_types();
        $failures = 0;

        foreach ($assisting_types as $type)
        {
            $repository_content_object_alias = $this->get_alias(ContentObject :: get_table_name());
            $repository_type_alias = $this->get_alias($type);

            $query = 'SELECT * ';
            $query .= ' FROM ' . $this->escape_table_name(ContentObject :: get_table_name()) . ' as ' . $repository_content_object_alias;
            $query .= ' JOIN ' . $this->escape_table_name($type) . ' AS ' . $repository_type_alias . ' ON ' . $this->escape_column_name(
                    ContentObject :: PROPERTY_ID, $repository_content_object_alias) . ' = ' . $this->escape_column_name(
                    ComplexContentObjectItem :: PROPERTY_ID, $repository_type_alias);

            $condition = new EqualityCondition('reference_id', $object->get_id(), $repository_type_alias, true);
            $version_condition = new NotCondition(
                    new EqualityCondition(ContentObject :: PROPERTY_CURRENT, ContentObject :: CURRENT_OLD));
            $condition = new AndCondition(array($condition, $version_condition));

            $assisting_objects = $this->retrieve_object_set($query, ContentObject :: get_table_name(), $condition);

            while ($assisting_object = $assisting_objects->next_result())
            {
                if (! RepositoryDataManager :: delete_clois_for_content_object($assisting_object))
                {
                    $failures ++;
                }

                if (! $assisting_object->delete())
                {
                    $failures ++;
                }
            }
        }

        return ($failures == 0);
    }

    function retrieve_doubles_in_repository($condition, $order_property, $offset, $count)
    {
        $co_table = $this->escape_table_name(ContentObject :: get_table_name());
        $co_alias = $this->get_alias(ContentObject :: get_table_name());

        $sql = 'SELECT ' . $co_alias . '.id, title, description, type, count(content_hash) as content_hash FROM ' . $co_table . ' as ' . $co_alias;

        $version_condition = new NotCondition(
                new EqualityCondition(ContentObject :: PROPERTY_CURRENT, ContentObject :: CURRENT_OLD));
        $condition = ! isset($condition) ? $version_condition : new AndCondition(array($condition, $version_condition));

        if (isset($condition))
        {
            $translator = ConditionTranslator :: factory($this, $co_alias);
            $sql .= $translator->render_query($condition);
        }

        $sql .= ' GROUP BY content_hash HAVING count(content_hash) > 1';

        return $this->retrieve_object_set($sql, ContentObject :: get_table_name(), null, $offset, $count,
                $order_property);
    }

    function count_doubles_in_repository($condition)
    {
        $co_table = $this->escape_table_name(ContentObject :: get_table_name());
        $co_alias = $this->get_alias(ContentObject :: get_table_name());

        $sql = 'SELECT COUNT(*) FROM ' . $co_table . ' as ' . $co_alias;

        $version_condition = new NotCondition(
                new EqualityCondition(ContentObject :: PROPERTY_CURRENT, ContentObject :: CURRENT_OLD));
        $condition = ! isset($condition) ? $version_condition : new AndCondition(array($condition, $version_condition));

        if (isset($condition))
        {
            $translator = ConditionTranslator :: factory($this, $co_alias);
            $sql .= $translator->render_query($condition);
        }

        $sql .= ' GROUP BY content_hash HAVING count(content_hash) > 1';

        return $this->count_result_set($sql, ContentObject :: get_table_name());
    }

    function create_external_setting(ExternalSetting $external_setting)
    {
        return $this->create($external_setting);
    }

    function update_external_setting(ExternalSetting $external_setting)
    {
        $condition = new EqualityCondition(ExternalSetting :: PROPERTY_ID, $external_setting->get_id());
        return $this->update($external_setting, $condition);
    }

    function delete_external_setting(ExternalSetting $external_setting)
    {
        $condition = new EqualityCondition(ExternalSetting :: PROPERTY_ID, $external_setting->get_id());
        return $this->delete($external_setting->get_table_name(), $condition);
    }

    /**
     *
     * @param $id int
     */
    function retrieve_external_setting($id)
    {
        $condition = new EqualityCondition(ExternalSetting :: PROPERTY_ID, $id);
        return $this->retrieve_object(ExternalSetting :: get_table_name(), $condition, array(),
                ExternalSetting :: CLASS_NAME);
    }

    function retrieve_external_settings($condition = null, $order_by = array(), $offset = 0, $max_objects = -1)
    {
        return $this->retrieve_objects(ExternalSetting :: get_table_name(), $condition, $offset, $max_objects,
                $order_by, ExternalSetting :: CLASS_NAME);
    }

    /**
     *
     * @param $id int
     */
    function retrieve_external_user_setting($id)
    {
        $condition = new EqualityCondition(ExternalUserSetting :: PROPERTY_ID, $id);
        return $this->retrieve_object(ExternalUserSetting :: get_table_name(), $condition, array(),
                ExternalUserSetting :: CLASS_NAME);
    }

    function retrieve_external_user_settings($condition = null, $order_by = array(), $offset = 0, $max_objects = -1)
    {
        return $this->retrieve_objects(ExternalUserSetting :: get_table_name(), $condition, $offset, $max_objects,
                $order_by, ExternalUserSetting :: CLASS_NAME);
    }

    function retrieve_external_setting_from_variable_name($variable, $external_id)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(ExternalSetting :: PROPERTY_EXTERNAL_ID, $external_id);
        $conditions[] = new EqualityCondition(ExternalSetting :: PROPERTY_VARIABLE, $variable);
        $condition = new AndCondition($conditions);
        return $this->retrieve_object(ExternalSetting :: get_table_name(), $condition, array(),
                ExternalSetting :: CLASS_NAME);
    }

    function create_external_user_setting(ExternalUserSetting $external_user_setting)
    {
        return $this->create($external_user_setting);
    }

    function update_external_user_setting(ExternalUserSetting $external_user_setting)
    {
        $condition = new EqualityCondition(ExternalUserSetting :: PROPERTY_ID, $external_user_setting->get_id());
        return $this->update($external_user_setting, $condition);
    }

    function delete_external_user_setting(ExternalUserSetting $external_user_setting)
    {
        $condition = new EqualityCondition(ExternalUserSetting :: PROPERTY_ID, $external_user_setting->get_id());
        return $this->delete($external_user_setting->get_table_name(), $condition);
    }

    function delete_external_user_settings($condition = null)
    {
        return $this->delete_objects(ExternalUserSetting :: get_table_name(), $condition);
    }

    function update_external_instance($external_instance)
    {
        $condition = new EqualityCondition(ExternalInstance :: PROPERTY_ID, $external_instance->get_id());
        return $this->update($external_instance, $condition);
    }

    function delete_external_instance($external_instance)
    {
        $condition = new EqualityCondition(ExternalInstance :: PROPERTY_ID, $external_instance->get_id());
        return $this->delete(ExternalInstance :: get_table_name(), $condition);
    }

    function create_external_repository_user_quotum(ExternalRepositoryUserQuotum $external_repository_user_quotum)
    {
        return $this->create($external_repository_user_quotum);
    }

    function update_external_repository_user_quotum(ExternalRepositoryUserQuotum $external_repository_user_quotum)
    {
        $condition = new EqualityCondition(ExternalRepositoryUserQuotum :: PROPERTY_ID,
                $external_repository_user_quotum->get_id());
        return $this->update($external_repository_user_quotum, $condition);
    }

    function delete_external_repository_user_quotum(ExternalRepositoryUserQuotum $external_repository_user_quotum)
    {
        $condition = new EqualityCondition(ExternalRepositoryUserQuotum :: PROPERTY_ID,
                $external_repository_user_quotum->get_id());
        return $this->delete(ExternalRepositoryUserQuotum :: get_table_name(), $condition);
    }

    /*
     * Content Object User Share
     */
    function delete_content_object_user_share(ContentObjectUserShare $content_object_user_share)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObjectUserShare :: PROPERTY_CONTENT_OBJECT_ID,
                $content_object_user_share->get_content_object_id());
        $conditions[] = new EqualityCondition(ContentObjectUserShare :: PROPERTY_USER_ID,
                $content_object_user_share->get_user_id());
        $condition = new AndCondition($conditions);

        return $this->delete(ContentObjectUserShare :: get_table_name(), $condition);
    }

    /**
     * Deletes the share of a content object with a specific user
     *
     * @param $content_object_id integer
     * @param $user_id integer
     */
    function delete_content_object_user_share_by_content_object_and_user_id($content_object_id, $user_id)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObjectUserShare :: PROPERTY_CONTENT_OBJECT_ID, $content_object_id);
        $conditions[] = new EqualityCondition(ContentObjectUserShare :: PROPERTY_USER_ID, $user_id);
        $condition = new AndCondition($conditions);

        return $this->delete(ContentObjectUserShare :: get_table_name(), $condition);
    }

    /**
     * Deletes all the user share given the content object id
     *
     * @param $content_object_user_share_id int
     * @return bool
     */
    function delete_all_content_object_user_shares_by_content_object_id($content_object_id)
    {
        $condition = new EqualityCondition(ContentObjectUserShare :: PROPERTY_CONTENT_OBJECT_ID, $content_object_id);

        return $this->delete(ContentObjectUserShare :: get_table_name(), $condition);
    }

    function update_content_object_user_share(ContentObjectUserShare $content_object_user_share)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObjectUserShare :: PROPERTY_CONTENT_OBJECT_ID,
                $content_object_user_share->get_content_object_id());
        $conditions[] = new EqualityCondition(ContentObjectUserShare :: PROPERTY_USER_ID,
                $content_object_user_share->get_user_id());
        $condition = new AndCondition($conditions);

        return $this->update($content_object_user_share, $condition);
    }

    function create_content_object_user_share(ContentObjectUserShare $content_object_user_share)
    {
        return $this->create($content_object_user_share);
    }

    function count_content_object_user_shares(Condition $condition = null)
    {
        return $this->count_objects(ContentObjectUserShare :: get_table_name(), $condition);
    }

    function retrieve_content_object_user_shares(Condition $condition = null, $offset = null, $count = null,
            ObjectTableOrder $order_property = null)
    {
        return $this->retrieve_objects(ContentObjectUserShare :: get_table_name(), $condition, $offset, $count,
                $order_property, ContentObjectUserShare :: CLASS_NAME);
    }

    /**
     * Deletes the share of a content object with a specific group
     *
     * @param $content_object_id integer
     * @param $group_id integer
     */
    function delete_content_object_group_share_by_content_object_and_group_id($content_object_id, $group_id)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObjectGroupShare :: PROPERTY_CONTENT_OBJECT_ID, $content_object_id);
        $conditions[] = new EqualityCondition(ContentObjectGroupShare :: PROPERTY_GROUP_ID, $group_id);
        $condition = new AndCondition($conditions);

        return $this->delete(ContentObjectGroupShare :: get_table_name(), $condition);
    }

    /**
     * Deletes all the group shares given the content object id
     *
     * @param $content_object_group_share_id int
     * @return bool
     */
    function delete_all_content_object_group_shares_by_content_object_id($content_object_id)
    {
        $condition = new EqualityCondition(ContentObjectGroupShare :: PROPERTY_CONTENT_OBJECT_ID, $content_object_id);

        return $this->delete(ContentObjectGroupShare :: get_table_name(), $condition);
    }

    /*
     * Content Object Group Share
     */
    function delete_content_object_group_share(ContentObjectGroupShare $content_object_group_share)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObjectGroupShare :: PROPERTY_CONTENT_OBJECT_ID,
                $content_object_group_share->get_content_object_id());
        $conditions[] = new EqualityCondition(ContentObjectGroupShare :: PROPERTY_GROUP_ID,
                $content_object_group_share->get_group_id());
        $condition = new AndCondition($conditions);

        return $this->delete(ContentObjectGroupShare :: get_table_name(), $condition);
    }

    function update_content_object_group_share(ContentObjectGroupShare $content_object_group_share)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObjectGroupShare :: PROPERTY_CONTENT_OBJECT_ID,
                $content_object_group_share->get_content_object_id());
        $conditions[] = new EqualityCondition(ContentObjectGroupShare :: PROPERTY_GROUP_ID,
                $content_object_group_share->get_group_id());
        $condition = new AndCondition($conditions);

        return $this->update($content_object_group_share, $condition);
    }

    function create_content_object_group_share(ContentObjectGroupShare $content_object_group_share)
    {
        return $this->create($content_object_group_share);
    }

    function count_content_object_group_shares(Condition $condition = null)
    {
        return $this->count_objects(ContentObjectGroupShare :: get_table_name(), $condition);
    }

    function retrieve_content_object_group_shares(Condition $condition = null, $offset = null, $count = null,
            ObjectTableOrder $order_property = null)
    {
        return $this->retrieve_objects(ContentObjectGroupShare :: get_table_name(), $condition, $offset, $count,
                $order_property, ContentObjectGroupShare :: CLASS_NAME);
    }

    function retrieve_shared_content_objects(Condition $condition = null, $offset = null, $count = null,
            ObjectTableOrder $order_property = null)
    {
        $content_object_alias = $this->get_alias(ContentObject :: get_table_name());
        $shared_content_object_rel_category_alias = $this->get_alias(SharedContentObjectRelCategory :: get_table_name());
        $query = 'SELECT ' . $content_object_alias . '.* FROM ';
        $query .= $this->escape_table_name(ContentObject :: get_table_name()) . ' AS ' . $content_object_alias;
        $query .= ' LEFT JOIN ';
        $query .= $this->escape_table_name(SharedContentObjectRelCategory :: get_table_name()) . ' AS ' . $shared_content_object_rel_category_alias;
        $query .= ' ON ';
        $query .= $this->escape_column_name(ContentObject :: PROPERTY_ID, $content_object_alias) . ' = ' . $this->escape_column_name(
                SharedContentObjectRelCategory :: PROPERTY_CONTENT_OBJECT_ID, $shared_content_object_rel_category_alias);

        return $this->retrieve_content_objects($condition, $order_property, $offset, $count, $query);
    }

    function retrieve_shared_type_content_objects($type, Condition $condition = null, $offset = null, $count = null,
            ObjectTableOrder $order_property = null)
    {
        $query = $this->get_shared_objects_query($type);
        $condition = $this->get_type_condition($type, $condition);

        $version_condition = new NotCondition(
                new EqualityCondition(ContentObject :: PROPERTY_CURRENT, ContentObject :: CURRENT_OLD));
        $condition = ! isset($condition) ? $version_condition : new AndCondition(array($condition, $version_condition));

        return $this->retrieve_object_set($query, ContentObject :: get_table_name(), $condition, $offset, $count,
                $order_property, ContentObject :: CLASS_NAME);
    }

    function count_shared_content_objects(Condition $condition = null)
    {
        $content_object_alias = $this->get_alias(ContentObject :: get_table_name());
        $shared_content_object_rel_category_alias = $this->get_alias(SharedContentObjectRelCategory :: get_table_name());
        $query = 'SELECT COUNT(1) FROM ';
        $query .= $this->escape_table_name(ContentObject :: get_table_name()) . ' AS ' . $content_object_alias;
        $query .= ' LEFT JOIN ';
        $query .= $this->escape_table_name(SharedContentObjectRelCategory :: get_table_name()) . ' AS ' . $shared_content_object_rel_category_alias;
        $query .= ' ON ';
        $query .= $this->escape_column_name(ContentObject :: PROPERTY_ID, $content_object_alias) . ' = ' . $this->escape_column_name(
                SharedContentObjectRelCategory :: PROPERTY_CONTENT_OBJECT_ID, $shared_content_object_rel_category_alias);

        return $this->count_content_objects($condition, $query);
    }

    /**
     * Retrieves shared_content_object_rel_categories
     *
     * @param $condition type
     * @param $offset type
     * @param $count type
     * @param $order_property ObjectTableOrder
     * @return Mdb2ResultSet
     */
    function retrieve_shared_content_object_rel_categories($condition = null, $offset = null, $count = null,
            ObjectTableOrder $order_property = null)
    {
        $shared_content_object_rel_category_alias = $this->get_alias(SharedContentObjectRelCategory :: get_table_name());
        $repository_category_alias = $this->get_alias(RepositoryCategory :: get_table_name());

        $query = 'SELECT ' . $shared_content_object_rel_category_alias . '.*';
        $query .= ' FROM ' . $this->escape_table_name(SharedContentObjectRelCategory :: get_table_name()) . ' AS ' . $shared_content_object_rel_category_alias;
        $query .= ' JOIN ' . $this->escape_table_name(RepositoryCategory :: get_table_name()) . ' AS ' . $repository_category_alias;
        $query .= ' ON ';
        $query .= $this->escape_column_name(SharedContentObjectRelCategory :: PROPERTY_CATEGORY_ID,
                $shared_content_object_rel_category_alias) . ' = ' . $this->escape_column_name(
                RepositoryCategory :: PROPERTY_ID, $repository_category_alias);

        return $this->retrieve_object_set($query, SharedContentObjectRelCategory :: get_table_name(), $condition,
                $offset, $count, $order_property, SharedContentObjectRelCategory :: class_name());
    }

    function count_shared_type_content_objects($type, Condition $condition = null)
    {
        $query = $this->get_shared_objects_query($type, true);
        $condition = $this->get_type_condition($type, $condition);

        $version_condition = new NotCondition(
                new EqualityCondition(ContentObject :: PROPERTY_CURRENT, ContentObject :: CURRENT_OLD));
        $condition = ! isset($condition) ? $version_condition : new AndCondition(array($condition, $version_condition));

        return $this->count_result_set($query, ContentObject :: get_table_name(), $condition);
    }

    private function get_type_condition($type, $condition)
    {
        if (! ContentObject :: is_extended_type($type))
        {
            if (! $condition)
            {
                $condition = new EqualityCondition(ContentObject :: PROPERTY_TYPE, $type,
                        $this->get_alias(ContentObject :: get_table_name()));
            }
            else
            {
                $conditions = array();
                $conditions[] = $condition;
                $conditions[] = new EqualityCondition(ContentObject :: PROPERTY_TYPE, $type,
                        $this->get_alias(ContentObject :: get_table_name()));
                $condition = new AndCondition($conditions);
            }
        }

        return $condition;
    }

    private function get_shared_objects_query($type = null, $count = false)
    {
        $content_object_table = $this->escape_table_name(ContentObject :: get_table_name());
        $content_object_alias = $this->get_alias(ContentObject :: get_table_name());
        $content_object_user_share_table = $this->escape_table_name(ContentObjectUserShare :: get_table_name());
        $content_object_user_share_alias = $this->get_alias(ContentObjectUserShare :: get_table_name());
        $content_object_group_share_table = $this->escape_table_name(ContentObjectGroupShare :: get_table_name());
        $content_object_group_share_alias = $this->get_alias(ContentObjectGroupShare :: get_table_name());

        if (! $count)
        {
            $query = 'SELECT DISTINCT(' . $content_object_alias . '.id), ' . $content_object_alias . '.*, ' . $content_object_user_share_alias . '.right_id AS user_right, ' . $content_object_group_share_alias . '.right_id AS group_right';
        }
        else
        {
            $query = 'SELECT COUNT(DISTINCT(' . $content_object_alias . '.id))';
        }
        $query .= ' FROM ' . $content_object_table . ' AS ' . $content_object_alias;
        $query .= ' LEFT JOIN ' . $content_object_user_share_table . '  AS ' . $content_object_user_share_alias . ' ON ';
        $query .= $content_object_alias . '.' . ContentObject :: PROPERTY_ID . ' = ' . $content_object_user_share_alias . '.' . ContentObjectUserShare :: PROPERTY_CONTENT_OBJECT_ID;
        $query .= ' LEFT JOIN ' . $content_object_group_share_table . '  AS ' . $content_object_group_share_alias . ' ON ';
        $query .= $content_object_alias . '.' . ContentObject :: PROPERTY_ID . ' = ' . $content_object_group_share_alias . '.' . ContentObjectGroupShare :: PROPERTY_CONTENT_OBJECT_ID;

        if (! is_null($type))
        {
            $type_alias = $this->get_alias($type);

            if (ContentObject :: is_extended_type($type))
            {
                $query .= ' JOIN ' . $this->escape_table_name($type) . ' AS ' . $type_alias . ' ON ' . $this->escape_column_name(
                        ContentObject :: PROPERTY_ID, $content_object_alias) . ' = ' . $this->escape_column_name(
                        ContentObject :: PROPERTY_ID, $type_alias);
            }
        }

        return $query;
    }

    function retrieve_content_object_user_share($content_object_id, $user_id)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObjectUserShare :: PROPERTY_CONTENT_OBJECT_ID, $content_object_id);
        $conditions[] = new EqualityCondition(ContentObjectUserShare :: PROPERTY_USER_ID, $user_id);
        $condition = new AndCondition($conditions);

        return $this->retrieve_object(ContentObjectUserShare :: get_table_name(), $condition, array(),
                ContentObjectUserShare :: CLASS_NAME);
    }

    function retrieve_content_object_group_share($content_object_id, $group_id)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObjectGroupShare :: PROPERTY_CONTENT_OBJECT_ID, $content_object_id);
        $conditions[] = new EqualityCondition(ContentObjectGroupShare :: PROPERTY_GROUP_ID, $group_id);
        $condition = new AndCondition($conditions);

        return $this->retrieve_object(ContentObjectGroupShare :: get_table_name(), $condition, array(),
                ContentObjectGroupShare :: CLASS_NAME);
    }

    function retrieve_active_external_instances($manager_types = array(), $types = array())
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(ExternalInstance :: PROPERTY_ENABLED, 1);
        foreach ($manager_types as $manager_type)
        {
            foreach ($types as $type)
            {
                $namespaces[] = 'common\extensions\\' . $manager_type . '\implementation\\' . $type;
            }
        }

        if (count($namespaces) > 0)
        {
            $conditions[] = new InCondition(ExternalInstance :: PROPERTY_TYPE, $namespaces);
        }

        $condition = new AndCondition($conditions);

        return $this->retrieve_external_instances($condition);
    }

    function content_object_title_exists($title, $parent_id = null, $content_object_id = null)
    {
        $conditions = array();
        if (! is_null($parent_id))
        {
            $conditions[] = new EqualityCondition(ContentObject :: PROPERTY_PARENT_ID, $parent_id);
        }
        if (! is_null($content_object_id))
        {
            $conditions[] = new NotCondition(new EqualityCondition(ContentObject :: PROPERTY_ID, $content_object_id));
        }

        $conditions[] = new EqualityCondition(ContentObject :: PROPERTY_TITLE, $title);
        $conditions[] = new EqualityCondition(ContentObject :: PROPERTY_OWNER_ID, Session :: get_user_id());
        $conditions[] = new InCondition(ContentObject :: PROPERTY_TYPE, RepositoryDataManager :: get_registered_types());
        $condition = new AndCondition($conditions);

        $count = RepositoryDataManager :: get_instance()->count_content_objects($condition);
        return ($count > 0);
    }

    function update_external_instance_objects($condition, $properties)
    {
        return $this->update_objects(ExternalSync :: get_table_name(), $properties, $condition);
    }

    function deactivate_content_object_type($condition)
    {
        $properties = array();
        $properties[ContentObject :: PROPERTY_STATE] = $this->escape_column_name(ContentObject :: PROPERTY_STATE) . ' + ' . ContentObject :: STATE_INACTIVE;
        return $this->update_objects(ContentObject :: get_table_name(), $properties, $condition);
    }

    function activate_content_object_type($condition)
    {
        $properties = array();
        $properties[ContentObject :: PROPERTY_STATE] = $this->escape_column_name(ContentObject :: PROPERTY_STATE) . ' - ' . ContentObject :: STATE_INACTIVE;
        return $this->update_objects(ContentObject :: get_table_name(), $properties, $condition);
    }
}
?>