<?php
namespace repository;

use common\libraries\Condition;
use common\libraries\ObjectTableOrder;

/**
 *
 * @package repository.lib
 *
 *          This is an interface for a data manager for the Repository
 *          application.
 *          Data managers must implement this class.
 *
 * @author Hans De Bisschop
 */
interface RepositoryDataManagerInterface
{

    /**
     * Is the object attached to another one ?
     *
     * @param
     *            ContentObject The object.
     * @return boolean Is Attached.
     */
    function is_attached($object, $type = null);

    /**
     * Gets all ids of all children/grandchildren/...
     * of a given object.
     *
     * @param $object ContentObject
     *            The learning object
     * @return array The requested id's
     */
    function get_children_ids($object);

    /**
     * Get number of times a physical document is used by an object's
     * versions.
     *
     * @param $path String
     *            The document path
     * @return boolean True if the physical document occurs only once, else
     *         False.
     */
    function is_only_document_occurence($path);

    /**
     * Gets all ids of all versions of a given object.
     *
     * @param $object ContentObject
     *            The object
     * @return array The requested id's
     */
    function get_version_ids($object);

    /**
     * Initializes the data manager.
     */
    function initialize();

    /**
     * Determines the type of the object with the given ID.
     *
     * @param $id int
     *            The ID of the object.
     * @return string The object type.
     */
    function determine_content_object_type($id);

    /**
     * Retrieves the object with the given ID from persistent
     * storage.
     * If the type of object is known, it should be
     * passed in order to save time.
     *
     * @param $id int
     *            The ID of the object.
     * @param $type string
     *            The type of the object. May be omitted.
     * @return ContentObject The object.
     */
    function retrieve_content_object($id, $type = null);

    /**
     * Retrieves the objects that match the given criteria from
     * persistent storage.
     * As far as ordering goes, there are two things to take into account:
     * - If, after applying the passed conditions, there is no order between
     * two objects, the display order index should be taken into
     * account.
     * - Regardless of what the order specification states, objects
     * of the "category" types must always come before others.
     * Finally, there are some limitations to this method:
     * - For now, you can only use the standard object properties,
     * not the type-specific ones IF you do not specify a single type of
     * object to retrieve.
     * - Future versions may include statistical functions.
     *
     * @param $type string
     *            The type of objects to retrieve, if any.
     *            If you do not specify a type, or the type is not
     *            known in advance, you will only be able to select
     *            on default properties; also, there will be a
     *            significant performance decrease. In this case,
     *            the values of the additional properties will not
     *            yet be known; they will be retrieved JIT, i.e.
     *            right before they are accessed.
     * @param $condition Condition
     *            The condition to use for learning object
     *            selection, structured as a Condition
     *            object. Please consult the appropriate
     *            documentation.
     * @param $order_by array
     *            An array of properties to sort the objects on.
     * @param $offset int
     *            The index of the first object to return. If
     *            omitted or negative, the result set will start
     *            from the first object.
     * @param $max_objects int
     *            The maximum number of objects to return. If
     *            omitted or non-positive, every object from the
     *            first index will be returned.
     * @param $state int
     *            The state the objects should have. Any of
     *            the ContentObject :: STATE_* constants. A negative
     *            number means the state should be ignored. Defaults
     *            to ContentObject :: STATE_NORMAL. You can just as
     *            easily use your own condition for this; this
     *            parameter is merely for convenience, and to ensure
     *            that the function does not apply to recycled objects
     *            by default.
     * @return ResultSet A set of matching objects.
     */
    function retrieve_content_objects($condition = null, $order_by = array (), $offset = 0, $max_objects = -1, $query = null);

    function retrieve_type_content_objects($type, $condition = null, $order_by = array (), $offset = 0, $max_objects = -1);

    /**
     * Retrieves the additional properties of the given learning object.
     *
     * @param $content_object ContentObject
     *            The object for which to
     *            fetch additional properties.
     * @return array The properties as an associative array.
     */
    function retrieve_additional_content_object_properties($content_object);

    /**
     * Returns the number of objects that match the given criteria.
     * This method has the same limitations as retrieve_content_objects.
     *
     * @param $type string
     *            The type of objects to search for, if any.
     *            If you do not specify a type, or the type is not
     *            known in advance, you will only be able to select
     *            on default properties; also, there will be a
     *            significant performance decrease.
     * @param $condition Condition
     *            The condition to use for object
     *            selection, structured as a Condition
     *            object. Please consult the appropriate
     *            documentation.
     * @param $state int
     *            The state the objects should have. Any of
     *            the ContentObject :: STATE_* constants. A negative
     *            number means the state should be ignored. Defaults
     *            to ContentObject :: STATE_NORMAL. You can just as
     *            easily use your own condition for this; this
     *            parameter is merely for convenience, and to ensure
     *            that the function does not apply to recycled objects
     *            by default.
     * @return int The number of matching objects.
     */
    function count_content_objects($condition = null, $query = null);

    function count_type_content_objects($type, $condition = null);

    /**
     * Returns the next available object number.
     *
     * @return int The ID.
     */
    function get_next_content_object_number();

    /**
     * Makes the given object persistent.
     *
     * @param $object ContentObject
     *            The object.
     * @return boolean True if creation succceeded, false otherwise.
     */
    function create_content_object($object, $type);

    /**
     * Updates the given object in persistent storage.
     *
     * @param $object ContentObject
     *            The object.
     * @return boolean True if the update succceeded, false otherwise.
     */
    function update_content_object($object);

    /**
     * Deletes the given object from persistent storage.
     * This function deletes
     * - all children of the given object (using this function
     * recursively)
     * - links from this object to other objects (so called attachments)
     * - links from other objects to this object (so called attachments)
     * - the object itself
     *
     * @param $object ContentObject
     *            The object.
     * @return boolean True if the given object was succesfully deleted, false
     *         otherwise. Deletion fails when the object is used
     *         somewhere in an application or if one of its children
     *         is in use.
     */
    function delete_content_object($object);

    /**
     * Creates a new complex object in the database
     *
     * @param $clo ComplexContentObject
     *            - The complex object
     * @return True if success
     */
    function create_complex_content_object_item($clo_item);

    /**
     * Updates a complex object in the database
     *
     * @param $clo ComplexContentObject
     *            - The complex object
     * @return True if success
     */
    function update_complex_content_object_item($clo_item);

    /**
     * Deletes a complex object in the database
     *
     * @param $clo ComplexContentObject
     *            - The complex object
     * @return True if success
     */
    function delete_complex_content_object_item($clo_item);

    /**
     * Retrieves a complex object from the database with a given id
     *
     * @param $clo_id Int
     * @return ComplexContentObjectItem The complex object
     */
    function retrieve_complex_content_object_item($clo_item_id);

    /**
     * Counts the available complex objects with the given condition
     *
     * @param $condition Condition
     * @return Int the amount of complex objects
     */
    function count_complex_content_object_items($condition);

    /**
     * Retrieves the complex object items with the given condition
     *
     * @param
     *            Condition
     */
    function retrieve_complex_content_object_items($condition = null, $order_by = array (), $offset = 0, $max_objects = -1);

    /**
     * Deletes the given object version from persistent storage.
     * This function deletes
     * - the selected version
     * This function updates
     * - the latest version entry if necessary
     *
     * @param $object ContentObject
     *            The object.
     * @return boolean True if the given version was succesfully deleted, false
     *         otherwise. Deletion fails when the version is used
     *         somewhere in an application or if one of its children
     *         is in use.
     */
    function delete_content_object_version($object);

    /**
     * Gets all objects from this user id, and removes them
     */
    function retrieve_content_object_by_user($user_id);

    function delete_content_object_attachments(Condition $condition);

    function delete_content_object_includes($object);

    function delete_assisting_content_objects($object);

    /**
     * Deletes all known objects from persistent storage.
     * @note Only for testing purpuses. This function also deletes the root
     * category of a user's repository.
     */
    function delete_all_content_objects();

    /**
     * Returns the objects that are attached to the object
     * with the given ID.
     *
     * @param $condition Condition
     *            The conditions to apply to the query
     * @return array The attached objects.
     */
    function retrieve_attached_content_objects(Condition $condition);

    /**
     * Counts the content objects to which the selected content object are
     * attached to
     *
     * @param $object ContentObject
     *            The content object
     * @return int The count
     */
    function count_objects_to_which_object_is_attached($object);

    /**
     * Returns the content objects to which the selected content object are
     * attached to
     *
     * @param $object ContentObject
     *            The content object
     * @return array The the content objects to which the selected content
     *         object are attached to
     */
    function retrieve_objects_to_which_object_is_attached($object);

    /**
     * Returns the objects that are included into the object
     * with the given ID.
     *
     * @param $object ContentObject
     *            The object for which to retrieve
     *            includes.
     * @return array The included objects.
     */
    function retrieve_included_content_objects($object);

    /**
     * Returns the content objects in which the selected content object are
     * included
     *
     * @param $object ContentObject
     *            The content object
     * @return array The the content objects in which the selected content
     *         object are included
     */
    function retrieve_objects_in_which_object_is_included($object);

    function is_content_object_already_included($content_object, $include_object_id);

    function is_content_object_already_attached($content_object, $attachment_id);

    /**
     * Counts the content objects in which the selected content object are
     * included
     *
     * @param $object ContentObject
     *            The content object
     * @return int The count
     */
    function count_objects_in_which_object_is_included($object);

    function retrieve_content_object_versions($object, $include_last = true, $include_self = true);

    function get_latest_version_id($object);

    /**
     * Adds an object to another's attachment list.
     *
     * @param $object ContentObject
     *            The object to attach the other
     *            object to.
     * @param $attachment_id int
     *            The ID of the object to attach.
     */
    function attach_content_object($object, $attachment_id, $type);

    /**
     * Removes an object from another's attachment list.
     *
     * @param $object ContentObject
     *            The object to detach the other
     *            object from.
     * @param $attachment_id int
     *            The ID of the object to detach.
     * @return boolean True if the attachment was removed, false if it did not
     *         exist.
     */
    function detach_content_object($object, $attachment_id, $type);

    /**
     * Adds an object to another's include list.
     *
     * @param $object ContentObject
     *            The object to include into the other
     *            object.
     * @param $attachment_id int
     *            The ID of the object to include.
     */
    function include_content_object($object, $include_id);

    /**
     * Removes an object from another's include list.
     *
     * @param $object ContentObject
     *            The object to exclude from the other
     *            object.
     * @param $attachment_id int
     *            The ID of the object to exclude.
     * @return boolean True if the include was removed, false if it did not
     *         exist.
     */
    function exclude_content_object($object, $include_id);

    /**
     * Sets the requested objects' state to one of the STATE_*
     * constants defined in the ContentObject class.
     * This function's main use
     * is to make an object's children inherit its state.
     *
     * @param $object_ids array
     *            The learning object IDs.
     * @param $state int
     *            The new state.
     * @return boolean True upon success, false upon failure.
     */
    function set_content_object_states($object_ids, $state);

    /**
     * Gets the disk space consumed by the given user.
     *
     * @param $user int
     *            The user ID.
     * @return int The number of bytes used.
     */
    function get_used_disk_space($user);

    /**
     * Creates a storage unit
     *
     * @param $name string
     *            Name of the storage unit
     * @param $properties array
     *            Properties of the storage unit
     * @param $indexes array
     *            The indexes which should be defined in the created
     *            storage unit
     */
    function create_storage_unit($name, $properties, $indexes);

    function select_next_category_display_order($parent_category_id, $user_id, $type);

    function delete_category($category);

    function update_category($category);

    function create_category($category);

    function count_categories($conditions = null);

    function retrieve_categories($condition = null, $offset = null, $count = null, $order_property = null);

    function delete_user_view($user_view);

    function update_user_view($user_view);

    function create_user_view($user_view);

    function count_user_views($conditions = null);

    function retrieve_user_views($condition = null, $offset = null, $count = null, $order_property = null);

    function update_user_view_rel_content_object($user_view_rel_content_object);

    function create_user_view_rel_content_object($user_view_rel_content_object);

    function create_content_object_pub_feedback($content_object_publication_feedback);

    function update_content_object_pub_feedback($content_object_publication_feedback);

    function delete_content_object_pub_feedback($content_object_publication_feedback);

    function retrieve_user_view_rel_content_objects($condition = null, $offset = null, $count = null, $order_property = null);

    function retrieve_content_object_pub_feedback($condition = null, $offset = null, $count = null, $order_property = null);

    function retrieve_last_post_forum_topics($forum_id);

    function retrieve_last_post_forum_subforums($forum_id);

    function retrieve_doubles_in_repository($condition, $order_property, $offset, $count);

    function count_doubles_in_repository($condition);

    /*
     * Content Object User Share
     */
    function delete_content_object_user_share(ContentObjectUserShare $content_object_user_share);

    function update_content_object_user_share(ContentObjectUserShare $content_object_user_share);

    function create_content_object_user_share(ContentObjectUserShare $content_object_user_share);

    function count_content_object_user_shares(Condition $condition = null);

    function retrieve_content_object_user_shares(Condition $condition = null, $offset = null, $count = null, ObjectTableOrder $order_property = null);

    /*
     * Content Object Group Share
     */
    function delete_content_object_group_share(ContentObjectGroupShare $content_object_group_share);

    function update_content_object_group_share(ContentObjectGroupShare $content_object_group_share);

    function create_content_object_group_share(ContentObjectGroupShare $content_object_group_share);

    function count_content_object_group_shares(Condition $condition = null);

    function retrieve_content_object_group_shares(Condition $condition = null, $offset = null, $count = null, ObjectTableOrder $order_property = null);

    function retrieve_content_object_user_share($content_object_id, $user_id);

    function retrieve_content_object_group_share($content_object_id, $group_id);

}
?>