<?php

namespace repository;

use Exception;
use common\libraries\Translation;
use common\libraries\SubManager;
use common\libraries\Theme;
use common\libraries\Utilities;

/**
 * Class that describes the actions to share content objects from within a context. 
 */
class ShareManager extends SubManager
{
    const PARAM_SHARE_ACTION = 'share_action';
    const PARAM_TARGET_USERS = 'target_users';
    const PARAM_TARGET_GROUPS = 'target_groups';
    
    const ACTION_BROWSE = 'browser';
    const ACTION_ADD_ENTITIES = 'create';
    const ACTION_REMOVE_ENTITY = 'delete';
    const ACTION_UPDATE_ENTITY = 'update';
    
    /**
     * The content objects for which the share rights need to be editted
     * @var Array
     */
    private $content_objects;

    /**
     * Launches the rights editor with the additional properties
     * @param Application $application
     * @param Array $locations
     * @param Array $entities
     */
    static function launch($application, $content_objects)
    {
        $component = self :: construct($application);
        
        if (empty($content_objects))
        {
            throw new Exception(Translation :: get('NoContentObjectsGiven'));
        }
        
        $component->set_content_objects($content_objects);
        $component->run();
    }

    // DEFAULT FUNCTIONS
    

    /**
     * Returns the parameter with which this submanager's actions are described
     * @return String
     */
    static function get_action_parameter()
    {
        return self :: PARAM_SHARE_ACTION;
    }

    /**
     * Returns the default action for this submannager
     * @return String
     */
    static function get_default_action()
    {
        return self :: ACTION_BROWSE;
    }

    /**
     * Returns the path to the components folder
     * @return String
     */
    function get_application_component_path()
    {
        return dirname(__FILE__) . '/component/';
    }

    /**
     * Returns the content objects
     * @return Array
     */
    function get_content_objects()
    {
        return $this->content_objects;
    }

    /**
     * Sets the content objects
     * @param array $content_objects
     */
    function set_content_objects(array $content_objects)
    {
        $this->content_objects = $content_objects;
    }

    /**
     * Returns the content object ids
     * @return array
     */
    function get_content_object_ids()
    {
        $content_object_ids = array();
        
        foreach ($this->content_objects as $content_object)
        {
            $content_object_ids[] = $content_object->get_id();
        }
        
        return $content_object_ids;
    }

    /**
     * Displays the content objects
     * @return String
     */
    function display_content_objects()
    {
        $html = array();
        $html[] = '<div class="content_object padding_10">';
        
        $html[] = '<div class="title">' . Translation :: get('SelectedObjects', array(
                'OBJECTS' => Translation :: get('ContentObjects')), Utilities :: COMMON_LIBRARIES) . '</div>';
        
        $html[] = '<div class="description">';
        $html[] = '<ul class="attachments_list">';
        
        foreach ($this->content_objects as $content_object)
        {
            $namespace = ContentObject :: get_content_object_type_namespace($content_object->get_type());
            
            $html[] = '<li><img src="' . Theme :: get_image_path($namespace) . 'logo/' . Theme :: ICON_MINI . '.png" alt="' . htmlentities(Translation :: get('TypeName', null, $namespace)) . '"/> ' . $content_object->get_title() . '</li>';
        }
        
        $html[] = '</ul>';
        $html[] = '</div>';
        $html[] = '</div>';
        
        return implode("\n", $html);
    }

    /**
     * Retrieves the location in the user type tree the user is currently browsing
     * e.g content object or category
     */
    static function get_current_user_tree_location($user_id, $content_object_id)
    {
        if (! is_null($content_object_id))
        {
            return RepositoryRights :: get_instance()->get_location_by_identifier_from_users_subtree(RepositoryRights :: TYPE_USER_CONTENT_OBJECT, $content_object_id, $user_id);
        }
        $category_id = Request :: get(RepositoryManager :: PARAM_CATEGORY_ID);
        if (! is_null($category_id))
        {
            return RepositoryRights :: get_instance()->get_location_by_identifier_from_users_subtree(RepositoryRights :: TYPE_USER_CATEGORY, $content_object_id, $user_id);
        }
        else
        {
            return RepositoryRights :: get_instance()->get_user_root($user_id);
        }
    }
}

?>
