<?php
namespace repository\quota;

use common\libraries\Filesystem;
use user\UserManager;
use common\libraries\InequalityCondition;
use common\libraries\ObjectTableOrder;
use group\GroupDataManager;
use group\Group;
use repository\RepositoryManager;
use repository\ContentObject;
use repository\RepositoryDataManager;
use user\User;
use common\libraries\Path;
use common\libraries\PlatformSetting;
use common\libraries\EqualityCondition;
use common\libraries\NotCondition;
use common\libraries\InCondition;
use common\libraries\AndCondition;

/**
 * This class provides some functionality to manage user disk quota
 *
 * @package repository.quota
 * @author Bart Mollet
 * @author Dieter De Neef
 * @author Hans De Bisschop
 */
class Calculator
{
    const POLICY_USER = 0;
    const POLICY_GROUP_HIGHEST = 1;
    const POLICY_GROUP_LOWEST = 2;
    const POLICY_HIGHEST = 3;
    const POLICY_LOWEST = 4;

    /**
     * The user
     *
     * @var \user\User
     */
    private $user;

    /**
     * Create a new Calculator
     *
     * @param $user \user\User
     */
    public function __construct(User $user, $reset = false)
    {
        $this->user = $user;
        if ($reset)
        {
            $this->reset_cache();
        }
    }

    /**
     * USER DISK QUOTA
     */
    public function get_used_user_disk_quota()
    {
        if (is_null($this->used_user_disk_quota))
        {
            $this->used_user_disk_quota = RepositoryDataManager :: get_instance()->get_used_disk_space(
                    $this->user->get_id());
        }
        return $this->used_user_disk_quota;
    }

    public function get_maximum_user_disk_quota()
    {
        if (is_null($this->maximum_user_disk_quota))
        {
            $policy = PlatformSetting :: get('quota_policy', __NAMESPACE__);
            $fallback = PlatformSetting :: get('quota_fallback', __NAMESPACE__);
            $fallback_user = PlatformSetting :: get('quota_fallback_user', __NAMESPACE__);

            switch ($policy)
            {
                case self :: POLICY_USER :
                    if ($this->user->get_disk_quota() || ! $fallback)
                    {
                        $this->maximum_user_disk_quota = $this->user->get_disk_quota();
                    }
                    else
                    {
                        $this->maximum_user_disk_quota = ($fallback_user == 0 ? $this->get_group_highest() : $this->get_group_lowest());
                    }
                    break;
                case self :: POLICY_GROUP_HIGHEST :
                    $group = $this->get_group_highest();
                    $this->maximum_user_disk_quota = $group || ! $fallback ? $group : $this->user->get_disk_quota();
                    break;
                case self :: POLICY_GROUP_LOWEST :
                    $group = $this->get_group_lowest();
                    $this->maximum_user_disk_quota = $group || ! $fallback ? $group : $this->user->get_disk_quota();
                    break;
                case self :: POLICY_HIGHEST :
                    $group = $this->get_group_highest();
                    $this->maximum_user_disk_quota = ($group > $this->user->get_disk_quota() ? $group : $this->user->get_disk_quota());
                    break;
                case self :: POLICY_LOWEST :
                    $group = $this->get_group_lowest();
                    $this->maximum_user_disk_quota = ($group > $this->user->get_disk_quota() || ! $group ? $this->user->get_disk_quota() : $group);
                    break;
                default :
                    $this->maximum_user_disk_quota = $this->user->get_disk_quota();
                    break;
            }
        }
        return $this->maximum_user_disk_quota;
    }

    function get_group_lowest()
    {
        $user_group_ids = $this->user->get_groups(true);

        $conditions = array();
        $conditions[] = new InCondition(Group :: PROPERTY_ID, $user_group_ids);
        $conditions[] = new InequalityCondition(Group :: PROPERTY_DISK_QUOTA, InequalityCondition :: GREATER_THAN, 0);
        $condition = new AndCondition($conditions);

        $group = GroupDataManager :: get_instance()->retrieve_object(Group :: get_table_name(), $condition,
                array(new ObjectTableOrder(Group :: PROPERTY_DISK_QUOTA, SORT_ASC)), Group :: CLASS_NAME);

        return $group instanceof Group ? $group->get_disk_quota() : 0;
    }

    function get_group_highest()
    {
        $user_group_ids = $this->user->get_groups(true);

        $conditions = array();
        $conditions[] = new InCondition(Group :: PROPERTY_ID, $user_group_ids);
        $conditions[] = new InequalityCondition(Group :: PROPERTY_DISK_QUOTA, InequalityCondition :: GREATER_THAN, 0);
        $condition = new AndCondition($conditions);

        $group = GroupDataManager :: get_instance()->retrieve_object(Group :: get_table_name(), $condition,
                array(new ObjectTableOrder(Group :: PROPERTY_DISK_QUOTA, SORT_DESC)), Group :: CLASS_NAME);

        return $group instanceof Group ? $group->get_disk_quota() : 0;
    }

    public function get_available_user_disk_quota()
    {
        $quota = $this->get_maximum_user_disk_quota() - $this->get_used_user_disk_quota();
        if ($quota > 0)
        {
            return $quota;
        }
        else
        {
            return 0;
        }
    }

    public function get_user_disk_quota_percentage()
    {
        return 100 * $this->get_used_user_disk_quota() / $this->get_maximum_user_disk_quota();
    }

    /**
     * AGGREGATED USER DISK QUOTA
     */
    public function get_used_aggregated_user_disk_quota()
    {
        if (is_null($this->used_aggregated_user_disk_quota))
        {
            $this->used_aggregated_user_disk_quota = RepositoryDataManager :: get_instance()->get_used_disk_space();
        }
        return $this->used_aggregated_user_disk_quota;
    }

    public function get_maximum_aggregated_user_disk_quota()
    {
        if (is_null($this->maximum_aggregated_user_disk_quota))
        {
            $this->maximum_aggregated_user_disk_quota = \user\DataManager :: get_total_user_disk_quota();
        }
        return $this->maximum_aggregated_user_disk_quota;
    }

    public function get_available_aggregated_user_disk_quota()
    {
        $quota = $this->get_maximum_aggregated_user_disk_quota() - $this->get_used_aggregated_user_disk_quota();
        if ($quota > 0)
        {
            return $quota;
        }
        else
        {
            return 0;
        }
    }

    public function get_aggregated_user_disk_quota_percentage()
    {
        return 100 * $this->get_used_aggregated_user_disk_quota() / $this->get_maximum_aggregated_user_disk_quota();
    }

    /**
     * RESERVED DISK SPACE
     */
    public function get_used_reserved_disk_space()
    {
        return $this->get_maximum_aggregated_user_disk_quota();
    }

    public function get_maximum_reserved_disk_space()
    {
        return disk_total_space(Path :: get(SYS_FILE_PATH));
    }

    public function get_available_reserved_disk_space()
    {
        $quota = $this->get_maximum_reserved_disk_space() - $this->get_used_reserved_disk_space();
        if ($quota > 0)
        {
            return $quota;
        }
        else
        {
            return 0;
        }
    }

    public function get_reserved_disk_space_percentage()
    {
        return 100 * $this->get_used_reserved_disk_space() / $this->get_maximum_reserved_disk_space();
    }

    /**
     * ALLOCATED DISK SPACE
     */
    public function get_used_allocated_disk_space()
    {
        return $this->get_used_aggregated_user_disk_quota();
    }

    public function get_maximum_allocated_disk_space()
    {
        return disk_total_space(Path :: get(SYS_FILE_PATH));
    }

    public function get_available_allocated_disk_space()
    {
        $quota = $this->get_maximum_allocated_disk_space() - $this->get_used_allocated_disk_space();
        if ($quota > 0)
        {
            return $quota;
        }
        else
        {
            return 0;
        }
    }

    public function get_allocated_disk_space_percentage()
    {
        return 100 * $this->get_used_allocated_disk_space() / $this->get_maximum_allocated_disk_space();
    }

    /**
     * DATABASE SPACE
     */
    public function get_used_database_quota()
    {
        if (is_null($this->used_database_quota))
        {
            $condition = new AndCondition(
                    new EqualityCondition(ContentObject :: PROPERTY_OWNER_ID, $this->user->get_id()),
                    new NotCondition(
                            new InCondition(ContentObject :: PROPERTY_TYPE,
                                    RepositoryDataManager :: get_active_helper_types())));
            $this->used_database_quota = RepositoryDataManager :: get_instance()->count_content_objects($condition);
        }
        return $this->used_database_quota;
    }

    public function get_maximum_database_quota()
    {
        if (is_null($this->maximum_database_quota))
        {
            $this->maximum_database_quota = $this->user->get_database_quota();
        }
        return $this->maximum_database_quota;
    }

    public function get_available_database_quota()
    {
        $quota = $this->get_maximum_database_quota() - $this->get_used_database_quota();
        if ($quota > 0)
        {
            return $quota;
        }
        else
        {
            return 0;
        }
    }

    public function get_user_database_percentage()
    {
        return 100 * $this->get_used_database_quota() / $this->get_maximum_database_quota();
    }

    public function upgrade_allowed()
    {
        $quota_step = (int) PlatformSetting :: get('step', __NAMESPACE__);
        $allow_upgrade = (boolean) PlatformSetting :: get('allow_upgrade', __NAMESPACE__);
        $maximum_user_disk_space = (int) PlatformSetting :: get('maximum_user', __NAMESPACE__);
        $policy = PlatformSetting :: get('quota_policy', __NAMESPACE__);

        if (! $this->uses_user_disk_quota())
        {
            return false;
        }

        if (\repository\quota\rights\Rights :: get_instance()->quota_is_allowed() && $this->get_available_allocated_disk_space() > $quota_step)
        {
            return true;
        }

        if ($allow_upgrade)
        {
            if ($maximum_user_disk_space == 0)
            {
                if ($this->get_available_allocated_disk_space() > $quota_step)
                {
                    return true;
                }
            }
            else
            {
                if ($this->user->get_disk_quota() < $maximum_user_disk_space)
                {
                    if ($this->get_available_allocated_disk_space() > $quota_step)
                    {
                        return true;
                    }
                }
            }
        }

        return false;
    }

    public function request_allowed()
    {
        $quota_step = (int) PlatformSetting :: get('step', __NAMESPACE__);
        $allow_request = PlatformSetting :: get('allow_request', __NAMESPACE__);
        $policy = PlatformSetting :: get('quota_policy', __NAMESPACE__);

        if (! $this->uses_user_disk_quota())
        {
            return false;
        }

        if (\repository\quota\rights\Rights :: get_instance()->quota_is_allowed() && $this->get_available_allocated_disk_space() > $quota_step)
        {
            return true;
        }

        if ($allow_request)
        {
            if ($this->get_available_allocated_disk_space() > $quota_step)
            {
                return true;
            }
        }

        return false;
    }

    function uses_user_disk_quota()
    {
        $policy = PlatformSetting :: get('quota_policy', __NAMESPACE__);
        $fallback = PlatformSetting :: get('quota_fallback', __NAMESPACE__);
        $fallback_user = PlatformSetting :: get('quota_fallback_user', __NAMESPACE__);

        switch ($policy)
        {
            case self :: POLICY_USER :
                return ($this->user->get_disk_quota() || ! $fallback ? true : false);
                break;
            case self :: POLICY_GROUP_HIGHEST :
                $group = $this->get_group_highest();
                return $group || ! $fallback ? false : true;
                break;
            case self :: POLICY_GROUP_LOWEST :
                $group = $this->get_group_lowest();
                return $group || ! $fallback ? false : true;
                break;
            case self :: POLICY_HIGHEST :
                $group = $this->get_group_highest();
                return ($group > $this->user->get_disk_quota() ? false : true);
                break;
            case self :: POLICY_LOWEST :
                $group = $this->get_group_lowest();
                return ($group > $this->user->get_disk_quota() || ! $group ? true : false);
                break;
            default :
                return true;
                break;
        }
    }

    /**
     * Build a bar-view of the used quota.
     *
     * @param $percent float The percentage of the bar that is in use
     * @param $status string A status message which will be displayed below the bar.
     * @return string HTML representation of the requested bar.
     */
    static function get_bar($percent, $status)
    {
        $html = array();

        $html[] = '<div class="usage_information">';
        $html[] = '<div class="usage_bar">';

        for($i = 0; $i < 100; $i ++)
        {
            if ($percent > $i)
            {
                if ($i >= 90)
                {
                    $class = 'very_critical';
                }
                elseif ($i >= 80)
                {
                    $class = 'critical';
                }
                else
                {
                    $class = 'used';
                }
            }
            else
            {
                $class = '';
            }
            $html[] = '<div class="' . $class . '"></div>';
        }

        $html[] = '</div>';
        $html[] = '<div class="usage_status">' . $status . ' &ndash; ' . round($percent, 2) . ' %</div>';
        $html[] = '</div>';

        return implode("\n", $html);
    }

    function reset_cache()
    {
        $cache_file = Path :: get_cache_path(UserManager :: context()) . 'total_user_disk_quota';
        Filesystem :: remove($cache_file);
    }
}
?>