<?php
namespace repository\quota;

use common\libraries\Theme;
use common\libraries\ToolbarItem;
use common\libraries\ActionBarRenderer;
use common\libraries\Mail;
use common\libraries\Filesystem;
use common\libraries\PlatformSetting;
use common\libraries\Utilities;
use common\libraries\NotAllowedException;
use common\libraries\Translation;
use common\libraries\NoObjectSelectedException;

class DenierComponent extends Manager
{

    function run()
    {
        if (! \repository\quota\rights\Rights :: get_instance()->quota_is_allowed())
        {
            throw new NotAllowedException();
        }

        $ids = \common\libraries\Request :: get(self :: PARAM_REQUEST_ID);

        if (! empty($ids))
        {
            if (! is_array($ids))
            {
                $failures = $this->single_deny($ids);
                $ids = array($ids);
            }
            else
            {
                $failures = $this->multiple_denies($ids);
            }

            if ($failures)
            {
                if (count($ids) == 1)
                {
                    $message = 'ObjectNotDenied';
                    $parameter = array('OBJECT' => Translation :: get('Request'));
                }
                elseif (count($ids) > $failures)
                {
                    $message = 'SomeObjectsNotDenied';
                    $parameter = array('OBJECTS' => Translation :: get('Requests'));
                }
                else
                {
                    $message = 'ObjectsNotDenied';
                    $parameter = array('OBJECTS' => Translation :: get('Requests'));
                }
            }
            else
            {
                if (count($ids) == 1)
                {
                    $message = 'ObjectDenied';
                    $parameter = array('OBJECT' => Translation :: get('Request'));
                }
                else
                {
                    $message = 'ObjectsDenied';
                    $parameter = array('OBJECTS' => Translation :: get('Requests'));
                }
            }

            $this->redirect(Translation :: get($message, $parameter, Utilities :: COMMON_LIBRARIES),
                    ($failures ? true : false), array(self :: PARAM_ACTION => self :: ACTION_BROWSE));
        }
        else
        {
            $this->display_error_page(
                    htmlentities(
                            Translation :: get('NoObjectSelected', array('OBJECT' => Translation :: get('Request')),
                                    Utilities :: COMMON_LIBRARIES)));
        }
    }

    function single_deny($id)
    {
        $request = DataManager :: retrieve(Request :: class_name(), (int) $id);

        if (! \repository\quota\rights\Rights :: get_instance()->is_target_user($this->get_user(), $request->get_user_id()) && ! $this->get_user()->is_platform_admin())
        {
            throw new NotAllowedException();
        }

        $failures = 0;

        $form = new RequestForm($request,
                $this->get_url(
                        array(self :: PARAM_ACTION => self :: ACTION_DENY,
                                self :: PARAM_REQUEST_ID => $request->get_id())));

        if ($form->validate())
        {
            $values = $form->exportValues();

            $request->set_decision(Request :: DECISION_DENIED);
            $request->set_decision_date(time());
            $request->set_decision_motivation($values[Request :: PROPERTY_DECISION_MOTIVATION]);

            if (! $request->update())
            {
                $failures ++;
            }
            else
            {
                $this->send_mail($request);
            }

            return $failures;
        }
        else
        {
            $this->display_header();
            echo $this->get_action_bar()->as_html();
            $form->freeze(array('quota_step', Request :: PROPERTY_QUOTA, Request :: PROPERTY_MOTIVATION));
            $form->display();
            $this->display_footer();
            exit();
        }
    }

    function multiple_denies($ids)
    {
        $failures = 0;

        foreach ($ids as $id)
        {
            $request = DataManager :: retrieve(Request :: class_name(), (int) $id);

            if (! \repository\quota\rights\Rights :: get_instance()->is_target_user($this->get_user(),
                    $request->get_user_id()) && ! $this->get_user()->is_platform_admin())
            {
                $failures ++;
            }
            else
            {
                if (! $request->is_pending())
                {
                    $failures ++;
                }
                else
                {
                    $request->set_decision(Request :: DECISION_DENIED);
                    $request->set_decision_date(time());

                    if (! $request->update())
                    {
                        $failures ++;
                    }
                    else
                    {
                        $this->send_mail($request);
                    }
                }
            }
        }

        return $failures;
    }

    function send_mail(Request $request)
    {
        set_time_limit(3600);

        $recipient = $request->get_user();

        $title = Translation :: get('RequestDeniedMailTitle',
                array('PLATFORM' => PlatformSetting :: get('site_name'),
                        'QUOTA' => Filesystem :: format_file_size($request->get_quota())));

        if (strlen($request->get_decision_motivation()) > 0)
        {
            $variable = 'RequestDeniedMailBody';
        }
        else
        {
            $variable = 'RequestDeniedMailBodySimple';
        }

        $body = Translation :: get($variable,
                array('USER' => $recipient->get_fullname(), 'PLATFORM' => PlatformSetting :: get('site_name'),
                        'QUOTA' => Filesystem :: format_file_size($request->get_quota()),
                        'MOTIVATION' => $request->get_decision_motivation()));

        $mail = Mail :: factory($title, $body,
                array(Mail :: NAME => $recipient->get_fullname(), Mail :: EMAIL => $recipient->get_email()),
                array(
                        Mail :: NAME => PlatformSetting :: get('administrator_firstname') . '_' . PlatformSetting :: get(
                                'administrator_surname'), Mail :: EMAIL => PlatformSetting :: get('administrator_email')));

        $mail->send();
    }

    function get_action_bar()
    {
        $calculator = new Calculator($this->get_user());
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);

        $allow_upgrade = PlatformSetting :: get('allow_upgrade', __NAMESPACE__);
        $maximum_user_disk_space = PlatformSetting :: get('maximum_user', __NAMESPACE__);

        if ($calculator->upgrade_allowed())
        {
            $action_bar->add_common_action(
                    new ToolbarItem(Translation :: get('UpgradeQuota'), Theme :: get_image_path() . 'action/upgrade.png',
                            $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_UPGRADE))));
        }

        if ($calculator->request_allowed())
        {
            $action_bar->add_common_action(
                    new ToolbarItem(Translation :: get('RequestUpgrade'),
                            Theme :: get_image_path() . 'action/request.png',
                            $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_CREATE))));
        }

        $action_bar->add_tool_action(
                new ToolbarItem(Translation :: get('BackToOverview'), Theme :: get_image_path() . 'action/browser.png',
                        $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_BROWSE))));

        return $action_bar;
    }
}
?>