<?php
namespace rights;

use common\libraries\Theme;
use rights\RightsDataManager;
use user\UserDataManager;
use common\libraries\EqualityCondition;
use user\UserRightsTemplate;
use common\libraries\InCondition;
use group\GroupRightsTemplate;
use group\GroupDataManager;
use common\libraries\Translation;
use common\libraries\Utilities;

/**
 * Class that describes the templates for the rights editor
 *
 * @author Sven Vanpoucke
 */
class TemplateEntity implements RightsEntity
{
    const ENTITY_NAME = 'template';
    const ENTITY_TYPE = 3;

    private $template_cache;

    private static $instance;

    static function get_instance()
    {
        if (! isset(self :: $instance))
        {
            self :: $instance = new self();
        }
        return self :: $instance;
    }

    /**
     * Retrieves the items for this entity
     *
     * @param $condition Condition
     * @param $offset int
     * @param $count int
     * @param $order_property Array
     * @return ObjectResultSet
     */
    function retrieve_entity_items($condition = null, $offset = null, $count = null, $order_property = null)
    {
        $condition = $this->get_condition($condition);
        return RightsDataManager :: get_instance()->retrieve_rights_templates($condition, $offset, $max_objects,
                $order_by);
    }

    /**
     * Retrieves the entity item ids relevant for a given user
     *
     * @param $user_id integer
     * @return array
     */
    function retrieve_entity_item_ids_linked_to_user($user_id)
    {
        if (is_null($this->template_cache[$user_id]))
        {
            $this->template_cache[$user_id] = array();
            // templates linked to platform groups

            $subscribed_platform_group_ids = PlatformGroupEntity :: get_instance()->retrieve_entity_item_ids_linked_to_user(
                    $user_id);

            $gdm = GroupDataManager :: get_instance();
            $conditions = new InCondition(GroupRightsTemplate :: PROPERTY_GROUP_ID, $subscribed_platform_group_ids);
            $group_templates = $gdm->retrieve_group_rights_templates($condition);

            while ($template = $group_templates->next_result())
            {
                $this->template_cache[$user_id][] = $template->get_rights_template_id();
            }

            // templates linked to the user
            $udm = UserDataManager :: get_instance();
            $condition = new EqualityCondition(UserRightsTemplate :: PROPERTY_USER_ID, $user_id);
            $user_templates = $udm->retrieve_user_rights_templates($condition);

            while ($template = $user_templates->next_result())
            {
                $this->template_cache[$user_id][] = $template->get_rights_template_id();
            }
        }
        return array_unique($this->template_cache[$user_id]);
    }

    /**
     * Counts the items for this entity
     *
     * @param $condition Condition
     * @return int
     */
    function count_entity_items($condition = null)
    {
        $condition = $this->get_condition($condition);
        return RightsDataManager :: get_instance()->count_rights_templates($condition);
    }

    /**
     * Returns the name of this entity
     *
     * @return String
     */
    function get_entity_name()
    {
        return self :: ENTITY_NAME;
    }

    /**
     * Returns the translated name of this entiry for displaying purposes only!
     *
     * @return String Translated name of the entity
     */
    function get_entity_translated_name()
    {
        return Translation :: get(Utilities :: underscores_to_camelcase(self :: ENTITY_NAME));
    }

    /**
     * Returns the type of this entity
     *
     * @return int
     */
    function get_entity_type()
    {
        return self :: ENTITY_TYPE;
    }

    /**
     * Returns the path to the icon of the entity
     *
     * @return String
     */
    function get_entity_icon()
    {
        return Theme :: get_image_path('common\extensions\rights_editor_manager') . 'place_template.png';
    }

    /**
     * Returns the cell renderer of this entity
     *
     * @param $browser Application
     * @return null
     */
    function get_entity_cell_renderer($browser)
    {
        return null;
    }

    /**
     * Returns the column model of this entity
     *
     * @param $browser Application
     * @return LocationTemplateBrowserTableColumnModel
     */
    function get_entity_column_model($browser)
    {
        return new LocationTemplateBrowserTableColumnModel($browser);
    }

    /**
     * Returns the properties on which can be searched
     *
     * @return Array
     */
    function get_search_properties()
    {
        return array(RightsTemplate :: PROPERTY_NAME);
    }

    /**
     * Function that can be filled in extensions of this class to limit the template
     *
     * @return Condition
     */
    public function get_condition($condition)
    {
        return $condition;
    }

    /**
     * Retrieves the type for the advanced element finder for the simple rights editor
     */
    function get_element_finder_type()
    {
        return null;
    }

    /**
     * Retrieves an element for the advanced element finder for the simple rights editor with the given id
     */
    function get_element_finder_element($id)
    {
        return null;
    }

    function get_users()
    {
        return UserDataManager :: get_instance()->retrieve_users_from_template_by_template_id($this->get_id());
    }

}

?>
