<?php
namespace rights;

use common\libraries\Utilities;
use common\libraries\DataClass;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\InequalityCondition;
use common\libraries\NotCondition;
use common\libraries\ObjectTableOrder;
use common\libraries\NestedTreeNode;

/**
 * $Id: location.class.php 214 2009-11-13 13:57:37Z vanpouckesven $
 * @package rights.lib
 * @author Hans de Bisschop
 */

class RightsLocation extends NestedTreeNode
{
    //Keep track of the context so we know which table to call
    private $context;
    
    const CLASS_NAME = __CLASS__;
    
    const PROPERTY_TREE_IDENTIFIER = 'tree_identifier';
    const PROPERTY_TREE_TYPE = 'tree_type';
    const PROPERTY_TYPE = 'type';
    const PROPERTY_IDENTIFIER = 'identifier';
    const PROPERTY_INHERIT = 'inherit';
    const PROPERTY_LOCKED = 'locked';

    /**
     * Get the default properties of all users.
     * @return array The property names.
     */
    static function get_default_property_names()
    {
        return parent :: get_default_property_names(array(self :: PROPERTY_TYPE, self :: PROPERTY_IDENTIFIER, 
                self :: PROPERTY_TREE_IDENTIFIER, self :: PROPERTY_TREE_TYPE, self :: PROPERTY_INHERIT, 
                self :: PROPERTY_LOCKED));
    }

    function get_context()
    {
        return $this->context;
    }

    function set_context($context)
    {
        $this->context = $context;
    }

    /**
     * inherited
     */
    function get_data_manager()
    {
        return RightsDataManager :: get_instance();
    }

    function get_type()
    {
        return $this->get_default_property(self :: PROPERTY_TYPE);
    }

    function set_type($type)
    {
        $this->set_default_property(self :: PROPERTY_TYPE, $type);
    }

    function set_type_from_object($object)
    {
        $this->set_type(Utilities :: get_classname_from_object($object, true));
    }

    function get_identifier()
    {
        return $this->get_default_property(self :: PROPERTY_IDENTIFIER);
    }

    function set_identifier($identifier)
    {
        $this->set_default_property(self :: PROPERTY_IDENTIFIER, $identifier);
    }

    function get_inherit()
    {
        return $this->get_default_property(self :: PROPERTY_INHERIT);
    }

    function set_inherit($inherit)
    {
        $this->set_default_property(self :: PROPERTY_INHERIT, $inherit);
    }

    function get_tree_identifier()
    {
        return $this->get_default_property(self :: PROPERTY_TREE_IDENTIFIER);
    }

    function set_tree_identifier($tree_identifier)
    {
        $this->set_default_property(self :: PROPERTY_TREE_IDENTIFIER, $tree_identifier);
    }

    function get_tree_type()
    {
        return $this->get_default_property(self :: PROPERTY_TREE_TYPE);
    }

    function set_tree_type($tree_type)
    {
        $this->set_default_property(self :: PROPERTY_TREE_TYPE, $tree_type);
    }

    function inherits()
    {
        return $this->get_inherit();
    }

    function switch_inherit()
    {
        if ($this->inherits())
        {
            $this->set_inherit(false);
        }
        else
        {
            $this->set_inherit(true);
        }
    }

    function inherit()
    {
        $this->set_inherit(true);
    }

    function disinherit()
    {
        $this->set_inherit(false);
    }

    function get_locked()
    {
        return $this->get_default_property(self :: PROPERTY_LOCKED);
    }

    function set_locked($locked)
    {
        $this->set_default_property(self :: PROPERTY_LOCKED, $locked);
    }

    function is_locked()
    {
        return $this->get_locked();
    }

    function lock()
    {
        $this->set_locked(true);
    }

    function unlock()
    {
        $this->set_locked(false);
    }

    function switch_lock()
    {
        if ($this->is_locked())
        {
            $this->unlock();
        }
        else
        {
            $this->lock();
        }
    }

    function is_root()
    {
        $parent = $this->get_parent();
        return ($parent == 0);
    }

    function get_locked_parent()
    {
        $rdm = RightsDataManager :: get_instance();
        
        $locked_parent_conditions = array();
        
        $locked_parent_conditions[] = new EqualityCondition(RightsLocation :: PROPERTY_TREE_TYPE, $this->get_tree_type());
        $locked_parent_conditions[] = new EqualityCondition(RightsLocation :: PROPERTY_TREE_IDENTIFIER, $this->get_tree_identifier());
        $locked_parent_conditions[] = new EqualityCondition(RightsLocation :: PROPERTY_LOCKED, true);
        $locked_parent_conditions[] = new InequalityCondition(RightsLocation :: PROPERTY_LEFT_VALUE, InequalityCondition :: LESS_THAN, $this->get_left_value());
        $locked_parent_conditions[] = new InequalityCondition(RightsLocation :: PROPERTY_RIGHT_VALUE, InequalityCondition :: GREATER_THAN, $this->get_right_value());
        
        $locked_parent_condition = new AndCondition($locked_parent_conditions);
        //$order[] = new ObjectTableOrder(Location :: PROPERTY_LEFT_VALUE);
        

        return $rdm->retrieve_locked_parent($this->get_context(), $locked_parent_condition);
    
    }

    static function get_table_name()
    {
        return Utilities :: get_classname_from_namespace(self :: CLASS_NAME, true);
    }

    function get_nested_tree_node_condition()
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(self :: PROPERTY_TREE_TYPE, $this->get_tree_type());
        $conditions[] = new EqualityCondition(self :: PROPERTY_TREE_IDENTIFIER, $this->get_tree_identifier());
        
        return new AndCondition($conditions);
    }

    /**
     * Removes the rights set on this location
     */
    function clear_rights()
    {
        $rdm = RightsDataManager :: get_instance();
        return $rdm->delete_rights_location_entity_rights($this);
    }

    function clear_right($right_id)
    {
        $rdm = RightsDataManager :: get_instance();
        return $rdm->delete_rights_location_entity_rights($this, null, null, $right_id);
    }

    /**
     * Retrieves the rights entities linked to this location
     *
     * @param int $right_id - [OPTIONAL] default null
     *
     * @return ResultSet
     */
    function get_rights_entities($right_id = null)
    {
        $rdm = RightsDataManager :: get_instance();

        $conditions = array();

        $conditions[] = new EqualityCondition(RightsLocationEntityRight :: PROPERTY_LOCATION_ID, $this->get_id());

        if(!is_null($right_id))
        {
            $conditions[] = new EqualityCondition(RightsLocationEntityRight :: PROPERTY_RIGHT_ID, $right_id);
        }

	$condition = new AndCondition($conditions);

        $entity_rights = $rdm->retrieve_rights_location_rights($this->get_context(), $condition);
        if ($entity_rights)
        {
            return $entity_rights->as_array();
        }
        else
        {
            throw new \Exception(Translation :: get('InvalidDataRetrievedFromDatabase'));
        }
    
    }

    function delete()
    {
        $this->clear_rights();
        return parent :: delete();
    }
}
?>
