<?php
namespace user;

use common\libraries\Filesystem;
use repository\quota\Calculator;
use admin\AdminDataManager;
use common\libraries\CommonDataManager;
use common\libraries\Registration;
use common\libraries\DataClassCountParameters;
use common\libraries\DataClassCountCache;
use common\libraries\DataClassResultSetCache;
use common\libraries\UserRegistrationSupport;
use common\libraries\Utilities;
use common\libraries\Path;
use common\libraries\Authentication;
use common\libraries\Translation;
use common\libraries\Configuration;
use common\libraries\EqualityCondition;
use common\libraries\PlatformSetting;
use common\libraries\DataClassRetrievesParameters;
use common\libraries\OrCondition;
use common\libraries\AndCondition;
use common\libraries\InCondition;
use common\libraries\DataClassCache;
use Exception;

/**
 *
 * @author Hans De Bisschop
 * @author Sven Vanpoucke
 * @package user.lib This is a skeleton for a data manager for the User application.
 */
class DataManager extends \common\libraries\DataManager
{

    /**
     * Gets the type of DataManager to be instantiated
     *
     * @return string
     */
    static function get_type()
    {
        return 'doctrine';
    }

    /**
     * Magic method to handle calls to non-existing methods and providing *some* backward compatibility
     *
     * @param $name string
     * @param $arguments multitype:mixed
     * @throws Exception
     * @return boolean
     */
    function __callStatic($name, $arguments)
    {
        switch ($name)
        {
            case 'retrieve_users' :
                return static :: retrieves(User :: class_name(), $arguments[0], $arguments[1], $arguments[2],
                       $arguments[3]);
                break;
            case 'count_users' :
                return static :: count(User :: class_name(), $arguments[0], $arguments[1], $arguments[2], $arguments[3]);
                break;
            case 'retrieve_user_settings' :
                return static :: retrieves(UserSetting :: class_name(),
                        new DataClassRetrievesParameters($arguments[0], $arguments[1], $arguments[2], $arguments[3]));
                break;
            case 'retrieve_user_setting' :
                $conditions = array();
                $conditions[] = new EqualityCondition(UserSetting :: PROPERTY_SETTING_ID, $arguments[0]);
                $conditions[] = new EqualityCondition(UserSetting :: PROPERTY_USER_ID, $arguments[1]);
                $condition = new AndCondition($conditions);
                return static :: retrieve(UserSetting :: class_name(), $condition);
                break;
            case 'delete_user_settings' :
                return static :: deletes(UserSetting :: class_name(), $arguments[0]);
                break;
            case 'delete_user_rights_templates' :
                return static :: deletes(UserRightsTemplate :: class_name(), $arguments[0]);
                break;
            case 'retrieve_user_rights_templates' :
                return static :: retrieves(UserRightsTemplate :: class_name(), $arguments[0], $arguments[1],
                        $arguments[2], $arguments[3]);
                break;
            case 'retrieve_buddy_list_items' :
                return static :: retrieves(BuddyListItem :: class_name(), $arguments[0], $arguments[1], $arguments[2],
                        $arguments[3]);
                break;
            case 'retrieve_buddy_list_categories' :
                return static :: retrieves(BuddyListCategory :: class_name(), $arguments[0], $arguments[1],
                        $arguments[2], $arguments[3]);
                break;
            case 'retrieve_chat_messages' :
                return static :: retrieves(ChatMessage :: class_name(), $arguments[0], $arguments[1], $arguments[2],
                        $arguments[3]);
                break;
            // case '' :
            // return static :: method();
            // break;
        }

        $name_parts = explode('_', $name, 2);
        $class = static :: context() . '\\' . Utilities :: underscores_to_camelcase($name_parts[1]);

        switch ($name_parts[0])
        {
            case 'retrieve' :
                return static :: retrieve_by_id($class, (int) $arguments[0]);
                break;
            default :
                throw new Exception(
                        'Your call to the method <i>' . $name . '</i> in <i>' . $class .
                                 'DataManager</i> could not be resolved.');
        }
    }

    /**
     * Logs a user in to the platform
     *
     * @param $username string
     * @param $password string
     */
    static function login($username, $password = null)
    {
        // Trim the username to make sure users are not created twice
        $username = trim($username);

        // If username is available, try to login
        if (! self :: is_username_available($username))
        {
            $condition = new EqualityCondition(User :: PROPERTY_USERNAME, $username);
            $user = self :: retrieve(User :: class_name(), $condition);
            $authentication_method = $user->get_auth_source();
            $authentication = Authentication :: factory($authentication_method);
            $succes = $authentication->check_login($user, $username, $password);

            if ($succes)
            {
                return $user;
            }

            return $authentication->get_message();
        }
        // If username is not available, check if an authentication method is able to register
        // the user in the platform
        else
        {
            $errmsg = Translation :: get('UsernameNotAvailable'); // default error message
            $authentication_dir = dir(Path :: get_library_path() . 'shared/authentication/');
            while (false !== ($authentication_method = $authentication_dir->read()))
            {
                if (strpos($authentication_method, '.') === false && is_dir(
                        $authentication_dir->path . '/' . $authentication_method) && PlatformSetting :: get(
                        'enable_' . $authentication_method . '_authentication'))
                {
                    $authentication_class_file = $authentication_dir->path . $authentication_method . '/' .
                             $authentication_method . '_authentication.class.php';
                    $authentication_class = 'common\libraries\\' . ucfirst($authentication_method) . 'Authentication';
                    $authentication = new $authentication_class();
                    if ($authentication instanceof UserRegistrationSupport)
                    {
                        if ($authentication->register_new_user($username, $password))
                        {
                            $authentication_dir->close();
                            return self :: get_instance()->retrieve_user_by_username($username);
                        }
                        else
                        {
                            // get authentication specific error message
                            $errmsg = $authentication->get_message();
                        }
                    }
                }
            }
            $authentication_dir->close();
            return $errmsg;
        }
    }

    /**
     * Logs the user out of the system
     */
    static function logout()
    {
        $user = self :: retrieve(User :: class_name(), intval(\common\libraries\Session :: get_user_id()));
        $authentication = Authentication :: factory($user->get_auth_source());
        if ($authentication->logout($user))
        {
            return true;
        }
        return false;
    }

    /**
     * Checks whether the user is allowed to be deleted Unfinished.
     */
    static function user_deletion_allowed($user)
    {
        // //A check to not delete a user when he's an active teacher
        // // Weblcms is an optional application, so first check if the app is installed:
        // $weblcms_registration = CommonDataManager :: get_registration(WeblcmsManager :: APPLICATION_NAME);
        // if ($weblcms_registration)
        // {
        // // if active
        // if ($weblcms_registration->get_status() == Registration :: STATUS_ACTIVE)
        // {
        // $courses = WeblcmsDataManager :: get_instance()->count_courses(new EqualityCondition(Course ::
        // PROPERTY_TITULAR, $user->get_id()));
        // if ($courses > 0)
        // {
        return false;
        // }
        // }
        // }
        // return true;
    }

    static function official_code_exists($official_code)
    {
        $condition = new EqualityCondition(User :: PROPERTY_OFFICIAL_CODE, $official_code);
        return (self :: count(User :: class_name(), $condition) > 0);
    }

    /**
     * Retrieve the users who are currently active
     *
     * @param $condition \common\libraries\Condition
     * @param $count int
     * @param $offset int
     * @param $order_by multitype:\common\libraries\ObjectTableOrder
     * @return \common\libraries\DataClassResultSet
     */
    static function retrieve_active_users($condition = null, $count = null, $offset = null, $order_by = array())
    {
        $conditions = array();
        if ($condition)
        {
            $conditions[] = $condition;
        }

        $conditions[] = new EqualityCondition(User :: PROPERTY_ACTIVE, 1);
        $condition = new AndCondition($conditions);

        $parameters = new DataClassRetrievesParameters($condition, $count, $offset, $order_by);

        return self :: retrieves(User :: class_name(), $parameters);
    }

    /**
     * Retrieve the users who are currently approved
     *
     * @param $condition \common\libraries\Condition
     * @param $count int
     * @param $offset int
     * @param $order_by multitype:\common\libraries\ObjectTableOrder
     * @return \common\libraries\DataClassResultSet
     */
    static function retrieve_approved_users($condition = null, $count = null, $offset = null, $order_by = array())
    {
        $conditions = array();
        if ($condition)
        {
            $conditions[] = $condition;
        }

        $conditions[] = new EqualityCondition(User :: PROPERTY_APPROVED, 1);
        $condition = new AndCondition($conditions);

        $parameters = new DataClassRetrievesParameters($condition, $count, $offset, $order_by);

        return self :: retrieves(User :: class_name(), $parameters);
    }

    /**
     * Retrieve the users who are currently not approved
     *
     * @param $condition \common\libraries\Condition
     * @param $count int
     * @param $offset int
     * @param $order_by multitype:\common\libraries\ObjectTableOrder
     * @return \common\libraries\DataClassResultSet
     */
    static function retrieve_approval_users($condition = null, $count = null, $offset = null, $order_by = array())
    {
        $conditions = array();
        if ($condition)
        {
            $conditions[] = $condition;
        }

        $conditions[] = new EqualityCondition(User :: PROPERTY_APPROVED, 0);
        $condition = new AndCondition($conditions);

        $parameters = new DataClassRetrievesParameters($condition, $count, $offset, $order_by);

        return self :: retrieves(User :: class_name(), $parameters);
    }

    /**
     * Retrieve the (first) user whose status is "anonymous"
     *
     * @return user\User boolean
     */
    static function retrieve_anonymous_user()
    {
        $condition = new EqualityCondition(User :: PROPERTY_STATUS, User :: STATUS_ANONYMOUS);
        return self :: retrieve(User :: CLASS_NAME, $condition);
    }

    /**
     * Return the full name for a specific user, or a translation if provided or if the user does not exist
     *
     * @param $id int
     * @param $unknown_user_translation string
     * @return string
     */
    static function get_fullname_from_user($id, $unknown_user_translation = null)
    {
        $user = self :: retrieve_by_id(User :: CLASS_NAME, $id);
        if ($user instanceof User)
        {
            return $user->get_fullname();
        }

        if ($unknown_user_translation)
        {
            return $unknown_user_translation;
        }

        return Translation :: get('UnknownUser');
    }

    /**
     * Returns the User currently registered in the session
     *
     * @return user\User
     */
    static function get_current_user()
    {
        return self :: retrieve_by_id(User :: CLASS_NAME, \common\libraries\Session :: get_user_id());
    }

    /**
     * Retrieve a User based on the official code property
     *
     * @param $official_code string
     * @return user\User
     */
    static function retrieve_user_by_official_code($official_code)
    {
        $condition = new EqualityCondition(User :: PROPERTY_OFFICIAL_CODE, $official_code);
        return self :: retrieve(User :: CLASS_NAME, $condition);
    }

    /**
     * Retrieve a User based on the username property
     *
     * @param $username string
     * @return user\User
     */
    static function retrieve_user_by_username($username)
    {
        $condition = new EqualityCondition(User :: PROPERTY_USERNAME, $username);
        return self :: retrieve(User :: CLASS_NAME, $condition);
    }

    /**
     * Retrieve a User based on the external user id property
     *
     * @param $external_uid string
     * @return user\User
     */
    static function retrieve_user_by_external_uid($external_uid)
    {
        $condition = new EqualityCondition(User :: PROPERTY_EXTERNAL_UID, $external_uid);
        return self :: retrieve(User :: CLASS_NAME, $condition);
    }

    /**
     * Retrieve a User based on the security token
     *
     * @param $security_token string
     * @return user\User
     */
    static function retrieve_user_by_security_token($security_token)
    {
        $condition = new EqualityCondition(User :: PROPERTY_SECURITY_TOKEN, $security_token);
        return self :: retrieve(User :: CLASS_NAME, $condition);
    }

    /**
     * Retrieve a user-id based on the official code property
     *
     * @param $official_code string
     * @return int
     */
    static function retrieve_user_id_by_official_code($official_code)
    {
        $user = self :: retrieve_user_by_official_code($official_code);
        return ($user instanceof User ? $user->get_id() : null);
    }

    /**
     * Retrieve a DataClassResultSet of Users based on a set of official codes
     *
     * @param $official_codes multitype:string
     * @return \common\libraries\DataClassResultSet
     */
    static function retrieve_users_by_official_codes($official_codes)
    {
        $condition = new InCondition(User :: PROPERTY_OFFICIAL_CODE, $official_codes);
        return self :: retrieves(User :: class_name(), $condition);
    }

    /**
     * Retrieve a DataClassResultSet of Users based on a set of email addresses
     *
     * @param $email multitype:string
     * @return \common\libraries\DataClassResultSet
     * @deprecated Should no longer return an array, calls should be changed to use a while-loop now instead of a for
     * each-loop
     */
    static function retrieve_users_by_email($email)
    {
        $condition = new EqualityCondition(User :: PROPERTY_EMAIL, $email);
        return self :: retrieves(User :: class_name(), $condition)->as_array();
    }

    /**
     * Is the username still available in the storage layer or not?
     *
     * @param $username string
     * @param $user_id int
     * @return boolean
     */
    static function is_username_available($username, $user_id = null)
    {
        $condition = new EqualityCondition(User :: PROPERTY_USERNAME, $username);

        if ($user_id)
        {
            $conditions = array();
            $conditions[] = new EqualityCondition(User :: PROPERTY_USERNAME, $username);
            $conditions = new EqualityCondition(User :: PROPERTY_ID, $user_id);
            $condition = new AndCondition($conditions);
        }

        return ! (self :: count(User :: class_name(), $condition) == 1);
    }

    /**
     * Retrieve a User based on the username property
     *
     * @param $username string
     * @return user\User
     * @deprecated Use UserDataManager :: retrieve_user_by_username() now
     */
    static function retrieve_user_info($username)
    {
        return self :: retrieve_user_by_username($username);
    }

    /**
     * Attempt to retrieve a user based on his full name (first name + last name)
     *
     * @param $fullname string
     * @return user\User
     */
    static function retrieve_user_by_fullname($fullname)
    {
        $name = explode(' ', $fullname);
        $firstname = $name[0];
        $lastname = $name[1];

        $conditions = array();
        $conditions1 = array();
        $conditions2 = array();

        $conditions1[] = new EqualityCondition(User :: PROPERTY_FIRSTNAME, $firstname);
        $conditions1[] = new EqualityCondition(User :: PROPERTY_LASTNAME, $lastname);
        $conditions[] = new AndCondition($conditions1);

        $conditions2[] = new EqualityCondition(User :: PROPERTY_FIRSTNAME, $lastname);
        $conditions2[] = new EqualityCondition(User :: PROPERTY_LASTNAME, $firstname);
        $conditions[] = new AndCondition($conditions2);

        $condition = new OrCondition($conditions);
        return self :: retrieve(User :: class_name(), $condition);
    }

    /**
     * Count the users who are currently active
     *
     * @param $condition \common\libraries\Condition
     * @return \common\libraries\DataClassResultSet
     */
    static function count_active_users($condition = null)
    {
        $conditions = array();
        if ($condition)
        {
            $conditions[] = $condition;
        }

        $conditions[] = new EqualityCondition(User :: PROPERTY_ACTIVE, 1);
        $condition = new AndCondition($conditions);

        return self :: count(User :: class_name(), $condition);
    }

    /**
     * Count the users who are currently approved
     *
     * @param $condition \common\libraries\Condition
     * @return \common\libraries\DataClassResultSet
     */
    static function count_approved_users($condition = null)
    {
        $conditions = array();
        if ($condition)
        {
            $conditions[] = $condition;
        }

        $conditions[] = new EqualityCondition(User :: PROPERTY_APPROVED, 1);
        $condition = new AndCondition($conditions);

        return self :: count(User :: class_name(), $condition);
    }

    /**
     * Count the users who are currently not approved
     *
     * @param $condition \common\libraries\Condition
     * @return \common\libraries\DataClassResultSet
     */
    static function count_approval_users($condition = null)
    {
        $conditions = array();
        if ($condition)
        {
            $conditions[] = $condition;
        }

        $conditions[] = new EqualityCondition(User :: PROPERTY_APPROVED, 0);
        $condition = new AndCondition($conditions);

        return self :: count(User :: class_name(), $condition);
    }

    static function delete_all_users()
    {
        while ($user = self :: retrieves(User :: class_name())->next_result())
        {
            $user->delete();
        }
    }

    static function retrieve_users_from_template_by_template_id($template_id)
    {
        $condition = new EqualityCondition(UserRightsTemplate :: PROPERTY_RIGHTS_TEMPLATE_ID, $template_id);
        return self :: retrieve_users_from_template($condition)->as_array();
    }

    static function retrieve_users_from_template(DataClassRetrievesParameters $parameters)
    {
        if (! $parameters instanceof DataClassRetrievesParameters)
        {
            $parameters = DataClassRetrievesParameters :: generate($parameters);
        }

        return self :: get_instance()->retrieve_users_from_template($parameters);
    }

    static function retrieve_users_by_groups($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        $parameters = new DataClassRetrievesParameters($condition, $max_objects, $offset, $offset);
        return self :: get_instance()->new_retrieve_users_by_groups(User :: class_name(), $parameters);
    }

    static function count_users_by_groups($condition = null)
    {
        $parameters = new DataClassCountParameters($condition);
        return self :: get_instance()->new_count_users_by_groups(User :: class_name(), $parameters);
    }

    static function get_total_user_disk_quota($reset = false)
    {
        $cache_file = Path :: get_cache_path(__NAMESPACE__) . 'total_user_disk_quota';

        if (! file_exists($cache_file) || $reset)
        {
            $policy = PlatformSetting :: get('quota_policy', __NAMESPACE__);
            $fallback = PlatformSetting :: get('quota_fallback', __NAMESPACE__);

            if ($policy == Calculator :: POLICY_USER && ! $fallback)
            {
                $total_quota = self :: get_instance()->get_total_user_disk_quota();
            }
            else
            {
                $users = self :: retrieves(User :: class_name());

                $total_quota = 0;

                while ($user = $users->next_result())
                {
                    $calculator = new Calculator($user);
                    $total_quota += $calculator->get_maximum_user_disk_quota();
                }

                $total_quota;
            }

            Filesystem :: write_to_file($cache_file, serialize($total_quota));
        }
        else
        {
            return unserialize(file_get_contents($cache_file));
        }
    }
}
?>