<?php
namespace user;

use group\GroupRelUser;
use group\GroupDataManager;

use common\libraries\DoctrineDatabase;

/**
 * Doctrine implementation of the user\DataManager
 *
 * @author Hans De Bisschop
 * @package common.libraries
 */
class DoctrineDataManager extends DoctrineDatabase implements UserDataManagerInterface
{

    /**
     * Retrieve users by one or more groups
     *
     * @param $class_name string
     * @param $parameters \common\libraries\DataClassRetrievesParameters
     * @return \common\libraries\Mdb2ResultSet
     */
    function new_retrieve_users_by_groups($class_name, $parameters)
    {
        return $this->retrieve_users_by_groups($parameters->get_condition(), $parameters->get_offset(), $parameters->get_count(), $parameters->get_order_by());
    }

    /**
     * Retrieve users by one or more groups
     *
     * @param $condition \common\libraries\Condition
     * @param $offset int
     * @param $max_objects int
     * @param $order_by multitype:\common\libraries\ObjectTableOrder
     * @return \common\libraries\Mdb2ResultSet
     * @deprecated Use new_retrieve_users_by_groups()
     */
    function retrieve_users_by_groups($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        $group_data_manager = GroupDataManager :: get_instance();
        $group_rel_user_alias = $group_data_manager->get_alias(GroupRelUser :: get_table_name());
        $user_alias = $this->get_alias(User :: get_table_name());

        $query = 'SELECT DISTINCT ' . $user_alias . '.*';
        $query .= ' FROM ' . $this->escape_table_name(User :: get_table_name()) . ' as ' . $user_alias;
        $query .= ' JOIN ' . $group_data_manager->escape_table_name(GroupRelUser :: get_table_name()) . ' as ' . $group_rel_user_alias . ' on ' . $this->escape_column_name(User :: PROPERTY_ID, $user_alias) . ' = ' . $this->escape_column_name(GroupRelUser :: PROPERTY_USER_ID, $group_rel_user_alias);

        return $this->retrieve_object_set($query, User :: get_table_name(), $condition, $offset, $max_objects, $order_by, User :: CLASS_NAME);
    }

    /**
     * Count users by one or more groups
     *
     * @param $class_name string
     * @param $parameters \common\libraries\DataClassCountParameters
     * @return int
     */
    function new_count_users_by_groups($class_name, $parameters)
    {
        return $this->count_users_by_groups($parameters->get_condition());
    }

    /**
     * Count users by one or more groups
     *
     * @param $condition unknown_type
     * @return int
     * @deprecated Use new_count_users_by_groups()
     */
    function count_users_by_groups($condition = null)
    {
        $group_data_manager = GroupDataManager :: get_instance();
        $group_rel_user_alias = $group_data_manager->get_alias(GroupRelUser :: get_table_name());
        $user_alias = $this->get_alias(User :: get_table_name());

        $query = 'SELECT COUNT(DISTINCT ' . $this->escape_table_name(User :: PROPERTY_ID, $user_alias) . ')';
        $query .= ' FROM ' . $this->escape_table_name(User :: get_table_name()) . ' as ' . $user_alias;
        $query .= ' JOIN ' . $group_data_manager->escape_table_name(GroupRelUser :: get_table_name()) . ' as ' . $group_rel_user_alias . ' on ' . $this->escape_column_name(User :: PROPERTY_ID, $user_alias) . ' = ' . $this->escape_column_name(GroupRelUser :: PROPERTY_USER_ID, $group_rel_user_alias);

        return $this->count_result_set($query, User :: get_table_name(), $condition);
    }

    /**
     * Retrieve users in combination with one or more templates
     *
     * @param $parameters \common\libraries\DataClassRetrievesParameters
     * @return \common\libraries\Mdb2ResultSet
     */
    function retrieve_users_from_template(DataClassRetrievesParameters $parameters)
    {
        $user_alias = $this->get_alias(User :: get_table_name());
        $template_alias = $this->get_alias(UserRightsTemplate :: get_table_name());

        $query = 'SELECT ' . $this->escape_column_name('*', $user_alias);
        $query .= ' FROM ' . $this->escape_table_name(User :: get_table_name()) . ' AS ' . $user_alias;
        $query .= ' JOIN ' . $this->escape_table_name(UserRightsTemplate :: get_table_name()) . ' as ' . $template_alias;
        $query .= ' ON ' . $this->escape_column_name(User :: PROPERTY_ID, $user_alias) . '=' . $this->escape_column_name(UserRightsTemplate :: PROPERTY_USER_ID, $template_alias);

        return $this->retrieve_object_set($query, User :: get_table_name(), $parameters->get_condition(), $parameters->get_offset(), $parameters->get_count(), $parameters->get_order_by(), User :: CLASS_NAME);
    }

    function get_total_user_disk_quota()
    {
        $query_builder = $this->get_connection()->createQueryBuilder();
        $query_builder->addSelect('SUM(' . $this->escape_column_name(User :: PROPERTY_DISK_QUOTA) . ') AS disk_quota');
        $query_builder->from($this->escape_table_name(User :: get_table_name()));

        $statement = $this->query($query_builder->getSQL());

        if (! $statement instanceof \PDOException)
        {
            $record = $statement->fetch(\PDO :: FETCH_ASSOC);
        }
        else
        {
            $this->error_handling($statement);
            return false;
        }

        if ($record instanceof \PDOException)
        {
            $this->error_handling($record);
            return false;
        }
        else
        {
            return $record['disk_quota'];
        }
    }
}
?>