<?php
namespace user;

use common\libraries\AndCondition;

use common\libraries\NotCondition;

use group\GroupRelUser;

use repository\RepositoryRights;

use common\libraries\Utilities;
use common\libraries\Path;
use common\libraries\Filesystem;
use common\libraries\Translation;
use common\libraries\DataClass;
use common\libraries\Theme;
use common\libraries\EqualityCondition;
use common\libraries\InCondition;
use common\libraries\PlatformSetting;

use repository\RepositoryDataManager;

use group\GroupDataManager;
use group\Group;
use common\libraries\ImageManipulation;

/**
 * $Id: user.class.php 211 2009-11-13 13:28:39Z vanpouckesven $
 *
 * @package user.lib
 */
/**
 * This class represents a user. User objects have a number of default properties: - user_id: the numeric ID of the
 * user; - lastname: the lastname of the user; - firstname: the firstname of the user; - password: the password for this
 * user; - auth_source: - external_uid: the external authentication system unique id of the user (eg: Shibboleth uid,
 * OpenID uid, ...) - email: the email address of this user; - status: the status of this user: 1 is teacher, 5 is a
 * student; - phone: the phone number of the user; - official_code; the official code of this user; - picture_uri: the
 * URI location of the picture of this user; - creator_id: the user_id of the user who created this user; - language:
 * the language setting of this user; - disk quota: the disk quota for this user; - database_quota: the database quota
 * for this user; - version_quota: the default quota for this user of no quota for a specific learning object type is
 * set.
 *
 * @author Hans de Bisschop
 * @author Dieter De Neef
 */

class User extends DataClass
{
    const CLASS_NAME = __CLASS__;
    const CONTEXT = __NAMESPACE__;

    const PROPERTY_LASTNAME = 'lastname';
    const PROPERTY_FIRSTNAME = 'firstname';
    const PROPERTY_USERNAME = 'username';
    const PROPERTY_PASSWORD = 'password';
    const PROPERTY_AUTH_SOURCE = 'auth_source';
    const PROPERTY_EXTERNAL_UID = 'external_uid';
    const PROPERTY_EMAIL = 'email';
    const PROPERTY_STATUS = 'status';
    const PROPERTY_PLATFORMADMIN = 'admin';
    const PROPERTY_PHONE = 'phone';
    const PROPERTY_OFFICIAL_CODE = 'official_code';
    const PROPERTY_PICTURE_URI = 'picture_uri';
    const PROPERTY_CREATOR_ID = 'creator_id';
    const PROPERTY_DISK_QUOTA = 'disk_quota';
    const PROPERTY_DATABASE_QUOTA = 'database_quota';
    const PROPERTY_ACTIVATION_DATE = 'activation_date';
    const PROPERTY_EXPIRATION_DATE = 'expiration_date';
    const PROPERTY_REGISTRATION_DATE = 'registration_date';
    const PROPERTY_ACTIVE = 'active';
    const PROPERTY_SECURITY_TOKEN = 'security_token';
    const PROPERTY_APPROVED = 'approved';

    const ACTION_CREATE_USER = 'create';

    const NAME_FORMAT_FIRST = 0;
    const NAME_FORMAT_LAST = 1;

    const ANONYMOUS_ID = "1";

    const STATUS_ANONYMOUS = 0;
    const STATUS_TEACHER = 1;
    const STATUS_STUDENT = 5;

    /**
     *
     * @var array
     */
    private $allowed_groups;

    /**
     * Get the default properties of all users.
     *
     * @return array The property names.
     */
    static function get_default_property_names($extended_property_names = array())
    {
        return parent :: get_default_property_names(array(self :: PROPERTY_LASTNAME, self :: PROPERTY_FIRSTNAME,
                self :: PROPERTY_USERNAME, self :: PROPERTY_PASSWORD, self :: PROPERTY_AUTH_SOURCE,
                self :: PROPERTY_EXTERNAL_UID, self :: PROPERTY_EMAIL, self :: PROPERTY_STATUS,
                self :: PROPERTY_PLATFORMADMIN, self :: PROPERTY_PHONE, self :: PROPERTY_OFFICIAL_CODE,
                self :: PROPERTY_PICTURE_URI, self :: PROPERTY_CREATOR_ID, self :: PROPERTY_DISK_QUOTA,
                self :: PROPERTY_DATABASE_QUOTA, self :: PROPERTY_ACTIVATION_DATE,
                self :: PROPERTY_EXPIRATION_DATE, self :: PROPERTY_REGISTRATION_DATE, self :: PROPERTY_ACTIVE,
                self :: PROPERTY_SECURITY_TOKEN, self :: PROPERTY_APPROVED));
    }

    /**
     * inherited
     */
    function get_data_manager()
    {
        return UserDataManager :: get_instance();
    }

    /**
     * Returns the lastname of this user.
     *
     * @return String The lastname
     */
    function get_lastname()
    {
        return $this->get_default_property(self :: PROPERTY_LASTNAME);
    }

    /**
     * Returns the firstname of this user.
     *
     * @return String The firstname
     */
    function get_firstname()
    {
        return $this->get_default_property(self :: PROPERTY_FIRSTNAME);
    }

    /**
     * Returns the fullname of this user
     *
     * @return string The fullname
     */
    function get_fullname()
    {
        $format = PlatformSetting :: get('fullname_format', 'user');

        switch ($format)
        {
            case self :: NAME_FORMAT_FIRST :
                return $this->get_firstname() . ' ' . $this->get_lastname();
                break;
            case self :: NAME_FORMAT_LAST :
                return $this->get_lastname() . ' ' . $this->get_firstname();
                break;
            default :
                return $this->get_firstname() . ' ' . $this->get_lastname();
        }
    }

    /**
     * Returns the username of this user.
     *
     * @return String The username
     */
    function get_username()
    {
        return $this->get_default_property(self :: PROPERTY_USERNAME);
    }

    /**
     * Returns the password of this user.
     *
     * @return String The password
     */
    function get_password()
    {
        return $this->get_default_property(self :: PROPERTY_PASSWORD);
    }

    /**
     * Returns the auth_source for this user.
     *
     * @return String The auth_source
     */
    function get_auth_source()
    {
        return $this->get_default_property(self :: PROPERTY_AUTH_SOURCE);
    }

    /**
     * Returns the external authentication system unique id for this user (useful for instance with : Shibboleth,
     * OpenID, LDAP, . ..)
     *
     * @return String The external unique id
     */
    function get_external_uid()
    {
        return $this->get_default_property(self :: PROPERTY_EXTERNAL_UID);
    }

    /**
     * Returns the email for this user.
     *
     * @return String The email address
     */
    function get_email()
    {
        return $this->get_default_property(self :: PROPERTY_EMAIL);
    }

    /**
     * Returns the status for this user.
     *
     * @return Int The status
     */
    function get_status()
    {
        return $this->get_default_property(self :: PROPERTY_STATUS);
    }

    /**
     * Returns if the user is platformadmin or not.
     *
     * @return Int platformadmin
     */
    function get_platformadmin()
    {
        return $this->get_default_property(self :: PROPERTY_PLATFORMADMIN);
    }

    /**
     * Returns the official code for this user.
     *
     * @return String The official code
     */
    function get_official_code()
    {
        return $this->get_default_property(self :: PROPERTY_OFFICIAL_CODE);
    }

    /**
     * Returns the phone number for this user.
     *
     * @return String The phone number
     */
    function get_phone()
    {
        return $this->get_default_property(self :: PROPERTY_PHONE);
    }

    /**
     * Returns the Picture URI for this user.
     *
     * @return String The URI
     */
    function get_picture_uri()
    {
        return $this->get_default_property(self :: PROPERTY_PICTURE_URI);
    }

    /**
     * Returns the creator ID for this user.
     *
     * @return Int The ID
     */
    function get_creator_id()
    {
        return $this->get_default_property(self :: PROPERTY_CREATOR_ID);
    }

    /**
     * Returns the disk quota for this user.
     *
     * @return Int the disk quota
     */
    function get_disk_quota()
    {
        return $this->get_default_property(self :: PROPERTY_DISK_QUOTA);
    }

    /**
     * Returns the database quota for this user.
     *
     * @return Int the database quota
     */
    function get_database_quota()
    {
        return $this->get_default_property(self :: PROPERTY_DATABASE_QUOTA);
    }

    function get_activation_date()
    {
        return $this->get_default_property(self :: PROPERTY_ACTIVATION_DATE);
    }

    /**
     * Returns the expiration date for this user.
     *
     * @return string the theme
     */
    function get_expiration_date()
    {
        return $this->get_default_property(self :: PROPERTY_EXPIRATION_DATE);
    }

    function get_registration_date()
    {
        return $this->get_default_property(self :: PROPERTY_REGISTRATION_DATE);
    }

    function get_active()
    {
        return $this->get_default_property(self :: PROPERTY_ACTIVE);
    }

    function get_security_token()
    {
        return $this->get_default_property(self :: PROPERTY_SECURITY_TOKEN);
    }

    /**
     * Sets the lastname of this user.
     *
     * @param $lastname String the lastname.
     */
    function set_lastname($lastname)
    {
        $this->set_default_property(self :: PROPERTY_LASTNAME, $lastname);
    }

    /**
     * Sets the firstname of this user.
     *
     * @param $firstname String the firstname.
     */
    function set_firstname($firstname)
    {
        $this->set_default_property(self :: PROPERTY_FIRSTNAME, $firstname);
    }

    /**
     * Sets the username of this user.
     *
     * @param $username String the username.
     */
    function set_username($username)
    {
        $this->set_default_property(self :: PROPERTY_USERNAME, $username);
    }

    /**
     * Sets the password of this user. If Chamilo configuration is set to encrypt the password, this function will also
     * take care of that.
     *
     * @param $password String the password.
     */
    function set_password($password)
    {
        $this->set_default_property(self :: PROPERTY_PASSWORD, $password);
    }

    /**
     * Sets the Auth_source for this user.
     *
     * @param $auth_source String the auth source.
     */
    function set_auth_source($auth_source)
    {
        $this->set_default_property(self :: PROPERTY_AUTH_SOURCE, $auth_source);
    }

    /**
     * Sets the external authentication system unique id for this user (useful for instance with : Shibboleth, OpenID,
     * LDAP, . ..)
     *
     * @param $external_uid String the external unique id
     */
    function set_external_uid($external_uid)
    {
        $this->set_default_property(self :: PROPERTY_EXTERNAL_UID, $external_uid);
    }

    /**
     * Sets the email for this user.
     *
     * @param $email String the email.
     */
    function set_email($email)
    {
        $this->set_default_property(self :: PROPERTY_EMAIL, $email);
    }

    /**
     * Sets the status for this user.
     *
     * @param $status Int the status.
     */
    function set_status($status)
    {
        $this->set_default_property(self :: PROPERTY_STATUS, $status);
    }

    /**
     * Sets the platformadmin property for this user.
     *
     * @param $admin Int the platformadmin status.
     */
    function set_platformadmin($admin)
    {
        $this->set_default_property(self :: PROPERTY_PLATFORMADMIN, $admin);
    }

    /**
     * Sets the official code for this user.
     *
     * @param $official_code String the official code.
     */
    function set_official_code($official_code)
    {
        $this->set_default_property(self :: PROPERTY_OFFICIAL_CODE, $official_code);
    }

    /**
     * Sets the phone number for this user.
     *
     * @param $phone String the phone number
     */
    function set_phone($phone)
    {
        $this->set_default_property(self :: PROPERTY_PHONE, $phone);
    }

    /**
     * Sets the picture uri for this user object
     *
     * @param $picture_uri String the picture URI
     */
    function set_picture_uri($picture_uri)
    {
        $this->set_default_property(self :: PROPERTY_PICTURE_URI, $picture_uri);
    }

    function set_security_token($security_token)
    {
        $this->set_default_property(self :: PROPERTY_SECURITY_TOKEN, $security_token);
    }

    /**
     * Determines if this user has uploaded a picture
     *
     * @return boolean
     */
    function has_picture()
    {
        $uri = $this->get_picture_uri();
        return ((strlen($uri) > 0) && (Utilities :: is_web_uri($uri) || file_exists(Path :: get(SYS_USER_PATH) . $uri)) );
    }

    function get_full_picture_url()
    {
        if ($this->has_picture())
        {
            if(Utilities :: is_web_uri($this->get_picture_uri())){
                //also allow full uri values (if uri set by LDAP parser class)
                return $this->get_picture_uri();
            }else{
                return Path :: get(WEB_USER_PATH) . $this->get_picture_uri();
            }
        }
        else
        {
            return Theme :: get_common_image_path() . 'unknown.jpg';
        }
    }

    function get_full_picture_path()
    {
        if ($this->has_picture())
        {
            return Path :: get(SYS_USER_PATH) . $this->get_picture_uri();
        }
        else
        {
            return Theme :: get_common_image_system_path() . 'unknown.jpg';
        }
    }

    /**
     * Sets the picture file
     *
     * @param array The information of the uploaded file (from the $_FILES- array)
     * @todo Make image resizing configurable
     */
    function set_picture_file($file_info)
    {
        $this->delete_picture();
        $path = Path :: get(SYS_USER_PATH);
        Filesystem :: create_dir($path);
        $img_file = Filesystem :: create_unique_name($path, $this->get_id() . '-' . $file_info['name']);
        move_uploaded_file($file_info['tmp_name'], $path . $img_file);
        $image_manipulation = ImageManipulation :: factory($path . $img_file);
        // Scale image to fit in 400x400 box. Should be configurable somewhere
        $image_manipulation->scale(400, 400);
        $image_manipulation->write_to_file();
        $this->set_picture_uri($img_file);
    }

    /**
     * Removes the picture connected to this user
     */
    function delete_picture()
    {
        if ($this->has_picture())
        {
            $path = Path :: get(SYS_USER_PATH) . $this->get_picture_uri();
            Filesystem :: remove($path);
            $this->set_picture_uri(null);
        }
    }

    /**
     * Sets the creator ID for this user.
     *
     * @param $creator_id String the creator ID.
     */
    function set_creator_id($creator_id)
    {
        $this->set_default_property(self :: PROPERTY_CREATOR_ID, $creator_id);
    }

    /**
     * Sets the disk quota for this user.
     *
     * @param $disk_quota Int The disk quota.
     */
    function set_disk_quota($disk_quota)
    {
        $this->set_default_property(self :: PROPERTY_DISK_QUOTA, $disk_quota);
    }

    /**
     * Sets the database_quota for this user.
     *
     * @param $database_quota Int The database quota.
     */
    function set_database_quota($database_quota)
    {
        $this->set_default_property(self :: PROPERTY_DATABASE_QUOTA, $database_quota);
    }

    function set_activation_date($activation_date)
    {
        $this->set_default_property(self :: PROPERTY_ACTIVATION_DATE, $activation_date);
    }

    /**
     * Sets the default theme for this user.
     *
     * @param $theme string The theme.
     */
    function set_expiration_date($expiration_date)
    {
        $this->set_default_property(self :: PROPERTY_EXPIRATION_DATE, $expiration_date);
    }

    function set_registration_date($registration_date)
    {
        $this->set_default_property(self :: PROPERTY_REGISTRATION_DATE, $registration_date);
    }

    function set_active($active)
    {
        $this->set_default_property(self :: PROPERTY_ACTIVE, $active);
    }

    function set_approved($approved)
    {
        $this->set_default_property(self :: PROPERTY_APPROVED, $approved);
    }

    function get_approved()
    {
        return $this->get_default_property(self :: PROPERTY_APPROVED);
    }

    /**
     * Checks if this user is a platform admin or not
     *
     * @return boolean true if the user is a platforma admin, false otherwise
     */
    function is_platform_admin()
    {
        return ($this->get_platformadmin() == 1 ? true : false);
    }

    /*
     *
     */
    function is_anonymous_user()
    {
        return ($this->get_status() == self :: STATUS_ANONYMOUS ? true : false);
    }

    /**
     * Checks if this user is a teacher or not
     *
     * @return boolean true if the user is a teacher, false otherwise
     */
    function is_teacher()
    {
        return ($this->get_status() == self :: STATUS_TEACHER ? true : false);
    }

    function is_active()
    {
        if ($this->get_active())
        {
            if ($this->get_activation_date() == 0 || time() >= $this->get_activation_date())
            {
                if ($this->get_expiration_date() == 0 || time() <= $this->get_expiration_date())
                {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * Instructs the Datamanager to create this user.
     *
     * @return boolean True if success, false otherwise
     */
    function create()
    {
        $udm = UserDataManager :: get_instance();
        $this->set_registration_date(time());
        $this->set_security_token(sha1(time() . uniqid()));

        if (! parent :: create($this))
        {
            return false;
        }

        if (! RepositoryRights :: get_instance()->create_user_root($this->get_id()))
        {
            return false;
        }

        // if (! UserRights :: create_location_in_users_subtree($this->get_fullname(), $this->get_id(), UserRights ::
        // get_users_subtree_root_id()))
        // {
        // $this->delete();
        // return false;
        // }

        return true;
    }

    function delete()
    {
        $group_data_manager = GroupDataManager :: get_instance();
        $group_rel_user_condition = new EqualityCondition(GroupRelUser :: PROPERTY_USER_ID, $this->get_id());
        $success = $group_data_manager->delete(GroupRelUser :: get_table_name(), $group_rel_user_condition);

        if ($success)
        {
            return parent :: delete();
        }
        else
        {
            return false;
        }
    }

    static function get_table_name()
    {
        return Utilities :: get_classname_from_namespace(self :: CLASS_NAME, true);
    }

    function get_groups($only_retrieve_ids = false)
    {
        $gdm = GroupDataManager :: get_instance();

        $group_ids = $gdm->retrieve_all_subscribed_groups_array($this->get_id(), true);

        // $user_groups = $gdm->retrieve_user_groups($this->get_id());
        // $group_ids = array();
        // while ($user_group = $user_groups->next_result())
        // {
        // $group_ids[] = $user_group->get_group_id();
        // $group = $gdm->retrieve_group($user_group->get_group_id());
        // if (! $group)
        // {
        // continue;
        // }
        //
        // $subgroups = $group->get_parents(false);
        //
        // while ($subgroup = $subgroups->next_result())
        // {
        // $subgroup_id = $subgroup->get_id();
        // if (! in_array($subgroup_id, $group_ids))
        // {
        // $group_ids[] = $subgroup_id;
        // }
        // }
        // }
        //
        if ($only_retrieve_ids)
        {
            return $group_ids;
        }
        //
        // TODO: refactor part below (=> retrieve_all_subscribed_groups_array($this->get_id(), $only_retrieve_ids);
        if (count($group_ids) > 0)
        {
            $condition = new InCondition(Group :: PROPERTY_ID, $group_ids);
            return $gdm->retrieve_groups($condition);
        }
        else
        {
            return null;
        }
    }

    function get_user_groups()
    {
        $gdm = GroupDataManager :: get_instance();
        return $gdm->retrieve_user_groups($this->get_id());
    }

    function get_rights_templates()
    {
        $udm = UserDataManager :: get_instance();
        $condition = new EqualityCondition(UserRightsTemplate :: PROPERTY_USER_ID, $this->get_id());

        return $udm->retrieve_user_rights_templates($condition);
    }

    function add_rights_template_link($rights_template_id)
    {
        $template = new UserRightsTemplate();
        $template->set_user_id($this->get_id());
        $template->set_rights_template_id($rights_template_id);
        return $template->create();
    }

    function delete_rights_template_link($rights_template_id)
    {
        $conditions = array();
        $conditions = new EqualityCondition(UserRightsTemplate :: PROPERTY_USER_ID, $this->get_id());
        $conditions = new EqualityCondition(UserRightsTemplate :: PROPERTY_RIGHTS_TEMPLATE_ID, $rights_template_id);
        $condition = new AndCondition($conditions);

        return DataManager :: deletes(UserRightsTemplate :: class_name(), $condition);
    }

    function update_rights_template_links($rights_templates)
    {
        // Delete the no longer existing links
        $conditions = array();
        $conditions = new NotCondition(new InCondition(UserRightsTemplate :: PROPERTY_RIGHTS_TEMPLATE_ID, $rights_templates));
        $conditions = new EqualityCondition(UserRightsTemplate :: PROPERTY_USER_ID, $this->get_id());
        $condition = new AndCondition($conditions);

        $success = DataManager :: deletes(UserRightsTemplate :: class_name(), $condition);
        if (! $success)
        {
            return false;
        }

        // Get the group's rights_templates
        $condition = new EqualityCondition(UserRightsTemplate :: PROPERTY_USER_ID, $this->get_id());
        $user_rights_templates = DataManager :: retrieves(UserRightsTemplate :: class_name(), $condition);
        $existing_rights_templates = array();

        while ($user_rights_template = $user_rights_templates->next_result())
        {
            $existing_rights_templates[] = $user_rights_template->get_rights_template_id();
        }

        // Add the new links
        foreach ($rights_templates as $rights_template)
        {
            if (! in_array($rights_template, $existing_rights_templates))
            {
                $template = new UserRightsTemplate();
                $template->set_user_id($this->get_id());
                $template->set_rights_template_id($rights_template);
                if (! $template->create())
                {
                    return false;
                }
            }
        }

        return true;
    }

    function get_status_name()
    {
        if ($this->get_platformadmin() == '1')
        {
            return Translation :: get('PlatformAdministrator');
        }

        switch ($this->get_status())
        {
            case self :: STATUS_ANONYMOUS :
                return Translation :: get('Anonymous');
                break;
            case self :: STATUS_STUDENT :
                return Translation :: get('Student');
                break;
            case self :: STATUS_TEACHER :
                return Translation :: get('CourseAdmin');
                break;
            default :
                return Translation :: get('Student');
        }
    }

    function get_fullname_format_options()
    {
        $options = array();
        $options[self :: NAME_FORMAT_FIRST] = Translation :: get('FirstName') . ' ' . Translation :: get('LastName');
        $options[self :: NAME_FORMAT_LAST] = Translation :: get('LastName') . ' ' . Translation :: get('FirstName');
        return $options;
    }

    /**
     *
     * @return ArrayResultSet
     */
    function get_allowed_groups()
    {
        if (! isset($this->allowed_groups))
        {
            $this->allowed_groups = array();
            $groups = $this->get_groups();

            if (! is_null($groups))
            {
                while ($group = $groups->next_result())
                {
                    foreach ($group->get_allowed_groups() as $group_id)
                    {
                        if (! in_array($group_id, $this->allowed_groups))
                        {
                            $this->allowed_groups[] = $group_id;
                        }
                    }
                }
            }
        }

        return $this->allowed_groups;
    }

    /**
     * Returns all (unique) properties by which a DataClass object can be cached
     *
     * @param $extended_property_names multitype:string
     * @return multitype:string
     */
    static function get_cacheable_property_names($extended_property_names = array())
    {
        return parent :: get_cacheable_property_names(array(self :: PROPERTY_USERNAME));
    }
}
?>